@@ -1,13 +1,12 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-using System.Threading.Tasks;
 using Microsoft.AspNetCore.Components.RenderTree;
 
 namespace Microsoft.AspNetCore.Components.Analyzers.Tests.TestFiles.ComponentInternalUsageDiagnosticsAnalyzerTest
 {
-    /*MM*/class UsesRendererAsBaseClass : Renderer
+    /*MM*/
+    class UsesRendererAsBaseClass : Renderer
     {
         public UsesRendererAsBaseClass()
             : base(null, null)
