@@ -1,9 +1,8 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-namespace Microsoft.AspNetCore.Components.WebView.Document
+namespace Microsoft.AspNetCore.Components.WebView.Document;
+
+internal class LogicalContainerNode : ContainerNode
 {
-    internal class LogicalContainerNode : ContainerNode
-    {
-    }
 }
