@@ -3,10 +3,9 @@
 
 using Newtonsoft.Json.Linq;
 
-namespace Microsoft.AspNetCore.JsonPatch
+namespace Microsoft.AspNetCore.JsonPatch;
+
+public class ObjectWithJObject
 {
-    public class ObjectWithJObject
-    {
-        public JObject CustomData { get; set; } = new JObject();
-    }
+    public JObject CustomData { get; set; } = new JObject();
 }
