@@ -4,14 +4,13 @@
 using System.ComponentModel.DataAnnotations;
 using Microsoft.AspNetCore.Mvc.ModelBinding;
 
-namespace ApiExplorerWebSite
-{
-    public class Product
-    {
-        [BindRequired]
-        public int Id { get; set; }
+namespace ApiExplorerWebSite;
 
-        [Required]
-        public string Name { get; set; }
-    }
-}
\ No newline at end of file
+public class Product
+{
+    [BindRequired]
+    public int Id { get; set; }
+
+    [Required]
+    public string Name { get; set; }
+}
