@@ -1,9 +1,8 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-namespace BasicWebSite.Models
+namespace BasicWebSite.Models;
+
+public class Product_Xml : Product
 {
-    public class Product_Xml : Product
-    {
-    }
-}
\ No newline at end of file
+}
