@@ -1,10 +1,9 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-namespace ControllersFromServicesWebSite
+namespace ControllersFromServicesWebSite;
+
+public class ValueService
 {
-    public class ValueService
-    {
-        public int Value { get; } = 3;
-    }
+    public int Value { get; } = 3;
 }
