@@ -3,12 +3,10 @@
 
 using Microsoft.AspNetCore.Mvc;
 
-namespace RazorPagesWebSite
-{
-    [Area("Accounts")]
-    public class HomeController : Controller
-    {
-        public IActionResult Index() => View();
-    }
+namespace RazorPagesWebSite;
 
-}
\ No newline at end of file
+[Area("Accounts")]
+public class HomeController : Controller
+{
+    public IActionResult Index() => View();
+}
