@@ -22,10 +22,10 @@ internal static class Utilities
 
     internal static readonly TimeSpan DefaultTimeout = TimeSpan.FromSeconds(15);
 
-    internal static HttpSysListener CreateHttpServer(out string baseAddress)
+    internal static HttpSysListener CreateHttpServer(out string baseAddress, bool respectHttp10KeepAlive = false)
     {
         string root;
-        return CreateDynamicHttpServer(string.Empty, out root, out baseAddress);
+        return CreateDynamicHttpServer(string.Empty, out root, out baseAddress, respectHttp10KeepAlive);
     }
 
     internal static HttpSysListener CreateHttpServerReturnRoot(string path, out string root)
@@ -34,7 +34,7 @@ internal static class Utilities
         return CreateDynamicHttpServer(path, out root, out baseAddress);
     }
 
-    internal static HttpSysListener CreateDynamicHttpServer(string basePath, out string root, out string baseAddress)
+    internal static HttpSysListener CreateDynamicHttpServer(string basePath, out string root, out string baseAddress, bool respectHttp10KeepAlive = false)
     {
         lock (PortLock)
         {
@@ -47,6 +47,7 @@ internal static class Utilities
                 var options = new HttpSysOptions();
                 options.UrlPrefixes.Add(prefix);
                 options.RequestQueueName = prefix.Port; // Convention for use with CreateServerOnExistingQueue
+                options.RespectHttp10KeepAlive = respectHttp10KeepAlive;
                 var listener = new HttpSysListener(options, new LoggerFactory());
                 try
                 {
