@@ -15,6 +15,7 @@ internal sealed class Http1ChunkedEncodingMessageBody : Http1MessageBody
 {
     // byte consts don't have a data type annotation so we pre-cast it
     private const byte ByteCR = (byte)'\r';
+    private const byte ByteLF = (byte)'\n';
     // "7FFFFFFF\r\n" is the largest chunk size that could be returned as an int.
     private const int MaxChunkPrefixBytes = 10;
 
@@ -26,6 +27,8 @@ internal sealed class Http1ChunkedEncodingMessageBody : Http1MessageBody
     private readonly Pipe _requestBodyPipe;
     private ReadResult _readResult;
 
+    private static readonly bool InsecureChunkedParsing = AppContext.TryGetSwitch("Microsoft.AspNetCore.Server.Kestrel.EnableInsecureChunkedRequestParsing", out var value) && value;
+
     public Http1ChunkedEncodingMessageBody(Http1Connection context, bool keepAlive)
         : base(context, keepAlive)
     {
@@ -343,15 +346,31 @@ internal sealed class Http1ChunkedEncodingMessageBody : Http1MessageBody
         KestrelBadHttpRequestException.Throw(RequestRejectionReason.BadChunkSizeData);
     }
 
+    // https://www.rfc-editor.org/rfc/rfc9112#section-7.1
+    // chunk          = chunk-size [ chunk-ext ] CRLF
+    // chunk-data CRLF
+
+    // https://www.rfc-editor.org/rfc/rfc9112#section-7.1.1
+    // chunk-ext      = *( BWS ";" BWS chunk-ext-name
+    //                     [BWS "=" BWS chunk-ext-val] )
+    // chunk-ext-name = token
+    // chunk-ext-val  = token / quoted-string
     private void ParseExtension(ReadOnlySequence<byte> buffer, out SequencePosition consumed, out SequencePosition examined)
     {
-        // Chunk-extensions not currently parsed
-        // Just drain the data
-        examined = buffer.Start;
+        // Chunk-extensions parsed for \r\n and throws for unpaired \r or \n.
 
         do
         {
-            SequencePosition? extensionCursorPosition = buffer.PositionOf(ByteCR);
+            SequencePosition? extensionCursorPosition;
+            if (InsecureChunkedParsing)
+            {
+                extensionCursorPosition = buffer.PositionOf(ByteCR);
+            }
+            else
+            {
+                extensionCursorPosition = buffer.PositionOfAny(ByteCR, ByteLF);
+            }
+
             if (extensionCursorPosition == null)
             {
                 // End marker not found yet
@@ -359,9 +378,10 @@ internal sealed class Http1ChunkedEncodingMessageBody : Http1MessageBody
                 examined = buffer.End;
                 AddAndCheckObservedBytes(buffer.Length);
                 return;
-            };
+            }
 
             var extensionCursor = extensionCursorPosition.Value;
+
             var charsToByteCRExclusive = buffer.Slice(0, extensionCursor).Length;
 
             var suffixBuffer = buffer.Slice(extensionCursor);
@@ -376,7 +396,9 @@ internal sealed class Http1ChunkedEncodingMessageBody : Http1MessageBody
             suffixBuffer = suffixBuffer.Slice(0, 2);
             var suffixSpan = suffixBuffer.ToSpan();
 
-            if (suffixSpan[1] == '\n')
+            if (InsecureChunkedParsing
+                ? (suffixSpan[1] == ByteLF)
+                : (suffixSpan[0] == ByteCR && suffixSpan[1] == ByteLF))
             {
                 // We consumed the \r\n at the end of the extension, so switch modes.
                 _mode = _inputLength > 0 ? Mode.Data : Mode.Trailer;
@@ -385,13 +407,22 @@ internal sealed class Http1ChunkedEncodingMessageBody : Http1MessageBody
                 examined = suffixBuffer.End;
                 AddAndCheckObservedBytes(charsToByteCRExclusive + 2);
             }
-            else
+            else if (InsecureChunkedParsing)
             {
+                examined = buffer.Start;
                 // Don't consume suffixSpan[1] in case it is also a \r.
                 buffer = buffer.Slice(charsToByteCRExclusive + 1);
                 consumed = extensionCursor;
                 AddAndCheckObservedBytes(charsToByteCRExclusive + 1);
             }
+            else
+            {
+                consumed = suffixBuffer.End;
+                examined = suffixBuffer.End;
+
+                // We have \rX or \nX, that's an invalid extension.
+                KestrelBadHttpRequestException.Throw(RequestRejectionReason.BadChunkExtension);
+            }
         } while (_mode == Mode.Extension);
     }
 
