@@ -388,15 +388,15 @@ internal class Http3InMemory
         return bufferSize ?? 0;
     }
 
-    internal ValueTask<Http3ControlStream> CreateControlStream()
+    internal ValueTask<Http3ControlStream> CreateControlStream(PipeScheduler clientWriterScheduler = null)
     {
-        return CreateControlStream(id: 0);
+        return CreateControlStream(id: 0, clientWriterScheduler);
     }
 
-    internal async ValueTask<Http3ControlStream> CreateControlStream(int? id)
+    internal async ValueTask<Http3ControlStream> CreateControlStream(int? id, PipeScheduler clientWriterScheduler = null)
     {
         var testStreamContext = new TestStreamContext(canRead: true, canWrite: false, this);
-        testStreamContext.Initialize(streamId: 2);
+        testStreamContext.Initialize(streamId: 2, clientWriterScheduler);
 
         var stream = new Http3ControlStream(this, testStreamContext);
         _runningStreams[stream.StreamId] = stream;
@@ -409,16 +409,17 @@ internal class Http3InMemory
         return stream;
     }
 
-    internal async ValueTask<Http3RequestStream> CreateRequestStream(IEnumerable<KeyValuePair<string, string>> headers, Http3RequestHeaderHandler headerHandler = null, bool endStream = false, TaskCompletionSource tsc = null)
+    internal async ValueTask<Http3RequestStream> CreateRequestStream(IEnumerable<KeyValuePair<string, string>> headers,
+        Http3RequestHeaderHandler headerHandler = null, bool endStream = false, TaskCompletionSource tsc = null, PipeScheduler clientWriterScheduler = null)
     {
-        var stream = CreateRequestStreamCore(headerHandler);
+        var stream = CreateRequestStreamCore(headerHandler, clientWriterScheduler);
 
         if (tsc is not null)
         {
             stream.StartStreamDisposeTcs = new TaskCompletionSource(TaskCreationOptions.RunContinuationsAsynchronously);
         }
 
-        if (headers is not null)
+        if (headers is not null && headers.Any())
         {
             await stream.SendHeadersAsync(headers, endStream);
         }
@@ -430,9 +431,10 @@ internal class Http3InMemory
         return stream;
     }
 
-    internal async ValueTask<Http3RequestStream> CreateRequestStream(Http3HeadersEnumerator headers, Http3RequestHeaderHandler headerHandler = null, bool endStream = false, TaskCompletionSource tsc = null)
+    internal async ValueTask<Http3RequestStream> CreateRequestStream(Http3HeadersEnumerator headers, Http3RequestHeaderHandler headerHandler = null,
+        bool endStream = false, TaskCompletionSource tsc = null, PipeScheduler clientWriterScheduler = null)
     {
-        var stream = CreateRequestStreamCore(headerHandler);
+        var stream = CreateRequestStreamCore(headerHandler, clientWriterScheduler);
 
         if (tsc is not null)
         {
@@ -448,7 +450,7 @@ internal class Http3InMemory
         return stream;
     }
 
-    private Http3RequestStream CreateRequestStreamCore(Http3RequestHeaderHandler headerHandler)
+    private Http3RequestStream CreateRequestStreamCore(Http3RequestHeaderHandler headerHandler, PipeScheduler clientWriterScheduler)
     {
         var requestStreamId = GetStreamId(0x00);
         if (!_streamContextPool.TryDequeue(out var testStreamContext))
@@ -459,7 +461,7 @@ internal class Http3InMemory
         {
             Logger.LogDebug($"Reusing context for request stream {requestStreamId}.");
         }
-        testStreamContext.Initialize(requestStreamId);
+        testStreamContext.Initialize(requestStreamId, clientWriterScheduler);
 
         return new Http3RequestStream(this, Connection, testStreamContext, headerHandler ?? new Http3RequestHeaderHandler());
     }
@@ -559,7 +561,7 @@ internal class Http3StreamBase
                     throw new InvalidOperationException("No data received.");
                 }
 
-                if (Http3FrameReader.TryReadFrame(ref buffer, frame, out var framePayload))
+                if (Http3FrameReader.TryReadFrame(ref buffer, frame, isContinuedFrame: false, out var framePayload))
                 {
                     consumed = examined = framePayload.End;
                     frame.Payload = framePayload.ToArray();
@@ -837,16 +839,14 @@ internal class Http3ControlStream : Http3StreamBase
         var settings = new Dictionary<long, long>();
         while (true)
         {
-            var id = VariableLengthIntegerHelper.GetInteger(payload, out var consumed, out _);
-            if (id == -1)
+            if (!VariableLengthIntegerHelper.TryGetInteger(payload, out var consumed, out var id))
             {
                 break;
             }
 
             payload = payload.Slice(consumed);
 
-            var value = VariableLengthIntegerHelper.GetInteger(payload, out consumed, out _);
-            if (value == -1)
+            if (!VariableLengthIntegerHelper.TryGetInteger(payload, out consumed, out var value))
             {
                 break;
             }
@@ -927,9 +927,9 @@ internal class Http3ControlStream : Http3StreamBase
             {
                 if (!readableBuffer.IsEmpty)
                 {
-                    var id = VariableLengthIntegerHelper.GetInteger(readableBuffer, out consumed, out examined);
-                    if (id != -1)
+                    if (VariableLengthIntegerHelper.TryGetInteger(readableBuffer, out consumed, out var id))
                     {
+                        examined = consumed;
                         return id;
                     }
                 }
@@ -1005,6 +1005,7 @@ internal class TestMultiplexedConnectionContext : MultiplexedConnectionContext,
         Features.Set<IProtocolErrorCodeFeature>(this);
         Features.Set<IConnectionMetricsContextFeature>(this);
         ConnectionClosedRequested = ConnectionClosingCts.Token;
+        ConnectionClosed = ConnectionClosedCts.Token;
     }
 
     public override string ConnectionId { get; set; }
@@ -1017,6 +1018,8 @@ internal class TestMultiplexedConnectionContext : MultiplexedConnectionContext,
 
     public CancellationTokenSource ConnectionClosingCts { get; set; } = new CancellationTokenSource();
 
+    public CancellationTokenSource ConnectionClosedCts { get; set; } = new CancellationTokenSource();
+
     public long Error
     {
         get => _error ?? -1;
@@ -1033,6 +1036,7 @@ internal class TestMultiplexedConnectionContext : MultiplexedConnectionContext,
     {
         ToServerAcceptQueue.Writer.TryComplete();
         ToClientAcceptQueue.Writer.TryComplete();
+        ConnectionClosedCts.Cancel();
     }
 
     public override async ValueTask<ConnectionContext> AcceptAsync(CancellationToken cancellationToken = default)
@@ -1106,39 +1110,31 @@ internal class TestStreamContext : ConnectionContext, IStreamDirectionFeature, I
         _testBase = testBase;
     }
 
-    public void Initialize(long streamId)
+    public void Initialize(long streamId, PipeScheduler clientWriterScheduler = null)
     {
-        if (!_isComplete)
-        {
-            // Create new pipes when test stream context is reused rather than reseting them.
-            // This is required because the client tests read from these directly from these pipes.
-            // When a request is finished they'll check to see whether there is anymore content
-            // in the Application.Output pipe. If it has been reset then that code will error.
-            var inputOptions = Http3InMemory.GetInputPipeOptions(_testBase._serviceContext, _testBase._memoryPool, PipeScheduler.ThreadPool);
-            var outputOptions = Http3InMemory.GetOutputPipeOptions(_testBase._serviceContext, _testBase._memoryPool, PipeScheduler.ThreadPool);
-
-            _inputPipe = new Pipe(inputOptions);
-            _outputPipe = new Pipe(outputOptions);
-
-            _transportPipeReader = new CompletionPipeReader(_inputPipe.Reader);
-            _transportPipeWriter = new CompletionPipeWriter(_outputPipe.Writer);
-
-            _pair = new DuplexPipePair(
-                new DuplexPipe(_transportPipeReader, _transportPipeWriter),
-                new DuplexPipe(_outputPipe.Reader, _inputPipe.Writer));
-        }
-        else
+        if (_isComplete)
         {
             _pair.Application.Input.Complete();
             _pair.Application.Output.Complete();
-
-            _transportPipeReader.Reset();
-            _transportPipeWriter.Reset();
-
-            _inputPipe.Reset();
-            _outputPipe.Reset();
         }
 
+        // Create new pipes when test stream context is reused rather than reseting them.
+        // This is required because the client tests read from these directly from these pipes.
+        // When a request is finished they'll check to see whether there is anymore content
+        // in the Application.Output pipe. If it has been reset then that code will error.
+        var inputOptions = Http3InMemory.GetInputPipeOptions(_testBase._serviceContext, _testBase._memoryPool, clientWriterScheduler ?? PipeScheduler.ThreadPool);
+        var outputOptions = Http3InMemory.GetOutputPipeOptions(_testBase._serviceContext, _testBase._memoryPool, PipeScheduler.ThreadPool);
+
+        _inputPipe = new Pipe(inputOptions);
+        _outputPipe = new Pipe(outputOptions);
+
+        _transportPipeReader = new CompletionPipeReader(_inputPipe.Reader);
+        _transportPipeWriter = new CompletionPipeWriter(_outputPipe.Writer);
+
+        _pair = new DuplexPipePair(
+            new DuplexPipe(_transportPipeReader, _transportPipeWriter),
+            new DuplexPipe(_outputPipe.Reader, _inputPipe.Writer));
+
         Features.Set<IStreamDirectionFeature>(this);
         Features.Set<IStreamIdFeature>(this);
         Features.Set<IStreamAbortFeature>(this);
