@@ -7,9 +7,11 @@ export { AbortError, HttpError, TimeoutError } from "./Errors";
 export { HttpClient, HttpRequest, HttpResponse } from "./HttpClient";
 export { DefaultHttpClient } from "./DefaultHttpClient";
 export { IHttpConnectionOptions } from "./IHttpConnectionOptions";
+export { IStatefulReconnectOptions } from "./IStatefulReconnectOptions";
 export { HubConnection, HubConnectionState } from "./HubConnection";
 export { HubConnectionBuilder } from "./HubConnectionBuilder";
-export { MessageType, MessageHeaders, HubMessage, HubMessageBase, HubInvocationMessage, InvocationMessage, StreamInvocationMessage, StreamItemMessage, CompletionMessage,
+export { AckMessage, SequenceMessage, MessageType, MessageHeaders, HubMessage, HubMessageBase,
+    HubInvocationMessage, InvocationMessage, StreamInvocationMessage, StreamItemMessage, CompletionMessage,
     PingMessage, CloseMessage, CancelInvocationMessage, IHubProtocol } from "./IHubProtocol";
 export { ILogger, LogLevel } from "./ILogger";
 export { HttpTransportType, TransferFormat, ITransport } from "./ITransport";
