@@ -1,10 +1,9 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-namespace Microsoft.AspNetCore.Testing
+namespace Microsoft.AspNetCore.Testing;
+
+public class TestAssemblyFixture
 {
-    public class TestAssemblyFixture
-    {
-        public int Count { get; set; }
-    }
+    public int Count { get; set; }
 }
