@@ -1,6 +1,5 @@
 ﻿// Copyright (c) Microsoft.  All Rights Reserved.  Licensed under the MIT license.  See License.txt in the project root for license information.
 
-using System;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Testing;
 using Test.Utilities;
@@ -16,10 +15,12 @@ using VerifyVB = Test.Utilities.VisualBasicCodeFixVerifier<
 
 namespace Microsoft.CodeAnalysis.BannedApiAnalyzers.UnitTests
 {
-    // For specification of document comment IDs see https://docs.microsoft.com/en-us/dotnet/csharp/language-reference/language-specification/documentation-comments#processing-the-documentation-file
+    // For specification of document comment IDs see https://learn.microsoft.com/dotnet/csharp/language-reference/language-specification/documentation-comments#processing-the-documentation-file
 
     public class SymbolIsBannedAnalyzerTests
     {
+        private const string BannedSymbolsFileName = "BannedSymbols.txt";
+
         private static DiagnosticResult GetCSharpResultAt(int markupKey, DiagnosticDescriptor descriptor, string bannedMemberName, string message)
             => VerifyCS.Diagnostic(descriptor)
                 .WithLocation(markupKey)
@@ -37,7 +38,7 @@ namespace Microsoft.CodeAnalysis.BannedApiAnalyzers.UnitTests
                 TestState =
                 {
                     Sources = { source },
-                    AdditionalFiles = { (SymbolIsBannedAnalyzer.BannedSymbolsFileName, bannedApiText) },
+                    AdditionalFiles = { (BannedSymbolsFileName, bannedApiText) },
                 },
             };
 
@@ -52,7 +53,7 @@ namespace Microsoft.CodeAnalysis.BannedApiAnalyzers.UnitTests
                 TestState =
                 {
                     Sources = { source },
-                    AdditionalFiles = { (SymbolIsBannedAnalyzer.BannedSymbolsFileName, bannedApiText) },
+                    AdditionalFiles = { (BannedSymbolsFileName, bannedApiText) },
                 },
             };
 
@@ -80,6 +81,31 @@ End Class");
             await VerifyCSharpAnalyzerAsync(source, bannedText);
         }
 
+        [Fact]
+        public async Task NoDiagnosticReportedForMultilineBlankBannedTextAsync()
+        {
+            var source = @"
+
+
+";
+
+            var bannedText = @"";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText);
+        }
+
+        [Fact]
+        public async Task NoDiagnosticReportedForMultilineMessageOnlyBannedTextAsync()
+        {
+            var source = @"";
+            var bannedText = @"
+;first
+  ;second
+;third // comment";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText);
+        }
+
         [Fact]
         public async Task DiagnosticReportedForDuplicateBannedApiLinesAsync()
         {
@@ -113,6 +139,264 @@ End Class");
             await VerifyCSharpAnalyzerAsync(source, bannedText, expected);
         }
 
+        [Fact]
+        public async Task DiagnosticReportedForDuplicateBannedApiLinesInDifferentFiles()
+        {
+            var test = new VerifyCS.Test
+            {
+                TestState =
+                {
+                    Sources = { @"" },
+                    AdditionalFiles =
+                    {
+                        ("BannedSymbols.txt", @"{|#0:T:System.Console|}") ,
+                        ("BannedSymbols.Other.txt", @"{|#1:T:System.Console|}")
+                    },
+                },
+                ExpectedDiagnostics =
+                {
+                    new DiagnosticResult(SymbolIsBannedAnalyzer.DuplicateBannedSymbolRule)
+                        .WithLocation(0)
+                        .WithLocation(1)
+                        .WithArguments("System.Console")
+                }
+            };
+
+            await test.RunAsync();
+        }
+
+        #region Comments in BannedSymbols.txt tests
+
+        [Fact]
+        public async Task DiagnosticReportedForDuplicateBannedApiLinesWithCommentsAsync()
+        {
+            var source = @"";
+            var bannedText = @"
+{|#0:T:System.Console|} " + '\t' + @" //comment here with whitespace before it
+{|#1:T:System.Console|}//should not affect the reported duplicate";
+
+            var expected = new DiagnosticResult(SymbolIsBannedAnalyzer.DuplicateBannedSymbolRule)
+                .WithLocation(1)
+                .WithLocation(0)
+                .WithArguments("System.Console");
+            await VerifyCSharpAnalyzerAsync(source, bannedText, expected);
+        }
+
+        [Fact]
+        public async Task DiagnosticReportedForDuplicateBannedApiLinesWithCommentsInDifferentFiles()
+        {
+            var test = new VerifyCS.Test
+            {
+                TestState =
+                {
+                    Sources = { @"" },
+                    AdditionalFiles =
+                    {
+                        ("BannedSymbols.txt", @"{|#0:T:System.Console|} //ignore this") ,
+                        ("BannedSymbols.Other.txt", @"{|#1:T:System.Console|} //comment will not affect result")
+                    },
+                },
+                ExpectedDiagnostics =
+                {
+                    new DiagnosticResult(SymbolIsBannedAnalyzer.DuplicateBannedSymbolRule)
+                        .WithLocation(0)
+                        .WithLocation(1)
+                        .WithArguments("System.Console")
+                }
+            };
+
+            await test.RunAsync();
+        }
+
+        [Fact]
+        public async Task NoDiagnosticReportedForCommentLine()
+        {
+            var source = @"";
+            var bannedText = @"// this comment line will be ignored";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText);
+            await VerifyBasicAnalyzerAsync(source, bannedText);
+        }
+
+        [Fact]
+        public async Task NoDiagnosticReportedForCommentedOutBannedApiLineAsync()
+        {
+            var bannedText = @"//T:N.Banned";
+
+            var csharpSource = @"
+namespace N
+{
+    class Banned { }
+    class C
+    {
+        void M()
+        {
+            var c = new Banned();
+        }
+    }
+}";
+
+            await VerifyCSharpAnalyzerAsync(csharpSource, bannedText);
+
+            var visualBasicSource = @"
+Namespace N
+    Class Banned : End Class
+    Class C
+        Sub M()
+            Dim c As New Banned()
+        End Sub
+    End Class
+End Namespace";
+
+            await VerifyBasicAnalyzerAsync(visualBasicSource, bannedText);
+        }
+
+        [Fact]
+        public async Task NoDiagnosticReportedForCommentLineThatBeginsWithWhitespaceAsync()
+        {
+            var bannedText = @"  // comment here";
+
+            var csharpSource = @"
+namespace N
+{
+    class Banned { }
+    class C
+    {
+        void M()
+        {
+            var c = new Banned();
+        }
+    }
+}";
+
+            await VerifyCSharpAnalyzerAsync(csharpSource, bannedText);
+
+            var visualBasicSource = @"
+Namespace N
+    Class Banned : End Class
+    Class C
+        Sub M()
+            Dim c As New Banned()
+        End Sub
+    End Class
+End Namespace";
+
+            await VerifyBasicAnalyzerAsync(visualBasicSource, bannedText);
+        }
+
+        [Fact]
+        public async Task NoDiagnosticReportedForCommentedOutDuplicateBannedApiLinesAsync()
+        {
+            var source = @"";
+            var bannedText = @"
+//T:System.Console
+//T:System.Console";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText);
+            await VerifyBasicAnalyzerAsync(source, bannedText);
+        }
+
+        [Fact]
+        public async Task DiagnosticReportedForBannedApiLineWithCommentAtTheEndAsync()
+        {
+            var bannedText = @"T:N.Banned//comment here";
+
+            var csharpSource = @"
+namespace N
+{
+    class Banned { }
+    class C
+    {
+        void M()
+        {
+            var c = {|#0:new Banned()|};
+        }
+    }
+}";
+
+            await VerifyCSharpAnalyzerAsync(csharpSource, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
+
+            var visualBasicSource = @"
+Namespace N
+    Class Banned : End Class
+    Class C
+        Sub M()
+            Dim c As {|#0:New Banned()|}
+        End Sub
+    End Class
+End Namespace";
+
+            await VerifyBasicAnalyzerAsync(visualBasicSource, bannedText, GetBasicResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
+        }
+
+        [Fact]
+        public async Task DiagnosticReportedForBannedApiLineWithWhitespaceThenCommentAtTheEndAsync()
+        {
+            var bannedText = "T:N.Banned \t //comment here";
+
+            var csharpSource = @"
+namespace N
+{
+    class Banned { }
+    class C
+    {
+        void M()
+        {
+            var c = {|#0:new Banned()|};
+        }
+    }
+}";
+
+            await VerifyCSharpAnalyzerAsync(csharpSource, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
+
+            var visualBasicSource = @"
+Namespace N
+    Class Banned : End Class
+    Class C
+        Sub M()
+            Dim c As {|#0:New Banned()|}
+        End Sub
+    End Class
+End Namespace";
+
+            await VerifyBasicAnalyzerAsync(visualBasicSource, bannedText, GetBasicResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
+        }
+
+        [Fact]
+        public async Task DiagnosticReportedForBannedApiLineWithMessageThenWhitespaceFollowedByCommentAtTheEndMustNotIncludeWhitespaceInMessageAsync()
+        {
+            var bannedText = "T:N.Banned;message \t //comment here";
+
+            var csharpSource = @"
+namespace N
+{
+    class Banned { }
+    class C
+    {
+        void M()
+        {
+            var c = {|#0:new Banned()|};
+        }
+    }
+}";
+
+            await VerifyCSharpAnalyzerAsync(csharpSource, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ": message"));
+
+            var visualBasicSource = @"
+Namespace N
+    Class Banned : End Class
+    Class C
+        Sub M()
+            Dim c As {|#0:New Banned()|}
+        End Sub
+    End Class
+End Namespace";
+
+            await VerifyBasicAnalyzerAsync(visualBasicSource, bannedText, GetBasicResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ": message"));
+        }
+
+        #endregion Comments in BannedSymbols.txt tests
+
         [Fact]
         public async Task CSharp_BannedApiFile_MessageIncludedInDiagnosticAsync()
         {
@@ -198,6 +482,48 @@ namespace N
             await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
         }
 
+        [Fact]
+        public async Task CSharp_BannedApi_MultipleFiles()
+        {
+            var source = @"
+namespace N
+{
+    class BannedA { }
+    class BannedB { }
+    class NotBanned { }
+    class C
+    {
+        void M()
+        {
+            var a = {|#0:new BannedA()|};
+            var b = {|#1:new BannedB()|};
+            var c = new NotBanned();
+        }
+    }
+}";
+
+            var test = new VerifyCS.Test
+            {
+                TestState =
+                {
+                    Sources = { source },
+                    AdditionalFiles =
+                    {
+                        ("BannedSymbols.txt", @"T:N.BannedA") ,
+                        ("BannedSymbols.Other.txt", @"T:N.BannedB"),
+                        ("OtherFile.txt", @"T:N.NotBanned")
+                    },
+                },
+                ExpectedDiagnostics =
+                {
+                    GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "BannedA", ""),
+                    GetCSharpResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "BannedB", "")
+                }
+            };
+
+            await test.RunAsync();
+        }
+
         [Fact]
         public async Task CSharp_BannedType_ConstructorAsync()
         {
@@ -220,6 +546,159 @@ T:N.Banned";
             await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
         }
 
+        [Fact]
+        public async Task CSharp_BannedNamespace_ConstructorAsync()
+        {
+            var source = @"
+namespace N
+{
+    class C
+    {
+        void M()
+        {
+            var c = {|#0:new N.C()|};
+        }
+    }
+}
+";
+
+            var bannedText = @"
+N:N";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""));
+        }
+
+        [Fact]
+        public async Task CSharp_BannedNamespace_Parent_ConstructorAsync()
+        {
+            var source = @"
+namespace N.NN
+{
+    class C
+    {
+        void M()
+        {
+            var a = {|#0:new N.NN.C()|};
+        }
+    }
+}
+";
+
+            var bannedText = @"
+N:N";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""));
+        }
+
+        [Fact]
+        public async Task CSharp_BannedNamespace_MethodGroupAsync()
+        {
+            var source = @"
+namespace N
+{
+    delegate void D();
+    class C
+    {
+        void M()
+        {
+            D d = {|#0:M|};
+        }
+    }
+}
+";
+
+            var bannedText = @"
+N:N";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""));
+        }
+
+        [Fact]
+        public async Task CSharp_BannedNamespace_PropertyAsync()
+        {
+            var source = @"
+namespace N
+{
+    class C
+    {
+        public int P { get; set; }
+        void M()
+        {
+            {|#0:P|} = {|#1:P|};
+        }
+    }
+}
+";
+
+            var bannedText = @"
+N:N";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText,
+                GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""),
+                GetCSharpResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""));
+        }
+
+        [Fact]
+        public async Task CSharp_BannedNamespace_MethodAsync()
+        {
+            var source = @"
+namespace N
+{
+    interface I
+    {
+        void M();
+    }
+}
+
+class C
+{
+    void M()
+    {
+        N.I i = null;
+        {|#0:i.M()|};
+    }
+}";
+            var bannedText = @"N:N";
+
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""));
+        }
+
+        [Fact]
+        public async Task CSharp_BannedNamespace_TypeOfArgument()
+        {
+            var source = @"
+namespace N
+{
+    class Banned {  }
+}
+class C
+{
+    void M()
+    {
+        var type = {|#0:typeof(N.Banned)|};
+    }
+}
+";
+            var bannedText = @"N:N";
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "N", ""));
+        }
+
+        [Fact]
+        public async Task CSharp_BannedNamespace_Constituent()
+        {
+            var source = @"
+class C
+{
+    void M()
+    {
+        var thread = {|#0:new System.Threading.Thread((System.Threading.ThreadStart)null)|};
+    }
+}
+";
+            var bannedText = @"N:System.Threading";
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "System.Threading", ""));
+        }
+
         [Fact]
         public async Task CSharp_BannedGenericType_ConstructorAsync()
         {
@@ -582,6 +1061,46 @@ namespace N
                 GetCSharpResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned.Banned(int)", ""));
         }
 
+        [Fact]
+        public async Task Csharp_BannedConstructor_AttributeAsync()
+        {
+            var source = @"
+using System;
+
+[AttributeUsage(AttributeTargets.All, Inherited = true)]
+class BannedAttribute : Attribute
+{
+    public BannedAttribute() {}
+    public BannedAttribute(int banned) {}
+    public BannedAttribute(string notBanned) {}
+}
+
+class C
+{
+    [{|#0:Banned|}]
+    public int SomeProperty { get; }
+
+    [{|#1:Banned(1)|}]
+    public void SomeMethod() {}
+
+    [Banned("""")]
+    class D {}
+}
+";
+            var bannedText1 = @"M:BannedAttribute.#ctor";
+            var bannedText2 = @"M:BannedAttribute.#ctor(System.Int32)";
+
+            await VerifyCSharpAnalyzerAsync(
+                source,
+                bannedText1,
+                GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "BannedAttribute.BannedAttribute()", ""));
+
+            await VerifyCSharpAnalyzerAsync(
+                source,
+                bannedText2,
+                GetCSharpResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "BannedAttribute.BannedAttribute(int)", ""));
+        }
+
         [Fact]
         public async Task CSharp_BannedMethodAsync()
         {
@@ -752,7 +1271,6 @@ namespace N
         }
     }
 }";
-
             var bannedText = @"M:N.C.Banned";
 
             await VerifyCSharpAnalyzerAsync(
@@ -761,6 +1279,40 @@ namespace N
                 GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "C.Banned()", ""));
         }
 
+        [Fact]
+        public async Task CSharp_NoDiagnosticClass_TypeOfArgument()
+        {
+            var source = @"
+class Banned {  }
+class C
+{
+    void M()
+    {
+        var type = {|#0:typeof(C)|};
+    }
+}
+";
+            var bannedText = @"T:Banned";
+            await VerifyCSharpAnalyzerAsync(source, bannedText);
+        }
+
+        [Fact]
+        public async Task CSharp_BannedClass_TypeOfArgument()
+        {
+            var source = @"
+class Banned {  }
+class C
+{
+    void M()
+    {
+        var type = {|#0:typeof(Banned)|};
+    }
+}
+";
+            var bannedText = @"T:Banned";
+            await VerifyCSharpAnalyzerAsync(source, bannedText, GetCSharpResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Banned", ""));
+        }
+
         [Fact, WorkItem(3295, "https://github.com/dotnet/roslyn-analyzers/issues/3295")]
         public async Task CSharp_BannedAbstractVirtualMemberAlsoBansOverrides_RootLevelIsBannedAsync()
         {
@@ -1036,6 +1588,45 @@ namespace N
             await VerifyCSharpAnalyzerAsync(source, bannedText);
         }
 
+        [Fact]
+        public async Task VisualBasic_BannedApi_MultipleFiles()
+        {
+            var source = @"
+Namespace N
+    Class BannedA : End Class
+    Class BannedB : End Class
+    Class NotBanned : End Class
+    Class C
+        Sub M()
+            Dim a As {|#0:New BannedA()|}
+            Dim b As {|#1:New BannedB()|}
+            Dim c As New NotBanned()
+        End Sub
+    End Class
+End Namespace";
+
+            var test = new VerifyVB.Test
+            {
+                TestState =
+                {
+                    Sources = { source },
+                    AdditionalFiles =
+                    {
+                        ("BannedSymbols.txt", @"T:N.BannedA") ,
+                        ("BannedSymbols.Other.txt", @"T:N.BannedB"),
+                        ("OtherFile.txt", @"T:N.NotBanned")
+                    },
+                },
+                ExpectedDiagnostics =
+                {
+                    GetBasicResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "BannedA", ""),
+                    GetBasicResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "BannedB", "")
+                }
+            };
+
+            await test.RunAsync();
+        }
+
         [Fact]
         public async Task VisualBasic_BannedType_ConstructorAsync()
         {
@@ -1317,6 +1908,46 @@ End Namespace";
                 GetBasicResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Public Sub New(I As Integer)", ""));
         }
 
+        [Fact]
+        public async Task VisualBasic_BannedConstructor_AttributeAsync()
+        {
+            var source = @"
+Imports System
+
+<AttributeUsage(System.AttributeTargets.All, Inherited:=True)>
+Class BannedAttribute
+    Inherits System.Attribute
+
+    Sub New : End Sub
+    Sub New(ByVal Banned As Integer) : End Sub
+    Sub New(ByVal NotBanned As String) : End Sub
+End Class
+
+Class C
+    <{|#0:Banned|}>
+    Public ReadOnly Property SomeProperty As Integer
+
+    <{|#1:Banned(1)|}>
+    Public Sub SomeMethod : End Sub
+
+    <Banned("""")>
+    Class D : End Class
+End Class
+";
+            var bannedText1 = @"M:BannedAttribute.#ctor";
+            var bannedText2 = @"M:BannedAttribute.#ctor(System.Int32)";
+
+            await VerifyBasicAnalyzerAsync(
+                source,
+                bannedText1,
+                GetBasicResultAt(0, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Public Sub New()", ""));
+
+            await VerifyBasicAnalyzerAsync(
+                source,
+                bannedText2,
+                GetBasicResultAt(1, SymbolIsBannedAnalyzer.SymbolIsBannedRule, "Public Sub New(Banned As Integer)", ""));
+        }
+
         [Fact]
         public async Task VisualBasic_BannedMethodAsync()
         {
