@@ -1,9 +1,10 @@
-// Copyright (c) Microsoft.  All Rights Reserved.  Licensed under the MIT license.  See License.txt in the project root for license information.
+﻿// Copyright (c) Microsoft.  All Rights Reserved.  Licensed under the MIT license.  See License.txt in the project root for license information.
 
 using System.Collections.Immutable;
 using System.Linq;
 using Analyzer.Utilities;
 using Analyzer.Utilities.Extensions;
+using Analyzer.Utilities.Lightup;
 using Microsoft.CodeAnalysis;
 using Microsoft.CodeAnalysis.Diagnostics;
 using Microsoft.CodeAnalysis.Operations;
@@ -13,7 +14,7 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
     using static MicrosoftCodeQualityAnalyzersResources;
 
     /// <summary>
-    /// CA1063: Implement IDisposable Correctly
+    /// CA1063: <inheritdoc cref="ImplementIDisposableCorrectlyTitle"/>
     /// </summary>
     [DiagnosticAnalyzer(LanguageNames.CSharp, LanguageNames.VisualBasic)]
     public sealed class ImplementIDisposableCorrectlyAnalyzer : DiagnosticAnalyzer
@@ -281,7 +282,7 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
             {
                 if (implementsDisposableInBaseType)
                 {
-                    context.ReportDiagnostic(type.CreateDiagnostic(IDisposableReimplementationRule, type.Name, type.BaseType.Name));
+                    context.ReportDiagnostic(type.CreateDiagnostic(IDisposableReimplementationRule, type.Name, type.BaseType!.Name));
                 }
             }
 
@@ -316,7 +317,7 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
                 if (method.MethodKind == MethodKind.Ordinary && method.IsOverride && method.ReturnsVoid && method.Parameters.IsEmpty)
                 {
                     bool isDisposeOverride = false;
-                    for (IMethodSymbol m = method.OverriddenMethod; m != null; m = m.OverriddenMethod)
+                    for (IMethodSymbol? m = method.OverriddenMethod; m != null; m = m.OverriddenMethod)
                     {
                         if (Equals(m, FindDisposeMethod(m.ContainingType)))
                         {
@@ -443,13 +444,13 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
             /// <summary>
             /// Returns method defined in the nearest ancestor: void Dispose(bool)
             /// </summary>
-            private IMethodSymbol? FindInheritedDisposeBoolMethod(INamedTypeSymbol type)
+            private IMethodSymbol? FindInheritedDisposeBoolMethod(INamedTypeSymbol? type)
             {
                 IMethodSymbol? method = null;
 
                 while (type != null && method == null && ImplementsDisposableInBaseType(type))
                 {
-                    type = type.BaseType;
+                    type = type.BaseType!;
                     method = FindDisposeBoolMethod(type);
                 }
 
@@ -532,7 +533,7 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
             {
                 foreach (IOperation operation in operations)
                 {
-                    if (!operation.IsImplicit && !ValidateOperation(operation))
+                    if (!operation.IsImplicit && operation.Kind != OperationKindEx.Attribute && !ValidateOperation(operation))
                     {
                         return false;
                     }
@@ -660,7 +661,7 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
                     // call to the base finalizer in the finally section. We need to validate the contents
                     // of the try block
                     // Also analyze the implicit expression statement created for expression bodied implementation.
-                    var shouldAnalyze = !operation.IsImplicit || operation.Kind == OperationKind.Try || operation.Kind == OperationKind.ExpressionStatement;
+                    var shouldAnalyze = (!operation.IsImplicit && operation.Kind != OperationKindEx.Attribute) || operation.Kind == OperationKind.Try || operation.Kind == OperationKind.ExpressionStatement;
                     if (shouldAnalyze && !ValidateOperation(operation))
                     {
                         return false;
@@ -707,7 +708,7 @@ namespace Microsoft.CodeQuality.Analyzers.ApiDesignGuidelines
                     IMethodSymbol methodSymbol = invocation.TargetMethod;
                     IInstanceReferenceOperation receiver = (IInstanceReferenceOperation)invocation.Instance;
 
-                    return methodSymbol.IsFinalizer() && Equals(receiver.Type.OriginalDefinition, _type.BaseType.OriginalDefinition);
+                    return methodSymbol.IsFinalizer() && Equals(receiver.Type?.OriginalDefinition, _type.BaseType.OriginalDefinition);
                 }
 
                 return false;
