@@ -18,6 +18,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
     using static MicrosoftNetCoreAnalyzersResources;
 
     /// <summary>
+    /// CA2252: <inheritdoc cref="DetectPreviewFeaturesTitle"/>
     /// Detect the use of [RequiresPreviewFeatures] in assemblies that have not opted into preview features
     /// </summary>
     public abstract class DetectPreviewFeatureAnalyzer : DiagnosticAnalyzer
@@ -28,7 +29,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
         private static readonly LocalizableString s_localizableDescription = CreateLocalizableResourceString(nameof(DetectPreviewFeaturesDescription));
         private static readonly ImmutableArray<SymbolKind> s_symbols = ImmutableArray.Create(SymbolKind.NamedType, SymbolKind.Method, SymbolKind.Property, SymbolKind.Field, SymbolKind.Event);
 
-        internal static DiagnosticDescriptor GeneralPreviewFeatureAttributeRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor GeneralPreviewFeatureAttributeRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                     s_localizableTitle,
                                                                                                                     CreateLocalizableResourceString(nameof(DetectPreviewFeaturesMessage)),
                                                                                                                     DiagnosticCategory.Usage,
@@ -36,7 +37,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                     s_localizableDescription,
                                                                                                                     isPortedFxCopRule: false,
                                                                                                                     isDataflowRule: false);
-        internal static DiagnosticDescriptor GeneralPreviewFeatureAttributeRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor GeneralPreviewFeatureAttributeRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                     s_localizableTitle,
                                                                                                                     CreateLocalizableResourceString(nameof(DetectPreviewFeaturesMessageWithCustomMessagePlaceholder)),
                                                                                                                     DiagnosticCategory.Usage,
@@ -45,7 +46,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                     isPortedFxCopRule: false,
                                                                                                                     isDataflowRule: false);
 
-        internal static DiagnosticDescriptor ImplementsPreviewInterfaceRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor ImplementsPreviewInterfaceRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                 s_localizableTitle,
                                                                                                                 CreateLocalizableResourceString(nameof(ImplementsPreviewInterfaceMessage)),
                                                                                                                 DiagnosticCategory.Usage,
@@ -53,7 +54,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                 s_localizableDescription,
                                                                                                                 isPortedFxCopRule: false,
                                                                                                                 isDataflowRule: false);
-        internal static DiagnosticDescriptor ImplementsPreviewInterfaceRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor ImplementsPreviewInterfaceRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                 s_localizableTitle,
                                                                                                                 CreateLocalizableResourceString(nameof(ImplementsPreviewInterfaceMessageWithCustomMessagePlaceholder)),
                                                                                                                 DiagnosticCategory.Usage,
@@ -62,7 +63,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                 isPortedFxCopRule: false,
                                                                                                                 isDataflowRule: false);
 
-        internal static DiagnosticDescriptor ImplementsPreviewMethodRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor ImplementsPreviewMethodRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                 s_localizableTitle,
                                                                                                                 CreateLocalizableResourceString(nameof(ImplementsPreviewMethodMessage)),
                                                                                                                 DiagnosticCategory.Usage,
@@ -70,7 +71,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                 s_localizableDescription,
                                                                                                                 isPortedFxCopRule: false,
                                                                                                                 isDataflowRule: false);
-        internal static DiagnosticDescriptor ImplementsPreviewMethodRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor ImplementsPreviewMethodRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                 s_localizableTitle,
                                                                                                                 CreateLocalizableResourceString(nameof(ImplementsPreviewMethodMessageWithCustomMessagePlaceholder)),
                                                                                                                 DiagnosticCategory.Usage,
@@ -79,7 +80,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                 isPortedFxCopRule: false,
                                                                                                                 isDataflowRule: false);
 
-        internal static DiagnosticDescriptor OverridesPreviewMethodRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor OverridesPreviewMethodRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                 s_localizableTitle,
                                                                                                                 CreateLocalizableResourceString(nameof(OverridesPreviewMethodMessage)),
                                                                                                                 DiagnosticCategory.Usage,
@@ -87,7 +88,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                 s_localizableDescription,
                                                                                                                 isPortedFxCopRule: false,
                                                                                                                 isDataflowRule: false);
-        internal static DiagnosticDescriptor OverridesPreviewMethodRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor OverridesPreviewMethodRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                 s_localizableTitle,
                                                                                                                 CreateLocalizableResourceString(nameof(OverridesPreviewMethodMessageWithCustomMessagePlaceholder)),
                                                                                                                 DiagnosticCategory.Usage,
@@ -96,7 +97,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                 isPortedFxCopRule: false,
                                                                                                                 isDataflowRule: false);
 
-        internal static DiagnosticDescriptor DerivesFromPreviewClassRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor DerivesFromPreviewClassRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                              s_localizableTitle,
                                                                                                              CreateLocalizableResourceString(nameof(DerivesFromPreviewClassMessage)),
                                                                                                              DiagnosticCategory.Usage,
@@ -104,7 +105,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                              s_localizableDescription,
                                                                                                              isPortedFxCopRule: false,
                                                                                                              isDataflowRule: false);
-        internal static DiagnosticDescriptor DerivesFromPreviewClassRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor DerivesFromPreviewClassRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                              s_localizableTitle,
                                                                                                              CreateLocalizableResourceString(nameof(DerivesFromPreviewClassMessageWithCustomMessagePlaceholder)),
                                                                                                              DiagnosticCategory.Usage,
@@ -113,7 +114,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                              isPortedFxCopRule: false,
                                                                                                              isDataflowRule: false);
 
-        internal static DiagnosticDescriptor UsesPreviewTypeParameterRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor UsesPreviewTypeParameterRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                               s_localizableTitle,
                                                                                                               CreateLocalizableResourceString(nameof(UsesPreviewTypeParameterMessage)),
                                                                                                               DiagnosticCategory.Usage,
@@ -121,7 +122,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                               s_localizableDescription,
                                                                                                               isPortedFxCopRule: false,
                                                                                                               isDataflowRule: false);
-        internal static DiagnosticDescriptor UsesPreviewTypeParameterRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor UsesPreviewTypeParameterRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                               s_localizableTitle,
                                                                                                               CreateLocalizableResourceString(nameof(UsesPreviewTypeParameterMessageWithCustomMessagePlaceholder)),
                                                                                                               DiagnosticCategory.Usage,
@@ -130,7 +131,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                               isPortedFxCopRule: false,
                                                                                                               isDataflowRule: false);
 
-        internal static DiagnosticDescriptor MethodReturnsPreviewTypeRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor MethodReturnsPreviewTypeRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                               s_localizableTitle,
                                                                                                               CreateLocalizableResourceString(nameof(MethodReturnsPreviewTypeMessage)),
                                                                                                               DiagnosticCategory.Usage,
@@ -138,7 +139,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                               s_localizableDescription,
                                                                                                               isPortedFxCopRule: false,
                                                                                                               isDataflowRule: false);
-        internal static DiagnosticDescriptor MethodReturnsPreviewTypeRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor MethodReturnsPreviewTypeRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                               s_localizableTitle,
                                                                                                               CreateLocalizableResourceString(nameof(MethodReturnsPreviewTypeMessageWithCustomMessagePlaceholder)),
                                                                                                               DiagnosticCategory.Usage,
@@ -147,7 +148,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                               isPortedFxCopRule: false,
                                                                                                               isDataflowRule: false);
 
-        internal static DiagnosticDescriptor MethodUsesPreviewTypeAsParameterRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor MethodUsesPreviewTypeAsParameterRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                       s_localizableTitle,
                                                                                                                       CreateLocalizableResourceString(nameof(MethodUsesPreviewTypeAsParameterMessage)),
                                                                                                                       DiagnosticCategory.Usage,
@@ -155,7 +156,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                       s_localizableDescription,
                                                                                                                       isPortedFxCopRule: false,
                                                                                                                       isDataflowRule: false);
-        internal static DiagnosticDescriptor MethodUsesPreviewTypeAsParameterRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor MethodUsesPreviewTypeAsParameterRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                       s_localizableTitle,
                                                                                                                       CreateLocalizableResourceString(nameof(MethodUsesPreviewTypeAsParameterMessageWithCustomMessagePlaceholder)),
                                                                                                                       DiagnosticCategory.Usage,
@@ -163,7 +164,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                       s_localizableDescription,
                                                                                                                       isPortedFxCopRule: false,
                                                                                                                       isDataflowRule: false);
-        internal static DiagnosticDescriptor FieldOrEventIsPreviewTypeRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor FieldOrEventIsPreviewTypeRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                         s_localizableTitle,
                                                                                                         CreateLocalizableResourceString(nameof(FieldIsPreviewTypeMessage)),
                                                                                                         DiagnosticCategory.Usage,
@@ -171,7 +172,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                         s_localizableDescription,
                                                                                                         isPortedFxCopRule: false,
                                                                                                         isDataflowRule: false);
-        internal static DiagnosticDescriptor FieldOrEventIsPreviewTypeRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor FieldOrEventIsPreviewTypeRuleWithCustomMessage = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                         s_localizableTitle,
                                                                                                         CreateLocalizableResourceString(nameof(FieldIsPreviewTypeMessageWithCustomMessagePlaceholder)),
                                                                                                         DiagnosticCategory.Usage,
@@ -180,7 +181,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                         isPortedFxCopRule: false,
                                                                                                         isDataflowRule: false);
 
-        internal static DiagnosticDescriptor StaticAbstractIsPreviewFeatureRule = DiagnosticDescriptorHelper.Create(RuleId,
+        internal static readonly DiagnosticDescriptor StaticAbstractIsPreviewFeatureRule = DiagnosticDescriptorHelper.Create(RuleId,
                                                                                                                     s_localizableTitle,
                                                                                                                     CreateLocalizableResourceString(nameof(StaticAndAbstractRequiresPreviewFeatures)),
                                                                                                                     DiagnosticCategory.Usage,
@@ -189,7 +190,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                                                                                                                     isPortedFxCopRule: false,
                                                                                                                     isDataflowRule: false);
 
-        public override ImmutableArray<DiagnosticDescriptor> SupportedDiagnostics => ImmutableArray.Create(
+        public override ImmutableArray<DiagnosticDescriptor> SupportedDiagnostics { get; } = ImmutableArray.Create(
             GeneralPreviewFeatureAttributeRule,
             ImplementsPreviewInterfaceRule,
             ImplementsPreviewMethodRule,
@@ -233,7 +234,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                     return;
                 }
 
-                if (context.Compilation.Assembly.HasAttribute(previewFeaturesAttribute))
+                if (context.Compilation.Assembly.HasAnyAttribute(previewFeaturesAttribute))
                 {
                     // This assembly has enabled preview attributes.
                     return;
@@ -304,6 +305,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                         return previewSymbol;
                     }
                 }
+
                 if (SymbolIsAnnotatedAsPreview(typeParameter, requiresPreviewFeaturesSymbols, previewFeatureAttributeSymbol))
                 {
                     return typeParameter;
@@ -417,11 +419,6 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                 }
             }
 
-            if (ProcessTypeAttributeForPreviewness(symbol, requiresPreviewFeaturesSymbols, previewFeatureAttributeSymbol, out SyntaxReference? attributeSyntaxReference, out string? attributeName, out ISymbol? previewAttributeSymbol))
-            {
-                ReportDiagnosticWithCustomMessageIfItExists(context, attributeSyntaxReference.GetSyntax(context.CancellationToken), previewAttributeSymbol, requiresPreviewFeaturesSymbols, GeneralPreviewFeatureAttributeRule, GeneralPreviewFeatureAttributeRuleWithCustomMessage, attributeName);
-            }
-
             INamedTypeSymbol? baseType = symbol.BaseType;
             if (baseType != null)
             {
@@ -440,40 +437,6 @@ namespace Microsoft.NetCore.Analyzers.Runtime
             }
         }
 
-        private static bool ProcessTypeAttributeForPreviewness(ISymbol symbol,
-                                                               ConcurrentDictionary<ISymbol, (bool isPreview, string? message, string? url)> requiresPreviewFeaturesSymbols,
-                                                               INamedTypeSymbol previewFeatureAttribute,
-                                                               [NotNullWhen(true)] out SyntaxReference? attributeSyntaxReference,
-                                                               [NotNullWhen(true)] out string? attributeName,
-                                                               [NotNullWhen(true)] out ISymbol? previewSymbol)
-        {
-            ImmutableArray<AttributeData> attributes = symbol.GetAttributes();
-            for (int i = 0; i < attributes.Length; i++)
-            {
-                AttributeData attribute = attributes[i];
-                if (SymbolIsAnnotatedAsPreview(attribute.AttributeClass, requiresPreviewFeaturesSymbols, previewFeatureAttribute))
-                {
-                    attributeName = attribute.AttributeClass.Name;
-                    attributeSyntaxReference = attribute.ApplicationSyntaxReference;
-                    previewSymbol = attribute.AttributeClass;
-                    return true;
-                }
-
-                if (SymbolIsAnnotatedAsPreview(attribute.AttributeConstructor, requiresPreviewFeaturesSymbols, previewFeatureAttribute))
-                {
-                    attributeName = attribute.AttributeClass.Name;
-                    attributeSyntaxReference = attribute.ApplicationSyntaxReference;
-                    previewSymbol = attribute.AttributeConstructor;
-                    return true;
-                }
-            }
-
-            attributeName = null;
-            attributeSyntaxReference = null;
-            previewSymbol = null;
-            return false;
-        }
-
         private SyntaxNode? GetPreviewSyntaxNodeFromSymbols(ISymbol symbol,
                                                             ISymbol previewType)
         {
@@ -615,7 +578,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
 
                 if (SymbolIsAnnotatedAsPreview(methodReturnType, requiresPreviewFeaturesSymbols, previewFeatureAttributeSymbol))
                 {
-                    SyntaxNode? returnTypeNode = GetPreviewReturnTypeSyntaxNodeForMethodOrProperty(method.IsPropertyGetter() ? method.AssociatedSymbol : method, methodReturnType);
+                    SyntaxNode? returnTypeNode = GetPreviewReturnTypeSyntaxNodeForMethodOrProperty(method.IsPropertyGetter() ? method.AssociatedSymbol! : method, methodReturnType);
                     if (returnTypeNode != null)
                     {
                         ReportDiagnosticWithCustomMessageIfItExists(context, returnTypeNode, methodReturnType, requiresPreviewFeaturesSymbols, MethodReturnsPreviewTypeRule, MethodReturnsPreviewTypeRuleWithCustomMessage, propertyOrMethodSymbol.Name, methodReturnType.Name);
@@ -631,7 +594,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                     ISymbol? innerPreviewSymbol = GetPreviewSymbolForGenericTypesFromTypeArguments(typeSymbol.TypeArguments, requiresPreviewFeaturesSymbols, previewFeatureAttributeSymbol);
                     if (innerPreviewSymbol != null)
                     {
-                        SyntaxNode? returnTypeNode = GetPreviewReturnTypeSyntaxNodeForMethodOrProperty(method.IsPropertyGetter() ? method.AssociatedSymbol : method, innerPreviewSymbol);
+                        SyntaxNode? returnTypeNode = GetPreviewReturnTypeSyntaxNodeForMethodOrProperty(method.IsPropertyGetter() ? method.AssociatedSymbol! : method, innerPreviewSymbol);
                         if (returnTypeNode != null)
                         {
                             ReportDiagnosticWithCustomMessageIfItExists(context, returnTypeNode, innerPreviewSymbol, requiresPreviewFeaturesSymbols, MethodReturnsPreviewTypeRule, MethodReturnsPreviewTypeRuleWithCustomMessage, propertyOrMethodSymbol.Name, innerPreviewSymbol.Name);
@@ -718,7 +681,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
             {
                 ProcessTypeSymbolAttributes(context, typeSymbol, requiresPreviewFeaturesSymbols, previewFeatureAttributeSymbol);
             }
-            else if (symbol is IMethodSymbol || symbol is IPropertySymbol)
+            else if (symbol is IMethodSymbol or IPropertySymbol)
             {
                 ProcessPropertyOrMethodAttributes(context, symbol, requiresPreviewFeaturesSymbols, virtualStaticsInInterfaces, previewFeatureAttributeSymbol);
             }
@@ -739,9 +702,10 @@ namespace Microsoft.NetCore.Analyzers.Runtime
             if (OperationUsesPreviewFeatures(context, requiresPreviewFeaturesSymbols, previewFeatureAttributeSymbol, out ISymbol? symbol))
             {
                 IOperation operation = context.Operation;
-                if (operation is ICatchClauseOperation catchClauseOperation)
+                if (operation is ICatchClauseOperation catchClauseOperation &&
+                    catchClauseOperation.ExceptionDeclarationOrExpression is { } exceptionOperation)
                 {
-                    operation = catchClauseOperation.ExceptionDeclarationOrExpression;
+                    operation = exceptionOperation;
                 }
 
                 ReportDiagnosticWithCustomMessageIfItExists(context, operation, symbol, requiresPreviewFeaturesSymbols, GeneralPreviewFeatureAttributeRule, GeneralPreviewFeatureAttributeRuleWithCustomMessage, symbol.Name);
@@ -863,9 +827,9 @@ namespace Microsoft.NetCore.Analyzers.Runtime
                 _ => null,
             };
 
-        private static ISymbol SymbolFromArrayCreationOperation(IArrayCreationOperation operation)
+        private static ISymbol? SymbolFromArrayCreationOperation(IArrayCreationOperation operation)
         {
-            ISymbol ret = operation.Type;
+            ISymbol? ret = operation.Type;
             while (ret is IArrayTypeSymbol arrayTypeSymbol)
             {
                 ret = arrayTypeSymbol.ElementType;
@@ -917,14 +881,16 @@ namespace Microsoft.NetCore.Analyzers.Runtime
             {
                 foreach (KeyValuePair<string, TypedConstant> namedArgument in namedArguments)
                 {
-                    if (namedArgument.Key == "Message")
+                    if (namedArgument.Key == "Message" &&
+                        namedArgument.Value.Value is string messageValue)
                     {
-                        message = (string)namedArgument.Value.Value;
+                        message = messageValue;
                     }
 
-                    if (namedArgument.Key == "Url")
+                    if (namedArgument.Key == "Url" &&
+                        namedArgument.Value.Value is string urlValue)
                     {
-                        url = (string)namedArgument.Value.Value;
+                        url = urlValue;
                     }
                 }
             }
@@ -942,7 +908,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
         {
             if (!requiresPreviewFeaturesSymbols.TryGetValue(symbol, out (bool isPreview, string? message, string? url) existing))
             {
-                Debug.Assert(true, $"Should never reach this line. This means the symbol {symbol.Name} was not processed in this analyzer");
+                Debug.Fail($"Should never reach this line. This means the symbol {symbol.Name} was not processed in this analyzer");
             }
             else
             {
@@ -969,7 +935,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
         {
             if (!requiresPreviewFeaturesSymbols.TryGetValue(previewSymbol, out (bool isPreview, string? message, string? url) existing))
             {
-                Debug.Assert(true, $"Should never reach this line. This means the symbol {previewSymbol.Name} was not processed in this analyzer");
+                Debug.Fail($"Should never reach this line. This means the symbol {previewSymbol.Name} was not processed in this analyzer");
             }
             else
             {
@@ -985,32 +951,6 @@ namespace Microsoft.NetCore.Analyzers.Runtime
             }
         }
 
-        private static void ReportDiagnosticWithCustomMessageIfItExists(SymbolAnalysisContext context,
-                                                                        SyntaxNode node,
-                                                                        ISymbol previewSymbol,
-                                                                        ConcurrentDictionary<ISymbol, (bool isPreview, string? message, string? url)> requiresPreviewFeaturesSymbols,
-                                                                        DiagnosticDescriptor diagnosticDescriptor,
-                                                                        DiagnosticDescriptor diagnosticDescriptorWithPlaceholdersForCustomMessage,
-                                                                        string diagnosticMessageArgument0)
-        {
-            if (!requiresPreviewFeaturesSymbols.TryGetValue(previewSymbol, out (bool isPreview, string? message, string? url) existing))
-            {
-                Debug.Assert(true, $"Should never reach this line. This means the symbol {previewSymbol.Name} was not processed in this analyzer");
-            }
-            else
-            {
-                string url = existing.url ?? DefaultURL;
-                if (existing.message is string customMessage)
-                {
-                    context.ReportDiagnostic(node.CreateDiagnostic(diagnosticDescriptorWithPlaceholdersForCustomMessage, diagnosticMessageArgument0, url, customMessage));
-                }
-                else
-                {
-                    context.ReportDiagnostic(node.CreateDiagnostic(diagnosticDescriptor, diagnosticMessageArgument0, url));
-                }
-            }
-        }
-
         private static void ReportDiagnosticWithCustomMessageIfItExists(SymbolAnalysisContext context,
                                                                         ISymbol previewSymbol,
                                                                         ISymbol symbolToRaiseDiagnosticOn,
@@ -1022,7 +962,7 @@ namespace Microsoft.NetCore.Analyzers.Runtime
         {
             if (!requiresPreviewFeaturesSymbols.TryGetValue(previewSymbol, out (bool isPreview, string? message, string? url) existing))
             {
-                Debug.Assert(true, $"Should never reach this line. This means the symbol {previewSymbol.Name} was not processed in this analyzer");
+                Debug.Fail($"Should never reach this line. This means the symbol {previewSymbol.Name} was not processed in this analyzer");
             }
             else
             {
@@ -1049,15 +989,11 @@ namespace Microsoft.NetCore.Analyzers.Runtime
 
             if (!requiresPreviewFeaturesSymbols.TryGetValue(symbol, out (bool isPreview, string? message, string? url) existing))
             {
-                ImmutableArray<AttributeData> attributes = symbol.GetAttributes();
-                foreach (var attribute in attributes)
+                if (symbol.GetAttribute(previewFeatureAttribute) is { } attribute)
                 {
-                    if (attribute.AttributeClass.Equals(previewFeatureAttribute))
-                    {
-                        string? message = GetMessageAndURLFromAttributeConstructor(attribute, out string? url);
-                        requiresPreviewFeaturesSymbols.GetOrAdd(symbol, new ValueTuple<bool, string?, string?>(true, message, url));
-                        return true;
-                    }
+                    string? message = GetMessageAndURLFromAttributeConstructor(attribute, out string? url);
+                    requiresPreviewFeaturesSymbols.GetOrAdd(symbol, new ValueTuple<bool, string?, string?>(true, message, url));
+                    return true;
                 }
 
                 ISymbol? parent = symbol.ContainingSymbol;
