@@ -18,6 +18,7 @@ namespace Microsoft.NetFramework.Analyzers
     using static MicrosoftNetFrameworkAnalyzersResources;
 
     /// <summary>
+    /// CA3075: <inheritdoc cref="InsecureXmlDtdProcessing"/>
     /// Secure DTD processing and entity resolution in XML
     /// </summary>
     [DiagnosticAnalyzer(LanguageNames.CSharp, LanguageNames.VisualBasic)]
@@ -163,9 +164,6 @@ namespace Microsoft.NetFramework.Analyzers
                 internal bool IsSecureResolver { get; set; }
                 internal bool IsConstructedInCodeBlock { get; set; }
 
-                // this constructor is used for keep track of XmlReaderSettings not created in the code block
-                internal XmlReaderSettingsEnvironment() { }
-
                 // this constructor is used for keep track of XmlReaderSettings craeted in the code block
                 internal XmlReaderSettingsEnvironment(bool isTargetFrameworkSecure)
                 {
@@ -236,6 +234,7 @@ namespace Microsoft.NetFramework.Analyzers
                         {
                             AnalyzeObjectCreationInternal(context, field, fieldInitializer.Value);
                         }
+
                         break;
                     case OperationKind.VariableDeclarator:
                         var declarator = (IVariableDeclaratorOperation)context.Operation;
