@@ -5,6 +5,7 @@
 using System;
 using System.Collections.Immutable;
 using System.Linq;
+using Analyzer.Utilities.Lightup;
 using Microsoft.CodeAnalysis;
 using Microsoft.CodeAnalysis.Operations;
 
@@ -99,7 +100,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
             ImmutableHashSet<object>.Builder? distinctReferencedConstantsBuilder = null;
 
             // Explicit user applied attribute.
-            if (operationBlock.Kind == OperationKind.None &&
+            if ((operationBlock.Kind is OperationKind.None or OperationKindEx.Attribute) &&
                 hasAnyExplicitExpression(operationBlock))
             {
                 executableLinesOfCode += 1;
@@ -154,6 +155,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                             countOperator(operation);
                             countOperand(field);
                         }
+
                         continue;
                     case OperationKind.PropertyInitializer:
                         foreach (var property in ((IPropertyInitializerOperation)operation).InitializedProperties)
@@ -161,6 +163,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                             countOperator(operation);
                             countOperand(property);
                         }
+
                         continue;
                     case OperationKind.ParameterInitializer:
                         countOperator(operation);
@@ -176,6 +179,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                         {
                             countOperand(variableDeclarator.Symbol);
                         }
+
                         continue;
 
                     // Invocations and Object creations.
@@ -186,6 +190,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                         {
                             countOperand(invocation.TargetMethod);
                         }
+
                         continue;
                     case OperationKind.ObjectCreation:
                         countOperator(operation);
@@ -224,6 +229,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                         {
                             countOperator(operation);
                         }
+
                         continue;
 
                     // Other common operators.
@@ -270,6 +276,7 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                         {
                             countOperator(operation);
                         }
+
                         continue;
                 }
             }
@@ -318,8 +325,8 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
 
             static bool hasAnyExplicitExpression(IOperation operation)
             {
-                // Check if all descendants are either implicit or are explicit non-branch operations with no constant value or type, indicating it is not user written code.
-                return !operation.DescendantsAndSelf().All(o => o.IsImplicit || (!o.ConstantValue.HasValue && o.Type == null && o.Kind != OperationKind.Branch));
+                // Check if all descendants are either implicit or are explicit non-branch, non-attribute operations with no constant value or type, indicating it is not user written code.
+                return !operation.DescendantsAndSelf().All(o => o.IsImplicit || (!o.ConstantValue.HasValue && o.Type == null && o.Kind is not (OperationKind.Branch or OperationKindEx.Attribute)));
             }
 
             void countOperator(IOperation operation)
@@ -391,10 +398,12 @@ namespace Microsoft.CodeAnalysis.CodeMetrics
                 {
                     count += _distinctBinaryOperatorKinds.Count - 1;
                 }
+
                 if (_distinctUnaryOperatorKinds.Count > 1)
                 {
                     count += _distinctUnaryOperatorKinds.Count - 1;
                 }
+
                 if (_distinctCaseKinds.Count > 1)
                 {
                     count += _distinctCaseKinds.Count - 1;
