@@ -5,7 +5,6 @@
 using System;
 using System.Collections.Immutable;
 using System.Diagnostics.CodeAnalysis;
-using System.IO;
 using System.Linq;
 using System.Runtime.CompilerServices;
 using Analyzer.Utilities.Extensions;
@@ -128,7 +127,7 @@ namespace Analyzer.Utilities
             var analyzerConfigOptions = options.GetOrComputeCategorizedAnalyzerConfigOptions(compilation);
             return analyzerConfigOptions.GetOptionValue(
                 optionName, tree, rule,
-                tryParseValue: (string value, out TEnum result) => Enum.TryParse(value, ignoreCase: true, result: out result),
+                tryParseValue: static (string value, out TEnum result) => Enum.TryParse(value, ignoreCase: true, result: out result),
                 defaultValue: defaultValue);
         }
 
@@ -159,9 +158,7 @@ namespace Analyzer.Utilities
             }
         }
 
-#pragma warning disable IDE0051 // Remove unused private members - Used in some projects that include this shared project.
         private static TEnum GetNonFlagsEnumOptionValue<TEnum>(
-#pragma warning restore IDE0051 // Remove unused private members
             this AnalyzerOptions options,
             string optionName,
             DiagnosticDescriptor rule,
@@ -169,13 +166,7 @@ namespace Analyzer.Utilities
             Compilation compilation,
             TEnum defaultValue)
             where TEnum : struct
-        {
-            var analyzerConfigOptions = options.GetOrComputeCategorizedAnalyzerConfigOptions(compilation);
-            return analyzerConfigOptions.GetOptionValue(
-                optionName, tree, rule,
-                tryParseValue: (string value, out TEnum result) => Enum.TryParse(value, ignoreCase: true, result: out result),
-                defaultValue: defaultValue);
-        }
+            => GetFlagsEnumOptionValue(options, optionName, rule, tree, compilation, defaultValue);
 
         public static bool GetBoolOptionValue(
             this AnalyzerOptions options,
@@ -191,7 +182,7 @@ namespace Analyzer.Utilities
         public static bool GetBoolOptionValue(
             this AnalyzerOptions options,
             string optionName,
-            DiagnosticDescriptor rule,
+            DiagnosticDescriptor? rule,
             SyntaxTree tree,
             Compilation compilation,
             bool defaultValue)
@@ -430,6 +421,20 @@ namespace Analyzer.Utilities
             Compilation compilation)
             => options.GetSymbolNamesWithValueOption<Unit>(EditorConfigOptionNames.AdditionalUseResultsMethods, rule, tree, compilation, static name => new SymbolNamesWithValueOption<Unit>.NameParts(name, Unit.Default), namePrefix: "M:");
 
+        public static SymbolNamesWithValueOption<Unit> GetEnumerationMethodsOption(
+            this AnalyzerOptions options,
+            DiagnosticDescriptor rule,
+            SyntaxTree tree,
+            Compilation compilation)
+            => options.GetSymbolNamesWithValueOption<Unit>(EditorConfigOptionNames.EnumerationMethods, rule, tree, compilation, static name => new SymbolNamesWithValueOption<Unit>.NameParts(name, Unit.Default), namePrefix: "M:");
+
+        public static SymbolNamesWithValueOption<Unit> GetLinqChainMethodsOption(
+            this AnalyzerOptions options,
+            DiagnosticDescriptor rule,
+            SyntaxTree tree,
+            Compilation compilation)
+            => options.GetSymbolNamesWithValueOption<Unit>(EditorConfigOptionNames.LinqChainMethods, rule, tree, compilation, static name => new SymbolNamesWithValueOption<Unit>.NameParts(name, Unit.Default), namePrefix: "M:");
+
         private static SymbolNamesWithValueOption<TValue> GetSymbolNamesWithValueOption<TValue>(
             this AnalyzerOptions options,
             string optionName,
@@ -442,17 +447,23 @@ namespace Analyzer.Utilities
             string? optionForcedValue = null)
         {
             var analyzerConfigOptions = options.GetOrComputeCategorizedAnalyzerConfigOptions(compilation);
-            return analyzerConfigOptions.GetOptionValue(optionName, tree, rule, TryParse, defaultValue: GetDefaultValue());
+            return analyzerConfigOptions.GetOptionValue(
+                optionName,
+                tree,
+                rule,
+                TryParse,
+                (compilation, getTypeAndSuffixFunc, namePrefix, optionForcedValue),
+                defaultValue: GetDefaultValue());
 
             // Local functions.
-            bool TryParse(string s, out SymbolNamesWithValueOption<TValue> option)
+            static bool TryParse(string s, (Compilation compilation, Func<string, SymbolNamesWithValueOption<TValue>.NameParts> getTypeAndSuffixFunc, string? namePrefix, string? optionForcedValue) arg, out SymbolNamesWithValueOption<TValue> option)
             {
                 var optionValue = s;
 
-                if (!RoslynString.IsNullOrEmpty(optionForcedValue) &&
-                    (optionValue == null || !optionValue.Contains(optionForcedValue, StringComparison.Ordinal)))
+                if (!RoslynString.IsNullOrEmpty(arg.optionForcedValue) &&
+                    (optionValue == null || !optionValue.Contains(arg.optionForcedValue, StringComparison.Ordinal)))
                 {
-                    optionValue = $"{optionForcedValue}|{optionValue}";
+                    optionValue = $"{arg.optionForcedValue}|{optionValue}";
                 }
 
                 if (string.IsNullOrEmpty(optionValue))
@@ -462,7 +473,7 @@ namespace Analyzer.Utilities
                 }
 
                 var names = optionValue.Split(new[] { '|' }, StringSplitOptions.RemoveEmptyEntries).ToImmutableArray();
-                option = SymbolNamesWithValueOption<TValue>.Create(names, compilation, namePrefix, getTypeAndSuffixFunc);
+                option = SymbolNamesWithValueOption<TValue>.Create(names, arg.compilation, arg.namePrefix, arg.getTypeAndSuffixFunc);
                 return true;
             }
 
@@ -484,7 +495,7 @@ namespace Analyzer.Utilities
 
                 RoslynDebug.Assert(optionValue != null);
 
-                return TryParse(optionValue, out var option)
+                return TryParse(optionValue, (compilation, getTypeAndSuffixFunc, namePrefix, optionForcedValue), out var option)
                     ? option
                     : SymbolNamesWithValueOption<TValue>.Empty;
             }
@@ -506,7 +517,11 @@ namespace Analyzer.Utilities
 
             var analyzerConfigOptions = options.GetOrComputeCategorizedAnalyzerConfigOptions(compilation);
             return analyzerConfigOptions.GetOptionValue(optionName, tree, rule: null,
-                tryParseValue: (string value, out string? result) => { result = value; return true; },
+                tryParseValue: static (string value, out string? result) =>
+                {
+                    result = value;
+                    return true;
+                },
                 defaultValue: null, OptionKind.BuildProperty);
         }
 
@@ -527,7 +542,11 @@ namespace Analyzer.Utilities
             var propertyOptionName = MSBuildItemOptionNamesHelpers.GetPropertyNameForItemOptionName(itemOptionName);
             var analyzerConfigOptions = options.GetOrComputeCategorizedAnalyzerConfigOptions(compilation);
             var propertyValue = analyzerConfigOptions.GetOptionValue(propertyOptionName, tree, rule: null,
-                tryParseValue: (string value, out string? result) => { result = value; return true; },
+                tryParseValue: static (string value, out string? result) =>
+                {
+                    result = value;
+                    return true;
+                },
                 defaultValue: null, OptionKind.BuildProperty);
             return MSBuildItemOptionNamesHelpers.ParseItemOptionValue(propertyValue);
         }
@@ -587,21 +606,24 @@ namespace Analyzer.Utilities
         private static ICategorizedAnalyzerConfigOptions GetOrComputeCategorizedAnalyzerConfigOptions(
             this AnalyzerOptions options, Compilation compilation)
         {
-            if (options.AdditionalFiles.Any(f => Path.GetFileName(f.Path).Equals(".editorconfig", StringComparison.OrdinalIgnoreCase)))
-            {
-                throw new InvalidOperationException("Passing '.editorconfig' files as additional files is no longer needed. It will be implicitly discovered (if the file is in the project's directory or any ancestor directory), or it should be converted into a 'globalconfig'. See 'https://docs.microsoft.com/dotnet/fundamentals/code-analysis/configuration-files'.");
-            }
-
             // TryGetValue upfront to avoid allocating createValueCallback if the entry already exists.
             if (s_cachedOptions.TryGetValue(options, out var categorizedAnalyzerConfigOptions))
             {
                 return categorizedAnalyzerConfigOptions;
             }
 
-            var createValueCallback = new ConditionalWeakTable<AnalyzerOptions, ICategorizedAnalyzerConfigOptions>.CreateValueCallback(_ =>
-                AggregateCategorizedAnalyzerConfigOptions.Create(options.AnalyzerConfigOptionsProvider, compilation)
-            );
-            return s_cachedOptions.GetValue(options, createValueCallback);
+            // Fall back to a slow version of the method, which allocates both for the CreateValueCallback and for
+            // capturing the Compilation argument.
+            return GetOrComputeCategorizedAnalyzerConfigOptions_Slow(options, compilation);
+
+            static ICategorizedAnalyzerConfigOptions GetOrComputeCategorizedAnalyzerConfigOptions_Slow(
+                AnalyzerOptions options,
+                Compilation compilation)
+            {
+                return s_cachedOptions.GetValue(
+                    options,
+                    options => AggregateCategorizedAnalyzerConfigOptions.Create(options.AnalyzerConfigOptionsProvider, compilation));
+            }
         }
     }
 }
