@@ -85,7 +85,7 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
 
             public ImmutableHashSet<IFlowAnonymousFunctionOperation> VisitedLambdas => this._visitedLambdas.ToImmutable();
 
-            protected override PropertySetAbstractValue GetAbstractDefaultValue(ITypeSymbol type) => ValueDomain.Bottom;
+            protected override PropertySetAbstractValue GetAbstractDefaultValue(ITypeSymbol? type) => ValueDomain.Bottom;
 
             protected override bool HasAnyAbstractValue(PropertySetAnalysisData data) => data.Count > 0;
 
@@ -143,6 +143,9 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
             public override PropertySetAbstractValue VisitObjectCreation(IObjectCreationOperation operation, object? argument)
             {
                 PropertySetAbstractValue abstractValue = base.VisitObjectCreation(operation, argument);
+                if (operation.Type == null || operation.Constructor == null)
+                    return abstractValue;
+
                 if (this.TrackedTypeSymbols.Any(s => operation.Type.GetBaseTypesAndThis().Contains(s)))
                 {
                     ConstructorMapper constructorMapper = this.DataFlowAnalysisContext.ConstructorMapper;
@@ -208,6 +211,8 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
             protected override PropertySetAbstractValue VisitAssignmentOperation(IAssignmentOperation operation, object? argument)
             {
                 PropertySetAbstractValue? baseValue = base.VisitAssignmentOperation(operation, argument);
+                if (operation.Target.Type == null)
+                    return baseValue;
 
                 // If we need to evaluate hazardous usages on initializations, track assignments of properties and fields, so
                 // at the end of the CFG we can figure out which assignment operations to flag.
@@ -272,7 +277,7 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                 }
 
                 if (propertyReferenceOperation != null
-                    && propertyReferenceOperation.Instance != null
+                    && propertyReferenceOperation.Instance?.Type != null
                     && this.TrackedTypeSymbols.Any(s => propertyReferenceOperation.Instance.Type.GetBaseTypesAndThis().Contains(s))
                     && this.DataFlowAnalysisContext.PropertyMappers.TryGetPropertyMapper(
                         propertyReferenceOperation.Property.Name,
@@ -343,13 +348,13 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                 try
                 {
                     this.DataFlowAnalysisContext.HazardousUsageEvaluators.TryGetInitializationHazardousUsageEvaluator(
-                        out HazardousUsageEvaluator initializationHazardousUsageEvaluator);
+                        out var initializationHazardousUsageEvaluator);
 
                     foreach (KeyValuePair<AnalysisEntity, TrackedAssignmentData> kvp
                         in this.TrackedFieldPropertyAssignments)
                     {
                         if (!this.DataFlowAnalysisContext.PointsToAnalysisResult!.ExitBlockOutput.Data.TryGetValue(
-                                kvp.Key, out PointsToAbstractValue pointsToAbstractValue))
+                                kvp.Key, out var pointsToAbstractValue))
                         {
                             continue;
                         }
@@ -375,21 +380,21 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
 
                                     if (!exitBlockOutput.Data.TryGetValue(
                                             abstractLocation,
-                                            out PropertySetAbstractValue propertySetAbstractValue))
+                                            out var propertySetAbstractValue))
                                     {
                                         propertySetAbstractValue = PropertySetAbstractValue.Unknown;
                                     }
 
-                                    HazardousUsageEvaluationResult result =
-                                        initializationHazardousUsageEvaluator.ValueEvaluator!(propertySetAbstractValue);
-                                    if (result != HazardousUsageEvaluationResult.Unflagged)
+                                    HazardousUsageEvaluationResult? result =
+                                        initializationHazardousUsageEvaluator?.ValueEvaluator!(propertySetAbstractValue);
+                                    if (result is not null and not HazardousUsageEvaluationResult.Unflagged)
                                     {
                                         foreach (IAssignmentOperation assignmentOperation in assignments)
                                         {
                                             this.MergeHazardousUsageResult(
                                                 assignmentOperation.Syntax,
                                                 methodSymbol: null,    // No method invocation; just evaluating initialization value.
-                                                result);
+                                                result.Value);
                                         }
                                     }
                                 }
@@ -404,16 +409,16 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                         {
                             if (kvp.Value.AssignmentsWithUnknownLocation != null)
                             {
-                                HazardousUsageEvaluationResult result =
-                                    initializationHazardousUsageEvaluator.ValueEvaluator!(PropertySetAbstractValue.Unknown);
-                                if (result != HazardousUsageEvaluationResult.Unflagged)
+                                HazardousUsageEvaluationResult? result =
+                                    initializationHazardousUsageEvaluator?.ValueEvaluator!(PropertySetAbstractValue.Unknown);
+                                if (result is not null and not HazardousUsageEvaluationResult.Unflagged)
                                 {
                                     foreach (IAssignmentOperation assignmentOperation in kvp.Value.AssignmentsWithUnknownLocation)
                                     {
                                         this.MergeHazardousUsageResult(
                                             assignmentOperation.Syntax,
                                             methodSymbol: null,    // No method invocation; just evaluating initialization value.
-                                            result);
+                                            result.Value);
                                     }
                                 }
                             }
@@ -449,11 +454,11 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                 PropertySetAbstractValue baseValue = base.VisitInvocation_NonLambdaOrDelegateOrLocalFunction(method, visitedInstance, visitedArguments, invokedAsDelegate, originalOperation, defaultValue);
 
                 if (this.DataFlowAnalysisContext.HazardousUsageEvaluators.TryGetArgumentHazardousUsageEvaluator(
-                            out HazardousUsageEvaluator argumentHazardousUsageEvaluator))
+                            out var argumentHazardousUsageEvaluator))
                 {
                     foreach (IArgumentOperation visitedArgument in visitedArguments)
                     {
-                        if (visitedArgument.Value != null && this.TrackedTypeSymbols.Any(s => visitedArgument.Value.Type.GetBaseTypesAndThis().Contains(s)))
+                        if (visitedArgument.Value?.Type != null && this.TrackedTypeSymbols.Any(s => visitedArgument.Value.Type.GetBaseTypesAndThis().Contains(s)))
                         {
                             this.EvaluatePotentialHazardousUsage(
                                 visitedArgument.Value.Syntax,
@@ -467,11 +472,11 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                 // If we have a HazardousUsageEvaluator for a method within the tracked type,
                 // or for a method within a different type.
                 IOperation? propertySetInstance = visitedInstance;
-                if ((visitedInstance != null
+                if ((visitedInstance?.Type != null
                         && this.TrackedTypeSymbols.Any(s => visitedInstance.Type.GetBaseTypesAndThis().Contains(s))
                         && this.DataFlowAnalysisContext.HazardousUsageEvaluators.TryGetHazardousUsageEvaluator(
                                method.MetadataName,
-                               out HazardousUsageEvaluator? hazardousUsageEvaluator))
+                               out var hazardousUsageEvaluator))
                     || TryFindNonTrackedTypeHazardousUsageEvaluator(
                            method,
                            visitedArguments,
@@ -483,7 +488,7 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                         originalOperation.Syntax,
                         method,
                         propertySetInstance,
-                        (PropertySetAbstractValue abstractValue) => hazardousUsageEvaluator.InvocationEvaluator!(method, abstractValue));
+                        (PropertySetAbstractValue abstractValue) => hazardousUsageEvaluator!.InvocationEvaluator!(method, abstractValue));
                 }
                 else
                 {
@@ -521,7 +526,10 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                     foreach (IArgumentOperation argumentOperation in visitedArguments)
                     {
                         IOperation value = argumentOperation.Value;
-                        ITypeSymbol argumentTypeSymbol = value is IConversionOperation conversionOperation ? conversionOperation.Operand.Type : value.Type;
+                        ITypeSymbol? argumentTypeSymbol = value is IConversionOperation conversionOperation ? conversionOperation.Operand.Type : value.Type;
+                        if (argumentTypeSymbol == null || argumentOperation.Parameter == null)
+                            continue;
+
                         foreach (string hazardousUsageTypeName in hazardousUsageTypeNames)
                         {
                             if (this.TrackedTypeSymbols.Any(s => argumentTypeSymbol.GetBaseTypesAndThis().Contains(s))
@@ -550,7 +558,7 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                 hazardousUsageTypeNames = null;
                 if (this.DataFlowAnalysisContext.HazardousUsageTypesToNames.TryGetValue(
                         (containingType, false),
-                        out string containingTypeName))
+                        out var containingTypeName))
                 {
                     hazardousUsageTypeNames = PooledHashSet<string>.GetInstance();
                     hazardousUsageTypeNames.Add(containingTypeName);
@@ -562,10 +570,7 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                         (type, true),
                         out containingTypeName))
                     {
-                        if (hazardousUsageTypeNames == null)
-                        {
-                            hazardousUsageTypeNames = PooledHashSet<string>.GetInstance();
-                        }
+                        hazardousUsageTypeNames ??= PooledHashSet<string>.GetInstance();
 
                         hazardousUsageTypeNames.Add(containingTypeName);
                     }
@@ -589,10 +594,7 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
                 Func<PropertySetAbstractValue, HazardousUsageEvaluationResult> evaluationFunction,
                 Func<AbstractLocation, PropertySetAbstractValue>? locationToAbstractValueMapping = null)
             {
-                if (locationToAbstractValueMapping == null)
-                {
-                    locationToAbstractValueMapping = this.GetAbstractValue;
-                }
+                locationToAbstractValueMapping ??= this.GetAbstractValue;
 
                 PointsToAbstractValue pointsToAbstractValue = this.GetPointsToAbstractValue(propertySetInstance);
                 HazardousUsageEvaluationResult result = HazardousUsageEvaluationResult.Unflagged;
@@ -646,10 +648,10 @@ namespace Analyzer.Utilities.FlowAnalysis.Analysis.PropertySetAnalysis
             {
                 base.ProcessReturnValue(returnValue);
 
-                if (returnValue != null
+                if (returnValue?.Type != null
                     && this.TrackedTypeSymbols.Any(s => returnValue.Type.GetBaseTypesAndThis().Contains(s))
                     && this.DataFlowAnalysisContext.HazardousUsageEvaluators.TryGetReturnHazardousUsageEvaluator(
-                        out HazardousUsageEvaluator hazardousUsageEvaluator))
+                        out var hazardousUsageEvaluator))
                 {
                     this.EvaluatePotentialHazardousUsage(
                         returnValue.Syntax,
