@@ -13,4 +13,4 @@ while [[ -h $source ]]; do
 done
 
 scriptroot="$( cd -P "$( dirname "$source" )" && pwd )"
-"$scriptroot/eng/build.sh" --restore $@
\ No newline at end of file
+"$scriptroot/eng/build.sh" --restore --solution  Roslyn.sln $@
\ No newline at end of file
