@@ -3,9 +3,9 @@
 // See the LICENSE file in the project root for more information.
 
 using System.Reflection;
-using Microsoft.CodeAnalysis.CSharp.LanguageServices;
+using Microsoft.CodeAnalysis.CSharp.LanguageService;
 using Microsoft.CodeAnalysis.Diagnostics;
-using Microsoft.CodeAnalysis.LanguageServices;
+using Microsoft.CodeAnalysis.LanguageService;
 using Microsoft.CodeAnalysis.RemoveUnnecessarySuppressions;
 
 namespace Microsoft.CodeAnalysis.CSharp.RemoveUnnecessarySuppressions
@@ -22,5 +22,7 @@ namespace Microsoft.CodeAnalysis.CSharp.RemoveUnnecessarySuppressions
         protected override ISemanticFacts SemanticFacts => CSharpSemanticFacts.Instance;
         protected override (Assembly assembly, string typeName) GetCompilerDiagnosticAnalyzerInfo()
             => (typeof(SyntaxKind).Assembly, CompilerDiagnosticAnalyzerNames.CSharpCompilerAnalyzerTypeName);
+        protected override bool ContainsPragmaDirective(SyntaxNode root)
+            => root.ContainsDirective(SyntaxKind.PragmaWarningDirectiveTrivia);
     }
 }
