@@ -3,7 +3,9 @@
 // See the LICENSE file in the project root for more information.
 
 using System.Threading.Tasks;
+using Microsoft.CodeAnalysis.Formatting;
 using Microsoft.CodeAnalysis.Testing;
+using Roslyn.Test.Utilities;
 using Xunit;
 using VerifyCS = Microsoft.CodeAnalysis.Editor.UnitTests.CodeActions.CSharpCodeFixVerifier<
     Microsoft.CodeAnalysis.CSharp.FileHeaders.CSharpFileHeaderDiagnosticAnalyzer,
@@ -13,15 +15,15 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.FileHeaders
 {
     public class FileHeaderTests
     {
-        private const string TestSettings = @"
-[*.cs]
-file_header_template = Copyright (c) SomeCorp. All rights reserved.\nLicensed under the ??? license. See LICENSE file in the project root for full license information.
-";
+        private const string TestSettings = """
+            [*.cs]
+            file_header_template = Copyright (c) SomeCorp. All rights reserved.\nLicensed under the ??? license. See LICENSE file in the project root for full license information.
+            """;
 
-        private const string TestSettingsWithEmptyLines = @"
-[*.cs]
-file_header_template = \nCopyright (c) SomeCorp. All rights reserved.\n\nLicensed under the ??? license. See LICENSE file in the project root for full license information.\n
-";
+        private const string TestSettingsWithEmptyLines = """
+            [*.cs]
+            file_header_template = \nCopyright (c) SomeCorp. All rights reserved.\n\nLicensed under the ??? license. See LICENSE file in the project root for full license information.\n
+            """;
 
         /// <summary>
         /// Verifies that the analyzer will not report a diagnostic when the file header is not configured.
@@ -33,10 +35,11 @@ file_header_template = \nCopyright (c) SomeCorp. All rights reserved.\n\nLicense
         [InlineData("file_header_template = unset")]
         public async Task TestFileHeaderNotConfiguredAsync(string fileHeaderTemplate)
         {
-            var testCode = @"namespace N
-{
-}
-";
+            var testCode = """
+                namespace N
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -53,26 +56,35 @@ file_header_template = \nCopyright (c) SomeCorp. All rights reserved.\n\nLicense
         /// Verifies that the analyzer will report a diagnostic when the file is completely missing a header.
         /// </summary>
         /// <returns>A <see cref="Task"/> representing the asynchronous unit test.</returns>
-        [Fact]
-        public async Task TestNoFileHeaderAsync()
+        [Theory]
+        [InlineData("\n")]
+        [InlineData("\r\n")]
+        [WorkItem("https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1414432")]
+        public async Task TestNoFileHeaderAsync(string lineEnding)
         {
-            var testCode = @"[||]namespace N
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                [||]namespace N
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace N
-{
-}
-";
+                namespace N
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
-                TestCode = testCode,
-                FixedCode = fixedCode,
+                TestCode = testCode.ReplaceLineEndings(lineEnding),
+                FixedCode = fixedCode.ReplaceLineEndings(lineEnding),
                 EditorConfig = TestSettings,
+                Options =
+                {
+                    { FormattingOptions2.NewLine, lineEnding },
+                },
             }.RunAsync();
         }
 
@@ -83,21 +95,23 @@ namespace N
         [Fact]
         public async Task TestNoFileHeaderWithUsingDirectiveAsync()
         {
-            var testCode = @"[||]using System;
+            var testCode = """
+                [||]using System;
 
-namespace N
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                namespace N
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-using System;
+                using System;
 
-namespace N
-{
-}
-";
+                namespace N
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -114,22 +128,24 @@ namespace N
         [Fact]
         public async Task TestNoFileHeaderWithBlankLineAndUsingDirectiveAsync()
         {
-            var testCode = @"[||]
-using System;
+            var testCode = """
+                [||]
+                using System;
 
-namespace N
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                namespace N
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-using System;
+                using System;
 
-namespace N
-{
-}
-";
+                namespace N
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -146,22 +162,24 @@ namespace N
         [Fact]
         public async Task TestNoFileHeaderWithWhitespaceLineAsync()
         {
-            var testCode = "[||]    " + @"
-using System;
+            var testCode = "[||]    " + """
 
-namespace N
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                using System;
 
-using System;
+                namespace N
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace N
-{
-}
-";
+                using System;
+
+                namespace N
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -178,22 +196,24 @@ namespace N
         [Fact]
         public async Task TestFileNameBuiltInVariableAsync()
         {
-            var editorConfig = @"
-[*.cs]
-file_header_template = {fileName} Copyright (c) SomeCorp. All rights reserved.\nLicensed under the ??? license. See LICENSE file in the project root for full license information.
-";
+            var editorConfig = """
+                [*.cs]
+                file_header_template = {fileName} Copyright (c) SomeCorp. All rights reserved.\nLicensed under the ??? license. See LICENSE file in the project root for full license information.
+                """;
 
-            var testCode = @"[||]namespace N
-{
-}
-";
-            var fixedCode = @"// Test0.cs Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                [||]namespace N
+                {
+                }
+                """;
+            var fixedCode = """
+                // Test0.cs Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace N
-{
-}
-";
+                namespace N
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -210,13 +230,14 @@ namespace N
         [Fact]
         public async Task TestValidFileHeaderWithSingleLineCommentsAsync()
         {
-            var testCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -233,14 +254,15 @@ namespace Bar
         [Fact]
         public async Task TestValidFileHeaderWithMultiLineComments1Async()
         {
-            var testCode = @"/* Copyright (c) SomeCorp. All rights reserved.
- * Licensed under the ??? license. See LICENSE file in the project root for full license information.
- */
+            var testCode = """
+                /* Copyright (c) SomeCorp. All rights reserved.
+                 * Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                 */
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -257,13 +279,14 @@ namespace Bar
         [Fact]
         public async Task TestValidFileHeaderWithMultiLineComments2Async()
         {
-            var testCode = @"/* Copyright (c) SomeCorp. All rights reserved.
-   Licensed under the ??? license. See LICENSE file in the project root for full license information. */
+            var testCode = """
+                /* Copyright (c) SomeCorp. All rights reserved.
+                   Licensed under the ??? license. See LICENSE file in the project root for full license information. */
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -281,9 +304,10 @@ namespace Bar
         [Fact]
         public async Task TestValidFileHeaderWithMultiLineComments3Async()
         {
-            var testCode = @"/* Copyright (c) SomeCorp. All rights reserved.
-   Licensed under the ??? license. See LICENSE file in the project root for full license information.
-";
+            var testCode = """
+                /* Copyright (c) SomeCorp. All rights reserved.
+                   Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                """;
 
             await new VerifyCS.Test
             {
@@ -312,15 +336,15 @@ namespace Bar
 
 namespace Bar
 {{
-}}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+}}";
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -337,20 +361,22 @@ namespace Bar
         [Fact]
         public async Task TestInvalidFileHeaderWithWrongTextAsync()
         {
-            var testCode = @"[|//|] Copyright (c) OtherCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                [|//|] Copyright (c) OtherCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace Bar
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                namespace Bar
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -367,25 +393,27 @@ namespace Bar
         [Fact]
         public async Task TestInvalidFileHeaderWithWrongText2Async()
         {
-            var testCode = @"[|/*|] Copyright (c) OtherCorp. All rights reserved.
- * Licensed under the ??? license. See LICENSE file in the project root for full license information.
- */
+            var testCode = """
+                [|/*|] Copyright (c) OtherCorp. All rights reserved.
+                 * Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                 */
 
-namespace Bar
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                namespace Bar
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-/* Copyright (c) OtherCorp. All rights reserved.
- * Licensed under the ??? license. See LICENSE file in the project root for full license information.
- */
+                /* Copyright (c) OtherCorp. All rights reserved.
+                 * Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                 */
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -462,15 +490,17 @@ namespace Bar
         [Fact]
         public async Task TestInvalidFileHeaderWithWrongTextInUnterminatedMultiLineComment1Async()
         {
-            var testCode = @"{|CS1035:|}[|/*|] Copyright (c) OtherCorp. All rights reserved.
- * Licensed under the ??? license. See LICENSE file in the project root for full license information.
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                {|CS1035:|}[|/*|] Copyright (c) OtherCorp. All rights reserved.
+                 * Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-{|CS1035:|}/* Copyright (c) OtherCorp. All rights reserved.
- * Licensed under the ??? license. See LICENSE file in the project root for full license information.
-";
+                {|CS1035:|}/* Copyright (c) OtherCorp. All rights reserved.
+                 * Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                """;
 
             await new VerifyCS.Test
             {
@@ -487,13 +517,15 @@ namespace Bar
         [Fact]
         public async Task TestInvalidFileHeaderWithWrongTextInUnterminatedMultiLineComment2Async()
         {
-            var testCode = @"{|CS1035:|}[|/*|]/
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                {|CS1035:|}[|/*|]/
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-{|CS1035:|}/*/
-";
+                {|CS1035:|}/*/
+                """;
 
             await new VerifyCS.Test
             {
@@ -518,15 +550,15 @@ namespace Bar
 
 namespace Bar
 {{
-}}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+}}";
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -543,24 +575,26 @@ namespace Bar
         [Fact]
         public async Task TestInvalidFileHeaderWithWrongTextFollowedByCommentAsync()
         {
-            var testCode = @"[|//|] Copyright (c) OtherCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                [|//|] Copyright (c) OtherCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-//using System;
+                //using System;
 
-namespace Bar
-{
-}
-";
-            var fixedCode = @"// Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                namespace Bar
+                {
+                }
+                """;
+            var fixedCode = """
+                // Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-//using System;
+                //using System;
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
@@ -573,23 +607,25 @@ namespace Bar
         [Fact]
         public async Task TestHeaderMissingRequiredNewLinesAsync()
         {
-            var testCode = @"[|//|] Copyright (c) SomeCorp. All rights reserved.
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
+            var testCode = """
+                [|//|] Copyright (c) SomeCorp. All rights reserved.
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
 
-namespace Bar
-{
-}
-";
-            var fixedCode = @"//
-// Copyright (c) SomeCorp. All rights reserved.
-//
-// Licensed under the ??? license. See LICENSE file in the project root for full license information.
-//
+                namespace Bar
+                {
+                }
+                """;
+            var fixedCode = """
+                //
+                // Copyright (c) SomeCorp. All rights reserved.
+                //
+                // Licensed under the ??? license. See LICENSE file in the project root for full license information.
+                //
 
-namespace Bar
-{
-}
-";
+                namespace Bar
+                {
+                }
+                """;
 
             await new VerifyCS.Test
             {
