@@ -7,6 +7,7 @@ using System.Collections.Generic;
 using System.IO;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Shared.Extensions;
+using Roslyn.Test.Utilities;
 using Xunit;
 using VerifyCS = Microsoft.CodeAnalysis.Editor.UnitTests.CodeActions.CSharpCodeFixVerifier<
     Microsoft.CodeAnalysis.CSharp.Analyzers.MatchFolderAndNamespace.CSharpMatchFolderAndNamespaceDiagnosticAnalyzer,
@@ -16,7 +17,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Analyzers.UnitTests.MatchFolderAndNamesp
 {
     public class CSharpMatchFolderAndNamespaceTests
     {
-        private static readonly string Directory = Path.Combine("Test", "Directory");
+        private static readonly string Directory = "/0/";
 
         // DefaultNamespace gets exposed as RootNamespace in the build properties
         private const string DefaultNamespace = "Test.Root.Namespace";
@@ -80,13 +81,14 @@ build_property.RootNamespace = {DefaultNamespace}
             // No change namespace action because the folder name is not valid identifier
             var folder = CreateFolderPath(new[] { "3B", "C" });
             var code =
-@"
-namespace A.B
-{    
-    class Class1
-    {
-    }
-}";
+                """
+                namespace A.B
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
 
             return RunTestAsync(
                 "File1.cs",
@@ -100,13 +102,13 @@ namespace A.B
             // No change namespace action because the folder name is not valid identifier
             var folder = CreateFolderPath(new[] { "3B", "C" });
             var code =
-@"
-namespace A.B;
+                """
+                namespace A.B;
 
-class Class1
-{
-}
-";
+                class Class1
+                {
+                }
+                """;
 
             return RunTestAsync(
                 "File1.cs",
@@ -120,13 +122,14 @@ class Class1
             // No change namespace action because the folder name is not valid identifier
             var folder = CreateFolderPath(new[] { "B.3C", "D" });
             var code =
-@"
-namespace A.B
-{    
-    class Class1
-    {
-    }
-}";
+                """
+                namespace A.B
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
 
             return RunTestAsync(
                 "File1.cs",
@@ -140,13 +143,14 @@ namespace A.B
             // No change namespace action because the folder name is not valid identifier
             var folder = CreateFolderPath(new[] { ".folder", "..subfolder", "name" });
             var code =
-@"
-namespace A.B
-{    
-    class Class1
-    {
-    }
-}";
+                """
+                namespace A.B
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
 
             return RunTestAsync(
                 "File1.cs",
@@ -161,7 +165,7 @@ namespace A.B
             var code =
 @$"
 namespace {DefaultNamespace}.a.b
-{{    
+{{
     class Class1
     {{
     }}
@@ -173,17 +177,43 @@ namespace {DefaultNamespace}.a.b
                 directory: folder);
         }
 
+        [Fact]
+        public async Task CodeStyleOptionIsFalse()
+        {
+            var folder = CreateFolderPath("B", "C");
+            var code =
+                """
+                namespace A.B
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
+
+            await RunTestAsync(
+                fileName: "Class1.cs",
+                fileContents: code,
+                directory: folder,
+                editorConfig: EditorConfig + """
+                dotnet_style_namespace_match_folder = false
+                """
+);
+        }
+
         [Fact]
         public async Task SingleDocumentNoReference()
         {
             var folder = CreateFolderPath("B", "C");
             var code =
-@"namespace [|A.B|]
-{
-    class Class1
-    {
-    }
-}";
+                """
+                namespace [|A.B|]
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
 
             var fixedCode =
 @$"namespace {DefaultNamespace}.B.C
@@ -204,20 +234,20 @@ namespace {DefaultNamespace}.a.b
         {
             var folder = CreateFolderPath("B", "C");
             var code =
-@"namespace [|A.B|];
+                """
+                namespace [|A.B|];
 
-class Class1
-{
-}
-";
+                class Class1
+                {
+                }
+                """;
 
             var fixedCode =
 @$"namespace {DefaultNamespace}.B.C;
 
 class Class1
 {{
-}}
-";
+}}";
             await RunTestAsync(
                 fileName: "Class1.cs",
                 fileContents: code,
@@ -235,12 +265,14 @@ build_property.ProjectDir = {Directory}
 
             var folder = CreateFolderPath("B", "C");
             var code =
-@"namespace [|A.B|]
-{
-    class Class1
-    {
-    }
-}";
+                """
+                namespace [|A.B|]
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
 
             var fixedCode =
 @$"namespace B.C
@@ -269,20 +301,22 @@ build_property.ProjectDir = {Directory}
 
             var folder = CreateFolderPath("B", "C");
             var code =
-@"namespace [|A.B|];
+                """
+                namespace [|A.B|];
 
-class Class1
-{
-}
-";
+                class Class1
+                {
+                }
+                """;
 
             var fixedCode =
-@$"namespace B.C;
+                """
+                namespace B.C;
 
-class Class1
-{{
-}}
-";
+                class Class1
+                {
+                }
+                """;
             await RunTestAsync(
                 fileName: "Class1.cs",
                 fileContents: code,
@@ -314,24 +348,26 @@ class Class1
         [Fact]
         public async Task NestedNamespaces_NoDiagnostic()
         {
-            // The code fix doesn't currently support nested namespaces for sync, so 
-            // diagnostic does not report. 
+            // The code fix doesn't currently support nested namespaces for sync, so
+            // diagnostic does not report.
 
             var folder = CreateFolderPath("B", "C");
             var code =
-@"namespace A.B
-{
-    namespace C.D
-    {
-        class CDClass
-        {
-        }
-    }
+                """
+                namespace A.B
+                {
+                    namespace C.D
+                    {
+                        class CDClass
+                        {
+                        }
+                    }
 
-    class ABClass
-    {
-    }
-}";
+                    class ABClass
+                    {
+                    }
+                }
+                """;
 
             await RunTestAsync(
                 fileName: "Class1.cs",
@@ -342,27 +378,31 @@ class Class1
         [Fact]
         public async Task PartialTypeWithMultipleDeclarations_NoDiagnostic()
         {
-            // The code fix doesn't currently support nested namespaces for sync, so 
-            // diagnostic does not report. 
+            // The code fix doesn't currently support nested namespaces for sync, so
+            // diagnostic does not report.
 
             var folder = CreateFolderPath("B", "C");
             var code1 =
-@"namespace A.B
-{
-    partial class ABClass
-    {
-        void M1() {}
-    }
-}";
+                """
+                namespace A.B
+                {
+                    partial class ABClass
+                    {
+                        void M1() {}
+                    }
+                }
+                """;
 
             var code2 =
-@"namespace A.B
-{
-    partial class ABClass
-    {
-        void M2() {}
-    }
-}";
+                """
+                namespace A.B
+                {
+                    partial class ABClass
+                    {
+                        void M2() {}
+                    }
+                }
+                """;
 
             var sources = new[]
             {
@@ -414,7 +454,7 @@ namespace [|{@namespace}|]
 
     class Class2 : {@namespace}.Class1
     {{
-        {@namespace}.D1 d;  
+        {@namespace}.D1 d;
 
         void {@namespace}.Class1.M1(){{}}
     }}
@@ -809,19 +849,13 @@ namespace {fixedNamespace3}
     }}
 }}";
 
-            var sources = new[]
-            {
-                (Path.Combine(folder1, "Class1.cs"), code1),
-                (Path.Combine(folder2, "Class2.cs"), code2),
-                (Path.Combine(folder3, "Class3.cs"), code3),
-            };
-
-            var fixedSources = new[]
-            {
-                (Path.Combine(folder1, "Class1.cs"), fixed1),
-                (Path.Combine(folder2, "Class2.cs"), fixed2),
-                (Path.Combine(folder3, "Class3.cs"), fixed3),
-            };
+            var project2Directory = "/Project2/";
+            var project2folder = Path.Combine(project2Directory, "A", "B", "C");
+            var project2EditorConfig = @$"
+is_global=true
+build_property.ProjectDir = {project2Directory}
+build_property.RootNamespace = {DefaultNamespace}
+";
 
             var project2Source =
 @$"using {declaredNamespace};
@@ -835,7 +869,7 @@ namespace [|Project2.Test|]
 }}";
 
             var project2FixedSource =
-$@"namespace Test.Root.Namespace.A.B.C
+$@"namespace {fixedNamespace1}
 {{
     class P
     {{
@@ -850,41 +884,143 @@ $@"namespace Test.Root.Namespace.A.B.C
                 LanguageVersion = LanguageVersion.CSharp10,
                 TestState =
                 {
-                    AdditionalProjects = {
-                        ["Project2"] = {
-                            Sources = { },
-                            AnalyzerConfigFiles = { ("/.editorconfig", EditorConfig) }
+                    Sources =
+                    {
+                        (Path.Combine(folder1, "Class1.cs"), code1),
+                        (Path.Combine(folder2, "Class2.cs"), code2),
+                        (Path.Combine(folder3, "Class3.cs"), code3),
+                    },
+                    AdditionalProjects =
+                    {
+                        ["Project2"] =
+                        {
+                            AdditionalProjectReferences = { "TestProject" },
+                            Sources = { (Path.Combine(project2folder, "P.cs"), project2Source) },
+                            AnalyzerConfigFiles = { (Path.Combine(project2Directory, ".editorconfig"), project2EditorConfig) },
                         },
-                    }
+                    },
                 },
                 FixedState =
                 {
+                    Sources =
+                    {
+                        (Path.Combine(folder1, "Class1.cs"), fixed1),
+                        (Path.Combine(folder2, "Class2.cs"), fixed2),
+                        (Path.Combine(folder3, "Class3.cs"), fixed3),
+                    },
                     AdditionalProjects =
                     {
-                        ["Project2"] = {
-                            Sources = { },
-                            AnalyzerConfigFiles = { ("/.editorconfig", EditorConfig) }
+                        ["Project2"] =
+                        {
+                            AdditionalProjectReferences = { "TestProject" },
+                            Sources = { (Path.Combine(project2folder, "P.cs"), project2FixedSource) },
+                            AnalyzerConfigFiles = { (Path.Combine(project2Directory, ".editorconfig"), project2EditorConfig) },
                         }
                     }
                 }
             };
 
-            foreach (var (fileName, content) in sources)
-                testState.TestState.Sources.Add((fileName, content));
-
-            fixedSources ??= Array.Empty<(string, string)>();
-            foreach (var (fileName, content) in fixedSources)
-                testState.FixedState.Sources.Add((fileName, content));
-
-            var project2 = testState.TestState.AdditionalProjects["Project2"];
-            project2.AdditionalProjectReferences.Add(testState.TestState.Name);
-            project2.Sources.Add((Path.Combine(folder1, "P.cs"), project2Source));
-
-            var fixedProject2 = testState.FixedState.AdditionalProjects["Project2"];
-            fixedProject2.AdditionalProjectReferences.Add(testState.TestState.Name);
-            fixedProject2.Sources.Add((Path.Combine(folder1, "P.cs"), project2FixedSource));
+            testState.SolutionTransforms.Add((solution, projectId) =>
+            {
+                foreach (var id in solution.ProjectIds)
+                {
+                    var project = solution.GetRequiredProject(id);
+                    solution = project.WithDefaultNamespace(DefaultNamespace).Solution;
+                }
+                return solution;
+            });
 
             await testState.RunAsync();
         }
+
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/58372")]
+        public async Task InvalidProjectName_ChangeNamespace()
+        {
+            var defaultNamespace = "Invalid-Namespace";
+            var editorConfig = @$"
+is_global=true
+build_property.ProjectDir = {Directory}
+build_property.RootNamespace = {defaultNamespace}
+";
+
+            var folder = CreateFolderPath(new[] { "B", "C" });
+            var code =
+                """
+                namespace [|A.B|]
+                {
+                    class Class1
+                    {
+                    }
+                }
+                """;
+
+            // The project name is invalid so the default namespace is not prepended
+            var fixedCode =
+@$"namespace B.C
+{{
+    class Class1
+    {{
+    }}
+}}";
+
+            await RunTestAsync(
+                "Class1.cs",
+                fileContents: code,
+                fixedCode: fixedCode,
+                directory: folder,
+                editorConfig: editorConfig,
+                defaultNamespace: defaultNamespace);
+        }
+
+        [Fact]
+        public async Task InvalidProjectName_DocumentAtRoot_ChangeNamespace()
+        {
+            var defaultNamespace = "Invalid-Namespace";
+            var editorConfig = @$"
+is_global=true
+build_property.ProjectDir = {Directory}
+build_property.RootNamespace = {defaultNamespace}
+";
+
+            var folder = CreateFolderPath();
+
+            var code =
+$@"namespace Test.Code
+{{
+    class C {{ }}
+}}";
+
+            await RunTestAsync(
+                "Class1.cs",
+                fileContents: code,
+                directory: folder,
+                editorConfig: editorConfig,
+                defaultNamespace: defaultNamespace);
+        }
+
+        [Fact]
+        public async Task InvalidRootNamespace_DocumentAtRoot_ChangeNamespace()
+        {
+            var editorConfig = @$"
+is_global=true
+build_property.ProjectDir = {Directory}
+build_property.RootNamespace = Test.Code # not an editorconfig comment even though it looks like one
+";
+
+            var folder = CreateFolderPath();
+
+            var code =
+$@"namespace Test.Code
+{{
+    class C {{ }}
+}}";
+
+            await RunTestAsync(
+                "Class1.cs",
+                fileContents: code,
+                directory: folder,
+                editorConfig: editorConfig,
+                defaultNamespace: "Invalid-Namespace");
+        }
     }
 }
