@@ -134,12 +134,12 @@ namespace Microsoft.CodeAnalysis.Diagnostics
             ReportDiagnostic effectiveSeverity,
             ImmutableArray<Location> additionalLocations,
             ImmutableArray<Location> additionalUnnecessaryLocations,
-            ImmutableDictionary<string, string?> properties,
+            ImmutableDictionary<string, string?>? properties,
             params object[] messageArgs)
         {
             if (additionalUnnecessaryLocations.IsEmpty)
             {
-                return Create(descriptor, location, effectiveSeverity, additionalLocations, ImmutableDictionary<string, string?>.Empty, messageArgs);
+                return Create(descriptor, location, effectiveSeverity, additionalLocations, properties, messageArgs);
             }
 
             var tagIndices = ImmutableDictionary<string, IEnumerable<int>>.Empty
@@ -180,7 +180,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
             ReportDiagnostic effectiveSeverity,
             IEnumerable<Location> additionalLocations,
             IDictionary<string, IEnumerable<int>> tagIndices,
-            ImmutableDictionary<string, string?> properties,
+            ImmutableDictionary<string, string?>? properties,
             params object[] messageArgs)
         {
             Contract.ThrowIfTrue(additionalLocations.IsEmpty());
@@ -257,15 +257,23 @@ namespace Microsoft.CodeAnalysis.Diagnostics
 
         public static string? GetHelpLinkForDiagnosticId(string id)
         {
+            // TODO: Add documentation for Regex and Json analyzer
+            // Tracked with https://github.com/dotnet/roslyn/issues/48530
             if (id == "RE0001")
+                return null;
+
+            if (id.StartsWith("JSON", StringComparison.Ordinal))
+                return null;
+
+            // These diagnostics are hidden and not configurable, so help link can never be shown and is not applicable.
+            if (id == RemoveUnnecessaryImports.RemoveUnnecessaryImportsConstants.DiagnosticFixableId ||
+                id == "IDE0005_gen")
             {
-                // TODO: Add documentation for Regex analyzer
-                // Tracked with https://github.com/dotnet/roslyn/issues/48530
                 return null;
             }
 
             Debug.Assert(id.StartsWith("IDE", StringComparison.Ordinal));
-            return $"https://docs.microsoft.com/dotnet/fundamentals/code-analysis/style-rules/{id.ToLowerInvariant()}";
+            return $"https://learn.microsoft.com/dotnet/fundamentals/code-analysis/style-rules/{id.ToLowerInvariant()}";
         }
 
         public sealed class LocalizableStringWithArguments : LocalizableString, IObjectWritable
@@ -307,7 +315,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                 }
                 else
                 {
-                    using var argumentsBuilderDisposer = ArrayBuilder<string>.GetInstance(length, out var argumentsBuilder);
+                    using var _ = ArrayBuilder<string>.GetInstance(length, out var argumentsBuilder);
                     for (var i = 0; i < length; i++)
                     {
                         argumentsBuilder.Add(reader.ReadString());
@@ -333,9 +341,9 @@ namespace Microsoft.CodeAnalysis.Diagnostics
             protected override string GetText(IFormatProvider? formatProvider)
             {
                 var messageFormat = _messageFormat.ToString(formatProvider);
-                return messageFormat != null ?
-                    (_formatArguments.Length > 0 ? string.Format(formatProvider, messageFormat, _formatArguments) : messageFormat) :
-                    string.Empty;
+                return messageFormat != null
+                    ? (_formatArguments.Length > 0 ? string.Format(formatProvider, messageFormat, _formatArguments) : messageFormat)
+                    : string.Empty;
             }
 
             protected override bool AreEqual(object? other)
