@@ -6,7 +6,9 @@ using System.Collections.Immutable;
 using System.Linq;
 using Microsoft.CodeAnalysis.CodeStyle;
 using Microsoft.CodeAnalysis.Diagnostics;
-using Microsoft.CodeAnalysis.LanguageServices;
+using Microsoft.CodeAnalysis.LanguageService;
+using Microsoft.CodeAnalysis.Options;
+using Microsoft.CodeAnalysis.Shared.Collections;
 using Microsoft.CodeAnalysis.Text;
 
 namespace Microsoft.CodeAnalysis.UseObjectInitializer
@@ -18,7 +20,9 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
         TObjectCreationExpressionSyntax,
         TMemberAccessExpressionSyntax,
         TAssignmentStatementSyntax,
-        TVariableDeclaratorSyntax>
+        TLocalDeclarationStatementSyntax,
+        TVariableDeclaratorSyntax,
+        TAnalyzer>
         : AbstractBuiltInCodeStyleDiagnosticAnalyzer
         where TSyntaxKind : struct
         where TExpressionSyntax : SyntaxNode
@@ -26,31 +30,65 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
         where TObjectCreationExpressionSyntax : TExpressionSyntax
         where TMemberAccessExpressionSyntax : TExpressionSyntax
         where TAssignmentStatementSyntax : TStatementSyntax
+        where TLocalDeclarationStatementSyntax : TStatementSyntax
         where TVariableDeclaratorSyntax : SyntaxNode
+        where TAnalyzer : AbstractUseNamedMemberInitializerAnalyzer<
+            TExpressionSyntax,
+            TStatementSyntax,
+            TObjectCreationExpressionSyntax,
+            TMemberAccessExpressionSyntax,
+            TAssignmentStatementSyntax,
+            TLocalDeclarationStatementSyntax,
+            TVariableDeclaratorSyntax,
+            TAnalyzer>, new()
     {
+        private static readonly DiagnosticDescriptor s_descriptor = CreateDescriptorWithId(
+            IDEDiagnosticIds.UseObjectInitializerDiagnosticId,
+            EnforceOnBuildValues.UseObjectInitializer,
+            new LocalizableResourceString(nameof(AnalyzersResources.Simplify_object_initialization), AnalyzersResources.ResourceManager, typeof(AnalyzersResources)),
+            new LocalizableResourceString(nameof(AnalyzersResources.Object_initialization_can_be_simplified), AnalyzersResources.ResourceManager, typeof(AnalyzersResources)),
+            isUnnecessary: false);
+
+        private static readonly DiagnosticDescriptor s_unnecessaryCodeDescriptor = CreateDescriptorWithId(
+            IDEDiagnosticIds.UseObjectInitializerDiagnosticId,
+            EnforceOnBuildValues.UseObjectInitializer,
+            new LocalizableResourceString(nameof(AnalyzersResources.Simplify_object_initialization), AnalyzersResources.ResourceManager, typeof(AnalyzersResources)),
+            new LocalizableResourceString(nameof(AnalyzersResources.Object_initialization_can_be_simplified), AnalyzersResources.ResourceManager, typeof(AnalyzersResources)),
+            isUnnecessary: true);
+
         protected abstract bool FadeOutOperatorToken { get; }
+        protected abstract TAnalyzer GetAnalyzer();
 
         protected AbstractUseObjectInitializerDiagnosticAnalyzer()
-            : base(IDEDiagnosticIds.UseObjectInitializerDiagnosticId,
-                   EnforceOnBuildValues.UseObjectInitializer,
-                   CodeStyleOptions2.PreferObjectInitializer,
-                   new LocalizableResourceString(nameof(AnalyzersResources.Simplify_object_initialization), AnalyzersResources.ResourceManager, typeof(AnalyzersResources)),
-                   new LocalizableResourceString(nameof(AnalyzersResources.Object_initialization_can_be_simplified), AnalyzersResources.ResourceManager, typeof(AnalyzersResources)))
+            : base(ImmutableDictionary<DiagnosticDescriptor, IOption2>.Empty
+                    .Add(s_descriptor, CodeStyleOptions2.PreferObjectInitializer)
+                    .Add(s_unnecessaryCodeDescriptor, CodeStyleOptions2.PreferObjectInitializer))
         {
         }
 
-        protected override void InitializeWorker(AnalysisContext context)
+        protected abstract ISyntaxFacts GetSyntaxFacts();
+
+        protected sealed override void InitializeWorker(AnalysisContext context)
         {
-            var syntaxKinds = GetSyntaxFacts().SyntaxKinds;
             context.RegisterCompilationStartAction(context =>
             {
                 if (!AreObjectInitializersSupported(context.Compilation))
-                {
                     return;
-                }
 
-                context.RegisterSyntaxNodeAction(
-                    AnalyzeNode, syntaxKinds.Convert<TSyntaxKind>(syntaxKinds.ObjectCreationExpression));
+                var syntaxKinds = GetSyntaxFacts().SyntaxKinds;
+                using var matchKinds = TemporaryArray<TSyntaxKind>.Empty;
+                matchKinds.Add(syntaxKinds.Convert<TSyntaxKind>(syntaxKinds.ObjectCreationExpression));
+                if (syntaxKinds.ImplicitObjectCreationExpression != null)
+                    matchKinds.Add(syntaxKinds.Convert<TSyntaxKind>(syntaxKinds.ImplicitObjectCreationExpression.Value));
+                var matchKindsArray = matchKinds.ToImmutableAndClear();
+
+                // We wrap the SyntaxNodeAction within a CodeBlockStartAction, which allows us to
+                // get callbacks for object creation expression nodes, but analyze nodes across the entire code block
+                // and eventually report fading diagnostics with location outside this node.
+                // Without the containing CodeBlockStartAction, our reported diagnostic would be classified
+                // as a non-local diagnostic and would not participate in lightbulb for computing code fixes.
+                context.RegisterCodeBlockStartAction<TSyntaxKind>(blockStartContext =>
+                    blockStartContext.RegisterSyntaxNodeAction(AnalyzeNode, matchKindsArray));
             });
         }
 
@@ -60,9 +98,10 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
 
         private void AnalyzeNode(SyntaxNodeAnalysisContext context)
         {
+            var semanticModel = context.SemanticModel;
             var objectCreationExpression = (TObjectCreationExpressionSyntax)context.Node;
             var language = objectCreationExpression.Language;
-            var option = context.GetOption(CodeStyleOptions2.PreferObjectInitializer, language);
+            var option = context.GetAnalyzerOptions().PreferObjectInitializer;
             if (!option.Value)
             {
                 // not point in analyzing if the option is off.
@@ -70,42 +109,33 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
             }
 
             var syntaxFacts = GetSyntaxFacts();
-            var matches = ObjectCreationExpressionAnalyzer<TExpressionSyntax, TStatementSyntax, TObjectCreationExpressionSyntax, TMemberAccessExpressionSyntax, TAssignmentStatementSyntax, TVariableDeclaratorSyntax>.Analyze(
-                context.SemanticModel, syntaxFacts, objectCreationExpression, context.CancellationToken);
+            using var analyzer = GetAnalyzer();
+            var matches = analyzer.Analyze(semanticModel, syntaxFacts, objectCreationExpression, context.CancellationToken);
 
-            if (matches == null || matches.Value.Length == 0)
-            {
+            if (matches.IsDefaultOrEmpty)
                 return;
-            }
 
             var containingStatement = objectCreationExpression.FirstAncestorOrSelf<TStatementSyntax>();
             if (containingStatement == null)
-            {
                 return;
-            }
 
             if (!IsValidContainingStatement(containingStatement))
-            {
                 return;
-            }
 
-            var nodes = ImmutableArray.Create<SyntaxNode>(containingStatement).AddRange(matches.Value.Select(m => m.Statement));
+            var nodes = ImmutableArray.Create<SyntaxNode>(containingStatement).AddRange(matches.Select(m => m.Statement));
             if (syntaxFacts.ContainsInterleavedDirective(nodes, context.CancellationToken))
-            {
                 return;
-            }
 
             var locations = ImmutableArray.Create(objectCreationExpression.GetLocation());
 
-            var severity = option.Notification.Severity;
             context.ReportDiagnostic(DiagnosticHelper.Create(
-                Descriptor,
-                objectCreationExpression.GetLocation(),
-                severity,
-                additionalLocations: locations,
+                s_descriptor,
+                objectCreationExpression.GetFirstToken().GetLocation(),
+                option.Notification.Severity,
+                locations,
                 properties: null));
 
-            FadeOutCode(context, matches.Value, locations);
+            FadeOutCode(context, matches, locations);
         }
 
         private void FadeOutCode(
@@ -115,13 +145,6 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
         {
             var syntaxTree = context.Node.SyntaxTree;
 
-            var fadeOutCode = context.GetOption(
-                CodeStyleOptions2.PreferObjectInitializer_FadeOutCode, context.Node.Language);
-            if (!fadeOutCode)
-            {
-                return;
-            }
-
             var syntaxFacts = GetSyntaxFacts();
 
             foreach (var match in matches)
@@ -136,7 +159,7 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
                 if (match.Statement.Span.End > match.Initializer.FullSpan.End)
                 {
                     context.ReportDiagnostic(DiagnosticHelper.CreateWithLocationTags(
-                        Descriptor,
+                        s_unnecessaryCodeDescriptor,
                         location1,
                         ReportDiagnostic.Default,
                         additionalLocations: locations,
@@ -146,14 +169,12 @@ namespace Microsoft.CodeAnalysis.UseObjectInitializer
                 else
                 {
                     context.ReportDiagnostic(Diagnostic.Create(
-                        Descriptor, location1, additionalLocations: locations));
+                        s_unnecessaryCodeDescriptor, location1, additionalLocations: locations));
                 }
             }
         }
 
-        protected abstract ISyntaxFacts GetSyntaxFacts();
-
-        public override DiagnosticAnalyzerCategory GetAnalyzerCategory()
+        public sealed override DiagnosticAnalyzerCategory GetAnalyzerCategory()
             => DiagnosticAnalyzerCategory.SemanticSpanAnalysis;
     }
 }
