@@ -73,9 +73,9 @@ namespace Microsoft.CodeAnalysis.CSharp
         }
 
         // Return the nearest enclosing node being bound as a nameof(...) argument, if any, or null if none.
-        protected virtual SyntaxNode? EnclosingNameofArgument => null;
+        protected virtual SyntaxNode? EnclosingNameofArgument => NextRequired.EnclosingNameofArgument;
 
-        private bool IsInsideNameof => this.EnclosingNameofArgument != null;
+        internal virtual bool IsInsideNameof => NextRequired.IsInsideNameof;
 
         /// <summary>
         /// Get the next binder in which to look up a name, if not found by this binder.
@@ -129,7 +129,7 @@ namespace Microsoft.CodeAnalysis.CSharp
         /// context is unchecked unless external factors (such as compiler switches and execution 
         /// environment configuration) call for checked evaluation.
         /// </remarks>
-        protected bool CheckOverflowAtRuntime
+        internal bool CheckOverflowAtRuntime
         {
             get
             {
@@ -156,6 +156,8 @@ namespace Microsoft.CodeAnalysis.CSharp
             }
         }
 
+        internal bool UseUpdatedEscapeRules => Compilation.SourceModule.UseUpdatedEscapeRules;
+
         /// <summary>
         /// Some nodes have special binders for their contents (like Blocks)
         /// </summary>
@@ -257,8 +259,9 @@ namespace Microsoft.CodeAnalysis.CSharp
                 Syntax.NullableContextState.State.Disabled => false,
                 Syntax.NullableContextState.State.ExplicitlyRestored => GetGlobalAnnotationState(),
                 Syntax.NullableContextState.State.Unknown =>
-                    !csTree.IsGeneratedCode(this.Compilation.Options.SyntaxTreeOptionsProvider, CancellationToken.None)
-                    && AreNullableAnnotationsGloballyEnabled(),
+                    // IsGeneratedCode may be slow, check global state first:
+                    AreNullableAnnotationsGloballyEnabled() &&
+                    !csTree.IsGeneratedCode(this.Compilation.Options.SyntaxTreeOptionsProvider, CancellationToken.None),
                 _ => throw ExceptionUtilities.UnexpectedValue(context.AnnotationsState)
             };
         }
@@ -269,12 +272,6 @@ namespace Microsoft.CodeAnalysis.CSharp
             return AreNullableAnnotationsEnabled(token.SyntaxTree, token.SpanStart);
         }
 
-        internal bool IsGeneratedCode(SyntaxToken token)
-        {
-            var tree = (CSharpSyntaxTree)token.SyntaxTree!;
-            return tree.IsGeneratedCode(Compilation.Options.SyntaxTreeOptionsProvider, CancellationToken.None);
-        }
-
         internal virtual bool AreNullableAnnotationsGloballyEnabled()
         {
             RoslynDebug.Assert(Next is object);
@@ -436,7 +433,6 @@ namespace Microsoft.CodeAnalysis.CSharp
             }
         }
 
-
         /// <summary>
         /// Returns true if the binder is binding top-level script code.
         /// </summary>
@@ -743,7 +739,7 @@ namespace Microsoft.CodeAnalysis.CSharp
             }
         }
 
-        internal static void ReportDiagnosticsIfUnmanagedCallersOnly(BindingDiagnosticBag diagnostics, MethodSymbol symbol, Location location, bool isDelegateConversion)
+        internal static void ReportDiagnosticsIfUnmanagedCallersOnly(BindingDiagnosticBag diagnostics, MethodSymbol symbol, SyntaxNodeOrToken syntax, bool isDelegateConversion)
         {
             var unmanagedCallersOnlyAttributeData = symbol.GetUnmanagedCallersOnlyAttributeData(forceComplete: false);
             if (unmanagedCallersOnlyAttributeData != null)
@@ -751,13 +747,14 @@ namespace Microsoft.CodeAnalysis.CSharp
                 // Either we haven't yet bound the attributes of this method, or there is an UnmanagedCallersOnly present.
                 // In the former case, we use a lazy diagnostic that may end up being ignored later, to avoid causing a
                 // binding cycle.
+                Debug.Assert(syntax.GetLocation() != null);
                 diagnostics.Add(unmanagedCallersOnlyAttributeData == UnmanagedCallersOnlyAttributeData.Uninitialized
-                                    ? (DiagnosticInfo)new LazyUnmanagedCallersOnlyMethodCalledDiagnosticInfo(symbol, isDelegateConversion)
+                                    ? new LazyUnmanagedCallersOnlyMethodCalledDiagnosticInfo(symbol, isDelegateConversion)
                                     : new CSDiagnosticInfo(isDelegateConversion
                                                                ? ErrorCode.ERR_UnmanagedCallersOnlyMethodsCannotBeConvertedToDelegate
                                                                : ErrorCode.ERR_UnmanagedCallersOnlyMethodsCannotBeCalledDirectly,
                                                            symbol),
-                                location);
+                                syntax.GetLocation()!);
             }
         }
 
@@ -814,6 +811,21 @@ namespace Microsoft.CodeAnalysis.CSharp
             GetWellKnownTypeMember(compilation, attributeMember, diagnostics, location, syntax, isOptional);
         }
 
+        /// <summary>
+        /// Adds diagnostics that should be reported when using a synthesized attribute. 
+        /// </summary>
+        internal static void AddUseSiteDiagnosticForSynthesizedAttribute(
+            CSharpCompilation compilation,
+            WellKnownMember attributeMember,
+            ref CompoundUseSiteInfo<AssemblySymbol> useSiteInfo)
+        {
+            GetWellKnownTypeMember(compilation,
+                attributeMember,
+                out var memberUseSiteInfo,
+                isOptional: WellKnownMembers.IsSynthesizedAttributeOptional(attributeMember));
+            useSiteInfo.Add(memberUseSiteInfo);
+        }
+
         public CompoundUseSiteInfo<AssemblySymbol> GetNewCompoundUseSiteInfo(BindingDiagnosticBag futureDestination)
         {
             return new CompoundUseSiteInfo<AssemblySymbol>(futureDestination, Compilation.Assembly);
@@ -871,7 +883,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 return statement;
             }
 
-            return new BoundBlock(statement.Syntax, locals, localFunctions,
+            return new BoundBlock(statement.Syntax, locals, localFunctions, hasUnsafeModifier: false, instrumentation: null,
                                   ImmutableArray.Create(statement))
             { WasCompilerGenerated = true };
         }
