@@ -229,7 +229,7 @@ namespace Microsoft.CodeAnalysis.CSharp
             public override Binder VisitDestructorDeclaration(DestructorDeclarationSyntax parent)
             {
                 // If the position isn't in the scope of the method, then proceed to the parent syntax node.
-                if (!LookupPosition.IsInMethodDeclaration(_position, parent))
+                if (!LookupPosition.IsInBody(_position, parent))
                 {
                     return VisitCore(parent.Parent);
                 }
@@ -542,19 +542,50 @@ namespace Microsoft.CodeAnalysis.CSharp
 
             private Symbol GetMemberSymbol(string memberName, TextSpan memberSpan, NamedTypeSymbol container, SymbolKind kind)
             {
-                // return container.GetMembers(methodSyntax.Identifier.ValueText).OfType<SourceMethodSymbol>().Single(m => m.Locations.Any(l => l.SourceTree == tree && methodSyntax.Span.Contains(l.SourceSpan)));
-                foreach (Symbol sym in container.GetMembers(memberName))
+                Debug.Assert(kind is SymbolKind.Method or SymbolKind.Property or SymbolKind.Event);
+
+                if (container is SourceMemberContainerTypeSymbol { HasPrimaryConstructor: true } sourceMemberContainerTypeSymbol)
                 {
+                    foreach (Symbol sym in sourceMemberContainerTypeSymbol.GetMembersToMatchAgainstDeclarationSpan())
+                    {
+                        if (sym.IsAccessor())
+                        {
+                            continue;
+                        }
+
+                        if (sym.Name == memberName && checkSymbol(sym, memberSpan, kind, out Symbol result))
+                        {
+                            return result;
+                        }
+                    }
+                }
+                else
+                {
+                    foreach (Symbol sym in container.GetMembers(memberName))
+                    {
+                        if (checkSymbol(sym, memberSpan, kind, out Symbol result))
+                        {
+                            return result;
+                        }
+                    }
+                }
+
+                return null;
+
+                bool checkSymbol(Symbol sym, TextSpan memberSpan, SymbolKind kind, out Symbol result)
+                {
+                    result = sym;
+
                     if (sym.Kind != kind)
                     {
-                        continue;
+                        return false;
                     }
 
                     if (sym.Kind == SymbolKind.Method)
                     {
-                        if (InSpan(sym.Locations[0], this.syntaxTree, memberSpan))
+                        if (InSpan(sym.GetFirstLocation(), this.syntaxTree, memberSpan))
                         {
-                            return sym;
+                            return true;
                         }
 
                         // If this is a partial method, the method represents the defining part,
@@ -563,19 +594,20 @@ namespace Microsoft.CodeAnalysis.CSharp
                         var implementation = ((MethodSymbol)sym).PartialImplementationPart;
                         if ((object)implementation != null)
                         {
-                            if (InSpan(implementation.Locations[0], this.syntaxTree, memberSpan))
+                            if (InSpan(implementation.GetFirstLocation(), this.syntaxTree, memberSpan))
                             {
-                                return implementation;
+                                result = implementation;
+                                return true;
                             }
                         }
                     }
                     else if (InSpan(sym.Locations, this.syntaxTree, memberSpan))
                     {
-                        return sym;
+                        return true;
                     }
-                }
 
-                return null;
+                    return false;
+                }
             }
 
             /// <summary>
@@ -679,8 +711,11 @@ namespace Microsoft.CodeAnalysis.CSharp
                 // and position is more likely to be in the body, so lets check for "inBody" first.
                 if (parent.OpenBraceToken != default &&
                     parent.CloseBraceToken != default &&
-                    (LookupPosition.IsBetweenTokens(_position, parent.OpenBraceToken, parent.CloseBraceToken) ||
-                     LookupPosition.IsInAttributeSpecification(_position, parent.AttributeLists)))
+                    LookupPosition.IsBetweenTokens(_position, parent.OpenBraceToken, parent.CloseBraceToken))
+                {
+                    extraInfo = NodeUsage.NamedTypeBodyOrTypeParameters;
+                }
+                else if (LookupPosition.IsInAttributeSpecification(_position, parent.AttributeLists))
                 {
                     extraInfo = NodeUsage.NamedTypeBodyOrTypeParameters;
                 }
@@ -723,6 +758,8 @@ namespace Microsoft.CodeAnalysis.CSharp
                         }
                         else
                         {
+                            resultBinder = new WithPrimaryConstructorParametersBinder(typeSymbol, resultBinder);
+
                             resultBinder = new InContainerBinder(typeSymbol, resultBinder);
 
                             if (parent.TypeParameterList != null)
@@ -1111,7 +1148,6 @@ namespace Microsoft.CodeAnalysis.CSharp
 
                 XmlNameAttributeElementKind elementKind = parent.GetElementKind();
 
-
                 NodeUsage extraInfo;
                 switch (elementKind)
                 {
@@ -1184,11 +1220,11 @@ namespace Microsoft.CodeAnalysis.CSharp
                     return new WithParametersBinder(method.Parameters, nextBinder);
                 }
 
-                if (memberSyntax is RecordDeclarationSyntax { ParameterList: { ParameterCount: > 0 } })
+                if (memberSyntax is TypeDeclarationSyntax { ParameterList: { ParameterCount: > 0 } })
                 {
                     Binder outerBinder = VisitCore(memberSyntax);
-                    SourceNamedTypeSymbol recordType = ((NamespaceOrTypeSymbol)outerBinder.ContainingMemberOrLambda).GetSourceTypeMember((TypeDeclarationSyntax)memberSyntax);
-                    var primaryConstructor = recordType.GetMembersUnordered().OfType<SynthesizedRecordConstructor>().SingleOrDefault();
+                    SourceNamedTypeSymbol type = ((NamespaceOrTypeSymbol)outerBinder.ContainingMemberOrLambda).GetSourceTypeMember((TypeDeclarationSyntax)memberSyntax);
+                    var primaryConstructor = type.PrimaryConstructor;
 
                     if (primaryConstructor.SyntaxRef.SyntaxTree == memberSyntax.SyntaxTree &&
                         primaryConstructor.GetSyntax() == memberSyntax)
@@ -1320,9 +1356,9 @@ namespace Microsoft.CodeAnalysis.CSharp
 
             Binder getBinder(BaseTypeDeclarationSyntax baseTypeDeclaration)
             {
-                if (baseTypeDeclaration is RecordDeclarationSyntax { SemicolonToken: { RawKind: (int)SyntaxKind.SemicolonToken } } recordDeclaration)
+                if (baseTypeDeclaration is TypeDeclarationSyntax { SemicolonToken: { RawKind: (int)SyntaxKind.SemicolonToken }, OpenBraceToken: { RawKind: (int)SyntaxKind.None } } noBlockBodyTypeDeclarationWithSemicolon)
                 {
-                    return factory.GetInRecordBodyBinder(recordDeclaration);
+                    return factory.GetInTypeBodyBinder(noBlockBodyTypeDeclarationWithSemicolon);
                 }
 
                 return factory.GetBinder(baseTypeDeclaration, baseTypeDeclaration.OpenBraceToken.SpanStart);
