@@ -77,10 +77,10 @@ namespace Microsoft.CodeAnalysis.CSharp
             {
                 reportAction = GetDiagnosticReport(ErrorFacts.GetSeverity(ErrorCode.WRN_ALinkWarn),
                     d.IsEnabledByDefault,
+                    d.Code,
                     CSharp.MessageProvider.Instance.GetIdForErrorCode((int)ErrorCode.WRN_ALinkWarn),
                     ErrorFacts.GetWarningLevel(ErrorCode.WRN_ALinkWarn),
                     d.Location,
-                    d.Category,
                     warningLevelOption,
                     nullableOption,
                     generalDiagnosticOption,
@@ -93,10 +93,10 @@ namespace Microsoft.CodeAnalysis.CSharp
             {
                 reportAction = GetDiagnosticReport(d.Severity,
                     d.IsEnabledByDefault,
+                    d.Code,
                     d.Id,
                     d.WarningLevel,
                     d.Location,
-                    d.Category,
                     warningLevelOption,
                     nullableOption,
                     generalDiagnosticOption,
@@ -131,10 +131,10 @@ namespace Microsoft.CodeAnalysis.CSharp
         internal static ReportDiagnostic GetDiagnosticReport(
             DiagnosticSeverity severity,
             bool isEnabledByDefault,
+            int errorCode,
             string id,
             int diagnosticWarningLevel,
             Location location,
-            string category,
             int warningLevelOption,
             NullableContextOptions nullableOption,
             ReportDiagnostic generalDiagnosticOption,
@@ -159,7 +159,8 @@ namespace Microsoft.CodeAnalysis.CSharp
                     Syntax.NullableContextState.State.Disabled => false,
                     Syntax.NullableContextState.State.ExplicitlyRestored => nullableOption.WarningsEnabled(),
                     Syntax.NullableContextState.State.Unknown =>
-                        tree?.IsGeneratedCode(syntaxTreeOptions, cancellationToken) != true && nullableOption.WarningsEnabled(),
+                        // IsGeneratedCode may be slow, check the option first:
+                        nullableOption.WarningsEnabled() && tree?.IsGeneratedCode(syntaxTreeOptions, cancellationToken) != true,
                     null => nullableOption.WarningsEnabled(),
                     _ => throw ExceptionUtilities.UnexpectedValue(warningsState)
                 };
@@ -290,6 +291,14 @@ namespace Microsoft.CodeAnalysis.CSharp
                 }
             }
 
+            if (!isSpecified && errorCode == (int)ErrorCode.WRN_Experimental)
+            {
+                // Special handling for [Experimental] warning (treat as error severity by default)
+                Debug.Assert(isEnabledByDefault);
+                Debug.Assert(!specifiedWarnAsErrorMinus);
+                report = ReportDiagnostic.Error;
+            }
+
             return report;
 
             bool promoteToAnError()
