@@ -7,9 +7,12 @@ using System.Collections.Generic;
 using System.Collections.Immutable;
 using System.Diagnostics;
 using System.IO;
+using System.Linq;
 using System.Reflection.Metadata;
 using System.Threading;
 using Microsoft.CodeAnalysis.CodeGen;
+using Microsoft.CodeAnalysis.Collections;
+using Microsoft.CodeAnalysis.CSharp.Symbols;
 using Microsoft.CodeAnalysis.Emit;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Roslyn.Utilities;
@@ -63,8 +66,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Emit
 
             if (testData != null)
             {
-                moduleBeingBuilt.SetMethodTestData(testData.Methods);
-                testData.Module = moduleBeingBuilt;
+                moduleBeingBuilt.SetTestData(testData);
             }
 
             var definitionMap = moduleBeingBuilt.PreviousDefinitions;
@@ -82,25 +84,32 @@ namespace Microsoft.CodeAnalysis.CSharp.Emit
                 filterOpt: s => changes.RequiresCompilation(s.GetISymbol()),
                 cancellationToken: cancellationToken))
             {
-                // Map the definitions from the previous compilation to the current compilation.
-                // This must be done after compiling above since synthesized definitions
-                // (generated when compiling method bodies) may be required.
-                var mappedBaseline = MapToCompilation(compilation, moduleBeingBuilt);
+                if (!ContainsPreviousAnonymousDelegates(definitionMap, baseline.SynthesizedTypes.AnonymousDelegatesWithIndexedNames, compilation.AnonymousTypeManager.GetCreatedAnonymousDelegateTypesWithIndexedNames()))
+                {
+                    diagnostics.Add(ErrorCode.ERR_EncUpdateFailedDelegateTypeChanged, Location.None);
+                }
+                else
+                {
+                    // Map the definitions from the previous compilation to the current compilation.
+                    // This must be done after compiling above since synthesized definitions
+                    // (generated when compiling method bodies) may be required.
+                    var mappedBaseline = MapToCompilation(compilation, moduleBeingBuilt);
 
-                newBaseline = compilation.SerializeToDeltaStreams(
-                    moduleBeingBuilt,
-                    mappedBaseline,
-                    definitionMap,
-                    changes,
-                    metadataStream,
-                    ilStream,
-                    pdbStream,
-                    updatedMethods,
-                    changedTypes,
-                    diagnostics,
-                    testData?.SymWriterFactory,
-                    emitOptions.PdbFilePath,
-                    cancellationToken);
+                    newBaseline = compilation.SerializeToDeltaStreams(
+                        moduleBeingBuilt,
+                        mappedBaseline,
+                        definitionMap,
+                        changes,
+                        metadataStream,
+                        ilStream,
+                        pdbStream,
+                        updatedMethods,
+                        changedTypes,
+                        diagnostics,
+                        testData?.SymWriterFactory,
+                        emitOptions.PdbFilePath,
+                        cancellationToken);
+                }
             }
 
             return new EmitDifferenceResult(
@@ -111,6 +120,36 @@ namespace Microsoft.CodeAnalysis.CSharp.Emit
                 changedTypes: changedTypes.ToImmutableAndFree());
         }
 
+        private static bool ContainsPreviousAnonymousDelegates(
+            CSharpDefinitionMap definitionMap,
+            ImmutableSegmentedDictionary<string, AnonymousTypeValue> previousDictionary,
+            IEnumerable<Cci.ITypeDefinition> currentTypes)
+        {
+            if (previousDictionary.Count == 0)
+            {
+                return true;
+            }
+
+            var currentTypesByName = currentTypes.ToImmutableDictionary(getName);
+            if (previousDictionary.Count > currentTypesByName.Count)
+            {
+                return false;
+            }
+
+            foreach (var previousType in previousDictionary)
+            {
+                if (!currentTypesByName.TryGetValue(getName(previousType.Value.Type), out var currentType) ||
+                    definitionMap.MapDefinition(currentType) is null)
+                {
+                    return false;
+                }
+            }
+
+            return true;
+
+            static string getName(Cci.ITypeDefinition type) => ((Cci.INamedEntity)type).Name!;
+        }
+
         /// <summary>
         /// Return a version of the baseline with all definitions mapped to this compilation.
         /// Definitions from the initial generation, from metadata, are not mapped since
@@ -135,42 +174,47 @@ namespace Microsoft.CodeAnalysis.CSharp.Emit
 
             RoslynDebug.AssertNotNull(previousGeneration.Compilation);
             RoslynDebug.AssertNotNull(previousGeneration.PEModuleBuilder);
+            RoslynDebug.AssertNotNull(moduleBeingBuilt.EncSymbolChanges);
 
+            var synthesizedTypes = moduleBeingBuilt.GetSynthesizedTypes();
             var currentSynthesizedMembers = moduleBeingBuilt.GetAllSynthesizedMembers();
+            var currentDeletedMembers = moduleBeingBuilt.EncSymbolChanges.GetAllDeletedMembers();
 
             // Mapping from previous compilation to the current.
-            var anonymousTypeMap = moduleBeingBuilt.GetAnonymousTypeMap();
-            var synthesizedDelegates = moduleBeingBuilt.GetSynthesizedDelegates();
             var sourceAssembly = ((CSharpCompilation)previousGeneration.Compilation).SourceAssembly;
             var sourceContext = new EmitContext((PEModuleBuilder)previousGeneration.PEModuleBuilder, null, new DiagnosticBag(), metadataOnly: false, includePrivateMembers: true);
             var otherContext = new EmitContext(moduleBeingBuilt, null, new DiagnosticBag(), metadataOnly: false, includePrivateMembers: true);
 
             var matcher = new CSharpSymbolMatcher(
-                anonymousTypeMap,
-                synthesizedDelegates,
                 sourceAssembly,
                 sourceContext,
                 compilation.SourceAssembly,
                 otherContext,
-                currentSynthesizedMembers);
+                synthesizedTypes,
+                currentSynthesizedMembers,
+                currentDeletedMembers);
 
-            var mappedSynthesizedMembers = matcher.MapSynthesizedMembers(previousGeneration.SynthesizedMembers, currentSynthesizedMembers);
+            var mappedSynthesizedMembers = matcher.MapSynthesizedOrDeletedMembers(previousGeneration.SynthesizedMembers, currentSynthesizedMembers, isDeletedMemberMapping: false);
+
+            // Deleted members are mapped the same way as synthesized members, so we can just call the same method.
+            var mappedDeletedMembers = matcher.MapSynthesizedOrDeletedMembers(previousGeneration.DeletedMembers, currentDeletedMembers, isDeletedMemberMapping: true);
 
             // TODO: can we reuse some data from the previous matcher?
             var matcherWithAllSynthesizedMembers = new CSharpSymbolMatcher(
-                anonymousTypeMap,
-                synthesizedDelegates,
                 sourceAssembly,
                 sourceContext,
                 compilation.SourceAssembly,
                 otherContext,
-                mappedSynthesizedMembers);
+                synthesizedTypes,
+                mappedSynthesizedMembers,
+                mappedDeletedMembers);
 
             return matcherWithAllSynthesizedMembers.MapBaselineToCompilation(
                 previousGeneration,
                 compilation,
                 moduleBeingBuilt,
-                mappedSynthesizedMembers);
+                mappedSynthesizedMembers,
+                mappedDeletedMembers);
         }
     }
 }
