@@ -140,7 +140,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             return null;
         }
 
-
         Cci.INestedTypeReference Cci.ITypeReference.AsNestedTypeReference
         {
             get
@@ -215,7 +214,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
 
         void Cci.IReference.Dispatch(Cci.MetadataVisitor visitor)
         {
-            throw ExceptionUtilities.Unreachable;
+            throw ExceptionUtilities.Unreachable();
             //We've not yet discovered a scenario in which we need this.
             //If you're hitting this exception. Uncomment the code below
             //and add a unit test.
@@ -508,8 +507,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 {
                     case TypeKind.Enum:
                     case TypeKind.Delegate:
-                    //C# interfaces don't have fields so the flag doesn't really matter, but Dev10 omits it
-                    case TypeKind.Interface:
                         return false;
                 }
 
@@ -756,6 +753,16 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             }
         }
 
+#nullable enable
+        string? Cci.INamedTypeReference.AssociatedFileIdentifier
+        {
+            get
+            {
+                return AdaptedNamedTypeSymbol.GetFileLocalTypeMetadataNamePrefix();
+            }
+        }
+#nullable disable
+
         string Cci.INamedEntity.Name
         {
             get
@@ -1034,7 +1041,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             if (underlyingNamedTypeSymbol is NativeIntegerTypeSymbol)
             {
                 // Emit should use underlying symbol only.
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             }
         }
 
