@@ -82,6 +82,11 @@ namespace Microsoft.CodeAnalysis.CSharp
         IDS_FeatureAutoPropertyInitializer = MessageBase + 12649,
 
         IDS_SK_TYPE_OR_NAMESPACE = MessageBase + 12652,
+        IDS_SK_ARRAY = MessageBase + 12653,
+        IDS_SK_POINTER = MessageBase + 12654,
+        IDS_SK_FUNCTION_POINTER = MessageBase + 12655,
+        IDS_SK_DYNAMIC = MessageBase + 12656,
+
         IDS_Contravariant = MessageBase + 12659,
         IDS_Contravariantly = MessageBase + 12660,
         IDS_Covariant = MessageBase + 12661,
@@ -230,17 +235,52 @@ namespace Microsoft.CodeAnalysis.CSharp
         IDS_FeatureLambdaReturnType = MessageBase + 12804,
         IDS_AsyncMethodBuilderOverride = MessageBase + 12805,
         IDS_FeatureImplicitImplementationOfNonPublicMembers = MessageBase + 12806,
-        IDS_FeatureLineSpanDirective = MessageBase + 12807,
+        // IDS_FeatureLineSpanDirective = MessageBase + 12807, // feature no longer gated on LangVer
         IDS_FeatureImprovedInterpolatedStrings = MessageBase + 12808,
         IDS_FeatureFileScopedNamespace = MessageBase + 12809,
         IDS_FeatureParameterlessStructConstructors = MessageBase + 12810,
         IDS_FeatureStructFieldInitializers = MessageBase + 12811,
+
         IDS_FeatureGenericAttributes = MessageBase + 12812,
+
+        IDS_FeatureNewLinesInInterpolations = MessageBase + 12813,
+        IDS_FeatureListPattern = MessageBase + 12814,
+        // IDS_ParameterNullChecking = MessageBase + 12815, // feature removed from C# 11
+
+        IDS_FeatureCacheStaticMethodGroupConversion = MessageBase + 12816,
+        IDS_FeatureRawStringLiterals = MessageBase + 12817,
+        IDS_FeatureSpanCharConstantPattern = MessageBase + 12818,
+        IDS_FeatureDisposalPattern = MessageBase + 12819,
+        IDS_FeatureAutoDefaultStructs = MessageBase + 12820,
+
+        IDS_FeatureCheckedUserDefinedOperators = MessageBase + 12821,
+        IDS_FeatureUtf8StringLiterals = MessageBase + 12822,
+
+        IDS_FeatureUnsignedRightShift = MessageBase + 12823,
+
+        IDS_FeatureRelaxedShiftOperator = MessageBase + 12824,
+        IDS_FeatureRequiredMembers = MessageBase + 12825,
+        IDS_FeatureRefFields = MessageBase + 12826,
+        IDS_FeatureFileTypes = MessageBase + 12827,
+        IDS_ArrayAccess = MessageBase + 12828,
+        IDS_PointerElementAccess = MessageBase + 12829,
+        IDS_Missing = MessageBase + 12830,
+        IDS_FeatureLambdaOptionalParameters = MessageBase + 12831,
+        IDS_FeatureLambdaParamsArray = MessageBase + 12832,
+
+        IDS_FeaturePrimaryConstructors = MessageBase + 12833,
+        IDS_FeatureUsingTypeAlias = MessageBase + 12834,
+
+        IDS_FeatureInstanceMemberInNameof = MessageBase + 12835,
+
+        IDS_FeatureInlineArrays = MessageBase + 12836,
+        IDS_FeatureCollectionExpressions = MessageBase + 12837,
+        IDS_FeatureRefReadonlyParameters = MessageBase + 12838,
     }
 
     // Message IDs may refer to strings that need to be localized.
     // This struct makes an IFormattable wrapper around a MessageID
-    internal struct LocalizableErrorArgument : IFormattable
+    internal readonly struct LocalizableErrorArgument : IFormattable
     {
         private readonly MessageID _id;
 
@@ -285,16 +325,87 @@ namespace Microsoft.CodeAnalysis.CSharp
             }
         }
 
+        internal static bool CheckFeatureAvailability(
+            this MessageID feature,
+            DiagnosticBag diagnostics,
+            SyntaxNode syntax,
+            Location? location = null)
+        {
+            return CheckFeatureAvailability(
+                feature,
+                diagnostics,
+                syntax.SyntaxTree.Options,
+                static tuple => tuple.location ?? tuple.syntax.Location,
+                (syntax, location));
+        }
+
+        internal static bool CheckFeatureAvailability(
+            this MessageID feature,
+            DiagnosticBag diagnostics,
+            SyntaxToken syntax,
+            Location? location = null)
+        {
+            return CheckFeatureAvailability(
+                feature,
+                diagnostics,
+                syntax.SyntaxTree!.Options,
+                static tuple => tuple.location ?? tuple.syntax.GetLocation(),
+                (syntax, location));
+        }
+
         internal static bool CheckFeatureAvailability(
             this MessageID feature,
             BindingDiagnosticBag diagnostics,
             SyntaxNode syntax,
             Location? location = null)
         {
-            var diag = GetFeatureAvailabilityDiagnosticInfo(feature, (CSharpParseOptions)syntax.SyntaxTree.Options);
-            if (diag is object)
+            return CheckFeatureAvailability(
+                feature,
+                diagnostics,
+                syntax.SyntaxTree.Options,
+                static tuple => tuple.location ?? tuple.syntax.Location,
+                (syntax, location));
+        }
+
+        internal static bool CheckFeatureAvailability(
+            this MessageID feature,
+            BindingDiagnosticBag diagnostics,
+            SyntaxToken syntax,
+            Location? location = null)
+        {
+            return CheckFeatureAvailability(
+                feature,
+                diagnostics,
+                syntax.SyntaxTree!.Options,
+                static tuple => tuple.location ?? tuple.syntax.GetLocation(),
+                (syntax, location));
+        }
+
+        private static bool CheckFeatureAvailability<TData>(
+            this MessageID feature,
+            DiagnosticBag diagnostics,
+            ParseOptions parseOptions,
+            Func<TData, Location> getLocation,
+            TData data)
+        {
+            if (GetFeatureAvailabilityDiagnosticInfo(feature, (CSharpParseOptions)parseOptions) is { } diagInfo)
             {
-                diagnostics.Add(diag, location ?? syntax.GetLocation());
+                diagnostics.Add(diagInfo, getLocation(data));
+                return false;
+            }
+            return true;
+        }
+
+        private static bool CheckFeatureAvailability<TData>(
+            this MessageID feature,
+            BindingDiagnosticBag diagnostics,
+            ParseOptions parseOptions,
+            Func<TData, Location> getLocation,
+            TData data)
+        {
+            if (GetFeatureAvailabilityDiagnosticInfo(feature, (CSharpParseOptions)parseOptions) is { } diagInfo)
+            {
+                diagnostics.Add(diagInfo, getLocation(data));
                 return false;
             }
             return true;
@@ -342,10 +453,40 @@ namespace Microsoft.CodeAnalysis.CSharp
             // Checks are in the LanguageParser unless otherwise noted.
             switch (feature)
             {
+                // PREFER reporting diagnostics in binding when diagnostics do not affect the shape of the syntax tree
+
                 // C# preview features.
+                //case MessageID.IDS_NewFeature:
+                //    return LanguageVersion.Preview;
+
+                // C# 12.0 features.
+                case MessageID.IDS_FeatureLambdaOptionalParameters: // semantic check
+                case MessageID.IDS_FeatureLambdaParamsArray: // semantic check
+                case MessageID.IDS_FeaturePrimaryConstructors: // declaration table check
+                case MessageID.IDS_FeatureUsingTypeAlias: // semantic check
+                case MessageID.IDS_FeatureInstanceMemberInNameof: // semantic check
+                case MessageID.IDS_FeatureInlineArrays: // semantic check
+                case MessageID.IDS_FeatureCollectionExpressions: // semantic check
+                case MessageID.IDS_FeatureRefReadonlyParameters: // semantic check
+                    return LanguageVersion.CSharp12;
+
+                // C# 11.0 features.
+                case MessageID.IDS_FeatureRawStringLiterals:
                 case MessageID.IDS_FeatureStaticAbstractMembersInInterfaces: // semantic check
+                case MessageID.IDS_FeatureNewLinesInInterpolations: // semantic check
+                case MessageID.IDS_FeatureListPattern: // semantic check
+                case MessageID.IDS_FeatureRequiredMembers: // semantic check
+                case MessageID.IDS_FeatureCacheStaticMethodGroupConversion: // lowering check
+                case MessageID.IDS_FeatureSpanCharConstantPattern:
+                case MessageID.IDS_FeatureAutoDefaultStructs: // semantic check
+                case MessageID.IDS_FeatureCheckedUserDefinedOperators: // semantic check for declarations, parsing check for doc comments
+                case MessageID.IDS_FeatureUtf8StringLiterals: // semantic check
+                case MessageID.IDS_FeatureUnsignedRightShift: // semantic check for declarations and consumption, parsing check for doc comments
+                case MessageID.IDS_FeatureRelaxedShiftOperator: // semantic check
+                case MessageID.IDS_FeatureRefFields: // semantic check
+                case MessageID.IDS_FeatureFileTypes: // semantic check
                 case MessageID.IDS_FeatureGenericAttributes: // semantic check
-                    return LanguageVersion.Preview;
+                    return LanguageVersion.CSharp11;
 
                 // C# 10.0 features.
                 case MessageID.IDS_FeatureMixedDeclarationsAndExpressionsInDeconstruction: // semantic check
@@ -358,56 +499,56 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case MessageID.IDS_FeatureGlobalUsing:
                 case MessageID.IDS_FeatureInferredDelegateType: // semantic check
                 case MessageID.IDS_FeatureLambdaAttributes: // semantic check
-                case MessageID.IDS_FeatureExtendedPropertyPatterns:
+                case MessageID.IDS_FeatureExtendedPropertyPatterns: // semantic check
                 case MessageID.IDS_FeatureLambdaReturnType: // semantic check
                 case MessageID.IDS_AsyncMethodBuilderOverride: // semantic check
                 case MessageID.IDS_FeatureConstantInterpolatedStrings: // semantic check
                 case MessageID.IDS_FeatureImplicitImplementationOfNonPublicMembers: // semantic check
-                case MessageID.IDS_FeatureLineSpanDirective:
-                case MessageID.IDS_FeatureFileScopedNamespace: // syntax check
+                case MessageID.IDS_FeatureFileScopedNamespace: // semantic check
                 case MessageID.IDS_FeatureParameterlessStructConstructors: // semantic check
                 case MessageID.IDS_FeatureStructFieldInitializers: // semantic check
                     return LanguageVersion.CSharp10;
 
                 // C# 9.0 features.
                 case MessageID.IDS_FeatureLambdaDiscardParameters: // semantic check
-                case MessageID.IDS_FeatureFunctionPointers:
+                case MessageID.IDS_FeatureFunctionPointers: // semantic check
                 case MessageID.IDS_FeatureLocalFunctionAttributes: // syntax check
-                case MessageID.IDS_FeatureExternLocalFunctions: // syntax check
-                case MessageID.IDS_FeatureImplicitObjectCreation: // syntax check
+                case MessageID.IDS_FeatureExternLocalFunctions: // semantic check
+                case MessageID.IDS_FeatureImplicitObjectCreation: // semantic check
                 case MessageID.IDS_FeatureMemberNotNull:
-                case MessageID.IDS_FeatureAndPattern:
-                case MessageID.IDS_FeatureNotPattern:
-                case MessageID.IDS_FeatureOrPattern:
-                case MessageID.IDS_FeatureParenthesizedPattern:
-                case MessageID.IDS_FeatureTypePattern:
-                case MessageID.IDS_FeatureRelationalPattern:
+                case MessageID.IDS_FeatureAndPattern: // semantic check
+                case MessageID.IDS_FeatureNotPattern: // semantic check
+                case MessageID.IDS_FeatureOrPattern: // semantic check
+                case MessageID.IDS_FeatureParenthesizedPattern: // semantic check
+                case MessageID.IDS_FeatureTypePattern: // semantic check
+                case MessageID.IDS_FeatureRelationalPattern: // semantic check
                 case MessageID.IDS_FeatureExtensionGetEnumerator: // semantic check
                 case MessageID.IDS_FeatureExtensionGetAsyncEnumerator: // semantic check
                 case MessageID.IDS_FeatureNativeInt:
                 case MessageID.IDS_FeatureExtendedPartialMethods: // semantic check
                 case MessageID.IDS_TopLevelStatements:
                 case MessageID.IDS_FeatureInitOnlySetters: // semantic check
-                case MessageID.IDS_FeatureRecords:
+                case MessageID.IDS_FeatureRecords: // semantic check
                 case MessageID.IDS_FeatureTargetTypedConditional:  // semantic check
                 case MessageID.IDS_FeatureCovariantReturnsForOverrides: // semantic check
-                case MessageID.IDS_FeatureStaticAnonymousFunction: // syntax check
+                case MessageID.IDS_FeatureStaticAnonymousFunction: // semantic check
                 case MessageID.IDS_FeatureModuleInitializers: // semantic check on method attribute
-                case MessageID.IDS_FeatureDefaultTypeParameterConstraint:
+                case MessageID.IDS_FeatureDefaultTypeParameterConstraint: // semantic check
                 case MessageID.IDS_FeatureVarianceSafetyForStaticInterfaceMembers: // semantic check
                     return LanguageVersion.CSharp9;
 
                 // C# 8.0 features.
-                case MessageID.IDS_FeatureAltInterpolatedVerbatimStrings:
-                case MessageID.IDS_FeatureCoalesceAssignmentExpression:
+                case MessageID.IDS_FeatureAltInterpolatedVerbatimStrings: // semantic check
+                case MessageID.IDS_FeatureCoalesceAssignmentExpression: // semantic check
                 case MessageID.IDS_FeatureUnconstrainedTypeParameterInNullCoalescingOperator:
                 case MessageID.IDS_FeatureNullableReferenceTypes: // syntax and semantic check
                 case MessageID.IDS_FeatureIndexOperator: // semantic check
                 case MessageID.IDS_FeatureRangeOperator: // semantic check
-                case MessageID.IDS_FeatureAsyncStreams:
-                case MessageID.IDS_FeatureRecursivePatterns:
-                case MessageID.IDS_FeatureUsingDeclarations:
-                case MessageID.IDS_FeatureStaticLocalFunctions:
+                case MessageID.IDS_FeatureAsyncStreams: // semantic check
+                case MessageID.IDS_FeatureRecursivePatterns: // semantic check
+                case MessageID.IDS_FeatureUsingDeclarations: // semantic check
+                case MessageID.IDS_FeatureDisposalPattern: //semantic check
+                case MessageID.IDS_FeatureStaticLocalFunctions: // semantic check
                 case MessageID.IDS_FeatureNameShadowingInNestedFunctions:
                 case MessageID.IDS_FeatureUnmanagedConstructedTypes: // semantic check
                 case MessageID.IDS_FeatureObsoleteOnPropertyAccessor:
@@ -425,13 +566,13 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case MessageID.IDS_FeatureAttributesOnBackingFields: // semantic check
                 case MessageID.IDS_FeatureImprovedOverloadCandidates: // semantic check
                 case MessageID.IDS_FeatureTupleEquality: // semantic check
-                case MessageID.IDS_FeatureRefReassignment:
+                case MessageID.IDS_FeatureRefReassignment: // semantic check
                 case MessageID.IDS_FeatureRefFor:
-                case MessageID.IDS_FeatureRefForEach:
+                case MessageID.IDS_FeatureRefForEach: // semantic check
                 case MessageID.IDS_FeatureEnumGenericTypeConstraint: // semantic check
                 case MessageID.IDS_FeatureDelegateGenericTypeConstraint: // semantic check
                 case MessageID.IDS_FeatureUnmanagedGenericTypeConstraint: // semantic check
-                case MessageID.IDS_FeatureStackAllocInitializer:
+                case MessageID.IDS_FeatureStackAllocInitializer: // semantic check
                 case MessageID.IDS_FeatureExpressionVariablesInQueriesAndInitializers: // semantic check
                 case MessageID.IDS_FeatureExtensibleFixedStatement:  // semantic check
                 case MessageID.IDS_FeatureIndexingMovableFixedBuffers: //semantic check
@@ -441,16 +582,16 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case MessageID.IDS_FeatureNonTrailingNamedArguments: // semantic check
                 case MessageID.IDS_FeatureLeadingDigitSeparator:
                 case MessageID.IDS_FeaturePrivateProtected:
-                case MessageID.IDS_FeatureReadOnlyReferences:
-                case MessageID.IDS_FeatureRefStructs:
-                case MessageID.IDS_FeatureReadOnlyStructs:
+                case MessageID.IDS_FeatureReadOnlyReferences: // semantic check
+                case MessageID.IDS_FeatureRefStructs: // semantic check
+                case MessageID.IDS_FeatureReadOnlyStructs: // semantic check
                 case MessageID.IDS_FeatureRefExtensionMethods:
                 case MessageID.IDS_FeatureRefConditional:
                     return LanguageVersion.CSharp7_2;
 
                 // C# 7.1 features.
                 case MessageID.IDS_FeatureAsyncMain:
-                case MessageID.IDS_FeatureDefaultLiteral:
+                case MessageID.IDS_FeatureDefaultLiteral: // semantic check
                 case MessageID.IDS_FeatureInferredTupleNames:
                 case MessageID.IDS_FeatureGenericPatternMatching:
                     return LanguageVersion.CSharp7_1;
@@ -458,67 +599,67 @@ namespace Microsoft.CodeAnalysis.CSharp
                 // C# 7 features.
                 case MessageID.IDS_FeatureBinaryLiteral:
                 case MessageID.IDS_FeatureDigitSeparator:
-                case MessageID.IDS_FeatureLocalFunctions:
+                case MessageID.IDS_FeatureLocalFunctions: // semantic check
                 case MessageID.IDS_FeatureRefLocalsReturns:
-                case MessageID.IDS_FeaturePatternMatching:
-                case MessageID.IDS_FeatureThrowExpression:
+                case MessageID.IDS_FeaturePatternMatching: // semantic check
+                case MessageID.IDS_FeatureThrowExpression: // semantic check
                 case MessageID.IDS_FeatureTuples:
-                case MessageID.IDS_FeatureOutVar:
+                case MessageID.IDS_FeatureOutVar: // semantic check
                 case MessageID.IDS_FeatureExpressionBodiedAccessor:
                 case MessageID.IDS_FeatureExpressionBodiedDeOrConstructor:
                 case MessageID.IDS_FeatureDiscards:
                     return LanguageVersion.CSharp7;
 
                 // C# 6 features.
-                case MessageID.IDS_FeatureExceptionFilter:
-                case MessageID.IDS_FeatureAutoPropertyInitializer:
-                case MessageID.IDS_FeatureNullPropagatingOperator:
+                case MessageID.IDS_FeatureExceptionFilter: // semantic check
+                case MessageID.IDS_FeatureAutoPropertyInitializer: // semantic check
+                case MessageID.IDS_FeatureNullPropagatingOperator: // semantic check
                 case MessageID.IDS_FeatureExpressionBodiedMethod:
-                case MessageID.IDS_FeatureExpressionBodiedProperty:
-                case MessageID.IDS_FeatureExpressionBodiedIndexer:
+                case MessageID.IDS_FeatureExpressionBodiedProperty: // semantic check
+                case MessageID.IDS_FeatureExpressionBodiedIndexer: // semantic check
                 case MessageID.IDS_FeatureNameof:
-                case MessageID.IDS_FeatureDictionaryInitializer:
-                case MessageID.IDS_FeatureUsingStatic:
-                case MessageID.IDS_FeatureInterpolatedStrings:
+                case MessageID.IDS_FeatureDictionaryInitializer: // semantic check
+                case MessageID.IDS_FeatureUsingStatic: // semantic check
+                case MessageID.IDS_FeatureInterpolatedStrings: // semantic check
                 case MessageID.IDS_AwaitInCatchAndFinally:
                 case MessageID.IDS_FeatureReadonlyAutoImplementedProperties:
                     return LanguageVersion.CSharp6;
 
                 // C# 5 features.
-                case MessageID.IDS_FeatureAsync:
+                case MessageID.IDS_FeatureAsync: // semantic check
                     return LanguageVersion.CSharp5;
 
                 // C# 4 features.
                 case MessageID.IDS_FeatureDynamic: // Checked in the binder.
-                case MessageID.IDS_FeatureTypeVariance:
-                case MessageID.IDS_FeatureNamedArgument:
-                case MessageID.IDS_FeatureOptionalParameter:
+                case MessageID.IDS_FeatureTypeVariance: // semantic check
+                case MessageID.IDS_FeatureNamedArgument: // semantic check
+                case MessageID.IDS_FeatureOptionalParameter: // semantic check
                     return LanguageVersion.CSharp4;
 
                 // C# 3 features.
-                case MessageID.IDS_FeatureImplicitArray:
-                case MessageID.IDS_FeatureAnonymousTypes:
+                case MessageID.IDS_FeatureImplicitArray: // semantic check
+                case MessageID.IDS_FeatureAnonymousTypes: // semantic check
                 case MessageID.IDS_FeatureObjectInitializer:
                 case MessageID.IDS_FeatureCollectionInitializer:
-                case MessageID.IDS_FeatureLambda:
-                case MessageID.IDS_FeatureQueryExpression:
+                case MessageID.IDS_FeatureLambda: // semantic check
+                case MessageID.IDS_FeatureQueryExpression: // semantic check
                 case MessageID.IDS_FeatureExtensionMethod:
-                case MessageID.IDS_FeaturePartialMethod:
+                case MessageID.IDS_FeaturePartialMethod: // semantic check
                 case MessageID.IDS_FeatureImplicitLocal: // Checked in the binder.
                 case MessageID.IDS_FeatureAutoImplementedProperties:
                     return LanguageVersion.CSharp3;
 
                 // C# 2 features.
                 case MessageID.IDS_FeatureGenerics: // Also affects crefs.
-                case MessageID.IDS_FeatureAnonDelegates:
+                case MessageID.IDS_FeatureAnonDelegates: // semantic check
                 case MessageID.IDS_FeatureGlobalNamespace: // Also affects crefs.
-                case MessageID.IDS_FeatureFixedBuffer:
-                case MessageID.IDS_FeatureStaticClasses:
-                case MessageID.IDS_FeaturePartialTypes:
+                case MessageID.IDS_FeatureFixedBuffer: // semantic check
+                case MessageID.IDS_FeatureStaticClasses: // semantic check
+                case MessageID.IDS_FeaturePartialTypes: // semantic check
                 case MessageID.IDS_FeaturePropertyAccessorMods:
-                case MessageID.IDS_FeatureExternAlias:
-                case MessageID.IDS_FeatureIterators:
-                case MessageID.IDS_FeatureDefault:
+                case MessageID.IDS_FeatureExternAlias: // semantic check
+                case MessageID.IDS_FeatureIterators: // semantic check
+                case MessageID.IDS_FeatureDefault: // semantic check
                 case MessageID.IDS_FeatureNullable:
                 case MessageID.IDS_FeaturePragma: // Checked in the directive parser.
                 case MessageID.IDS_FeatureSwitchOnBool: // Checked in the binder.
