@@ -152,7 +152,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                             PropertySymbol property = p.Property;
                             var outputTemp = new BoundDagTemp(p.Syntax, property.Type, p);
                             BoundExpression output = _tempAllocator.GetTemp(outputTemp);
-                            return _factory.AssignmentExpression(output, _factory.Property(input, property));
+                            return _factory.AssignmentExpression(output, _localRewriter.MakePropertyAccess(_factory.Syntax, input, property, LookupResultKind.Viable, property.Type, isLeftOfAssignment: false));
                         }
 
                     case BoundDagDeconstructEvaluation d:
@@ -208,7 +208,9 @@ namespace Microsoft.CodeAnalysis.CSharp
                             var outputTemp = new BoundDagTemp(t.Syntax, type, t);
                             BoundExpression output = _tempAllocator.GetTemp(outputTemp);
                             CompoundUseSiteInfo<AssemblySymbol> useSiteInfo = _localRewriter.GetNewCompoundUseSiteInfo();
-                            Conversion conversion = _factory.Compilation.Conversions.ClassifyBuiltInConversion(inputType, output.Type, ref useSiteInfo);
+                            Conversion conversion = _factory.Compilation.Conversions.ClassifyBuiltInConversion(inputType, output.Type, isChecked: false, ref useSiteInfo);
+
+                            Debug.Assert(!conversion.IsUserDefined);
                             _localRewriter._diagnostics.Add(t.Syntax, useSiteInfo);
                             BoundExpression evaluated;
                             if (conversion.Exists)
@@ -242,12 +244,90 @@ namespace Microsoft.CodeAnalysis.CSharp
                             TypeSymbol type = e.Property.GetMethod.ReturnType;
                             var outputTemp = new BoundDagTemp(e.Syntax, type, e);
                             BoundExpression output = _tempAllocator.GetTemp(outputTemp);
-                            return _factory.AssignmentExpression(output, _factory.Call(input, e.Property.GetMethod, _factory.Literal(e.Index)));
+                            return _factory.AssignmentExpression(output, _factory.Indexer(input, e.Property, _factory.Literal(e.Index)));
                         }
 
+                    case BoundDagIndexerEvaluation e:
+                        {
+                            // this[Index]
+                            // this[int]
+                            // array[Index]
+
+                            var indexerAccess = e.IndexerAccess;
+                            if (indexerAccess is BoundImplicitIndexerAccess implicitAccess)
+                            {
+                                indexerAccess = implicitAccess.WithLengthOrCountAccess(_tempAllocator.GetTemp(e.LengthTemp));
+                            }
+
+                            var placeholderValues = PooledDictionary<BoundEarlyValuePlaceholderBase, BoundExpression>.GetInstance();
+                            placeholderValues.Add(e.ReceiverPlaceholder, input);
+                            placeholderValues.Add(e.ArgumentPlaceholder, makeUnloweredIndexArgument(e.Index));
+                            indexerAccess = PlaceholderReplacer.Replace(placeholderValues, indexerAccess);
+                            placeholderValues.Free();
+
+                            var access = (BoundExpression)_localRewriter.Visit(indexerAccess);
+
+                            var outputTemp = new BoundDagTemp(e.Syntax, e.IndexerType, e);
+                            BoundExpression output = _tempAllocator.GetTemp(outputTemp);
+                            return _factory.AssignmentExpression(output, access);
+                        }
+
+                    case BoundDagSliceEvaluation e:
+                        {
+                            // this[Range]
+                            // Slice(int, int)
+                            // string.Substring(int, int)
+                            // array[Range]
+
+                            var indexerAccess = e.IndexerAccess;
+                            if (indexerAccess is BoundImplicitIndexerAccess implicitAccess)
+                            {
+                                indexerAccess = implicitAccess.WithLengthOrCountAccess(_tempAllocator.GetTemp(e.LengthTemp));
+                            }
+
+                            var placeholderValues = PooledDictionary<BoundEarlyValuePlaceholderBase, BoundExpression>.GetInstance();
+                            placeholderValues.Add(e.ReceiverPlaceholder, input);
+                            placeholderValues.Add(e.ArgumentPlaceholder, makeUnloweredRangeArgument(e));
+                            indexerAccess = PlaceholderReplacer.Replace(placeholderValues, indexerAccess);
+                            placeholderValues.Free();
+
+                            var access = (BoundExpression)_localRewriter.Visit(indexerAccess);
+
+                            var outputTemp = new BoundDagTemp(e.Syntax, e.SliceType, e);
+                            BoundExpression output = _tempAllocator.GetTemp(outputTemp);
+                            return _factory.AssignmentExpression(output, access);
+                        }
+
+                    case BoundDagAssignmentEvaluation:
                     default:
                         throw ExceptionUtilities.UnexpectedValue(evaluation);
                 }
+
+                BoundExpression makeUnloweredIndexArgument(int index)
+                {
+                    // LocalRewriter.MakePatternIndexOffsetExpression understands this format
+                    var ctor = (MethodSymbol)_factory.WellKnownMember(WellKnownMember.System_Index__ctor);
+
+                    if (index < 0)
+                    {
+                        return new BoundFromEndIndexExpression(_factory.Syntax, _factory.Literal(-index),
+                            methodOpt: ctor, _factory.WellKnownType(WellKnownType.System_Index));
+                    }
+
+                    return _factory.New(ctor, _factory.Literal(index), _factory.Literal(false));
+                }
+
+                BoundExpression makeUnloweredRangeArgument(BoundDagSliceEvaluation e)
+                {
+                    // LocalRewriter.VisitRangeImplicitIndexerAccess understands this format
+                    var indexCtor = (MethodSymbol)_factory.WellKnownMember(WellKnownMember.System_Index__ctor);
+                    var end = new BoundFromEndIndexExpression(_factory.Syntax, _factory.Literal(-e.EndIndex),
+                        methodOpt: indexCtor, _factory.WellKnownType(WellKnownType.System_Index));
+
+                    var rangeCtor = (MethodSymbol)_factory.WellKnownMember(WellKnownMember.System_Range__ctor);
+                    return new BoundRangeExpression(e.Syntax, makeUnloweredIndexArgument(e.StartIndex), end,
+                        methodOpt: rangeCtor, _factory.WellKnownType(WellKnownType.System_Range));
+                }
             }
 
             /// <summary>
@@ -286,6 +366,8 @@ namespace Microsoft.CodeAnalysis.CSharp
 
             private BoundExpression MakeNullCheck(SyntaxNode syntax, BoundExpression rewrittenExpr, BinaryOperatorKind operatorKind)
             {
+                Debug.Assert(!rewrittenExpr.Type.IsSpanOrReadOnlySpanChar());
+
                 if (rewrittenExpr.Type.IsPointerOrFunctionPointer())
                 {
                     TypeSymbol objectType = _factory.SpecialType(SpecialType.System_Object);
@@ -304,6 +386,11 @@ namespace Microsoft.CodeAnalysis.CSharp
 
             protected BoundExpression MakeValueTest(SyntaxNode syntax, BoundExpression input, ConstantValue value)
             {
+                if (value.IsString && input.Type.IsSpanOrReadOnlySpanChar())
+                {
+                    return MakeSpanStringTest(input, value);
+                }
+
                 TypeSymbol comparisonType = input.Type.EnumUnderlyingTypeOrSelf();
                 var operatorType = Binder.RelationalOperatorType(comparisonType);
                 Debug.Assert(operatorType != BinaryOperatorKind.Error);
@@ -341,6 +428,22 @@ namespace Microsoft.CodeAnalysis.CSharp
                 return this._localRewriter.MakeBinaryOperator(_factory.Syntax, operatorKind, input, literal, _factory.SpecialType(SpecialType.System_Boolean), method: null, constrainedToTypeOpt: null);
             }
 
+            private BoundExpression MakeSpanStringTest(BoundExpression input, ConstantValue value)
+            {
+                var isReadOnlySpan = input.Type.IsReadOnlySpanChar();
+                // Binder.ConvertPatternExpression() has checked for these well-known members.
+                var sequenceEqual =
+                    ((MethodSymbol)_factory.WellKnownMember(isReadOnlySpan
+                        ? WellKnownMember.System_MemoryExtensions__SequenceEqual_ReadOnlySpan_T
+                        : WellKnownMember.System_MemoryExtensions__SequenceEqual_Span_T))
+                    .Construct(_factory.SpecialType(SpecialType.System_Char));
+                var asSpan = (MethodSymbol)_factory.WellKnownMember(WellKnownMember.System_MemoryExtensions__AsSpan_String);
+
+                Debug.Assert(sequenceEqual != null && asSpan != null);
+
+                return _factory.Call(null, sequenceEqual, input, _factory.Call(null, asSpan, _factory.StringLiteral(value)));
+            }
+
             /// <summary>
             /// Lower a test followed by an evaluation into a side-effect followed by a test. This permits us to optimize
             /// a type test followed by a cast into an `as` expression followed by a null check. Returns true if the optimization
@@ -378,7 +481,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 // case 2: null check followed by cast to a base type
                 if (test is BoundDagNonNullTest nonNullTest &&
                     evaluation is BoundDagTypeEvaluation typeEvaluation2 &&
-                    _factory.Compilation.Conversions.ClassifyBuiltInConversion(test.Input.Type, typeEvaluation2.Type, ref useSiteInfo) is Conversion conv &&
+                    _factory.Compilation.Conversions.ClassifyBuiltInConversion(test.Input.Type, typeEvaluation2.Type, isChecked: false, ref useSiteInfo) is Conversion conv &&
                     (conv.IsIdentity || conv.Kind == ConversionKind.ImplicitReference || conv.IsBoxing) &&
                     typeEvaluation2.Input == nonNullTest.Input)
                 {
@@ -410,7 +513,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 // We share input variables if there is no when clause (because a when clause might mutate them).
                 bool anyWhenClause =
                     decisionDag.TopologicallySortedNodes
-                    .Any(node => node is BoundWhenDecisionDagNode { WhenExpression: { ConstantValue: null } });
+                    .Any(static node => node is BoundWhenDecisionDagNode { WhenExpression: { ConstantValueOpt: null } });
 
                 var inputDagTemp = BoundDagTemp.ForOriginalInput(loweredInput);
                 if ((loweredInput.Kind == BoundKind.Local || loweredInput.Kind == BoundKind.Parameter)
@@ -450,7 +553,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                     !loweredInput.Type.OriginalDefinition.Equals(_factory.Compilation.GetWellKnownType(WellKnownType.System_ValueTuple_TRest)) &&
                     loweredInput.Syntax.Kind() == SyntaxKind.TupleExpression &&
                     loweredInput is BoundObjectCreationExpression expr &&
-                    !decisionDag.TopologicallySortedNodes.Any(n => usesOriginalInput(n)))
+                    !decisionDag.TopologicallySortedNodes.Any(static n => usesOriginalInput(n)))
                 {
                     // If the switch governing expression is a tuple literal whose whole value is not used anywhere,
                     // (though perhaps its component parts are used), then we can save the component parts
@@ -477,7 +580,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                     switch (node)
                     {
                         case BoundWhenDecisionDagNode n:
-                            return n.Bindings.Any(b => b.TempContainingValue.IsOriginalInput);
+                            return n.Bindings.Any(static b => b.TempContainingValue.IsOriginalInput);
                         case BoundTestDecisionDagNode t:
                             return t.Test.Input.IsOriginalInput;
                         case BoundEvaluationDecisionDagNode e:
@@ -536,6 +639,8 @@ namespace Microsoft.CodeAnalysis.CSharp
 
                 void storeToTemp(BoundDagTemp temp, BoundExpression expr)
                 {
+                    Debug.Assert(!IsCapturedPrimaryConstructorParameter(expr));
+
                     if (canShareInputs && (expr.Kind == BoundKind.Parameter || expr.Kind == BoundKind.Local) && _tempAllocator.TrySetTemp(temp, expr))
                     {
                         // we've arranged to use the input value from the variable it is already stored in
@@ -547,7 +652,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                     }
                 }
 
-                BoundDecisionDagNode makeReplacement(BoundDecisionDagNode node, Func<BoundDecisionDagNode, BoundDecisionDagNode> replacement)
+                static BoundDecisionDagNode makeReplacement(BoundDecisionDagNode node, IReadOnlyDictionary<BoundDecisionDagNode, BoundDecisionDagNode> replacement)
                 {
                     switch (node)
                     {
@@ -559,7 +664,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                                 field.TupleElementIndex is int i)
                             {
                                 // The elements of an input tuple were evaluated beforehand, so don't need to be evaluated now.
-                                return replacement(evalNode.Next);
+                                return replacement[evalNode.Next];
                             }
 
                             // Since we are performing an optimization whose precondition is that the original
