@@ -549,8 +549,11 @@ namespace Microsoft.CodeAnalysis.CSharp
 
             switch (loweredReceiver.Kind)
             {
-                case BoundKind.Local:
                 case BoundKind.Parameter:
+                    Debug.Assert(!LocalRewriter.IsCapturedPrimaryConstructorParameter(loweredReceiver));
+                    goto case BoundKind.Local;
+
+                case BoundKind.Local:
                 case BoundKind.ArrayAccess:
                 case BoundKind.ThisReference:
                 case BoundKind.PointerIndirectionOperator:
@@ -800,7 +803,11 @@ namespace Microsoft.CodeAnalysis.CSharp
             return synthesizedType.Construct(delegateSignature);
 
             // The distinction between by-ref kinds is ignored for dynamic call-sites.
-            static RefKind getRefKind(RefKind refKind) => refKind == RefKind.None ? RefKind.None : RefKind.Ref;
+            static RefKind getRefKind(RefKind refKind)
+            {
+                Debug.Assert(refKind != RefKind.RefReadOnlyParameter);
+                return refKind == RefKind.None ? RefKind.None : RefKind.Ref;
+            }
         }
 
         private BoundExpression GetArgumentInfo(
@@ -863,7 +870,7 @@ namespace Microsoft.CodeAnalysis.CSharp
             //
             // as well as the alternative where x is a const null of type object.
 
-            if (boundArgument.ConstantValue != null)
+            if (boundArgument.ConstantValueOpt != null)
             {
                 flags |= CSharpArgumentInfoFlags.Constant;
             }
