@@ -8,6 +8,7 @@ using System;
 using System.Collections.Immutable;
 using System.Diagnostics;
 using System.Linq;
+using Microsoft.CodeAnalysis.CSharp.CodeGen;
 using Microsoft.CodeAnalysis.CSharp.Symbols;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Roslyn.Utilities;
@@ -20,12 +21,18 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         private readonly SyntheticBoundNodeFactory _F;
         private readonly PooledDictionary<LocalSymbol, LocalSymbol> _tempSubstitution;
+        private readonly PooledDictionary<LocalSymbol, BoundComplexConditionalReceiver> _receiverSubstitution;
 
-        private SpillSequenceSpiller(MethodSymbol method, SyntaxNode syntaxNode, TypeCompilationState compilationState, PooledDictionary<LocalSymbol, LocalSymbol> tempSubstitution, BindingDiagnosticBag diagnostics)
+        private SpillSequenceSpiller(
+            MethodSymbol method, SyntaxNode syntaxNode, TypeCompilationState compilationState,
+            PooledDictionary<LocalSymbol, LocalSymbol> tempSubstitution,
+            PooledDictionary<LocalSymbol, BoundComplexConditionalReceiver> receiverSubstitution,
+            BindingDiagnosticBag diagnostics)
         {
             _F = new SyntheticBoundNodeFactory(method, syntaxNode, compilationState, diagnostics);
             _F.CurrentFunction = method;
             _tempSubstitution = tempSubstitution;
+            _receiverSubstitution = receiverSubstitution;
         }
 
         private sealed class BoundSpillSequenceBuilder : BoundExpression
@@ -175,21 +182,29 @@ namespace Microsoft.CodeAnalysis.CSharp
         private sealed class LocalSubstituter : BoundTreeRewriterWithStackGuardWithoutRecursionOnTheLeftOfBinaryOperator
         {
             private readonly PooledDictionary<LocalSymbol, LocalSymbol> _tempSubstitution;
+            private readonly PooledDictionary<LocalSymbol, BoundComplexConditionalReceiver> _receiverSubstitution;
 
-            private LocalSubstituter(PooledDictionary<LocalSymbol, LocalSymbol> tempSubstitution, int recursionDepth = 0)
+            private LocalSubstituter(
+                PooledDictionary<LocalSymbol, LocalSymbol> tempSubstitution,
+                PooledDictionary<LocalSymbol, BoundComplexConditionalReceiver> receiverSubstitution,
+                int recursionDepth = 0)
                 : base(recursionDepth)
             {
                 _tempSubstitution = tempSubstitution;
+                _receiverSubstitution = receiverSubstitution;
             }
 
-            public static BoundNode Rewrite(PooledDictionary<LocalSymbol, LocalSymbol> tempSubstitution, BoundNode node)
+            public static BoundNode Rewrite(
+                PooledDictionary<LocalSymbol, LocalSymbol> tempSubstitution,
+                PooledDictionary<LocalSymbol, BoundComplexConditionalReceiver> receiverSubstitution,
+                BoundNode node)
             {
                 if (tempSubstitution.Count == 0)
                 {
                     return node;
                 }
 
-                var substituter = new LocalSubstituter(tempSubstitution);
+                var substituter = new LocalSubstituter(tempSubstitution, receiverSubstitution);
                 return substituter.Visit(node);
             }
 
@@ -200,8 +215,15 @@ namespace Microsoft.CodeAnalysis.CSharp
                     LocalSymbol longLived;
                     if (_tempSubstitution.TryGetValue(node.LocalSymbol, out longLived))
                     {
+                        Debug.Assert(!_receiverSubstitution.ContainsKey(node.LocalSymbol));
+
                         return node.Update(longLived, node.ConstantValueOpt, node.Type);
                     }
+
+                    if (_receiverSubstitution.TryGetValue(node.LocalSymbol, out var receiver))
+                    {
+                        return Visit(receiver);
+                    }
                 }
 
                 return base.VisitLocal(node);
@@ -211,10 +233,12 @@ namespace Microsoft.CodeAnalysis.CSharp
         internal static BoundStatement Rewrite(BoundStatement body, MethodSymbol method, TypeCompilationState compilationState, BindingDiagnosticBag diagnostics)
         {
             var tempSubstitution = PooledDictionary<LocalSymbol, LocalSymbol>.GetInstance();
-            var spiller = new SpillSequenceSpiller(method, body.Syntax, compilationState, tempSubstitution, diagnostics);
+            var receiverSubstitution = PooledDictionary<LocalSymbol, BoundComplexConditionalReceiver>.GetInstance();
+            var spiller = new SpillSequenceSpiller(method, body.Syntax, compilationState, tempSubstitution, receiverSubstitution, diagnostics);
             BoundNode result = spiller.Visit(body);
-            result = LocalSubstituter.Rewrite(tempSubstitution, result);
+            result = LocalSubstituter.Rewrite(tempSubstitution, receiverSubstitution, result);
             tempSubstitution.Free();
+            receiverSubstitution.Free();
             return (BoundStatement)result;
         }
 
@@ -311,11 +335,43 @@ namespace Microsoft.CodeAnalysis.CSharp
                         continue;
 
                     case BoundKind.Sequence:
-                        // neither the side-effects nor the value of the sequence contains await 
-                        // (otherwise it would be converted to a SpillSequenceBuilder).
-                        if (refKind != RefKind.None)
+                        if (refKind != RefKind.None || expression.Type?.IsRefLikeType == true)
                         {
-                            return expression;
+                            var sequence = (BoundSequence)expression;
+
+                            PromoteAndAddLocals(builder, sequence.Locals);
+                            builder.AddExpressions(sequence.SideEffects);
+                            expression = sequence.Value;
+                            continue;
+                        }
+
+                        goto default;
+
+                    case BoundKind.AssignmentOperator:
+                        var assignment = (BoundAssignmentOperator)expression;
+                        if (assignment.IsRef &&
+                            assignment is not { Left.Kind: BoundKind.Local, Right.Kind: BoundKind.ArrayAccess }) // Optimize for some known to be safe scenarios.
+                        {
+                            if (sideEffectsOnly &&
+                                IsComplexConditionalInitializationOfReceiverRef(
+                                    assignment,
+                                    out LocalSymbol receiverRefLocal,
+                                    out BoundComplexConditionalReceiver complexReceiver,
+                                    out BoundLocal valueTypeReceiver,
+                                    out BoundLocal referenceTypeReceiver))
+                            {
+                                Debug.Assert(receiverRefLocal.IsKnownToReferToTempIfReferenceType);
+                                builder.AddStatement(_F.ExpressionStatement(complexReceiver));
+
+                                _receiverSubstitution.Add(receiverRefLocal, complexReceiver.Update(valueTypeReceiver, referenceTypeReceiver, complexReceiver.Type));
+                                return null;
+                            }
+                            else
+                            {
+                                var left = Spill(builder, assignment.Left, RefKind.Ref);
+                                var right = Spill(builder, assignment.Right, RefKind.Ref);
+                                expression = assignment.Update(left, right, assignment.IsRef, assignment.Type);
+                            }
                         }
 
                         goto default;
@@ -373,6 +429,49 @@ namespace Microsoft.CodeAnalysis.CSharp
                         // later, if needed
                         return expression;
 
+                    case BoundKind.Call:
+                        var call = (BoundCall)expression;
+
+                        // If the method is known to have no observable side-effects, we can spill its receiver and arguments,
+                        // and call it later, after some other (unrelated) side-effects are evaluated.
+                        // It is similar to spilling a field/array access.
+                        if (refKind != RefKind.None)
+                        {
+                            if (call.Method.OriginalDefinition is SynthesizedInlineArrayFirstElementRefMethod or SynthesizedInlineArrayFirstElementRefReadOnlyMethod)
+                            {
+                                Debug.Assert(call.Arguments.Length == 1);
+                                return call.Update(ImmutableArray.Create(Spill(builder, call.Arguments[0], call.ArgumentRefKindsOpt[0])));
+                            }
+                            else if (call.Method.OriginalDefinition is SynthesizedInlineArrayElementRefMethod or SynthesizedInlineArrayElementRefReadOnlyMethod)
+                            {
+                                return spillInlineArrayHelperWithTwoArguments(builder, call);
+                            }
+                            else if (call.Method.OriginalDefinition == _F.Compilation.GetWellKnownTypeMember(WellKnownMember.System_Span_T__get_Item) ||
+                                call.Method.OriginalDefinition == _F.Compilation.GetWellKnownTypeMember(WellKnownMember.System_ReadOnlySpan_T__get_Item))
+                            {
+                                Debug.Assert(call.Arguments.Length == 1);
+                                return call.Update(Spill(builder, call.ReceiverOpt, ReceiverSpillRefKind(call.ReceiverOpt)),
+                                                   initialBindingReceiverIsSubjectToCloning: ThreeState.Unknown,
+                                                   call.Method,
+                                                   ImmutableArray.Create(Spill(builder, call.Arguments[0])));
+                            }
+                        }
+                        else if (call.Method.OriginalDefinition is SynthesizedInlineArrayAsSpanMethod or SynthesizedInlineArrayAsReadOnlySpanMethod)
+                        {
+                            return spillInlineArrayHelperWithTwoArguments(builder, call);
+                        }
+                        else if (call.Method.OriginalDefinition == _F.Compilation.GetWellKnownTypeMember(WellKnownMember.System_Span_T__Slice_Int_Int) ||
+                                 call.Method.OriginalDefinition == _F.Compilation.GetWellKnownTypeMember(WellKnownMember.System_ReadOnlySpan_T__Slice_Int_Int))
+                        {
+                            Debug.Assert(call.Arguments.Length == 2);
+                            return call.Update(Spill(builder, call.ReceiverOpt, ReceiverSpillRefKind(call.ReceiverOpt)),
+                                               initialBindingReceiverIsSubjectToCloning: ThreeState.Unknown,
+                                               call.Method,
+                                               ImmutableArray.Create(Spill(builder, call.Arguments[0]), Spill(builder, call.Arguments[1])));
+                        }
+
+                        goto default;
+
                     default:
                         if (expression.Type.IsVoidType() || sideEffectsOnly)
                         {
@@ -396,6 +495,68 @@ namespace Microsoft.CodeAnalysis.CSharp
                         }
                 }
             }
+
+            BoundExpression spillInlineArrayHelperWithTwoArguments(BoundSpillSequenceBuilder builder, BoundCall call)
+            {
+                Debug.Assert(call.Arguments.Length == 2);
+                return call.Update(ImmutableArray.Create(Spill(builder, call.Arguments[0], call.ArgumentRefKindsOpt[0]),
+                                                         call.Arguments[1].ConstantValueOpt is { } ? call.Arguments[1] : Spill(builder, call.Arguments[1])));
+            }
+        }
+
+        internal static bool IsComplexConditionalInitializationOfReceiverRef(
+            BoundAssignmentOperator assignment,
+            out LocalSymbol outReceiverRefLocal,
+            out BoundComplexConditionalReceiver outComplexReceiver,
+            out BoundLocal outValueTypeReceiver,
+            out BoundLocal outReferenceTypeReceiver)
+        {
+            if (assignment is
+                {
+                    IsRef: true,
+                    Left: BoundLocal { LocalSymbol: { SynthesizedKind: SynthesizedLocalKind.LoweringTemp, RefKind: RefKind.Ref } receiverRefLocal },
+                    Right: BoundComplexConditionalReceiver
+                    {
+                        ValueTypeReceiver: BoundLocal { LocalSymbol: { SynthesizedKind: SynthesizedLocalKind.LoweringTemp, RefKind: RefKind.Ref } } valueTypeReceiver,
+                        ReferenceTypeReceiver: BoundSequence
+                        {
+                            Locals.IsEmpty: true,
+                            SideEffects:
+                            [
+                            BoundAssignmentOperator
+                            {
+                                IsRef: false,
+                                Left: BoundLocal { LocalSymbol: { SynthesizedKind: SynthesizedLocalKind.LoweringTemp, RefKind: RefKind.None } referenceTypeClone },
+                                Right: BoundLocal { LocalSymbol: { SynthesizedKind: SynthesizedLocalKind.LoweringTemp, RefKind: RefKind.Ref } originalReceiverReference }
+                            }
+                            ],
+                            Value: BoundLocal { LocalSymbol: { SynthesizedKind: SynthesizedLocalKind.LoweringTemp, RefKind: RefKind.None } } referenceTypeReceiver
+                        }
+                    } complexReceiver,
+                }
+                && (object)referenceTypeClone == referenceTypeReceiver.LocalSymbol
+                && (object)originalReceiverReference == valueTypeReceiver.LocalSymbol
+                && (object)receiverRefLocal != valueTypeReceiver.LocalSymbol
+                && (object)receiverRefLocal != referenceTypeClone
+                && receiverRefLocal.Type.IsTypeParameter()
+                && !receiverRefLocal.Type.IsReferenceType
+                && !receiverRefLocal.Type.IsValueType
+                && valueTypeReceiver.Type.Equals(receiverRefLocal.Type, TypeCompareKind.AllIgnoreOptions)
+                && referenceTypeReceiver.Type.Equals(receiverRefLocal.Type, TypeCompareKind.AllIgnoreOptions)
+            )
+            {
+                outReceiverRefLocal = receiverRefLocal;
+                outComplexReceiver = complexReceiver;
+                outValueTypeReceiver = valueTypeReceiver;
+                outReferenceTypeReceiver = referenceTypeReceiver;
+                return true;
+            }
+
+            outReceiverRefLocal = null;
+            outComplexReceiver = null;
+            outValueTypeReceiver = null;
+            outReferenceTypeReceiver = null;
+            return false;
         }
 
         private ImmutableArray<BoundExpression> VisitExpressionList(
@@ -482,7 +643,7 @@ namespace Microsoft.CodeAnalysis.CSharp
         {
             BoundSpillSequenceBuilder builder = null;
             var expression = VisitExpression(ref builder, node.Expression);
-            return UpdateStatement(builder, node.Update(expression, node.Cases, node.DefaultLabel, node.EqualityMethod));
+            return UpdateStatement(builder, node.Update(expression, node.Cases, node.DefaultLabel, node.LengthBasedStringSwitchDataOpt));
         }
 
         public override BoundNode VisitThrowStatement(BoundThrowStatement node)
@@ -512,7 +673,7 @@ namespace Microsoft.CodeAnalysis.CSharp
         {
             BoundSpillSequenceBuilder builder = null;
             var expression = VisitExpression(ref builder, node.ExpressionOpt);
-            return UpdateStatement(builder, node.Update(node.RefKind, expression));
+            return UpdateStatement(builder, node.Update(node.RefKind, expression, @checked: node.Checked));
         }
 
         public override BoundNode VisitYieldReturnStatement(BoundYieldReturnStatement node)
@@ -561,7 +722,7 @@ namespace Microsoft.CodeAnalysis.CSharp
             // the spilling will occur in the enclosing node.
             BoundSpillSequenceBuilder builder = null;
             var expr = VisitExpression(ref builder, node.Expression);
-            return UpdateExpression(builder, node.Update(expr, node.AwaitableInfo, node.Type));
+            return UpdateExpression(builder, node.Update(expr, node.AwaitableInfo, node.DebugInfo, node.Type));
         }
 
         public override BoundNode VisitSpillSequence(BoundSpillSequence node)
@@ -667,7 +828,9 @@ namespace Microsoft.CodeAnalysis.CSharp
         {
             BoundSpillSequenceBuilder builder = null;
             var operand = VisitExpression(ref builder, node.Operand);
-            return UpdateExpression(builder, node.Update(operand, node.TargetType, node.Conversion, node.Type));
+            Debug.Assert(node.OperandPlaceholder is null);
+            Debug.Assert(node.OperandConversion is null);
+            return UpdateExpression(builder, node.Update(operand, node.TargetType, node.OperandPlaceholder, node.OperandConversion, node.Type));
         }
 
         public override BoundNode VisitAssignmentOperator(BoundAssignmentOperator node)
@@ -790,7 +953,7 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public override BoundNode VisitUserDefinedConditionalLogicalOperator(BoundUserDefinedConditionalLogicalOperator node)
         {
-            throw ExceptionUtilities.Unreachable;
+            throw ExceptionUtilities.Unreachable();
         }
 
         public override BoundNode VisitBinaryOperator(BoundBinaryOperator node)
@@ -826,7 +989,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 }
             }
 
-            return UpdateExpression(builder, node.Update(node.OperatorKind, node.ConstantValue, node.Method, node.ConstrainedToType, node.ResultKind, left, right, node.Type));
+            return UpdateExpression(builder, node.Update(node.OperatorKind, node.ConstantValueOpt, node.Method, node.ConstrainedToType, node.ResultKind, left, right, node.Type));
         }
 
         public override BoundNode VisitCall(BoundCall node)
@@ -847,12 +1010,41 @@ namespace Microsoft.CodeAnalysis.CSharp
                 receiver = node.ReceiverOpt;
                 RefKind refKind = ReceiverSpillRefKind(receiver);
 
+                Debug.Assert(refKind == RefKind.None || !receiver.Type.IsReferenceType);
+
                 receiver = Spill(receiverBuilder, VisitExpression(ref receiverBuilder, receiver), refKind: refKind);
+
+                if (refKind != RefKind.None &&
+                    CodeGenerator.IsPossibleReferenceTypeReceiverOfConstrainedCall(receiver) &&
+                    !CodeGenerator.ReceiverIsKnownToReferToTempIfReferenceType(receiver) &&
+                    !CodeGenerator.IsSafeToDereferenceReceiverRefAfterEvaluatingArguments(node.Arguments))
+                {
+                    var receiverType = receiver.Type;
+                    Debug.Assert(!receiverType.IsReferenceType);
+
+                    // A case where T is actually a class must be handled specially.
+                    // Taking a reference to a class instance is fragile because the value behind the 
+                    // reference might change while arguments are evaluated. However, the call should be
+                    // performed on the instance that is behind reference at the time we push the
+                    // reference to the stack. So, for a class we need to emit a reference to a temporary
+                    // location, rather than to the original location
+
+                    var save_Syntax = _F.Syntax;
+                    _F.Syntax = node.Syntax;
+
+                    var cache = _F.Local(_F.SynthesizedLocal(receiverType));
+                    receiverBuilder.AddLocal(cache.LocalSymbol);
+                    receiverBuilder.AddStatement(_F.ExpressionStatement(new BoundComplexConditionalReceiver(node.Syntax, cache, _F.Sequence(new[] { _F.AssignmentExpression(cache, receiver) }, cache), receiverType) { WasCompilerGenerated = true }));
+
+                    receiver = _F.ComplexConditionalReceiver(receiver, cache);
+                    _F.Syntax = save_Syntax;
+                }
+
                 receiverBuilder.Include(builder);
                 builder = receiverBuilder;
             }
 
-            return UpdateExpression(builder, node.Update(receiver, node.Method, arguments));
+            return UpdateExpression(builder, node.Update(receiver, initialBindingReceiverIsSubjectToCloning: ThreeState.Unknown, node.Method, arguments));
         }
 
         private static RefKind ReceiverSpillRefKind(BoundExpression receiver)
@@ -866,6 +1058,28 @@ namespace Microsoft.CodeAnalysis.CSharp
             return result;
         }
 
+        public override BoundNode VisitFunctionPointerInvocation(BoundFunctionPointerInvocation node)
+        {
+            BoundSpillSequenceBuilder builder = null;
+            var arguments = this.VisitExpressionList(ref builder, node.Arguments, node.ArgumentRefKindsOpt);
+
+            BoundExpression invokedExpression;
+            if (builder == null)
+            {
+                invokedExpression = VisitExpression(ref builder, node.InvokedExpression);
+            }
+            else
+            {
+                var invokedExpressionBuilder = new BoundSpillSequenceBuilder(builder.Syntax);
+
+                invokedExpression = Spill(invokedExpressionBuilder, VisitExpression(ref invokedExpressionBuilder, node.InvokedExpression));
+                invokedExpressionBuilder.Include(builder);
+                builder = invokedExpressionBuilder;
+            }
+
+            return UpdateExpression(builder, node.Update(invokedExpression, arguments, node.ArgumentRefKindsOpt, node.ResultKind, node.Type));
+        }
+
         public override BoundNode VisitConditionalOperator(BoundConditionalOperator node)
         {
             BoundSpillSequenceBuilder conditionBuilder = null;
@@ -937,14 +1151,14 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public override BoundNode VisitMethodGroup(BoundMethodGroup node)
         {
-            throw ExceptionUtilities.Unreachable;
+            throw ExceptionUtilities.Unreachable();
         }
 
         public override BoundNode VisitDelegateCreationExpression(BoundDelegateCreationExpression node)
         {
             BoundSpillSequenceBuilder builder = null;
             var argument = VisitExpression(ref builder, node.Argument);
-            return UpdateExpression(builder, node.Update(argument, node.MethodOpt, node.IsExtensionMethod, node.Type));
+            return UpdateExpression(builder, node.Update(argument, node.MethodOpt, node.IsExtensionMethod, node.WasTargetTyped, node.Type));
         }
 
         public override BoundNode VisitFieldAccess(BoundFieldAccess node)
@@ -958,7 +1172,7 @@ namespace Microsoft.CodeAnalysis.CSharp
         {
             BoundSpillSequenceBuilder builder = null;
             var operand = VisitExpression(ref builder, node.Operand);
-            return UpdateExpression(builder, node.Update(operand, node.TargetType, node.Conversion, node.Type));
+            return UpdateExpression(builder, node.Update(operand, node.TargetType, node.ConversionKind, node.Type));
         }
 
         public override BoundNode VisitMakeRefOperator(BoundMakeRefOperator node)
@@ -970,6 +1184,8 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public override BoundNode VisitNullCoalescingOperator(BoundNullCoalescingOperator node)
         {
+            Debug.Assert(node.LeftPlaceholder is null);
+            Debug.Assert(node.LeftConversion is null);
             BoundSpillSequenceBuilder builder = null;
             var right = VisitExpression(ref builder, node.RightOperand);
             BoundExpression left;
@@ -993,7 +1209,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 return UpdateExpression(leftBuilder, _F.Local(tmp));
             }
 
-            return UpdateExpression(builder, node.Update(left, right, node.LeftConversion, node.OperatorResultKind, node.Type));
+            return UpdateExpression(builder, node.Update(left, right, node.LeftPlaceholder, node.LeftConversion, node.OperatorResultKind, @checked: node.Checked, node.Type));
         }
 
         public override BoundNode VisitLoweredConditionalAccess(BoundLoweredConditionalAccess node)
@@ -1011,7 +1227,7 @@ namespace Microsoft.CodeAnalysis.CSharp
 
             if (whenNotNullBuilder == null && whenNullBuilder == null)
             {
-                return UpdateExpression(receiverBuilder, node.Update(receiver, node.HasValueMethodOpt, whenNotNull, whenNullOpt, node.Id, node.Type));
+                return UpdateExpression(receiverBuilder, node.Update(receiver, node.HasValueMethodOpt, whenNotNull, whenNullOpt, node.Id, node.ForceCopyOfNullableValueType, node.Type));
             }
 
             if (receiverBuilder == null) receiverBuilder = new BoundSpillSequenceBuilder((whenNotNullBuilder ?? whenNullBuilder).Syntax);
@@ -1177,14 +1393,14 @@ namespace Microsoft.CodeAnalysis.CSharp
                 builder = expressionBuilder;
             }
 
-            return UpdateExpression(builder, node.Update(expression, index, node.Checked, node.Type));
+            return UpdateExpression(builder, node.Update(expression, index, node.Checked, node.RefersToLocation, node.Type));
         }
 
         public override BoundNode VisitPointerIndirectionOperator(BoundPointerIndirectionOperator node)
         {
             BoundSpillSequenceBuilder builder = null;
             var operand = VisitExpression(ref builder, node.Operand);
-            return UpdateExpression(builder, node.Update(operand, node.Type));
+            return UpdateExpression(builder, node.Update(operand, node.RefersToLocation, node.Type));
         }
 
         public override BoundNode VisitSequence(BoundSequence node)
@@ -1234,7 +1450,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 {
                     builder.AddLocal(local);
                 }
-                else
+                else if (!_receiverSubstitution.ContainsKey(local))
                 {
                     LocalSymbol longLived = local.WithSynthesizedLocalKindAndSyntax(SynthesizedLocalKind.Spill, _F.Syntax);
                     _tempSubstitution.Add(local, longLived);
