@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System;
 
 namespace Microsoft.CodeAnalysis.CSharp
@@ -18,5 +16,8 @@ namespace Microsoft.CodeAnalysis.CSharp
         NullableContextAttribute = 0x10,
         NullablePublicOnlyAttribute = 0x20,
         NativeIntegerAttribute = 0x40,
+        ScopedRefAttribute = 0x80,
+        RefSafetyRulesAttribute = 0x100,
+        RequiresLocationAttribute = 0x200,
     }
 }
