@@ -47,6 +47,10 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 return mangleName;
             }
         }
+
+        internal sealed override bool IsFileLocal => false;
+        internal sealed override FileIdentifier? AssociatedFileIdentifier => null;
+
         /// <summary>
         /// Get the arity of the missing type.
         /// </summary>
@@ -355,7 +359,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 return other;
             }
 
-            internal sealed override bool IsNativeIntegerType => _isNativeInt;
+            internal sealed override bool IsNativeIntegerWrapperType => _isNativeInt;
 
             internal sealed override NamedTypeSymbol? NativeIntegerUnderlyingType => _isNativeInt ? AsNativeInteger(asNativeInt: false) : null;
 
@@ -430,7 +434,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 }
             }
 
-
             public override SpecialType SpecialType
             {
                 get
@@ -441,7 +444,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
 
             protected override NamedTypeSymbol WithTupleDataCore(TupleExtraData newData)
             {
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             }
 
             public override int GetHashCode()
