@@ -40,7 +40,8 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 {
                     // force resolution of bases in containing type
                     // to make base resolution errors more deterministic
-                    if ((object)ContainingType != null)
+                    if ((object)ContainingType != null &&
+                        TypeKind is not (TypeKind.Enum or TypeKind.Delegate or TypeKind.Submission))
                     {
                         var tmp = ContainingType.BaseTypeNoUseSiteDiagnostics;
                     }
@@ -113,7 +114,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             if (singleDeclaration != null)
             {
                 var corLibrary = this.ContainingAssembly.CorLibrary;
-                var conversions = new TypeConversions(corLibrary);
+                var conversions = corLibrary.TypeConversions;
                 var location = singleDeclaration.NameLocation;
 
                 localBase.CheckAllConstraints(DeclaringCompilation, conversions, location, diagnostics);
@@ -159,7 +160,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             if (singleDeclaration != null)
             {
                 var corLibrary = this.ContainingAssembly.CorLibrary;
-                var conversions = new TypeConversions(corLibrary);
+                var conversions = corLibrary.TypeConversions;
                 var location = singleDeclaration.NameLocation;
 
                 foreach (var pair in interfaces)
@@ -324,18 +325,24 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                             {
                                 baseType = partBase;
                                 baseTypeLocation = decl.NameLocation;
-                                continue;
                             }
-                            else if (containsOnlyOblivious(partBase))
+                            else if (!containsOnlyOblivious(partBase))
                             {
-                                continue;
+                                reportBaseType();
                             }
                         }
+                        else
+                        {
+                            reportBaseType();
+                        }
 
-                        var info = diagnostics.Add(ErrorCode.ERR_PartialMultipleBases, Locations[0], this);
-                        baseType = new ExtendedErrorTypeSymbol(baseType, LookupResultKind.Ambiguous, info);
-                        baseTypeLocation = decl.NameLocation;
-                        reportedPartialConflict = true;
+                        void reportBaseType()
+                        {
+                            var info = diagnostics.Add(ErrorCode.ERR_PartialMultipleBases, GetFirstLocation(), this);
+                            baseType = new ExtendedErrorTypeSymbol(baseType, LookupResultKind.Ambiguous, info);
+                            baseTypeLocation = decl.NameLocation;
+                            reportedPartialConflict = true;
+                        }
 
                         static bool containsOnlyOblivious(TypeSymbol type)
                         {
@@ -384,6 +391,11 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                     // Inconsistent accessibility: base class '{1}' is less accessible than class '{0}'
                     diagnostics.Add(ErrorCode.ERR_BadVisBaseClass, baseTypeLocation, this, baseType);
                 }
+
+                if (baseType.HasFileLocalTypes() && !this.HasFileLocalTypes())
+                {
+                    diagnostics.Add(ErrorCode.ERR_FileTypeBase, baseTypeLocation, baseType, this);
+                }
             }
 
             var baseInterfacesRO = baseInterfaces.ToImmutableAndFree();
@@ -396,12 +408,17 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                         // Inconsistent accessibility: base interface '{1}' is less accessible than interface '{0}'
                         diagnostics.Add(ErrorCode.ERR_BadVisBaseInterface, interfaceLocations[i], this, i);
                     }
+
+                    if (i.HasFileLocalTypes() && !this.HasFileLocalTypes())
+                    {
+                        diagnostics.Add(ErrorCode.ERR_FileTypeBase, interfaceLocations[i], i, this);
+                    }
                 }
             }
 
             interfaceLocations.Free();
 
-            diagnostics.Add(Locations[0], useSiteInfo);
+            diagnostics.Add(GetFirstLocation(), useSiteInfo);
 
             return new Tuple<NamedTypeSymbol, ImmutableArray<NamedTypeSymbol>>(baseType, baseInterfacesRO);
         }
@@ -547,13 +564,13 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                         if (this.IsStatic)
                         {
                             // '{0}': static classes cannot implement interfaces
-                            diagnostics.Add(ErrorCode.ERR_StaticClassInterfaceImpl, location, this, baseType);
+                            diagnostics.Add(ErrorCode.ERR_StaticClassInterfaceImpl, location, this);
                         }
 
                         if (this.IsRefLikeType)
                         {
                             // '{0}': ref structs cannot implement interfaces
-                            diagnostics.Add(ErrorCode.ERR_RefStructInterfaceImpl, location, this, baseType);
+                            diagnostics.Add(ErrorCode.ERR_RefStructInterfaceImpl, location, this);
                         }
 
                         if (baseType.ContainsDynamic())
@@ -614,7 +631,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             void checkPrimaryConstructorBaseType(BaseTypeSyntax baseTypeSyntax, TypeSymbol baseType)
             {
                 if (baseTypeSyntax is PrimaryConstructorBaseTypeSyntax primaryConstructorBaseType &&
-                    (!IsRecord || TypeKind != TypeKind.Class || baseType.TypeKind == TypeKind.Interface || ((RecordDeclarationSyntax)decl.SyntaxReference.GetSyntax()).ParameterList is null))
+                    (TypeKind != TypeKind.Class || baseType.TypeKind == TypeKind.Interface || ((TypeDeclarationSyntax)decl.SyntaxReference.GetSyntax()).ParameterList is null))
                 {
                     diagnostics.Add(ErrorCode.ERR_UnexpectedArgumentList, primaryConstructorBaseType.ArgumentList.Location);
                 }
@@ -660,7 +677,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                     if (BaseTypeAnalysis.TypeDependsOn(depends: t, on: this))
                     {
                         result.Add(new ExtendedErrorTypeSymbol(t, LookupResultKind.NotReferencable,
-                            diagnostics.Add(ErrorCode.ERR_CycleInInterfaceInheritance, Locations[0], this, t)));
+                            diagnostics.Add(ErrorCode.ERR_CycleInInterfaceInheritance, GetFirstLocation(), this, t)));
                         continue;
                     }
                     else
@@ -684,7 +701,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                     }
                 }
 
-                diagnostics.Add(Locations[0], useSiteInfo);
+                diagnostics.Add(GetFirstLocation(), useSiteInfo);
             }
 
             return isInterface ? result.ToImmutableAndFree() : declaredInterfaces;
@@ -738,7 +755,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             if (BaseTypeAnalysis.TypeDependsOn(declaredBase, this))
             {
                 return new ExtendedErrorTypeSymbol(declaredBase, LookupResultKind.NotReferencable,
-                    diagnostics.Add(ErrorCode.ERR_CircularBase, Locations[0], declaredBase, this));
+                    diagnostics.Add(ErrorCode.ERR_CircularBase, GetFirstLocation(), declaredBase, this));
             }
 
             this.SetKnownToHaveNoDeclaredBaseCycles();
@@ -758,7 +775,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             }
             while ((object)current != null);
 
-            diagnostics.Add(useSiteInfo.Diagnostics.IsNullOrEmpty() ? Location.None : (FindBaseRefSyntax(declaredBase) ?? Locations[0]), useSiteInfo);
+            diagnostics.Add(useSiteInfo.Diagnostics.IsNullOrEmpty() ? Location.None : (FindBaseRefSyntax(declaredBase) ?? GetFirstLocation()), useSiteInfo);
 
             return declaredBase;
         }
