@@ -237,7 +237,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             }
         }
 
-
         public sealed override ImmutableArray<CustomModifier> RefCustomModifiers
         {
             get
@@ -353,24 +352,32 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
 
         internal override int CalculateLocalSyntaxOffset(int localPosition, SyntaxTree localTree)
         {
-            throw ExceptionUtilities.Unreachable;
+            throw ExceptionUtilities.Unreachable();
         }
 
-        internal override bool IsNullableAnalysisEnabled() => throw ExceptionUtilities.Unreachable;
+        internal override bool IsNullableAnalysisEnabled() => throw ExceptionUtilities.Unreachable();
 
         private int ComputeHashCode()
         {
             int code = this.OriginalDefinition.GetHashCode();
+            int containingHashCode;
 
-            // If the containing type of the original definition is the same as our containing type
-            // it's possible that we will compare equal to the original definition under certain conditions 
-            // (e.g, ignoring nullability) and want to retain the same hashcode. As such, consider only
-            // the original definition for the hashcode when we know equality is possible
-            var containingHashCode = _containingType.GetHashCode();
-            if (containingHashCode == this.OriginalDefinition.ContainingType.GetHashCode() &&
-                wasConstructedForAnnotations(this))
+            if (OriginalDefinition is SynthesizedGlobalMethodSymbol global)
             {
-                return code;
+                containingHashCode = System.Runtime.CompilerServices.RuntimeHelpers.GetHashCode(global.ContainingPrivateImplementationDetailsType);
+            }
+            else
+            {
+                // If the containing type of the original definition is the same as our containing type
+                // it's possible that we will compare equal to the original definition under certain conditions 
+                // (e.g, ignoring nullability) and want to retain the same hashcode. As such, consider only
+                // the original definition for the hashcode when we know equality is possible
+                containingHashCode = _containingType.GetHashCode();
+                if (containingHashCode == this.OriginalDefinition.ContainingType.GetHashCode() &&
+                    wasConstructedForAnnotations(this))
+                {
+                    return code;
+                }
             }
 
             code = Hash.Combine(containingHashCode, code);
