@@ -11,6 +11,7 @@ using System.Threading;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.RuntimeMembers;
 using Roslyn.Utilities;
+using ReferenceEqualityComparer = Roslyn.Utilities.ReferenceEqualityComparer;
 
 namespace Microsoft.CodeAnalysis.CSharp.Symbols
 {
@@ -46,7 +47,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
 
             if (numElements <= 1)
             {
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             }
 
             NamedTypeSymbol underlyingType = getTupleUnderlyingType(elementTypesWithAnnotations, syntax, compilation, diagnostics);
@@ -316,7 +317,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
         {
             if (arity > ValueTupleRestPosition)
             {
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             }
             return tupleTypes[arity - 1];
         }
@@ -344,7 +345,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
         {
             if (arity > 8)
             {
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             }
             return tupleCtors[arity - 1];
         }
@@ -447,7 +448,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 return 0;
             }
 
-            return matchesCanonicalElementName(name);
+            return MatchesCanonicalTupleElementName(name);
 
             static bool isElementNameForbidden(string name)
             {
@@ -465,25 +466,25 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                         return false;
                 }
             }
+        }
 
-            // Returns 3 for "Item3".
-            // Returns -1 otherwise.
-            static int matchesCanonicalElementName(string name)
+        // Returns 3 for "Item3".
+        // Returns -1 otherwise.
+        internal static int MatchesCanonicalTupleElementName(string name)
+        {
+            if (name.StartsWith("Item", StringComparison.Ordinal))
             {
-                if (name.StartsWith("Item", StringComparison.Ordinal))
+                string tail = name.Substring("Item".Length);
+                if (int.TryParse(tail, out int number))
                 {
-                    string tail = name.Substring(4);
-                    if (int.TryParse(tail, out int number))
+                    if (number > 0 && string.Equals(name, TupleMemberName(number), StringComparison.Ordinal))
                     {
-                        if (number > 0 && string.Equals(name, TupleMemberName(number), StringComparison.Ordinal))
-                        {
-                            return number;
-                        }
+                        return number;
                     }
                 }
-
-                return -1;
             }
+
+            return -1;
         }
 
         /// <summary>
@@ -506,7 +507,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                     useSiteInfo = useSiteInfo.AdjustDiagnosticInfo(null);
                 }
 
-                diagnostics.Add(useSiteInfo, syntax?.GetLocation() ?? Location.None);
+                diagnostics.Add(useSiteInfo, static syntax => syntax?.Location ?? Location.None, syntax);
             }
 
             return member;
@@ -567,36 +568,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
         public sealed override ImmutableArray<FieldSymbol> TupleElements
             => IsTupleType ? TupleData!.TupleElements(this) : default;
 
-        /// <summary>
-        /// For tuple fields that aren't TupleElementFieldSymbol or TupleErrorFieldSymbol, we cache their tuple element index.
-        /// This supports <see cref="FieldSymbol.TupleElementIndex"/>.
-        /// For those fields, we map from their definition to an index.
-        /// </summary>
-        public SmallDictionary<FieldSymbol, int>? TupleFieldDefinitionsToIndexMap
-        {
-            get
-            {
-                if (!IsTupleType)
-                {
-                    return null;
-                }
-
-                if (!IsDefinition)
-                {
-                    return this.OriginalDefinition.TupleFieldDefinitionsToIndexMap;
-                }
-
-                return TupleData!.GetFieldDefinitionsToIndexMap(this);
-            }
-        }
-
-        public virtual void InitializeTupleFieldDefinitionsToIndexMap()
-        {
-            Debug.Assert(this.IsTupleType);
-            Debug.Assert(this.IsDefinition); // we only store a map for definitions
-            _ = this.GetMembers();
-        }
-
         public TMember? GetTupleMemberSymbolForUnderlyingMember<TMember>(TMember? underlyingMemberOpt) where TMember : Symbol
         {
             return IsTupleType ? TupleData!.GetTupleMemberSymbolForUnderlyingMember(underlyingMemberOpt) : null;
@@ -611,10 +582,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             var elementsMatchedByFields = ArrayBuilder<bool>.GetInstance(elementTypes.Length, fillWithValue: false);
             var members = ArrayBuilder<Symbol>.GetInstance(currentMembers.Length);
 
-            // For tuple fields that aren't TupleElementFieldSymbol or TupleErrorFieldSymbol, we cache/map their tuple element index
-            // corresponding to their definition. We only need to do that for the definition of ValueTuple types.
-            var fieldDefinitionsToIndexMap = IsDefinition ? new SmallDictionary<FieldSymbol, int>(ReferenceEqualityComparer.Instance) : null;
-
             NamedTypeSymbol currentValueTuple = this;
             int currentNestingLevel = 0;
 
@@ -692,7 +659,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                                     if (IsDefinition)
                                     {
                                         defaultTupleField = field;
-                                        fieldDefinitionsToIndexMap!.Add(field, tupleFieldIndex);
                                     }
                                     else
                                     {
@@ -819,10 +785,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
             }
 
             elementsMatchedByFields.Free();
-            if (fieldDefinitionsToIndexMap is object)
-            {
-                this.TupleData!.SetFieldDefinitionsToIndexMap(fieldDefinitionsToIndexMap);
-            }
             return members;
 
             // Returns the nested type at a certain depth.
@@ -847,11 +809,11 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 for (int i = 0; i < fieldsPerType; i++)
                 {
                     WellKnownMember wellKnownTupleField = GetTupleTypeMember(arity, i + 1);
-                    fieldsForElements.Add((FieldSymbol?)GetWellKnownMemberInType(members, wellKnownTupleField));
+                    fieldsForElements.Add((FieldSymbol?)getWellKnownMemberInType(members, wellKnownTupleField));
                 }
             }
 
-            static Symbol? GetWellKnownMemberInType(ImmutableArray<Symbol> members, WellKnownMember relativeMember)
+            static Symbol? getWellKnownMemberInType(ImmutableArray<Symbol> members, WellKnownMember relativeMember)
             {
                 Debug.Assert(relativeMember >= WellKnownMember.System_ValueTuple_T1__Item1 && relativeMember <= WellKnownMember.System_ValueTuple_TRest__ctor);
 
@@ -954,13 +916,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
 
             private ImmutableArray<FieldSymbol> _lazyDefaultElementFields;
 
-            /// <summary>
-            /// For tuple fields that aren't TupleElementFieldSymbol or TupleErrorFieldSymbol, we cache their tuple element index.
-            /// This supports <see cref="FieldSymbol.TupleElementIndex"/>.
-            /// For those fields, we map from their definition to an index.
-            /// </summary>
-            private SmallDictionary<FieldSymbol, int>? _lazyFieldDefinitionsToIndexMap;
-
             private SmallDictionary<Symbol, Symbol>? _lazyUnderlyingDefinitionToMemberMap;
 
             /// <summary>
@@ -1093,26 +1048,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Symbols
                 }
             }
 
-            internal SmallDictionary<FieldSymbol, int> GetFieldDefinitionsToIndexMap(NamedTypeSymbol tuple)
-            {
-                Debug.Assert(tuple.IsTupleType);
-                Debug.Assert(tuple.IsDefinition); // we only store a map for definitions
-                if (_lazyFieldDefinitionsToIndexMap is null)
-                {
-                    tuple.InitializeTupleFieldDefinitionsToIndexMap();
-                }
-
-                Debug.Assert(_lazyFieldDefinitionsToIndexMap is object);
-                return _lazyFieldDefinitionsToIndexMap;
-            }
-
-            internal void SetFieldDefinitionsToIndexMap(SmallDictionary<FieldSymbol, int> map)
-            {
-                Debug.Assert(map.Keys.All(k => k.IsDefinition));
-                Debug.Assert(map.Values.All(v => v >= 0));
-                Interlocked.CompareExchange(ref _lazyFieldDefinitionsToIndexMap, map, null);
-            }
-
             internal SmallDictionary<Symbol, Symbol> UnderlyingDefinitionToMemberMap
             {
                 get
