@@ -202,7 +202,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Syntax
 
             if (methodDecl.TypeParameterList == null)
             {
-                // no type parameters => nothing can be in their scope
+                // no type parameters => they are not in scope
                 return false;
             }
 
@@ -220,14 +220,41 @@ namespace Microsoft.CodeAnalysis.CSharp.Syntax
 
             var explicitInterfaceSpecifier = methodDecl.ExplicitInterfaceSpecifier;
             var firstNameToken = explicitInterfaceSpecifier == null ? methodDecl.Identifier : explicitInterfaceSpecifier.GetFirstToken();
-
-            var typeParams = methodDecl.TypeParameterList;
-            var firstPostNameToken = typeParams == null ? methodDecl.ParameterList.OpenParenToken : typeParams.LessThanToken;
+            var firstPostNameToken = methodDecl.TypeParameterList.LessThanToken;
 
             // Scope does not include method name.
             return !IsBetweenTokens(position, firstNameToken, firstPostNameToken);
         }
 
+        internal static bool IsInLocalFunctionTypeParameterScope(int position, LocalFunctionStatementSyntax localFunction)
+        {
+            Debug.Assert(localFunction != null);
+
+            if (localFunction.TypeParameterList == null)
+            {
+                // no type parameters => they are not in scope
+                return false;
+            }
+
+            // optimization for a common case - when position is in the ReturnType, we can see type parameters
+            if (localFunction.ReturnType.FullSpan.Contains(position))
+            {
+                return true;
+            }
+
+            // Must be in the local function, but not in an attribute on the method.
+            if (IsInAttributeSpecification(position, localFunction.AttributeLists))
+            {
+                return false;
+            }
+
+            var firstNameToken = localFunction.Identifier;
+            var firstPostNameToken = localFunction.TypeParameterList.LessThanToken;
+
+            // Scope does not include local function name.
+            return !IsBetweenTokens(position, firstNameToken, firstPostNameToken);
+        }
+
         /// <remarks>
         /// Used to determine whether it would be appropriate to use the binder for the statement (if any).
         /// Not used to determine whether the position is syntactically within the statement.
