@@ -45,6 +45,14 @@ namespace Microsoft.CodeAnalysis.CSharp
             {
                 case SyntaxKind.AssemblyKeyword:
                 case SyntaxKind.ModuleKeyword:
+                case SyntaxKind.EventKeyword:
+                case SyntaxKind.FieldKeyword:
+                case SyntaxKind.MethodKeyword:
+                case SyntaxKind.ParamKeyword:
+                case SyntaxKind.PropertyKeyword:
+                case SyntaxKind.ReturnKeyword:
+                case SyntaxKind.TypeKeyword:
+                case SyntaxKind.TypeVarKeyword:
                     return true;
                 default:
                     return false;
@@ -143,7 +151,7 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public static bool IsPunctuation(SyntaxKind kind)
         {
-            return kind >= SyntaxKind.TildeToken && kind <= SyntaxKind.QuestionQuestionEqualsToken;
+            return kind >= SyntaxKind.TildeToken && kind <= SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken;
         }
 
         public static bool IsLanguagePunctuation(SyntaxKind kind)
@@ -164,7 +172,7 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public static IEnumerable<SyntaxKind> GetPunctuationKinds()
         {
-            for (int i = (int)SyntaxKind.TildeToken; i <= (int)SyntaxKind.PercentEqualsToken; i++)
+            for (int i = (int)SyntaxKind.TildeToken; i <= (int)SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken; i++)
             {
                 yield return (SyntaxKind)i;
             }
@@ -181,12 +189,16 @@ namespace Microsoft.CodeAnalysis.CSharp
             {
                 case SyntaxKind.IdentifierToken:
                 case SyntaxKind.StringLiteralToken:
+                case SyntaxKind.Utf8StringLiteralToken:
+                case SyntaxKind.SingleLineRawStringLiteralToken:
+                case SyntaxKind.Utf8SingleLineRawStringLiteralToken:
+                case SyntaxKind.MultiLineRawStringLiteralToken:
+                case SyntaxKind.Utf8MultiLineRawStringLiteralToken:
                 case SyntaxKind.CharacterLiteralToken:
                 case SyntaxKind.NumericLiteralToken:
                 case SyntaxKind.XmlTextLiteralToken:
                 case SyntaxKind.XmlTextLiteralNewLineToken:
                 case SyntaxKind.XmlEntityLiteralToken:
-                    //case SyntaxKind.Unknown:
                     return true;
                 default:
                     return false;
@@ -201,12 +213,17 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case SyntaxKind.InterpolatedStringToken:
                 case SyntaxKind.InterpolatedStringStartToken:
                 case SyntaxKind.InterpolatedVerbatimStringStartToken:
+                case SyntaxKind.InterpolatedMultiLineRawStringStartToken:
+                case SyntaxKind.InterpolatedSingleLineRawStringStartToken:
                 case SyntaxKind.InterpolatedStringTextToken:
                 case SyntaxKind.InterpolatedStringEndToken:
+                case SyntaxKind.InterpolatedRawStringEndToken:
                 case SyntaxKind.LoadKeyword:
                 case SyntaxKind.NullableKeyword:
                 case SyntaxKind.EnableKeyword:
                 case SyntaxKind.UnderscoreToken:
+                case SyntaxKind.MultiLineRawStringLiteralToken:
+                case SyntaxKind.SingleLineRawStringLiteralToken:
                     return true;
                 default:
                     return false;
@@ -468,6 +485,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case SyntaxKind.GreaterThanToken:
                 case SyntaxKind.GreaterThanEqualsToken:
                 case SyntaxKind.GreaterThanGreaterThanToken:
+                case SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
                 case SyntaxKind.ExclamationEqualsToken:
                     return true;
                 default:
@@ -530,25 +548,22 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public static SyntaxKind GetLiteralExpression(SyntaxKind token)
         {
-            switch (token)
+            return token switch
             {
-                case SyntaxKind.StringLiteralToken:
-                    return SyntaxKind.StringLiteralExpression;
-                case SyntaxKind.CharacterLiteralToken:
-                    return SyntaxKind.CharacterLiteralExpression;
-                case SyntaxKind.NumericLiteralToken:
-                    return SyntaxKind.NumericLiteralExpression;
-                case SyntaxKind.NullKeyword:
-                    return SyntaxKind.NullLiteralExpression;
-                case SyntaxKind.TrueKeyword:
-                    return SyntaxKind.TrueLiteralExpression;
-                case SyntaxKind.FalseKeyword:
-                    return SyntaxKind.FalseLiteralExpression;
-                case SyntaxKind.ArgListKeyword:
-                    return SyntaxKind.ArgListExpression;
-                default:
-                    return SyntaxKind.None;
-            }
+                SyntaxKind.StringLiteralToken => SyntaxKind.StringLiteralExpression,
+                SyntaxKind.Utf8StringLiteralToken => SyntaxKind.Utf8StringLiteralExpression,
+                SyntaxKind.SingleLineRawStringLiteralToken => SyntaxKind.StringLiteralExpression,
+                SyntaxKind.Utf8SingleLineRawStringLiteralToken => SyntaxKind.Utf8StringLiteralExpression,
+                SyntaxKind.MultiLineRawStringLiteralToken => SyntaxKind.StringLiteralExpression,
+                SyntaxKind.Utf8MultiLineRawStringLiteralToken => SyntaxKind.Utf8StringLiteralExpression,
+                SyntaxKind.CharacterLiteralToken => SyntaxKind.CharacterLiteralExpression,
+                SyntaxKind.NumericLiteralToken => SyntaxKind.NumericLiteralExpression,
+                SyntaxKind.NullKeyword => SyntaxKind.NullLiteralExpression,
+                SyntaxKind.TrueKeyword => SyntaxKind.TrueLiteralExpression,
+                SyntaxKind.FalseKeyword => SyntaxKind.FalseLiteralExpression,
+                SyntaxKind.ArgListKeyword => SyntaxKind.ArgListExpression,
+                _ => SyntaxKind.None,
+            };
         }
 
         public static bool IsInstanceExpression(SyntaxKind token)
@@ -611,6 +626,8 @@ namespace Microsoft.CodeAnalysis.CSharp
                     return SyntaxKind.LeftShiftExpression;
                 case SyntaxKind.GreaterThanGreaterThanToken:
                     return SyntaxKind.RightShiftExpression;
+                case SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
+                    return SyntaxKind.UnsignedRightShiftExpression;
                 case SyntaxKind.PlusToken:
                     return SyntaxKind.AddExpression;
                 case SyntaxKind.MinusToken:
@@ -640,6 +657,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case SyntaxKind.ExclusiveOrAssignmentExpression:
                 case SyntaxKind.LeftShiftAssignmentExpression:
                 case SyntaxKind.RightShiftAssignmentExpression:
+                case SyntaxKind.UnsignedRightShiftAssignmentExpression:
                 case SyntaxKind.AddAssignmentExpression:
                 case SyntaxKind.SubtractAssignmentExpression:
                 case SyntaxKind.MultiplyAssignmentExpression:
@@ -662,6 +680,7 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case SyntaxKind.CaretEqualsToken:
                 case SyntaxKind.LessThanLessThanEqualsToken:
                 case SyntaxKind.GreaterThanGreaterThanEqualsToken:
+                case SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken:
                 case SyntaxKind.PlusEqualsToken:
                 case SyntaxKind.MinusEqualsToken:
                 case SyntaxKind.AsteriskEqualsToken:
@@ -688,6 +707,8 @@ namespace Microsoft.CodeAnalysis.CSharp
                     return SyntaxKind.LeftShiftAssignmentExpression;
                 case SyntaxKind.GreaterThanGreaterThanEqualsToken:
                     return SyntaxKind.RightShiftAssignmentExpression;
+                case SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken:
+                    return SyntaxKind.UnsignedRightShiftAssignmentExpression;
                 case SyntaxKind.PlusEqualsToken:
                     return SyntaxKind.AddAssignmentExpression;
                 case SyntaxKind.MinusEqualsToken:
@@ -979,21 +1000,39 @@ namespace Microsoft.CodeAnalysis.CSharp
         {
             switch (operatorMetadataName)
             {
-                case WellKnownMemberNames.AdditionOperatorName: return SyntaxKind.PlusToken;
+                case WellKnownMemberNames.CheckedAdditionOperatorName:
+                case WellKnownMemberNames.AdditionOperatorName:
+                    return SyntaxKind.PlusToken;
+
                 case WellKnownMemberNames.BitwiseAndOperatorName: return SyntaxKind.AmpersandToken;
                 case WellKnownMemberNames.BitwiseOrOperatorName: return SyntaxKind.BarToken;
                 // case WellKnownMemberNames.ConcatenateOperatorName:
-                case WellKnownMemberNames.DecrementOperatorName: return SyntaxKind.MinusMinusToken;
-                case WellKnownMemberNames.DivisionOperatorName: return SyntaxKind.SlashToken;
+
+                case WellKnownMemberNames.CheckedDecrementOperatorName:
+                case WellKnownMemberNames.DecrementOperatorName:
+                    return SyntaxKind.MinusMinusToken;
+
+                case WellKnownMemberNames.CheckedDivisionOperatorName:
+                case WellKnownMemberNames.DivisionOperatorName:
+                    return SyntaxKind.SlashToken;
+
                 case WellKnownMemberNames.EqualityOperatorName: return SyntaxKind.EqualsEqualsToken;
                 case WellKnownMemberNames.ExclusiveOrOperatorName: return SyntaxKind.CaretToken;
-                case WellKnownMemberNames.ExplicitConversionName: return SyntaxKind.ExplicitKeyword;
+
+                case WellKnownMemberNames.CheckedExplicitConversionName:
+                case WellKnownMemberNames.ExplicitConversionName:
+                    return SyntaxKind.ExplicitKeyword;
+
                 // case WellKnownMemberNames.ExponentOperatorName:
                 case WellKnownMemberNames.FalseOperatorName: return SyntaxKind.FalseKeyword;
                 case WellKnownMemberNames.GreaterThanOperatorName: return SyntaxKind.GreaterThanToken;
                 case WellKnownMemberNames.GreaterThanOrEqualOperatorName: return SyntaxKind.GreaterThanEqualsToken;
                 case WellKnownMemberNames.ImplicitConversionName: return SyntaxKind.ImplicitKeyword;
-                case WellKnownMemberNames.IncrementOperatorName: return SyntaxKind.PlusPlusToken;
+
+                case WellKnownMemberNames.CheckedIncrementOperatorName:
+                case WellKnownMemberNames.IncrementOperatorName:
+                    return SyntaxKind.PlusPlusToken;
+
                 case WellKnownMemberNames.InequalityOperatorName: return SyntaxKind.ExclamationEqualsToken;
                 //case WellKnownMemberNames.IntegerDivisionOperatorName: 
                 case WellKnownMemberNames.LeftShiftOperatorName: return SyntaxKind.LessThanLessThanToken;
@@ -1002,18 +1041,50 @@ namespace Microsoft.CodeAnalysis.CSharp
                 // case WellKnownMemberNames.LikeOperatorName:
                 case WellKnownMemberNames.LogicalNotOperatorName: return SyntaxKind.ExclamationToken;
                 case WellKnownMemberNames.ModulusOperatorName: return SyntaxKind.PercentToken;
-                case WellKnownMemberNames.MultiplyOperatorName: return SyntaxKind.AsteriskToken;
+
+                case WellKnownMemberNames.CheckedMultiplyOperatorName:
+                case WellKnownMemberNames.MultiplyOperatorName:
+                    return SyntaxKind.AsteriskToken;
+
                 case WellKnownMemberNames.OnesComplementOperatorName: return SyntaxKind.TildeToken;
                 case WellKnownMemberNames.RightShiftOperatorName: return SyntaxKind.GreaterThanGreaterThanToken;
-                case WellKnownMemberNames.SubtractionOperatorName: return SyntaxKind.MinusToken;
+                case WellKnownMemberNames.UnsignedRightShiftOperatorName: return SyntaxKind.GreaterThanGreaterThanGreaterThanToken;
+
+                case WellKnownMemberNames.CheckedSubtractionOperatorName:
+                case WellKnownMemberNames.SubtractionOperatorName:
+                    return SyntaxKind.MinusToken;
+
                 case WellKnownMemberNames.TrueOperatorName: return SyntaxKind.TrueKeyword;
-                case WellKnownMemberNames.UnaryNegationOperatorName: return SyntaxKind.MinusToken;
+
+                case WellKnownMemberNames.CheckedUnaryNegationOperatorName:
+                case WellKnownMemberNames.UnaryNegationOperatorName:
+                    return SyntaxKind.MinusToken;
+
                 case WellKnownMemberNames.UnaryPlusOperatorName: return SyntaxKind.PlusToken;
                 default:
                     return SyntaxKind.None;
             }
         }
 
+        public static bool IsCheckedOperator(string operatorMetadataName)
+        {
+            switch (operatorMetadataName)
+            {
+                case WellKnownMemberNames.CheckedDecrementOperatorName:
+                case WellKnownMemberNames.CheckedIncrementOperatorName:
+                case WellKnownMemberNames.CheckedUnaryNegationOperatorName:
+                case WellKnownMemberNames.CheckedAdditionOperatorName:
+                case WellKnownMemberNames.CheckedDivisionOperatorName:
+                case WellKnownMemberNames.CheckedMultiplyOperatorName:
+                case WellKnownMemberNames.CheckedSubtractionOperatorName:
+                case WellKnownMemberNames.CheckedExplicitConversionName:
+                    return true;
+
+                default:
+                    return false;
+            }
+        }
+
         public static SyntaxKind GetPreprocessorKeywordKind(string text)
         {
             switch (text)
@@ -1075,7 +1146,7 @@ namespace Microsoft.CodeAnalysis.CSharp
 
         public static IEnumerable<SyntaxKind> GetContextualKeywordKinds()
         {
-            for (int i = (int)SyntaxKind.YieldKeyword; i <= (int)SyntaxKind.UnmanagedKeyword; i++)
+            for (int i = (int)SyntaxKind.YieldKeyword; i <= (int)SyntaxKind.FileKeyword; i++)
             {
                 yield return (SyntaxKind)i;
             }
@@ -1128,6 +1199,9 @@ namespace Microsoft.CodeAnalysis.CSharp
                 case SyntaxKind.RecordKeyword:
                 case SyntaxKind.ManagedKeyword:
                 case SyntaxKind.UnmanagedKeyword:
+                case SyntaxKind.RequiredKeyword:
+                case SyntaxKind.ScopedKeyword:
+                case SyntaxKind.FileKeyword:
                     return true;
                 default:
                     return false;
@@ -1247,6 +1321,12 @@ namespace Microsoft.CodeAnalysis.CSharp
                     return SyntaxKind.ManagedKeyword;
                 case "unmanaged":
                     return SyntaxKind.UnmanagedKeyword;
+                case "required":
+                    return SyntaxKind.RequiredKeyword;
+                case "scoped":
+                    return SyntaxKind.ScopedKeyword;
+                case "file":
+                    return SyntaxKind.FileKeyword;
                 default:
                     return SyntaxKind.None;
             }
@@ -1364,6 +1444,10 @@ namespace Microsoft.CodeAnalysis.CSharp
                     return ">>";
                 case SyntaxKind.GreaterThanGreaterThanEqualsToken:
                     return ">>=";
+                case SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
+                    return ">>>";
+                case SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken:
+                    return ">>>=";
                 case SyntaxKind.SlashEqualsToken:
                     return "/=";
                 case SyntaxKind.AsteriskEqualsToken:
@@ -1682,6 +1766,12 @@ namespace Microsoft.CodeAnalysis.CSharp
                     return "managed";
                 case SyntaxKind.UnmanagedKeyword:
                     return "unmanaged";
+                case SyntaxKind.RequiredKeyword:
+                    return "required";
+                case SyntaxKind.ScopedKeyword:
+                    return "scoped";
+                case SyntaxKind.FileKeyword:
+                    return "file";
                 default:
                     return string.Empty;
             }
