@@ -362,7 +362,6 @@ Diagnostic(ErrorCode.ERR_AmbigUDConv, "default(N?)").WithArguments("N.implicit o
 Diagnostic(ErrorCode.ERR_AmbigUDConv, "default(P?)").WithArguments("P.implicit operator G(P)", "P.implicit operator G(P?)", "P?", "G?")
                 );
 
-
             // More cases where the specification indicates that a conversion should be legal,
             // but the native compiler disallows it. Roslyn follows the native compiler in these cases.
 
@@ -665,7 +664,6 @@ class Z
 }
 ";
 
-
             var comp = CreateCompilation(source1 + source3);
             comp.VerifyDiagnostics();
 
@@ -1283,7 +1281,6 @@ class D<T> : C<T>
     }
 }";
 
-
             var verifier = CompileAndVerify(source, expectedOutput: "23");
         }
 
@@ -1538,7 +1535,7 @@ class InArgument<T>
     public static implicit operator InArgument<T>(T t) { return default(InArgument<T>); }
 }
 
-public struct start
+public struct @start
 {
     static public void Main()
     {
@@ -1647,5 +1644,293 @@ class C<T>
                 //         return (C<object>) M1();
                 Diagnostic(ErrorCode.ERR_NoExplicitConv, "(C<object>) M1()").WithArguments("void", "C<object>").WithLocation(9, 16));
         }
+
+        [Fact, WorkItem(56646, "https://github.com/dotnet/roslyn/issues/56646")]
+        public void LiftedConversion_InvalidTypeArgument01()
+        {
+            var code = @"
+int? i = null;
+C c = i;
+
+class C
+{
+    public static implicit operator C(long l) => throw null;
+}
+
+namespace System
+{
+    public class Object {}
+    public class String {}
+    public class Exception {}
+    public class ValueType : Object {}
+    public struct Void {}
+    public struct Int32 {}
+    public struct Nullable<T> where T : struct {}
+    public ref struct Int64 {}
+}
+";
+
+            var comp = CreateEmptyCompilation(code);
+            comp.VerifyDiagnostics(
+                // (3,7): error CS0266: Cannot implicitly convert type 'int?' to 'C'. An explicit conversion exists (are you missing a cast?)
+                // C c = i;
+                Diagnostic(ErrorCode.ERR_NoImplicitConvCast, "i").WithArguments("int?", "C").WithLocation(3, 7)
+            );
+        }
+
+        [Fact, WorkItem(56646, "https://github.com/dotnet/roslyn/issues/56646")]
+        public void LiftedConversion_InvalidTypeArgument02()
+        {
+            var code = @"
+C c = null;
+M(c);
+
+void M(long? i) => throw null;
+
+class C
+{
+    public static implicit operator int(C c) => throw null;
+}
+
+namespace System
+{
+    public class Object {}
+    public class String {}
+    public class Exception {}
+    public class ValueType : Object {}
+    public struct Void {}
+    public ref struct Int32 {}
+    public struct Nullable<T> where T : struct { public Nullable(T value) {} }
+    public struct Int64 {}
+    public struct Boolean { }
+    public class Attribute { }
+    public class AttributeUsageAttribute : Attribute
+    {
+        public AttributeUsageAttribute(AttributeTargets t) { }
+        public bool AllowMultiple { get; set; }
+        public bool Inherited { get; set; }
+    }
+    public struct Enum { }
+    public enum AttributeTargets { }
+}
+";
+
+            var comp = CreateEmptyCompilation(code);
+            var verifier = CompileAndVerify(comp, verify: Verification.Skipped);
+
+            // Note that no int? is being created.
+            verifier.VerifyIL("<top-level-statements-entry-point>", @"
+{
+  // Code size       18 (0x12)
+  .maxstack  1
+  IL_0000:  ldnull
+  IL_0001:  call       ""int C.op_Implicit(C)""
+  IL_0006:  conv.i8
+  IL_0007:  newobj     ""long?..ctor(long)""
+  IL_000c:  call       ""void Program.<<Main>$>g__M|0_0(long?)""
+  IL_0011:  ret
+}
+");
+        }
+
+        [Fact, WorkItem(56646, "https://github.com/dotnet/roslyn/issues/56646")]
+        public void LiftedConversion_InvalidTypeArgument03()
+        {
+            var code = @"
+int? i = null;
+C c = (C)i;
+
+class C
+{
+    public static explicit operator C(long l) => throw null;
+}
+
+namespace System
+{
+    public class Object {}
+    public class String {}
+    public class Exception {}
+    public class ValueType : Object {}
+    public struct Void {}
+    public struct Int32 {}
+    public struct Nullable<T> where T : struct { public T Value { get => throw null; } }
+    public ref struct Int64 {}
+    public struct Boolean { }
+    public class Attribute { }
+    public class AttributeUsageAttribute : Attribute
+    {
+        public AttributeUsageAttribute(AttributeTargets t) { }
+        public bool AllowMultiple { get; set; }
+        public bool Inherited { get; set; }
+    }
+    public struct Enum { }
+    public enum AttributeTargets { }
+}
+";
+
+            var comp = CreateEmptyCompilation(code);
+            var verifier = CompileAndVerify(comp, verify: Verification.Skipped);
+
+            verifier.VerifyIL("<top-level-statements-entry-point>", @"
+{
+  // Code size       23 (0x17)
+  .maxstack  1
+  .locals init (int? V_0) //i
+  IL_0000:  ldloca.s   V_0
+  IL_0002:  initobj    ""int?""
+  IL_0008:  ldloca.s   V_0
+  IL_000a:  call       ""int int?.Value.get""
+  IL_000f:  conv.i8
+  IL_0010:  call       ""C C.op_Explicit(long)""
+  IL_0015:  pop
+  IL_0016:  ret
+}
+");
+        }
+
+        [Fact, WorkItem(56646, "https://github.com/dotnet/roslyn/issues/56646")]
+        public void LiftedConversion_InvalidTypeArgument04()
+        {
+            var code = @"
+C c = null;
+M((long?)c);
+
+void M(long? i) => throw null;
+
+class C
+{
+    public static explicit operator int(C c) => throw null;
+}
+
+namespace System
+{
+    public class Object {}
+    public class String {}
+    public class Exception {}
+    public class ValueType : Object {}
+    public struct Void {}
+    public ref struct Int32 {}
+    public struct Nullable<T> where T : struct { public Nullable(T value) {} }
+    public struct Int64 {}
+    public struct Boolean { }
+    public class Attribute { }
+    public class AttributeUsageAttribute : Attribute
+    {
+        public AttributeUsageAttribute(AttributeTargets t) { }
+        public bool AllowMultiple { get; set; }
+        public bool Inherited { get; set; }
+    }
+    public struct Enum { }
+    public enum AttributeTargets { }
+}
+";
+
+            var comp = CreateEmptyCompilation(code);
+            var verifier = CompileAndVerify(comp, verify: Verification.Skipped);
+
+            // Note that no int? is being created.
+            verifier.VerifyIL("<top-level-statements-entry-point>", @"
+{
+  // Code size       18 (0x12)
+  .maxstack  1
+  IL_0000:  ldnull
+  IL_0001:  call       ""int C.op_Explicit(C)""
+  IL_0006:  conv.i8
+  IL_0007:  newobj     ""long?..ctor(long)""
+  IL_000c:  call       ""void Program.<<Main>$>g__M|0_0(long?)""
+  IL_0011:  ret
+}
+");
+        }
+
+        [Fact, WorkItem(56646, "https://github.com/dotnet/roslyn/issues/56646")]
+        public void LiftedConversion_InvalidTypeArgument05()
+        {
+            var code = @"
+unsafe
+{
+    S? s = null;
+    void* f = s;
+    System.Console.WriteLine((int)f);
+}
+
+public struct S
+{
+    public static unsafe implicit operator void*(S v) => throw null;
+}
+";
+
+            var comp = CreateCompilation(code, options: TestOptions.UnsafeReleaseExe);
+            var verifier = CompileAndVerify(comp, expectedOutput: "0", verify: Verification.Skipped);
+
+            verifier.VerifyIL("<top-level-statements-entry-point>", @"
+{
+  // Code size       42 (0x2a)
+  .maxstack  1
+  .locals init (S? V_0)
+  IL_0000:  ldloca.s   V_0
+  IL_0002:  initobj    ""S?""
+  IL_0008:  ldloc.0
+  IL_0009:  stloc.0
+  IL_000a:  ldloca.s   V_0
+  IL_000c:  call       ""bool S?.HasValue.get""
+  IL_0011:  brtrue.s   IL_0017
+  IL_0013:  ldc.i4.0
+  IL_0014:  conv.u
+  IL_0015:  br.s       IL_0023
+  IL_0017:  ldloca.s   V_0
+  IL_0019:  call       ""S S?.GetValueOrDefault()""
+  IL_001e:  call       ""void* S.op_Implicit(S)""
+  IL_0023:  conv.i4
+  IL_0024:  call       ""void System.Console.WriteLine(int)""
+  IL_0029:  ret
+}
+");
+        }
+
+        [Fact, WorkItem(56646, "https://github.com/dotnet/roslyn/issues/56646")]
+        public void LiftedConversion_InvalidTypeArgument06()
+        {
+            var code = @"
+unsafe
+{
+    S? s = null;
+    void* f = (void*)s;
+    System.Console.WriteLine((int)f);
+}
+
+public struct S
+{
+    public static unsafe explicit operator void*(S v) => throw null;
+}
+";
+
+            var comp = CreateCompilation(code, options: TestOptions.UnsafeReleaseExe);
+            var verifier = CompileAndVerify(comp, expectedOutput: "0", verify: Verification.Skipped);
+
+            verifier.VerifyIL("<top-level-statements-entry-point>", @"
+{
+  // Code size       42 (0x2a)
+  .maxstack  1
+  .locals init (S? V_0)
+  IL_0000:  ldloca.s   V_0
+  IL_0002:  initobj    ""S?""
+  IL_0008:  ldloc.0
+  IL_0009:  stloc.0
+  IL_000a:  ldloca.s   V_0
+  IL_000c:  call       ""bool S?.HasValue.get""
+  IL_0011:  brtrue.s   IL_0017
+  IL_0013:  ldc.i4.0
+  IL_0014:  conv.u
+  IL_0015:  br.s       IL_0023
+  IL_0017:  ldloca.s   V_0
+  IL_0019:  call       ""S S?.GetValueOrDefault()""
+  IL_001e:  call       ""void* S.op_Explicit(S)""
+  IL_0023:  conv.i4
+  IL_0024:  call       ""void System.Console.WriteLine(int)""
+  IL_0029:  ret
+}
+");
+        }
     }
 }
