@@ -595,7 +595,7 @@ namespace Microsoft.CodeAnalysis.CSharp.UnitTests
                 ForByte, ForSByte, ForShort, ForUShort,
                 ForInt, ForUInt, ForLong, ForULong,
                 ForFloat, ForDouble, ForDecimal, ForNint,
-                ForNuint, ForChar
+                ForNuint, ForChar, ForLength,
                 })
             {
                 for (int i = 0; i < 100; i++)
@@ -619,7 +619,7 @@ namespace Microsoft.CodeAnalysis.CSharp.UnitTests
                 ForByte, ForSByte, ForShort, ForUShort,
                 ForInt, ForUInt, ForLong, ForULong,
                 ForDecimal, ForNint,
-                ForNuint, ForChar })
+                ForNuint, ForChar, ForLength })
             {
                 for (int i = 0; i < 100; i++)
                 {
@@ -869,7 +869,8 @@ namespace Microsoft.CodeAnalysis.CSharp.UnitTests
                     ForString,
                     ForDecimal,
                     ForNint,
-                    ForNuint
+                    ForNuint,
+                    ForLength,
             };
             ConstantValue badConstant = ConstantValue.Bad;
             foreach (IValueSetFactory fac in ctors)
