@@ -27,94 +27,10 @@ namespace Microsoft.CodeAnalysis.CSharp.UnitTests
             return SyntaxFactory.ParseExpression(text, options: (options ?? TestOptions.Regular).WithLanguageVersion(LanguageVersion.CSharp8));
         }
 
-        [Fact, WorkItem(32318, "https://github.com/dotnet/roslyn/issues/32318")]
-        public void AwaitUsingDeclaration_WithCSharp73()
-        {
-            string source = @"
-class C
-{
-    async void M()
-    {
-        await using (var x = this)
-        {
-        }
-    }
-}
-";
-            var tree = SyntaxFactory.ParseSyntaxTree(source, options: TestOptions.Regular.WithLanguageVersion(LanguageVersion.CSharp7_3));
-            tree.GetDiagnostics().Verify(
-                // (6,9): error CS8652: The feature 'asynchronous using' is not available in C# 7.3. Please use language version 8.0 or greater.
-                //         await using (var x = this)
-                Diagnostic(ErrorCode.ERR_FeatureNotAvailableInVersion7_3, "await").WithArguments("asynchronous using", "8.0").WithLocation(6, 9)
-                );
-
-            UsingTree(source);
-            N(SyntaxKind.CompilationUnit);
-            {
-                N(SyntaxKind.ClassDeclaration);
-                {
-                    N(SyntaxKind.ClassKeyword);
-                    N(SyntaxKind.IdentifierToken, "C");
-                    N(SyntaxKind.OpenBraceToken);
-                    N(SyntaxKind.MethodDeclaration);
-                    {
-                        N(SyntaxKind.AsyncKeyword);
-                        N(SyntaxKind.PredefinedType);
-                        {
-                            N(SyntaxKind.VoidKeyword);
-                        }
-                        N(SyntaxKind.IdentifierToken, "M");
-                        N(SyntaxKind.ParameterList);
-                        {
-                            N(SyntaxKind.OpenParenToken);
-                            N(SyntaxKind.CloseParenToken);
-                        }
-                        N(SyntaxKind.Block);
-                        {
-                            N(SyntaxKind.OpenBraceToken);
-                            N(SyntaxKind.UsingStatement);
-                            {
-                                N(SyntaxKind.AwaitKeyword);
-                                N(SyntaxKind.UsingKeyword);
-                                N(SyntaxKind.OpenParenToken);
-                                N(SyntaxKind.VariableDeclaration);
-                                {
-                                    N(SyntaxKind.IdentifierName);
-                                    {
-                                        N(SyntaxKind.IdentifierToken, "var");
-                                    }
-                                    N(SyntaxKind.VariableDeclarator);
-                                    {
-                                        N(SyntaxKind.IdentifierToken, "x");
-                                        N(SyntaxKind.EqualsValueClause);
-                                        {
-                                            N(SyntaxKind.EqualsToken);
-                                            N(SyntaxKind.ThisExpression);
-                                            {
-                                                N(SyntaxKind.ThisKeyword);
-                                            }
-                                        }
-                                    }
-                                }
-                                N(SyntaxKind.CloseParenToken);
-                                N(SyntaxKind.Block);
-                                {
-                                    N(SyntaxKind.OpenBraceToken);
-                                    N(SyntaxKind.CloseBraceToken);
-                                }
-                            }
-                            N(SyntaxKind.CloseBraceToken);
-                        }
-                    }
-                    N(SyntaxKind.CloseBraceToken);
-                }
-                N(SyntaxKind.EndOfFileToken);
-            }
-            EOF();
-        }
-
-        [Fact]
-        public void AwaitUsingDeclaration()
+        [Theory, WorkItem(32318, "https://github.com/dotnet/roslyn/issues/32318")]
+        [InlineData(true)]
+        [InlineData(false)]
+        public void AwaitUsingDeclaration(bool useCSharp8)
         {
             UsingTree(@"
 class C
@@ -126,7 +42,7 @@ class C
         }
     }
 }
-");
+", options: useCSharp8 ? TestOptions.Regular8 : TestOptions.Regular7_3);
             N(SyntaxKind.CompilationUnit);
             {
                 N(SyntaxKind.ClassDeclaration);
@@ -267,7 +183,25 @@ class C
         }
     }
 }
-");
+",
+                // (6,15): error CS4003: 'await' cannot be used as an identifier within an async method or lambda expression
+                //         using await (this)
+                Diagnostic(ErrorCode.ERR_BadAwaitAsIdentifier, "await").WithLocation(6, 15),
+                // (6,21): error CS1001: Identifier expected
+                //         using await (this)
+                Diagnostic(ErrorCode.ERR_IdentifierExpected, "(").WithLocation(6, 21),
+                // (6,21): error CS1528: Expected ; or = (cannot specify constructor arguments in declaration)
+                //         using await (this)
+                Diagnostic(ErrorCode.ERR_BadVarDecl, "(this").WithLocation(6, 21),
+                // (6,21): error CS1003: Syntax error, '[' expected
+                //         using await (this)
+                Diagnostic(ErrorCode.ERR_SyntaxError, "(").WithArguments("[").WithLocation(6, 21),
+                // (6,26): error CS1003: Syntax error, ']' expected
+                //         using await (this)
+                Diagnostic(ErrorCode.ERR_SyntaxError, ")").WithArguments("]").WithLocation(6, 26),
+                // (6,27): error CS1002: ; expected
+                //         using await (this)
+                Diagnostic(ErrorCode.ERR_SemicolonExpected, "").WithLocation(6, 27));
             N(SyntaxKind.CompilationUnit);
             {
                 N(SyntaxKind.ClassDeclaration);
@@ -334,85 +268,10 @@ class C
             EOF();
         }
 
-        [Fact]
-        public void AwaitForeach_WithCSharp73()
-        {
-            string source = @"
-class C
-{
-    async void M()
-    {
-        await foreach (var i in collection)
-        {
-        }
-    }
-}
-";
-            var tree = SyntaxFactory.ParseSyntaxTree(source, options: TestOptions.Regular.WithLanguageVersion(LanguageVersion.CSharp7_3));
-            tree.GetDiagnostics().Verify(
-                // (6,9): error CS8652: The feature 'async streams' is not available in C# 7.3. Please use language version 8.0 or greater.
-                //         await foreach (var i in collection)
-                Diagnostic(ErrorCode.ERR_FeatureNotAvailableInVersion7_3, "await").WithArguments("async streams", "8.0").WithLocation(6, 9)
-                );
-
-            UsingTree(source);
-            N(SyntaxKind.CompilationUnit);
-            {
-                N(SyntaxKind.ClassDeclaration);
-                {
-                    N(SyntaxKind.ClassKeyword);
-                    N(SyntaxKind.IdentifierToken, "C");
-                    N(SyntaxKind.OpenBraceToken);
-                    N(SyntaxKind.MethodDeclaration);
-                    {
-                        N(SyntaxKind.AsyncKeyword);
-                        N(SyntaxKind.PredefinedType);
-                        {
-                            N(SyntaxKind.VoidKeyword);
-                        }
-                        N(SyntaxKind.IdentifierToken, "M");
-                        N(SyntaxKind.ParameterList);
-                        {
-                            N(SyntaxKind.OpenParenToken);
-                            N(SyntaxKind.CloseParenToken);
-                        }
-                        N(SyntaxKind.Block);
-                        {
-                            N(SyntaxKind.OpenBraceToken);
-                            N(SyntaxKind.ForEachStatement);
-                            {
-                                N(SyntaxKind.AwaitKeyword);
-                                N(SyntaxKind.ForEachKeyword);
-                                N(SyntaxKind.OpenParenToken);
-                                N(SyntaxKind.IdentifierName);
-                                {
-                                    N(SyntaxKind.IdentifierToken, "var");
-                                }
-                                N(SyntaxKind.IdentifierToken, "i");
-                                N(SyntaxKind.InKeyword);
-                                N(SyntaxKind.IdentifierName);
-                                {
-                                    N(SyntaxKind.IdentifierToken, "collection");
-                                }
-                                N(SyntaxKind.CloseParenToken);
-                                N(SyntaxKind.Block);
-                                {
-                                    N(SyntaxKind.OpenBraceToken);
-                                    N(SyntaxKind.CloseBraceToken);
-                                }
-                            }
-                            N(SyntaxKind.CloseBraceToken);
-                        }
-                    }
-                    N(SyntaxKind.CloseBraceToken);
-                }
-                N(SyntaxKind.EndOfFileToken);
-            }
-            EOF();
-        }
-
-        [Fact]
-        public void AwaitForeach()
+        [Theory]
+        [InlineData(true)]
+        [InlineData(false)]
+        public void AwaitForeach(bool useCSharp8)
         {
             UsingTree(@"
 class C
@@ -424,7 +283,7 @@ class C
         }
     }
 }
-");
+", options: useCSharp8 ? TestOptions.Regular8 : TestOptions.Regular7_3);
             N(SyntaxKind.CompilationUnit);
             {
                 N(SyntaxKind.ClassDeclaration);
@@ -493,7 +352,37 @@ class C
         }
     }
 }
-");
+",
+                // (6,17): error CS1003: Syntax error, '(' expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_SyntaxError, "await").WithArguments("(").WithLocation(6, 17),
+                // (6,28): error CS1026: ) expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_CloseParenExpected, "i").WithLocation(6, 28),
+                // (6,28): error CS1515: 'in' expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_InExpected, "i").WithLocation(6, 28),
+                // (6,28): error CS0230: Type and identifier are both required in a foreach statement
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_BadForeachDecl, "i").WithLocation(6, 28),
+                // (6,30): error CS1026: ) expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_CloseParenExpected, "in").WithLocation(6, 30),
+                // (6,30): error CS1525: Invalid expression term 'in'
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_InvalidExprTerm, "in").WithArguments("in").WithLocation(6, 30),
+                // (6,30): error CS1002: ; expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_SemicolonExpected, "in").WithLocation(6, 30),
+                // (6,30): error CS1513: } expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_RbraceExpected, "in").WithLocation(6, 30),
+                // (6,43): error CS1002: ; expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_SemicolonExpected, ")").WithLocation(6, 43),
+                // (6,43): error CS1513: } expected
+                //         foreach await (var i in collection)
+                Diagnostic(ErrorCode.ERR_RbraceExpected, ")").WithLocation(6, 43));
             N(SyntaxKind.CompilationUnit);
             {
                 N(SyntaxKind.ClassDeclaration);
@@ -572,101 +461,10 @@ class C
             EOF();
         }
 
-        [Fact]
-        public void DeconstructionAwaitForeach_WithCSharp73()
-        {
-            string source = @"
-class C
-{
-    async void M()
-    {
-        await foreach (var (i, j) in collection)
-        {
-        }
-    }
-}
-";
-            var tree = SyntaxFactory.ParseSyntaxTree(source, options: TestOptions.Regular.WithLanguageVersion(LanguageVersion.CSharp7_3));
-            tree.GetDiagnostics().Verify(
-                // (6,9): error CS8652: The feature 'async streams' is not available in C# 7.3. Please use language version 8.0 or greater.
-                //         await foreach (var (i, j) in collection)
-                Diagnostic(ErrorCode.ERR_FeatureNotAvailableInVersion7_3, "await").WithArguments("async streams", "8.0").WithLocation(6, 9)
-                );
-
-            UsingTree(source);
-            N(SyntaxKind.CompilationUnit);
-            {
-                N(SyntaxKind.ClassDeclaration);
-                {
-                    N(SyntaxKind.ClassKeyword);
-                    N(SyntaxKind.IdentifierToken, "C");
-                    N(SyntaxKind.OpenBraceToken);
-                    N(SyntaxKind.MethodDeclaration);
-                    {
-                        N(SyntaxKind.AsyncKeyword);
-                        N(SyntaxKind.PredefinedType);
-                        {
-                            N(SyntaxKind.VoidKeyword);
-                        }
-                        N(SyntaxKind.IdentifierToken, "M");
-                        N(SyntaxKind.ParameterList);
-                        {
-                            N(SyntaxKind.OpenParenToken);
-                            N(SyntaxKind.CloseParenToken);
-                        }
-                        N(SyntaxKind.Block);
-                        {
-                            N(SyntaxKind.OpenBraceToken);
-                            N(SyntaxKind.ForEachVariableStatement);
-                            {
-                                N(SyntaxKind.AwaitKeyword);
-                                N(SyntaxKind.ForEachKeyword);
-                                N(SyntaxKind.OpenParenToken);
-                                N(SyntaxKind.DeclarationExpression);
-                                {
-                                    N(SyntaxKind.IdentifierName);
-                                    {
-                                        N(SyntaxKind.IdentifierToken, "var");
-                                    }
-                                    N(SyntaxKind.ParenthesizedVariableDesignation);
-                                    {
-                                        N(SyntaxKind.OpenParenToken);
-                                        N(SyntaxKind.SingleVariableDesignation);
-                                        {
-                                            N(SyntaxKind.IdentifierToken, "i");
-                                        }
-                                        N(SyntaxKind.CommaToken);
-                                        N(SyntaxKind.SingleVariableDesignation);
-                                        {
-                                            N(SyntaxKind.IdentifierToken, "j");
-                                        }
-                                        N(SyntaxKind.CloseParenToken);
-                                    }
-                                }
-                                N(SyntaxKind.InKeyword);
-                                N(SyntaxKind.IdentifierName);
-                                {
-                                    N(SyntaxKind.IdentifierToken, "collection");
-                                }
-                                N(SyntaxKind.CloseParenToken);
-                                N(SyntaxKind.Block);
-                                {
-                                    N(SyntaxKind.OpenBraceToken);
-                                    N(SyntaxKind.CloseBraceToken);
-                                }
-                            }
-                            N(SyntaxKind.CloseBraceToken);
-                        }
-                    }
-                    N(SyntaxKind.CloseBraceToken);
-                }
-                N(SyntaxKind.EndOfFileToken);
-            }
-            EOF();
-        }
-
-        [Fact]
-        public void DeconstructionAwaitForeach()
+        [Theory]
+        [InlineData(true)]
+        [InlineData(false)]
+        public void DeconstructionAwaitForeach(bool useCSharp8)
         {
             UsingTree(@"
 class C
@@ -678,7 +476,7 @@ class C
         }
     }
 }
-");
+", options: useCSharp8 ? TestOptions.Regular8 : TestOptions.Regular7_3);
             N(SyntaxKind.CompilationUnit);
             {
                 N(SyntaxKind.ClassDeclaration);
