@@ -1813,7 +1813,6 @@ option3 = config2
 option2 = config3
 ", "/path/to/.editorconfig"));
 
-
             var options = GetAnalyzerConfigOptions(
                  new[] { "/path/to/file1.cs", "/path/file1.cs", "/file1.cs" },
                  configs);
@@ -1968,7 +1967,6 @@ dotnet_diagnostic.cs000.severity = error
                 configs);
             configs.Free();
 
-
             Assert.Equal(new[] {
                 SyntaxTree.EmptyDiagnosticOptions,
                 CreateImmutableDictionary(("cs000", ReportDiagnostic.Error))
@@ -2046,7 +2044,6 @@ dotnet_diagnostic.cs000.severity = warning
                 configs);
             configs.Free();
 
-
             Assert.Equal(new[] {
                 CreateImmutableDictionary(("cs000", ReportDiagnostic.Suppress)),
                 CreateImmutableDictionary(("cs000", ReportDiagnostic.Warn))
@@ -2115,7 +2112,6 @@ root = true
 option2 = config3
 ", "/path/to/.editorconfig"));
 
-
             var options = GetAnalyzerConfigOptions(
                  new[] { "/path/to/file1.cs", "/path/file1.cs", "/file1.cs" },
                  configs);
@@ -2536,6 +2532,28 @@ a = b
             }
         }
 
+        [Fact]
+        public void CorrectlyMergeGlobalConfigWithEscapedPaths()
+        {
+            var configs = ArrayBuilder<AnalyzerConfig>.GetInstance();
+            configs.Add(Parse(@"
+is_global = true
+[/Test.cs]
+a = a
+[/\Test.cs]
+b = b
+", "/.editorconfig"));
+
+            var configSet = AnalyzerConfigSet.Create(configs, out var diagnostics);
+            configs.Free();
+
+            var options = configSet.GetOptionsForSourcePath("/Test.cs");
+
+            Assert.Equal(2, options.AnalyzerOptions.Count);
+            Assert.Equal("a", options.AnalyzerOptions["a"]);
+            Assert.Equal("b", options.AnalyzerOptions["b"]);
+        }
+
         #endregion
     }
 }
