@@ -74,7 +74,7 @@ namespace Microsoft.CodeAnalysis.CommandLine
         public static BuildRequest Create(RequestLanguage language,
                                           IList<string> args,
                                           string workingDirectory,
-                                          string tempDirectory,
+                                          string? tempDirectory,
                                           string compilerHash,
                                           Guid? requestId = null,
                                           string? keepAlive = null,
@@ -290,6 +290,9 @@ namespace Microsoft.CodeAnalysis.CommandLine
 
             // The server hash did not match the one supplied by the client
             IncorrectHash,
+
+            // Cannot connect to the server
+            CannotConnect,
         }
 
         public abstract ResponseType Type { get; }
@@ -364,6 +367,9 @@ namespace Microsoft.CodeAnalysis.CommandLine
                         return ShutdownBuildResponse.Create(reader);
                     case ResponseType.Rejected:
                         return RejectedBuildResponse.Create(reader);
+
+                    // Intentional fall through
+                    case ResponseType.CannotConnect:
                     default:
                         throw new InvalidOperationException("Received invalid response type from server.");
                 }
@@ -493,6 +499,9 @@ namespace Microsoft.CodeAnalysis.CommandLine
         }
     }
 
+    /// <summary>
+    /// The <see cref="BuildRequest"/> was rejected by the server.
+    /// </summary>
     internal sealed class RejectedBuildResponse : BuildResponse
     {
         public string Reason;
@@ -517,6 +526,16 @@ namespace Microsoft.CodeAnalysis.CommandLine
         }
     }
 
+    /// <summary>
+    /// Used when the client cannot connect to the server.
+    /// </summary>
+    internal sealed class CannotConnectResponse : BuildResponse
+    {
+        public override ResponseType Type => ResponseType.CannotConnect;
+
+        protected override void AddResponseBody(BinaryWriter writer) { }
+    }
+
     // The id numbers below are just random. It's useful to use id numbers
     // that won't occur accidentally for debugging.
     internal enum RequestLanguage
