@@ -275,8 +275,14 @@ namespace Microsoft.CodeAnalysis
             {
                 try
                 {
-                    // Check some ancient reserved device names, such as COM1,..9, LPT1..9, PRN, CON, or AUX etc., and bail out earlier
-                    // Win32 API - GetFullFileName - will resolve them, say 'COM1', as "\\.\COM1" 
+                    // Windows 10 and earlier placed restrictions on file names that originally appeared as device 
+                    // names. For example COM1, PRN, CON, AUX, etc ... Files could not be created with those names even 
+                    // with extensions like .txt. When those restricted names are passed to GetFullPath the 
+                    // runtime will escape them with \\.\. For example GetFullPath("aux.txt") will return "\\.\aux.txt".
+                    // The compiler detects these illegal names and bails out early
+                    //
+                    // Windows 11 removed this restriction though and hence the names are now legal. Cannot find documentation
+                    // to support this but experimentally it can be validated. 
                     resolvedPath = Path.GetFullPath(resolvedPath);
                     // preserve possible invalid path info for diagnostic purpose
                     invalidPath = resolvedPath;
@@ -308,7 +314,7 @@ namespace Microsoft.CodeAnalysis
         /// <summary>
         /// Trims all '.' and whitespace from the end of the path
         /// </summary>
-        [return: NotNullIfNotNull("path")]
+        [return: NotNullIfNotNull(nameof(path))]
         internal static string? RemoveTrailingSpacesAndDots(string? path)
         {
             if (path == null)
@@ -348,7 +354,7 @@ namespace Microsoft.CodeAnalysis
                 var kv = SplitWithDoubledSeparatorEscaping(kEqualsV, '=');
                 if (kv.Length != 2)
                 {
-                    errors.Add(Diagnostic.Create(_messageProvider, _messageProvider.ERR_InvalidPathMap, kEqualsV));
+                    errors.Add(Diagnostic.Create(_messageProvider, _messageProvider.ERR_InvalidPathMap));
                     continue;
                 }
 
@@ -357,7 +363,7 @@ namespace Microsoft.CodeAnalysis
 
                 if (from.Length == 0 || to.Length == 0)
                 {
-                    errors.Add(Diagnostic.Create(_messageProvider, _messageProvider.ERR_InvalidPathMap, kEqualsV));
+                    errors.Add(Diagnostic.Create(_messageProvider, _messageProvider.ERR_InvalidPathMap));
                 }
                 else
                 {
@@ -700,11 +706,11 @@ namespace Microsoft.CodeAnalysis
         /// </param>
         internal static bool TryParseClientArgs(
             IEnumerable<string> args,
-            out List<string>? parsedArgs,
+            [NotNullWhen(true)] out List<string>? parsedArgs,
             out bool containsShared,
             out string? keepAliveValue,
             out string? pipeName,
-            out string? errorMessage)
+            [NotNullWhen(false)] out string? errorMessage)
         {
             containsShared = false;
             keepAliveValue = null;
@@ -838,7 +844,6 @@ namespace Microsoft.CodeAnalysis
                 length -= offset;
             }
 
-
             if (length >= 1)
             {
                 filePath = RemoveQuotesAndSlashes(parts[offset + 0]);
@@ -888,7 +893,7 @@ namespace Microsoft.CodeAnalysis
         /// function is called RemoveQuotesAndSlashes.  It has virtually the same behavior except for a few 
         /// quirks in error cases.  
         /// </remarks>
-        [return: NotNullIfNotNull(parameterName: "arg")]
+        [return: NotNullIfNotNull(parameterName: nameof(arg))]
         internal static string? RemoveQuotesAndSlashes(string? arg) =>
             arg is not null
                 ? RemoveQuotesAndSlashes(arg.AsMemory())
