@@ -15,6 +15,7 @@ using System.Reflection.PortableExecutable;
 using System.Runtime.CompilerServices;
 using System.Runtime.Versioning;
 using System.Threading;
+using Microsoft.CodeAnalysis.PooledObjects;
 using Roslyn.Utilities;
 
 namespace Microsoft.CodeAnalysis.Diagnostics
@@ -198,9 +199,6 @@ namespace Microsoft.CodeAnalysis.Diagnostics
 
         private static AnalyzerLoadFailureEventArgs CreateAnalyzerFailedArgs(Exception e, string? typeName = null)
         {
-            // unwrap:
-            e = (e as TargetInvocationException) ?? e;
-
             // remove all line breaks from the exception message
             string message = e.Message.Replace("\r", "").Replace("\n", "");
 
@@ -211,7 +209,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
             return new AnalyzerLoadFailureEventArgs(errorCode, message, e, typeName);
         }
 
-        internal ImmutableSortedDictionary<string, ImmutableSortedSet<string>> GetAnalyzerTypeNameMap()
+        internal ImmutableSortedDictionary<string, ImmutableHashSet<string>> GetAnalyzerTypeNameMap()
         {
             return _diagnosticAnalyzers.GetExtensionTypeNameMap();
         }
@@ -222,7 +220,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
         /// <exception cref="BadImageFormatException">The PE image format is invalid.</exception>
         /// <exception cref="IOException">IO error reading the metadata.</exception>
         [PerformanceSensitive("https://github.com/dotnet/roslyn/issues/30449")]
-        private static ImmutableSortedDictionary<string, ImmutableSortedSet<string>> GetAnalyzerTypeNameMap(string fullPath, Type attributeType, AttributeLanguagesFunc languagesFunc)
+        private static ImmutableSortedDictionary<string, ImmutableHashSet<string>> GetAnalyzerTypeNameMap(string fullPath, Type attributeType, AttributeLanguagesFunc languagesFunc)
         {
             using var assembly = AssemblyMetadata.CreateFromFile(fullPath);
 
@@ -238,7 +236,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                               from supportedLanguage in supportedLanguages
                               group typeName by supportedLanguage;
 
-            return typeNameMap.ToImmutableSortedDictionary(g => g.Key, g => g.ToImmutableSortedSet(StringComparer.OrdinalIgnoreCase), StringComparer.OrdinalIgnoreCase);
+            return typeNameMap.ToImmutableSortedDictionary(g => g.Key, g => g.ToImmutableHashSet(), StringComparer.OrdinalIgnoreCase);
         }
 
         private static IEnumerable<string> GetSupportedLanguages(TypeDefinition typeDef, PEModule peModule, Type attributeType, AttributeLanguagesFunc languagesFunc)
@@ -360,7 +358,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
             private readonly Func<object?, TExtension?>? _coerceFunction;
             private ImmutableArray<TExtension> _lazyAllExtensions;
             private ImmutableDictionary<string, ImmutableArray<TExtension>> _lazyExtensionsPerLanguage;
-            private ImmutableSortedDictionary<string, ImmutableSortedSet<string>>? _lazyExtensionTypeNameMap;
+            private ImmutableSortedDictionary<string, ImmutableHashSet<string>>? _lazyExtensionTypeNameMap;
 
             internal Extensions(AnalyzerFileReference reference, Type attributeType, AttributeLanguagesFunc languagesFunc, bool allowNetFramework, Func<object?, TExtension?>? coerceFunction = null)
             {
@@ -435,7 +433,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                 return builder.ToImmutable();
             }
 
-            internal ImmutableSortedDictionary<string, ImmutableSortedSet<string>> GetExtensionTypeNameMap()
+            internal ImmutableSortedDictionary<string, ImmutableHashSet<string>> GetExtensionTypeNameMap()
             {
                 if (_lazyExtensionTypeNameMap == null)
                 {
@@ -448,7 +446,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
 
             internal void AddExtensions(ImmutableSortedDictionary<string, ImmutableArray<TExtension>>.Builder builder)
             {
-                ImmutableSortedDictionary<string, ImmutableSortedSet<string>> analyzerTypeNameMap;
+                ImmutableSortedDictionary<string, ImmutableHashSet<string>> analyzerTypeNameMap;
                 Assembly analyzerAssembly;
 
                 try
@@ -460,6 +458,10 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                     }
 
                     analyzerAssembly = _reference.GetAssembly();
+                    if (CheckAssemblyReferencesNewerCompiler(analyzerAssembly))
+                    {
+                        return;
+                    }
                 }
                 catch (Exception e)
                 {
@@ -492,7 +494,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
 
             internal void AddExtensions(ImmutableArray<TExtension>.Builder builder, string language, Func<TExtension, bool>? shouldInclude = null)
             {
-                ImmutableSortedDictionary<string, ImmutableSortedSet<string>> analyzerTypeNameMap;
+                ImmutableSortedDictionary<string, ImmutableHashSet<string>> analyzerTypeNameMap;
                 Assembly analyzerAssembly;
 
                 try
@@ -506,7 +508,7 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                     }
 
                     analyzerAssembly = _reference.GetAssembly();
-                    if (analyzerAssembly == null)
+                    if (analyzerAssembly == null || CheckAssemblyReferencesNewerCompiler(analyzerAssembly))
                     {
                         // This can be null if NoOpAnalyzerAssemblyLoader is used.
                         return;
@@ -539,9 +541,26 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                 }
             }
 
-            private ImmutableArray<TExtension> GetLanguageSpecificAnalyzers(Assembly analyzerAssembly, ImmutableSortedDictionary<string, ImmutableSortedSet<string>> analyzerTypeNameMap, string language, ref bool reportedError)
+            bool CheckAssemblyReferencesNewerCompiler(Assembly analyzerAssembly)
             {
-                ImmutableSortedSet<string>? languageSpecificAnalyzerTypeNames;
+                var runningCompilerAssemblyName = typeof(AnalyzerFileReference).Assembly.GetName();
+                foreach (var referencedAssemblyName in analyzerAssembly.GetReferencedAssemblies())
+                {
+                    if (string.Equals(referencedAssemblyName.Name, runningCompilerAssemblyName.Name, StringComparison.OrdinalIgnoreCase)
+                        && referencedAssemblyName.Version > runningCompilerAssemblyName.Version)
+                    {
+                        // note: we introduce an actual message for this scenario when handling the failed event.
+                        _reference.AnalyzerLoadFailed?.Invoke(_reference, new AnalyzerLoadFailureEventArgs(AnalyzerLoadFailureEventArgs.FailureErrorCode.ReferencesNewerCompiler, message: "") { ReferencedCompilerVersion = referencedAssemblyName.Version });
+                        return true;
+                    }
+                }
+
+                return false;
+            }
+
+            private ImmutableArray<TExtension> GetLanguageSpecificAnalyzers(Assembly analyzerAssembly, ImmutableSortedDictionary<string, ImmutableHashSet<string>> analyzerTypeNameMap, string language, ref bool reportedError)
+            {
+                ImmutableHashSet<string>? languageSpecificAnalyzerTypeNames;
                 if (!analyzerTypeNameMap.TryGetValue(language, out languageSpecificAnalyzerTypeNames))
                 {
                     return ImmutableArray<TExtension>.Empty;
@@ -549,12 +568,14 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                 return this.GetAnalyzersForTypeNames(analyzerAssembly, languageSpecificAnalyzerTypeNames, ref reportedError);
             }
 
-            private ImmutableArray<TExtension> GetAnalyzersForTypeNames(Assembly analyzerAssembly, IEnumerable<string> analyzerTypeNames, ref bool reportedError)
+            private ImmutableArray<TExtension> GetAnalyzersForTypeNames(Assembly analyzerAssembly, ImmutableHashSet<string> analyzerTypeNames, ref bool reportedError)
             {
-                var analyzers = ImmutableArray.CreateBuilder<TExtension>();
+                var builder = ArrayBuilder<(string typeName, TExtension analyzer)>.GetInstance();
 
                 // Given the type names, get the actual System.Type and try to create an instance of the type through reflection.
-                foreach (var typeName in analyzerTypeNames)
+                // Randomize the order we instantiate analyzers to avoid static constructor/JIT contention, but still return
+                // the list of analyzers in the order of the sorted type names for deterministic purpose.
+                foreach (var typeName in shuffle(analyzerTypeNames))
                 {
                     Type? type;
                     try
@@ -599,11 +620,36 @@ namespace Microsoft.CodeAnalysis.Diagnostics
                     TExtension? analyzer = typeInstance as TExtension ?? _coerceFunction?.Invoke(typeInstance);
                     if (analyzer != null)
                     {
-                        analyzers.Add(analyzer);
+                        builder.Add((typeName, analyzer));
                     }
                 }
 
-                return analyzers.ToImmutable();
+                builder.Sort(static (x, y) => string.Compare(x.typeName, y.typeName, StringComparison.OrdinalIgnoreCase));
+                var analyzers = builder.SelectAsArray(x => x.analyzer);
+                builder.Free();
+
+                return analyzers;
+
+                static IEnumerable<string> shuffle(ImmutableHashSet<string> source)
+                {
+                    var random =
+#if NET6_0_OR_GREATER
+                        Random.Shared;
+#else
+                        new Random();
+#endif
+                    var builder = ArrayBuilder<string>.GetInstance(source.Count);
+                    builder.AddRange(source);
+
+                    for (var i = builder.Count - 1; i >= 0; i--)
+                    {
+                        var swapIndex = random.Next(i + 1);
+                        yield return builder[swapIndex];
+                        builder[swapIndex] = builder[i];
+                    }
+
+                    builder.Free();
+                }
             }
         }
 
