@@ -25,7 +25,7 @@ namespace Microsoft.CodeAnalysis.Operations
 
         internal ImmutableArray<IOperation> Children { get; }
 
-        protected override IOperation GetCurrent(int slot, int index)
+        internal override IOperation GetCurrent(int slot, int index)
             => slot switch
             {
                 0 when index < Children.Length
@@ -33,7 +33,9 @@ namespace Microsoft.CodeAnalysis.Operations
                 _ => throw ExceptionUtilities.UnexpectedValue((slot, index))
             };
 
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
+        internal override int ChildOperationsCount => Children.Length;
+
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
         {
             switch (previousSlot)
             {
@@ -50,6 +52,15 @@ namespace Microsoft.CodeAnalysis.Operations
             }
         }
 
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex)
+            => previousSlot switch
+            {
+                int.MaxValue when !Children.IsEmpty => (true, 0, Children.Length - 1),
+                0 when previousIndex > 0 => (true, 0, previousIndex - 1),
+                int.MaxValue or 0 or -1 => (false, -1, 0),
+                _ => throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex))
+            };
+
         public override ITypeSymbol? Type { get; }
         internal override ConstantValue? OperationConstantValue { get; }
         public override OperationKind Kind => OperationKind.None;
@@ -68,6 +79,7 @@ namespace Microsoft.CodeAnalysis.Operations
     internal partial class ConversionOperation
     {
         public IMethodSymbol? OperatorMethod => Conversion.MethodSymbol;
+        public ITypeSymbol? ConstrainedToType => Conversion.ConstrainedToType;
     }
 
     internal sealed partial class InvalidOperation : Operation, IInvalidOperation
@@ -84,7 +96,7 @@ namespace Microsoft.CodeAnalysis.Operations
 
         internal ImmutableArray<IOperation> Children { get; }
 
-        protected override IOperation GetCurrent(int slot, int index)
+        internal override IOperation GetCurrent(int slot, int index)
             => slot switch
             {
                 0 when index < Children.Length
@@ -92,7 +104,9 @@ namespace Microsoft.CodeAnalysis.Operations
                 _ => throw ExceptionUtilities.UnexpectedValue((slot, index))
             };
 
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
+        internal override int ChildOperationsCount => Children.Length;
+
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
         {
             switch (previousSlot)
             {
@@ -109,6 +123,15 @@ namespace Microsoft.CodeAnalysis.Operations
             }
         }
 
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex)
+            => previousSlot switch
+            {
+                int.MaxValue when !Children.IsEmpty => (true, 0, Children.Length - 1),
+                0 when previousIndex > 0 => (true, 0, previousIndex - 1),
+                int.MaxValue or 0 or -1 => (false, -1, 0),
+                _ => throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex))
+            };
+
         public override ITypeSymbol? Type { get; }
         internal override ConstantValue? OperationConstantValue { get; }
         public override OperationKind Kind => OperationKind.Invalid;
@@ -137,8 +160,10 @@ namespace Microsoft.CodeAnalysis.Operations
         }
         public IMethodSymbol Symbol => Original.Symbol;
 
-        protected override IOperation GetCurrent(int slot, int index) => throw ExceptionUtilities.UnexpectedValue((slot, index));
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex) => (false, int.MinValue, int.MinValue);
+        internal override IOperation GetCurrent(int slot, int index) => throw ExceptionUtilities.UnexpectedValue((slot, index));
+        internal override int ChildOperationsCount => 0;
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex) => (false, int.MinValue, int.MinValue);
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex) => (false, int.MinValue, int.MinValue);
 
         public override OperationKind Kind => OperationKind.FlowAnonymousFunction;
         public override ITypeSymbol? Type => null;
@@ -177,6 +202,7 @@ namespace Microsoft.CodeAnalysis.Operations
     internal sealed partial class FieldReferenceOperation
     {
         public override ISymbol Member => Field;
+        public override ITypeSymbol? ConstrainedToType => null;
     }
 
     internal sealed partial class RangeCaseClauseOperation
@@ -233,7 +259,9 @@ namespace Microsoft.CodeAnalysis.Operations
         internal override ConstantValue? OperationConstantValue => null;
         public override OperationKind Kind => OperationKind.DynamicObjectCreation;
 
-        protected override IOperation GetCurrent(int slot, int index)
+        internal override int ChildOperationsCount => (Initializer is null ? 0 : 1) + Arguments.Length;
+
+        internal override IOperation GetCurrent(int slot, int index)
             => slot switch
             {
                 0 when index < Arguments.Length
@@ -243,7 +271,7 @@ namespace Microsoft.CodeAnalysis.Operations
                 _ => throw ExceptionUtilities.UnexpectedValue((slot, index)),
             };
 
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
         {
             switch (previousSlot)
             {
@@ -267,6 +295,30 @@ namespace Microsoft.CodeAnalysis.Operations
             }
         }
 
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex)
+        {
+            switch (previousSlot)
+            {
+                case int.MaxValue:
+                    if (Initializer == null) goto case 1;
+                    else return (true, 1, 0);
+
+                case 1:
+                    if (!Arguments.IsEmpty) return (true, 0, Arguments.Length - 1);
+                    else goto case 0;
+
+                case 0 when previousIndex > 0:
+                    return (true, 0, previousIndex - 1);
+
+                case 0:
+                case -1:
+                    return (false, -1, 0);
+
+                default:
+                    throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex));
+            }
+        }
+
         public override void Accept(OperationVisitor visitor)
         {
             visitor.VisitDynamicObjectCreation(this);
@@ -285,7 +337,9 @@ namespace Microsoft.CodeAnalysis.Operations
             Operation = SetParentOperation(operation, this);
         }
 
-        protected override IOperation GetCurrent(int slot, int index)
+        internal override int ChildOperationsCount => (Operation is null ? 0 : 1) + Arguments.Length;
+
+        internal override IOperation GetCurrent(int slot, int index)
             => slot switch
             {
                 0 when Operation != null
@@ -295,7 +349,7 @@ namespace Microsoft.CodeAnalysis.Operations
                 _ => throw ExceptionUtilities.UnexpectedValue((slot, index)),
             };
 
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
         {
             switch (previousSlot)
             {
@@ -319,6 +373,30 @@ namespace Microsoft.CodeAnalysis.Operations
             }
         }
 
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex)
+        {
+            switch (previousSlot)
+            {
+                case int.MaxValue:
+                    if (!Arguments.IsEmpty) return (true, 1, Arguments.Length - 1);
+                    else goto case 1;
+
+                case 1 when previousIndex > 0:
+                    return (true, 1, previousIndex - 1);
+
+                case 1:
+                    if (Operation != null) return (true, 0, 0);
+                    else goto case 0;
+
+                case 0:
+                case -1:
+                    return (false, -1, 0);
+
+                default:
+                    throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex));
+            }
+        }
+
         public IOperation Operation { get; }
         internal override ConstantValue? OperationConstantValue => null;
         public override OperationKind Kind => OperationKind.DynamicInvocation;
@@ -345,7 +423,9 @@ namespace Microsoft.CodeAnalysis.Operations
         internal override ConstantValue? OperationConstantValue => null;
         public override OperationKind Kind => OperationKind.DynamicIndexerAccess;
 
-        protected override IOperation GetCurrent(int slot, int index)
+        internal override int ChildOperationsCount => (Operation is null ? 0 : 1) + Arguments.Length;
+
+        internal override IOperation GetCurrent(int slot, int index)
             => slot switch
             {
                 0 when Operation != null
@@ -355,7 +435,7 @@ namespace Microsoft.CodeAnalysis.Operations
                 _ => throw ExceptionUtilities.UnexpectedValue((slot, index)),
             };
 
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
         {
             switch (previousSlot)
             {
@@ -379,6 +459,30 @@ namespace Microsoft.CodeAnalysis.Operations
             }
         }
 
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex)
+        {
+            switch (previousSlot)
+            {
+                case int.MaxValue:
+                    if (!Arguments.IsEmpty) return (true, 1, Arguments.Length - 1);
+                    else goto case 1;
+
+                case 1 when previousIndex > 0:
+                    return (true, 1, previousIndex - 1);
+
+                case 1:
+                    if (Operation != null) return (true, 0, 0);
+                    else goto case 0;
+
+                case 0:
+                case -1:
+                    return (false, -1, 0);
+
+                default:
+                    throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex));
+            }
+        }
+
         public override void Accept(OperationVisitor visitor)
         {
             visitor.VisitDynamicIndexerAccess(this);
@@ -406,7 +510,7 @@ namespace Microsoft.CodeAnalysis.Operations
 
     internal sealed partial class WhileLoopOperation
     {
-        protected override IOperation GetCurrent(int slot, int index)
+        internal override IOperation GetCurrent(int slot, int index)
         {
             return ConditionIsTop ? getCurrentSwitchTop() : getCurrentSwitchBottom();
 
@@ -435,7 +539,7 @@ namespace Microsoft.CodeAnalysis.Operations
                 };
         }
 
-        protected override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNext(int previousSlot, int previousIndex)
         {
             return ConditionIsTop ? moveNextConditionIsTop() : moveNextConditionIsBottom();
 
@@ -482,13 +586,60 @@ namespace Microsoft.CodeAnalysis.Operations
             }
         }
 
+        internal override (bool hasNext, int nextSlot, int nextIndex) MoveNextReversed(int previousSlot, int previousIndex)
+        {
+            return ConditionIsTop ? moveNextConditionIsTop() : moveNextConditionIsBottom();
+
+            (bool hasNext, int nextSlot, int nextIndex) moveNextConditionIsTop()
+            {
+                switch (previousSlot)
+                {
+                    case int.MaxValue:
+                        if (IgnoredCondition != null) return (true, 2, 0);
+                        else goto case 2;
+                    case 2:
+                        if (Body != null) return (true, 1, 0);
+                        else goto case 1;
+                    case 1:
+                        if (Condition != null) return (true, 0, 0);
+                        else goto case 0;
+                    case 0:
+                    case -1:
+                        return (false, -1, 0);
+                    default:
+                        throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex));
+                }
+            }
+
+            (bool hasNext, int nextSlot, int nextIndex) moveNextConditionIsBottom()
+            {
+                switch (previousSlot)
+                {
+                    case int.MaxValue:
+                        if (IgnoredCondition != null) return (true, 2, 0);
+                        else goto case 2;
+                    case 2:
+                        if (Condition != null) return (true, 1, 0);
+                        else goto case 1;
+                    case 1:
+                        if (Body != null) return (true, 0, 0);
+                        else goto case 0;
+                    case 0:
+                    case -1:
+                        return (false, -1, 0);
+                    default:
+                        throw ExceptionUtilities.UnexpectedValue((previousSlot, previousIndex));
+                }
+            }
+        }
+
         public override LoopKind LoopKind => LoopKind.While;
     }
 
     internal sealed partial class FlowCaptureReferenceOperation
     {
-        public FlowCaptureReferenceOperation(int id, SyntaxNode syntax, ITypeSymbol? type, ConstantValue? constantValue) :
-            this(new CaptureId(id), semanticModel: null, syntax: syntax, type: type, constantValue: constantValue, isImplicit: true)
+        public FlowCaptureReferenceOperation(int id, SyntaxNode syntax, ITypeSymbol? type, ConstantValue? constantValue, bool isInitialization = false) :
+            this(new CaptureId(id), isInitialization, semanticModel: null, syntax: syntax, type: type, constantValue: constantValue, isImplicit: true)
         {
         }
     }
