@@ -17,6 +17,12 @@ namespace Microsoft.Cci
     {
         private readonly HashSet<IReferenceOrISignature> _alreadySeen = new();
         private readonly HashSet<IReferenceOrISignature> _alreadyHasToken = new();
+
+        /// <summary>
+        /// Set true before a type reference is visited but only if a token needs to be created for the type reference.
+        /// E.g. not set before return type of a method is visited since the type reference is encoded in the signature blob of the method.
+        /// On the other hand, it is true before the type of an event definition is visited since Event table stores the type of the event as a coded token (TypeDef/Ref/Spec).
+        /// </summary>
         protected bool typeReferenceNeedsToken;
 
         internal ReferenceIndexerBase(EmitContext context)
@@ -38,6 +44,7 @@ namespace Microsoft.Cci
         {
             this.typeReferenceNeedsToken = true;
             this.Visit(customModifier.GetModifier(Context));
+            Debug.Assert(!this.typeReferenceNeedsToken);
         }
 
         public override void Visit(IEventDefinition eventDefinition)
@@ -60,6 +67,7 @@ namespace Microsoft.Cci
                 return;
             }
 
+            this.Visit(fieldReference.RefCustomModifiers);
             this.Visit((ITypeMemberReference)fieldReference);
             this.Visit(fieldReference.GetType(Context));
             ReserveFieldToken(fieldReference);
@@ -299,7 +307,6 @@ namespace Microsoft.Cci
             }
         }
 
-
         private void VisitTypeReferencesThatNeedTokens(ITypeReference typeReference)
         {
             this.typeReferenceNeedsToken = true;
