@@ -66,36 +66,33 @@ namespace Microsoft.CodeAnalysis
                 }
             }
 
-            public override IEnumerable<TAssemblySymbol> AvailableSymbols
+            public override ImmutableArray<TAssemblySymbol> AvailableSymbols
             {
                 get
                 {
-                    throw ExceptionUtilities.Unreachable;
+                    throw ExceptionUtilities.Unreachable();
                 }
             }
 
             public override AssemblyReferenceBinding[] BindAssemblyReferences(
-                ImmutableArray<AssemblyData> assemblies,
+                MultiDictionary<string, (AssemblyData DefinitionData, int DefinitionIndex)> assemblies,
                 AssemblyIdentityComparer assemblyIdentityComparer)
             {
                 var boundReferences = new AssemblyReferenceBinding[_referencedAssemblies.Length];
 
                 for (int i = 0; i < _referencedAssemblyData.Length; i++)
                 {
-                    Debug.Assert(ReferenceEquals(_referencedAssemblyData[i], assemblies[i + 1]));
-                    boundReferences[i] = new AssemblyReferenceBinding(assemblies[i + 1].Identity, i + 1);
+                    Debug.Assert(assemblies[_referencedAssemblyData[i].Identity.Name].Contains((_referencedAssemblyData[i], i + 1)));
+                    boundReferences[i] = new AssemblyReferenceBinding(_referencedAssemblyData[i].Identity, i + 1);
                 }
 
-                // references from added modules shouldn't resolve against the assembly being built (definition #0)
-                const int definitionStartIndex = 1;
-
                 // resolve references coming from linked modules:
                 for (int i = _referencedAssemblyData.Length; i < _referencedAssemblies.Length; i++)
                 {
                     boundReferences[i] = ResolveReferencedAssembly(
                         _referencedAssemblies[i],
                         assemblies,
-                        definitionStartIndex,
+                        resolveAgainstAssemblyBeingBuilt: false, // references from added modules shouldn't resolve against the assembly being built (definition #0)
                         assemblyIdentityComparer);
                 }
 
@@ -104,14 +101,14 @@ namespace Microsoft.CodeAnalysis
 
             public override bool IsMatchingAssembly(TAssemblySymbol? assembly)
             {
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             }
 
             public override bool ContainsNoPiaLocalTypes
             {
                 get
                 {
-                    throw ExceptionUtilities.Unreachable;
+                    throw ExceptionUtilities.Unreachable();
                 }
             }
 
