@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
+using System;
 using System.Collections.Generic;
 using System.Diagnostics;
 
@@ -190,6 +191,7 @@ namespace Microsoft.CodeAnalysis
 
         System_Collections_IList,
         System_Collections_ICollection,
+        System_Collections_Immutable_ImmutableArray_T,
         System_Collections_Generic_EqualityComparer_T,
         System_Collections_Generic_List_T,
         System_Collections_Generic_IDictionary_KV,
@@ -251,11 +253,11 @@ namespace Microsoft.CodeAnalysis
 
         System_ValueTuple,
         System_ValueTuple_T1,
-        System_ValueTuple_T2,
-        System_ValueTuple_T3,
 
         ExtSentinel, // Not a real type, just a marker for types above 255 and strictly below 512
 
+        System_ValueTuple_T2,
+        System_ValueTuple_T3,
         System_ValueTuple_T4,
         System_ValueTuple_T5,
         System_ValueTuple_T6,
@@ -265,12 +267,14 @@ namespace Microsoft.CodeAnalysis
         System_Runtime_CompilerServices_TupleElementNamesAttribute,
 
         Microsoft_CodeAnalysis_Runtime_Instrumentation,
+        Microsoft_CodeAnalysis_Runtime_LocalStoreTracker,
         System_Runtime_CompilerServices_NullableAttribute,
         System_Runtime_CompilerServices_NullableContextAttribute,
         System_Runtime_CompilerServices_NullablePublicOnlyAttribute,
         System_Runtime_CompilerServices_ReferenceAssemblyAttribute,
 
         System_Runtime_CompilerServices_IsReadOnlyAttribute,
+        System_Runtime_CompilerServices_RequiresLocationAttribute,
         System_Runtime_CompilerServices_IsByRefLikeAttribute,
         System_Runtime_InteropServices_InAttribute,
         System_ObsoleteAttribute,
@@ -310,13 +314,30 @@ namespace Microsoft.CodeAnalysis
 
         System_Runtime_CompilerServices_IsExternalInit,
         System_Runtime_InteropServices_OutAttribute,
+        System_Runtime_InteropServices_MemoryMarshal,
+        System_Runtime_InteropServices_CollectionsMarshal,
+        System_Runtime_InteropServices_ImmutableCollectionsMarshal,
 
         System_Text_StringBuilder,
 
         System_Runtime_CompilerServices_DefaultInterpolatedStringHandler,
+        System_Runtime_CompilerServices_ScopedRefAttribute,
+        System_Runtime_CompilerServices_RefSafetyRulesAttribute,
+
+        System_ArgumentNullException,
+
+        System_Runtime_CompilerServices_RequiredMemberAttribute,
+        System_Diagnostics_CodeAnalysis_SetsRequiredMembersAttribute,
+        System_MemoryExtensions,
+
+        System_Runtime_CompilerServices_CompilerFeatureRequiredAttribute,
+        System_Diagnostics_CodeAnalysis_UnscopedRefAttribute,
+
+        System_MissingMethodException,
+        System_Runtime_CompilerServices_MetadataUpdateOriginalTypeAttribute,
+        System_Runtime_CompilerServices_Unsafe,
 
         NextAvailable,
-
         // Remember to update the AllWellKnownTypes tests when making changes here
     }
 
@@ -500,6 +521,7 @@ namespace Microsoft.CodeAnalysis
 
             "System.Collections.IList",
             "System.Collections.ICollection",
+            "System.Collections.Immutable.ImmutableArray`1",
             "System.Collections.Generic.EqualityComparer`1",
             "System.Collections.Generic.List`1",
             "System.Collections.Generic.IDictionary`2",
@@ -560,11 +582,11 @@ namespace Microsoft.CodeAnalysis
 
             "System.ValueTuple",
             "System.ValueTuple`1",
-            "System.ValueTuple`2",
-            "System.ValueTuple`3",
 
             "", // extension marker
 
+            "System.ValueTuple`2",
+            "System.ValueTuple`3",
             "System.ValueTuple`4",
             "System.ValueTuple`5",
             "System.ValueTuple`6",
@@ -574,6 +596,7 @@ namespace Microsoft.CodeAnalysis
             "System.Runtime.CompilerServices.TupleElementNamesAttribute",
 
             "Microsoft.CodeAnalysis.Runtime.Instrumentation",
+            "Microsoft.CodeAnalysis.Runtime.LocalStoreTracker",
 
             "System.Runtime.CompilerServices.NullableAttribute",
             "System.Runtime.CompilerServices.NullableContextAttribute",
@@ -581,6 +604,7 @@ namespace Microsoft.CodeAnalysis
             "System.Runtime.CompilerServices.ReferenceAssemblyAttribute",
 
             "System.Runtime.CompilerServices.IsReadOnlyAttribute",
+            "System.Runtime.CompilerServices.RequiresLocationAttribute",
             "System.Runtime.CompilerServices.IsByRefLikeAttribute",
             "System.Runtime.InteropServices.InAttribute",
             "System.ObsoleteAttribute",
@@ -620,9 +644,24 @@ namespace Microsoft.CodeAnalysis
             "System.Runtime.CompilerServices.NativeIntegerAttribute",
             "System.Runtime.CompilerServices.IsExternalInit",
             "System.Runtime.InteropServices.OutAttribute",
+            "System.Runtime.InteropServices.MemoryMarshal",
+            "System.Runtime.InteropServices.CollectionsMarshal",
+            "System.Runtime.InteropServices.ImmutableCollectionsMarshal",
 
             "System.Text.StringBuilder",
             "System.Runtime.CompilerServices.DefaultInterpolatedStringHandler",
+            "System.Runtime.CompilerServices.ScopedRefAttribute",
+            "System.Runtime.CompilerServices.RefSafetyRulesAttribute",
+            "System.ArgumentNullException",
+
+            "System.Runtime.CompilerServices.RequiredMemberAttribute",
+            "System.Diagnostics.CodeAnalysis.SetsRequiredMembersAttribute",
+            "System.MemoryExtensions",
+            "System.Runtime.CompilerServices.CompilerFeatureRequiredAttribute",
+            "System.Diagnostics.CodeAnalysis.UnscopedRefAttribute",
+            "System.MissingMethodException",
+            "System.Runtime.CompilerServices.MetadataUpdateOriginalTypeAttribute",
+            "System.Runtime.CompilerServices.Unsafe",
         };
 
         private static readonly Dictionary<string, WellKnownType> s_nameToTypeIdMap = new Dictionary<string, WellKnownType>((int)Count);
@@ -675,7 +714,7 @@ namespace Microsoft.CodeAnalysis
                     typeIdName = typeIdName.Substring(0, separator);
                 }
 
-                Debug.Assert(name == typeIdName, "Enum name and type name must match");
+                Debug.Assert(name == typeIdName, $"Enum name ({typeIdName}) and type name ({name}) must match at {i}");
             }
 
             Debug.Assert((int)WellKnownType.ExtSentinel == 255);
