@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System;
 using System.Collections.Generic;
 using System.Collections.Immutable;
@@ -14,22 +12,55 @@ using System.Reflection.PortableExecutable;
 using System.Text;
 using System.Xml.Linq;
 using Microsoft.CodeAnalysis.CodeGen;
+using Microsoft.CodeAnalysis.CSharp.Symbols;
 using Microsoft.CodeAnalysis.CSharp.Symbols.Metadata.PE;
 using Microsoft.CodeAnalysis.Emit;
 using Microsoft.CodeAnalysis.Operations;
+using Microsoft.CodeAnalysis.Text;
 using Roslyn.Test.Utilities;
 using Roslyn.Utilities;
 using Xunit;
 
 namespace Microsoft.CodeAnalysis.Test.Utilities
 {
-    public enum Verification
+    [Flags]
+    public enum VerificationStatus
     {
+        /// <summary>
+        /// default(<see cref="Verification"/>) should be passing.
+        /// </summary>
         Passes = 0,
-        Fails,
-        Skipped
+
+        Skipped = 1 << 1,
+
+        FailsPEVerify = 1 << 2,
+        FailsILVerify = 1 << 3,
+        Fails = FailsPEVerify | FailsILVerify,
+
+        PassesOrFailFast = 1 << 4,
     }
 
+    public readonly struct Verification
+    {
+        public VerificationStatus Status { get; init; }
+        public string? ILVerifyMessage { get; init; }
+        public string? PEVerifyMessage { get; init; }
+
+        /// <summary>
+        /// True if the expected messages include member tokens and MVIDs.
+        /// </summary>
+        public bool IncludeTokensAndModuleIds { get; init; }
+
+        public static readonly Verification Skipped = new() { Status = VerificationStatus.Skipped };
+        public static readonly Verification Passes = new() { Status = VerificationStatus.Passes };
+        public static readonly Verification FailsPEVerify = new() { Status = VerificationStatus.FailsPEVerify };
+        public static readonly Verification FailsILVerify = new() { Status = VerificationStatus.FailsILVerify };
+        public static readonly Verification Fails = new() { Status = VerificationStatus.Fails };
+        public static readonly Verification PassesOrFailFast = new() { Status = VerificationStatus.PassesOrFailFast };
+    }
+
+#nullable disable
+
     /// <summary>
     /// Base class for all language specific tests.
     /// </summary>
@@ -46,10 +77,11 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
             Action<IModuleSymbol> symbolValidator = null,
             SignatureDescription[] expectedSignatures = null,
             string expectedOutput = null,
+            bool trimOutput = true,
             int? expectedReturnCode = null,
             string[] args = null,
             EmitOptions emitOptions = null,
-            Verification verify = Verification.Passes)
+            Verification verify = default)
         {
             Assert.NotNull(compilation);
 
@@ -70,6 +102,7 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                                 manifestResources,
                                 expectedSignatures,
                                 expectedOutput,
+                                trimOutput,
                                 expectedReturnCode,
                                 args ?? Array.Empty<string>(),
                                 assemblyValidator,
@@ -142,6 +175,7 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
             IEnumerable<ResourceDescription> manifestResources,
             SignatureDescription[] expectedSignatures,
             string expectedOutput,
+            bool trimOutput,
             int? expectedReturnCode,
             string[] args,
             Action<PEAssembly> assemblyValidator,
@@ -151,12 +185,12 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
         {
             var verifier = new CompilationVerifier(compilation, VisualizeRealIL, dependencies);
 
-            verifier.Emit(expectedOutput, expectedReturnCode, args, manifestResources, emitOptions, verify, expectedSignatures);
+            verifier.Emit(expectedOutput, trimOutput, expectedReturnCode, args, manifestResources, emitOptions, verify, expectedSignatures);
 
             if (assemblyValidator != null || symbolValidator != null)
             {
                 // We're dual-purposing emitters here.  In this context, it
-                // tells the validator the version of Emit that is calling it. 
+                // tells the validator the version of Emit that is calling it.
                 RunValidators(verifier, assemblyValidator, symbolValidator);
             }
 
@@ -292,9 +326,33 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
             IEnumerable<MetadataReference> referencedAssemblies = null,
             IEnumerable<Compilation> referencedCompilations = null)
         {
+            return CreateCSharpCompilation(assemblyName, assemblyIdentity: null, code, parseOptions, compilationOptions, referencedAssemblies, referencedCompilations);
+        }
+
+        protected CSharp.CSharpCompilation CreateCSharpCompilation(
+            AssemblyIdentity assemblyIdentity,
+            string code,
+            CSharp.CSharpParseOptions parseOptions = null,
+            CSharp.CSharpCompilationOptions compilationOptions = null,
+            IEnumerable<MetadataReference> referencedAssemblies = null,
+            IEnumerable<Compilation> referencedCompilations = null)
+        {
+            return CreateCSharpCompilation(assemblyName: null, assemblyIdentity, code, parseOptions, compilationOptions, referencedAssemblies, referencedCompilations);
+        }
+
+        private CSharp.CSharpCompilation CreateCSharpCompilation(
+            string assemblyName,
+            AssemblyIdentity assemblyIdentity,
+            string code,
+            CSharp.CSharpParseOptions parseOptions = null,
+            CSharp.CSharpCompilationOptions compilationOptions = null,
+            IEnumerable<MetadataReference> referencedAssemblies = null,
+            IEnumerable<Compilation> referencedCompilations = null)
+        {
+            Debug.Assert(assemblyName == null || assemblyIdentity == null || assemblyIdentity.Name == assemblyName);
             if (assemblyName == null)
             {
-                assemblyName = GetUniqueName();
+                assemblyName = assemblyIdentity?.Name ?? GetUniqueName();
             }
 
             if (parseOptions == null)
@@ -324,9 +382,16 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
 
             AddReferencedCompilations(referencedCompilations, references);
 
-            var tree = CSharp.SyntaxFactory.ParseSyntaxTree(code, options: parseOptions);
+            var tree = CSharp.SyntaxFactory.ParseSyntaxTree(SourceText.From(code, encoding: null, SourceHashAlgorithms.Default), options: parseOptions);
 
-            return CSharp.CSharpCompilation.Create(assemblyName, new[] { tree }, references, compilationOptions);
+            var compilation = CSharp.CSharpCompilation.Create(assemblyName, new[] { tree }, references, compilationOptions);
+
+            if (assemblyIdentity != null)
+            {
+                ((SourceAssemblySymbol)compilation.Assembly).lazyAssemblyIdentity = assemblyIdentity;
+            }
+
+            return compilation;
         }
 
         protected VisualBasic.VisualBasicCompilation CreateVisualBasicCompilation(
@@ -407,10 +472,9 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
             var trees = new SyntaxTree[files.Length];
             for (int i = 0; i < files.Length; i++)
             {
-                trees[i] = VisualBasic.VisualBasicSyntaxTree.ParseText(files[i], options: parseOptions, encoding: encoding, path: sourceFileNames?[i]);
+                trees[i] = VisualBasic.VisualBasicSyntaxTree.ParseText(SourceText.From(files[i], encoding, SourceHashAlgorithms.Default), options: parseOptions, path: sourceFileNames?[i]);
             }
 
-
             return VisualBasic.VisualBasicCompilation.Create(assemblyName, trees, references, compilationOptions);
         }
 
@@ -549,7 +613,7 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
         private static void CollectParentOperations(IOperation operation, Dictionary<IOperation, IOperation> map)
         {
             // walk down to collect all parent operation map for this tree
-            foreach (var child in operation.Children.WhereNotNull())
+            foreach (var child in operation.ChildOperations)
             {
                 map.Add(child, operation);
 
@@ -618,7 +682,7 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                 // all operations from spine should belong to the operation tree set
                 VerifyOperationTreeSpine(semanticModel, set, child.Syntax);
 
-                // operation tree's node must be part of root of semantic model which is 
+                // operation tree's node must be part of root of semantic model which is
                 // owner of operation's lifetime
                 Assert.True(semanticModel.Root.FullSpan.Contains(child.Syntax.FullSpan));
             }
