@@ -11,6 +11,7 @@ using System.Diagnostics;
 using System.Diagnostics.CodeAnalysis;
 using System.Linq;
 using Microsoft.CodeAnalysis.CSharp;
+using Microsoft.CodeAnalysis.CSharp.Symbols;
 using Microsoft.CodeAnalysis.CSharp.Syntax;
 using Microsoft.CodeAnalysis.FlowAnalysis;
 using Microsoft.CodeAnalysis.Operations;
@@ -64,6 +65,10 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                     graph = ControlFlowGraph.Create(parameterInitializerOperation);
                     break;
 
+                case IAttributeOperation attributeOperation:
+                    graph = ControlFlowGraph.Create(attributeOperation);
+                    break;
+
                 default:
                     return default;
             }
@@ -310,7 +315,6 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                     validateBranch(block, nextBranch);
                 }
 
-
                 if (currentRegion.LastBlockOrdinal == block.Ordinal && i != blocks.Length - 1)
                 {
                     leaveRegions(block.EnclosingRegion, block.Ordinal);
@@ -807,6 +811,14 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                 foreach (IFlowCaptureReferenceOperation reference in operation.DescendantsAndSelf().OfType<IFlowCaptureReferenceOperation>())
                 {
                     CaptureId id = reference.Id;
+
+                    if (reference.IsInitialization)
+                    {
+                        AssertTrueWithGraph(state.Add(id), $"Multiple initialization of [{id}]", finalGraph);
+                        AssertTrueWithGraph(block.EnclosingRegion.CaptureIds.Contains(id), $"Flow capture initialization [{id}] should come from the containing region.", finalGraph);
+                        continue;
+                    }
+
                     referencedIds.Add(id);
 
                     if (isLongLivedCaptureReference(reference, block.EnclosingRegion))
@@ -819,14 +831,34 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
 
                     // Except for a few specific scenarios, any references to captures should either be long-lived capture references,
                     // or they should come from the enclosing region.
-                    AssertTrueWithGraph(block.EnclosingRegion.CaptureIds.Contains(id) || longLivedIds.Contains(id) ||
-                                ((isFirstOperandOfDynamicOrUserDefinedLogicalOperator(reference) ||
-                                     isIncrementedNullableForToLoopControlVariable(reference) ||
-                                     isConditionalAccessReceiver(reference) ||
-                                     isCoalesceAssignmentTarget(reference) ||
-                                     isObjectInitializerInitializedObjectTarget(reference)) &&
-                                 block.EnclosingRegion.EnclosingRegion.CaptureIds.Contains(id)),
-                        $"Operation [{operationIndex}] in [{getBlockId(block)}] uses capture [{id.Value}] from another region. Should the regions be merged?", finalGraph);
+                    if (block.EnclosingRegion.CaptureIds.Contains(id) || longLivedIds.Contains(id))
+                    {
+                        continue;
+                    }
+
+                    if (block.EnclosingRegion.EnclosingRegion.CaptureIds.Contains(id))
+                    {
+                        AssertTrueWithGraph(
+                            isFirstOperandOfDynamicOrUserDefinedLogicalOperator(reference)
+                            || isIncrementedNullableForToLoopControlVariable(reference)
+                            || isConditionalAccessReceiver(reference)
+                            || isCoalesceAssignmentTarget(reference)
+                            || isObjectInitializerInitializedObjectTarget(reference)
+                            || isInterpolatedStringArgumentCapture(reference)
+                            || isInterpolatedStringHandlerCapture(reference),
+                            $"Operation [{operationIndex}] in [{getBlockId(block)}] uses capture [{id.Value}] from another region. Should the regions be merged?", finalGraph);
+                    }
+                    else if (block.EnclosingRegion.EnclosingRegion?.EnclosingRegion.CaptureIds.Contains(id) ?? false)
+                    {
+                        AssertTrueWithGraph(
+                            isInterpolatedStringArgumentCapture(reference)
+                            || isInterpolatedStringHandlerCapture(reference),
+                            $"Operation [{operationIndex}] in [{getBlockId(block)}] uses capture [{id.Value}] from another region. Should the regions be merged?", finalGraph);
+                    }
+                    else
+                    {
+                        AssertTrueWithGraph(false, $"Operation [{operationIndex}] in [{getBlockId(block)}] uses capture [{id.Value}] from another region. Should the regions be merged?", finalGraph);
+                    }
                 }
             }
 
@@ -892,6 +924,59 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                 } && left == referenceSyntax;
             }
 
+            bool isInterpolatedStringArgumentCapture(IFlowCaptureReferenceOperation reference)
+            {
+                if (reference.Language != LanguageNames.CSharp)
+                {
+                    return false;
+                }
+
+                IOperation containingArgument = reference;
+                do
+                {
+                    containingArgument = containingArgument.Parent;
+                }
+                while (containingArgument is not (null or IArgumentOperation));
+
+#pragma warning disable IDE0055 // Fix formatting
+                return containingArgument is
+                       {
+                           Parent: IObjectCreationOperation
+                           {
+                               Parent: IFlowCaptureOperation,
+                               Constructor.ContainingType: INamedTypeSymbol ctorContainingType,
+                               Arguments: { Length: >= 3 } arguments,
+                               Syntax: CSharpSyntaxNode syntax
+                           }
+                       }
+                       && applyParenthesizedOrNullSuppressionIfAnyCS(syntax) is CSharp.Syntax.InterpolatedStringExpressionSyntax or CSharp.Syntax.BinaryExpressionSyntax
+                       && ctorContainingType.GetSymbol().IsInterpolatedStringHandlerType
+                       && arguments[0].Value.Type.SpecialType == SpecialType.System_Int32
+                       && arguments[1].Value.Type.SpecialType == SpecialType.System_Int32;
+#pragma warning restore IDE0055
+            }
+
+            bool isInterpolatedStringHandlerCapture(IFlowCaptureReferenceOperation reference)
+            {
+                if (reference.Language != LanguageNames.CSharp)
+                {
+                    return false;
+                }
+
+#pragma warning disable IDE0055 // Fix formatting
+                return reference is
+                       {
+                           Parent: IInvocationOperation
+                           {
+                               Instance: { } instance,
+                               TargetMethod: { Name: BoundInterpolatedString.AppendFormattedMethod or BoundInterpolatedString.AppendLiteralMethod, ContainingType: INamedTypeSymbol containingType }
+                           }
+                       }
+                       && ReferenceEquals(instance, reference)
+                       && containingType.GetSymbol().IsInterpolatedStringHandlerType;
+#pragma warning restore IDE0055
+            }
+
             bool isFirstOperandOfDynamicOrUserDefinedLogicalOperator(IFlowCaptureReferenceOperation reference)
             {
                 if (reference.Parent is IBinaryOperation binOp)
@@ -1058,6 +1143,12 @@ namespace Microsoft.CodeAnalysis.Test.Utilities
                                         return true;
                                     }
                                     break;
+                                case CSharp.SyntaxKind.CollectionExpression:
+                                    if (((CSharp.Syntax.CollectionExpressionSyntax)syntax).Elements.Any())
+                                    {
+                                        return true;
+                                    }
+                                    break;
                             }
 
                             if (syntax.Parent is CSharp.Syntax.WithExpressionSyntax withExpr
@@ -1825,6 +1916,9 @@ endRegion:
                     var instanceReference = (IInstanceReferenceOperation)n;
                     return instanceReference.ReferenceKind == InstanceReferenceKind.ContainingTypeInstance ||
                         instanceReference.ReferenceKind == InstanceReferenceKind.PatternInput ||
+                        // Will be removed when CFG support for interpolated string handlers is implemented, tracked by
+                        // https://github.com/dotnet/roslyn/issues/54718
+                        instanceReference.ReferenceKind == InstanceReferenceKind.InterpolatedStringHandler ||
                         (instanceReference.ReferenceKind == InstanceReferenceKind.ImplicitReceiver &&
                          n.Type.IsAnonymousType &&
                          n.Parent is IPropertyReferenceOperation propertyReference &&
@@ -1836,12 +1930,14 @@ endRegion:
                 case OperationKind.None:
                     return !(n is IPlaceholderOperation);
 
+                case OperationKind.FunctionPointerInvocation:
                 case OperationKind.Invalid:
                 case OperationKind.YieldReturn:
                 case OperationKind.ExpressionStatement:
                 case OperationKind.Stop:
                 case OperationKind.RaiseEvent:
                 case OperationKind.Literal:
+                case OperationKind.Utf8String:
                 case OperationKind.Conversion:
                 case OperationKind.Invocation:
                 case OperationKind.ArrayElementReference:
@@ -1904,6 +2000,14 @@ endRegion:
                 case OperationKind.NegatedPattern:
                 case OperationKind.BinaryPattern:
                 case OperationKind.TypePattern:
+                case OperationKind.InterpolatedStringAppendFormatted:
+                case OperationKind.InterpolatedStringAppendLiteral:
+                case OperationKind.InterpolatedStringAppendInvalid:
+                case OperationKind.SlicePattern:
+                case OperationKind.ListPattern:
+                case OperationKind.ImplicitIndexerReference:
+                case OperationKind.Attribute:
+                case OperationKind.InlineArrayAccess:
                     return true;
             }
 
