@@ -35,8 +35,8 @@ namespace Microsoft.CodeAnalysis.CSharp.UnitTests
 
             TypeCompilationState compilationState = new TypeCompilationState(method.ContainingType, compilation, module);
 
-            var diagnostics = DiagnosticBag.GetInstance();
-            var block = MethodCompiler.BindMethodBody(method, compilationState, new BindingDiagnosticBag(diagnostics));
+            var diagnostics = BindingDiagnosticBag.GetInstance(withDiagnostics: true, withDependencies: false);
+            var block = MethodCompiler.BindSynthesizedMethodBody(method, compilationState, diagnostics);
             diagnostics.Free();
             return block;
         }
