@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System.ComponentModel.Composition;
 using Microsoft.VisualStudio.Utilities;
 
@@ -21,11 +19,11 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.ContentType
         // From Microsoft.VisualStudio.LanguageServer.Client.CodeRemoteContentDefinition.CodeRemoteBaseTypeName
         // We cannot directly reference the LSP client package in EditorFeatures as it is a VS dependency.
         [BaseDefinition("code-languageserver-base")]
-        public static readonly ContentTypeDefinition CSharpContentTypeDefinition;
+        public static readonly ContentTypeDefinition CSharpContentTypeDefinition = null!;
 
         [Export]
         [Name(ContentTypeNames.CSharpSignatureHelpContentType)]
         [BaseDefinition("sighelp")]
-        public static readonly ContentTypeDefinition SignatureHelpContentTypeDefinition;
+        public static readonly ContentTypeDefinition SignatureHelpContentTypeDefinition = null!;
     }
 }
