@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CSharp.Test.Utilities;
 using Microsoft.CodeAnalysis.Editor.UnitTests.BraceMatching;
@@ -14,12 +12,13 @@ using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
 {
+    [Trait(Traits.Feature, Traits.Features.BraceMatching)]
     public class CSharpBraceMatcherTests : AbstractBraceMatcherTests
     {
         protected override TestWorkspace CreateWorkspaceFromCode(string code, ParseOptions options)
             => TestWorkspace.CreateCSharp(code, options);
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestEmptyFile()
         {
             var code = @"$$";
@@ -28,7 +27,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestAtFirstPositionInFile()
         {
             var code = @"$$public class C { }";
@@ -37,7 +36,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestAtLastPositionInFile()
         {
             var code = @"public class C { }$$";
@@ -46,7 +45,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestCurlyBrace1()
         {
             var code = @"public class C $${ }";
@@ -55,7 +54,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestCurlyBrace2()
         {
             var code = @"public class C {$$ }";
@@ -64,7 +63,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestCurlyBrace3()
         {
             var code = @"public class C { $$}";
@@ -73,7 +72,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestCurlyBrace4()
         {
             var code = @"public class C { }$$";
@@ -82,7 +81,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestParen1()
         {
             var code = @"public class C { void Goo$$() { } }";
@@ -91,7 +90,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestParen2()
         {
             var code = @"public class C { void Goo($$) { } }";
@@ -100,7 +99,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestParen3()
         {
             var code = @"public class C { void Goo($$ ) { } }";
@@ -109,7 +108,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestParen4()
         {
             var code = @"public class C { void Goo( $$) { } }";
@@ -118,7 +117,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestParen5()
         {
             var code = @"public class C { void Goo( )$$ { } }";
@@ -127,7 +126,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestParen6()
         {
             var code = @"public class C { void Goo()$$ { } }";
@@ -136,7 +135,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestSquareBracket1()
         {
             var code = @"public class C { int$$[] i; }";
@@ -145,7 +144,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestSquareBracket2()
         {
             var code = @"public class C { int[$$] i; }";
@@ -154,7 +153,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestSquareBracket3()
         {
             var code = @"public class C { int[$$ ] i; }";
@@ -163,7 +162,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestSquareBracket4()
         {
             var code = @"public class C { int[ $$] i; }";
@@ -172,7 +171,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestSquareBracket5()
         {
             var code = @"public class C { int[ ]$$ i; }";
@@ -181,7 +180,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestSquareBracket6()
         {
             var code = @"public class C { int[]$$ i; }";
@@ -190,7 +189,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestAngleBracket1()
         {
             var code = @"public class C { Goo$$<int> f; }";
@@ -199,7 +198,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestAngleBracket2()
         {
             var code = @"public class C { Goo<$$int> f; }";
@@ -208,7 +207,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestAngleBracket3()
         {
             var code = @"public class C { Goo<int$$> f; }";
@@ -217,7 +216,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestAngleBracket4()
         {
             var code = @"public class C { Goo<int>$$ f; }";
@@ -226,7 +225,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket1()
         {
             var code = @"public class C { Func$$<Func<int,int>> f; }";
@@ -235,7 +234,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket2()
         {
             var code = @"public class C { Func<$$Func<int,int>> f; }";
@@ -244,7 +243,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket3()
         {
             var code = @"public class C { Func<Func$$<int,int>> f; }";
@@ -253,7 +252,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket4()
         {
             var code = @"public class C { Func<Func<$$int,int>> f; }";
@@ -262,7 +261,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket5()
         {
             var code = @"public class C { Func<Func<int,int$$>> f; }";
@@ -271,7 +270,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket6()
         {
             var code = @"public class C { Func<Func<int,int>$$> f; }";
@@ -280,7 +279,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket7()
         {
             var code = @"public class C { Func<Func<int,int> $$> f; }";
@@ -289,7 +288,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestNestedAngleBracket8()
         {
             var code = @"public class C { Func<Func<int,int>>$$ f; }";
@@ -298,7 +297,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestString1()
         {
             var code = @"public class C { string s = $$""Goo""; }";
@@ -307,7 +306,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestString2()
         {
             var code = @"public class C { string s = ""$$Goo""; }";
@@ -316,7 +315,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestString3()
         {
             var code = @"public class C { string s = ""Goo$$""; }";
@@ -325,7 +324,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestString4()
         {
             var code = @"public class C { string s = ""Goo""$$; }";
@@ -334,7 +333,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestString5()
         {
             var code = @"public class C { string s = ""Goo$$ ";
@@ -343,7 +342,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestVerbatimString1()
         {
             var code = @"public class C { string s = $$@""Goo""; }";
@@ -352,7 +351,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestVerbatimString2()
         {
             var code = @"public class C { string s = @$$""Goo""; }";
@@ -361,7 +360,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestVerbatimString3()
         {
             var code = @"public class C { string s = @""$$Goo""; }";
@@ -370,7 +369,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestVerbatimString4()
         {
             var code = @"public class C { string s = @""Goo$$""; }";
@@ -379,7 +378,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestVerbatimString5()
         {
             var code = @"public class C { string s = @""Goo""$$; }";
@@ -388,7 +387,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString1()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""$${x}, {y}""; }";
@@ -397,7 +396,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString2()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{$$x}, {y}""; }";
@@ -406,7 +405,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString3()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{x$$}, {y}""; }";
@@ -415,7 +414,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString4()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{x}$$, {y}""; }";
@@ -424,7 +423,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString5()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{x}, $${y}""; }";
@@ -433,7 +432,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString6()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{x}, {$$y}""; }";
@@ -442,7 +441,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString7()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{x}, {y$$}""; }";
@@ -451,7 +450,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString8()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $""{x}, {y}$$""; }";
@@ -460,7 +459,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString9()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $$[||]$""{x}, {y}""; }";
@@ -469,7 +468,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString10()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $[||]$$""{x}, {y}""; }";
@@ -478,7 +477,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString11()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $$[||]$@""{x}, {y}""; }";
@@ -487,7 +486,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString12()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $[||]$$@""{x}, {y}""; }";
@@ -496,7 +495,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestInterpolatedString13()
         {
             var code = @"public class C { void M() { var x = ""Hello""; var y = ""World""; var s = $@$$""{x}, {y}""; }";
@@ -505,253 +504,390 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.BraceMatching
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
+        public async Task TestUtf8String1()
+        {
+            var code = @"public class C { string s = $$""Goo""u8; }";
+            var expected = @"public class C { string s = ""Goo[|""u8|]; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestUtf8String2()
+        {
+            var code = @"public class C { string s = ""$$Goo""u8; }";
+            var expected = @"public class C { string s = ""Goo[|""u8|]; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestUtf8String3()
+        {
+            var code = @"public class C { string s = ""Goo$$""u8; }";
+            var expected = @"public class C { string s = [|""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestUtf8String4()
+        {
+            var code = @"public class C { string s = ""Goo""$$u8; }";
+            var expected = @"public class C { string s = [|""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestUtf8String5()
+        {
+            var code = @"public class C { string s = ""Goo""u$$8; }";
+            var expected = @"public class C { string s = [|""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestUtf8String6()
+        {
+            var code = @"public class C { string s = ""Goo""u8$$; }";
+            var expected = @"public class C { string s = [|""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String1()
+        {
+            var code = @"public class C { string s = $$@""Goo""u8; }";
+            var expected = @"public class C { string s = @""Goo[|""u8|]; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String2()
+        {
+            var code = @"public class C { string s = @$$""Goo""u8; }";
+            var expected = @"public class C { string s = @""Goo[|""u8|]; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String3()
+        {
+            var code = @"public class C { string s = @""$$Goo""u8; }";
+            var expected = @"public class C { string s = @""Goo[|""u8|]; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String4()
+        {
+            var code = @"public class C { string s = @""Goo$$""u8; }";
+            var expected = @"public class C { string s = [|@""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String5()
+        {
+            var code = @"public class C { string s = @""Goo""$$u8; }";
+            var expected = @"public class C { string s = [|@""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String6()
+        {
+            var code = @"public class C { string s = @""Goo""u$$8; }";
+            var expected = @"public class C { string s = [|@""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [Fact]
+        public async Task TestVerbatimUtf8String7()
+        {
+            var code = @"public class C { string s = @""Goo""u8$$; }";
+            var expected = @"public class C { string s = [|@""|]Goo""u8; }";
+
+            await TestAsync(code, expected);
+        }
+
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestConditionalDirectiveWithSingleMatchingDirective()
         {
-            var code = @"
-public class C 
-{
-#if$$ CHK 
-#endif
-}";
-            var expected = @"
-public class C 
-{
-#if$$ CHK 
-[|#endif|]
-}";
+            var code = """
+                public class C 
+                {
+                #if$$ CHK 
+                #endif
+                }
+                """;
+            var expected = """
+                public class C 
+                {
+                #if$$ CHK 
+                [|#endif|]
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestConditionalDirectiveWithTwoMatchingDirectives()
         {
-            var code = @"
-public class C 
-{
-#if$$ CHK 
-#else
-#endif
-}";
-            var expected = @"
-public class C 
-{
-#if$$ CHK 
-[|#else|]
-#endif
-}";
+            var code = """
+                public class C 
+                {
+                #if$$ CHK 
+                #else
+                #endif
+                }
+                """;
+            var expected = """
+                public class C 
+                {
+                #if$$ CHK 
+                [|#else|]
+                #endif
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestConditionalDirectiveWithAllMatchingDirectives()
         {
-            var code = @"
-public class C 
-{
-#if CHK 
-#elif RET
-#else
-#endif$$
-}";
-            var expected = @"
-public class C 
-{
-[|#if|] CHK 
-#elif RET
-#else
-#endif
-}";
+            var code = """
+                public class C 
+                {
+                #if CHK 
+                #elif RET
+                #else
+                #endif$$
+                }
+                """;
+            var expected = """
+                public class C 
+                {
+                [|#if|] CHK 
+                #elif RET
+                #else
+                #endif
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestRegionDirective()
         {
-            var code = @"
-public class C 
-{
-$$#region test
-#endregion
-}";
-            var expected = @"
-public class C 
-{
-#region test
-[|#endregion|]
-}";
+            var code = """
+                public class C 
+                {
+                $$#region test
+                #endregion
+                }
+                """;
+            var expected = """
+                public class C 
+                {
+                #region test
+                [|#endregion|]
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestInterleavedDirectivesInner()
         {
-            var code = @"
-#define CHK
-public class C 
-{
-    void Test()
-    {
-#if CHK
-$$#region test
-    var x = 5;
-#endregion
-#else
-    var y = 6;
-#endif
-    }
-}";
-            var expected = @"
-#define CHK
-public class C 
-{
-    void Test()
-    {
-#if CHK
-#region test
-    var x = 5;
-[|#endregion|]
-#else
-    var y = 6;
-#endif
-    }
-}";
+            var code = """
+                #define CHK
+                public class C 
+                {
+                    void Test()
+                    {
+                #if CHK
+                $$#region test
+                    var x = 5;
+                #endregion
+                #else
+                    var y = 6;
+                #endif
+                    }
+                }
+                """;
+            var expected = """
+                #define CHK
+                public class C 
+                {
+                    void Test()
+                    {
+                #if CHK
+                #region test
+                    var x = 5;
+                [|#endregion|]
+                #else
+                    var y = 6;
+                #endif
+                    }
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestInterleavedDirectivesOuter()
         {
-            var code = @"
-#define CHK
-public class C 
-{
-    void Test()
-    {
-#if$$ CHK
-#region test
-    var x = 5;
-#endregion
-#else
-    var y = 6;
-#endif
-    }
-}";
-            var expected = @"
-#define CHK
-public class C 
-{
-    void Test()
-    {
-#if CHK
-#region test
-    var x = 5;
-#endregion
-[|#else|]
-    var y = 6;
-#endif
-    }
-}";
+            var code = """
+                #define CHK
+                public class C 
+                {
+                    void Test()
+                    {
+                #if$$ CHK
+                #region test
+                    var x = 5;
+                #endregion
+                #else
+                    var y = 6;
+                #endif
+                    }
+                }
+                """;
+            var expected = """
+                #define CHK
+                public class C 
+                {
+                    void Test()
+                    {
+                #if CHK
+                #region test
+                    var x = 5;
+                #endregion
+                [|#else|]
+                    var y = 6;
+                #endif
+                    }
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestUnmatchedDirective1()
         {
-            var code = @"
-public class C 
-{
-$$#region test
-}";
-            var expected = @"
-public class C 
-{
-#region test
-}";
+            var code = """
+                public class C 
+                {
+                $$#region test
+                }
+                """;
+            var expected = """
+                public class C 
+                {
+                #region test
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7120, "https://github.com/dotnet/roslyn/issues/7120")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7120")]
+        [WpfFact]
         public async Task TestUnmatchedDirective2()
         {
-            var code = @"
-#d$$efine CHK
-public class C 
-{
-}";
-            var expected = @"
-#define CHK
-public class C 
-{
-}";
+            var code = """
+                #d$$efine CHK
+                public class C 
+                {
+                }
+                """;
+            var expected = """
+                #define CHK
+                public class C 
+                {
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7534, "https://github.com/dotnet/roslyn/issues/7534")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7534")]
+        [WpfFact]
         public async Task TestUnmatchedConditionalDirective()
         {
-            var code = @"
-class Program
-{
-    static void Main(string[] args)
-    {#if$$
+            var code = """
+                class Program
+                {
+                    static void Main(string[] args)
+                    {#if$$
 
-    }
-}";
-            var expected = @"
-class Program
-{
-    static void Main(string[] args)
-    {#if
+                    }
+                }
+                """;
+            var expected = """
+                class Program
+                {
+                    static void Main(string[] args)
+                    {#if
 
-    }
-}";
+                    }
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [WorkItem(7534, "https://github.com/dotnet/roslyn/issues/7534")]
-        [WpfFact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [WorkItem("https://github.com/dotnet/roslyn/issues/7534")]
+        [WpfFact]
         public async Task TestUnmatchedConditionalDirective2()
         {
-            var code = @"
-class Program
-{
-    static void Main(string[] args)
-    {#else$$
+            var code = """
+                class Program
+                {
+                    static void Main(string[] args)
+                    {#else$$
 
-    }
-}";
-            var expected = @"
-class Program
-{
-    static void Main(string[] args)
-    {#else
+                    }
+                }
+                """;
+            var expected = """
+                class Program
+                {
+                    static void Main(string[] args)
+                    {#else
 
-    }
-}";
+                    }
+                }
+                """;
 
             await TestAsync(code, expected);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task StartTupleDeclaration()
         {
             var code = @"public class C { $$(int, int, int, int, int, int, int, int) x; }";
@@ -760,7 +896,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task EndTupleDeclaration()
         {
             var code = @"public class C { (int, int, int, int, int, int, int, int)$$ x; }";
@@ -769,7 +905,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task StartTupleLiteral()
         {
             var code = @"public class C { var x = $$(1, 2, 3, 4, 5, 6, 7, 8); }";
@@ -778,7 +914,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task EndTupleLiteral()
         {
             var code = @"public class C { var x = (1, 2, 3, 4, 5, 6, 7, 8)$$; }";
@@ -787,7 +923,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task StartNestedTupleLiteral()
         {
             var code = @"public class C { var x = $$((1, 1, 1), 2, 3, 4, 5, 6, 7, 8); }";
@@ -796,7 +932,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task StartInnerNestedTupleLiteral()
         {
             var code = @"public class C { var x = ($$(1, 1, 1), 2, 3, 4, 5, 6, 7, 8); }";
@@ -805,7 +941,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task EndNestedTupleLiteral()
         {
             var code = @"public class C { var x = (1, 2, 3, 4, 5, 6, 7, (8, 8, 8))$$; }";
@@ -814,7 +950,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task EndInnerNestedTupleLiteral()
         {
             var code = @"public class C { var x = ((1, 1, 1)$$, 2, 3, 4, 5, 6, 7, 8); }";
@@ -823,7 +959,7 @@ class Program
             await TestAsync(code, expected, TestOptions.Regular);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.BraceMatching)]
+        [Fact]
         public async Task TestFunctionPointer()
         {
             var code = @"public unsafe class C { delegate*<$$int, int> functionPointer; }";
