@@ -13,29 +13,30 @@ using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Completion.CompletionProviders
 {
+    [Trait(Traits.Feature, Traits.Features.Completion)]
     public class ExplicitInterfaceTypeCompletionProviderTests : AbstractCSharpCompletionProviderTests
     {
         internal override Type GetCompletionProviderType()
             => typeof(ExplicitInterfaceTypeCompletionProvider);
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestAtStartOfClass()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int $$
-}
-";
+                class C : IList
+                {
+                    int $$
+                }
+                """;
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
             await VerifyItemExistsAsync(markup, "ICollection");
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Theory]
         [InlineData("record")]
         [InlineData("record class")]
         [InlineData("record struct")]
@@ -61,30 +62,29 @@ using System.Collections;
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
-        [WorkItem(459044, "https://devdiv.visualstudio.com/DefaultCollection/DevDiv/_workitems?id=459044")]
+        [Fact, WorkItem("https://devdiv.visualstudio.com/DefaultCollection/DevDiv/_workitems?id=459044")]
         public async Task TestInMisplacedUsing()
         {
-            var markup = @"
-class C
-{
-    using ($$)
-}
-";
+            var markup = """
+                class C
+                {
+                    using ($$)
+                }
+                """;
             await VerifyNoItemsExistAsync(markup); // no crash
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestAtStartOfStruct()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-struct C : IList
-{
-    int $$
-}
-";
+                struct C : IList
+                {
+                    int $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -92,18 +92,18 @@ struct C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestAfterField()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int i;
-    int $$
-}
-";
+                class C : IList
+                {
+                    int i;
+                    int $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -111,18 +111,18 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestAfterMethod_01()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    void Goo() { }
-    int $$
-}
-";
+                class C : IList
+                {
+                    void Goo() { }
+                    int $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -130,18 +130,18 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestAfterMethod_02()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-interface C : IList
-{
-    void Goo() { }
-    int $$
-}
-";
+                interface C : IList
+                {
+                    void Goo() { }
+                    int $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -149,18 +149,18 @@ interface C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestAfterExpressionBody()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int Goo() => 0;
-    int $$
-}
-";
+                class C : IList
+                {
+                    int Goo() => 0;
+                    int $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -168,21 +168,21 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestWithAttributeFollowing()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int Goo() => 0;
-    int $$
+                class C : IList
+                {
+                    int Goo() => 0;
+                    int $$
 
-    [Attr]
-    int Bar();
-}
-";
+                    [Attr]
+                    int Bar();
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -190,20 +190,20 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestWithModifierFollowing()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int Goo() => 0;
-    int $$
+                class C : IList
+                {
+                    int Goo() => 0;
+                    int $$
 
-    public int Bar();
-}
-";
+                    public int Bar();
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -211,20 +211,20 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestWithTypeFollowing()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int Goo() => 0;
-    int $$
+                class C : IList
+                {
+                    int Goo() => 0;
+                    int $$
 
-    int Bar();
-}
-";
+                    int Bar();
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -232,20 +232,20 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestWithTypeFollowing2()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    int Goo() => 0;
-    int $$
+                class C : IList
+                {
+                    int Goo() => 0;
+                    int $$
 
-    X Bar();
-}
-";
+                    X Bar();
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -253,50 +253,50 @@ class C : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task NotInMember()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    void Goo()
-    {
-        int $$
-    }
-}
-";
+                class C : IList
+                {
+                    void Goo()
+                    {
+                        int $$
+                    }
+                }
+                """;
 
             await VerifyNoItemsExistAsync(markup);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task NotWithAccessibility()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-class C : IList
-{
-    public int $$
-}
-";
+                class C : IList
+                {
+                    public int $$
+                }
+                """;
 
             await VerifyNoItemsExistAsync(markup);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestInInterface()
         {
-            var markup = @"
-using System.Collections;
+            var markup = """
+                using System.Collections;
 
-interface I : IList
-{
-    int $$
-}
-";
+                interface I : IList
+                {
+                    int $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IEnumerable");
@@ -304,19 +304,20 @@ interface I : IList
             await VerifyItemExistsAsync(markup, "IList");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Completion)]
+        [Fact]
         public async Task TestImplementedAsAsync()
         {
-            var markup = @"
-interface IGoo
-{
-    Task Goo();
-}
+            var markup = """
+                interface IGoo
+                {
+                    Task Goo();
+                }
 
-class MyGoo : IGoo
-{
-     async Task $$
-}";
+                class MyGoo : IGoo
+                {
+                     async Task $$
+                }
+                """;
 
             await VerifyAnyItemExistsAsync(markup, hasSuggestionModeItem: true);
             await VerifyItemExistsAsync(markup, "IGoo");
