@@ -18,6 +18,7 @@ using Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces;
 using Microsoft.CodeAnalysis.Options;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.Shared.Extensions;
+using Microsoft.CodeAnalysis.Simplification;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Microsoft.CodeAnalysis.Text;
 using Microsoft.CodeAnalysis.UnitTests.Diagnostics;
@@ -49,6 +50,9 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Diagnostics.UserDiagnos
             // https://github.com/dotnet/roslyn/issues/44682 - Add to all in one
             missingSyntaxNodes.Add(SyntaxKind.WithExpression);
             missingSyntaxNodes.Add(SyntaxKind.RecordDeclaration);
+            missingSyntaxNodes.Add(SyntaxKind.CollectionExpression);
+            missingSyntaxNodes.Add(SyntaxKind.ExpressionElement);
+            missingSyntaxNodes.Add(SyntaxKind.SpreadElement);
 
             var analyzer = new CSharpTrackingDiagnosticAnalyzer();
             using var workspace = TestWorkspace.CreateCSharp(source, TestOptions.Regular, composition: s_compositionWithMockDiagnosticUpdateSourceRegistrationService);
@@ -67,19 +71,19 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Diagnostics.UserDiagnos
             analyzer.VerifyOnCodeBlockCalledForAllSymbolAndMethodKinds(symbolKindsWithNoCodeBlocks, true);
         }
 
-        [Fact, WorkItem(908658, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/908658")]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/908658")]
         public async Task DiagnosticAnalyzerDriverVsAnalyzerDriverOnCodeBlock()
         {
             var methodNames = new string[] { "Initialize", "AnalyzeCodeBlock" };
-            var source = @"
-[System.Obsolete]
-class C
-{
-    int P { get; set; }
-    delegate void A();
-    delegate string F();
-}
-";
+            var source = """
+                [System.Obsolete]
+                class C
+                {
+                    int P { get; set; }
+                    delegate void A();
+                    delegate string F();
+                }
+                """;
 
             var ideEngineAnalyzer = new CSharpTrackingDiagnosticAnalyzer();
             using (var ideEngineWorkspace = TestWorkspace.CreateCSharp(source, composition: s_compositionWithMockDiagnosticUpdateSourceRegistrationService))
@@ -111,8 +115,7 @@ class C
             }
         }
 
-        [Fact]
-        [WorkItem(759, "https://github.com/dotnet/roslyn/issues/759")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/759")]
         public async Task DiagnosticAnalyzerDriverIsSafeAgainstAnalyzerExceptions()
         {
             var source = TestResource.AllInOneCSharpCode;
@@ -129,8 +132,7 @@ class C
             });
         }
 
-        [WorkItem(908621, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/908621")]
-        [Fact]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/908621")]
         public void DiagnosticServiceIsSafeAgainstAnalyzerExceptions_1()
         {
             var analyzer = new ThrowingDiagnosticAnalyzer<SyntaxKind>();
@@ -138,8 +140,7 @@ class C
             AccessSupportedDiagnostics(analyzer);
         }
 
-        [WorkItem(908621, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/908621")]
-        [Fact]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/908621")]
         public void DiagnosticServiceIsSafeAgainstAnalyzerExceptions_2()
         {
             var analyzer = new ThrowingDoNotCatchDiagnosticAnalyzer<SyntaxKind>();
@@ -170,7 +171,7 @@ class C
 
             workspace.TryApplyChanges(workspace.CurrentSolution
                 .WithAnalyzerReferences(new[] { analyzerReference })
-                .AddAdditionalDocument(additionalDocId, "add.config", additionalText.GetText()));
+                .AddAdditionalDocument(additionalDocId, "add.config", additionalText.GetText()!));
 
             var sourceDocument = workspace.CurrentSolution.Projects.Single().Documents.Single();
             await DiagnosticProviderTestUtilities.GetAllDiagnosticsAsync(workspace, sourceDocument, new TextSpan(0, sourceDocument.GetTextAsync().Result.Length));
@@ -185,9 +186,9 @@ class C
 
         private class ThrowingDoNotCatchDiagnosticAnalyzer<TLanguageKindEnum> : ThrowingDiagnosticAnalyzer<TLanguageKindEnum>, IBuiltInAnalyzer where TLanguageKindEnum : struct
         {
-            public CodeActionRequestPriority RequestPriority => CodeActionRequestPriority.Normal;
+            public bool IsHighPriority => false;
 
-            public bool OpenFileOnly(OptionSet options) => false;
+            public bool OpenFileOnly(SimplifierOptions? options) => false;
 
             public DiagnosticAnalyzerCategory GetAnalyzerCategory()
                 => DiagnosticAnalyzerCategory.SyntaxTreeWithoutSemanticsAnalysis | DiagnosticAnalyzerCategory.SemanticDocumentAnalysis | DiagnosticAnalyzerCategory.ProjectAnalysis;
@@ -243,16 +244,16 @@ class C
         [Fact]
         public async Task CodeBlockAnalyzersOnlyAnalyzeExecutableCode()
         {
-            var source = @"
-using System;
-class C
-{
-    void F(int x = 0)
-    {
-        Console.WriteLine(0);
-    }
-}
-";
+            var source = """
+                using System;
+                class C
+                {
+                    void F(int x = 0)
+                    {
+                        Console.WriteLine(0);
+                    }
+                }
+                """;
 
             var analyzer = new CodeBlockAnalyzerFactory();
             using (var ideEngineWorkspace = TestWorkspace.CreateCSharp(source, composition: s_compositionWithMockDiagnosticUpdateSourceRegistrationService))
@@ -266,16 +267,16 @@ class C
                 Assert.Equal(2, diagnosticsFromAnalyzer.Count());
             }
 
-            source = @"
-using System;
-class C
-{
-    void F(int x = 0, int y = 1, int z = 2)
-    {
-        Console.WriteLine(0);
-    }
-}
-";
+            source = """
+                using System;
+                class C
+                {
+                    void F(int x = 0, int y = 1, int z = 2)
+                    {
+                        Console.WriteLine(0);
+                    }
+                }
+                """;
 
             using (var compilerEngineWorkspace = TestWorkspace.CreateCSharp(source, composition: s_compositionWithMockDiagnosticUpdateSourceRegistrationService))
             {
@@ -341,7 +342,7 @@ class C
             var compilerEngineCompilation = (CSharpCompilation)(await compilerEngineWorkspace.CurrentSolution.Projects.Single().GetRequiredCompilationAsync(CancellationToken.None));
 
             var diagnostics = compilerEngineCompilation.GetAnalyzerDiagnostics(new[] { analyzer });
-            AssertEx.Any(diagnostics, d => d.Id == AnalyzerHelper.AnalyzerExceptionDiagnosticId);
+            AssertEx.Any(diagnostics, d => d.Id == DocumentAnalysisExecutor.AnalyzerExceptionDiagnosticId);
         }
 
         private class InvalidSpanAnalyzer : DiagnosticAnalyzer
@@ -358,7 +359,7 @@ class C
                 => context.ReportDiagnostic(Diagnostic.Create(Descriptor, Location.Create(context.Tree, TextSpan.FromBounds(1000, 2000))));
         }
 
-        [Fact, WorkItem(18818, "https://github.com/dotnet/roslyn/issues/18818")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/18818")]
         public async Task TestNuGetAndVsixAnalyzer_ReportsSameId()
         {
             // NuGet and VSIX analyzer reporting same diagnostic IDs.
@@ -417,7 +418,7 @@ class C
                 });
         }
 
-        [Fact, WorkItem(18818, "https://github.com/dotnet/roslyn/issues/18818")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/18818")]
         public async Task TestNuGetAndVsixAnalyzer_NuGetAnalyzerReportsSubsetOfVsixAnalyzerIds()
         {
             // NuGet analyzer reports subset of diagnostic IDs reported by VSIX analyzer.
@@ -469,7 +470,7 @@ class C
                 });
         }
 
-        [Fact, WorkItem(18818, "https://github.com/dotnet/roslyn/issues/18818")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/18818")]
         public async Task TestNuGetAndVsixAnalyzer_VsixAnalyzerReportsSubsetOfNuGetAnalyzerIds()
         {
             // VSIX analyzer reports subset of diagnostic IDs reported by NuGet analyzer.
@@ -520,7 +521,7 @@ class C
                 });
         }
 
-        [Fact, WorkItem(18818, "https://github.com/dotnet/roslyn/issues/18818")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/18818")]
         public async Task TestNuGetAndVsixAnalyzer_MultipleNuGetAnalyzersCollectivelyReportSameIds()
         {
             // Multiple NuGet analyzers collectively report same diagnostic IDs reported by Vsix analyzer.
@@ -593,7 +594,7 @@ class C
                 });
         }
 
-        [Fact, WorkItem(46942, "https://github.com/dotnet/roslyn/issues/46942")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/46942")]
         public async Task TestNuGetAndVsixAnalyzer_SuppressorSuppressesVsixAnalyzer()
         {
             // Multiple NuGet analyzers do not overlap with the VSIX analyzer or suppressor
