@@ -12,6 +12,7 @@ using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CodeActions;
 using Microsoft.CodeAnalysis.CodeFixes;
+using Microsoft.CodeAnalysis.CodeStyle;
 using Microsoft.CodeAnalysis.CSharp;
 using Microsoft.CodeAnalysis.CSharp.CodeFixes.Suppression;
 using Microsoft.CodeAnalysis.CSharp.Diagnostics.SimplifyTypeNames;
@@ -21,7 +22,6 @@ using Microsoft.CodeAnalysis.Diagnostics.CSharp;
 using Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics;
 using Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces;
 using Microsoft.CodeAnalysis.ErrorLogger;
-using Microsoft.CodeAnalysis.Formatting;
 using Microsoft.CodeAnalysis.Host.Mef;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Roslyn.Test.Utilities;
@@ -45,12 +45,13 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Diagnostics.Suppression
                 get { return 0; }
             }
 
+            [Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
             public class CompilerDiagnosticSuppressionTests : CSharpPragmaWarningDisableSuppressionTests
             {
                 internal override Tuple<DiagnosticAnalyzer, IConfigurationFixProvider> CreateDiagnosticProviderAndFixer(Workspace workspace)
                     => Tuple.Create<DiagnosticAnalyzer, IConfigurationFixProvider>(null, new CSharpSuppressionCodeFixProvider());
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestPragmaWarningDirective()
                 {
                     await TestAsync(
@@ -74,8 +75,7 @@ class Class
 }}");
                 }
 
-                [WorkItem(26015, "https://github.com/dotnet/roslyn/issues/26015")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/26015")]
                 public async Task TestPragmaWarningDirectiveAroundMultiLineStatement()
                 {
                     await TestAsync(
@@ -101,7 +101,45 @@ line"";
 }}");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/56165")]
+                public async Task TestPragmaWarningDirectiveAroundMultiLineInterpolatedString()
+                {
+                    await TestAsync(
+            @"
+using System;
+
+[Obsolete]
+class Session { }
+
+class Class
+{
+    void Method()
+    {
+        var s = $@""
+hi {[|new Session()|]}
+"";
+    }
+}",
+            $@"
+using System;
+
+[Obsolete]
+class Session {{ }}
+
+class Class
+{{
+    void Method()
+    {{
+#pragma warning disable CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+        var s = $@""
+hi {{new Session()}}
+"";
+#pragma warning restore CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+    }}
+}}");
+                }
+
+                [Fact]
                 public async Task TestMultilineStatementPragmaWarningDirective()
                 {
                     await TestAsync(
@@ -121,13 +159,39 @@ class Class
     {{
 #pragma warning disable CS0219 // {CSharpResources.WRN_UnreferencedVarAssg_Title}
         int x = 0
-#pragma warning restore CS0219 // {CSharpResources.WRN_UnreferencedVarAssg_Title}
               + 1;
+#pragma warning restore CS0219 // {CSharpResources.WRN_UnreferencedVarAssg_Title}
     }}
 }}");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
+                public async Task TestMultilineStatementPragmaWarningDirective2()
+                {
+                    await TestAsync(
+        @"
+class Class
+{
+    void Method()
+    {
+        [|int x = 0,
+            y = 1;|]
+    }
+}",
+        $@"
+class Class
+{{
+    void Method()
+    {{
+#pragma warning disable CS0219 // {CSharpResources.WRN_UnreferencedVarAssg_Title}
+        int x = 0,
+            y = 1;
+#pragma warning restore CS0219 // {CSharpResources.WRN_UnreferencedVarAssg_Title}
+    }}
+}}");
+                }
+
+                [Fact]
                 public async Task TestPragmaWarningDirectiveWithExistingTrivia()
                 {
                     await TestAsync(
@@ -156,8 +220,7 @@ class Class
 }}");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(16681, "https://github.com/dotnet/roslyn/issues/16681")]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/16681")]
                 public async Task TestPragmaWarningDirectiveWithDocumentationComment1()
                 {
                     await TestAsync(
@@ -181,8 +244,139 @@ sealed class Class
 }}");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(16681, "https://github.com/dotnet/roslyn/issues/16681")]
+                [Fact]
+                public async Task TestPragmaWarningExpressionBodiedMember1()
+                {
+                    await TestAsync(
+        @"
+sealed class Class
+{
+    [|protected int Method()|] => 1;
+}",
+        $@"
+sealed class Class
+{{
+#pragma warning disable CS0628 // {CSharpResources.WRN_ProtectedInSealed_Title}
+    protected int Method() => 1;
+#pragma warning restore CS0628 // {CSharpResources.WRN_ProtectedInSealed_Title}
+}}");
+                }
+
+                [Fact]
+                public async Task TestPragmaWarningExpressionBodiedMember2()
+                {
+                    await TestAsync(
+            @"
+using System;
+
+[Obsolete]
+class Session { }
+
+class Class
+{
+    string Method()
+        => @$""hi
+        {[|new Session()|]}
+        "";
+}",
+            $@"
+using System;
+
+[Obsolete]
+class Session {{ }}
+
+class Class
+{{
+    string Method()
+#pragma warning disable CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+        => @$""hi
+        {{new Session()}}
+        "";
+#pragma warning restore CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+}}");
+                }
+
+                [Fact]
+                public async Task TestPragmaWarningExpressionBodiedLocalFunction()
+                {
+                    await TestAsync(
+            @"
+using System;
+
+[Obsolete]
+class Session { }
+
+class Class
+{
+    void M()
+    {
+        string Method()
+            => @$""hi
+            {[|new Session()|]}
+            "";
+    }
+}",
+            $@"
+using System;
+
+[Obsolete]
+class Session {{ }}
+
+class Class
+{{
+    void M()
+    {{
+#pragma warning disable CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+        string Method()
+            => @$""hi
+            {{new Session()}}
+            "";
+#pragma warning restore CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+    }}
+}}");
+                }
+
+                [Fact]
+                public async Task TestPragmaWarningExpressionBodiedLambda()
+                {
+                    await TestAsync(
+            @"
+using System;
+
+[Obsolete]
+class Session { }
+
+class Class
+{
+    void M()
+    {
+        new Func<string>(()
+            => @$""hi
+            {[|new Session()|]}
+            "");
+    }
+}",
+            $@"
+using System;
+
+[Obsolete]
+class Session {{ }}
+
+class Class
+{{
+    void M()
+    {{
+#pragma warning disable CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+        new Func<string>(()
+            => @$""hi
+            {{new Session()}}
+            "");
+#pragma warning restore CS0612 // {CSharpResources.WRN_DeprecatedSymbol_Title}
+    }}
+}}");
+                }
+
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/16681")]
                 public async Task TestPragmaWarningDirectiveWithDocumentationComment2()
                 {
                     await TestAsync(
@@ -213,7 +407,7 @@ sealed class Class
 }}", new CSharpParseOptions(documentationMode: DocumentationMode.Diagnose));
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestMultipleInstancesOfPragmaWarningDirective()
                 {
                     await TestAsync(
@@ -237,8 +431,7 @@ class Class
 }}");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(3311, "https://github.com/dotnet/roslyn/issues/3311")]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/3311")]
                 public async Task TestNoDuplicateSuppressionCodeFixes()
                 {
                     var source = @"
@@ -267,10 +460,10 @@ class Class
                         SpecializedCollections.EmptyEnumerable<Lazy<CodeFixProvider, CodeChangeProviderMetadata>>(),
                         SpecializedCollections.SingletonEnumerable(suppressionProviderFactory));
                     var document = GetDocumentAndSelectSpan(workspace, out var span);
-                    var diagnostics = await diagnosticService.GetDiagnosticsForSpanAsync(document, span);
+                    var diagnostics = await diagnosticService.GetDiagnosticsForSpanAsync(document, span, CancellationToken.None);
                     Assert.Equal(2, diagnostics.Where(d => d.Id == "CS0219").Count());
 
-                    var allFixes = (await fixService.GetFixesAsync(document, span, includeConfigurationFixes: true, cancellationToken: CancellationToken.None))
+                    var allFixes = (await fixService.GetFixesAsync(document, span, CodeActionOptions.DefaultProvider, CancellationToken.None))
                         .SelectMany(fixCollection => fixCollection.Fixes);
 
                     var cs0219Fixes = allFixes.Where(fix => fix.PrimaryDiagnostic.Id == "CS0219").ToArray();
@@ -289,7 +482,7 @@ class Class
                     Assert.NotEqual(cs0219EquivalenceKey, cs0168EquivalenceKey);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestErrorAndWarningScenario()
                 {
                     await TestAsync(
@@ -315,8 +508,7 @@ class Class
 }}");
                 }
 
-                [WorkItem(956453, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/956453")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/956453")]
                 public async Task TestWholeFilePragmaWarningDirective()
                 {
                     await TestAsync(
@@ -326,8 +518,7 @@ class Class {{ void Method() {{ int x = 0; }} }}
 #pragma warning restore CS0219 // {CSharpResources.WRN_UnreferencedVarAssg_Title}");
                 }
 
-                [WorkItem(970129, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/970129")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/970129")]
                 public async Task TestSuppressionAroundSingleToken()
                 {
                     await TestAsync(
@@ -357,8 +548,7 @@ class Program
 }}");
                 }
 
-                [WorkItem(1066576, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
                 public async Task TestPragmaWarningDirectiveAroundTrivia1()
                 {
                     await TestAsync(
@@ -381,11 +571,11 @@ class Class
 class Class
 {{
     void Method()
+#pragma warning disable CS1633 // {CSharpResources.WRN_IllegalPragma_Title}
     {{
 
-        // Comment
-        // Comment
-#pragma warning disable CS1633 // {CSharpResources.WRN_IllegalPragma_Title}
+// Comment
+// Comment
 #pragma abcde
 
     }}    // Comment   
@@ -396,8 +586,7 @@ class Class
 }}");
                 }
 
-                [WorkItem(1066576, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
                 public async Task TestPragmaWarningDirectiveAroundTrivia2()
                 {
                     await TestAsync(
@@ -407,8 +596,7 @@ class Class
 #pragma warning restore CS1633 // {CSharpResources.WRN_IllegalPragma_Title}");
                 }
 
-                [WorkItem(1066576, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
                 public async Task TestPragmaWarningDirectiveAroundTrivia3()
                 {
                     await TestAsync(
@@ -418,8 +606,7 @@ class Class
 #pragma warning restore CS1633 // {CSharpResources.WRN_IllegalPragma_Title}");
                 }
 
-                [WorkItem(1066576, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
                 public async Task TestPragmaWarningDirectiveAroundTrivia4()
                 {
                     await TestAsync(
@@ -439,8 +626,7 @@ class C {{ }}
 ");
                 }
 
-                [WorkItem(1066576, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
                 public async Task TestPragmaWarningDirectiveAroundTrivia5()
                 {
                     await TestAsync(
@@ -456,8 +642,7 @@ class C2 {{ }}
 class C3 {{ }}");
                 }
 
-                [WorkItem(1066576, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1066576")]
                 public async Task TestPragmaWarningDirectiveAroundTrivia6()
                 {
                     await TestAsync(
@@ -568,23 +753,23 @@ class Class
                 internal override Tuple<DiagnosticAnalyzer, IConfigurationFixProvider> CreateDiagnosticProviderAndFixer(Workspace workspace)
                 {
                     return new Tuple<DiagnosticAnalyzer, IConfigurationFixProvider>(
-                        new FormattingDiagnosticAnalyzer(), new CSharpSuppressionCodeFixProvider());
+                        new CSharpFormattingAnalyzer(), new CSharpSuppressionCodeFixProvider());
                 }
 
-                protected override Task<(ImmutableArray<CodeAction>, CodeAction actionToInvoke)> GetCodeActionsAsync(TestWorkspace workspace, TestParameters parameters)
+                protected override async Task<(ImmutableArray<CodeAction>, CodeAction actionToInvoke)> GetCodeActionsAsync(TestWorkspace workspace, TestParameters parameters)
                 {
                     var solution = workspace.CurrentSolution;
                     var compilationOptions = solution.Projects.Single().CompilationOptions;
                     var specificDiagnosticOptions = new[] { KeyValuePairUtil.Create(IDEDiagnosticIds.FormattingDiagnosticId, ReportDiagnostic.Warn) };
                     compilationOptions = compilationOptions.WithSpecificDiagnosticOptions(specificDiagnosticOptions);
                     var updatedSolution = solution.WithProjectCompilationOptions(solution.ProjectIds.Single(), compilationOptions);
-                    workspace.ChangeSolution(updatedSolution);
+                    await workspace.ChangeSolutionAsync(updatedSolution);
 
-                    return base.GetCodeActionsAsync(workspace, parameters);
+                    return await base.GetCodeActionsAsync(workspace, parameters);
                 }
 
                 [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(38587, "https://github.com/dotnet/roslyn/issues/38587")]
+                [WorkItem("https://github.com/dotnet/roslyn/issues/38587")]
                 public async Task TestFormattingDiagnosticSuppressed()
                 {
                     await TestAsync(
@@ -757,8 +942,8 @@ using System;
                     new UserDiagnosticAnalyzer(), new CSharpSuppressionCodeFixProvider());
             }
 
-            [WorkItem(2764, "https://github.com/dotnet/roslyn/issues/2764")]
             [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+            [WorkItem("https://github.com/dotnet/roslyn/issues/2764")]
             public async Task TestPragmaWarningDirectiveAroundMultilineDiagnostic()
             {
                 await TestAsync(
@@ -782,10 +967,7 @@ class Class
 
         public abstract partial class CSharpGlobalSuppressMessageSuppressionTests : CSharpSuppressionTests
         {
-            protected sealed override int CodeActionIndex
-            {
-                get { return 1; }
-            }
+            protected sealed override int CodeActionIndex => 1;
 
             public class CompilerDiagnosticSuppressionTests : CSharpGlobalSuppressMessageSuppressionTests
             {
@@ -813,23 +995,23 @@ class Class
                 internal override Tuple<DiagnosticAnalyzer, IConfigurationFixProvider> CreateDiagnosticProviderAndFixer(Workspace workspace)
                 {
                     return Tuple.Create<DiagnosticAnalyzer, IConfigurationFixProvider>(
-                        new FormattingDiagnosticAnalyzer(), new CSharpSuppressionCodeFixProvider());
+                        new CSharpFormattingAnalyzer(), new CSharpSuppressionCodeFixProvider());
                 }
 
-                protected override Task<(ImmutableArray<CodeAction>, CodeAction actionToInvoke)> GetCodeActionsAsync(TestWorkspace workspace, TestParameters parameters)
+                protected override async Task<(ImmutableArray<CodeAction>, CodeAction actionToInvoke)> GetCodeActionsAsync(TestWorkspace workspace, TestParameters parameters)
                 {
                     var solution = workspace.CurrentSolution;
                     var compilationOptions = solution.Projects.Single().CompilationOptions;
                     var specificDiagnosticOptions = new[] { KeyValuePairUtil.Create(IDEDiagnosticIds.FormattingDiagnosticId, ReportDiagnostic.Warn) };
                     compilationOptions = compilationOptions.WithSpecificDiagnosticOptions(specificDiagnosticOptions);
                     var updatedSolution = solution.WithProjectCompilationOptions(solution.ProjectIds.Single(), compilationOptions);
-                    workspace.ChangeSolution(updatedSolution);
+                    await workspace.ChangeSolutionAsync(updatedSolution);
 
-                    return base.GetCodeActionsAsync(workspace, parameters);
+                    return await base.GetCodeActionsAsync(workspace, parameters);
                 }
 
                 [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(38587, "https://github.com/dotnet/roslyn/issues/38587")]
+                [WorkItem("https://github.com/dotnet/roslyn/issues/38587")]
                 public async Task TestCompilerDiagnosticsCannotBeSuppressed()
                 {
                     // Another test verifies we have a pragma warning action for this source, this verifies there are no other suppression actions.
@@ -869,12 +1051,13 @@ class Class
                 }
             }
 
+            [Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
             public partial class UserInfoDiagnosticSuppressionTests : CSharpGlobalSuppressMessageSuppressionTests
             {
                 private class UserDiagnosticAnalyzer : DiagnosticAnalyzer
                 {
                     public static readonly DiagnosticDescriptor Descriptor =
-                        new DiagnosticDescriptor("InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", DiagnosticSeverity.Info, isEnabledByDefault: true);
+                        new("InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", DiagnosticSeverity.Info, isEnabledByDefault: true);
 
                     public override ImmutableArray<DiagnosticDescriptor> SupportedDiagnostics
                     {
@@ -939,8 +1122,7 @@ class Class
                         new UserDiagnosticAnalyzer(), new CSharpSuppressionCodeFixProvider());
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(37529, "https://github.com/dotnet/roslyn/issues/37529")]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/37529")]
                 public async Task GeneratedCodeShouldNotHaveTrailingWhitespace()
                 {
                     var expected =
@@ -972,8 +1154,7 @@ using System;
                 private static bool HasTrailingWhitespace(string line)
                     => line.LastOrNull() is char last && char.IsWhiteSpace(last);
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(37529, "https://github.com/dotnet/roslyn/issues/37529")]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/37529")]
                 public async Task GeneratedCodeShouldNotHaveLeadingBlankLines()
                 {
                     var expected =
@@ -1003,8 +1184,7 @@ using System;
 }", expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(37529, "https://github.com/dotnet/roslyn/issues/37529")]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/37529")]
                 public async Task GeneratedCodeShouldNotHaveMoreThanOneTrailingBlankLine()
                 {
                     var expected =
@@ -1019,7 +1199,7 @@ using System.Diagnostics.CodeAnalysis;
 ";
 
                     var lines = Regex.Split(expected, "\r?\n");
-                    Assert.False(string.IsNullOrWhiteSpace(lines[lines.Length - 2]));
+                    Assert.False(string.IsNullOrWhiteSpace(lines[^2]));
 
                     await TestAsync(
             @"
@@ -1034,7 +1214,7 @@ using System;
 }", expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnSimpleType()
                 {
                     await TestAsync(
@@ -1076,7 +1256,7 @@ using System.Diagnostics.CodeAnalysis;
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnNamespace()
                 {
                     await TestInRegularAndScriptAsync(
@@ -1124,7 +1304,7 @@ using System.Diagnostics.CodeAnalysis;
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnTypeInsideNamespace()
                 {
                     await TestAsync(
@@ -1178,7 +1358,7 @@ namespace N1
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnNestedType()
                 {
                     await TestAsync(
@@ -1232,7 +1412,7 @@ namespace N
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnMethod()
                 {
                     await TestAsync(
@@ -1286,7 +1466,7 @@ namespace N
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnOverloadedMethod()
                 {
                     await TestAsync(
@@ -1386,7 +1566,7 @@ using System.Diagnostics.CodeAnalysis;
 ");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnGenericMethod()
                 {
                     await TestAsync(
@@ -1440,7 +1620,7 @@ namespace N
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnProperty()
                 {
                     await TestAsync(
@@ -1494,7 +1674,7 @@ namespace N
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnField()
                 {
                     await TestAsync(
@@ -1530,8 +1710,7 @@ class Class
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-                [WorkItem(6379, "https://github.com/dotnet/roslyn/issues/6379")]
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/6379")]
                 public async Task TestSuppressionOnTriviaBetweenFields()
                 {
                     await TestAsync(
@@ -1581,7 +1760,7 @@ enum E
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnField2()
                 {
                     await TestAsync(
@@ -1617,7 +1796,7 @@ class Class
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnEvent()
                 {
                     await TestAsync(
@@ -1681,7 +1860,7 @@ class Class
 }");
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionWithExistingGlobalSuppressionsDocument()
                 {
                     var initialMarkup = @"<Workspace>
@@ -1721,7 +1900,7 @@ using System.Diagnostics.CodeAnalysis;
                     await TestAsync(initialMarkup, expectedText);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionWithExistingGlobalSuppressionsDocument2()
                 {
                     // Own custom file named GlobalSuppressions.cs
@@ -1757,7 +1936,7 @@ using System.Diagnostics.CodeAnalysis;
                     await TestAsync(initialMarkup, expectedText);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionWithExistingGlobalSuppressionsDocument3()
                 {
                     // Own custom file named GlobalSuppressions.cs + existing GlobalSuppressions2.cs with global suppressions
@@ -1804,7 +1983,7 @@ using System.Diagnostics.CodeAnalysis;
                     await TestAsync(initialMarkup, expectedText);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionWithUsingDirectiveInExistingGlobalSuppressionsDocument()
                 {
                     var initialMarkup = @"<Workspace>
@@ -1836,7 +2015,7 @@ using System.Diagnostics.CodeAnalysis;
                     await TestAsync(initialMarkup, expectedText);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionWithoutUsingDirectiveInExistingGlobalSuppressionsDocument()
                 {
                     var initialMarkup = @"<Workspace>
@@ -1870,25 +2049,17 @@ using System.Diagnostics.CodeAnalysis;
 
         public abstract class CSharpLocalSuppressMessageSuppressionTests : CSharpSuppressionTests
         {
-            protected sealed override int CodeActionIndex
-            {
-                get { return 2; }
-            }
+            protected sealed override int CodeActionIndex => 2;
 
+            [Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
             public class UserInfoDiagnosticSuppressionTests : CSharpLocalSuppressMessageSuppressionTests
             {
                 private class UserDiagnosticAnalyzer : DiagnosticAnalyzer
                 {
                     private readonly DiagnosticDescriptor _descriptor =
-                        new DiagnosticDescriptor("InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", DiagnosticSeverity.Info, isEnabledByDefault: true);
+                        new("InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", "InfoDiagnostic", DiagnosticSeverity.Info, isEnabledByDefault: true);
 
-                    public override ImmutableArray<DiagnosticDescriptor> SupportedDiagnostics
-                    {
-                        get
-                        {
-                            return ImmutableArray.Create(_descriptor);
-                        }
-                    }
+                    public override ImmutableArray<DiagnosticDescriptor> SupportedDiagnostics => ImmutableArray.Create(_descriptor);
 
                     public override void Initialize(AnalysisContext context)
                         => context.RegisterSyntaxNodeAction(AnalyzeNode, SyntaxKind.ClassDeclaration, SyntaxKind.NamespaceDeclaration, SyntaxKind.MethodDeclaration);
@@ -1921,7 +2092,7 @@ using System.Diagnostics.CodeAnalysis;
                         new UserDiagnosticAnalyzer(), new CSharpSuppressionCodeFixProvider());
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnSimpleType()
                 {
                     var initial = @"
@@ -1955,7 +2126,7 @@ class Class
                     await TestMissingAsync(expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnSimpleType2()
                 {
                     // Type already has attributes.
@@ -1993,7 +2164,7 @@ class Class
                     await TestMissingAsync(expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnSimpleType3()
                 {
                     // Type already has attributes with trailing trivia.
@@ -2033,7 +2204,7 @@ class Class
                     await TestMissingAsync(expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnTypeInsideNamespace()
                 {
                     var initial = @"
@@ -2076,7 +2247,7 @@ namespace N1
                     await TestMissingAsync(expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnNestedType()
                 {
                     var initial = @"
@@ -2119,7 +2290,7 @@ namespace N
                     await TestMissingAsync(expected);
                 }
 
-                [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
+                [Fact]
                 public async Task TestSuppressionOnMethod()
                 {
                     var initial = @"
@@ -2161,6 +2332,114 @@ namespace N
                     expected = expected.Replace("int Method()", "[|int Method()|]");
                     await TestMissingAsync(expected);
                 }
+
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/47427")]
+                public async Task TestSuppressionOnMethodWithXmlDoc()
+                {
+                    var initial = @"
+using System;
+
+namespace ClassLibrary10
+{
+    public class Class1
+    {
+        int x;
+
+        /// <summary>
+        /// This is a description
+        /// </summary>
+        [|public void Method(int unused)|] { }
+    }
+}";
+                    var expected = $@"
+using System;
+
+namespace ClassLibrary10
+{{
+    public class Class1
+    {{
+        int x;
+
+        /// <summary>
+        /// This is a description
+        /// </summary>
+        [System.Diagnostics.CodeAnalysis.SuppressMessage(""InfoDiagnostic"", ""InfoDiagnostic:InfoDiagnostic"", Justification = ""{FeaturesResources.Pending}"")]
+        public void Method(int unused) {{ }}
+    }}
+}}";
+                    await TestAsync(initial, expected);
+
+                    // Also verify that the added attribute does indeed suppress the diagnostic.
+                    expected = expected.Replace("public void Method(int unused)", "[|public void Method(int unused)|]");
+                    await TestMissingAsync(expected);
+                }
+
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/47427")]
+                public async Task TestSuppressionOnMethodWithNoTrivia()
+                {
+                    var initial = @"
+using System;
+
+namespace ClassLibrary10
+{
+    public class Class1
+    {
+        int x;
+[|public void Method(int unused)|] { }
+    }
+}";
+                    var expected = $@"
+using System;
+
+namespace ClassLibrary10
+{{
+    public class Class1
+    {{
+        int x;
+        [System.Diagnostics.CodeAnalysis.SuppressMessage(""InfoDiagnostic"", ""InfoDiagnostic:InfoDiagnostic"", Justification = ""{FeaturesResources.Pending}"")]
+        public void Method(int unused) {{ }}
+    }}
+}}";
+                    await TestAsync(initial, expected);
+
+                    // Also verify that the added attribute does indeed suppress the diagnostic.
+                    expected = expected.Replace("public void Method(int unused)", "[|public void Method(int unused)|]");
+                    await TestMissingAsync(expected);
+                }
+
+                [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/47427")]
+                public async Task TestSuppressionOnMethodWithTriviaStartsOnTheSameLine()
+                {
+                    var initial = @"
+using System;
+
+namespace ClassLibrary10
+{
+    public class Class1
+    {
+        int x;
+        /*test*/[|public void Method(int unused)|] { }
+    }
+}";
+                    var expected = $@"
+using System;
+
+namespace ClassLibrary10
+{{
+    public class Class1
+    {{
+        int x;
+        /*test*/
+        [System.Diagnostics.CodeAnalysis.SuppressMessage(""InfoDiagnostic"", ""InfoDiagnostic:InfoDiagnostic"", Justification = ""{FeaturesResources.Pending}"")]
+        public void Method(int unused) {{ }}
+    }}
+}}";
+                    await TestAsync(initial, expected);
+
+                    // Also verify that the added attribute does indeed suppress the diagnostic.
+                    expected = expected.Replace("public void Method(int unused)", "[|public void Method(int unused)|]");
+                    await TestMissingAsync(expected);
+                }
             }
         }
 
@@ -2173,15 +2452,10 @@ namespace N
             private class UserDiagnosticAnalyzer : DiagnosticAnalyzer
             {
                 public static readonly DiagnosticDescriptor Descriptor =
-                    new DiagnosticDescriptor("NoLocationDiagnostic", "NoLocationDiagnostic", "NoLocationDiagnostic", "NoLocationDiagnostic", DiagnosticSeverity.Info, isEnabledByDefault: true);
+                    new("NoLocationDiagnostic", "NoLocationDiagnostic", "NoLocationDiagnostic", "NoLocationDiagnostic", DiagnosticSeverity.Info, isEnabledByDefault: true);
 
                 public override ImmutableArray<DiagnosticDescriptor> SupportedDiagnostics
-                {
-                    get
-                    {
-                        return ImmutableArray.Create(Descriptor);
-                    }
-                }
+                    => ImmutableArray.Create(Descriptor);
 
                 public override void Initialize(AnalysisContext context)
                     => context.RegisterSyntaxNodeAction(AnalyzeNode, SyntaxKind.ClassDeclaration);
@@ -2196,16 +2470,10 @@ namespace N
                     new UserDiagnosticAnalyzer(), new CSharpSuppressionCodeFixProvider());
             }
 
-            protected override int CodeActionIndex
-            {
-                get
-                {
-                    return 0;
-                }
-            }
+            protected override int CodeActionIndex => 0;
 
             [Fact, Trait(Traits.Feature, Traits.Features.CodeActionsSuppression)]
-            [WorkItem(1073825, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1073825")]
+            [WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/1073825")]
             public async Task TestDiagnosticWithoutLocationCanBeSuppressed()
             {
                 await TestAsync(
