@@ -7,14 +7,17 @@
 using System;
 using System.Collections.Generic;
 using System.Collections.Immutable;
+using System.IO;
 using System.Linq;
+using System.Text;
 using System.Threading;
 using System.Threading.Tasks;
+using EnvDTE;
 using Microsoft.CodeAnalysis.CSharp.Syntax;
 using Microsoft.CodeAnalysis.CSharp.Test.Utilities;
 using Microsoft.CodeAnalysis.Differencing;
 using Microsoft.CodeAnalysis.EditAndContinue;
-using Microsoft.CodeAnalysis.EditAndContinue.Contracts;
+using Microsoft.CodeAnalysis.Contracts.EditAndContinue;
 using Microsoft.CodeAnalysis.EditAndContinue.UnitTests;
 using Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces;
 using Microsoft.CodeAnalysis.Test.Utilities;
@@ -32,6 +35,18 @@ namespace Microsoft.CodeAnalysis.CSharp.EditAndContinue.UnitTests
 
         #region Helpers
 
+        private static TestWorkspace CreateWorkspace()
+            => new(composition: s_composition);
+
+        private static Solution AddDefaultTestProject(Solution solution, string source)
+        {
+            var projectId = ProjectId.CreateNewId();
+
+            return solution.
+                AddProject(ProjectInfo.Create(projectId, VersionStamp.Create(), "proj", "proj", LanguageNames.CSharp)).GetProject(projectId).
+                AddDocument("test.cs", SourceText.From(source, Encoding.UTF8), filePath: Path.Combine(TempRoot.Root, "test.cs")).Project.Solution;
+        }
+
         private static void TestSpans(string source, Func<SyntaxNode, bool> hasLabel)
         {
             var tree = SyntaxFactory.ParseSyntaxTree(source);
@@ -192,7 +207,7 @@ namespace Microsoft.CodeAnalysis.CSharp.EditAndContinue.UnitTests
     
 }
 ";
-            TestSpans(source, node => SyntaxComparer.TopLevel.HasLabel(node));
+            TestSpans(source, SyntaxComparer.TopLevel.HasLabel);
         }
 
         [Fact]
@@ -248,7 +263,7 @@ class C
             // TODO: test
             // /*<span>*/F($$from a in b from c in d select a.x);/*</span>*/
             // /*<span>*/F(from a in b $$from c in d select a.x);/*</span>*/
-            TestSpans(source, kind => SyntaxComparer.Statement.HasLabel(kind));
+            TestSpans(source, SyntaxComparer.Statement.HasLabel);
         }
 
         /// <summary>
@@ -257,8 +272,8 @@ class C
         [Fact]
         public void ErrorSpansAllKinds()
         {
-            TestErrorSpansAllKinds(kind => SyntaxComparer.Statement.HasLabel(kind));
-            TestErrorSpansAllKinds(kind => SyntaxComparer.TopLevel.HasLabel(kind));
+            TestErrorSpansAllKinds(SyntaxComparer.Statement.HasLabel);
+            TestErrorSpansAllKinds(SyntaxComparer.TopLevel.HasLabel);
         }
 
         [Fact]
@@ -283,14 +298,14 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-            var oldSolution = workspace.CurrentSolution;
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var oldText = await oldDocument.GetTextAsync();
             var oldSyntaxRoot = await oldDocument.GetSyntaxRootAsync();
             var documentId = oldDocument.Id;
-            var newSolution = workspace.CurrentSolution.WithDocumentText(documentId, SourceText.From(source2));
+            var newSolution = oldSolution.WithDocumentText(documentId, SourceText.From(source2));
             var newDocument = newSolution.GetDocument(documentId);
             var newText = await newDocument.GetTextAsync();
             var newSyntaxRoot = await newDocument.GetSyntaxRootAsync();
@@ -350,12 +365,12 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-            var oldSolution = workspace.CurrentSolution;
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var documentId = oldDocument.Id;
-            var newSolution = workspace.CurrentSolution.WithDocumentText(documentId, SourceText.From(source2));
+            var newSolution = oldSolution.WithDocumentText(documentId, SourceText.From(source2));
 
             var result = await AnalyzeDocumentAsync(oldProject, newSolution.GetDocument(documentId));
 
@@ -377,8 +392,9 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source, composition: s_composition);
-            var oldProject = workspace.CurrentSolution.Projects.Single();
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source);
+            var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
 
             var result = await AnalyzeDocumentAsync(oldProject, oldDocument);
@@ -410,14 +426,13 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-
-            var oldSolution = workspace.CurrentSolution;
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var documentId = oldDocument.Id;
 
-            var newSolution = workspace.CurrentSolution.WithDocumentText(documentId, SourceText.From(source2));
+            var newSolution = oldSolution.WithDocumentText(documentId, SourceText.From(source2));
 
             var result = await AnalyzeDocumentAsync(oldProject, newSolution.GetDocument(documentId));
 
@@ -440,11 +455,15 @@ class C
 ";
             var experimentalFeatures = new Dictionary<string, string>(); // no experimental features to enable
             var experimental = TestOptions.Regular.WithFeatures(experimentalFeatures);
+            var root = SyntaxFactory.ParseCompilationUnit(source, options: experimental);
 
-            using var workspace = TestWorkspace.CreateCSharp(
-                source, parseOptions: experimental, compilationOptions: null, composition: s_composition);
+            using var workspace = CreateWorkspace();
+
+            var projectId = ProjectId.CreateNewId();
+            var oldSolution = workspace.CurrentSolution.
+                AddProject(ProjectInfo.Create(projectId, VersionStamp.Create(), "proj", "proj", LanguageNames.CSharp)).GetProject(projectId).
+                AddDocument("test.cs", root, filePath: Path.Combine(TempRoot.Root, "test.cs")).Project.Solution;
 
-            var oldSolution = workspace.CurrentSolution;
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var documentId = oldDocument.Id;
@@ -488,7 +507,7 @@ class C
                 var experimental = TestOptions.Regular.WithFeatures(featuresToEnable);
 
                 using var workspace = TestWorkspace.CreateCSharp(
-                    source1, parseOptions: experimental, compilationOptions: null, exportProvider: null);
+                    source1, parseOptions: experimental, compilationOptions: null);
 
                 var oldSolution = workspace.CurrentSolution;
                 var oldProject = oldSolution.Projects.Single();
@@ -520,9 +539,9 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source, composition: s_composition);
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source);
 
-            var oldSolution = workspace.CurrentSolution;
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var documentId = oldDocument.Id;
@@ -534,7 +553,7 @@ class C
             Assert.False(result.HasChangesAndSyntaxErrors);
         }
 
-        [Fact, WorkItem(10683, "https://github.com/dotnet/roslyn/issues/10683")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/10683")]
         public async Task AnalyzeDocumentAsync_SemanticErrorInMethodBody_Change()
         {
             var source1 = @"
@@ -558,14 +577,13 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-
-            var oldSolution = workspace.CurrentSolution;
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var documentId = oldDocument.Id;
 
-            var newSolution = workspace.CurrentSolution.WithDocumentText(documentId, SourceText.From(source2));
+            var newSolution = oldSolution.WithDocumentText(documentId, SourceText.From(source2));
 
             var result = await AnalyzeDocumentAsync(oldProject, newSolution.GetDocument(documentId));
 
@@ -576,7 +594,7 @@ class C
             Assert.False(result.HasChangesAndSyntaxErrors);
         }
 
-        [Fact, WorkItem(10683, "https://github.com/dotnet/roslyn/issues/10683")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/10683")]
         public async Task AnalyzeDocumentAsync_SemanticErrorInDeclaration_Change()
         {
             var source1 = @"
@@ -598,14 +616,13 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-
-            var oldSolution = workspace.CurrentSolution;
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
             var oldProject = oldSolution.Projects.Single();
             var oldDocument = oldProject.Documents.Single();
             var documentId = oldDocument.Id;
 
-            var newSolution = workspace.CurrentSolution.WithDocumentText(documentId, SourceText.From(source2));
+            var newSolution = oldSolution.WithDocumentText(documentId, SourceText.From(source2));
 
             var result = await AnalyzeDocumentAsync(oldProject, newSolution.GetDocument(documentId));
 
@@ -640,14 +657,13 @@ namespace N
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-            // fork the solution to introduce a change
-            var oldProject = workspace.CurrentSolution.Projects.Single();
-            var newDocId = DocumentId.CreateNewId(oldProject.Id);
-            var oldSolution = workspace.CurrentSolution;
-            var newSolution = oldSolution.AddDocument(newDocId, "goo.cs", SourceText.From(source2));
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
 
-            workspace.TryApplyChanges(newSolution);
+            // fork the solution to introduce a change
+            var oldProject = oldSolution.Projects.Single();
+            var newDocId = DocumentId.CreateNewId(oldProject.Id);
+            var newSolution = oldSolution.AddDocument(newDocId, "goo.cs", SourceText.From(source2), filePath: Path.Combine(TempRoot.Root, "goo.cs"));
 
             var newProject = newSolution.Projects.Single();
             var changes = newProject.GetChanges(oldProject);
@@ -665,8 +681,7 @@ namespace N
             }
 
             Assert.True(result.IsSingle());
-            Assert.Equal(1, result.Single().RudeEditErrors.Count());
-            Assert.Equal(RudeEditKind.Insert, result.Single().RudeEditErrors.Single().Kind);
+            Assert.Empty(result.Single().RudeEditErrors);
         }
 
         [Fact]
@@ -689,14 +704,12 @@ class D
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
 
-            var oldSolution = workspace.CurrentSolution;
             var oldProject = oldSolution.Projects.Single();
             var newDocId = DocumentId.CreateNewId(oldProject.Id);
-            var newSolution = oldSolution.AddDocument(newDocId, "goo.cs", SourceText.From(source2));
-
-            workspace.TryApplyChanges(newSolution);
+            var newSolution = oldSolution.AddDocument(newDocId, "goo.cs", SourceText.From(source2), filePath: Path.Combine(TempRoot.Root, "goo.cs"));
 
             var newProject = newSolution.Projects.Single();
             var changes = newProject.GetChanges(oldProject);
@@ -723,17 +736,17 @@ class D
             var source1 = @"class C {}";
             var source2 = @"class C { int x; }";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
-            var oldProject = workspace.CurrentSolution.Projects.Single();
+            var filePath = Path.Combine(TempRoot.Root, "src.cs");
+
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
+            var oldProject = oldSolution.Projects.Single();
             var documentId = DocumentId.CreateNewId(oldProject.Id);
-            var oldSolution = workspace.CurrentSolution;
-            var newSolution = oldSolution.AddDocument(documentId, "goo.cs", SourceText.From(source2), filePath: "src.cs");
+            var newSolution = oldSolution.AddDocument(documentId, "goo.cs", SourceText.From(source2), filePath: filePath);
             var newProject = newSolution.Projects.Single();
             var newDocument = newProject.GetDocument(documentId);
             var newSyntaxTree = await newDocument.GetSyntaxTreeAsync().ConfigureAwait(false);
 
-            workspace.TryApplyChanges(newSolution);
-
             var baseActiveStatements = AsyncLazy.Create(ActiveStatementsMap.Empty);
             var capabilities = AsyncLazy.Create(EditAndContinueTestHelpers.Net5RuntimeCapabilities);
 
@@ -747,11 +760,11 @@ class D
 
             var result = await analyzer.AnalyzeDocumentAsync(oldProject, baseActiveStatements, newDocument, ImmutableArray<LinePositionSpan>.Empty, capabilities, CancellationToken.None);
 
-            var expectedDiagnostic = outOfMemory ?
-                $"ENC0089: {string.Format(FeaturesResources.Modifying_source_file_0_requires_restarting_the_application_because_the_file_is_too_big, "src.cs")}" :
+            var expectedDiagnostic = outOfMemory
+                ? $"ENC0089: {string.Format(FeaturesResources.Modifying_source_file_0_requires_restarting_the_application_because_the_file_is_too_big, filePath)}"
                 // Because the error message that is formatted into this template string includes a stacktrace with newlines, we need to replicate that behavior
                 // here so that any trailing punctuation is removed from the translated template string.
-                $"ENC0080: {string.Format(FeaturesResources.Modifying_source_file_0_requires_restarting_the_application_due_to_internal_error_1, "src.cs", "System.NullReferenceException: NullRef!\n")}".Split('\n').First();
+                : $"ENC0080: {string.Format(FeaturesResources.Modifying_source_file_0_requires_restarting_the_application_due_to_internal_error_1, filePath, "System.NullReferenceException: NullRef!\n")}".Split('\n').First();
 
             AssertEx.Equal(new[] { expectedDiagnostic }, result.RudeEditErrors.Select(d => d.ToDiagnostic(newSyntaxTree))
                 .Select(d => $"{d.Id}: {d.GetMessage().Split(new[] { Environment.NewLine }, StringSplitOptions.None).First()}"));
@@ -779,12 +792,12 @@ class C
 }
 ";
 
-            using var workspace = TestWorkspace.CreateCSharp(source1, composition: s_composition);
+            using var workspace = CreateWorkspace();
+            var oldSolution = AddDefaultTestProject(workspace.CurrentSolution, source1);
 
-            var oldSolution = workspace.CurrentSolution;
             var oldProject = oldSolution.Projects.Single();
             var documentId = oldProject.Documents.Single().Id;
-            var newSolution = workspace.CurrentSolution.WithDocumentText(documentId, SourceText.From(source2));
+            var newSolution = oldSolution.WithDocumentText(documentId, SourceText.From(source2));
             var newDocument = newSolution.GetDocument(documentId);
 
             var result = await AnalyzeDocumentAsync(oldProject, newDocument, capabilities: EditAndContinueCapabilities.None);
