@@ -16,15 +16,14 @@ using Xunit.Abstractions;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
 {
+    [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
     public class SmartTokenFormatterFormatTokenTests : CSharpFormatterTestsBase
     {
         public SmartTokenFormatterFormatTokenTests(ITestOutputHelper output) : base(output)
         {
         }
 
-        [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
-        [WorkItem(44423, "https://github.com/dotnet/roslyn/issues/44423")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/44423")]
         public async Task EmptyFile1()
         {
             var code = @"{";
@@ -36,7 +35,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task EmptyFile2()
         {
             var code = @"}";
@@ -47,7 +45,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Namespace1()
         {
             var code = @"namespace NS
@@ -60,7 +57,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Namespace2()
         {
             var code = @"namespace NS
@@ -73,7 +69,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Namespace3()
         {
             var code = @"namespace NS
@@ -87,7 +82,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Class1()
         {
             var code = @"namespace NS
@@ -102,7 +96,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Class2()
         {
             var code = @"namespace NS
@@ -117,7 +110,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Class3()
         {
             var code = @"namespace NS
@@ -133,7 +125,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Method1()
         {
             var code = @"namespace NS
@@ -150,7 +141,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Method2()
         {
             var code = @"namespace NS
@@ -167,7 +157,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Method3()
         {
             var code = @"namespace NS
@@ -185,7 +174,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Property1()
         {
             var code = @"namespace NS
@@ -202,7 +190,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Property2()
         {
             var code = @"namespace NS
@@ -220,7 +207,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Event1()
         {
             var code = @"namespace NS
@@ -237,7 +223,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Event2()
         {
             var code = @"namespace NS
@@ -255,7 +240,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Indexer1()
         {
             var code = @"namespace NS
@@ -272,7 +256,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Indexer2()
         {
             var code = @"namespace NS
@@ -290,7 +273,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Block1()
         {
             var code = @"namespace NS
@@ -308,7 +290,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Block2()
         {
             var code = @"namespace NS
@@ -326,7 +307,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Block3()
         {
             var code = @"namespace NS
@@ -345,7 +325,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task Block4()
         {
             var code = @"namespace NS
@@ -364,7 +343,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
         }
 
         [Fact]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
         public async Task ArrayInitializer1()
         {
             var code = @"namespace NS
@@ -391,9 +369,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
                 indentationLine: 6);
         }
 
-        [Fact]
-        [WorkItem(537827, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/537827")]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/537827")]
         public async Task ArrayInitializer3()
         {
             var code = @"namespace NS
@@ -414,9 +390,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
                 expectedSpace: 12);
         }
 
-        [Fact]
-        [WorkItem(543142, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/543142")]
-        [Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/543142")]
         public async Task EnterWithTrailingWhitespace()
         {
             var code = @"class Class
@@ -433,8 +407,7 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Formatting.Indentation
                 expectedSpace: 8);
         }
 
-        [WorkItem(9216, "DevDiv_Projects/Roslyn")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem(9216, "DevDiv_Projects/Roslyn")]
         public async Task OpenBraceWithBaseIndentation()
         {
             var markup = @"
@@ -453,8 +426,7 @@ $${
             await AssertSmartTokenFormatterOpenBraceWithBaseIndentationAsync(markup, baseIndentation: 7, expectedIndentation: 11);
         }
 
-        [WorkItem(9216, "DevDiv_Projects/Roslyn")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem(9216, "DevDiv_Projects/Roslyn")]
         public async Task CloseBraceWithBaseIndentation()
         {
             var markup = @"
@@ -473,8 +445,7 @@ $$}
             await AssertSmartTokenFormatterCloseBraceWithBaseIndentation(markup, baseIndentation: 7, expectedIndentation: 11);
         }
 
-        [WorkItem(766159, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/766159")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/766159")]
         public async Task TestPreprocessor()
         {
             var code = @"
@@ -493,8 +464,7 @@ class C
             Assert.Equal(0, actualIndentation);
         }
 
-        [WorkItem(766159, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/766159")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/766159")]
         public async Task TestRegion()
         {
             var code = @"
@@ -513,8 +483,7 @@ class C
             Assert.Equal(8, actualIndentation);
         }
 
-        [WorkItem(766159, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/766159")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/766159")]
         public async Task TestEndRegion()
         {
             var code = @"
@@ -534,8 +503,7 @@ class C
             Assert.Equal(8, actualIndentation);
         }
 
-        [WorkItem(777467, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/777467")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/777467")]
         public async Task TestSelect()
         {
             var code = @"
@@ -559,8 +527,7 @@ class Program
             Assert.Equal(15, actualIndentation);
         }
 
-        [WorkItem(777467, "http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/777467")]
-        [Fact, Trait(Traits.Feature, Traits.Features.SmartTokenFormatting)]
+        [Fact, WorkItem("http://vstfdevdiv:8080/DevDiv2/DevDiv/_workitems/edit/777467")]
         public async Task TestWhere()
         {
             var code = @"
@@ -645,12 +612,9 @@ class Program
             // create tree service
             using var workspace = TestWorkspace.CreateCSharp(code);
 
-            workspace.TryApplyChanges(workspace.CurrentSolution.WithOptions(workspace.Options
-                .WithChangedOption(FormattingOptions2.UseTabs, LanguageNames.CSharp, useTabs)));
-
             var buffer = workspace.Documents.First().GetTextBuffer();
 
-            var actual = await TokenFormatAsync(workspace, buffer, indentationLine, '{');
+            var actual = await TokenFormatAsync(workspace, buffer, indentationLine, '{', useTabs);
             Assert.Equal(expected, actual);
         }
 
