@@ -19,26 +19,135 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.SymbolId
     [UseExportProvider]
     public class SymbolKeyTests
     {
-        [Fact, WorkItem(45437, "https://github.com/dotnet/roslyn/issues/45437")]
+        [Fact]
+        public async Task FileType_01()
+        {
+            var typeSource = """
+                file class C1
+                {
+                    public static void M() { }
+                }
+                """;
+
+            var workspaceXml = @$"
+<Workspace>
+    <Project Language=""C#"">
+        <CompilationOptions Nullable=""Enable""/>
+        <Document FilePath=""C.cs"">
+{typeSource}
+        </Document>
+    </Project>
+</Workspace>
+";
+            using var workspace = TestWorkspace.Create(workspaceXml);
+
+            var solution = workspace.CurrentSolution;
+            var project = solution.Projects.Single();
+
+            var compilation = await project.GetCompilationAsync();
+            var type = compilation.GlobalNamespace.GetMembers("C1").Single();
+            Assert.NotNull(type);
+            var symbolKey = SymbolKey.Create(type);
+            var resolved = symbolKey.Resolve(compilation).Symbol;
+            Assert.Same(type, resolved);
+        }
+
+        [Fact]
+        public async Task FileType_02()
+        {
+            var workspaceXml = $$"""
+<Workspace>
+    <Project Language="C#">
+        <CompilationOptions Nullable="Enable"/>
+        <Document FilePath="File0.cs">
+file class C
+{
+    public static void M() { }
+}
+        </Document>
+        <Document FilePath="File1.cs">
+file class C
+{
+    public static void M() { }
+}
+        </Document>
+    </Project>
+</Workspace>
+""";
+            using var workspace = TestWorkspace.Create(workspaceXml);
+
+            var solution = workspace.CurrentSolution;
+            var project = solution.Projects.Single();
+
+            var compilation = await project.GetCompilationAsync();
+
+            var members = compilation.GlobalNamespace.GetMembers("C").ToArray();
+            Assert.Equal(2, members.Length);
+
+            var type = members[0];
+            Assert.NotNull(type);
+            var symbolKey = SymbolKey.Create(type);
+            var resolved = symbolKey.Resolve(compilation).Symbol;
+            Assert.Same(type, resolved);
+
+            type = members[1];
+            Assert.NotNull(type);
+            symbolKey = SymbolKey.Create(type);
+            resolved = symbolKey.Resolve(compilation).Symbol;
+            Assert.Same(type, resolved);
+        }
+
+        [Fact]
+        public async Task FileType_03()
+        {
+            var workspaceXml = $$"""
+<Workspace>
+    <Project Language="C#">
+        <CompilationOptions Nullable="Enable"/>
+        <Document FilePath="File0.cs">
+file class C
+{
+    public class Inner { }
+}
+        </Document>
+    </Project>
+</Workspace>
+""";
+            using var workspace = TestWorkspace.Create(workspaceXml);
+
+            var solution = workspace.CurrentSolution;
+            var project = solution.Projects.Single();
+
+            var compilation = await project.GetCompilationAsync();
+
+            var type = compilation.GlobalNamespace.GetMembers("C").Single().GetMembers("Inner").Single();
+            Assert.NotNull(type);
+            var symbolKey = SymbolKey.Create(type);
+            var resolved = symbolKey.Resolve(compilation).Symbol;
+            Assert.Same(type, resolved);
+        }
+
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/45437")]
         public async Task TestGenericsAndNullability()
         {
-            var typeSource = @"
-#nullable enable
+            var typeSource = """
+                #nullable enable
 
-    public sealed class ConditionalWeakTableTest<TKey, TValue> /*: IEnumerable<KeyValuePair<TKey, TValue>>, IEnumerable*/
-        where TKey : class
-        where TValue : class
-    {
-        public ConditionalWeakTable() { }
-        public void Add(TKey key, TValue value) { }
-        public void AddOrUpdate(TKey key, TValue value) { }
-        public void Clear() { }
-        public TValue GetOrCreateValue(TKey key) => default;
-        public TValue GetValue(TKey key, ConditionalWeakTableTest<TKey, TValue>.CreateValueCallback createValueCallback) => default;
-        public bool Remove(TKey key) => false;
+                    public sealed class ConditionalWeakTableTest<TKey, TValue> /*: IEnumerable<KeyValuePair<TKey, TValue>>, IEnumerable*/
+                        where TKey : class
+                        where TValue : class
+                    {
+                        public ConditionalWeakTable() { }
+                        public void Add(TKey key, TValue value) { }
+                        public void AddOrUpdate(TKey key, TValue value) { }
+                        public void Clear() { }
+                        public TValue GetOrCreateValue(TKey key) => default;
+                        public TValue GetValue(TKey key, ConditionalWeakTableTest<TKey, TValue>.CreateValueCallback createValueCallback) => default;
+                        public bool Remove(TKey key) => false;
 
-        public delegate TValue CreateValueCallback(TKey key);
-    }".Replace("<", "&lt;").Replace(">", "&gt;");
+                        public delegate TValue CreateValueCallback(TKey key);
+                    }
+                """.Replace("<", "&lt;").Replace(">", "&gt;");
 
             var workspaceXml = @$"
 <Workspace>
@@ -69,10 +178,9 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.SymbolId
             Assert.Equal(method, resolved);
         }
 
-        [Fact]
-        [WorkItem(1178861, "https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1178861")]
-        [WorkItem(1192188, "https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1192188")]
-        [WorkItem(1192486, "https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1192486")]
+        [Fact, WorkItem("https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1178861")]
+        [WorkItem("https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1192188")]
+        [WorkItem("https://devdiv.visualstudio.com/DevDiv/_workitems/edit/1192486")]
         public async Task ResolveBodySymbolsInMultiProjectReferencesToOriginalProjectAsync()
         {
             var random = new Random(Seed: 0);
@@ -102,24 +210,26 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.SymbolId
 
             TestWorkspace GetWorkspace()
             {
-                var bodyProject = @"
-    <Project Language=""C#"" AssemblyName=""BodyProject"" CommonReferences=""true"">
-        <Document>
-class Program
-{
-    void M()
-    {
-        int local;
-    }
-}
-        </Document>
-    </Project>";
-                var referenceProject = @"
-    <Project Language=""C#"" AssemblyName=""ReferenceProject"" CommonReferences=""true"">
-        <ProjectReference>BodyProject</ProjectReference>
-        <Document>
-        </Document>
-    </Project>";
+                var bodyProject = """
+                        <Project Language="C#" AssemblyName="BodyProject" CommonReferences="true">
+                            <Document>
+                    class Program
+                    {
+                        void M()
+                        {
+                            int local;
+                        }
+                    }
+                            </Document>
+                        </Project>
+                    """;
+                var referenceProject = """
+                    <Project Language="C#" AssemblyName="ReferenceProject" CommonReferences="true">
+                        <ProjectReference>BodyProject</ProjectReference>
+                        <Document>
+                        </Document>
+                    </Project>
+                    """;
 
                 // Randomize the order of the projects in the workspace.
                 if (random.Next() % 2 == 0)
