@@ -6,7 +6,7 @@
 
 using System.Threading;
 using System.Threading.Tasks;
-using Microsoft.CodeAnalysis.LanguageServices;
+using Microsoft.CodeAnalysis.LanguageService;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Roslyn.Test.Utilities;
@@ -14,6 +14,7 @@ using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.TypeInferrer
 {
+    [Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
     public partial class TypeInferrerTests
     {
         private async Task TestDelegateAsync(string text, string expectedType)
@@ -34,151 +35,169 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.TypeInferrer
             Assert.Equal(expectedType, delegateType.ToNameDisplayString());
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestDeclaration1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    Func<int> q = [|here|];
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    Func<int> q = [|here|];
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestAssignment1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    Func<int> f;
-    f = [|here|]
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    Func<int> f;
+                    f = [|here|]
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestArgument1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    Bar([|here|]);
-  }
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    Bar([|here|]);
+                  }
 
-  void Bar(Func<int> f);
-}";
+                  void Bar(Func<int> f);
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestConstructor1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    new C([|here|]);
-  }
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    new C([|here|]);
+                  }
 
-  public C(Func<int> f);
-}";
+                  public C(Func<int> f);
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestDelegateConstructor1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    new Func<int>([|here|]);
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    new Func<int>([|here|]);
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestCastExpression1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    (Func<int>)[|here|]
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    (Func<int>)[|here|]
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestCastExpression2()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    (Func<int>)([|here|]
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    (Func<int>)([|here|]
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestReturnFromMethod()
         {
             var text =
-@"using System;
-class C
-{
-  Func<int> M()
-  {
-    return [|here|]
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  Func<int> M()
+                  {
+                    return [|here|]
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<int>");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.TypeInferenceService)]
+        [Fact]
         public async Task TestInsideLambda1()
         {
             var text =
-@"using System;
-class C
-{
-  void M()
-  {
-    Func<int,Func<string,bool>> f = i => [|here|]
-  }
-}";
+                """
+                using System;
+                class C
+                {
+                  void M()
+                  {
+                    Func<int,Func<string,bool>> f = i => [|here|]
+                  }
+                }
+                """;
 
             await TestDelegateAsync(text, "System.Func<string, bool>");
         }
