@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Roslyn.Test.Utilities;
@@ -11,417 +9,462 @@ using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Recommendations
 {
+    [Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
     public class AssemblyKeywordRecommenderTests : KeywordRecommenderTests
     {
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAtRoot_Interactive()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Script,
 @"$$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterClass_Interactive()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Script,
-@"class C { }
-$$");
+                """
+                class C { }
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterGlobalStatement_Interactive()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Script,
-@"System.Console.WriteLine();
-$$");
+                """
+                System.Console.WriteLine();
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterGlobalVariableDeclaration_Interactive()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Script,
-@"int i = 0;
-$$");
+                """
+                int i = 0;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInUsingAlias()
         {
             await VerifyAbsenceAsync(
 @"using Goo = $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInGlobalUsingAlias()
         {
             await VerifyAbsenceAsync(
 @"global using Goo = $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInEmptyStatement()
         {
             await VerifyAbsenceAsync(AddInsideMethod(
 @"$$"));
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeInsideClass()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    [$$");
+                """
+                class C {
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeAfterAttributeInsideClass()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    [Goo]
-    [$$");
+                """
+                class C {
+                    [Goo]
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeAfterMethod()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    void Goo() {
-    }
-    [$$");
+                """
+                class C {
+                    void Goo() {
+                    }
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeAfterProperty()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    int Goo {
-        get;
-    }
-    [$$");
+                """
+                class C {
+                    int Goo {
+                        get;
+                    }
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeAfterField()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    int Goo;
-    [$$");
+                """
+                class C {
+                    int Goo;
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeAfterEvent()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    event Action<int> Goo;
-    [$$");
+                """
+                class C {
+                    event Action<int> Goo;
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestInOuterAttribute()
         {
             await VerifyKeywordAsync(
 @"[$$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInAttributeNestClass()
         {
             await VerifyAbsenceAsync(
-@"class A
-{
-    [$$
-    class B
-    {
-    }
-}");
+                """
+                class A
+                {
+                    [$$
+                    class B
+                    {
+                    }
+                }
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeNamespace()
         {
             await VerifyKeywordAsync(
-@"[$$
-namespace Goo {");
+                """
+                [$$
+                namespace Goo {
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeFileScopedNamespace()
         {
             await VerifyKeywordAsync(
-@"[$$
-namespace Goo;");
+                """
+                [$$
+                namespace Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInAttributeBeforeNamespaceWithoutOpenBracket()
         {
             await VerifyAbsenceAsync(
-@"$$
-namespace Goo {}");
+                """
+                $$
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInAttributeBeforeNamespaceAndAfterUsingWithNoOpenBracket()
         {
             await VerifyAbsenceAsync(
-@"
-using System;
+                """
+                using System;
 
-$$
-namespace Goo {}");
+                $$
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeBeforeNamespaceAndAfterGlobalUsingWithNoOpenBracket()
         {
             await VerifyAbsenceAsync(
-@"
-global using System;
+                """
+                global using System;
 
-$$
-namespace Goo {}");
+                $$
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeNamespaceAndAfterUsingWithOpenBracket()
         {
             await VerifyKeywordAsync(
-@"
-using System;
+                """
+                using System;
 
-[$$
-namespace Goo {}");
+                [$$
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestInAttributeBeforeNamespaceAndAfterGlobalUsingWithOpenBracket()
         {
             await VerifyKeywordAsync(
-@"
-global using System;
+                """
+                global using System;
 
-[$$
-namespace Goo {}");
+                [$$
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeAssemblyWithOpenBracket()
         {
             await VerifyKeywordAsync(
-@"
-[$$
-[assembly: Whatever]
-namespace Goo {}");
+                """
+                [$$
+                [assembly: Whatever]
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeClass()
         {
             await VerifyKeywordAsync(
-@"
-[$$
-class Goo {}");
+                """
+                [$$
+                class Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeInterface()
         {
             await VerifyKeywordAsync(
-@"
-[$$
-interface IGoo {}");
+                """
+                [$$
+                interface IGoo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeStruct()
         {
             await VerifyKeywordAsync(
-@"
-[$$
-struct Goo {}");
+                """
+                [$$
+                struct Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInAttributeBeforeEnum()
         {
             await VerifyKeywordAsync(
-@"
-[$$
-enum Goo {}");
+                """
+                [$$
+                enum Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInAttributeBeforeOtherAttributeWithoutOpenBracket()
         {
             await VerifyAbsenceAsync(
-@"
-$$
-[assembly: Whatever]
-namespace Goo {}");
+                """
+                $$
+                [assembly: Whatever]
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInAttributeBeforeAssemblyAttributeAndAfterUsingWithoutOpenBracket()
         {
             await VerifyAbsenceAsync(
-@"
-using System;
+                """
+                using System;
 
-$$
-[assembly: Whatever]
-namespace Goo {}");
+                $$
+                [assembly: Whatever]
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInAttributeBeforeAssemblyAttributeAndAfterGlobalUsingWithoutOpenBracket()
         {
             await VerifyAbsenceAsync(
-@"
-global using System;
+                """
+                global using System;
 
-$$
-[assembly: Whatever]
-namespace Goo {}");
+                $$
+                [assembly: Whatever]
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestInBeforeAttributeAssemblyAttributeAndAfterUsingWithoutOpenBracket()
         {
             await VerifyKeywordAsync(
-@"
-using System;
+                """
+                using System;
 
-[$$
-[assembly: Whatever]
-namespace Goo {}");
+                [$$
+                [assembly: Whatever]
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestInBeforeAttributeAssemblyAttributeAndAfterGlobalUsingWithoutOpenBracket()
         {
             await VerifyKeywordAsync(
-@"
-global using System;
+                """
+                global using System;
 
-[$$
-[assembly: Whatever]
-namespace Goo {}");
+                [$$
+                [assembly: Whatever]
+                namespace Goo {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInOuterAttributeInNamespace()
         {
             await VerifyAbsenceAsync(
-@"namespace Goo {
-     [$$");
+                """
+                namespace Goo {
+                     [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInParameterAttribute()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    void Goo([$$");
+                """
+                class C {
+                    void Goo([$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInElementAccess()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    void Goo(string[] array) {
-        array[$$");
+                """
+                class C {
+                    void Goo(string[] array) {
+                        array[$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
-        [WorkItem(362, "https://github.com/dotnet/roslyn/issues/362")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/362")]
         public async Task TestNotInIndex()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    public int this[$$");
+                """
+                class C {
+                    public int this[$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInPropertyAttribute()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    int Goo { [$$");
+                """
+                class C {
+                    int Goo { [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInEventAttribute()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    event Action<int> Goo { [$$");
+                """
+                class C {
+                    event Action<int> Goo { [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInClassAssemblyParameters()
         {
             await VerifyAbsenceAsync(
 @"class C<[$$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInDelegateAssemblyParameters()
         {
             await VerifyAbsenceAsync(
 @"delegate void D<[$$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInMethodAssemblyParameters()
         {
             await VerifyAbsenceAsync(
-@"class C {
-    void M<[$$");
+                """
+                class C {
+                    void M<[$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInInterface()
         {
             await VerifyAbsenceAsync(
-@"interface I {
-    [$$");
+                """
+                interface I {
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInStruct()
         {
             await VerifyAbsenceAsync(
-@"struct S {
-    [$$");
+                """
+                struct S {
+                    [$$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInEnum()
         {
             await VerifyAbsenceAsync(
-@"enum E {
-    [$$");
+                """
+                enum E {
+                    [$$
+                """);
         }
     }
 }
