@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CSharp.Completion.KeywordRecommenders;
@@ -12,32 +10,29 @@ using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Recommendations
 {
-    public class ManagedKeywordRecommenderTests : RecommenderTests
+    [Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+    public class ManagedKeywordRecommenderTests : KeywordRecommenderTests
     {
-        private readonly ManagedKeywordRecommender _recommender = new ManagedKeywordRecommender();
-
-        public ManagedKeywordRecommenderTests()
-        {
-            this.keywordText = "managed";
-            this.RecommendKeywordsAsync = (position, context) => Task.FromResult(_recommender.RecommendKeywords(position, context, CancellationToken.None));
-        }
-
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestInFunctionPointerDeclaration()
         {
             await VerifyKeywordAsync(
-@"class Test {
-    unsafe void N() {
-        delegate* $$");
+                """
+                class Test {
+                    unsafe void N() {
+                        delegate* $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestInFunctionPointerDeclarationTouchingAsterisk()
         {
             await VerifyKeywordAsync(
-@"class Test {
-    unsafe void N() {
-        delegate*$$");
+                """
+                class Test {
+                    unsafe void N() {
+                        delegate*$$
+                """);
         }
     }
 }
