@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Roslyn.Test.Utilities;
@@ -11,47 +9,54 @@ using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.UnitTests.Recommendations
 {
+    [Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
     public class UsingKeywordRecommenderTests : KeywordRecommenderTests
     {
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInUsingAlias()
         {
             await VerifyAbsenceAsync(
 @"using Goo = $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInGlobalUsingAlias()
         {
             await VerifyAbsenceAsync(
 @"global using Goo = $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterClass()
         {
             await VerifyKeywordAsync(
-@"class C { }
-$$");
+                """
+                class C { }
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalStatement_Interactive()
         {
             await VerifyKeywordAsync(SourceCodeKind.Script,
-@"System.Console.WriteLine();
-$$");
+                """
+                System.Console.WriteLine();
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalVariableDeclaration_Interactive()
         {
             await VerifyKeywordAsync(SourceCodeKind.Script,
-@"int i = 0;
-$$");
+                """
+                int i = 0;
+                $$
+                """);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestInEmptyStatement(bool topLevelStatement)
         {
@@ -59,444 +64,538 @@ $$");
 @"$$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterAwait()
         {
             await VerifyKeywordAsync(
-@"class C
-{
-    async void M()
-    {
-        await $$
-    }
-}");
+                """
+                class C
+                {
+                    async void M()
+                    {
+                        await $$
+                    }
+                }
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterAwaitInAssignment()
         {
             await VerifyAbsenceAsync(
-@"class C
-{
-    async void M()
-    {
-        _ = await $$
-    }
-}");
+                """
+                class C
+                {
+                    async void M()
+                    {
+                        _ = await $$
+                    }
+                }
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAtRoot()
         {
             await VerifyKeywordAsync(
 @"$$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterUsingKeyword()
             => await VerifyAbsenceAsync(@"using $$");
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterPreviousUsing()
         {
             await VerifyKeywordAsync(
-@"using Goo;
-$$");
+                """
+                using Goo;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterPreviousGlobalUsing()
         {
             await VerifyKeywordAsync(
-@"global using Goo;
-$$");
+                """
+                global using Goo;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterExtern()
         {
             await VerifyKeywordAsync(
-@"extern alias goo;
-$$");
+                """
+                extern alias goo;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalAfterExtern()
         {
             await VerifyKeywordAsync(
-@"extern alias goo;
-global $$");
+                """
+                extern alias goo;
+                global $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalAfterExternBeforeUsing_01()
         {
             await VerifyKeywordAsync(
-@"extern alias goo;
-global $$
-using Goo;");
+                """
+                extern alias goo;
+                global $$
+                using Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalAfterExternBeforeUsing_02()
         {
             await VerifyKeywordAsync(
-@"extern alias goo;
-global $$
-global using Goo;");
+                """
+                extern alias goo;
+                global $$
+                global using Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeUsing()
         {
             await VerifyKeywordAsync(
-@"$$
-using Goo;");
+                """
+                $$
+                using Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeUsingAfterGlobal()
         {
             await VerifyKeywordAsync(
-@"global $$
-using Goo;");
+                """
+                global $$
+                using Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterUsingAlias()
         {
             await VerifyKeywordAsync(
-@"using Goo = Bar;
-$$");
+                """
+                using Goo = Bar;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalUsingAlias()
         {
             await VerifyKeywordAsync(
-@"global using Goo = Bar;
-$$");
+                """
+                global using Goo = Bar;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterNestedTypeDeclaration()
         {
-            await VerifyAbsenceAsync(@"class A {
-    class C {}
-    $$");
+            await VerifyAbsenceAsync("""
+                class A {
+                    class C {}
+                    $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestInsideNamespace()
         {
             await VerifyKeywordAsync(
-@"namespace N {
-    $$");
+                """
+                namespace N {
+                    $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterFileScopedNamespace()
         {
             await VerifyKeywordAsync(
-@"namespace N;
-$$");
+                """
+                namespace N;
+                $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterUsingKeyword_InsideNamespace()
         {
-            await VerifyAbsenceAsync(@"namespace N {
-    using $$");
+            await VerifyAbsenceAsync("""
+                namespace N {
+                    using $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterPreviousUsing_InsideNamespace()
         {
             await VerifyKeywordAsync(
-@"namespace N {
-   using Goo;
-   $$");
+                """
+                namespace N {
+                   using Goo;
+                   $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeUsing_InsideNamespace()
         {
             await VerifyKeywordAsync(
-@"namespace N {
-    $$
-    using Goo;");
+                """
+                namespace N {
+                    $$
+                    using Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterMember_InsideNamespace()
         {
-            await VerifyAbsenceAsync(@"namespace N {
-    class C {}
-    $$");
+            await VerifyAbsenceAsync("""
+                namespace N {
+                    class C {}
+                    $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterNestedMember_InsideNamespace()
         {
-            await VerifyAbsenceAsync(@"namespace N {
-    class A {
-      class C {}
-      $$");
+            await VerifyAbsenceAsync("""
+                namespace N {
+                    class A {
+                      class C {}
+                      $$
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotBeforeExtern()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Regular,
-@"$$
-extern alias Goo;");
+                """
+                $$
+                extern alias Goo;
+                """);
         }
 
-        [WpfFact(Skip = "https://github.com/dotnet/roslyn/issues/9880"), Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact(Skip = "https://github.com/dotnet/roslyn/issues/9880")]
         public async Task TestNotBeforeExtern_Interactive()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Script,
-@"$$
-extern alias Goo;");
+                """
+                $$
+                extern alias Goo;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotBeforeExternAfterGlobal()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Regular,
-@"global $$
-extern alias Goo;");
+                """
+                global $$
+                extern alias Goo;
+                """);
         }
 
-        [WpfFact(Skip = "https://github.com/dotnet/roslyn/issues/9880"), Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact(Skip = "https://github.com/dotnet/roslyn/issues/9880")]
         public async Task TestNotBeforeExternAfterGlobal_Interactive()
         {
             await VerifyAbsenceAsync(SourceCodeKind.Script,
-@"global $$
-extern alias Goo;");
+                """
+                global $$
+                extern alias Goo;
+                """);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestBeforeStatement(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"$$
-return true;", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                $$
+                return true;
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterStatement(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"return true;
-$$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                return true;
+                $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterBlock(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"if (true) {
-}
-$$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                if (true) {
+                }
+                $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterIf(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"if (true) 
-    $$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                if (true) 
+                    $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterDo(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"do 
-    $$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                do 
+                    $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterWhile(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"while (true) 
-    $$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                while (true) 
+                    $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterFor(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"for (int i = 0; i < 10; i++) 
-    $$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                for (int i = 0; i < 10; i++) 
+                    $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Theory, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Theory]
         [CombinatorialData]
         public async Task TestAfterForeach(bool topLevelStatement)
         {
             await VerifyKeywordAsync(AddInsideMethod(
-@"foreach (var v in bar)
-    $$", topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
+                """
+                foreach (var v in bar)
+                    $$
+                """, topLevelStatement: topLevelStatement), options: CSharp9ParseOptions);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterUsing()
         {
             await VerifyAbsenceAsync(
 @"using $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotAfterGlobalUsing()
         {
             await VerifyAbsenceAsync(
 @"global using $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestNotInClass()
         {
-            await VerifyAbsenceAsync(@"class C
-{
-  $$
-}");
+            await VerifyAbsenceAsync("""
+                class C
+                {
+                  $$
+                }
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBetweenUsings_01()
         {
             await VerifyKeywordAsync(
-@"using Goo;
-$$
-using Bar;");
+                """
+                using Goo;
+                $$
+                using Bar;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBetweenUsings_02()
         {
             await VerifyKeywordAsync(
-@"global using Goo;
-$$
-using Bar;");
+                """
+                global using Goo;
+                $$
+                using Bar;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalBetweenUsings_01()
         {
             await VerifyKeywordAsync(
-@"global using Goo;
-global $$
-using Bar;");
+                """
+                global using Goo;
+                global $$
+                using Bar;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobalBetweenUsings_02()
         {
             await VerifyKeywordAsync(
-@"global using Goo;
-global $$
-global using Bar;");
+                """
+                global using Goo;
+                global $$
+                global using Bar;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestAfterGlobal()
         {
             await VerifyKeywordAsync(
 @"global $$");
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeNamespace()
         {
             await VerifyKeywordAsync(
-@"$$
-namespace NS
-{}");
+                """
+                $$
+                namespace NS
+                {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeFileScopedNamespace()
         {
             await VerifyKeywordAsync(
-@"$$
-namespace NS;");
+                """
+                $$
+                namespace NS;
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeClass()
         {
             await VerifyKeywordAsync(
-@"$$
-class C1
-{}");
+                """
+                $$
+                class C1
+                {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeAttribute_01()
         {
             await VerifyKeywordAsync(
-@"$$
-[Call()]");
+                """
+                $$
+                [Call()]
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeAttribute_02()
         {
             await VerifyKeywordAsync(
-@"$$
-[assembly: Call()]");
+                """
+                $$
+                [assembly: Call()]
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeNamespaceAfterGlobal()
         {
             await VerifyKeywordAsync(
-@"global $$
-namespace NS
-{}");
+                """
+                global $$
+                namespace NS
+                {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeClassAfterGlobal()
         {
             await VerifyKeywordAsync(
-@"global $$
-class C1
-{}");
+                """
+                global $$
+                class C1
+                {}
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeStatementAfterGlobal()
         {
             await VerifyKeywordAsync(
-@"global $$
-Call();");
+                """
+                global $$
+                Call();
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeAttributeAfterGlobal_01()
         {
             await VerifyKeywordAsync(
-@"global $$
-[Call()]");
+                """
+                global $$
+                [Call()]
+                """);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.KeywordRecommending)]
+        [Fact]
         public async Task TestBeforeAttributeAfterGlobal_02()
         {
             await VerifyKeywordAsync(
-@"global $$
-[assembly: Call()]");
+                """
+                global $$
+                [assembly: Call()]
+                """);
         }
     }
 }
