@@ -731,6 +731,42 @@ namespace Microsoft.CodeAnalysis.Editor.Implementation.Classification
         }
         #endregion
 
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.TestCode)]
+        [Name(ClassificationTypeNames.TestCode)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class TestCodeFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public TestCodeFormatDefinition()
+            {
+                this.DisplayName = EditorFeaturesResources.Roslyn_Test_Code;
+                this.BackgroundColor = Color.FromRgb(0xe5, 0xe5, 0xe5);
+            }
+        }
+
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.TestCodeMarkdown)]
+        [Name(ClassificationTypeNames.TestCodeMarkdown)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class TestCodeMarkdownFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public TestCodeMarkdownFormatDefinition()
+            {
+                this.DisplayName = EditorFeaturesResources.Roslyn_Test_Code_Markup;
+                this.ForegroundColor = Color.FromRgb(0xff, 0x00, 0xc1);
+            }
+        }
+
         #region Regex
 
         [Export(typeof(EditorFormatDefinition))]
@@ -900,6 +936,164 @@ namespace Microsoft.CodeAnalysis.Editor.Implementation.Classification
         }
         #endregion
 
+        #region JSON
+
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonComment)]
+        [Name(ClassificationTypeNames.JsonComment)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonCommentFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonCommentFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Comment;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonNumber)]
+        [Name(ClassificationTypeNames.JsonNumber)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonNumberFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonNumberFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Number;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonString)]
+        [Name(ClassificationTypeNames.JsonString)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonStringFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonStringFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_String;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonKeyword)]
+        [Name(ClassificationTypeNames.JsonKeyword)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonKeywordFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonKeywordFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Keyword;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonText)]
+        [Name(ClassificationTypeNames.JsonText)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonTextFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonTextFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Text;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonOperator)]
+        [Name(ClassificationTypeNames.JsonOperator)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonOperatorFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonOperatorFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Operator;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonPunctuation)]
+        [Name(ClassificationTypeNames.JsonPunctuation)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonPunctuationFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonPunctuationFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Punctuation;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonObject)]
+        [Name(ClassificationTypeNames.JsonObject)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonObjectFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonObjectFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Object;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonArray)]
+        [Name(ClassificationTypeNames.JsonArray)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonArrayFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonArrayFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Array;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonPropertyName)]
+        [Name(ClassificationTypeNames.JsonPropertyName)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonPropertyNameFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonPropertyNameFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Property_Name;
+        }
+        [Export(typeof(EditorFormatDefinition))]
+        [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.JsonConstructorName)]
+        [Name(ClassificationTypeNames.JsonConstructorName)]
+        [Order(After = ClassificationTypeNames.StringLiteral)]
+        [Order(After = ClassificationTypeNames.VerbatimStringLiteral)]
+        [UserVisible(true)]
+        [ExcludeFromCodeCoverage]
+        private class JsonConstructorNameFormatDefinition : ClassificationFormatDefinition
+        {
+            [ImportingConstructor]
+            [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+            public JsonConstructorNameFormatDefinition()
+                => this.DisplayName = EditorFeaturesResources.JSON_in_string_literal_Constructor_Name;
+        }
+        #endregion
+
         #region VB XML Literals - Attribute Name
         [Export(typeof(EditorFormatDefinition))]
         [ClassificationType(ClassificationTypeNames = ClassificationTypeNames.XmlLiteralAttributeName)]
