@@ -3,16 +3,17 @@
 // See the LICENSE file in the project root for more information.
 
 using Microsoft.CodeAnalysis.Host;
+using Microsoft.CodeAnalysis.Options;
 using Microsoft.VisualStudio.InteractiveWindow;
 
-namespace Microsoft.CodeAnalysis.Editor.Interactive
+namespace Microsoft.CodeAnalysis.Interactive
 {
     internal sealed class InteractiveWindowWorkspace : InteractiveWorkspace
     {
         public IInteractiveWindow? Window { get; set; }
 
-        public InteractiveWindowWorkspace(HostServices hostServices)
-            : base(hostServices)
+        public InteractiveWindowWorkspace(HostServices hostServices, IGlobalOptionService globalOptions)
+            : base(hostServices, globalOptions)
         {
         }
     }
