@@ -6,12 +6,9 @@ using Microsoft.CodeAnalysis.Debugging;
 
 namespace Microsoft.CodeAnalysis.ExternalAccess.VSTypeScript.Api
 {
-    internal readonly struct VSTypeScriptDebugLocationInfoWrapper
+    internal readonly struct VSTypeScriptDebugLocationInfoWrapper(string name, int lineOffset)
     {
-        internal readonly DebugLocationInfo UnderlyingObject;
-
-        public VSTypeScriptDebugLocationInfoWrapper(string name, int lineOffset)
-            => UnderlyingObject = new DebugLocationInfo(name, lineOffset);
+        internal readonly DebugLocationInfo UnderlyingObject = new DebugLocationInfo(name, lineOffset);
 
         public readonly string Name => UnderlyingObject.Name;
         public readonly int LineOffset => UnderlyingObject.LineOffset;
