@@ -7,14 +7,18 @@
 using System;
 using System.Collections.Generic;
 using System.Collections.Immutable;
-using System.Diagnostics;
 using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CodeActions;
 using Microsoft.CodeAnalysis.CodeFixes;
+using Microsoft.CodeAnalysis.CodeFixesAndRefactorings;
+using Microsoft.CodeAnalysis.CodeRefactorings;
+using Microsoft.CodeAnalysis.Diagnostics;
 using Microsoft.CodeAnalysis.Editor.UnitTests.CodeActions;
 using Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces;
+using Microsoft.CodeAnalysis.Remote.Testing;
+using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Shared.Utilities;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Microsoft.CodeAnalysis.Text;
@@ -22,9 +26,10 @@ using Microsoft.CodeAnalysis.UnitTests.Diagnostics;
 using Roslyn.Test.Utilities;
 using Roslyn.Utilities;
 using Xunit;
-using Microsoft.CodeAnalysis.Diagnostics;
-using Microsoft.CodeAnalysis.Remote.Testing;
 using Xunit.Abstractions;
+using FixAllContext = Microsoft.CodeAnalysis.CodeFixes.FixAllContext;
+using FixAllProvider = Microsoft.CodeAnalysis.CodeFixes.FixAllProvider;
+using FixAllState = Microsoft.CodeAnalysis.CodeFixes.FixAllState;
 
 namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
 {
@@ -42,11 +47,12 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             TestWorkspace workspace, TestParameters parameters);
 
         private protected async Task TestDiagnosticsAsync(
-            string initialMarkup, TestParameters parameters = default, params DiagnosticDescription[] expected)
+            string initialMarkup, TestParameters parameters = null, params DiagnosticDescription[] expected)
         {
-            using var workspace = CreateWorkspaceFromOptions(initialMarkup, parameters);
+            var ps = parameters ?? TestParameters.Default;
+            using var workspace = CreateWorkspaceFromOptions(initialMarkup, ps);
 
-            var diagnostics = await GetDiagnosticsAsync(workspace, parameters).ConfigureAwait(false);
+            var diagnostics = await GetDiagnosticsAsync(workspace, ps).ConfigureAwait(false);
 
             // Special case for single diagnostic reported with annotated span.
             if (expected.Length == 1 && !expected[0].HasLocation)
@@ -84,6 +90,9 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             return dxs;
         }
 
+        internal override Task<CodeRefactoring> GetCodeRefactoringAsync(TestWorkspace workspace, TestParameters parameters)
+            => throw new NotImplementedException("No refactoring provided in diagnostic test");
+
         protected static void AddAnalyzerToWorkspace(Workspace workspace, DiagnosticAnalyzer analyzer, TestParameters parameters)
         {
             AnalyzerReference[] analyzeReferences;
@@ -113,59 +122,6 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             return workspace.CurrentSolution.GetDocument(hostDocument.Id);
         }
 
-        protected static bool TryGetDocumentAndSelectSpan(TestWorkspace workspace, out Document document, out TextSpan span)
-        {
-            var hostDocument = workspace.Documents.FirstOrDefault(d => d.SelectedSpans.Any());
-            if (hostDocument == null)
-            {
-                // If there wasn't a span, see if there was a $$ caret.  we'll create an empty span
-                // there if so.
-                hostDocument = workspace.Documents.FirstOrDefault(d => d.CursorPosition != null);
-                if (hostDocument == null)
-                {
-                    document = null;
-                    span = default;
-                    return false;
-                }
-
-                span = new TextSpan(hostDocument.CursorPosition.Value, 0);
-                document = workspace.CurrentSolution.GetDocument(hostDocument.Id);
-                return true;
-            }
-
-            span = hostDocument.SelectedSpans.Single();
-            document = workspace.CurrentSolution.GetDocument(hostDocument.Id);
-            return true;
-        }
-
-        protected static Document GetDocumentAndAnnotatedSpan(TestWorkspace workspace, out string annotation, out TextSpan span)
-        {
-            var annotatedDocuments = workspace.Documents.Where(d => d.AnnotatedSpans.Any());
-            Debug.Assert(!annotatedDocuments.IsEmpty(), "No annotated span found");
-            var hostDocument = annotatedDocuments.Single();
-            var annotatedSpan = hostDocument.AnnotatedSpans.Single();
-            annotation = annotatedSpan.Key;
-            span = annotatedSpan.Value.Single();
-            return workspace.CurrentSolution.GetDocument(hostDocument.Id);
-        }
-
-        protected static FixAllScope? GetFixAllScope(string annotation)
-        {
-            if (annotation == null)
-            {
-                return null;
-            }
-
-            return annotation switch
-            {
-                "FixAllInDocument" => FixAllScope.Document,
-                "FixAllInProject" => FixAllScope.Project,
-                "FixAllInSolution" => FixAllScope.Solution,
-                "FixAllInSelection" => FixAllScope.Custom,
-                _ => throw new InvalidProgramException("Incorrect FixAll annotation in test"),
-            };
-        }
-
         internal async Task<(ImmutableArray<Diagnostic>, ImmutableArray<CodeAction>, CodeAction actionToInvoke)> GetDiagnosticAndFixesAsync(
             IEnumerable<Diagnostic> diagnostics,
             CodeFixProvider fixer,
@@ -181,20 +137,17 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             }
 
             var scope = GetFixAllScope(annotation);
-            return await GetDiagnosticAndFixesAsync(
-                diagnostics, fixer, testDriver, document, span, scope, index);
-        }
 
-        private async Task<(ImmutableArray<Diagnostic>, ImmutableArray<CodeAction>, CodeAction actionToinvoke)> GetDiagnosticAndFixesAsync(
-            IEnumerable<Diagnostic> diagnostics,
-            CodeFixProvider fixer,
-            TestDiagnosticAnalyzerDriver testDriver,
-            Document document,
-            TextSpan span,
-            FixAllScope? scope,
-            int index)
-        {
-            Assert.NotEmpty(diagnostics);
+            if (scope is FixAllScope.ContainingMember or FixAllScope.ContainingType &&
+                document.GetLanguageService<IFixAllSpanMappingService>() is IFixAllSpanMappingService spanMappingService)
+            {
+                var documentsAndSpansToFix = await spanMappingService.GetFixAllSpansAsync(
+                    document, span, scope.Value, CancellationToken.None).ConfigureAwait(false);
+                if (documentsAndSpansToFix.IsEmpty)
+                {
+                    return (ImmutableArray<Diagnostic>.Empty, ImmutableArray<CodeAction>.Empty, null);
+                }
+            }
 
             var intersectingDiagnostics = diagnostics.Where(d => d.Location.SourceSpan.IntersectsWith(span))
                                                      .ToImmutableArray();
@@ -204,8 +157,11 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             foreach (var diagnostic in intersectingDiagnostics)
             {
                 var context = new CodeFixContext(
-                    document, diagnostic,
+                    document,
+                    diagnostic.Location.SourceSpan,
+                    ImmutableArray.Create(diagnostic),
                     (a, d) => fixes.Add(new CodeFix(document.Project, a, d)),
+                    testDriver.FallbackOptions,
                     CancellationToken.None);
 
                 await fixer.RegisterCodeFixesAsync(context);
@@ -213,34 +169,29 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
 
             VerifyCodeActionsRegisteredByProvider(fixer, fixes);
 
-            var actions = fixes.SelectAsArray(f => f.Action);
-
-            actions = MassageActions(actions);
+            var actions = MassageActions(fixes.SelectAsArray(f => f.Action));
 
             if (scope == null)
             {
                 // Simple code fix.
                 return (intersectingDiagnostics, actions, actions.Length == 0 ? null : actions[index]);
             }
-            else
-            {
 
-                var equivalenceKey = actions[index].EquivalenceKey;
+            var equivalenceKey = actions[index].EquivalenceKey;
 
-                // Fix all fix.
-                var fixAllProvider = fixer.GetFixAllProvider();
-                Assert.NotNull(fixAllProvider);
+            // Fix all fix.
+            var fixAllProvider = fixer.GetFixAllProvider();
+            Assert.NotNull(fixAllProvider);
 
-                var fixAllState = GetFixAllState(
-                    fixAllProvider, diagnostics, fixer, testDriver, document,
-                    scope.Value, equivalenceKey);
-                var fixAllContext = new FixAllContext(fixAllState, new ProgressTracker(), CancellationToken.None);
-                var fixAllFix = await fixAllProvider.GetFixAsync(fixAllContext);
+            var fixAllState = GetFixAllState(
+                fixAllProvider, diagnostics, fixer, testDriver, document,
+                scope.Value, equivalenceKey, testDriver.FallbackOptions);
+            var fixAllContext = new FixAllContext(fixAllState, new ProgressTracker(), CancellationToken.None);
+            var fixAllFix = await fixAllProvider.GetFixAsync(fixAllContext);
 
-                // We have collapsed the fixes down to the single fix-all fix, so we just let our
-                // caller know they should pull that entry out of the result.
-                return (intersectingDiagnostics, ImmutableArray.Create(fixAllFix), fixAllFix);
-            }
+            // We have collapsed the fixes down to the single fix-all fix, so we just let our
+            // caller know they should pull that entry out of the result.
+            return (intersectingDiagnostics, ImmutableArray.Create(fixAllFix), fixAllFix);
         }
 
         private static FixAllState GetFixAllState(
@@ -250,7 +201,8 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             TestDiagnosticAnalyzerDriver testDriver,
             Document document,
             FixAllScope scope,
-            string equivalenceKey)
+            string equivalenceKey,
+            CodeActionOptionsProvider optionsProvider)
         {
             Assert.NotEmpty(diagnostics);
 
@@ -258,7 +210,7 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             {
                 // Bulk fixing diagnostics in selected scope.                    
                 var diagnosticsToFix = ImmutableDictionary.CreateRange(SpecializedCollections.SingletonEnumerable(KeyValuePairUtil.Create(document, diagnostics.ToImmutableArray())));
-                return FixAllState.Create(fixAllProvider, diagnosticsToFix, fixer, equivalenceKey);
+                return FixAllState.Create(fixAllProvider, diagnosticsToFix, fixer, equivalenceKey, optionsProvider);
             }
 
             var diagnostic = diagnostics.First();
@@ -266,8 +218,8 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             var fixAllDiagnosticProvider = new FixAllDiagnosticProvider(testDriver, diagnosticIds);
 
             return diagnostic.Location.IsInSource
-                ? new FixAllState(fixAllProvider, document, fixer, scope, equivalenceKey, diagnosticIds, fixAllDiagnosticProvider)
-                : new FixAllState(fixAllProvider, document.Project, fixer, scope, equivalenceKey, diagnosticIds, fixAllDiagnosticProvider);
+                ? new FixAllState(fixAllProvider, diagnostic.Location.SourceSpan, document, document.Project, fixer, scope, equivalenceKey, diagnosticIds, fixAllDiagnosticProvider, optionsProvider)
+                : new FixAllState(fixAllProvider, diagnosticSpan: null, document: null, document.Project, fixer, scope, equivalenceKey, diagnosticIds, fixAllDiagnosticProvider, optionsProvider);
         }
 
         private protected Task TestActionCountInAllFixesAsync(
@@ -276,11 +228,12 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
             ParseOptions parseOptions = null,
             CompilationOptions compilationOptions = null,
             OptionsCollection options = null,
+            OptionsCollection globalOptions = null,
             object fixProviderData = null)
         {
             return TestActionCountInAllFixesAsync(
                 initialMarkup,
-                new TestParameters(parseOptions, compilationOptions, options, fixProviderData),
+                new TestParameters(parseOptions, compilationOptions, options, globalOptions, fixProviderData),
                 count);
         }
 
@@ -298,22 +251,23 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Diagnostics
         internal async Task TestSpansAsync(
             string initialMarkup,
             string diagnosticId = null,
-            TestParameters parameters = default)
+            TestParameters parameters = null)
         {
-            MarkupTestFile.GetSpans(initialMarkup, out var unused, out ImmutableArray<TextSpan> spansList);
+            MarkupTestFile.GetSpans(initialMarkup, out var unused, out var spansList);
 
+            var ps = parameters ?? TestParameters.Default;
             var expectedTextSpans = spansList.ToSet();
-            using var workspace = CreateWorkspaceFromOptions(initialMarkup, parameters);
+            using var workspace = CreateWorkspaceFromOptions(initialMarkup, ps);
 
             ISet<TextSpan> actualTextSpans;
             if (diagnosticId == null)
             {
-                var (diagnostics, _, _) = await GetDiagnosticAndFixesAsync(workspace, parameters);
+                var (diagnostics, _, _) = await GetDiagnosticAndFixesAsync(workspace, ps);
                 actualTextSpans = diagnostics.Select(d => d.Location.SourceSpan).ToSet();
             }
             else
             {
-                var diagnostics = await GetDiagnosticsAsync(workspace, parameters);
+                var diagnostics = await GetDiagnosticsAsync(workspace, ps);
                 actualTextSpans = diagnostics.Where(d => d.Id == diagnosticId).Select(d => d.Location.SourceSpan).ToSet();
             }
 
