@@ -11,12 +11,13 @@ using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CSharp.EditAndContinue;
-using Microsoft.CodeAnalysis.EditAndContinue.Contracts;
+using Microsoft.CodeAnalysis.Contracts.EditAndContinue;
 using Microsoft.CodeAnalysis.Editor.UnitTests;
 using Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces;
 using Microsoft.CodeAnalysis.Emit;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Microsoft.CodeAnalysis.Text;
+using Microsoft.CodeAnalysis.UnitTests;
 using Moq;
 using Roslyn.Test.Utilities;
 using Roslyn.Utilities;
@@ -31,7 +32,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
     [UseExportProvider]
     public class EditSessionActiveStatementsTests : TestBase
     {
-        private static readonly TestComposition s_composition = EditorTestCompositions.EditorFeatures.AddParts(typeof(DummyLanguageService));
+        private static readonly TestComposition s_composition = EditorTestCompositions.EditorFeatures.AddParts(typeof(NoCompilationLanguageService));
 
         private static EditSession CreateEditSession(
             Solution solution,
@@ -51,6 +52,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
                 solution,
                 mockDebuggerService,
                 mockCompilationOutputsProvider,
+                NullPdbMatchingSourceTextProvider.Instance,
                 SpecializedCollections.EmptyEnumerable<KeyValuePair<DocumentId, CommittedSolution.DocumentState>>(),
                 reportDiagnostics: true);
 
@@ -63,7 +65,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
             return debuggingSession.EditSession;
         }
 
-        private static Solution AddDefaultTestSolution(TestWorkspace workspace, string[] markedSources)
+        private static async Task<Solution> AddDefaultTestSolutionAsync(TestWorkspace workspace, string[] markedSources)
         {
             var solution = workspace.CurrentSolution;
 
@@ -76,12 +78,12 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
             for (var i = 0; i < markedSources.Length; i++)
             {
                 var name = $"test{i + 1}.cs";
-                var text = SourceText.From(ActiveStatementsDescription.ClearTags(markedSources[i]), Encoding.UTF8);
+                var text = SourceText.From(SourceMarkers.Clear(markedSources[i]), Encoding.UTF8);
                 var id = DocumentId.CreateNewId(project.Id, name);
                 solution = solution.AddDocument(id, name, text, filePath: Path.Combine(TempRoot.Root, name));
             }
 
-            workspace.ChangeSolution(solution);
+            await workspace.ChangeSolutionAsync(solution);
             return solution;
         }
 
@@ -172,10 +174,10 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             using var workspace = new TestWorkspace(composition: s_composition);
 
-            var solution = AddDefaultTestSolution(workspace, markedSources);
+            var solution = await AddDefaultTestSolutionAsync(workspace, markedSources);
             var projectId = solution.ProjectIds.Single();
-            var dummyProject = solution.AddProject("dummy_proj", "dummy_proj", DummyLanguageService.LanguageName);
-            solution = dummyProject.Solution.AddDocument(DocumentId.CreateNewId(dummyProject.Id, DummyLanguageService.LanguageName), "a.dummy", "");
+            var dummyProject = solution.AddProject("dummy_proj", "dummy_proj", NoCompilationConstants.LanguageName);
+            solution = dummyProject.Solution.AddDocument(DocumentId.CreateNewId(dummyProject.Id, NoCompilationConstants.LanguageName), "a.dummy", "");
             var project = solution.GetProject(projectId);
             var document1 = project.Documents.Single(d => d.Name == "test1.cs");
             var document2 = project.Documents.Single(d => d.Name == "test2.cs");
@@ -226,7 +228,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             // Exception Regions
 
-            var analyzer = solution.GetProject(projectId).LanguageServices.GetRequiredService<IEditAndContinueAnalyzer>();
+            var analyzer = solution.GetProject(projectId).Services.GetRequiredService<IEditAndContinueAnalyzer>();
             var oldActiveStatements1 = await baseActiveStatementsMap.GetOldActiveStatementsAsync(analyzer, document1, CancellationToken.None).ConfigureAwait(false);
 
             AssertEx.Equal(new[]
@@ -273,11 +275,11 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             AssertEx.Equal(new[]
             {
-                $"0x06000004 v1 | AS {document2.FilePath}: (8,20)-(8,25) δ=1",
-                $"0x06000004 v1 | ER {document2.FilePath}: (14,8)-(16,9) δ=1",
-                $"0x06000004 v1 | ER {document2.FilePath}: (10,10)-(12,11) δ=1",
-                $"0x06000003 v1 | AS {document2.FilePath}: (21,14)-(21,24) δ=0",
-                $"0x06000005 v1 | AS {document2.FilePath}: (26,20)-(26,25) δ=0"
+                $"0x06000004 v1 | AS {document2.FilePath}: (8,20)-(8,25) => (9,20)-(9,25)",
+                $"0x06000004 v1 | ER {document2.FilePath}: (14,8)-(16,9) => (15,8)-(17,9)",
+                $"0x06000004 v1 | ER {document2.FilePath}: (10,10)-(12,11) => (11,10)-(13,11)",
+                $"0x06000003 v1 | AS {document2.FilePath}: (21,14)-(21,24) => (21,14)-(21,24)",
+                $"0x06000005 v1 | AS {document2.FilePath}: (26,20)-(26,25) => (26,20)-(26,25)"
             }, nonRemappableRegions.Select(r => $"{r.Method.GetDebuggerDisplay()} | {r.Region.GetDebuggerDisplay()}"));
 
             AssertEx.Equal(new[]
@@ -292,7 +294,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
             }, activeStatementsInUpdatedMethods.Select(InspectActiveStatementUpdate));
         }
 
-        [Fact, WorkItem(24439, "https://github.com/dotnet/roslyn/issues/24439")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/24439")]
         public async Task BaseActiveStatementsAndExceptionRegions2()
         {
             var baseSource =
@@ -334,7 +336,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
                 });
 
             using var workspace = new TestWorkspace(composition: s_composition);
-            var solution = AddDefaultTestSolution(workspace, new[] { baseSource });
+            var solution = await AddDefaultTestSolutionAsync(workspace, new[] { baseSource });
             var project = solution.Projects.Single();
             var document = project.Documents.Single();
 
@@ -353,7 +355,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             // Exception Regions
 
-            var analyzer = solution.GetProject(project.Id).LanguageServices.GetRequiredService<IEditAndContinueAnalyzer>();
+            var analyzer = solution.GetProject(project.Id).Services.GetRequiredService<IEditAndContinueAnalyzer>();
             var oldActiveStatements = await baseActiveStatementMap.GetOldActiveStatementsAsync(analyzer, document, CancellationToken.None).ConfigureAwait(false);
 
             // Note that the spans correspond to the base snapshot (V2). 
@@ -388,10 +390,10 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             AssertEx.Equal(new[]
             {
-                $"0x06000001 v1 | AS {document.FilePath}: (6,18)-(6,23) δ=0",
-                $"0x06000001 v1 | ER {document.FilePath}: (8,8)-(12,9) δ=0",
-                $"0x06000002 v1 | AS {document.FilePath}: (18,14)-(18,36) δ=0",
-            }, nonRemappableRegions.OrderBy(r => r.Region.Span.Span.Start.Line).Select(r => $"{r.Method.GetDebuggerDisplay()} | {r.Region.GetDebuggerDisplay()}"));
+                $"0x06000001 v1 | AS {document.FilePath}: (6,18)-(6,23) => (6,18)-(6,23)",
+                $"0x06000001 v1 | ER {document.FilePath}: (8,8)-(12,9) => (8,8)-(12,9)",
+                $"0x06000002 v1 | AS {document.FilePath}: (18,14)-(18,36) => (18,14)-(18,36)",
+            }, nonRemappableRegions.OrderBy(r => r.Region.OldSpan.Span.Start.Line).Select(r => $"{r.Method.GetDebuggerDisplay()} | {r.Region.GetDebuggerDisplay()}"));
 
             AssertEx.Equal(new[]
             {
@@ -487,7 +489,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
                     ActiveStatementFlags.None | ActiveStatementFlags.NonLeafFrame,           // F4
                 });
 
-            var exceptionSpans = ActiveStatementsDescription.GetExceptionRegions(markedSourceV1);
+            var exceptionSpans = SourceMarkers.GetExceptionRegions(markedSourceV1);
 
             var filePath = activeStatementsPreRemap[0].DocumentName;
             var spanPreRemap2 = new SourceFileSpan(filePath, activeStatementsPreRemap[2].SourceSpan.ToLinePositionSpan());
@@ -503,20 +505,20 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
             {
                 { new ManagedMethodId(module1, 0x06000003, 1), ImmutableArray.Create(
                     // move AS:2 one line up:
-                    new NonRemappableRegion(spanPreRemap2, lineDelta: -1, isExceptionRegion: false),
+                    new NonRemappableRegion(spanPreRemap2, spanPreRemap2.AddLineDelta(-1), isExceptionRegion: false),
                     // move ER:2.0 and ER:2.1 two lines down:
-                    new NonRemappableRegion(erPreRemap20, lineDelta: +2, isExceptionRegion: true),
-                    new NonRemappableRegion(erPreRemap21, lineDelta: +2, isExceptionRegion: true)) },
+                    new NonRemappableRegion(erPreRemap20, erPreRemap20.AddLineDelta(+2), isExceptionRegion: true),
+                    new NonRemappableRegion(erPreRemap21, erPreRemap21.AddLineDelta(+2), isExceptionRegion: true)) },
                 { new ManagedMethodId(module1, 0x06000004, 1), ImmutableArray.Create(
                     // move AS:3 one line down:
-                    new NonRemappableRegion(spanPreRemap3, lineDelta: +1, isExceptionRegion: false),
+                    new NonRemappableRegion(spanPreRemap3, spanPreRemap3.AddLineDelta(+1), isExceptionRegion: false),
                     // move ER:3.0 and ER:3.1 one line down:
-                    new NonRemappableRegion(erPreRemap30, lineDelta: +1, isExceptionRegion: true),
-                    new NonRemappableRegion(erPreRemap31, lineDelta: +1, isExceptionRegion: true)) }
+                    new NonRemappableRegion(erPreRemap30, erPreRemap30.AddLineDelta(+1), isExceptionRegion: true),
+                    new NonRemappableRegion(erPreRemap31, erPreRemap31.AddLineDelta(+1), isExceptionRegion: true)) }
             }.ToImmutableDictionary();
 
             using var workspace = new TestWorkspace(composition: s_composition);
-            var solution = AddDefaultTestSolution(workspace, new[] { markedSourceV2 });
+            var solution = await AddDefaultTestSolutionAsync(workspace, new[] { markedSourceV2 });
             var project = solution.Projects.Single();
             var document = project.Documents.Single();
 
@@ -538,7 +540,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             // Exception Regions
 
-            var analyzer = solution.GetProject(project.Id).LanguageServices.GetRequiredService<IEditAndContinueAnalyzer>();
+            var analyzer = solution.GetProject(project.Id).Services.GetRequiredService<IEditAndContinueAnalyzer>();
             var oldActiveStatements = await baseActiveStatementMap.GetOldActiveStatementsAsync(analyzer, document, CancellationToken.None).ConfigureAwait(false);
 
             // Note that the spans correspond to the base snapshot (V2). 
@@ -582,16 +584,16 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
             // Note: Since no method have been remapped yet all the following spans are in their pre-remap locations: 
             AssertEx.Equal(new[]
             {
-                $"0x06000001 v2 | AS {document.FilePath}: (6,18)-(6,22) δ=0",
-                $"0x06000002 v2 | ER {document.FilePath}: (18,16)-(21,9) δ=-1",
-                $"0x06000002 v2 | AS {document.FilePath}: (20,18)-(20,22) δ=-1",
-                $"0x06000003 v1 | AS {document.FilePath}: (30,22)-(30,26) δ=-1", // AS:2 moved -1 in first edit, 0 in second
-                $"0x06000003 v1 | ER {document.FilePath}: (32,20)-(34,13) δ=2",  // ER:2.0 moved +2 in first edit, 0 in second
-                $"0x06000003 v1 | ER {document.FilePath}: (36,16)-(38,9) δ=2",   // ER:2.0 moved +2 in first edit, 0 in second
-                $"0x06000004 v1 | ER {document.FilePath}: (50,20)-(53,13) δ=3",  // ER:3.0 moved +1 in first edit, +2 in second              
-                $"0x06000004 v1 | AS {document.FilePath}: (52,22)-(52,26) δ=3",  // AS:3 moved +1 in first edit, +2 in second
-                $"0x06000004 v1 | ER {document.FilePath}: (55,16)-(57,9) δ=3",   // ER:3.1 moved +1 in first edit, +2 in second     
-            }, nonRemappableRegions.OrderBy(r => r.Region.Span.Span.Start.Line).Select(r => $"{r.Method.GetDebuggerDisplay()} | {r.Region.GetDebuggerDisplay()}"));
+                $"0x06000001 v2 | AS {document.FilePath}: (6,18)-(6,22) => (6,18)-(6,22)",
+                $"0x06000002 v2 | ER {document.FilePath}: (18,16)-(21,9) => (17,16)-(20,9)",
+                $"0x06000002 v2 | AS {document.FilePath}: (20,18)-(20,22) => (19,18)-(19,22)",
+                $"0x06000003 v1 | AS {document.FilePath}: (30,22)-(30,26) => (29,22)-(29,26)",  // AS:2 moved -1 in first edit, 0 in second
+                $"0x06000003 v1 | ER {document.FilePath}: (32,20)-(34,13) => (34,20)-(36,13)",  // ER:2.0 moved +2 in first edit, 0 in second
+                $"0x06000003 v1 | ER {document.FilePath}: (36,16)-(38,9) => (38,16)-(40,9)",    // ER:2.0 moved +2 in first edit, 0 in second
+                $"0x06000004 v1 | ER {document.FilePath}: (50,20)-(53,13) => (53,20)-(56,13)",  // ER:3.0 moved +1 in first edit, +2 in second              
+                $"0x06000004 v1 | AS {document.FilePath}: (52,22)-(52,26) => (55,22)-(55,26)",  // AS:3 moved +1 in first edit, +2 in second
+                $"0x06000004 v1 | ER {document.FilePath}: (55,16)-(57,9) => (58,16)-(60,9)",    // ER:3.1 moved +1 in first edit, +2 in second     
+            }, nonRemappableRegions.OrderBy(r => r.Region.OldSpan.Span.Start.Line).Select(r => $"{r.Method.GetDebuggerDisplay()} | {r.Region.GetDebuggerDisplay()}"));
 
             AssertEx.Equal(new[]
             {
@@ -651,7 +653,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
                 });
 
             using var workspace = new TestWorkspace(composition: s_composition);
-            var solution = AddDefaultTestSolution(workspace, markedSources);
+            var solution = await AddDefaultTestSolutionAsync(workspace, markedSources);
             var project = solution.Projects.Single();
             var document = project.Documents.Single();
 
@@ -684,7 +686,7 @@ namespace Microsoft.CodeAnalysis.EditAndContinue.UnitTests
 
             // Exception Regions
 
-            var analyzer = solution.GetProject(project.Id).LanguageServices.GetRequiredService<IEditAndContinueAnalyzer>();
+            var analyzer = solution.GetProject(project.Id).Services.GetRequiredService<IEditAndContinueAnalyzer>();
             var oldActiveStatements = await baseActiveStatementMap.GetOldActiveStatementsAsync(analyzer, document, CancellationToken.None).ConfigureAwait(false);
 
             AssertEx.Equal(new[]
