@@ -4,12 +4,15 @@
 
 using System;
 using System.Collections.Generic;
+using System.Collections.Immutable;
 using System.IO;
 using System.Linq;
 using System.Security;
 using System.Threading;
 using System.Threading.Tasks;
-using Microsoft.CodeAnalysis.Editor.CSharp.DecompiledSource;
+using Microsoft.CodeAnalysis.CodeStyle;
+using Microsoft.CodeAnalysis.CSharp.CodeGeneration;
+using Microsoft.CodeAnalysis.CSharp.DecompiledSource;
 using Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces;
 using Microsoft.CodeAnalysis.MetadataAsSource;
 using Microsoft.CodeAnalysis.Shared.Extensions;
@@ -36,7 +39,8 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                 bool includeXmlDocComments = false,
                 string? sourceWithSymbolReference = null,
                 string? languageVersion = null,
-                string? metadataLanguageVersion = null)
+                string? metadataLanguageVersion = null,
+                string? metadataCommonReferences = null)
             {
                 projectLanguage ??= LanguageNames.CSharp;
                 metadataSources ??= SpecializedCollections.EmptyEnumerable<string>();
@@ -46,7 +50,7 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
 
                 var workspace = CreateWorkspace(
                     projectLanguage, metadataSources, includeXmlDocComments,
-                    sourceWithSymbolReference, languageVersion, metadataLanguageVersion);
+                    sourceWithSymbolReference, languageVersion, metadataLanguageVersion, metadataCommonReferences);
 
                 return new TestContext(workspace);
             }
@@ -67,16 +71,20 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                 get { return this.CurrentSolution.Projects.First(); }
             }
 
-            public Task<MetadataAsSourceFile> GenerateSourceAsync(ISymbol symbol, Project? project = null, bool allowDecompilation = false)
+            public Task<MetadataAsSourceFile> GenerateSourceAsync(ISymbol symbol, Project? project = null, bool signaturesOnly = true)
             {
                 project ??= this.DefaultProject;
                 Contract.ThrowIfNull(symbol);
 
                 // Generate and hold onto the result so it can be disposed of with this context
-                return _metadataAsSourceService.GetGeneratedFileAsync(project, symbol, allowDecompilation);
+                return _metadataAsSourceService.GetGeneratedFileAsync(Workspace, project, symbol, signaturesOnly, MetadataAsSourceOptions.GetDefault(project.Services));
             }
 
-            public async Task<MetadataAsSourceFile> GenerateSourceAsync(string? symbolMetadataName = null, Project? project = null, bool allowDecompilation = false)
+            public async Task<MetadataAsSourceFile> GenerateSourceAsync(
+                string? symbolMetadataName = null,
+                Project? project = null,
+                bool signaturesOnly = true,
+                bool fileScopedNamespaces = false)
             {
                 symbolMetadataName ??= AbstractMetadataAsSourceTests.DefaultSymbolMetadataName;
                 project ??= this.DefaultProject;
@@ -88,7 +96,7 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                 var symbol = await ResolveSymbolAsync(symbolMetadataName, compilation);
                 Contract.ThrowIfNull(symbol);
 
-                if (allowDecompilation)
+                if (!signaturesOnly)
                 {
                     foreach (var reference in compilation.References)
                     {
@@ -110,8 +118,24 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                     }
                 }
 
+                var options = MetadataAsSourceOptions.GetDefault(project.Services);
+
+                if (fileScopedNamespaces)
+                {
+                    options = options with
+                    {
+                        GenerationOptions = options.GenerationOptions with
+                        {
+                            GenerationOptions = new CSharpCodeGenerationOptions
+                            {
+                                NamespaceDeclarations = new CodeStyleOption2<NamespaceDeclarationPreference>(NamespaceDeclarationPreference.FileScoped, NotificationOption2.Silent)
+                            }
+                        }
+                    };
+                }
+
                 // Generate and hold onto the result so it can be disposed of with this context
-                var result = await _metadataAsSourceService.GetGeneratedFileAsync(project, symbol, allowDecompilation);
+                var result = await _metadataAsSourceService.GetGeneratedFileAsync(Workspace, project, symbol, signaturesOnly, options);
 
                 return result;
             }
@@ -129,9 +153,9 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                 Assert.Equal(expectedSpan.End, actualSpan.End);
             }
 
-            public async Task GenerateAndVerifySourceAsync(string symbolMetadataName, string expected, Project? project = null, bool allowDecompilation = false)
+            public async Task GenerateAndVerifySourceAsync(string symbolMetadataName, string expected, Project? project = null, bool signaturesOnly = true, bool fileScopedNamespaces = false)
             {
-                var result = await GenerateSourceAsync(symbolMetadataName, project, allowDecompilation);
+                var result = await GenerateSourceAsync(symbolMetadataName, project, signaturesOnly, fileScopedNamespaces);
                 VerifyResult(result, expected);
             }
 
@@ -192,8 +216,8 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                             --lastDotIndex;
                         }
 
-                        var memberSymbolName = symbolMetadataName.Substring(lastDotIndex + 1);
-                        var namedTypeName = symbolMetadataName.Substring(0, lastDotIndex);
+                        var memberSymbolName = symbolMetadataName[(lastDotIndex + 1)..];
+                        var namedTypeName = symbolMetadataName[..lastDotIndex];
 
                         namedTypeSymbol = assemblySymbol.GetTypeByMetadataName(namedTypeName);
                         if (namedTypeSymbol != null)
@@ -236,7 +260,8 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
                 bool includeXmlDocComments,
                 string? sourceWithSymbolReference,
                 string? languageVersion,
-                string? metadataLanguageVersion)
+                string? metadataLanguageVersion,
+                string? metadataCommonReferences)
             {
                 var languageVersionAttribute = languageVersion is null ? "" : $@" LanguageVersion=""{languageVersion}""";
 
@@ -250,10 +275,11 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
 
                 foreach (var source in metadataSources)
                 {
+                    var commonReferencesAttributeName = metadataCommonReferences ?? "CommonReferences";
                     var metadataLanguage = DeduceLanguageString(source);
                     var metadataLanguageVersionAttribute = metadataLanguageVersion is null ? "" : $@" LanguageVersion=""{metadataLanguageVersion}""";
                     xmlString = string.Concat(xmlString, $@"
-        <MetadataReferenceFromSource Language=""{metadataLanguage}"" CommonReferences=""true"" {metadataLanguageVersionAttribute} IncludeXmlDocComments=""{includeXmlDocComments}"">
+        <MetadataReferenceFromSource Language=""{metadataLanguage}"" {commonReferencesAttributeName}= ""true"" {metadataLanguageVersionAttribute} IncludeXmlDocComments=""{includeXmlDocComments}"">
             <Document FilePath=""MetadataDocument"">
 {SecurityElement.Escape(source)}
             </Document>
@@ -273,7 +299,13 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.MetadataAsSource
     </Project>
 </Workspace>");
 
-                return TestWorkspace.Create(xmlString);
+                // We construct our own composition here because we only want the decompilation metadata as source provider
+                // to be available.
+                var composition = EditorTestCompositions.EditorFeatures
+                    .WithExcludedPartTypes(ImmutableHashSet.Create(typeof(IMetadataAsSourceFileProvider)))
+                    .AddParts(typeof(DecompilationMetadataAsSourceFileProvider));
+
+                return TestWorkspace.Create(xmlString, composition: composition);
             }
 
             internal Document GetDocument(MetadataAsSourceFile file)
