@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System;
 using System.Collections.Generic;
 using System.Collections.Immutable;
@@ -12,13 +10,13 @@ using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Classification;
-using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.Remote.Testing;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Test.Utilities;
 using Microsoft.CodeAnalysis.Text;
 using Roslyn.Test.Utilities;
 using Roslyn.Utilities;
+using Xunit;
 
 namespace Microsoft.CodeAnalysis.Editor.UnitTests.Classification
 {
@@ -27,7 +25,7 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Classification
     {
         protected AbstractClassifierTests() { }
 
-        protected abstract Task<ImmutableArray<ClassifiedSpan>> GetClassificationSpansAsync(string text, TextSpan span, ParseOptions parseOptions, TestHost testHost);
+        protected abstract Task<ImmutableArray<ClassifiedSpan>> GetClassificationSpansAsync(string text, TextSpan span, ParseOptions? parseOptions, TestHost testHost);
 
         protected abstract string WrapInClass(string className, string code);
         protected abstract string WrapInExpression(string code);
@@ -40,12 +38,31 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Classification
            string code,
            string allCode,
            TestHost testHost,
-           ParseOptions parseOptions,
+           ParseOptions? parseOptions,
            params FormattedClassification[] expected)
         {
-            var start = allCode.IndexOf(code, StringComparison.Ordinal);
-            var length = code.Length;
-            var span = new TextSpan(start, length);
+            TextSpan span;
+            if (code != allCode)
+            {
+                var start = allCode.IndexOf(code, StringComparison.Ordinal);
+                var length = code.Length;
+                span = new TextSpan(start, length);
+            }
+            else
+            {
+                MarkupTestFile.GetSpans(allCode, out var rewrittenCode, out var spans);
+                Assert.True(spans.Length < 2);
+                if (spans.Length == 1)
+                {
+                    allCode = rewrittenCode;
+                    span = spans.Single();
+                }
+                else
+                {
+                    span = new TextSpan(0, allCode.Length);
+                }
+            }
+
             var actual = await GetClassificationSpansAsync(allCode, span, parseOptions, testHost);
 
             var actualOrdered = actual.OrderBy((t1, t2) => t1.TextSpan.Start - t2.TextSpan.Start);
@@ -84,6 +101,15 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Classification
             await DefaultTestAsync(code, code, testHost, expected);
         }
 
+        protected async Task TestAsync(
+            string code,
+            TestHost testHost,
+            ParseOptions? parseOptions,
+            params FormattedClassification[] expected)
+        {
+            await TestAsync(code, code, testHost, parseOptions, expected);
+        }
+
         protected async Task TestAsync(
             string code,
             string allCode,
@@ -245,21 +271,22 @@ namespace Microsoft.CodeAnalysis.Editor.UnitTests.Classification
         protected static async Task<ImmutableArray<ClassifiedSpan>> GetSemanticClassificationsAsync(Document document, TextSpan span)
         {
             var service = document.GetRequiredLanguageService<IClassificationService>();
-            var options = ClassificationOptions.From(document.Project);
+            var options = ClassificationOptions.Default;
 
-            using var _ = ArrayBuilder<ClassifiedSpan>.GetInstance(out var result);
+            using var _ = Classifier.GetPooledList(out var result);
             await service.AddSemanticClassificationsAsync(document, span, options, result, CancellationToken.None);
-            return result.ToImmutable();
+            await service.AddEmbeddedLanguageClassificationsAsync(document, span, options, result, CancellationToken.None);
+            return result.ToImmutableArray();
         }
 
         protected static async Task<ImmutableArray<ClassifiedSpan>> GetSyntacticClassificationsAsync(Document document, TextSpan span)
         {
-            var root = await document.GetSyntaxRootAsync();
-            var service = document.GetLanguageService<ISyntaxClassificationService>();
+            var root = await document.GetRequiredSyntaxRootAsync(CancellationToken.None);
+            var service = document.GetRequiredLanguageService<ISyntaxClassificationService>();
 
-            using var _ = ArrayBuilder<ClassifiedSpan>.GetInstance(out var results);
+            using var _ = Classifier.GetPooledList(out var results);
             service.AddSyntacticClassifications(root, span, results, CancellationToken.None);
-            return results.ToImmutable();
+            return results.ToImmutableArray();
         }
 
         protected static async Task<ImmutableArray<ClassifiedSpan>> GetAllClassificationsAsync(Document document, TextSpan span)
