@@ -6,18 +6,20 @@
 
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CSharp;
+using Microsoft.CodeAnalysis.Test.Utilities;
 using Microsoft.VisualStudio.Composition;
 
 namespace Microsoft.CodeAnalysis.Editor.UnitTests.Workspaces
 {
     public class CSharpTestWorkspaceFixture : TestWorkspaceFixture
     {
-        protected override TestWorkspace CreateWorkspace(ExportProvider exportProvider = null)
+        protected override TestWorkspace CreateWorkspace(TestComposition composition = null)
         {
-            return TestWorkspace.CreateCSharp2(
-                new string[] { string.Empty, },
-                new CSharpParseOptions[] { new CSharpParseOptions(kind: SourceCodeKind.Regular), },
-                exportProvider: exportProvider);
+            return TestWorkspace.CreateWithSingleEmptySourceFile(
+                LanguageNames.CSharp,
+                compilationOptions: null,
+                parseOptions: new CSharpParseOptions(kind: SourceCodeKind.Regular),
+                composition: composition);
         }
     }
 }
