@@ -7,13 +7,16 @@ using System.Composition;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.BraceCompletion;
+using Microsoft.CodeAnalysis.CSharp.LanguageService;
 using Microsoft.CodeAnalysis.Host.Mef;
+using Microsoft.CodeAnalysis.LanguageService;
 using Roslyn.Utilities;
 
 namespace Microsoft.CodeAnalysis.CSharp.BraceCompletion
 {
+
     [Export(LanguageNames.CSharp, typeof(IBraceCompletionService)), Shared]
-    internal class CharLiteralBraceCompletionService : AbstractBraceCompletionService
+    internal class CharLiteralBraceCompletionService : AbstractCSharpBraceCompletionService
     {
         [ImportingConstructor]
         [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
@@ -25,8 +28,8 @@ namespace Microsoft.CodeAnalysis.CSharp.BraceCompletion
 
         protected override char ClosingBrace => SingleQuote.CloseCharacter;
 
-        public override Task<bool> AllowOverTypeAsync(BraceCompletionContext braceCompletionContext, CancellationToken cancellationToken)
-            => AllowOverTypeWithValidClosingTokenAsync(braceCompletionContext, cancellationToken);
+        public override bool AllowOverType(BraceCompletionContext braceCompletionContext, CancellationToken cancellationToken)
+            => AllowOverTypeWithValidClosingToken(braceCompletionContext);
 
         protected override bool IsValidOpeningBraceToken(SyntaxToken token) => token.IsKind(SyntaxKind.CharacterLiteralToken);
 
