@@ -10,6 +10,8 @@ using System.Diagnostics.CodeAnalysis;
 using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
+using Microsoft.CodeAnalysis.CodeActions;
+using Microsoft.CodeAnalysis.CodeFixes;
 using Microsoft.CodeAnalysis.CodeRefactorings;
 using Microsoft.CodeAnalysis.CSharp.CodeGeneration;
 using Microsoft.CodeAnalysis.CSharp.Extensions;
@@ -20,6 +22,7 @@ using Microsoft.CodeAnalysis.Host.Mef;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Simplification;
+using Microsoft.CodeAnalysis.Text;
 using Roslyn.Utilities;
 
 namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
@@ -28,36 +31,30 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
     using static SyntaxKind;
 
     [ExportCodeRefactoringProvider(LanguageNames.CSharp, Name = PredefinedCodeRefactoringProviderNames.UseRecursivePatterns), Shared]
-    internal sealed class UseRecursivePatternsCodeRefactoringProvider : CodeRefactoringProvider
+    internal sealed class UseRecursivePatternsCodeRefactoringProvider : SyntaxEditorBasedCodeRefactoringProvider
     {
         private static readonly PatternSyntax s_trueConstantPattern = ConstantPattern(LiteralExpression(TrueLiteralExpression));
         private static readonly PatternSyntax s_falseConstantPattern = ConstantPattern(LiteralExpression(FalseLiteralExpression));
 
-        private static readonly Func<IdentifierNameSyntax, SemanticModel, bool> s_canConvertToSubpattern =
-            (name, model) => model.GetSymbolInfo(name).Symbol is
-            {
-                IsStatic: false,
-                Kind: SymbolKind.Property or SymbolKind.Field,
-                ContainingType: not { SpecialType: SpecialType.System_Nullable_T }
-            };
-
         [ImportingConstructor]
-        [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+        [SuppressMessage("RoslynDiagnosticsReliability", "RS0033:Importing constructor should be [Obsolete]", Justification = "Used in test code: https://github.com/dotnet/roslyn/issues/42814")]
         public UseRecursivePatternsCodeRefactoringProvider()
         {
         }
 
+        protected override ImmutableArray<FixAllScope> SupportedFixAllScopes => AllFixAllScopes;
+
         public override async Task ComputeRefactoringsAsync(CodeRefactoringContext context)
         {
             var (document, textSpan, cancellationToken) = context;
-            if (document.Project.Solution.Workspace.Kind == WorkspaceKind.MiscellaneousFiles)
+            if (document.Project.Solution.WorkspaceKind == WorkspaceKind.MiscellaneousFiles)
                 return;
 
             if (textSpan.Length > 0)
                 return;
 
             var root = await document.GetRequiredSyntaxRootAsync(cancellationToken).ConfigureAwait(false);
-            if (((CSharpParseOptions)root.SyntaxTree.Options).LanguageVersion < LanguageVersion.CSharp9)
+            if (root.SyntaxTree.Options.LanguageVersion() < LanguageVersion.CSharp9)
                 return;
 
             var model = await document.GetRequiredSemanticModelAsync(cancellationToken).ConfigureAwait(false);
@@ -67,7 +64,7 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
                 return;
 
             context.RegisterRefactoring(
-                new MyCodeAction(
+                CodeAction.Create(
                     CSharpFeaturesResources.Use_recursive_patterns,
                     _ => Task.FromResult(document.WithSyntaxRoot(replacementFunc(root))),
                     nameof(CSharpFeaturesResources.Use_recursive_patterns)));
@@ -84,6 +81,17 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
                 _ => null
             };
 
+        private static bool IsFixableNode(SyntaxNode node)
+            => node switch
+            {
+                BinaryExpressionSyntax(LogicalAndExpression) => true,
+                CasePatternSwitchLabelSyntax { WhenClause: { } whenClause } => true,
+                SwitchExpressionArmSyntax { WhenClause: { } whenClause } => true,
+                WhenClauseSyntax { Parent: CasePatternSwitchLabelSyntax } => true,
+                WhenClauseSyntax { Parent: SwitchExpressionArmSyntax } => true,
+                _ => false
+            };
+
         private static Func<SyntaxNode, SyntaxNode>? CombineLogicalAndOperands(BinaryExpressionSyntax logicalAnd, SemanticModel model)
         {
             if (TryDetermineReceiver(logicalAnd.Left, model) is not var (leftReceiver, leftTarget, leftFlipped) ||
@@ -115,8 +123,6 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
                 {
                     var leftSubpattern = CreateSubpattern(leftNames, CreatePattern(leftReceiver, leftTarget, leftFlipped));
                     var rightSubpattern = CreateSubpattern(rightNames, CreatePattern(rightReceiver, rightTarget, rightFlipped));
-                    // If the common receiver is null, it's an implicit `this` reference in source.
-                    // For instance, `prop == 1 && field == 2` would be converted to `this is { prop: 1, field: 2 }`
                     var replacement = IsPatternExpression(commonReceiver, RecursivePattern(leftSubpattern, rightSubpattern));
                     return root.ReplaceNode(logicalAnd, AdjustBinaryExpressionOperands(logicalAnd, replacement));
                 };
@@ -237,12 +243,7 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
             {
                 if (left is null || right is null)
                     return left ?? right;
-                var leftSubpatterns = left.Subpatterns.GetWithSeparators();
-                if (leftSubpatterns.Any() && !leftSubpatterns.Last().IsToken)
-                    leftSubpatterns = leftSubpatterns.Add(Token(CommaToken));
-                var rightSubpatterns = right.Subpatterns.GetWithSeparators();
-                var list = new SyntaxNodeOrTokenList(leftSubpatterns.Concat(rightSubpatterns));
-                return left.WithSubpatterns(SeparatedList<SubpatternSyntax>(list));
+                return left.WithSubpatterns(left.Subpatterns.AddRange(right.Subpatterns));
             }
         }
 
@@ -295,13 +296,14 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
                 .Where(d => d.Identifier.ValueText == identifierName.Identifier.ValueText)
                 .FirstOrDefault();
 
-            if (designation is not { Parent: PatternSyntax containingPattern })
+            // Excluding list patterns because those cannot be combined with a recursive pattern.
+            if (designation is not { Parent: PatternSyntax(not SyntaxKind.ListPattern) containingPattern })
                 return null;
 
             // Only the following patterns can directly contain a variable designation.
             // Note: While a parenthesized designation can also contain other variables,
             // it is not a pattern, so it would not get past the PatternSyntax test above.
-            Debug.Assert(containingPattern.IsKind(SyntaxKind.VarPattern, SyntaxKind.DeclarationPattern, SyntaxKind.RecursivePattern));
+            Debug.Assert(containingPattern.Kind() is SyntaxKind.VarPattern or SyntaxKind.DeclarationPattern or SyntaxKind.RecursivePattern);
             return (containingPattern, names.ToImmutableOrNull());
         }
 
@@ -360,10 +362,21 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
         private static SubpatternSyntax CreateSubpattern(ImmutableArray<IdentifierNameSyntax> names, PatternSyntax pattern)
         {
             Debug.Assert(!names.IsDefaultOrEmpty);
-            var subpattern = Subpattern(names[0], pattern);
-            for (var i = 1; i < names.Length; i++)
-                subpattern = Subpattern(names[i], RecursivePattern(subpattern));
-            return subpattern;
+
+            if (names.Length > 1 && names[0].SyntaxTree.Options.LanguageVersion() >= LanguageVersion.CSharp10)
+            {
+                ExpressionSyntax expression = names[^1];
+                for (var i = names.Length - 2; i >= 0; i--)
+                    expression = MemberAccessExpression(SimpleMemberAccessExpression, expression, names[i]);
+                return SyntaxFactory.Subpattern(ExpressionColon(expression, Token(ColonToken)), pattern);
+            }
+            else
+            {
+                var subpattern = Subpattern(names[0], pattern);
+                for (var i = 1; i < names.Length; i++)
+                    subpattern = Subpattern(names[i], RecursivePattern(subpattern));
+                return subpattern;
+            }
         }
 
         private static SubpatternSyntax Subpattern(IdentifierNameSyntax name, PatternSyntax pattern)
@@ -409,6 +422,8 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
                 commonReceiver = GetInnermostReceiver(left, lastName, static (identifierName, lastName) => identifierName != lastName);
             }
 
+            // If the common receiver is null, it's an implicit `this` reference in source.
+            // For instance, `prop == 1 && field == 2` would be converted to `this is { prop: 1, field: 2 }`
             return (commonReceiver ?? ThisExpression(), leftNames.ToImmutable(), rightNames.ToImmutable());
 
             static bool TryGetInnermostReceiver(ExpressionSyntax node, ArrayBuilder<IdentifierNameSyntax> builder, [NotNullWhen(true)] out ExpressionSyntax? receiver, SemanticModel model)
@@ -438,7 +453,19 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
         }
 
         private static ExpressionSyntax? GetInnermostReceiver(ExpressionSyntax node, ArrayBuilder<IdentifierNameSyntax> builder, SemanticModel model)
-            => GetInnermostReceiver(node, model, s_canConvertToSubpattern, builder);
+        {
+            return GetInnermostReceiver(node, model, CanConvertToSubpattern, builder);
+
+            static bool CanConvertToSubpattern(IdentifierNameSyntax name, SemanticModel model)
+            {
+                return model.GetSymbolInfo(name).Symbol is
+                {
+                    IsStatic: false,
+                    Kind: SymbolKind.Property or SymbolKind.Field,
+                    ContainingType: not { SpecialType: SpecialType.System_Nullable_T }
+                };
+            }
+        }
 
         private static ExpressionSyntax? GetInnermostReceiver<TArg>(
             ExpressionSyntax node, TArg arg,
@@ -492,12 +519,47 @@ namespace Microsoft.CodeAnalysis.CSharp.CodeRefactorings.UseRecursivePatterns
             }
         }
 
-        private sealed class MyCodeAction : CodeActions.CodeAction.DocumentChangeAction
+        protected override async Task FixAllAsync(
+            Document document,
+            ImmutableArray<TextSpan> fixAllSpans,
+            SyntaxEditor editor,
+            CodeActionOptionsProvider optionsProvider,
+            string? equivalenceKey,
+            CancellationToken cancellationToken)
         {
-            public MyCodeAction(string title, Func<CancellationToken, Task<Document>> createChangedDocument, string equivalenceKey)
-                : base(title, createChangedDocument, equivalenceKey)
+            // Get all the descendant nodes to refactor.
+            // NOTE: We need to realize the nodes with 'ToArray' call here
+            // to ensure we strongly hold onto the nodes so that 'TrackNodes'
+            // invoked below, which does tracking based off a ConditionalWeakTable,
+            // tracks the nodes for the entire duration of this method.
+            var nodes = editor.OriginalRoot.DescendantNodes().Where(IsFixableNode).ToArray();
+
+            // We're going to be continually editing this tree. Track all the nodes we
+            // care about so we can find them across each edit.
+            document = document.WithSyntaxRoot(editor.OriginalRoot.TrackNodes(nodes));
+
+            // Process all nodes to refactor in reverse to ensure nested nodes
+            // are processed before the outer nodes to refactor.
+            foreach (var originalNode in nodes.Reverse())
             {
+                // Only process nodes fully within a fixAllSpan
+                if (!fixAllSpans.Any(fixAllSpan => fixAllSpan.Contains(originalNode.Span)))
+                    continue;
+
+                // Get current root, current node to refactor and semantic model.
+                var root = await document.GetRequiredSyntaxRootAsync(cancellationToken).ConfigureAwait(false);
+                var currentNode = root.GetCurrentNodes(originalNode).SingleOrDefault();
+                var semanticModel = await document.GetRequiredSemanticModelAsync(cancellationToken).ConfigureAwait(false);
+
+                var replacementFunc = GetReplacementFunc(currentNode, semanticModel);
+                if (replacementFunc == null)
+                    continue;
+
+                document = document.WithSyntaxRoot(replacementFunc(root));
             }
+
+            var updatedRoot = await document.GetRequiredSyntaxRootAsync(cancellationToken).ConfigureAwait(false);
+            editor.ReplaceNode(editor.OriginalRoot, updatedRoot);
         }
     }
 }
