@@ -13,8 +13,10 @@ using System.Threading.Tasks;
 using Microsoft.CodeAnalysis;
 using Microsoft.CodeAnalysis.CodeActions;
 using Microsoft.CodeAnalysis.CodeGeneration;
+using Microsoft.CodeAnalysis.CSharp.CodeGeneration;
 using Microsoft.CodeAnalysis.CSharp.CodeStyle;
 using Microsoft.CodeAnalysis.CSharp.Extensions;
+using Microsoft.CodeAnalysis.CSharp.LanguageService;
 using Microsoft.CodeAnalysis.CSharp.Syntax;
 using Microsoft.CodeAnalysis.Diagnostics.Analyzers.NamingStyles;
 using Microsoft.CodeAnalysis.Editing;
@@ -26,13 +28,12 @@ using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Simplification;
 using Roslyn.Utilities;
-using static Microsoft.CodeAnalysis.Diagnostics.Analyzers.NamingStyles.SymbolSpecification;
 
 namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
 {
     internal partial class CSharpMethodExtractor
     {
-        private abstract partial class CSharpCodeGenerator : CodeGenerator<StatementSyntax, ExpressionSyntax, SyntaxNode>
+        private abstract partial class CSharpCodeGenerator : CodeGenerator<StatementSyntax, ExpressionSyntax, SyntaxNode, CSharpCodeGenerationOptions>
         {
             private readonly SyntaxToken _methodName;
 
@@ -43,7 +44,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 InsertionPoint insertionPoint,
                 SelectionResult selectionResult,
                 AnalyzerResult analyzerResult,
-                OptionSet options,
+                CSharpCodeGenerationOptions options,
                 bool localFunction,
                 CancellationToken cancellationToken)
             {
@@ -55,7 +56,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 InsertionPoint insertionPoint,
                 SelectionResult selectionResult,
                 AnalyzerResult analyzerResult,
-                OptionSet options,
+                CSharpCodeGenerationOptions options,
                 bool localFunction)
             {
                 if (ExpressionCodeGenerator.IsExtractMethodOnExpression(selectionResult))
@@ -80,7 +81,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 InsertionPoint insertionPoint,
                 SelectionResult selectionResult,
                 AnalyzerResult analyzerResult,
-                OptionSet options,
+                CSharpCodeGenerationOptions options,
                 bool localFunction)
                 : base(insertionPoint, selectionResult, analyzerResult, options, localFunction)
             {
@@ -91,9 +92,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
             }
 
             private CSharpSelectionResult CSharpSelectionResult
-            {
-                get { return (CSharpSelectionResult)SelectionResult; }
-            }
+                => (CSharpSelectionResult)SelectionResult;
 
             protected override SyntaxNode GetPreviousMember(SemanticDocument document)
             {
@@ -131,7 +130,8 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 var firstStatementToRemove = GetFirstStatementOrInitializerSelectedAtCallSite();
                 var lastStatementToRemove = GetLastStatementOrInitializerSelectedAtCallSite();
 
-                Contract.ThrowIfFalse(firstStatementToRemove.Parent == lastStatementToRemove.Parent);
+                Contract.ThrowIfFalse(firstStatementToRemove.Parent == lastStatementToRemove.Parent
+                    || CSharpSyntaxFacts.Instance.AreStatementsInSameContainer(firstStatementToRemove, lastStatementToRemove));
 
                 var statementsToInsert = await CreateStatementsOrInitializerToInsertAtCallSiteAsync(cancellationToken).ConfigureAwait(false);
 
@@ -175,7 +175,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
             }
 
             protected override bool ShouldLocalFunctionCaptureParameter(SyntaxNode node)
-            => ((CSharpParseOptions)node.SyntaxTree.Options).LanguageVersion < LanguageVersion.CSharp8;
+            => node.SyntaxTree.Options.LanguageVersion() < LanguageVersion.CSharp8;
 
             private static bool IsExpressionBodiedMember(SyntaxNode node)
                 => node is MemberDeclarationSyntax member && member.GetExpressionBody() != null;
@@ -186,7 +186,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
             private SimpleNameSyntax CreateMethodNameForInvocation()
             {
                 return AnalyzerResult.MethodTypeParametersInDeclaration.Count == 0
-                    ? (SimpleNameSyntax)SyntaxFactory.IdentifierName(_methodName)
+                    ? SyntaxFactory.IdentifierName(_methodName)
                     : SyntaxFactory.GenericName(_methodName, SyntaxFactory.TypeArgumentList(CreateMethodCallTypeVariables()));
             }
 
@@ -228,9 +228,17 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 var isReadOnly = AnalyzerResult.ShouldBeReadOnly;
 
                 // Static local functions are only supported in C# 8.0 and later
-                var languageVersion = ((CSharpParseOptions)SemanticDocument.SyntaxTree.Options).LanguageVersion;
+                var languageVersion = SemanticDocument.SyntaxTree.Options.LanguageVersion();
 
-                if (LocalFunction && (!Options.GetOption(CSharpCodeStyleOptions.PreferStaticLocalFunction).Value || languageVersion < LanguageVersion.CSharp8))
+                if (LocalFunction && (!Options.PreferStaticLocalFunction.Value || languageVersion < LanguageVersion.CSharp8))
+                {
+                    isStatic = false;
+                }
+
+                // UseInstanceMember will be false for interface members, but extracting a non-static
+                // member to a static member has a very different meaning for interfaces so we need
+                // an extra check here.
+                if (!LocalFunction && IsNonStaticInterfaceMember())
                 {
                     isStatic = false;
                 }
@@ -242,11 +250,27 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                     isReadOnly: isReadOnly);
             }
 
+            private bool IsNonStaticInterfaceMember()
+            {
+                var typeDecl = SelectionResult.GetContainingScopeOf<BaseTypeDeclarationSyntax>();
+                if (typeDecl is null)
+                    return false;
+
+                if (!typeDecl.IsKind(SyntaxKind.InterfaceDeclaration))
+                    return false;
+
+                var memberDecl = SelectionResult.GetContainingScopeOf<MemberDeclarationSyntax>();
+                if (memberDecl is null)
+                    return false;
+
+                return !memberDecl.Modifiers.Any(SyntaxKind.StaticKeyword);
+            }
+
             private static SyntaxKind GetParameterRefSyntaxKind(ParameterBehavior parameterBehavior)
             {
-                return parameterBehavior == ParameterBehavior.Ref ?
-                        SyntaxKind.RefKeyword :
-                            parameterBehavior == ParameterBehavior.Out ?
+                return parameterBehavior == ParameterBehavior.Ref
+                        ? SyntaxKind.RefKeyword
+                            : parameterBehavior == ParameterBehavior.Out ?
                                 SyntaxKind.OutKeyword : SyntaxKind.None;
             }
 
@@ -439,7 +463,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 var replacements = new Dictionary<SyntaxNode, SyntaxNode>();
 
                 var declarations = statement.DescendantNodes()
-                    .Where(n => n.IsKind(SyntaxKind.DeclarationExpression, SyntaxKind.DeclarationPattern));
+                    .Where(n => n.Kind() is SyntaxKind.DeclarationExpression or SyntaxKind.DeclarationPattern);
 
                 foreach (var node in declarations)
                 {
@@ -606,7 +630,7 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
 
                 if (CSharpSelectionResult.ShouldCallConfigureAwaitFalse())
                 {
-                    if (AnalyzerResult.ReturnType.GetMembers().Any(x => x is IMethodSymbol
+                    if (AnalyzerResult.ReturnType.GetMembers().Any(static x => x is IMethodSymbol
                         {
                             Name: nameof(Task.ConfigureAwait),
                             Parameters: { Length: 1 } parameters
@@ -633,14 +657,24 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 ExpressionSyntax initialValue,
                 CancellationToken cancellationToken)
             {
-                var type = variable.GetVariableType(SemanticDocument);
+                var type = variable.GetVariableType();
                 var typeNode = type.GenerateTypeSyntax();
 
+                var originalIdentifierToken = variable.GetOriginalIdentifierToken(cancellationToken);
+
+                // Hierarchy being checked for to see if a using keyword is needed is
+                // Token -> VariableDeclarator -> VariableDeclaration -> LocalDeclaration
+                var usingKeyword = originalIdentifierToken.Parent?.Parent?.Parent is LocalDeclarationStatementSyntax { UsingKeyword.FullSpan.IsEmpty: false }
+                    ? SyntaxFactory.Token(SyntaxKind.UsingKeyword)
+                    : default;
+
                 var equalsValueClause = initialValue == null ? null : SyntaxFactory.EqualsValueClause(value: initialValue);
 
                 return SyntaxFactory.LocalDeclarationStatement(
                     SyntaxFactory.VariableDeclaration(typeNode)
-                          .AddVariables(SyntaxFactory.VariableDeclarator(SyntaxFactory.Identifier(variable.Name)).WithInitializer(equalsValueClause)));
+                          .AddVariables(SyntaxFactory.VariableDeclarator(SyntaxFactory.Identifier(variable.Name))
+                          .WithInitializer(equalsValueClause)))
+                    .WithUsingKeyword(usingKeyword);
             }
 
             protected override async Task<GeneratedCode> CreateGeneratedCodeAsync(OperationStatus status, SemanticDocument newDocument, CancellationToken cancellationToken)
@@ -824,14 +858,13 @@ namespace Microsoft.CodeAnalysis.CSharp.ExtractMethod
                 }
 
                 // For local functions, pascal case and camel case should be the most common and therefore we only consider those cases.
-                var namingPreferences = Options.GetOption(NamingStyleOptions.NamingPreferences, LanguageNames.CSharp);
-                var localFunctionPreferences = namingPreferences.SymbolSpecifications.Where(symbol => symbol.AppliesTo(new SymbolKindOrTypeKind(MethodKind.LocalFunction), CreateMethodModifiers(), null));
+                var localFunctionPreferences = Options.NamingStyle.SymbolSpecifications.Where(symbol => symbol.AppliesTo(new SymbolSpecification.SymbolKindOrTypeKind(MethodKind.LocalFunction), CreateMethodModifiers(), null));
 
-                var namingRules = namingPreferences.Rules.NamingRules;
-                var localFunctionKind = new SymbolKindOrTypeKind(MethodKind.LocalFunction);
+                var namingRules = Options.NamingStyle.Rules.NamingRules;
+                var localFunctionKind = new SymbolSpecification.SymbolKindOrTypeKind(MethodKind.LocalFunction);
                 if (LocalFunction)
                 {
-                    if (namingRules.Any(rule => rule.NamingStyle.CapitalizationScheme.Equals(Capitalization.CamelCase) && rule.SymbolSpecification.AppliesTo(localFunctionKind, CreateMethodModifiers(), null)))
+                    if (namingRules.Any(static (rule, arg) => rule.NamingStyle.CapitalizationScheme.Equals(Capitalization.CamelCase) && rule.SymbolSpecification.AppliesTo(arg.localFunctionKind, arg.self.CreateMethodModifiers(), null), (self: this, localFunctionKind)))
                     {
                         methodName = NewMethodCamelCaseStr;
                     }
