@@ -8,7 +8,7 @@ using System;
 using System.Composition;
 using Microsoft.CodeAnalysis.Host;
 using Microsoft.CodeAnalysis.Host.Mef;
-using Microsoft.CodeAnalysis.LanguageServices;
+using Microsoft.CodeAnalysis.LanguageService;
 
 namespace Microsoft.CodeAnalysis.Editor.CSharp.LanguageServices
 {
@@ -22,6 +22,6 @@ namespace Microsoft.CodeAnalysis.Editor.CSharp.LanguageServices
         }
 
         public ILanguageService CreateLanguageService(HostLanguageServices provider)
-            => new CSharpSymbolDisplayService(provider);
+            => new CSharpSymbolDisplayService(provider.LanguageServices);
     }
 }
