@@ -8,14 +8,14 @@ using System.Composition;
 using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
+using Microsoft.CodeAnalysis.CodeGeneration;
 using Microsoft.CodeAnalysis.CodeStyle;
-using Microsoft.CodeAnalysis.CSharp.CodeStyle;
+using Microsoft.CodeAnalysis.CSharp.CodeGeneration;
 using Microsoft.CodeAnalysis.CSharp.Extensions;
 using Microsoft.CodeAnalysis.CSharp.Syntax;
 using Microsoft.CodeAnalysis.Editing;
 using Microsoft.CodeAnalysis.Formatting;
 using Microsoft.CodeAnalysis.Host.Mef;
-using Microsoft.CodeAnalysis.Options;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.ReplacePropertyWithMethods;
 using Microsoft.CodeAnalysis.Shared.Extensions;
@@ -40,30 +40,31 @@ namespace Microsoft.CodeAnalysis.CSharp.ReplacePropertyWithMethods
             IFieldSymbol propertyBackingField,
             string desiredGetMethodName,
             string desiredSetMethodName,
+            CodeGenerationOptionsProvider fallbackOptions,
             CancellationToken cancellationToken)
         {
             if (propertyDeclarationNode is not PropertyDeclarationSyntax propertyDeclaration)
                 return ImmutableArray<SyntaxNode>.Empty;
 
-            var documentOptions = await document.GetOptionsAsync(cancellationToken).ConfigureAwait(false);
+            var options = (CSharpCodeGenerationOptions)await document.GetCodeGenerationOptionsAsync(fallbackOptions, cancellationToken).ConfigureAwait(false);
             var syntaxTree = await document.GetRequiredSyntaxTreeAsync(cancellationToken).ConfigureAwait(false);
-            var parseOptions = syntaxTree.Options;
+            var languageVersion = syntaxTree.Options.LanguageVersion();
 
             return ConvertPropertyToMembers(
-                documentOptions, parseOptions,
+                languageVersion,
                 SyntaxGenerator.GetGenerator(document), property,
                 propertyDeclaration, propertyBackingField,
-                desiredGetMethodName, desiredSetMethodName,
+                options.PreferExpressionBodiedMethods.Value, desiredGetMethodName, desiredSetMethodName,
                 cancellationToken);
         }
 
         private static ImmutableArray<SyntaxNode> ConvertPropertyToMembers(
-            DocumentOptionSet documentOptions,
-            ParseOptions parseOptions,
+            LanguageVersion languageVersion,
             SyntaxGenerator generator,
             IPropertySymbol property,
             PropertyDeclarationSyntax propertyDeclaration,
             IFieldSymbol? propertyBackingField,
+            ExpressionBodyPreference expressionBodyPreference,
             string desiredGetMethodName,
             string desiredSetMethodName,
             CancellationToken cancellationToken)
@@ -80,108 +81,146 @@ namespace Microsoft.CodeAnalysis.CSharp.ReplacePropertyWithMethods
             if (getMethod != null)
             {
                 result.Add(GetGetMethod(
-                    documentOptions, parseOptions,
+                    languageVersion,
                     generator, propertyDeclaration, propertyBackingField,
-                    getMethod, desiredGetMethodName,
-                    cancellationToken: cancellationToken));
+                    getMethod, desiredGetMethodName, expressionBodyPreference,
+                    cancellationToken));
             }
 
             var setMethod = property.SetMethod;
             if (setMethod != null)
             {
                 result.Add(GetSetMethod(
-                    documentOptions, parseOptions,
+                    languageVersion,
                     generator, propertyDeclaration, propertyBackingField,
-                    setMethod, desiredSetMethodName,
-                    cancellationToken: cancellationToken));
+                    setMethod, desiredSetMethodName, expressionBodyPreference,
+                    cancellationToken));
             }
 
             return result.ToImmutable();
         }
 
         private static SyntaxNode GetSetMethod(
-            DocumentOptionSet documentOptions,
-            ParseOptions parseOptions,
+            LanguageVersion languageVersion,
             SyntaxGenerator generator,
             PropertyDeclarationSyntax propertyDeclaration,
             IFieldSymbol? propertyBackingField,
             IMethodSymbol setMethod,
             string desiredSetMethodName,
+            ExpressionBodyPreference expressionBodyPreference,
             CancellationToken cancellationToken)
         {
-            var methodDeclaration = GetSetMethodWorker(
-                generator, propertyDeclaration, propertyBackingField,
-                setMethod, desiredSetMethodName, cancellationToken);
+            var methodDeclaration = GetSetMethodWorker();
 
             // The analyzer doesn't report diagnostics when the trivia contains preprocessor directives, so it's safe
             // to copy the complete leading trivia to both generated methods.
             methodDeclaration = CopyLeadingTrivia(propertyDeclaration, methodDeclaration, ConvertValueToParamRewriter.Instance);
 
             return UseExpressionOrBlockBodyIfDesired(
-                documentOptions, parseOptions, methodDeclaration,
-                createReturnStatementForExpression: false);
-        }
+                languageVersion, methodDeclaration, expressionBodyPreference,
+                createReturnStatementForExpression: false, cancellationToken);
 
-        private static MethodDeclarationSyntax GetSetMethodWorker(
-            SyntaxGenerator generator,
-            PropertyDeclarationSyntax propertyDeclaration,
-            IFieldSymbol? propertyBackingField,
-            IMethodSymbol setMethod,
-            string desiredSetMethodName,
-            CancellationToken cancellationToken)
-        {
-            var setAccessorDeclaration = (AccessorDeclarationSyntax)setMethod.DeclaringSyntaxReferences[0].GetSyntax(cancellationToken);
-            var methodDeclaration = (MethodDeclarationSyntax)generator.MethodDeclaration(setMethod, desiredSetMethodName);
+            MethodDeclarationSyntax GetSetMethodWorker()
+            {
+                var setAccessorDeclaration = (AccessorDeclarationSyntax)setMethod.DeclaringSyntaxReferences[0].GetSyntax(cancellationToken);
+                var methodDeclaration = (MethodDeclarationSyntax)generator.MethodDeclaration(setMethod, desiredSetMethodName);
 
-            // property has unsafe, but generator didn't add it to the method, so we have to add it here
-            if (propertyDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword)
-                && !methodDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword))
-            {
-                methodDeclaration = methodDeclaration.AddModifiers(SyntaxFactory.Token(SyntaxKind.UnsafeKeyword));
-            }
+                // property has unsafe, but generator didn't add it to the method, so we have to add it here
+                if (propertyDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword)
+                    && !methodDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword))
+                {
+                    methodDeclaration = methodDeclaration.AddModifiers(SyntaxFactory.Token(SyntaxKind.UnsafeKeyword));
+                }
 
-            if (setAccessorDeclaration.Body != null)
-            {
-                return methodDeclaration.WithBody(setAccessorDeclaration.Body)
-                                        .WithAdditionalAnnotations(Formatter.Annotation);
-            }
-            else if (setAccessorDeclaration.ExpressionBody != null)
-            {
-                return methodDeclaration.WithBody(null)
-                                        .WithExpressionBody(setAccessorDeclaration.ExpressionBody)
-                                        .WithSemicolonToken(setAccessorDeclaration.SemicolonToken);
-            }
-            else if (propertyBackingField != null)
-            {
-                return methodDeclaration.WithBody(SyntaxFactory.Block(
-                    (StatementSyntax)generator.ExpressionStatement(
-                        generator.AssignmentStatement(
-                            GetFieldReference(generator, propertyBackingField),
-                            generator.IdentifierName("value")))));
-            }
+                methodDeclaration = methodDeclaration.WithAttributeLists(setAccessorDeclaration.AttributeLists);
 
-            return methodDeclaration;
+                if (setAccessorDeclaration.Body != null)
+                {
+                    return methodDeclaration.WithBody(setAccessorDeclaration.Body)
+                                            .WithAdditionalAnnotations(Formatter.Annotation);
+                }
+                else if (setAccessorDeclaration.ExpressionBody != null)
+                {
+                    return methodDeclaration.WithBody(null)
+                                            .WithExpressionBody(setAccessorDeclaration.ExpressionBody)
+                                            .WithSemicolonToken(setAccessorDeclaration.SemicolonToken);
+                }
+                else if (propertyBackingField != null)
+                {
+                    return methodDeclaration.WithBody(SyntaxFactory.Block(
+                        (StatementSyntax)generator.ExpressionStatement(
+                            generator.AssignmentStatement(
+                                GetFieldReference(generator, propertyBackingField),
+                                generator.IdentifierName("value")))));
+                }
+
+                return methodDeclaration;
+            }
         }
 
         private static SyntaxNode GetGetMethod(
-            DocumentOptionSet documentOptions,
-            ParseOptions parseOptions,
+            LanguageVersion languageVersion,
             SyntaxGenerator generator,
             PropertyDeclarationSyntax propertyDeclaration,
             IFieldSymbol? propertyBackingField,
             IMethodSymbol getMethod,
             string desiredGetMethodName,
+            ExpressionBodyPreference expressionBodyPreference,
             CancellationToken cancellationToken)
         {
-            var methodDeclaration = GetGetMethodWorker(
-                generator, propertyDeclaration, propertyBackingField, getMethod,
-                desiredGetMethodName, cancellationToken);
+            var methodDeclaration = GetGetMethodWorker();
 
             methodDeclaration = CopyLeadingTrivia(propertyDeclaration, methodDeclaration, ConvertValueToReturnsRewriter.Instance);
 
             return UseExpressionOrBlockBodyIfDesired(
-                documentOptions, parseOptions, methodDeclaration,
-                createReturnStatementForExpression: true);
+                languageVersion, methodDeclaration, expressionBodyPreference,
+                createReturnStatementForExpression: true, cancellationToken);
+
+            MethodDeclarationSyntax GetGetMethodWorker()
+            {
+                var methodDeclaration = (MethodDeclarationSyntax)generator.MethodDeclaration(getMethod, desiredGetMethodName);
+
+                // property has unsafe, but generator didn't add it to the method, so we have to add it here
+                if (propertyDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword)
+                    && !methodDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword))
+                {
+                    methodDeclaration = methodDeclaration.AddModifiers(SyntaxFactory.Token(SyntaxKind.UnsafeKeyword));
+                }
+
+                if (propertyDeclaration.ExpressionBody != null)
+                {
+                    return methodDeclaration.WithBody(null)
+                                            .WithExpressionBody(propertyDeclaration.ExpressionBody)
+                                            .WithSemicolonToken(propertyDeclaration.SemicolonToken);
+                }
+                else
+                {
+                    var getAccessorDeclaration = (AccessorDeclarationSyntax)getMethod.DeclaringSyntaxReferences[0].GetSyntax(cancellationToken);
+
+                    methodDeclaration = methodDeclaration.WithAttributeLists(getAccessorDeclaration.AttributeLists);
+
+                    if (getAccessorDeclaration?.ExpressionBody != null)
+                    {
+                        return methodDeclaration.WithBody(null)
+                                                .WithExpressionBody(getAccessorDeclaration.ExpressionBody)
+                                                .WithSemicolonToken(getAccessorDeclaration.SemicolonToken);
+                    }
+                    else if (getAccessorDeclaration?.Body != null)
+                    {
+                        return methodDeclaration.WithBody(getAccessorDeclaration.Body)
+                                                .WithAdditionalAnnotations(Formatter.Annotation);
+                    }
+                    else if (propertyBackingField != null)
+                    {
+                        var fieldReference = GetFieldReference(generator, propertyBackingField);
+                        return methodDeclaration.WithBody(
+                            SyntaxFactory.Block(
+                                (StatementSyntax)generator.ReturnStatement(fieldReference)));
+                    }
+                }
+
+                return methodDeclaration;
+            }
         }
 
         private static MethodDeclarationSyntax CopyLeadingTrivia(
@@ -213,14 +252,16 @@ namespace Microsoft.CodeAnalysis.CSharp.ReplacePropertyWithMethods
         }
 
         private static SyntaxNode UseExpressionOrBlockBodyIfDesired(
-            DocumentOptionSet documentOptions, ParseOptions parseOptions,
-            MethodDeclarationSyntax methodDeclaration, bool createReturnStatementForExpression)
+            LanguageVersion languageVersion,
+            MethodDeclarationSyntax methodDeclaration,
+            ExpressionBodyPreference expressionBodyPreference,
+            bool createReturnStatementForExpression,
+            CancellationToken cancellationToken)
         {
-            var expressionBodyPreference = documentOptions.GetOption(CSharpCodeStyleOptions.PreferExpressionBodiedMethods).Value;
             if (methodDeclaration.Body != null && expressionBodyPreference != ExpressionBodyPreference.Never)
             {
                 if (methodDeclaration.Body.TryConvertToArrowExpressionBody(
-                        methodDeclaration.Kind(), parseOptions, expressionBodyPreference,
+                        methodDeclaration.Kind(), languageVersion, expressionBodyPreference, cancellationToken,
                         out var arrowExpression, out var semicolonToken))
                 {
                     return methodDeclaration.WithBody(null)
@@ -244,56 +285,6 @@ namespace Microsoft.CodeAnalysis.CSharp.ReplacePropertyWithMethods
             return methodDeclaration;
         }
 
-        private static MethodDeclarationSyntax GetGetMethodWorker(
-            SyntaxGenerator generator,
-            PropertyDeclarationSyntax propertyDeclaration,
-            IFieldSymbol? propertyBackingField,
-            IMethodSymbol getMethod,
-            string desiredGetMethodName,
-            CancellationToken cancellationToken)
-        {
-            var methodDeclaration = (MethodDeclarationSyntax)generator.MethodDeclaration(getMethod, desiredGetMethodName);
-
-            // property has unsafe, but generator didn't add it to the method, so we have to add it here
-            if (propertyDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword)
-                && !methodDeclaration.Modifiers.Any(SyntaxKind.UnsafeKeyword))
-            {
-                methodDeclaration = methodDeclaration.AddModifiers(SyntaxFactory.Token(SyntaxKind.UnsafeKeyword));
-            }
-
-            if (propertyDeclaration.ExpressionBody != null)
-            {
-                return methodDeclaration.WithBody(null)
-                                        .WithExpressionBody(propertyDeclaration.ExpressionBody)
-                                        .WithSemicolonToken(propertyDeclaration.SemicolonToken);
-            }
-            else
-            {
-                var getAccessorDeclaration = (AccessorDeclarationSyntax)getMethod.DeclaringSyntaxReferences[0].GetSyntax(cancellationToken);
-                if (getAccessorDeclaration?.ExpressionBody != null)
-                {
-                    return methodDeclaration.WithBody(null)
-                                            .WithExpressionBody(getAccessorDeclaration.ExpressionBody)
-                                            .WithSemicolonToken(getAccessorDeclaration.SemicolonToken);
-                }
-
-                if (getAccessorDeclaration?.Body != null)
-                {
-                    return methodDeclaration.WithBody(getAccessorDeclaration.Body)
-                                            .WithAdditionalAnnotations(Formatter.Annotation);
-                }
-                else if (propertyBackingField != null)
-                {
-                    var fieldReference = GetFieldReference(generator, propertyBackingField);
-                    return methodDeclaration.WithBody(
-                        SyntaxFactory.Block(
-                            (StatementSyntax)generator.ReturnStatement(fieldReference)));
-                }
-            }
-
-            return methodDeclaration;
-        }
-
         /// <summary>
         /// Used by the documentation comment rewriters to identify top-level <c>&lt;value&gt;</c> nodes.
         /// </summary>
@@ -334,17 +325,25 @@ namespace Microsoft.CodeAnalysis.CSharp.ReplacePropertyWithMethods
         {
             var parent = (AssignmentExpressionSyntax)compoundAssignment;
 
-            var operatorKind =
-                parent.IsKind(SyntaxKind.OrAssignmentExpression) ? SyntaxKind.BitwiseOrExpression :
-                parent.IsKind(SyntaxKind.AndAssignmentExpression) ? SyntaxKind.BitwiseAndExpression :
-                parent.IsKind(SyntaxKind.ExclusiveOrAssignmentExpression) ? SyntaxKind.ExclusiveOrExpression :
-                parent.IsKind(SyntaxKind.LeftShiftAssignmentExpression) ? SyntaxKind.LeftShiftExpression :
-                parent.IsKind(SyntaxKind.RightShiftAssignmentExpression) ? SyntaxKind.RightShiftExpression :
-                parent.IsKind(SyntaxKind.AddAssignmentExpression) ? SyntaxKind.AddExpression :
-                parent.IsKind(SyntaxKind.SubtractAssignmentExpression) ? SyntaxKind.SubtractExpression :
-                parent.IsKind(SyntaxKind.MultiplyAssignmentExpression) ? SyntaxKind.MultiplyExpression :
-                parent.IsKind(SyntaxKind.DivideAssignmentExpression) ? SyntaxKind.DivideExpression :
-                parent.IsKind(SyntaxKind.ModuloAssignmentExpression) ? SyntaxKind.ModuloExpression : SyntaxKind.None;
+            var operatorKind = parent.Kind() switch
+            {
+                SyntaxKind.AddAssignmentExpression => SyntaxKind.AddExpression,
+                SyntaxKind.AndAssignmentExpression => SyntaxKind.BitwiseAndExpression,
+                SyntaxKind.CoalesceAssignmentExpression => SyntaxKind.CoalesceExpression,
+                SyntaxKind.DivideAssignmentExpression => SyntaxKind.DivideExpression,
+                SyntaxKind.ExclusiveOrAssignmentExpression => SyntaxKind.ExclusiveOrExpression,
+                SyntaxKind.LeftShiftAssignmentExpression => SyntaxKind.LeftShiftExpression,
+                SyntaxKind.ModuloAssignmentExpression => SyntaxKind.ModuloExpression,
+                SyntaxKind.MultiplyAssignmentExpression => SyntaxKind.MultiplyExpression,
+                SyntaxKind.OrAssignmentExpression => SyntaxKind.BitwiseOrExpression,
+                SyntaxKind.RightShiftAssignmentExpression => SyntaxKind.RightShiftExpression,
+                SyntaxKind.SubtractAssignmentExpression => SyntaxKind.SubtractExpression,
+                SyntaxKind.UnsignedRightShiftAssignmentExpression => SyntaxKind.UnsignedRightShiftExpression,
+                _ => SyntaxKind.None,
+            };
+
+            if (operatorKind is SyntaxKind.None)
+                return parent;
 
             return SyntaxFactory.BinaryExpression(operatorKind, readExpression, parent.Right.Parenthesize());
         }
