@@ -2,15 +2,15 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-using System.Threading;
-using Microsoft.CodeAnalysis.CSharp.LanguageServices;
+using Microsoft.CodeAnalysis.CSharp.LanguageService;
+using Microsoft.CodeAnalysis.CSharp.Simplification;
 using Microsoft.CodeAnalysis.CSharp.Simplification.Simplifiers;
 using Microsoft.CodeAnalysis.CSharp.Syntax;
 using Microsoft.CodeAnalysis.Diagnostics;
-using Microsoft.CodeAnalysis.LanguageServices;
-using Microsoft.CodeAnalysis.Options;
+using Microsoft.CodeAnalysis.LanguageService;
+using Microsoft.CodeAnalysis.Simplification;
+using Microsoft.CodeAnalysis.Simplification.Simplifiers;
 using Microsoft.CodeAnalysis.SimplifyThisOrMe;
-using Microsoft.CodeAnalysis.Text;
 
 namespace Microsoft.CodeAnalysis.CSharp.SimplifyThisOrMe
 {
@@ -22,17 +22,12 @@ namespace Microsoft.CodeAnalysis.CSharp.SimplifyThisOrMe
             ThisExpressionSyntax,
             MemberAccessExpressionSyntax>
     {
-        protected override string GetLanguageName()
-            => LanguageNames.CSharp;
+        protected override ISyntaxKinds SyntaxKinds => CSharpSyntaxKinds.Instance;
 
-        protected override ISyntaxFacts GetSyntaxFacts()
-            => CSharpSyntaxFacts.Instance;
+        protected override ISimplification Simplification
+            => CSharpSimplification.Instance;
 
-        protected override bool CanSimplifyTypeNameExpression(
-            SemanticModel model, MemberAccessExpressionSyntax node, OptionSet optionSet,
-            out TextSpan issueSpan, CancellationToken cancellationToken)
-        {
-            return ExpressionSimplifier.Instance.TrySimplify(node, model, optionSet, out _, out issueSpan, cancellationToken);
-        }
+        protected override AbstractMemberAccessExpressionSimplifier<ExpressionSyntax, MemberAccessExpressionSyntax, ThisExpressionSyntax> Simplifier
+            => MemberAccessExpressionSimplifier.Instance;
     }
 }
