@@ -10,10 +10,12 @@ using System.Collections.Immutable;
 using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
-using Microsoft.CodeAnalysis.AddImports;
+using Microsoft.CodeAnalysis.AddImport;
+using Microsoft.CodeAnalysis.CodeActions;
 using Microsoft.CodeAnalysis.Diagnostics;
 using Microsoft.CodeAnalysis.Formatting;
-using Microsoft.CodeAnalysis.LanguageServices;
+using Microsoft.CodeAnalysis.Host;
+using Microsoft.CodeAnalysis.LanguageService;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Text;
@@ -42,16 +44,16 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
         public bool IsFixableDiagnostic(Diagnostic diagnostic)
             => SuppressionHelpers.CanBeSuppressed(diagnostic) || SuppressionHelpers.CanBeUnsuppressed(diagnostic);
 
-        protected abstract SyntaxTriviaList CreatePragmaDisableDirectiveTrivia(Diagnostic diagnostic, Func<SyntaxNode, SyntaxNode> formatNode, bool needsLeadingEndOfLine, bool needsTrailingEndOfLine);
-        protected abstract SyntaxTriviaList CreatePragmaRestoreDirectiveTrivia(Diagnostic diagnostic, Func<SyntaxNode, SyntaxNode> formatNode, bool needsLeadingEndOfLine, bool needsTrailingEndOfLine);
+        protected abstract SyntaxTriviaList CreatePragmaDisableDirectiveTrivia(Diagnostic diagnostic, Func<SyntaxNode, CancellationToken, SyntaxNode> formatNode, bool needsLeadingEndOfLine, bool needsTrailingEndOfLine, CancellationToken cancellationToken);
+        protected abstract SyntaxTriviaList CreatePragmaRestoreDirectiveTrivia(Diagnostic diagnostic, Func<SyntaxNode, CancellationToken, SyntaxNode> formatNode, bool needsLeadingEndOfLine, bool needsTrailingEndOfLine, CancellationToken cancellationToken);
 
         protected abstract SyntaxNode AddGlobalSuppressMessageAttribute(
             SyntaxNode newRoot,
             ISymbol targetSymbol,
             INamedTypeSymbol suppressMessageAttribute,
             Diagnostic diagnostic,
-            Workspace workspace,
-            Compilation compilation,
+            SolutionServices services,
+            SyntaxFormattingOptions options,
             IAddImportsService addImportsService,
             CancellationToken cancellationToken);
 
@@ -87,28 +89,74 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
             return diagnostic.Id;
         }
 
-        protected virtual SyntaxToken GetAdjustedTokenForPragmaDisable(SyntaxToken token, SyntaxNode root, TextLineCollection lines, int indexOfLine)
-            => token;
+        protected abstract SyntaxNode GetContainingStatement(SyntaxToken token);
+        protected abstract bool TokenHasTrailingLineContinuationChar(SyntaxToken token);
 
-        protected virtual SyntaxToken GetAdjustedTokenForPragmaRestore(SyntaxToken token, SyntaxNode root, TextLineCollection lines, int indexOfLine)
-            => token;
-
-        public Task<ImmutableArray<CodeFix>> GetFixesAsync(
-            Document document, TextSpan span, IEnumerable<Diagnostic> diagnostics, CancellationToken cancellationToken)
+        protected SyntaxToken GetAdjustedTokenForPragmaDisable(SyntaxToken token, SyntaxNode root, TextLineCollection lines)
         {
-            return GetSuppressionsAsync(document, span, diagnostics, skipSuppressMessage: false, skipUnsuppress: false, cancellationToken: cancellationToken);
+            var containingStatement = GetContainingStatement(token);
+
+            // The containing statement might not start on the same line as the token, but we don't want to split
+            // a statement in the middle, so we actually want to use the first token on the line that has the first token
+            // of the statement.
+            //
+            // eg, given: public void M() { int x = 1; }
+            //
+            // When trying to suppress an "unused local" for x, token would be "x", the first token
+            // of the containing statement is "int", but we want the pragma before "public".
+            if (containingStatement is not null && containingStatement.GetFirstToken() != token)
+            {
+                var indexOfLine = lines.IndexOf(containingStatement.GetFirstToken().SpanStart);
+                var line = lines[indexOfLine];
+                token = root.FindToken(line.Start);
+            }
+
+            return token;
         }
 
-        internal async Task<ImmutableArray<PragmaWarningCodeAction>> GetPragmaSuppressionsAsync(Document document, TextSpan span, IEnumerable<Diagnostic> diagnostics, CancellationToken cancellationToken)
+        private SyntaxToken GetAdjustedTokenForPragmaRestore(SyntaxToken token, SyntaxNode root, TextLineCollection lines, int indexOfLine)
         {
-            var codeFixes = await GetSuppressionsAsync(document, span, diagnostics, skipSuppressMessage: true, skipUnsuppress: true, cancellationToken: cancellationToken).ConfigureAwait(false);
+            var containingStatement = GetContainingStatement(token);
+
+            // As per above, the last token of the statement might not be the last token on the line
+            if (containingStatement is not null && containingStatement.GetLastToken() != token)
+            {
+                indexOfLine = lines.IndexOf(containingStatement.GetLastToken().SpanStart);
+            }
+
+            var line = lines[indexOfLine];
+            token = root.FindToken(line.End);
+
+            // VB has line continuation characters that can explicitly extend the line beyond the last
+            // token, so allow for that by just skipping over them
+            while (TokenHasTrailingLineContinuationChar(token))
+            {
+                indexOfLine = indexOfLine + 1;
+                token = root.FindToken(lines[indexOfLine].End, findInsideTrivia: true);
+            }
+
+            return token;
+        }
+
+        public Task<ImmutableArray<CodeFix>> GetFixesAsync(
+            TextDocument textDocument, TextSpan span, IEnumerable<Diagnostic> diagnostics, CodeActionOptionsProvider fallbackOptions, CancellationToken cancellationToken)
+        {
+            if (textDocument is not Document document)
+                return Task.FromResult(ImmutableArray<CodeFix>.Empty);
+
+            return GetSuppressionsAsync(document, span, diagnostics, fallbackOptions, skipSuppressMessage: false, skipUnsuppress: false, cancellationToken: cancellationToken);
+        }
+
+        internal async Task<ImmutableArray<PragmaWarningCodeAction>> GetPragmaSuppressionsAsync(Document document, TextSpan span, IEnumerable<Diagnostic> diagnostics, CodeActionOptionsProvider fallbackOptions, CancellationToken cancellationToken)
+        {
+            var codeFixes = await GetSuppressionsAsync(document, span, diagnostics, fallbackOptions, skipSuppressMessage: true, skipUnsuppress: true, cancellationToken: cancellationToken).ConfigureAwait(false);
             return codeFixes.SelectMany(fix => fix.Action.NestedCodeActions)
                             .OfType<PragmaWarningCodeAction>()
                             .ToImmutableArray();
         }
 
         private async Task<ImmutableArray<CodeFix>> GetSuppressionsAsync(
-            Document document, TextSpan span, IEnumerable<Diagnostic> diagnostics, bool skipSuppressMessage, bool skipUnsuppress, CancellationToken cancellationToken)
+            Document document, TextSpan span, IEnumerable<Diagnostic> diagnostics, CodeActionOptionsProvider fallbackOptions, bool skipSuppressMessage, bool skipUnsuppress, CancellationToken cancellationToken)
         {
             var suppressionTargetInfo = await GetSuppressionTargetInfoAsync(document, span, cancellationToken).ConfigureAwait(false);
             if (suppressionTargetInfo == null)
@@ -117,13 +165,11 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
             }
 
             return await GetSuppressionsAsync(
-                documentOpt: document, project: document.Project, diagnostics: diagnostics,
-                suppressionTargetInfo: suppressionTargetInfo, skipSuppressMessage: skipSuppressMessage,
-                skipUnsuppress: skipUnsuppress, cancellationToken: cancellationToken).ConfigureAwait(false);
+                document, document.Project, diagnostics, suppressionTargetInfo, fallbackOptions, skipSuppressMessage, skipUnsuppress, cancellationToken).ConfigureAwait(false);
         }
 
         public async Task<ImmutableArray<CodeFix>> GetFixesAsync(
-            Project project, IEnumerable<Diagnostic> diagnostics, CancellationToken cancellationToken)
+            Project project, IEnumerable<Diagnostic> diagnostics, CodeActionOptionsProvider fallbackOptions, CancellationToken cancellationToken)
         {
             if (!project.SupportsCompilation)
             {
@@ -133,13 +179,13 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
             var compilation = await project.GetCompilationAsync(cancellationToken).ConfigureAwait(false);
             var suppressionTargetInfo = new SuppressionTargetInfo() { TargetSymbol = compilation.Assembly };
             return await GetSuppressionsAsync(
-                documentOpt: null, project: project, diagnostics: diagnostics,
-                suppressionTargetInfo: suppressionTargetInfo, skipSuppressMessage: false,
-                skipUnsuppress: false, cancellationToken: cancellationToken).ConfigureAwait(false);
+                documentOpt: null, project, diagnostics, suppressionTargetInfo,
+                fallbackOptions, skipSuppressMessage: false, skipUnsuppress: false,
+                cancellationToken).ConfigureAwait(false);
         }
 
         private async Task<ImmutableArray<CodeFix>> GetSuppressionsAsync(
-            Document documentOpt, Project project, IEnumerable<Diagnostic> diagnostics, SuppressionTargetInfo suppressionTargetInfo, bool skipSuppressMessage, bool skipUnsuppress, CancellationToken cancellationToken)
+            Document documentOpt, Project project, IEnumerable<Diagnostic> diagnostics, SuppressionTargetInfo suppressionTargetInfo, CodeActionOptionsProvider fallbackOptions, bool skipSuppressMessage, bool skipUnsuppress, CancellationToken cancellationToken)
         {
             // We only care about diagnostics that can be suppressed/unsuppressed.
             diagnostics = diagnostics.Where(IsFixableDiagnostic);
@@ -156,6 +202,7 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
                 skipSuppressMessage = suppressMessageAttribute == null || !suppressMessageAttribute.IsAttribute();
             }
 
+            var lazyFormattingOptions = (SyntaxFormattingOptions)null;
             var result = ArrayBuilder<CodeFix>.GetInstance();
             foreach (var diagnostic in diagnostics)
             {
@@ -165,7 +212,8 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
                     if (diagnostic.Location.IsInSource && documentOpt != null)
                     {
                         // pragma warning disable.
-                        nestedActions.Add(PragmaWarningCodeAction.Create(suppressionTargetInfo, documentOpt, diagnostic, this));
+                        lazyFormattingOptions ??= await documentOpt.GetSyntaxFormattingOptionsAsync(fallbackOptions, cancellationToken).ConfigureAwait(false);
+                        nestedActions.Add(PragmaWarningCodeAction.Create(suppressionTargetInfo, documentOpt, lazyFormattingOptions, diagnostic, this));
                     }
 
                     // SuppressMessageAttribute suppression is not supported for compiler diagnostics.
@@ -173,7 +221,7 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
                     {
                         // global assembly-level suppress message attribute.
                         nestedActions.Add(new GlobalSuppressMessageCodeAction(
-                            suppressionTargetInfo.TargetSymbol, suppressMessageAttribute, project, diagnostic, this));
+                            suppressionTargetInfo.TargetSymbol, suppressMessageAttribute, project, diagnostic, this, fallbackOptions));
 
                         // local suppress message attribute
                         // please note that in order to avoid issues with existing unit tests referencing the code fix
@@ -194,7 +242,7 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
                 }
                 else if (!skipUnsuppress)
                 {
-                    var codeAction = await RemoveSuppressionCodeAction.CreateAsync(suppressionTargetInfo, documentOpt, project, diagnostic, this, cancellationToken).ConfigureAwait(false);
+                    var codeAction = await RemoveSuppressionCodeAction.CreateAsync(suppressionTargetInfo, documentOpt, project, diagnostic, this, fallbackOptions, cancellationToken).ConfigureAwait(false);
                     if (codeAction != null)
                     {
                         result.Add(new CodeFix(project, codeAction, diagnostic));
@@ -228,7 +276,7 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
             var indexOfLine = lines.IndexOf(span.Start);
             var lineAtPos = lines[indexOfLine];
             var startToken = root.FindToken(lineAtPos.Start);
-            startToken = GetAdjustedTokenForPragmaDisable(startToken, root, lines, indexOfLine);
+            startToken = GetAdjustedTokenForPragmaDisable(startToken, root, lines);
 
             // Find the end token to attach pragma restore warning directive.
             var spanEnd = Math.Max(startToken.Span.End, span.End);
@@ -287,11 +335,8 @@ namespace Microsoft.CodeAnalysis.CodeFixes.Suppression
                 }
             }
 
-            if (targetSymbol == null)
-            {
-                // Outside of a member declaration, suppress diagnostic for the entire assembly.
-                targetSymbol = semanticModel.Compilation.Assembly;
-            }
+            // Outside of a member declaration, suppress diagnostic for the entire assembly.
+            targetSymbol ??= semanticModel.Compilation.Assembly;
 
             return new SuppressionTargetInfo() { TargetSymbol = targetSymbol, NodeWithTokens = nodeWithTokens, StartToken = startToken, EndToken = endToken, TargetMemberNode = targetMemberNode };
         }
