@@ -20,17 +20,9 @@ namespace Microsoft.CodeAnalysis.CommentSelection
         public abstract string SingleLineCommentString { get; }
         public abstract bool SupportsBlockComment { get; }
 
-        public Task<Document> FormatAsync(Document document, ImmutableArray<TextSpan> changes, CancellationToken cancellationToken)
-        {
-            var root = document.GetRequiredSyntaxRootSynchronously(cancellationToken);
-            var formattingSpans = changes.Select(s => CommonFormattingHelpers.GetFormattingSpan(root, s));
-
-            return Formatter.FormatAsync(document, formattingSpans, cancellationToken: cancellationToken);
-        }
-
-        public Task<CommentSelectionInfo> GetInfoAsync(Document document, TextSpan textSpan, CancellationToken cancellationToken)
-            => Task.FromResult(SupportsBlockComment
-                ? new CommentSelectionInfo(true, SupportsBlockComment, SingleLineCommentString, BlockCommentStartString, BlockCommentEndString)
-                : new CommentSelectionInfo(true, SupportsBlockComment, SingleLineCommentString, "", ""));
+        public CommentSelectionInfo GetInfo()
+            => SupportsBlockComment
+                ? new(supportsSingleLineComment: true, SupportsBlockComment, SingleLineCommentString, BlockCommentStartString, BlockCommentEndString)
+                : new(supportsSingleLineComment: true, SupportsBlockComment, SingleLineCommentString, blockCommentStartString: "", blockCommentEndString: "");
     }
 }
