@@ -6,14 +6,11 @@ using System;
 using System.Collections.Generic;
 using System.Collections.Immutable;
 using System.Linq;
-using System.Runtime.CompilerServices;
 using System.Threading;
 using System.Threading.Tasks;
-using Microsoft.CodeAnalysis.Completion.Log;
 using Microsoft.CodeAnalysis.ErrorReporting;
 using Microsoft.CodeAnalysis.Internal.Log;
-using Microsoft.CodeAnalysis.LanguageServices;
-using Microsoft.CodeAnalysis.Options;
+using Microsoft.CodeAnalysis.LanguageService;
 using Microsoft.CodeAnalysis.PooledObjects;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Shared.Extensions.ContextQuery;
@@ -27,28 +24,27 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
     internal abstract partial class AbstractSymbolCompletionProvider<TSyntaxContext> : LSPCompletionProvider
         where TSyntaxContext : SyntaxContext
     {
-        // PERF: Many CompletionProviders derive AbstractSymbolCompletionProvider and therefore
-        // compute identical contexts. This actually shows up on the 2-core typing test.
-        // Cache the most recent document/position/computed SyntaxContext to reduce repeat computation.
-        private static readonly ConditionalWeakTable<Document, Tuple<int, AsyncLazy<TSyntaxContext>>> s_cachedDocuments = new();
-
-        private bool? _isTargetTypeCompletionFilterExperimentEnabled = null;
-
         protected AbstractSymbolCompletionProvider()
         {
         }
 
-        protected abstract Task<ImmutableArray<(ISymbol symbol, bool preselect)>> GetSymbolsAsync(CompletionContext? completionContext, TSyntaxContext syntaxContext, int position, OptionSet options, CancellationToken cancellationToken);
         protected abstract (string displayText, string suffix, string insertionText) GetDisplayAndSuffixAndInsertionText(ISymbol symbol, TSyntaxContext context);
 
-        protected virtual CompletionItemRules GetCompletionItemRules(ImmutableArray<(ISymbol symbol, bool preselect)> symbols)
-            => CompletionItemRules.Default;
+        protected abstract Task<ImmutableArray<SymbolAndSelectionInfo>> GetSymbolsAsync(
+            CompletionContext? completionContext,
+            TSyntaxContext syntaxContext,
+            int position,
+            CompletionOptions options,
+            CancellationToken cancellationToken);
 
-        protected bool IsTargetTypeCompletionFilterExperimentEnabled(OptionSet options)
-        {
-            _isTargetTypeCompletionFilterExperimentEnabled ??= options.GetOption(CompletionOptions.TargetTypedCompletionFilterFeatureFlag);
-            return _isTargetTypeCompletionFilterExperimentEnabled == true;
-        }
+        protected abstract CompletionItem CreateItem(
+            CompletionContext completionContext,
+            string displayText,
+            string displayTextSuffix,
+            string insertionText,
+            ImmutableArray<SymbolAndSelectionInfo> symbols,
+            TSyntaxContext context,
+            SupportedPlatformData? supportedPlatformData);
 
         /// <param name="typeConvertibilityCache">A cache to use for repeated lookups. This should be created with <see cref="SymbolEqualityComparer.Default"/>
         /// because we ignore nullability.</param>
@@ -103,22 +99,21 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
         /// </summary>
         private ImmutableArray<CompletionItem> CreateItems(
             CompletionContext completionContext,
-            ImmutableArray<(ISymbol symbol, bool preselect)> symbols,
-            Func<(ISymbol symbol, bool preselect), TSyntaxContext> contextLookup,
+            ImmutableArray<SymbolAndSelectionInfo> symbols,
+            Func<SymbolAndSelectionInfo, TSyntaxContext> contextLookup,
             Dictionary<ISymbol, List<ProjectId>>? invalidProjectMap,
-            List<ProjectId>? totalProjects,
-            TelemetryCounter telemetryCounter)
+            List<ProjectId>? totalProjects)
         {
             // We might get symbol w/o name but CanBeReferencedByName is still set to true, 
             // need to filter them out.
             // https://github.com/dotnet/roslyn/issues/47690
             var symbolGroups = from symbol in symbols
-                               let texts = GetDisplayAndSuffixAndInsertionText(symbol.symbol, contextLookup(symbol))
+                               let texts = GetDisplayAndSuffixAndInsertionText(symbol.Symbol, contextLookup(symbol))
                                where !string.IsNullOrWhiteSpace(texts.displayText)
                                group symbol by texts into g
                                select g;
 
-            var itemListBuilder = ImmutableArray.CreateBuilder<CompletionItem>();
+            using var _ = ArrayBuilder<CompletionItem>.GetInstance(out var itemListBuilder);
             var typeConvertibilityCache = new Dictionary<ITypeSymbol, bool>(SymbolEqualityComparer.Default);
 
             foreach (var symbolGroup in symbolGroups)
@@ -127,10 +122,8 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
                 var arbitraryFirstContext = contextLookup(symbolGroup.First());
                 var symbolList = symbolGroup.ToImmutableArray();
 
-                if (IsTargetTypeCompletionFilterExperimentEnabled(completionContext.Options))
+                if (completionContext.CompletionOptions.TargetTypedCompletionFilter)
                 {
-                    var tick = Environment.TickCount;
-
                     includeItemInTargetTypedCompletion = TryFindFirstSymbolMatchesTargetTypes(contextLookup, symbolList, typeConvertibilityCache, out var index);
                     if (includeItemInTargetTypedCompletion && index > 0)
                     {
@@ -140,13 +133,11 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
                         symbolList = symbolList.RemoveAt(index);
                         symbolList = symbolList.Insert(0, firstMatch);
                     }
-
-                    telemetryCounter.AddTick(Environment.TickCount - tick);
                 }
 
+                var supportedPlatformData = ComputeSupportedPlatformData(completionContext, symbolList, invalidProjectMap, totalProjects);
                 var item = CreateItem(
-                    completionContext, symbolGroup.Key.displayText, symbolGroup.Key.suffix, symbolGroup.Key.insertionText,
-                    symbolList, arbitraryFirstContext, invalidProjectMap, totalProjects);
+                    completionContext, symbolGroup.Key.displayText, symbolGroup.Key.suffix, symbolGroup.Key.insertionText, symbolList, arbitraryFirstContext, supportedPlatformData);
 
                 if (includeItemInTargetTypedCompletion)
                 {
@@ -160,8 +151,8 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
         }
 
         protected static bool TryFindFirstSymbolMatchesTargetTypes(
-            Func<(ISymbol symbol, bool preselect), TSyntaxContext> contextLookup,
-            ImmutableArray<(ISymbol symbol, bool preselect)> symbolList,
+            Func<SymbolAndSelectionInfo, TSyntaxContext> contextLookup,
+            ImmutableArray<SymbolAndSelectionInfo> symbolList,
             Dictionary<ITypeSymbol, bool> typeConvertibilityCache,
             out int index)
         {
@@ -169,35 +160,22 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
             {
                 var symbol = symbolList[index];
                 var syntaxContext = contextLookup(symbol);
-                if (ShouldIncludeInTargetTypedCompletionList(symbol.symbol, syntaxContext.InferredTypes, syntaxContext.SemanticModel, syntaxContext.Position, typeConvertibilityCache))
+                if (ShouldIncludeInTargetTypedCompletionList(symbol.Symbol, syntaxContext.InferredTypes, syntaxContext.SemanticModel, syntaxContext.Position, typeConvertibilityCache))
                     break;
             }
 
             return index < symbolList.Length;
         }
 
-        /// <summary>
-        /// Given a Symbol, creates the completion item for it.
-        /// </summary>
-        private CompletionItem CreateItem(
-            CompletionContext completionContext,
-            string displayText,
-            string displayTextSuffix,
-            string insertionText,
-            ImmutableArray<(ISymbol symbol, bool preselect)> symbols,
-            TSyntaxContext context,
-            Dictionary<ISymbol, List<ProjectId>>? invalidProjectMap,
-            List<ProjectId>? totalProjects)
+        private static SupportedPlatformData? ComputeSupportedPlatformData(CompletionContext completionContext, ImmutableArray<SymbolAndSelectionInfo> symbols, Dictionary<ISymbol, List<ProjectId>>? invalidProjectMap, List<ProjectId>? totalProjects)
         {
-            Contract.ThrowIfTrue(symbols.IsDefault);
-
             SupportedPlatformData? supportedPlatformData = null;
             if (invalidProjectMap != null)
             {
                 List<ProjectId>? invalidProjects = null;
                 foreach (var symbol in symbols)
                 {
-                    if (invalidProjectMap.TryGetValue(symbol.symbol, out invalidProjects))
+                    if (invalidProjectMap.TryGetValue(symbol.Symbol, out invalidProjects))
                         break;
                 }
 
@@ -205,44 +183,40 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
                     supportedPlatformData = new SupportedPlatformData(completionContext.Document.Project.Solution, invalidProjects, totalProjects);
             }
 
-            return CreateItem(
-                completionContext, displayText, displayTextSuffix, insertionText, symbols, context, supportedPlatformData);
+            return supportedPlatformData;
         }
 
-        protected virtual CompletionItem CreateItem(
-            CompletionContext completionContext,
+        protected static CompletionItem CreateItemDefault(
             string displayText,
             string displayTextSuffix,
             string insertionText,
-            ImmutableArray<(ISymbol symbol, bool preselect)> symbols,
+            ImmutableArray<SymbolAndSelectionInfo> symbols,
             TSyntaxContext context,
             SupportedPlatformData? supportedPlatformData)
         {
-            var preselect = symbols.Any(t => t.preselect);
+            var preselect = symbols.Any(static t => t.Preselect);
             return SymbolCompletionItem.CreateWithSymbolId(
                 displayText: displayText,
                 displayTextSuffix: displayTextSuffix,
                 insertionText: insertionText,
-                filterText: GetFilterText(symbols[0].symbol, displayText, context),
+                filterText: GetFilterTextDefault(symbols[0].Symbol, displayText, context),
                 contextPosition: context.Position,
-                symbols: symbols.SelectAsArray(t => t.symbol),
+                symbols: symbols.SelectAsArray(t => t.Symbol),
                 supportedPlatforms: supportedPlatformData,
-                rules: GetCompletionItemRules(symbols)
+                rules: CompletionItemRules.Default
                     .WithMatchPriority(preselect ? MatchPriority.Preselect : MatchPriority.Default)
                     .WithSelectionBehavior(context.IsRightSideOfNumericType ? CompletionItemSelectionBehavior.SoftSelection : CompletionItemSelectionBehavior.Default));
         }
 
-        protected virtual string GetFilterText(ISymbol symbol, string displayText, TSyntaxContext context)
+        protected static string GetFilterTextDefault(ISymbol symbol, string displayText, TSyntaxContext context)
         {
-            return (displayText == symbol.Name) ||
-                (displayText.Length > 0 && displayText[0] == '@') ||
-                (context.IsAttributeNameContext && symbol.IsAttribute())
+            return displayText == symbol.Name || displayText is ['@', ..] || (context.IsAttributeNameContext && symbol.IsAttribute())
                 ? displayText
                 : symbol.Name;
         }
 
-        protected override Task<CompletionDescription> GetDescriptionWorkerAsync(Document document, CompletionItem item, CancellationToken cancellationToken)
-            => SymbolCompletionItem.GetDescriptionAsync(item, document, cancellationToken);
+        internal override Task<CompletionDescription> GetDescriptionWorkerAsync(Document document, CompletionItem item, CompletionOptions options, SymbolDescriptionOptions displayOptions, CancellationToken cancellationToken)
+            => SymbolCompletionItem.GetDescriptionAsync(item, document, displayOptions, cancellationToken);
 
         public override async Task ProvideCompletionsAsync(CompletionContext completionContext)
         {
@@ -250,7 +224,7 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
             {
                 var document = completionContext.Document;
                 var position = completionContext.Position;
-                var options = completionContext.Options;
+                var options = completionContext.CompletionOptions;
                 var cancellationToken = completionContext.CancellationToken;
 
                 // If we were triggered by typing a character, then do a semantic check to make sure
@@ -267,15 +241,15 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
                 completionContext.IsExclusive = IsExclusive();
 
                 using (Logger.LogBlock(FunctionId.Completion_SymbolCompletionProvider_GetItemsWorker, cancellationToken))
-                using (var telemetryCounter = new TelemetryCounter(ShouldCollectTelemetryForTargetTypeCompletion && IsTargetTypeCompletionFilterExperimentEnabled(options)))
                 {
-                    var syntaxContext = await GetOrCreateContextAsync(document, position, cancellationToken).ConfigureAwait(false);
-                    var regularItems = await GetItemsAsync(completionContext, syntaxContext, document, position, options, telemetryCounter, cancellationToken).ConfigureAwait(false);
+                    var syntaxContext = await completionContext.GetSyntaxContextWithExistingSpeculativeModelAsync(document, cancellationToken).ConfigureAwait(false) as TSyntaxContext;
+                    Contract.ThrowIfNull(syntaxContext);
 
+                    var regularItems = await GetItemsAsync(completionContext, syntaxContext, document, position, options, cancellationToken).ConfigureAwait(false);
                     completionContext.AddItems(regularItems);
                 }
             }
-            catch (Exception e) when (FatalError.ReportAndCatchUnlessCanceled(e))
+            catch (Exception e) when (FatalError.ReportAndCatchUnlessCanceled(e, ErrorSeverity.General))
             {
                 // nop
             }
@@ -286,26 +260,24 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
             TSyntaxContext syntaxContext,
             Document document,
             int position,
-            OptionSet options,
-            TelemetryCounter telemetryCounter,
+            CompletionOptions options,
             CancellationToken cancellationToken)
         {
             var relatedDocumentIds = document.GetLinkedDocumentIds();
-            options = CompletionUtilities.GetUpdatedRecommendationOptions(options, document.Project.Language);
 
             if (relatedDocumentIds.IsEmpty)
             {
                 var itemsForCurrentDocument = await GetSymbolsAsync(completionContext, syntaxContext, position, options, cancellationToken).ConfigureAwait(false);
-                return CreateItems(completionContext, itemsForCurrentDocument, _ => syntaxContext, invalidProjectMap: null, totalProjects: null, telemetryCounter);
+                return CreateItems(completionContext, itemsForCurrentDocument, _ => syntaxContext, invalidProjectMap: null, totalProjects: null);
             }
 
-            var contextAndSymbolLists = await GetPerContextSymbolsAsync(completionContext, document, position, options, new[] { document.Id }.Concat(relatedDocumentIds), cancellationToken).ConfigureAwait(false);
+            var contextAndSymbolLists = await GetPerContextSymbolsAsync(completionContext, document, options, new[] { document.Id }.Concat(relatedDocumentIds), cancellationToken).ConfigureAwait(false);
             var symbolToContextMap = UnionSymbols(contextAndSymbolLists);
             var missingSymbolsMap = FindSymbolsMissingInLinkedContexts(symbolToContextMap, contextAndSymbolLists);
             var totalProjects = contextAndSymbolLists.Select(t => t.documentId.ProjectId).ToList();
 
             return CreateItems(
-                completionContext, symbolToContextMap.Keys.ToImmutableArray(), symbol => symbolToContextMap[symbol], missingSymbolsMap, totalProjects, telemetryCounter);
+                completionContext, symbolToContextMap.Keys.ToImmutableArray(), symbol => symbolToContextMap[symbol], missingSymbolsMap, totalProjects);
         }
 
         protected virtual bool IsExclusive()
@@ -314,18 +286,17 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
         protected virtual Task<bool> IsSemanticTriggerCharacterAsync(Document document, int characterPosition, CancellationToken cancellationToken)
             => SpecializedTasks.True;
 
-        private static Dictionary<(ISymbol symbol, bool preselect), TSyntaxContext> UnionSymbols(
-            ImmutableArray<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<(ISymbol symbol, bool preselect)> symbols)> linkedContextSymbolLists)
+        private static Dictionary<SymbolAndSelectionInfo, TSyntaxContext> UnionSymbols(
+            ImmutableArray<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<SymbolAndSelectionInfo> symbols)> linkedContextSymbolLists)
         {
             // To correctly map symbols back to their SyntaxContext, we do care about assembly identity.
-            var result = new Dictionary<(ISymbol symbol, bool preselect), TSyntaxContext>(CompletionLinkedFilesSymbolEquivalenceComparer.Instance);
-
             // We don't care about assembly identity when creating the union.
+            var result = new Dictionary<SymbolAndSelectionInfo, TSyntaxContext>();
             foreach (var (documentId, syntaxContext, symbols) in linkedContextSymbolLists)
             {
                 // We need to use the SemanticModel any particular symbol came from in order to generate its description correctly.
                 // Therefore, when we add a symbol to set of union symbols, add a mapping from it to its SyntaxContext.
-                foreach (var symbol in symbols.GroupBy(s => new { s.symbol.Name, s.symbol.Kind }).Select(g => g.First()))
+                foreach (var symbol in symbols.GroupBy(s => new { s.Symbol.Name, s.Symbol.Kind }).Select(g => g.First()))
                 {
                     if (!result.ContainsKey(symbol))
                         result.Add(symbol, syntaxContext);
@@ -335,20 +306,22 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
             return result;
         }
 
-        private async Task<ImmutableArray<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<(ISymbol symbol, bool preselect)> symbols)>> GetPerContextSymbolsAsync(
-            CompletionContext completionContext, Document document, int position, OptionSet options, IEnumerable<DocumentId> relatedDocuments, CancellationToken cancellationToken)
+        private async Task<ImmutableArray<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<SymbolAndSelectionInfo> symbols)>> GetPerContextSymbolsAsync(
+            CompletionContext completionContext, Document document, CompletionOptions options, IEnumerable<DocumentId> relatedDocuments, CancellationToken cancellationToken)
         {
             var solution = document.Project.Solution;
 
-            using var _1 = ArrayBuilder<Task<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<(ISymbol symbol, bool preselect)> symbols)>>.GetInstance(out var tasks);
-            using var _2 = ArrayBuilder<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<(ISymbol symbol, bool preselect)> symbols)>.GetInstance(out var perContextSymbols);
+            using var _1 = ArrayBuilder<Task<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<SymbolAndSelectionInfo> symbols)>>.GetInstance(out var tasks);
+            using var _2 = ArrayBuilder<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<SymbolAndSelectionInfo> symbols)>.GetInstance(out var perContextSymbols);
 
             foreach (var relatedDocumentId in relatedDocuments)
             {
                 tasks.Add(Task.Run(async () =>
                 {
                     var relatedDocument = solution.GetRequiredDocument(relatedDocumentId);
-                    var syntaxContext = await GetOrCreateContextAsync(relatedDocument, position, cancellationToken).ConfigureAwait(false);
+                    var syntaxContext = await completionContext.GetSyntaxContextWithExistingSpeculativeModelAsync(relatedDocument, cancellationToken).ConfigureAwait(false) as TSyntaxContext;
+
+                    Contract.ThrowIfNull(syntaxContext);
                     var symbols = await TryGetSymbolsForContextAsync(completionContext, syntaxContext, options, cancellationToken).ConfigureAwait(false);
 
                     return (relatedDocument.Id, syntaxContext, symbols);
@@ -370,42 +343,15 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
         /// <summary>
         /// If current context is in active region, returns available symbols. Otherwise, returns null.
         /// </summary>
-        protected async Task<ImmutableArray<(ISymbol symbol, bool preselect)>> TryGetSymbolsForContextAsync(
-            CompletionContext? completionContext, TSyntaxContext syntaxContext, OptionSet options, CancellationToken cancellationToken)
+        protected async Task<ImmutableArray<SymbolAndSelectionInfo>> TryGetSymbolsForContextAsync(
+            CompletionContext? completionContext, TSyntaxContext syntaxContext, CompletionOptions options, CancellationToken cancellationToken)
         {
-            var syntaxFacts = syntaxContext.GetLanguageService<ISyntaxFactsService>();
+            var syntaxFacts = syntaxContext.GetRequiredLanguageService<ISyntaxFactsService>();
             return syntaxFacts.IsInInactiveRegion(syntaxContext.SyntaxTree, syntaxContext.Position, cancellationToken)
                 ? default
                 : await GetSymbolsAsync(completionContext, syntaxContext, syntaxContext.Position, options, cancellationToken).ConfigureAwait(false);
         }
 
-        protected static async Task<TSyntaxContext> CreateContextAsync(Document document, int position, CancellationToken cancellationToken)
-        {
-            var semanticModel = await document.ReuseExistingSpeculativeModelAsync(position, cancellationToken).ConfigureAwait(false);
-
-            var service = document.GetRequiredLanguageService<ISyntaxContextService>();
-            return (TSyntaxContext)service.CreateContext(document, semanticModel, position, cancellationToken);
-        }
-
-        private static Task<TSyntaxContext> GetOrCreateContextAsync(Document document, int position, CancellationToken cancellationToken)
-        {
-            lock (s_cachedDocuments)
-            {
-                var (cachedPosition, cachedLazyContext) = s_cachedDocuments.GetValue(
-                    document, d => Tuple.Create(position, new AsyncLazy<TSyntaxContext>(ct => CreateContextAsync(d, position, ct), cacheResult: true)));
-
-                if (cachedPosition == position)
-                {
-                    return cachedLazyContext.GetValueAsync(cancellationToken);
-                }
-
-                var lazyContext = new AsyncLazy<TSyntaxContext>(ct => CreateContextAsync(document, position, ct), cacheResult: true);
-                s_cachedDocuments.Remove(document);
-                s_cachedDocuments.Add(document, Tuple.Create(position, lazyContext));
-                return lazyContext.GetValueAsync(cancellationToken);
-            }
-        }
-
         /// <summary>
         /// Given a list of symbols, determine which are not recommended at the same position in linked documents.
         /// </summary>
@@ -413,14 +359,14 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
         /// <param name="linkedContextSymbolLists">The symbols recommended in linked documents</param>
         /// <returns>The list of projects each recommended symbol did NOT appear in.</returns>
         private static Dictionary<ISymbol, List<ProjectId>> FindSymbolsMissingInLinkedContexts(
-            Dictionary<(ISymbol symbol, bool preselect), TSyntaxContext> symbolToContext,
-            ImmutableArray<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<(ISymbol symbol, bool preselect)> symbols)> linkedContextSymbolLists)
+            Dictionary<SymbolAndSelectionInfo, TSyntaxContext> symbolToContext,
+            ImmutableArray<(DocumentId documentId, TSyntaxContext syntaxContext, ImmutableArray<SymbolAndSelectionInfo> symbols)> linkedContextSymbolLists)
         {
             var missingSymbols = new Dictionary<ISymbol, List<ProjectId>>(LinkedFilesSymbolEquivalenceComparer.Instance);
 
             foreach (var (documentId, syntaxContext, symbols) in linkedContextSymbolLists)
             {
-                var symbolsMissingInLinkedContext = symbolToContext.Keys.Except(symbols, CompletionLinkedFilesSymbolEquivalenceComparer.Instance);
+                var symbolsMissingInLinkedContext = symbolToContext.Keys.Except(symbols);
                 foreach (var (symbol, _) in symbolsMissingInLinkedContext)
                     missingSymbols.GetOrAdd(symbol, m => new List<ProjectId>()).Add(documentId.ProjectId);
             }
@@ -443,28 +389,5 @@ namespace Microsoft.CodeAnalysis.Completion.Providers
         /// </summary>
         protected virtual string GetInsertionText(CompletionItem item, char ch)
             => SymbolCompletionItem.GetInsertionText(item);
-
-        // This is used to decide which provider we'd collect target type completion telemetry from.
-        protected virtual bool ShouldCollectTelemetryForTargetTypeCompletion => false;
-
-        private class TelemetryCounter : IDisposable
-        {
-            private readonly bool _shouldReport;
-            private int _tick;
-
-            public TelemetryCounter(bool shouldReport)
-                => _shouldReport = shouldReport;
-
-            public void AddTick(int tick)
-                => _tick += tick;
-
-            public void Dispose()
-            {
-                if (_shouldReport)
-                {
-                    CompletionProvidersLogger.LogTargetTypeCompletionTicksDataPoint(_tick);
-                }
-            }
-        }
     }
 }
