@@ -2,26 +2,23 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System.Collections.Generic;
+using System.Linq;
 using Roslyn.Utilities;
 
 namespace Microsoft.CodeAnalysis.Completion.Providers
 {
-    internal class UnionCompletionItemComparer : IEqualityComparer<CompletionItem>
+    internal sealed class UnionCompletionItemComparer : IEqualityComparer<CompletionItem>
     {
-        public static UnionCompletionItemComparer Instance { get; } = new UnionCompletionItemComparer();
+        public static readonly UnionCompletionItemComparer Instance = new();
 
         private UnionCompletionItemComparer()
         {
         }
 
-        public bool Equals(CompletionItem x, CompletionItem y)
-        {
-            return x.DisplayText == y.DisplayText &&
-                (x.Tags == y.Tags || System.Linq.Enumerable.SequenceEqual(x.Tags, y.Tags));
-        }
+        public bool Equals(CompletionItem? x, CompletionItem? y)
+            => ReferenceEquals(x, y) ||
+               x is not null && y is not null && x.DisplayText == y.DisplayText && x.Tags.SequenceEqual(y.Tags);
 
         public int GetHashCode(CompletionItem obj)
             => Hash.Combine(obj.DisplayText.GetHashCode(), obj.Tags.Length);
