@@ -12,16 +12,11 @@ namespace Microsoft.CodeAnalysis.ExternalAccess.VSTypeScript
 {
     [Shared]
     [ExportLanguageService(typeof(VSTypeScriptDiagnosticAnalyzerLanguageService), InternalLanguageNames.TypeScript)]
-    internal sealed class VSTypeScriptDiagnosticAnalyzerLanguageService : ILanguageService
+    [method: ImportingConstructor]
+    [method: Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
+    internal sealed class VSTypeScriptDiagnosticAnalyzerLanguageService(
+        [Import(AllowDefault = true)] IVSTypeScriptDiagnosticAnalyzerImplementation? implementation = null) : ILanguageService
     {
-        internal readonly IVSTypeScriptDiagnosticAnalyzerImplementation? Implementation;
-
-        [ImportingConstructor]
-        [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
-        public VSTypeScriptDiagnosticAnalyzerLanguageService(
-            [Import(AllowDefault = true)] IVSTypeScriptDiagnosticAnalyzerImplementation? implementation = null)
-        {
-            Implementation = implementation;
-        }
+        internal readonly IVSTypeScriptDiagnosticAnalyzerImplementation? Implementation = implementation;
     }
 }
