@@ -5,10 +5,12 @@
 #nullable disable
 
 using System.Collections.Immutable;
+using Microsoft.CodeAnalysis.CodeCleanup;
+using Microsoft.CodeAnalysis.CodeGeneration;
 
 namespace Microsoft.CodeAnalysis.ExtractInterface
 {
-    internal class ExtractInterfaceOptionsResult
+    internal sealed class ExtractInterfaceOptionsResult
     {
         public enum ExtractLocation
         {
@@ -23,13 +25,15 @@ namespace Microsoft.CodeAnalysis.ExtractInterface
         public string InterfaceName { get; }
         public string FileName { get; }
         public ExtractLocation Location { get; }
+        public CleanCodeGenerationOptionsProvider FallbackOptions { get; }
 
-        public ExtractInterfaceOptionsResult(bool isCancelled, ImmutableArray<ISymbol> includedMembers, string interfaceName, string fileName, ExtractLocation location)
+        public ExtractInterfaceOptionsResult(bool isCancelled, ImmutableArray<ISymbol> includedMembers, string interfaceName, string fileName, ExtractLocation location, CleanCodeGenerationOptionsProvider fallbackOptions)
         {
             IsCancelled = isCancelled;
             IncludedMembers = includedMembers;
             InterfaceName = interfaceName;
             Location = location;
+            FallbackOptions = fallbackOptions;
             FileName = fileName;
         }
 
