@@ -10,23 +10,28 @@ using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.CodeActions;
+using Microsoft.CodeAnalysis.CodeGeneration;
 using Microsoft.CodeAnalysis.Editing;
-using Microsoft.CodeAnalysis.LanguageServices;
+using Microsoft.CodeAnalysis.LanguageService;
 using Microsoft.CodeAnalysis.Operations;
-using Microsoft.CodeAnalysis.Options;
 using Microsoft.CodeAnalysis.PooledObjects;
+using Microsoft.CodeAnalysis.Shared.Collections;
 using Microsoft.CodeAnalysis.Shared.Extensions;
+using Microsoft.CodeAnalysis.Simplification;
 using Microsoft.CodeAnalysis.Text;
 using Roslyn.Utilities;
 
 namespace Microsoft.CodeAnalysis.InitializeParameter
 {
-    internal abstract partial class AbstractAddParameterCheckCodeRefactoringProvider<
+    using static InitializeParameterHelpersCore;
+
+    internal abstract class AbstractAddParameterCheckCodeRefactoringProvider<
         TTypeDeclarationSyntax,
         TParameterSyntax,
         TStatementSyntax,
         TExpressionSyntax,
-        TBinaryExpressionSyntax> : AbstractInitializeParameterCodeRefactoringProvider<
+        TBinaryExpressionSyntax,
+        TSimplifierOptions> : AbstractInitializeParameterCodeRefactoringProvider<
             TTypeDeclarationSyntax,
             TParameterSyntax,
             TStatementSyntax,
@@ -36,22 +41,22 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
         where TStatementSyntax : SyntaxNode
         where TExpressionSyntax : SyntaxNode
         where TBinaryExpressionSyntax : TExpressionSyntax
+        where TSimplifierOptions : SimplifierOptions
     {
-        private readonly Func<SyntaxNode, bool> _isFunctionDeclarationFunc;
-
-        protected AbstractAddParameterCheckCodeRefactoringProvider()
-        {
-            _isFunctionDeclarationFunc = IsFunctionDeclaration;
-        }
-
         protected abstract bool CanOffer(SyntaxNode body);
-        protected abstract bool PrefersThrowExpression(DocumentOptionSet options);
+        protected abstract bool PrefersThrowExpression(TSimplifierOptions options);
         protected abstract string EscapeResourceString(string input);
-        protected abstract TStatementSyntax CreateParameterCheckIfStatement(DocumentOptionSet options, TExpressionSyntax condition, TStatementSyntax ifTrueStatement);
+        protected abstract TStatementSyntax CreateParameterCheckIfStatement(TExpressionSyntax condition, TStatementSyntax ifTrueStatement, TSimplifierOptions options);
 
         protected override async Task<ImmutableArray<CodeAction>> GetRefactoringsForAllParametersAsync(
-            Document document, SyntaxNode functionDeclaration, IMethodSymbol methodSymbol,
-            IBlockOperation? blockStatementOpt, ImmutableArray<SyntaxNode> listOfParameterNodes, TextSpan parameterSpan, CancellationToken cancellationToken)
+            Document document,
+            SyntaxNode functionDeclaration,
+            IMethodSymbol methodSymbol,
+            IBlockOperation? blockStatementOpt,
+            ImmutableArray<SyntaxNode> listOfParameterNodes,
+            TextSpan parameterSpan,
+            CleanCodeGenerationOptionsProvider fallbackOptions,
+            CancellationToken cancellationToken)
         {
             // List to keep track of the valid parameters
             var listOfParametersOrdinals = new List<int>();
@@ -61,59 +66,61 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             {
                 var parameter = (IParameterSymbol)semanticModel.GetRequiredDeclaredSymbol(parameterNode, cancellationToken);
                 if (ParameterValidForNullCheck(document, parameter, semanticModel, blockStatementOpt, cancellationToken))
-                {
                     listOfParametersOrdinals.Add(parameter.Ordinal);
-                }
             }
 
             // Min 2 parameters to offer the refactoring
             if (listOfParametersOrdinals.Count < 2)
-            {
                 return ImmutableArray<CodeAction>.Empty;
-            }
 
             // Great.  The list has parameters that need null checks. Offer to add null checks for all.
-            return ImmutableArray.Create<CodeAction>(new MyCodeAction(
+            return ImmutableArray.Create<CodeAction>(CodeAction.Create(
                 FeaturesResources.Add_null_checks_for_all_parameters,
-                c => UpdateDocumentForRefactoringAsync(document, blockStatementOpt, listOfParametersOrdinals, parameterSpan, c),
+                c => UpdateDocumentForRefactoringAsync(document, blockStatementOpt, listOfParametersOrdinals, parameterSpan, fallbackOptions, c),
                 nameof(FeaturesResources.Add_null_checks_for_all_parameters)));
         }
 
         protected override async Task<ImmutableArray<CodeAction>> GetRefactoringsForSingleParameterAsync(
-            Document document, IParameterSymbol parameter, SyntaxNode functionDeclaration, IMethodSymbol methodSymbol,
-            IBlockOperation? blockStatementOpt, CancellationToken cancellationToken)
+            Document document,
+            TParameterSyntax parameterSyntax,
+            IParameterSymbol parameter,
+            SyntaxNode functionDeclaration,
+            IMethodSymbol methodSymbol,
+            IBlockOperation? blockStatementOpt,
+            CleanCodeGenerationOptionsProvider fallbackOptions,
+            CancellationToken cancellationToken)
         {
             var semanticModel = await document.GetRequiredSemanticModelAsync(cancellationToken).ConfigureAwait(false);
 
             // Only should provide null-checks for reference types and nullable types.
             if (!ParameterValidForNullCheck(document, parameter, semanticModel, blockStatementOpt, cancellationToken))
-            {
                 return ImmutableArray<CodeAction>.Empty;
-            }
+
+            var simplifierOptions = (TSimplifierOptions)await document.GetSimplifierOptionsAsync(fallbackOptions, cancellationToken).ConfigureAwait(false);
 
             // Great.  There was no null check.  Offer to add one.
-            using var _ = ArrayBuilder<CodeAction>.GetInstance(out var result);
-            result.Add(new MyCodeAction(
+            using var result = TemporaryArray<CodeAction>.Empty;
+            result.Add(CodeAction.Create(
                 FeaturesResources.Add_null_check,
-                c => AddNullCheckAsync(document, parameter, functionDeclaration, methodSymbol, blockStatementOpt, c),
+                cancellationToken => AddNullCheckAsync(document, parameter, functionDeclaration, methodSymbol, blockStatementOpt, simplifierOptions, cancellationToken),
                 nameof(FeaturesResources.Add_null_check)));
 
-            // Also, if this was a string, offer to add the special checks to 
-            // string.IsNullOrEmpty and string.IsNullOrWhitespace.
+            // Also, if this was a string, offer to add the special checks to string.IsNullOrEmpty and
+            // string.IsNullOrWhitespace.
             if (parameter.Type.SpecialType == SpecialType.System_String)
             {
-                result.Add(new MyCodeAction(
+                result.Add(CodeAction.Create(
                     FeaturesResources.Add_string_IsNullOrEmpty_check,
-                    c => AddStringCheckAsync(document, parameter, functionDeclaration, methodSymbol, blockStatementOpt, nameof(string.IsNullOrEmpty), c),
+                    cancellationToken => AddStringCheckAsync(document, parameter, functionDeclaration, methodSymbol, blockStatementOpt, nameof(string.IsNullOrEmpty), simplifierOptions, cancellationToken),
                     nameof(FeaturesResources.Add_string_IsNullOrEmpty_check)));
 
-                result.Add(new MyCodeAction(
+                result.Add(CodeAction.Create(
                     FeaturesResources.Add_string_IsNullOrWhiteSpace_check,
-                    c => AddStringCheckAsync(document, parameter, functionDeclaration, methodSymbol, blockStatementOpt, nameof(string.IsNullOrWhiteSpace), c),
+                    cancellationToken => AddStringCheckAsync(document, parameter, functionDeclaration, methodSymbol, blockStatementOpt, nameof(string.IsNullOrWhiteSpace), simplifierOptions, cancellationToken),
                     nameof(FeaturesResources.Add_string_IsNullOrWhiteSpace_check)));
             }
 
-            return result.ToImmutable();
+            return result.ToImmutableAndClear();
         }
 
         private async Task<Document> UpdateDocumentForRefactoringAsync(
@@ -121,59 +128,62 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             IBlockOperation? blockStatementOpt,
             List<int> listOfParametersOrdinals,
             TextSpan parameterSpan,
+            CleanCodeGenerationOptionsProvider fallbackOptions,
             CancellationToken cancellationToken)
         {
+            TSimplifierOptions? lazySimplifierOptions = null;
+
             foreach (var index in listOfParametersOrdinals)
             {
                 // Updates functionDeclaration and uses it to get the first valid ParameterNode using the ordinals (index).
                 var root = await document.GetRequiredSyntaxRootAsync(cancellationToken).ConfigureAwait(false);
 
                 var firstParameterNode = (TParameterSyntax)root.FindNode(parameterSpan);
-                var functionDeclaration = firstParameterNode.FirstAncestorOrSelf(_isFunctionDeclarationFunc);
+                var functionDeclaration = firstParameterNode.FirstAncestorOrSelf<SyntaxNode>(IsFunctionDeclaration);
                 if (functionDeclaration == null)
                     continue;
 
                 var generator = SyntaxGenerator.GetGenerator(document);
-                var parameterNodes = generator.GetParameters(functionDeclaration);
+                var parameterNodes = (IReadOnlyList<TParameterSyntax>)generator.GetParameters(functionDeclaration);
                 var semanticModel = await document.GetRequiredSemanticModelAsync(cancellationToken).ConfigureAwait(false);
-                var parameter = GetParameterAtOrdinal(index, parameterNodes, semanticModel, cancellationToken);
+                var (parameterSyntax, parameter) = GetParameterAtOrdinal(index, parameterNodes, semanticModel, cancellationToken);
                 if (parameter == null)
                     continue;
+                Contract.ThrowIfNull(parameterSyntax);
 
                 var syntaxFacts = document.GetRequiredLanguageService<ISyntaxFactsService>();
 
                 if (!CanOfferRefactoring(functionDeclaration, semanticModel, syntaxFacts, cancellationToken, out blockStatementOpt))
-                {
                     continue;
-                }
 
-                // If parameter is a string, default check would be IsNullOrEmpty. This is because IsNullOrEmpty is more commonly used in this regard according to telemetry and UX testing.
+                lazySimplifierOptions ??= (TSimplifierOptions)await document.GetSimplifierOptionsAsync(fallbackOptions, cancellationToken).ConfigureAwait(false);
+
+                // If parameter is a string, default check would be IsNullOrEmpty. This is because IsNullOrEmpty is more
+                // commonly used in this regard according to telemetry and UX testing.
                 if (parameter.Type.SpecialType == SpecialType.System_String)
                 {
-                    document = await AddStringCheckAsync(document, parameter, functionDeclaration, (IMethodSymbol)parameter.ContainingSymbol, blockStatementOpt, nameof(string.IsNullOrEmpty), cancellationToken).ConfigureAwait(false);
+                    document = await AddStringCheckAsync(document, parameter, functionDeclaration, (IMethodSymbol)parameter.ContainingSymbol, blockStatementOpt, nameof(string.IsNullOrEmpty), lazySimplifierOptions, cancellationToken).ConfigureAwait(false);
                     continue;
                 }
 
                 // For all other parameters, add null check - updates document
                 document = await AddNullCheckAsync(document, parameter, functionDeclaration,
-                    (IMethodSymbol)parameter.ContainingSymbol, blockStatementOpt, cancellationToken).ConfigureAwait(false);
+                    (IMethodSymbol)parameter.ContainingSymbol, blockStatementOpt, lazySimplifierOptions, cancellationToken).ConfigureAwait(false);
             }
 
             return document;
         }
 
-        private static IParameterSymbol? GetParameterAtOrdinal(int index, IReadOnlyList<SyntaxNode> parameterNodes, SemanticModel semanticModel, CancellationToken cancellationToken)
+        private static (TParameterSyntax?, IParameterSymbol?) GetParameterAtOrdinal(int index, IReadOnlyList<TParameterSyntax> parameterNodes, SemanticModel semanticModel, CancellationToken cancellationToken)
         {
             foreach (var parameterNode in parameterNodes)
             {
                 var parameter = (IParameterSymbol)semanticModel.GetRequiredDeclaredSymbol(parameterNode, cancellationToken);
                 if (index == parameter.Ordinal)
-                {
-                    return parameter;
-                }
+                    return (parameterNode, parameter);
             }
 
-            return null;
+            return default;
         }
 
         private static bool ContainsNullCoalesceCheck(
@@ -206,7 +216,7 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             if (statement is IConditionalOperation ifStatement)
             {
                 var condition = ifStatement.Condition;
-                condition = UnwrapImplicitConversion(condition);
+                condition = condition.UnwrapImplicitConversion();
 
                 if (condition is IBinaryOperation binaryOperator)
                 {
@@ -222,9 +232,7 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
                 {
                     // Look for code of the form "if (p is null)"
                     if (IsNullCheck(constantPattern.Value, isPatternOperation.Value, parameter))
-                    {
                         return true;
-                    }
                 }
                 else if (parameter.Type.SpecialType == SpecialType.System_String &&
                          IsStringCheck(condition, parameter))
@@ -243,9 +251,7 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             {
                 // Don't add null checks to things explicitly declared nullable
                 if (parameter.Type.NullableAnnotation == NullableAnnotation.Annotated)
-                {
                     return false;
-                }
             }
             else if (!parameter.Type.IsNullable())
             {
@@ -253,9 +259,10 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             }
 
             if (parameter.RefKind == RefKind.Out)
-            {
                 return false;
-            }
+
+            if (parameter.IsDiscard)
+                return false;
 
             var syntaxFacts = document.GetRequiredLanguageService<ISyntaxFactsService>();
 
@@ -268,16 +275,12 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             if (blockStatementOpt != null)
             {
                 if (!CanOffer(blockStatementOpt.Syntax))
-                {
                     return false;
-                }
 
                 foreach (var statement in blockStatementOpt.Operations)
                 {
                     if (IsIfNullCheck(statement, parameter))
-                    {
                         return false;
-                    }
 
                     if (ContainsNullCoalesceCheck(
                             syntaxFacts, semanticModel, statement,
@@ -299,39 +302,35 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             {
                 var targetMethod = invocation.TargetMethod;
                 if (targetMethod?.Name is nameof(string.IsNullOrEmpty) or nameof(string.IsNullOrWhiteSpace))
-                {
                     return targetMethod.ContainingType.SpecialType == SpecialType.System_String;
-                }
             }
 
             return false;
         }
 
         private static bool IsNullCheck(IOperation operand1, IOperation operand2, IParameterSymbol parameter)
-            => UnwrapImplicitConversion(operand1).IsNullLiteral() && IsParameterReference(operand2, parameter);
+            => operand1.UnwrapImplicitConversion().IsNullLiteral() && IsParameterReference(operand2, parameter);
 
         private async Task<Document> AddNullCheckAsync(
             Document document,
             IParameterSymbol parameter,
             SyntaxNode functionDeclaration,
             IMethodSymbol method,
-            IBlockOperation? blockStatementOpt,
+            IBlockOperation? blockStatement,
+            TSimplifierOptions options,
             CancellationToken cancellationToken)
         {
             // First see if we can convert a statement of the form "this.s = s" into "this.s = s ?? throw ...".
-            var documentOpt = await TryAddNullCheckToAssignmentAsync(
-                document, parameter, blockStatementOpt, cancellationToken).ConfigureAwait(false);
+            var modifiedDocument = await TryAddNullCheckToAssignmentAsync(
+                document, parameter, blockStatement, options, cancellationToken).ConfigureAwait(false);
 
-            if (documentOpt != null)
-            {
-                return documentOpt;
-            }
+            if (modifiedDocument != null)
+                return modifiedDocument;
 
             // If we can't, then just offer to add an "if (s == null)" statement.
-            var optionSet = await document.GetOptionsAsync(cancellationToken).ConfigureAwait(false);
             return await AddNullCheckStatementAsync(
-                document, parameter, functionDeclaration, method, blockStatementOpt,
-                (s, g) => CreateNullCheckStatement(optionSet, s, g, parameter),
+                document, parameter, functionDeclaration, method, blockStatement,
+                (s, g) => CreateNullCheckStatement(s, g, parameter, options),
                 cancellationToken).ConfigureAwait(false);
         }
 
@@ -342,12 +341,12 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             IMethodSymbol method,
             IBlockOperation? blockStatementOpt,
             string methodName,
+            TSimplifierOptions options,
             CancellationToken cancellationToken)
         {
-            var optionSet = await document.GetOptionsAsync(cancellationToken).ConfigureAwait(false);
             return await AddNullCheckStatementAsync(
                 document, parameter, functionDeclaration, method, blockStatementOpt,
-                (s, g) => CreateStringCheckStatement(optionSet, s.Compilation, g, parameter, methodName),
+                (s, g) => CreateStringCheckStatement(s.Compilation, g, parameter, methodName, options),
                 cancellationToken).ConfigureAwait(false);
         }
 
@@ -356,14 +355,14 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             IParameterSymbol parameter,
             SyntaxNode functionDeclaration,
             IMethodSymbol method,
-            IBlockOperation? blockStatementOpt,
+            IBlockOperation? blockStatement,
             Func<SemanticModel, SyntaxGenerator, TStatementSyntax> generateNullCheck,
             CancellationToken cancellationToken)
         {
             var semanticModel = await document.GetRequiredSemanticModelAsync(cancellationToken).ConfigureAwait(false);
             var root = await document.GetRequiredSyntaxRootAsync(cancellationToken).ConfigureAwait(false);
 
-            var editor = new SyntaxEditor(root, document.Project.Solution.Workspace);
+            var editor = new SyntaxEditor(root, document.Project.Solution.Services);
             var nullCheckStatement = generateNullCheck(semanticModel, editor.Generator);
 
             // We may be inserting a statement into a single-line container.  In that case,
@@ -377,22 +376,21 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             // and assignments in the constructor to match the order of parameters in the method
             // signature.
             var statementToAddAfter = GetStatementToAddNullCheckAfter(
-                semanticModel, parameter, blockStatementOpt, cancellationToken);
+                semanticModel, parameter, blockStatement, cancellationToken);
             InsertStatement(editor, functionDeclaration, method.ReturnsVoid, statementToAddAfter, nullCheckStatement);
 
             var newRoot = editor.GetChangedRoot();
             return document.WithSyntaxRoot(newRoot);
         }
 
-        private TStatementSyntax CreateNullCheckStatement(DocumentOptionSet optionSet, SemanticModel semanticModel, SyntaxGenerator generator, IParameterSymbol parameter)
+        private TStatementSyntax CreateNullCheckStatement(SemanticModel semanticModel, SyntaxGenerator generator, IParameterSymbol parameter, TSimplifierOptions options)
             => CreateParameterCheckIfStatement(
-                optionSet,
                 (TExpressionSyntax)generator.CreateNullCheckExpression(semanticModel, parameter.Name),
-                (TStatementSyntax)generator.CreateThrowArgumentNullExceptionStatement(semanticModel.Compilation, parameter));
+                (TStatementSyntax)generator.CreateThrowArgumentNullExceptionStatement(semanticModel.Compilation, parameter),
+                options);
 
         private TStatementSyntax CreateStringCheckStatement(
-            DocumentOptionSet optionSet, Compilation compilation, SyntaxGenerator generator,
-            IParameterSymbol parameter, string methodName)
+            Compilation compilation, SyntaxGenerator generator, IParameterSymbol parameter, string methodName, TSimplifierOptions options)
         {
             var stringType = compilation.GetSpecialType(SpecialType.System_String);
 
@@ -404,19 +402,17 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
                                 generator.Argument(generator.IdentifierName(parameter.Name)));
             var throwStatement = (TStatementSyntax)generator.ThrowStatement(CreateArgumentException(compilation, generator, parameter, methodName));
 
-            return CreateParameterCheckIfStatement(optionSet, condition, throwStatement);
+            return CreateParameterCheckIfStatement(condition, throwStatement, options);
         }
 
         private static SyntaxNode? GetStatementToAddNullCheckAfter(
             SemanticModel semanticModel,
             IParameterSymbol parameter,
-            IBlockOperation? blockStatementOpt,
+            IBlockOperation? blockStatement,
             CancellationToken cancellationToken)
         {
-            if (blockStatementOpt == null)
-            {
+            if (blockStatement == null)
                 return null;
-            }
 
             var methodSymbol = (IMethodSymbol)parameter.ContainingSymbol;
             var parameterIndex = methodSymbol.Parameters.IndexOf(parameter);
@@ -426,11 +422,9 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             for (var i = parameterIndex - 1; i >= 0; i--)
             {
                 var checkStatement = TryFindParameterCheckStatement(
-                    semanticModel, methodSymbol.Parameters[i], blockStatementOpt, cancellationToken);
+                    semanticModel, methodSymbol.Parameters[i], blockStatement, cancellationToken);
                 if (checkStatement != null)
-                {
                     return checkStatement.Syntax;
-                }
             }
 
             // look for an existing check for a parameter that comes before us.
@@ -438,11 +432,11 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             for (var i = parameterIndex + 1; i < methodSymbol.Parameters.Length; i++)
             {
                 var checkStatement = TryFindParameterCheckStatement(
-                    semanticModel, methodSymbol.Parameters[i], blockStatementOpt, cancellationToken);
+                    semanticModel, methodSymbol.Parameters[i], blockStatement, cancellationToken);
                 if (checkStatement != null)
                 {
-                    var statementIndex = blockStatementOpt.Operations.IndexOf(checkStatement);
-                    return statementIndex > 0 ? blockStatementOpt.Operations[statementIndex - 1].Syntax : null;
+                    var statementIndex = blockStatement.Operations.IndexOf(checkStatement);
+                    return statementIndex > 0 ? blockStatement.Operations[statementIndex - 1].Syntax : null;
                 }
             }
 
@@ -458,19 +452,17 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
         private static IOperation? TryFindParameterCheckStatement(
             SemanticModel semanticModel,
             IParameterSymbol parameterSymbol,
-            IBlockOperation? blockStatementOpt,
+            IBlockOperation? blockStatement,
             CancellationToken cancellationToken)
         {
-            if (blockStatementOpt != null)
+            if (blockStatement != null)
             {
-                foreach (var statement in blockStatementOpt.Operations)
+                foreach (var statement in blockStatement.Operations)
                 {
                     if (statement is IConditionalOperation ifStatement)
                     {
                         if (ContainsParameterReference(semanticModel, ifStatement.Condition, parameterSymbol, cancellationToken))
-                        {
                             return statement;
-                        }
 
                         continue;
                     }
@@ -486,35 +478,30 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
         private async Task<Document?> TryAddNullCheckToAssignmentAsync(
             Document document,
             IParameterSymbol parameter,
-            IBlockOperation? blockStatementOpt,
+            IBlockOperation? blockStatement,
+            TSimplifierOptions options,
             CancellationToken cancellationToken)
         {
             // tries to convert "this.s = s" into "this.s = s ?? throw ...".  Only supported
             // in languages that have a throw-expression, and only if the user has set the
             // preference that they like throw-expressions.
 
-            if (blockStatementOpt == null)
-            {
+            if (blockStatement == null)
                 return null;
-            }
 
             var syntaxTree = await document.GetRequiredSyntaxTreeAsync(cancellationToken).ConfigureAwait(false);
             var syntaxFacts = document.GetRequiredLanguageService<ISyntaxFactsService>();
             if (!syntaxFacts.SupportsThrowExpression(syntaxTree.Options))
-            {
                 return null;
-            }
 
-            var options = await document.GetOptionsAsync(cancellationToken).ConfigureAwait(false);
             if (!PrefersThrowExpression(options))
-            {
                 return null;
-            }
 
             // Look through all the top level statements in the block to see if we can
             // find an existing field/property assignment involving this parameter.
             var containingType = parameter.ContainingType;
-            foreach (var statement in blockStatementOpt.Operations)
+            var semanticModel = await document.GetRequiredSemanticModelAsync(cancellationToken).ConfigureAwait(false);
+            foreach (var statement in blockStatement.Operations)
             {
                 if (IsFieldOrPropertyAssignment(statement, containingType, out var assignmentExpression) &&
                     IsParameterReference(assignmentExpression.Value, parameter))
@@ -531,9 +518,14 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
                         generator.ThrowExpression(
                             CreateArgumentNullException(compilation, generator, parameter)));
 
-                    var newRoot = root.ReplaceNode<SyntaxNode>(assignmentExpression.Value.Syntax, coalesce);
+                    var newRoot = root.ReplaceNode(assignmentExpression.Value.Syntax, coalesce);
                     return document.WithSyntaxRoot(newRoot);
                 }
+
+                // Otherwise, if this statement references the parameter, then we need to stop looking.  We need the
+                // null check to go before the parameter is otherwise used.
+                if (ContainsParameterReference(semanticModel, statement, parameter, cancellationToken))
+                    return null;
             }
 
             return null;
@@ -568,7 +560,7 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
             {
                 nameof(string.IsNullOrEmpty) => new LocalizableResourceString(nameof(FeaturesResources._0_cannot_be_null_or_empty), FeaturesResources.ResourceManager, typeof(FeaturesResources)).ToString(),
                 nameof(string.IsNullOrWhiteSpace) => new LocalizableResourceString(nameof(FeaturesResources._0_cannot_be_null_or_whitespace), FeaturesResources.ResourceManager, typeof(FeaturesResources)).ToString(),
-                _ => throw ExceptionUtilities.Unreachable,
+                _ => throw ExceptionUtilities.Unreachable(),
             };
 
             // The resource string is written to be shown in a UI and is not necessarily valid code, but we're
@@ -608,16 +600,12 @@ namespace Microsoft.CodeAnalysis.InitializeParameter
 
             var index = text.IndexOf(Placeholder);
             if (index < 0)
-            {
                 return default;
-            }
 
             return (text[..index], text[(index + Placeholder.Length)..]);
         }
 
         private static SyntaxNode InterpolatedStringText(SyntaxGenerator generator, string content, string value)
-        {
-            return generator.InterpolatedStringText(generator.InterpolatedStringTextToken(content, value));
-        }
+            => generator.InterpolatedStringText(generator.InterpolatedStringTextToken(content, value));
     }
 }
