@@ -4,28 +4,35 @@
 
 #nullable disable
 
-using System.Collections.Generic;
 using System.Collections.Immutable;
 using System.Linq;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Completion;
-using Microsoft.CodeAnalysis.LanguageServer.Handler;
+using Microsoft.CodeAnalysis.Host;
+using Microsoft.CodeAnalysis.LanguageServer.Handler.Completion;
+using Microsoft.CodeAnalysis.LanguageService;
 using Microsoft.CodeAnalysis.Options;
+using Microsoft.CodeAnalysis.Shared.TestHooks;
 using Microsoft.CodeAnalysis.Text;
 using Microsoft.VisualStudio.LanguageServer.Protocol;
 using Microsoft.VisualStudio.Text.Adornments;
 using Newtonsoft.Json;
 using Roslyn.Test.Utilities;
 using Xunit;
+using Xunit.Abstractions;
 using LSP = Microsoft.VisualStudio.LanguageServer.Protocol;
 
 namespace Microsoft.CodeAnalysis.LanguageServer.UnitTests.Completion
 {
     public class CompletionResolveTests : AbstractLanguageServerProtocolTests
     {
-        [Fact]
-        public async Task TestResolveCompletionItemFromListAsync()
+        public CompletionResolveTests(ITestOutputHelper testOutputHelper) : base(testOutputHelper)
+        {
+        }
+
+        [Theory, CombinatorialData]
+        public async Task TestResolveCompletionItemFromListAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"class A
@@ -35,7 +42,7 @@ namespace Microsoft.CodeAnalysis.LanguageServer.UnitTests.Completion
         {|caret:|}
     }
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
+
             var clientCapabilities = new LSP.VSInternalClientCapabilities
             {
                 SupportsVisualStudioExtensions = true,
@@ -50,22 +57,22 @@ namespace Microsoft.CodeAnalysis.LanguageServer.UnitTests.Completion
                     }
                 }
             };
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace, clientCapabilities);
+
             var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(
                 testLspServer,
-                locations,
-                label: "A",
-                clientCapabilities).ConfigureAwait(false);
+                label: "A").ConfigureAwait(false);
 
             var description = new ClassifiedTextElement(CreateClassifiedTextRunForClass("A"));
-            var expected = CreateResolvedCompletionItem(clientCompletionItem, description, "class A", null);
+            var expected = CreateResolvedCompletionItem(clientCompletionItem, description, null);
 
             var results = (LSP.VSInternalCompletionItem)await RunResolveCompletionItemAsync(
-                testLspServer, clientCompletionItem, clientCapabilities).ConfigureAwait(false);
+                testLspServer, clientCompletionItem).ConfigureAwait(false);
             AssertJsonEquals(expected, results);
         }
 
-        [Fact]
-        public async Task TestResolveCompletionItemAsync()
+        [Theory, CombinatorialData]
+        public async Task TestResolveCompletionItemAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"class A
@@ -75,20 +82,19 @@ namespace Microsoft.CodeAnalysis.LanguageServer.UnitTests.Completion
         {|caret:|}
     }
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
-            var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(testLspServer, locations, label: "A").ConfigureAwait(false);
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace, new LSP.VSInternalClientCapabilities { SupportsVisualStudioExtensions = true });
+            var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(testLspServer, label: "A").ConfigureAwait(false);
 
             var description = new ClassifiedTextElement(CreateClassifiedTextRunForClass("A"));
-            var expected = CreateResolvedCompletionItem(clientCompletionItem, description, "class A", null);
+            var expected = CreateResolvedCompletionItem(clientCompletionItem, description, null);
 
             var results = (LSP.VSInternalCompletionItem)await RunResolveCompletionItemAsync(
                 testLspServer, clientCompletionItem).ConfigureAwait(false);
             AssertJsonEquals(expected, results);
         }
 
-        [Fact]
-        [WorkItem(51125, "https://github.com/dotnet/roslyn/issues/51125")]
-        public async Task TestResolveOverridesCompletionItemAsync()
+        [Theory, CombinatorialData]
+        public async Task TestResolveOverridesCompletionItemAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"abstract class A
@@ -100,8 +106,8 @@ class B : A
 {
     override {|caret:|}
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
-            var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(testLspServer, locations, label: "M()").ConfigureAwait(false);
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace, new LSP.VSInternalClientCapabilities { SupportsVisualStudioExtensions = true });
+            var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(testLspServer, label: "M()").ConfigureAwait(false);
             var results = (LSP.VSInternalCompletionItem)await RunResolveCompletionItemAsync(
                 testLspServer, clientCompletionItem).ConfigureAwait(false);
 
@@ -110,12 +116,11 @@ class B : A
             Assert.Equal(@"public override void M()
     {
         throw new System.NotImplementedException();
-    }", results.TextEdit.NewText);
+    }", results.TextEdit.Value.First.NewText);
         }
 
-        [Fact]
-        [WorkItem(51125, "https://github.com/dotnet/roslyn/issues/51125")]
-        public async Task TestResolveOverridesCompletionItem_SnippetsEnabledAsync()
+        [Theory, CombinatorialData, WorkItem("https://github.com/dotnet/roslyn/issues/51125")]
+        public async Task TestResolveOverridesCompletionItem_SnippetsEnabledAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"abstract class A
@@ -127,7 +132,7 @@ class B : A
 {
     override {|caret:|}
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
+
             // Explicitly enable snippets. This allows us to set the cursor with $0. Currently only applies to C# in Razor docs.
             var clientCapabilities = new LSP.VSInternalClientCapabilities
             {
@@ -143,26 +148,24 @@ class B : A
                     }
                 }
             };
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace, clientCapabilities);
             var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(
                 testLspServer,
-                locations,
-                label: "M()",
-                clientCapabilities).ConfigureAwait(false);
+                label: "M()").ConfigureAwait(false);
 
             var results = (LSP.VSInternalCompletionItem)await RunResolveCompletionItemAsync(
-                testLspServer, clientCompletionItem, clientCapabilities).ConfigureAwait(false);
+                testLspServer, clientCompletionItem).ConfigureAwait(false);
 
             Assert.NotNull(results.TextEdit);
             Assert.Null(results.InsertText);
             Assert.Equal(@"public override void M()
     {
         throw new System.NotImplementedException();$0
-    }", results.TextEdit.NewText);
+    }", results.TextEdit.Value.First.NewText);
         }
 
-        [Fact]
-        [WorkItem(51125, "https://github.com/dotnet/roslyn/issues/51125")]
-        public async Task TestResolveOverridesCompletionItem_SnippetsEnabled_CaretOutOfSnippetScopeAsync()
+        [Theory, CombinatorialData, WorkItem("https://github.com/dotnet/roslyn/issues/51125")]
+        public async Task TestResolveOverridesCompletionItem_SnippetsEnabled_CaretOutOfSnippetScopeAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"abstract class A
@@ -174,13 +177,13 @@ class B : A
 {
     override {|caret:|}
 }";
-            using var testLspServer = CreateTestLspServer(markup, out _);
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace);
 
             var document = testLspServer.GetCurrentSolution().Projects.First().Documents.First();
 
-            var selectedItem = CodeAnalysis.Completion.CompletionItem.Create(displayText: "M");
-            var textEdit = await CompletionResolveHandler.GenerateTextEditAsync(
-                document, new TestCaretOutOfScopeCompletionService(), selectedItem, snippetsSupported: true, CancellationToken.None).ConfigureAwait(false);
+            var selectedItem = CodeAnalysis.Completion.CompletionItem.Create(displayText: "M", isComplexTextEdit: true);
+            var (textEdit, _, _) = await AbstractLspCompletionResultCreationService.GenerateComplexTextEditAsync(
+                document, new TestCaretOutOfScopeCompletionService(testLspServer.TestWorkspace.Services.SolutionServices), selectedItem, snippetsSupported: true, insertNewPositionPlaceholder: true, CancellationToken.None).ConfigureAwait(false);
 
             Assert.Equal(@"public override void M()
     {
@@ -188,8 +191,8 @@ class B : A
     }", textEdit.NewText);
         }
 
-        [Fact]
-        public async Task TestResolveCompletionItemWithMarkupContentAsync()
+        [Theory, CombinatorialData]
+        public async Task TestResolveCompletionItemWithMarkupContentAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"
@@ -224,49 +227,47 @@ class A
         AMet{|caret:|}
     }
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
+            var clientCapabilities = new ClientCapabilities
+            {
+                TextDocument = new TextDocumentClientCapabilities
+                {
+                    Completion = new CompletionSetting
+                    {
+                        CompletionItem = new CompletionItemSetting
+                        {
+                            DocumentationFormat = new MarkupKind[] { MarkupKind.Markdown }
+                        }
+                    }
+                }
+            };
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace, clientCapabilities);
             var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.CompletionItem>(
                 testLspServer,
-                locations,
-                label: "AMethod",
-                new ClientCapabilities()).ConfigureAwait(false);
+                label: "AMethod").ConfigureAwait(false);
             Assert.True(clientCompletionItem is not VSInternalCompletionItem);
 
             var expected = @"```csharp
 void A.AMethod(int i)
 ```
   
-A&nbsp;cref&nbsp;A\.AMethod\(int\)  
-**strong&nbsp;text**  
-_italic&nbsp;text_  
-<u>underline&nbsp;text</u>  
+A cref&nbsp;A\.AMethod\(int\)  
+**strong text**  
+_italic text_  
+<u>underline text</u>  
   
-•&nbsp;Item&nbsp;1\.  
-•&nbsp;Item&nbsp;2\.  
+•&nbsp;Item 1\.  
+•&nbsp;Item 2\.  
   
 [link text](https://google.com)";
 
             var results = await RunResolveCompletionItemAsync(
                 testLspServer,
-                clientCompletionItem,
-                new ClientCapabilities
-                {
-                    TextDocument = new TextDocumentClientCapabilities
-                    {
-                        Completion = new CompletionSetting
-                        {
-                            CompletionItem = new CompletionItemSetting
-                            {
-                                DocumentationFormat = new MarkupKind[] { MarkupKind.Markdown }
-                            }
-                        }
-                    }
-                }).ConfigureAwait(false);
+                clientCompletionItem).ConfigureAwait(false);
             Assert.Equal(expected, results.Documentation.Value.Second.Value);
         }
 
-        [Fact]
-        public async Task TestResolveCompletionItemWithPlainTextAsync()
+        [Theory, CombinatorialData]
+        public async Task TestResolveCompletionItemWithPlainTextAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"
@@ -301,12 +302,10 @@ class A
         AMet{|caret:|}
     }
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace);
             var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.CompletionItem>(
                 testLspServer,
-                locations,
-                label: "AMethod",
-                new ClientCapabilities()).ConfigureAwait(false);
+                label: "AMethod").ConfigureAwait(false);
             Assert.True(clientCompletionItem is not VSInternalCompletionItem);
 
             var expected = @"void A.AMethod(int i)
@@ -322,13 +321,12 @@ link text";
 
             var results = await RunResolveCompletionItemAsync(
                 testLspServer,
-                clientCompletionItem,
-                new ClientCapabilities()).ConfigureAwait(false);
+                clientCompletionItem).ConfigureAwait(false);
             Assert.Equal(expected, results.Documentation.Value.Second.Value);
         }
 
-        [Fact]
-        public async Task TestResolveCompletionItemWithPrefixSuffixAsync()
+        [Theory, CombinatorialData]
+        public async Task TestResolveCompletionItemWithPrefixSuffixAsync(bool mutatingLspWorkspace)
         {
             var markup =
 @"class A
@@ -339,8 +337,8 @@ link text";
         a.{|caret:|}
     }
 }";
-            using var testLspServer = CreateTestLspServer(markup, out var locations);
-            var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(testLspServer, locations, label: "(byte)").ConfigureAwait(false);
+            await using var testLspServer = await CreateTestLspServerAsync(markup, mutatingLspWorkspace, new LSP.VSInternalClientCapabilities { SupportsVisualStudioExtensions = true });
+            var clientCompletionItem = await GetCompletionItemToResolveAsync<LSP.VSInternalCompletionItem>(testLspServer, label: "(byte)").ConfigureAwait(false);
 
             var results = (LSP.VSInternalCompletionItem)await RunResolveCompletionItemAsync(
                 testLspServer, clientCompletionItem).ConfigureAwait(false);
@@ -348,54 +346,79 @@ link text";
             Assert.NotNull(results.Description);
         }
 
-        private static async Task<LSP.CompletionItem> RunResolveCompletionItemAsync(TestLspServer testLspServer, LSP.CompletionItem completionItem, LSP.ClientCapabilities clientCapabilities = null)
+        private static async Task<LSP.CompletionItem> RunResolveCompletionItemAsync(TestLspServer testLspServer, LSP.CompletionItem completionItem)
         {
-            clientCapabilities ??= new LSP.VSInternalClientCapabilities { SupportsVisualStudioExtensions = true };
             return await testLspServer.ExecuteRequestAsync<LSP.CompletionItem, LSP.CompletionItem>(LSP.Methods.TextDocumentCompletionResolveName,
-                           completionItem, clientCapabilities, null, CancellationToken.None);
+                           completionItem, CancellationToken.None);
         }
 
-        private static LSP.VSInternalCompletionItem CreateResolvedCompletionItem(
+        private static VSInternalCompletionItem Clone(VSInternalCompletionItem completionItem)
+        {
+            return new VSInternalCompletionItem()
+            {
+                Label = completionItem.Label,
+                LabelDetails = completionItem.LabelDetails,
+                Kind = completionItem.Kind,
+                Detail = completionItem.Detail,
+                Documentation = completionItem.Documentation,
+                Preselect = completionItem.Preselect,
+                SortText = completionItem.SortText,
+                FilterText = completionItem.FilterText,
+                InsertText = completionItem.InsertText,
+                InsertTextFormat = completionItem.InsertTextFormat,
+                TextEdit = completionItem.TextEdit,
+                TextEditText = completionItem.TextEditText,
+                AdditionalTextEdits = completionItem.AdditionalTextEdits,
+                CommitCharacters = completionItem.CommitCharacters,
+                Command = completionItem.Command,
+                Data = completionItem.Data,
+                Icon = completionItem.Icon,
+                Description = completionItem.Description,
+                VsCommitCharacters = completionItem.VsCommitCharacters,
+                VsResolveTextEditOnCommit = completionItem.VsResolveTextEditOnCommit,
+            };
+        }
+
+        private static VSInternalCompletionItem CreateResolvedCompletionItem(
             VSInternalCompletionItem completionItem,
             ClassifiedTextElement description,
-            string detail,
             string documentation)
         {
-            completionItem.Detail = detail;
+            var expectedCompletionItem = Clone(completionItem);
+
             if (documentation != null)
             {
-                completionItem.Documentation = new LSP.MarkupContent()
+                expectedCompletionItem.Documentation = new MarkupContent()
                 {
                     Kind = LSP.MarkupKind.PlainText,
                     Value = documentation
                 };
             }
 
-            completionItem.Description = description;
-            return completionItem;
+            expectedCompletionItem.Description = description;
+            return expectedCompletionItem;
         }
 
         private static ClassifiedTextRun[] CreateClassifiedTextRunForClass(string className)
             => new ClassifiedTextRun[]
             {
+                new ClassifiedTextRun("whitespace", string.Empty),
                 new ClassifiedTextRun("keyword", "class"),
                 new ClassifiedTextRun("whitespace", " "),
-                new ClassifiedTextRun("class name", className)
+                new ClassifiedTextRun("class name", className),
+                new ClassifiedTextRun("whitespace", string.Empty),
             };
 
         private static async Task<T> GetCompletionItemToResolveAsync<T>(
             TestLspServer testLspServer,
-            Dictionary<string, IList<LSP.Location>> locations,
-            string label,
-            LSP.ClientCapabilities clientCapabilities = null) where T : LSP.CompletionItem
+            string label) where T : LSP.CompletionItem
         {
             var completionParams = CreateCompletionParams(
-                locations["caret"].Single(), LSP.VSInternalCompletionInvokeKind.Explicit, "\0", LSP.CompletionTriggerKind.Invoked);
+                testLspServer.GetLocations("caret").Single(), LSP.VSInternalCompletionInvokeKind.Explicit, "\0", LSP.CompletionTriggerKind.Invoked);
 
-            clientCapabilities ??= new LSP.VSInternalClientCapabilities { SupportsVisualStudioExtensions = true };
-            var completionList = await RunGetCompletionsAsync(testLspServer, completionParams, clientCapabilities);
+            var completionList = await RunGetCompletionsAsync(testLspServer, completionParams);
 
-            if (clientCapabilities.HasCompletionListDataCapability())
+            if (testLspServer.ClientCapabilities.HasCompletionListDataCapability())
             {
                 var vsCompletionList = Assert.IsAssignableFrom<VSInternalCompletionList>(completionList);
                 Assert.NotNull(vsCompletionList.Data);
@@ -408,14 +431,13 @@ link text";
 
         private static async Task<LSP.CompletionList> RunGetCompletionsAsync(
             TestLspServer testLspServer,
-            LSP.CompletionParams completionParams,
-            LSP.ClientCapabilities clientCapabilities)
+            LSP.CompletionParams completionParams)
         {
             var completionList = await testLspServer.ExecuteRequestAsync<LSP.CompletionParams, LSP.CompletionList>(LSP.Methods.TextDocumentCompletionName,
-                completionParams, clientCapabilities, null, CancellationToken.None);
+                completionParams, CancellationToken.None);
 
             // Emulate client behavior of promoting "Data" completion list properties onto completion items.
-            if (clientCapabilities.HasCompletionListDataCapability() &&
+            if (testLspServer.ClientCapabilities.HasCompletionListDataCapability() &&
                 completionList is VSInternalCompletionList vsCompletionList &&
                 vsCompletionList.Data != null)
             {
@@ -438,16 +460,19 @@ link text";
 
         private class TestCaretOutOfScopeCompletionService : CompletionService
         {
+            public TestCaretOutOfScopeCompletionService(SolutionServices services) : base(services, AsynchronousOperationListenerProvider.NullProvider)
+            {
+            }
+
             public override string Language => LanguageNames.CSharp;
 
-            public override Task<CodeAnalysis.Completion.CompletionList> GetCompletionsAsync(
-                Document document,
+            internal override Task<CodeAnalysis.Completion.CompletionList> GetCompletionsAsync(Document document,
                 int caretPosition,
+                CodeAnalysis.Completion.CompletionOptions options,
+                OptionSet passThroughOptions,
                 CompletionTrigger trigger = default,
                 ImmutableHashSet<string> roles = null,
-                OptionSet options = null,
-                CancellationToken cancellationToken = default)
-                => Task.FromResult(CodeAnalysis.Completion.CompletionList.Empty);
+                CancellationToken cancellationToken = default) => Task.FromResult(CodeAnalysis.Completion.CompletionList.Empty);
 
             public override Task<CompletionChange> GetChangeAsync(
                 Document document,
@@ -462,6 +487,15 @@ link text";
 
                 return Task.FromResult(CompletionChange.Create(textChange, newPosition: 0));
             }
+
+            internal override bool ShouldTriggerCompletion(Project project, LanguageServices languageServices, SourceText text, int caretPosition, CompletionTrigger trigger, CodeAnalysis.Completion.CompletionOptions options, OptionSet passthroughOptions, ImmutableHashSet<string> roles = null)
+                => false;
+
+            internal override CompletionRules GetRules(CodeAnalysis.Completion.CompletionOptions options)
+                => CompletionRules.Default;
+
+            internal override Task<CompletionDescription> GetDescriptionAsync(Document document, CodeAnalysis.Completion.CompletionItem item, CodeAnalysis.Completion.CompletionOptions options, SymbolDescriptionOptions displayOptions, CancellationToken cancellationToken = default)
+                => Task.FromResult(CompletionDescription.Empty);
         }
     }
 }
