@@ -4,9 +4,8 @@
 
 using System;
 using System.Xml.Linq;
-using Microsoft.CodeAnalysis;
+using Microsoft.CodeAnalysis.CodeStyle;
 using Microsoft.CodeAnalysis.Diagnostics.Analyzers.NamingStyles;
-using Microsoft.CodeAnalysis.Simplification;
 
 namespace Microsoft.VisualStudio.LanguageServices.CSharp.Options
 {
