@@ -8,7 +8,7 @@ using System;
 using System.Collections.Generic;
 using System.Windows.Data;
 using Microsoft.CodeAnalysis;
-using Microsoft.CodeAnalysis.AddImports;
+using Microsoft.CodeAnalysis.AddImport;
 using Microsoft.CodeAnalysis.CodeStyle;
 using Microsoft.CodeAnalysis.CSharp.CodeStyle;
 using Microsoft.VisualStudio.LanguageServices.Implementation.Options;
@@ -958,6 +958,71 @@ class Customer2
     }}
 //]
 }}
+";
+
+        private static readonly string s_preferMethodGroupConversion = $@"
+using System;
+
+class Customer1
+{{
+    public void M()
+    {{
+//[
+        // {ServicesVSResources.Prefer_colon}
+        Action<object> writeObject = Console.Write;
+//]
+    }}
+}}
+class Customer2
+{{
+    public void M()
+    {{
+//[
+        // {ServicesVSResources.Over_colon}
+        Action<object> writeObject = obj => Console.Write(obj);
+//]
+    }}
+//]
+}}
+";
+
+        private static readonly string s_preferTopLevelStatements = $@"
+using System;
+//[
+// {ServicesVSResources.Prefer_colon}
+Console.WriteLine(""Hello, World!"");
+//]
+
+//[
+// {ServicesVSResources.Over_colon}
+internal class Program
+{{
+    private static void Main(string[] args)
+    {{
+        Console.WriteLine(""Hello, World!"");
+    }}
+}}
+//]
+";
+
+        private static readonly string s_preferProgramMain = $@"
+using System;
+
+//[
+// {ServicesVSResources.Prefer_colon}
+internal class Program
+{{
+    private static void Main(string[] args)
+    {{
+        Console.WriteLine(""Hello, World!"");
+    }}
+}}
+//]
+
+//[
+// {ServicesVSResources.Over_colon}
+Console.WriteLine(""Hello, World!"");
+//]
 ";
 
         private static readonly string s_preferLocalFunctionOverAnonymousFunction = $@"
@@ -1068,6 +1133,30 @@ class Customer
 //]
     }}
 }}
+";
+
+        private static readonly string s_preferTupleSwap = $@"
+using System;
+
+class Customer
+{{
+    void M1(string[] args)
+    {{
+//[
+        // {ServicesVSResources.Prefer_colon}
+        (args[1], args[0]) = (args[0], args[1]);
+//]
+    }}
+    void M2(string[] args)
+    {{
+//[
+        // {ServicesVSResources.Over_colon}
+        var temp = args[0];
+        args[0] = args[1];
+        args[1] = temp;
+//]
+    }}
+}}
 ";
 
         private static readonly string s_preferIsNullOverReferenceEquals = $@"
@@ -1413,6 +1502,23 @@ class Customer2
 //]
 " };
 
+        private static readonly string s_preferReadOnlyStruct = $@"
+//[
+// {ServicesVSResources.Prefer_colon}
+readonly struct Point
+{{
+    public readonly int X, Y;
+}}
+//]
+//[
+// {ServicesVSResources.Over_colon}
+struct Point
+{{
+    public readonly int X, Y;
+}}
+//]
+";
+
         private static readonly string s_preferStaticLocalFunction = $@"
 class Customer1
 {{
@@ -1442,6 +1548,10 @@ class Customer2
 }}
 ";
 
+        #endregion
+
+        #region New Line Preferences
+
         private static readonly string s_allow_embedded_statements_on_same_line_true = $@"
 class Class2
 {{
@@ -1631,7 +1741,7 @@ class Class2
 }}
 ";
 
-        private static readonly string s_allow_bank_line_after_colon_in_constructor_initializer_true = $@"
+        private static readonly string s_allow_blank_line_after_colon_in_constructor_initializer_true = $@"
 class Class
 {{
 //[
@@ -1644,7 +1754,7 @@ class Class
 }}
 ";
 
-        private static readonly string s_allow_bank_line_after_colon_in_constructor_initializer_false = $@"
+        private static readonly string s_allow_blank_line_after_colon_in_constructor_initializer_false = $@"
 namespace NS1
 {{
     class Class
@@ -1671,6 +1781,90 @@ namespace NS2
     //]
     }}
 }}
+";
+
+        private static readonly string s_allow_blank_line_after_token_in_conditional_expression_true = $@"
+class Class
+{{
+//[
+    // {ServicesVSResources.Allow_colon}
+    void Method()
+    {{
+        var result = CheckCondition() ?
+            WhenTrue :
+            WhenFalse;
+    }}
+//]
+}}
+";
+
+        private static readonly string s_allow_blank_line_after_token_in_conditional_expression_false = $@"
+namespace NS1
+{{
+    class Class
+    {{
+    //[
+        // {ServicesVSResources.Require_colon}
+        void Method()
+        {{
+            var result = CheckCondition()
+                ? WhenTrue
+                : WhenFalse;
+        }}
+    //]
+    }}
+}}
+namespace NS2
+{{
+    class Class
+    {{
+    //[
+        // {ServicesVSResources.Over_colon}
+        void Method()
+        {{
+            var result = CheckCondition() ?
+                WhenTrue :
+                WhenFalse;
+        }}
+    //]
+    }}
+}}
+";
+
+        private static readonly string s_allow_blank_line_after_token_in_arrow_expression_clause_true = $@"
+class Class
+{{
+//[
+    // {ServicesVSResources.Allow_colon}
+    int Method() =>
+        1 + 1;
+//]
+}}
+";
+
+        private static readonly string s_allow_blank_line_after_token_in_arrow_expression_clause_false = $@"
+namespace NS1
+{{
+    class Class
+    {{
+    //[
+        // {ServicesVSResources.Require_colon}
+        int Method()
+            => 1 + 1;
+    //]
+    }}
+}}
+namespace NS2
+{{
+    class Class
+    {{
+    //[
+        // {ServicesVSResources.Over_colon}
+        int Method() =>
+            1 + 1;
+    //]
+    }}
+}}
 ";
 
         #endregion
@@ -2042,6 +2236,8 @@ class C2
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CodeStyleOptions2.PreferAutoProperties, ServicesVSResources.analyzer_Prefer_auto_properties, s_preferAutoProperties, s_preferAutoProperties, this, optionStore, codeBlockPreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferSimpleUsingStatement, ServicesVSResources.Prefer_simple_using_statement, s_preferSimpleUsingStatement, s_preferSimpleUsingStatement, this, optionStore, codeBlockPreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CodeStyleOptions2.PreferSystemHashCode, ServicesVSResources.Prefer_System_HashCode_in_GetHashCode, s_preferSystemHashCode, s_preferSystemHashCode, this, optionStore, codeBlockPreferencesGroupTitle));
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferMethodGroupConversion, ServicesVSResources.Prefer_method_group_conversion, s_preferMethodGroupConversion, s_preferMethodGroupConversion, this, optionStore, codeBlockPreferencesGroupTitle));
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferTopLevelStatements, ServicesVSResources.Prefer_top_level_statements, s_preferTopLevelStatements, s_preferProgramMain, this, optionStore, codeBlockPreferencesGroupTitle));
 
             AddParenthesesOptions(OptionStore);
 
@@ -2063,15 +2259,17 @@ class C2
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferIndexOperator, ServicesVSResources.Prefer_index_operator, s_preferIndexOperator, s_preferIndexOperator, this, optionStore, expressionPreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferRangeOperator, ServicesVSResources.Prefer_range_operator, s_preferRangeOperator, s_preferRangeOperator, this, optionStore, expressionPreferencesGroupTitle));
 
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferTupleSwap, ServicesVSResources.Prefer_tuple_swap, s_preferTupleSwap, s_preferTupleSwap, this, optionStore, expressionPreferencesGroupTitle));
+
+            AddExpressionBodyOptions(optionStore, expressionPreferencesGroupTitle);
+            AddUnusedValueOptions(optionStore, expressionPreferencesGroupTitle);
+
             // Pattern matching
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferPatternMatching, CSharpVSResources.Prefer_pattern_matching, s_preferPatternMatching, s_preferPatternMatching, this, optionStore, patternMatchingPreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferPatternMatchingOverIsWithCastCheck, CSharpVSResources.Prefer_pattern_matching_over_is_with_cast_check, s_preferPatternMatchingOverIsWithCastCheck, s_preferPatternMatchingOverIsWithCastCheck, this, optionStore, patternMatchingPreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferPatternMatchingOverAsWithNullCheck, CSharpVSResources.Prefer_pattern_matching_over_as_with_null_check, s_preferPatternMatchingOverAsWithNullCheck, s_preferPatternMatchingOverAsWithNullCheck, this, optionStore, patternMatchingPreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferNotPattern, CSharpVSResources.Prefer_pattern_matching_over_mixed_type_check, s_preferPatternMatchingOverMixedTypeCheck, s_preferPatternMatchingOverMixedTypeCheck, this, optionStore, patternMatchingPreferencesGroupTitle));
 
-            AddExpressionBodyOptions(optionStore, expressionPreferencesGroupTitle);
-            AddUnusedValueOptions(optionStore, expressionPreferencesGroupTitle);
-
             // Variable preferences
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferInlinedVariableDeclaration, ServicesVSResources.Prefer_inlined_variable_declaration, s_preferInlinedVariableDeclaration, s_preferInlinedVariableDeclaration, this, optionStore, variablePreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferDeconstructedVariableDeclaration, ServicesVSResources.Prefer_deconstructed_variable_declaration, s_preferDeconstructedVariableDeclaration, s_preferDeconstructedVariableDeclaration, this, optionStore, variablePreferencesGroupTitle));
@@ -2092,6 +2290,7 @@ class C2
 
             // Modifier preferences.
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CodeStyleOptions2.PreferReadonly, ServicesVSResources.Prefer_readonly_fields, s_preferReadonly, s_preferReadonly, this, optionStore, modifierGroupTitle));
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferReadOnlyStruct, ServicesVSResources.Prefer_read_only_struct, s_preferReadOnlyStruct, s_preferReadOnlyStruct, this, optionStore, modifierGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.PreferStaticLocalFunction, ServicesVSResources.Prefer_static_local_functions, s_preferStaticLocalFunction, s_preferStaticLocalFunction, this, optionStore, modifierGroupTitle));
 
             // Parameter preferences
@@ -2102,31 +2301,33 @@ class C2
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.AllowEmbeddedStatementsOnSameLine, CSharpVSResources.Allow_embedded_statements_on_same_line, s_allow_embedded_statements_on_same_line_true, s_allow_embedded_statements_on_same_line_false, this, optionStore, newLinePreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.AllowBlankLinesBetweenConsecutiveBraces, CSharpVSResources.Allow_blank_lines_between_consecutive_braces, s_allow_blank_line_between_consecutive_braces_true, s_allow_blank_line_between_consecutive_braces_false, this, optionStore, newLinePreferencesGroupTitle));
             CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CodeStyleOptions2.AllowStatementImmediatelyAfterBlock, ServicesVSResources.Allow_statement_immediately_after_block, s_allow_statement_immediately_after_block_true, s_allow_statement_immediately_after_block_false, this, optionStore, newLinePreferencesGroupTitle));
-            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.AllowBlankLineAfterColonInConstructorInitializer, CSharpVSResources.Allow_bank_line_after_colon_in_constructor_initializer, s_allow_bank_line_after_colon_in_constructor_initializer_true, s_allow_bank_line_after_colon_in_constructor_initializer_false, this, optionStore, newLinePreferencesGroupTitle));
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.AllowBlankLineAfterColonInConstructorInitializer, CSharpVSResources.Allow_blank_line_after_colon_in_constructor_initializer, s_allow_blank_line_after_colon_in_constructor_initializer_true, s_allow_blank_line_after_colon_in_constructor_initializer_false, this, optionStore, newLinePreferencesGroupTitle));
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.AllowBlankLineAfterTokenInConditionalExpression, CSharpVSResources.Allow_blank_line_after_token_in_conditional_expression, s_allow_blank_line_after_token_in_conditional_expression_true, s_allow_blank_line_after_token_in_conditional_expression_false, this, optionStore, newLinePreferencesGroupTitle));
+            CodeStyleItems.Add(new BooleanCodeStyleOptionViewModel(CSharpCodeStyleOptions.AllowBlankLineAfterTokenInArrowExpressionClause, CSharpVSResources.Allow_blank_line_after_token_in_arrow_expression_clause, s_allow_blank_line_after_token_in_arrow_expression_clause_true, s_allow_blank_line_after_token_in_arrow_expression_clause_false, this, optionStore, newLinePreferencesGroupTitle));
         }
 
         private void AddParenthesesOptions(OptionStore optionStore)
         {
             AddParenthesesOption(
-                LanguageNames.CSharp, optionStore, CodeStyleOptions2.ArithmeticBinaryParentheses,
+                optionStore, CodeStyleOptions2.ArithmeticBinaryParentheses,
                 CSharpVSResources.In_arithmetic_binary_operators,
                 new[] { s_arithmeticBinaryAlwaysForClarity, s_arithmeticBinaryNeverIfUnnecessary },
                 defaultAddForClarity: true);
 
             AddParenthesesOption(
-                LanguageNames.CSharp, optionStore, CodeStyleOptions2.OtherBinaryParentheses,
+                optionStore, CodeStyleOptions2.OtherBinaryParentheses,
                 CSharpVSResources.In_other_binary_operators,
                 new[] { s_otherBinaryAlwaysForClarity, s_otherBinaryNeverIfUnnecessary },
                 defaultAddForClarity: true);
 
             AddParenthesesOption(
-                LanguageNames.CSharp, optionStore, CodeStyleOptions2.RelationalBinaryParentheses,
+                optionStore, CodeStyleOptions2.RelationalBinaryParentheses,
                 CSharpVSResources.In_relational_binary_operators,
                 new[] { s_relationalBinaryAlwaysForClarity, s_relationalBinaryNeverIfUnnecessary },
                 defaultAddForClarity: true);
 
             AddParenthesesOption(
-                LanguageNames.CSharp, optionStore, CodeStyleOptions2.OtherParentheses,
+                optionStore, CodeStyleOptions2.OtherParentheses,
                 ServicesVSResources.In_other_operators,
                 new[] { s_otherParenthesesAlwaysForClarity, s_otherParenthesesNeverIfUnnecessary },
                 defaultAddForClarity: false);
@@ -2280,7 +2481,7 @@ class C2
                 s_avoidUnusedParametersAllMethods
             };
 
-            AddUnusedParameterOption(LanguageNames.CSharp, optionStore, parameterPreferencesGroupTitle, examples);
+            AddUnusedParameterOption(optionStore, parameterPreferencesGroupTitle, examples);
         }
     }
 }
