@@ -7,17 +7,23 @@ using System.Composition;
 using Microsoft.CodeAnalysis.Editor.EditorConfigSettings.Data;
 using Microsoft.CodeAnalysis.Host;
 using Microsoft.CodeAnalysis.Host.Mef;
+using Microsoft.CodeAnalysis.Options;
 
 namespace Microsoft.CodeAnalysis.Editor.EditorConfigSettings.DataProvider.Whitespace
 {
-    [ExportWorkspaceServiceFactory(typeof(IWorkspaceSettingsProviderFactory<WhitespaceSetting>)), Shared]
-    internal class CommonWhitespaceSettingsWorkspaceServiceFactory : IWorkspaceServiceFactory
+    [ExportWorkspaceServiceFactory(typeof(IWorkspaceSettingsProviderFactory<Setting>)), Shared]
+    internal sealed class CommonWhitespaceSettingsWorkspaceServiceFactory : IWorkspaceServiceFactory
     {
+        private readonly IGlobalOptionService _globalOptions;
+
         [ImportingConstructor]
         [Obsolete(MefConstruction.ImportingConstructorMessage, error: true)]
-        public CommonWhitespaceSettingsWorkspaceServiceFactory() { }
+        public CommonWhitespaceSettingsWorkspaceServiceFactory(IGlobalOptionService globalOptions)
+        {
+            _globalOptions = globalOptions;
+        }
 
         public IWorkspaceService CreateService(HostWorkspaceServices workspaceServices)
-            => new CommonWhitespaceSettingsProviderFactory(workspaceServices.Workspace);
+            => new CommonWhitespaceSettingsProviderFactory(workspaceServices.Workspace, _globalOptions);
     }
 }
