@@ -11,11 +11,11 @@ namespace Microsoft.VisualStudio.LanguageServices.EditorConfigSettings.Whitespac
 {
     internal partial class WhitespaceViewModel
     {
-        internal sealed class SettingsSnapshotFactory : SettingsSnapshotFactoryBase<WhitespaceSetting, SettingsEntriesSnapshot>
+        internal sealed class SettingsSnapshotFactory : SettingsSnapshotFactoryBase<Setting, SettingsEntriesSnapshot>
         {
-            public SettingsSnapshotFactory(ISettingsProvider<WhitespaceSetting> data) : base(data) { }
+            public SettingsSnapshotFactory(ISettingsProvider<Setting> data) : base(data) { }
 
-            protected override SettingsEntriesSnapshot CreateSnapshot(ImmutableArray<WhitespaceSetting> data, int currentVersionNumber)
+            protected override SettingsEntriesSnapshot CreateSnapshot(ImmutableArray<Setting> data, int currentVersionNumber)
                 => new(data, currentVersionNumber);
         }
     }
