@@ -18,6 +18,7 @@ using Xunit.Abstractions;
 namespace Roslyn.VisualStudio.IntegrationTests.CSharp
 {
     [Collection(nameof(SharedIntegrationHostFixture))]
+    [Trait(Traits.Feature, Traits.Features.Classification)]
     public class CSharpKeywordHighlighting : AbstractEditorTest
     {
         protected override string LanguageName => LanguageNames.CSharp;
@@ -27,7 +28,7 @@ namespace Roslyn.VisualStudio.IntegrationTests.CSharp
         {
         }
 
-        [WpfFact, Trait(Traits.Feature, Traits.Features.Classification)]
+        [WpfFact]
         public void Foreach()
         {
             var input = @"class C
@@ -38,7 +39,7 @@ namespace Roslyn.VisualStudio.IntegrationTests.CSharp
     }
 }";
 
-            Roslyn.Test.Utilities.MarkupTestFile.GetSpans(input, out var text, out ImmutableArray<TextSpan> spans);
+            Roslyn.Test.Utilities.MarkupTestFile.GetSpans(input, out var text, out var spans);
 
             VisualStudio.Editor.SetText(text);
 
@@ -48,7 +49,7 @@ namespace Roslyn.VisualStudio.IntegrationTests.CSharp
             Verify("in", ImmutableArray.Create<TextSpan>());
         }
 
-        [WpfFact, Trait(Traits.Feature, Traits.Features.Classification)]
+        [WpfFact]
         public void PreprocessorConditionals()
         {
             var input = @"
@@ -80,7 +81,7 @@ class PurchaseTransaction
             Verify("#endif", spans["else"]);
         }
 
-        [WpfFact, Trait(Traits.Feature, Traits.Features.Classification)]
+        [WpfFact]
         public void PreprocessorRegions()
         {
             var input = @"
@@ -93,10 +94,7 @@ class C
     [|#endregion|]
 }";
 
-            Test.Utilities.MarkupTestFile.GetSpans(
-                input,
-                out var text,
-                out ImmutableArray<TextSpan> spans);
+            MarkupTestFile.GetSpans(input, out var text, out var spans);
 
             VisualStudio.Editor.SetText(text);
 
@@ -110,7 +108,7 @@ class C
             VisualStudio.Workspace.WaitForAllAsyncOperations(
                 Helper.HangMitigatingTimeout,
                 FeatureAttribute.Workspace,
-                FeatureAttribute.SolutionCrawler,
+                FeatureAttribute.SolutionCrawlerLegacy,
                 FeatureAttribute.DiagnosticService,
                 FeatureAttribute.Classification,
                 FeatureAttribute.KeywordHighlighting);
