@@ -14,14 +14,17 @@ using System.Threading.Tasks;
 using System.Windows;
 using System.Windows.Controls;
 using System.Windows.Forms;
-using System.Windows.Media;
 using Microsoft.CodeAnalysis;
 using Microsoft.CodeAnalysis.Editor.Implementation.Highlighting;
 using Microsoft.CodeAnalysis.Editor.Shared.Extensions;
+using Microsoft.CodeAnalysis.UnitTests;
+using Microsoft.VisualStudio.Editor;
 using Microsoft.VisualStudio.IntegrationTest.Utilities.Common;
 using Microsoft.VisualStudio.IntegrationTest.Utilities.Input;
 using Microsoft.VisualStudio.Language.Intellisense;
 using Microsoft.VisualStudio.Language.Intellisense.AsyncCompletion;
+using Microsoft.VisualStudio.PlatformUI;
+using Microsoft.VisualStudio.Shell;
 using Microsoft.VisualStudio.Shell.Interop;
 using Microsoft.VisualStudio.Text;
 using Microsoft.VisualStudio.Text.Classification;
@@ -29,11 +32,10 @@ using Microsoft.VisualStudio.Text.Editor;
 using Microsoft.VisualStudio.Text.Outlining;
 using Microsoft.VisualStudio.Text.Tagging;
 using Microsoft.VisualStudio.TextManager.Interop;
+using Microsoft.VisualStudio.Threading;
 using Microsoft.VisualStudio.Utilities;
 using Roslyn.Utilities;
 using UIAutomationClient;
-using Xunit;
-using ThreadHelper = Microsoft.VisualStudio.Shell.ThreadHelper;
 
 namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
 {
@@ -58,13 +60,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
         protected override IWpfTextView GetActiveTextView()
             => GetActiveTextViewHost().TextView;
 
-        private static IVsTextView GetActiveVsTextView()
-        {
-            var (textView, hr) = TryGetActiveVsTextView();
-            Marshal.ThrowExceptionForHR(hr);
-            return textView;
-        }
-
         private static (IVsTextView textView, int hr) TryGetActiveVsTextView()
         {
             var vsTextManager = GetGlobalService<SVsTextManager, IVsTextManager>();
@@ -141,48 +136,25 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                     throw new InvalidOperationException($"{WellKnownCommandNames.Edit_ToggleCompletionMode} did not leave the editor in the expected state.");
                 }
             }
-        }
 
-        public void VerifyNotSaved()
-        {
-            Contract.ThrowIfTrue(GetDTE().ActiveDocument.ProjectItem.Saved);
-        }
+            if (!value)
+            {
+                // For blocking completion mode, make sure we don't have responsive completion interfering when
+                // integration tests run slowly.
+                ExecuteOnActiveView(view =>
+                {
+                    var options = view.Options.GlobalOptions;
+                    options.SetOptionValue(DefaultOptions.ResponsiveCompletionOptionId, false);
 
-        public string VerifySaved()
-        {
-            var activeDocument = GetDTE().ActiveDocument;
-            Contract.ThrowIfFalse(activeDocument.ProjectItem.Saved);
-            return activeDocument.FullName;
-        }
-
-        public string GetActiveBufferName()
-        {
-            return GetDTE().ActiveDocument.Name;
-        }
-
-        public string GetActiveWindowName()
-        {
-            return GetDTE().ActiveWindow.Caption;
-        }
-
-        public void WaitForActiveView(string expectedView)
-        {
-            using var cts = new CancellationTokenSource(Helper.HangMitigatingTimeout);
-            Retry(_ => GetActiveBufferName(), (actual, _) => actual == expectedView, TimeSpan.FromMilliseconds(100), cts.Token);
-        }
-
-        public void WaitForActiveWindow(string expectedWindow)
-        {
-            using var cts = new CancellationTokenSource(Helper.HangMitigatingTimeout);
-            Retry(_ => GetActiveWindowName(), (actual, _) => actual == expectedWindow, TimeSpan.FromMilliseconds(100), cts.Token);
+                    var latencyGuardOptionKey = new EditorOptionKey<bool>("EnableTypingLatencyGuard");
+                    options.SetOptionValue(latencyGuardOptionKey, false);
+                });
+            }
         }
 
         public void Activate()
             => GetDTE().ActiveDocument.Activate();
 
-        public bool IsProjectItemDirty()
-            => GetDTE().ActiveDocument.ProjectItem.IsDirty;
-
         public string GetText()
             => ExecuteOnActiveView(view => view.TextSnapshot.GetText());
 
@@ -241,7 +213,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 var line = bufferPosition.GetContainingLine();
                 var text = line.GetText();
 
-                return text.Substring(0, bufferPosition.Position - line.Start);
+                return text[..(bufferPosition.Position - line.Start)];
             });
 
         public string GetLineTextAfterCaret()
@@ -252,17 +224,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 var line = bufferPosition.GetContainingLine();
                 var text = line.GetText();
 
-                return text.Substring(bufferPosition.Position - line.Start);
-            });
-
-        public string GetSelectedText()
-            => ExecuteOnActiveView(view =>
-            {
-                var subjectBuffer = view.GetBufferContainingCaret();
-                Contract.ThrowIfNull(subjectBuffer);
-
-                var selectedSpan = view.Selection.SelectedSpans[0];
-                return subjectBuffer.CurrentSnapshot.GetText(selectedSpan);
+                return text[(bufferPosition.Position - line.Start)..];
             });
 
         public void MoveCaret(int position)
@@ -287,9 +249,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 return broker.IsSignatureHelpActive(view);
             });
 
-        public string[] GetErrorTags()
-            => GetTags<IErrorTag>();
-
         public string[] GetHighlightTags()
            => GetTags<ITextMarkerTag>(tag => tag.Type == KeywordHighlightTag.TagId);
 
@@ -302,10 +261,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             bool Filter(TTag tag)
                 => true;
 
-            if (filter == null)
-            {
-                filter = Filter;
-            }
+            filter ??= Filter;
 
             return ExecuteOnActiveView(view =>
             {
@@ -319,24 +275,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             });
         }
 
-        /// <remarks>
-        /// This method does not wait for async operations before
-        /// querying the editor
-        /// </remarks>
-        public Signature[] GetSignatures()
-            => ExecuteOnActiveView(view =>
-            {
-                var broker = GetComponentModelService<ISignatureHelpBroker>();
-
-                var sessions = broker.GetSessions(view);
-                if (sessions.Count != 1)
-                {
-                    throw new InvalidOperationException($"Expected exactly one session in the signature help, but found {sessions.Count}");
-                }
-
-                return sessions[0].Signatures.Select(s => new Signature(s)).ToArray();
-            });
-
         /// <remarks>
         /// This method does not wait for async operations before
         /// querying the editor
@@ -368,9 +306,9 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
 
         public ClassifiedToken[] GetLightbulbPreviewClassifications(string menuText)
         {
-            return ThreadHelper.JoinableTaskFactory.Run(async () =>
+            return JoinableTaskFactory.Run(async () =>
             {
-                await ThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync();
+                await JoinableTaskFactory.SwitchToMainThreadAsync();
 
                 var view = GetActiveTextView();
                 var broker = GetComponentModel().GetService<ILightBulbBroker>();
@@ -405,10 +343,12 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
 
             if (!string.IsNullOrEmpty(menuText))
             {
+#pragma warning disable CS0618 // Type or member is obsolete
                 if (activeSession.TryGetSuggestedActionSets(out var actionSets) != QuerySuggestedActionCompletionStatus.Completed)
                 {
                     actionSets = Array.Empty<SuggestedActionSet>();
                 }
+#pragma warning restore CS0618 // Type or member is obsolete
 
                 var set = actionSets.SelectMany(s => s.Actions).FirstOrDefault(a => a.DisplayText == menuText);
                 if (set == null)
@@ -422,8 +362,8 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 if (pane is System.Windows.Controls.UserControl)
                 {
                     var container = ((System.Windows.Controls.UserControl)pane).FindName("PreviewDockPanel") as DockPanel;
-                    var host = FindDescendants<UIElement>(container).OfType<IWpfTextViewHost>().LastOrDefault();
-                    preview = (host == null) ? null : host.TextView;
+                    var host = container?.FindDescendants<UIElement>().OfType<IWpfTextViewHost>().LastOrDefault();
+                    preview = host?.TextView;
                 }
 
                 if (preview == null)
@@ -441,29 +381,9 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             return Array.Empty<ClassifiedToken>();
         }
 
-        private static IEnumerable<T> FindDescendants<T>(DependencyObject? rootObject) where T : DependencyObject
-        {
-            if (rootObject != null)
-            {
-                for (var i = 0; i < VisualTreeHelper.GetChildrenCount(rootObject); i++)
-                {
-                    var child = VisualTreeHelper.GetChild(rootObject, i);
-
-                    if (child is not null and T)
-                        yield return (T)child;
-
-                    foreach (var descendant in FindDescendants<T>(child))
-                        yield return descendant;
-                }
-            }
-        }
-
-        public void MessageBox(string message)
-            => ExecuteOnActiveView(view => System.Windows.MessageBox.Show(message));
-
         public void VerifyDialog(string dialogAutomationId, bool isOpen)
         {
-            var dialogAutomationElement = DialogHelpers.FindDialogByAutomationId((IntPtr)GetDTE().MainWindow.HWnd, dialogAutomationId, isOpen);
+            var dialogAutomationElement = DialogHelpers.FindDialogByAutomationId(GetDTE().MainWindow.HWnd, dialogAutomationId, isOpen);
 
             if ((isOpen && dialogAutomationElement == null) ||
                 (!isOpen && dialogAutomationElement != null))
@@ -474,19 +394,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
 
         public void DialogSendKeys(string dialogAutomationName, object[] keys)
         {
-            var dialogAutomationElement = DialogHelpers.GetOpenDialogById((IntPtr)GetDTE().MainWindow.HWnd, dialogAutomationName);
-
-            dialogAutomationElement.SetFocus();
-            _sendKeys.Send(keys);
-        }
-
-        public void SendKeysToNavigateTo(object[] keys)
-        {
-            var dialogAutomationElement = FindNavigateTo();
-            if (dialogAutomationElement == null)
-            {
-                throw new InvalidOperationException($"Expected the NavigateTo dialog to be open, but it is not.");
-            }
+            var dialogAutomationElement = DialogHelpers.GetOpenDialogById(GetDTE().MainWindow.HWnd, dialogAutomationName);
 
             dialogAutomationElement.SetFocus();
             _sendKeys.Send(keys);
@@ -494,44 +402,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
 
         public void PressDialogButton(string dialogAutomationName, string buttonAutomationName)
         {
-            DialogHelpers.PressButton((IntPtr)GetDTE().MainWindow.HWnd, dialogAutomationName, buttonAutomationName);
-        }
-
-        private static IUIAutomationElement FindNavigateTo()
-        {
-            var vsAutomationElement = Helper.Automation.ElementFromHandle((IntPtr)GetDTE().MainWindow.HWnd);
-            return vsAutomationElement.FindDescendantByAutomationId("PART_SearchBox");
-        }
-
-        private T Retry<T>(Func<CancellationToken, T> action, Func<T, CancellationToken, bool> stoppingCondition, TimeSpan delay, CancellationToken cancellationToken)
-        {
-            do
-            {
-                cancellationToken.ThrowIfCancellationRequested();
-
-                T retval;
-                try
-                {
-                    retval = action(cancellationToken);
-                }
-                catch (COMException)
-                {
-                    // Devenv can throw COMExceptions if it's busy when we make DTE calls.
-
-                    Thread.Sleep(delay);
-                    continue;
-                }
-
-                if (stoppingCondition(retval, cancellationToken))
-                {
-                    return retval;
-                }
-                else
-                {
-                    Thread.Sleep(delay);
-                }
-            }
-            while (true);
+            DialogHelpers.PressButton(GetDTE().MainWindow.HWnd, dialogAutomationName, buttonAutomationName);
         }
 
         public void AddWinFormButton(string buttonName)
@@ -706,12 +577,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             return properties[propertyName].GetValue(button) as string;
         }
 
-        public void FormatDocumentViaCommand()
-            => ExecuteCommand(WellKnownCommandNames.Edit_FormatDocument);
-
-        public void Paste()
-            => ExecuteCommand(WellKnownCommandNames.Edit_Paste);
-
         public void Undo()
             => ExecuteCommand(WellKnownCommandNames.Edit_Undo);
 
@@ -743,45 +608,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             });
         }
 
-        public List<string> GetF1Keywords()
-        {
-            return InvokeOnUIThread(cancellationToken =>
-            {
-                var results = new List<string>();
-                GetActiveVsTextView().GetBuffer(out var textLines);
-                Marshal.ThrowExceptionForHR(textLines.GetLanguageServiceID(out var languageServiceGuid));
-                Marshal.ThrowExceptionForHR(Microsoft.VisualStudio.Shell.ServiceProvider.GlobalProvider.QueryService(languageServiceGuid, out var languageService));
-                var languageContextProvider = (IVsLanguageContextProvider)languageService;
-
-                var monitorUserContext = GetGlobalService<SVsMonitorUserContext, IVsMonitorUserContext>();
-                Marshal.ThrowExceptionForHR(monitorUserContext.CreateEmptyContext(out var emptyUserContext));
-                Marshal.ThrowExceptionForHR(GetActiveVsTextView().GetCaretPos(out var line, out var column));
-                var span = new TextManager.Interop.TextSpan()
-                {
-                    iStartLine = line,
-                    iStartIndex = column,
-                    iEndLine = line,
-                    iEndIndex = column
-                };
-
-                Marshal.ThrowExceptionForHR(languageContextProvider.UpdateLanguageContext(0, textLines, new[] { span }, emptyUserContext));
-                Marshal.ThrowExceptionForHR(emptyUserContext.CountAttributes("keyword", VSConstants.S_FALSE, out var count));
-                for (var i = 0; i < count; i++)
-                {
-                    emptyUserContext.GetAttribute(i, "keyword", VSConstants.S_FALSE, out var key, out var value);
-                    results.Add(value);
-                }
-
-                return results;
-            });
-        }
-
-        public void GoToDefinition()
-            => ExecuteCommand(WellKnownCommandNames.Edit_GoToDefinition);
-
-        public void GoToImplementation()
-            => ExecuteCommand(WellKnownCommandNames.Edit_GoToImplementation);
-
         /// <summary>
         /// Gets the spans where a particular tag appears in the active text view.
         /// </summary>
@@ -800,11 +626,26 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 return matchingTags.Select(t => t.Span.GetSpans(view.TextBuffer).Single().Span.ToTextSpan()).SelectMany(t => new List<int> { t.Start, t.Length }).ToArray();
             });
 
-        public void SendExplicitFocus()
-            => InvokeOnUIThread(cancellationToken =>
+        public void WaitForEditorOperations(TimeSpan timeout)
+        {
+            var joinableTaskCollection = InvokeOnUIThread(cancellationToken =>
             {
-                var view = GetActiveVsTextView();
-                view.SendExplicitFocus();
+                var shell = GetGlobalService<SVsShell, IVsShell>();
+                if (shell.IsPackageLoaded(DefGuidList.guidEditorPkg, out var editorPackage) == VSConstants.S_OK)
+                {
+                    var asyncPackage = (AsyncPackage)editorPackage;
+                    var collection = asyncPackage.GetPropertyValue<JoinableTaskCollection>("JoinableTaskCollection");
+                    return collection;
+                }
+
+                return null;
             });
+
+            if (joinableTaskCollection is not null)
+            {
+                using var cts = new CancellationTokenSource(timeout);
+                joinableTaskCollection.JoinTillEmptyAsync(cts.Token).Wait(cts.Token);
+            }
+        }
     }
 }
