@@ -81,33 +81,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             vsproject.References.Add(assemblyName);
         }
 
-        public void RemoveMetadataReference(string assemblyName, string projectName)
-        {
-            var project = GetProject(projectName);
-            var reference = ((VSProject)project.Object).References.Cast<Reference>().Where(x => x.Name == assemblyName).First();
-            reference.Remove();
-        }
-
-        public void AddAnalyzerReference(string filePath, string projectName)
-        {
-            var project = GetProject(projectName);
-            var vsProject = (VSProject3)project.Object;
-            vsProject.AnalyzerReferences.Add(filePath);
-        }
-
-        public void RemoveAnalyzerReference(string filePath, string projectName)
-        {
-            var project = GetProject(projectName);
-            ((VSProject3)project.Object).AnalyzerReferences.Remove(filePath);
-        }
-
-        public void SetLanguageVersion(string projectName, string languageVersion)
-        {
-            var project = GetProject(projectName);
-            var projectConfiguration = (CSharpProjectConfigurationProperties3)project.ConfigurationManager.ActiveConfiguration.Object;
-            projectConfiguration.LanguageVersion = languageVersion;
-        }
-
         public string DirectoryName => Path.GetDirectoryName(SolutionFileFullPath);
 
         public string SolutionFileFullPath
@@ -146,64 +119,16 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             Directory.CreateDirectory(solutionPath);
 
             var solution = GetGlobalService<SVsSolution, IVsSolution>();
-            ErrorHandler.ThrowOnFailure(solution.CreateSolution(solutionPath, solutionFileName, (uint)__VSCREATESOLUTIONFLAGS.CSF_SILENT));
-            ErrorHandler.ThrowOnFailure(solution.SaveSolutionElement((uint)__VSSLNSAVEOPTIONS.SLNSAVEOPT_ForceSave, null, 0));
+            InvokeOnUIThread(cancellationToken =>
+            {
+                ErrorHandler.ThrowOnFailure(solution.CreateSolution(solutionPath, solutionFileName, (uint)__VSCREATESOLUTIONFLAGS.CSF_SILENT));
+                ErrorHandler.ThrowOnFailure(solution.SaveSolutionElement((uint)__VSSLNSAVEOPTIONS.SLNSAVEOPT_ForceSave, null, 0));
+            });
 
             _solution = (Solution2)dte.Solution;
             _fileName = Path.Combine(solutionPath, solutionFileName);
         }
 
-        public string[] GetAssemblyReferences(string projectName)
-        {
-            var project = GetProject(projectName);
-            var references = ((VSProject)project.Object).References.Cast<Reference>()
-                .Where(x => x.SourceProject == null)
-                .Select(x => x.Name + "," + x.Version + "," + x.PublicKeyToken).ToArray();
-            return references;
-        }
-
-        public void RenameFile(string projectName, string oldFileName, string newFileName)
-        {
-            var project = GetProject(projectName);
-            var projectDirectory = Path.GetDirectoryName(project.FullName);
-
-            VsShellUtilities.RenameDocument(
-                ServiceProvider.GlobalProvider,
-                Path.Combine(projectDirectory, oldFileName),
-                Path.Combine(projectDirectory, newFileName));
-        }
-
-        public void RenameFileViaDTE(string projectName, string oldFileName, string newFileName)
-        {
-            var projectItem = GetProjectItem(projectName, oldFileName);
-
-            projectItem.Name = newFileName;
-        }
-
-        public void EditProjectFile(string projectName)
-        {
-            var solutionExplorer = ((DTE2)GetDTE()).ToolWindows.SolutionExplorer;
-            solutionExplorer.Parent.Activate();
-            var rootHierarchyItems = solutionExplorer.UIHierarchyItems.Cast<EnvDTE.UIHierarchyItem>();
-            var solution = rootHierarchyItems.First();
-            var solutionHierarchyItems = solution.UIHierarchyItems.Cast<EnvDTE.UIHierarchyItem>();
-            var project = solutionHierarchyItems.Where(x => x.Name == projectName).FirstOrDefault();
-            if (project == null)
-            {
-                throw new ArgumentException($"Could not find project file, current hierarchy items '{string.Join(", ", rootHierarchyItems.Select(x => x.Name))}'");
-            }
-
-            project.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeSelect);
-            ExecuteCommand("Project.EditProjectFile");
-        }
-
-        public string[] GetProjectReferences(string projectName)
-        {
-            var project = GetProject(projectName);
-            var references = ((VSProject)project.Object).References.Cast<Reference>().Where(x => x.SourceProject != null).Select(x => x.Name).ToArray();
-            return references;
-        }
-
         public void CreateSolution(string solutionName, string solutionElementString)
         {
             var solutionElement = XElement.Parse(solutionElementString);
@@ -304,36 +229,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             }
         }
 
-        public void RemoveProjectReference(string projectName, string projectReferenceName)
-        {
-            var project = GetProject(projectName);
-            var vsproject = (VSProject)project.Object;
-            var references = vsproject.References.Cast<Reference>();
-            var reference = references.Where(x => x.ContainingProject != null && x.Name == projectReferenceName).FirstOrDefault();
-            if (reference == null)
-            {
-                var projectReference = references.Where(x => x.ContainingProject != null).Select(x => x.Name);
-                throw new ArgumentException($"reference to project {projectReferenceName} not found, references: '{string.Join(", ", projectReference)}'");
-            }
-
-            reference.Remove();
-        }
-
-        public void OpenSolution(string path, bool saveExistingSolutionIfExists = false)
-        {
-            var dte = GetDTE();
-
-            if (dte.Solution.IsOpen)
-            {
-                CloseSolution(saveExistingSolutionIfExists);
-            }
-
-            dte.Solution.Open(path);
-
-            _solution = (EnvDTE80.Solution2)dte.Solution;
-            _fileName = path;
-        }
-
         private static string ConvertLanguageName(string languageName)
         {
             const string CSharp = nameof(CSharp);
@@ -360,18 +255,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             _solution.AddFromTemplate(projectTemplatePath, projectPath, projectName, Exclusive: false);
         }
 
-        public void AddCustomProject(string projectName, string projectFileExtension, string projectFileContent)
-        {
-            var projectPath = Path.Combine(DirectoryName, projectName);
-            Directory.CreateDirectory(projectPath);
-
-            var projectFilePath = Path.Combine(projectPath, projectName + projectFileExtension);
-            File.WriteAllText(projectFilePath, projectFileContent);
-
-            Contract.ThrowIfNull(_solution);
-            _solution.AddFromFile(projectFilePath);
-        }
-
         // TODO: Adjust language name based on whether we are using a web template
         private string GetProjectTemplatePath(string projectTemplate, string languageName)
         {
@@ -497,7 +380,11 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 solutionEvents.AfterCloseSolution += HandleAfterCloseSolution;
                 try
                 {
-                    ErrorHandler.ThrowOnFailure(solution.CloseSolutionElement((uint)__VSSLNCLOSEOPTIONS.SLNCLOSEOPT_DeleteProject | (uint)__VSSLNSAVEOPTIONS.SLNSAVEOPT_NoSave, null, 0));
+                    InvokeOnUIThread(cancellationToken =>
+                    {
+                        ErrorHandler.ThrowOnFailure(solution.CloseSolutionElement((uint)__VSSLNCLOSEOPTIONS.SLNCLOSEOPT_DeleteProject | (uint)__VSSLNSAVEOPTIONS.SLNSAVEOPT_NoSave, null, 0));
+                    });
+
                     semaphore.Wait();
                 }
                 finally
@@ -506,7 +393,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
                 }
             }
 
-            var waitingService = GetComponentModel().DefaultExportProvider.GetExportedValue<TestingOnly_WaitingService>();
+            var waitingService = new TestWaitingService(GetComponentModel().DefaultExportProvider.GetExportedValue<AsynchronousOperationListenerProvider>());
             waitingService.WaitForAsyncOperations(FeatureAttribute.Workspace, waitForWorkspaceFirst: true);
         }
 
@@ -684,45 +571,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             }
         }
 
-        /// <summary>
-        /// Adds a new standalone file to the Miscellaneous Files workspace.
-        /// </summary>
-        /// <param name="fileName">The name of the file to add.</param>
-        public void AddStandaloneFile(string fileName)
-        {
-            string itemTemplate;
-
-            var extension = Path.GetExtension(fileName).ToLowerInvariant();
-            switch (extension)
-            {
-                case ".cs":
-                    itemTemplate = @"General\Visual C# Class";
-                    break;
-                case ".csx":
-                    itemTemplate = @"Script\Visual C# Script";
-                    break;
-                case ".vb":
-                    itemTemplate = @"General\Visual Basic Class";
-                    break;
-                case ".txt":
-                    itemTemplate = @"General\Text File";
-                    break;
-                default:
-                    throw new NotSupportedException($"File type '{extension}' is not yet supported.");
-            }
-
-            GetDTE().ItemOperations.NewFile(itemTemplate, fileName);
-        }
-
-        public void SetFileContents(string projectName, string relativeFilePath, string contents)
-        {
-            var project = GetProject(projectName);
-            var projectPath = Path.GetDirectoryName(project.FullName);
-            var filePath = Path.Combine(projectPath, relativeFilePath);
-
-            File.WriteAllText(filePath, contents);
-        }
-
         public string GetFileContents(string projectName, string relativeFilePath)
         {
             var project = GetProject(projectName);
@@ -740,13 +588,7 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             WaitForBuildToFinish();
         }
 
-        public void ClearBuildOutputWindowPane()
-        {
-            var buildOutputWindowPane = GetBuildOutputWindowPane();
-            buildOutputWindowPane.Clear();
-        }
-
-        private EnvDTE.OutputWindowPane GetBuildOutputWindowPane()
+        private static EnvDTE.OutputWindowPane GetBuildOutputWindowPane()
         {
             var dte = (DTE2)GetDTE();
             var outputWindow = dte.ToolWindows.OutputWindow;
@@ -1034,14 +876,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             return Path.Combine(projectPath, relativeFilePath);
         }
 
-        public void ReloadProject(string projectRelativePath)
-        {
-            Contract.ThrowIfNull(_solution);
-            var solutionPath = Path.GetDirectoryName(_solution.FullName);
-            var projectPath = Path.Combine(solutionPath, projectRelativePath);
-            _solution.AddFromFile(projectPath);
-        }
-
         public bool RestoreNuGetPackages(string projectName)
         {
             using var cancellationTokenSource = new CancellationTokenSource(Helper.HangMitigatingTimeout);
@@ -1055,29 +889,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
         public void SaveAll()
             => ExecuteCommand(WellKnownCommandNames.File_SaveAll);
 
-        public void ShowErrorList()
-            => ExecuteCommand(WellKnownCommandNames.View_ErrorList);
-
-        public void ShowOutputWindow()
-            => ExecuteCommand(WellKnownCommandNames.View_Output);
-
-        public void UnloadProject(string projectName)
-        {
-            Contract.ThrowIfNull(_solution);
-            var projects = _solution.Projects;
-            EnvDTE.Project? project = null;
-            for (var i = 1; i <= projects.Count; i++)
-            {
-                project = projects.Item(i);
-                if (string.Compare(project.Name, projectName, StringComparison.Ordinal) == 0)
-                {
-                    break;
-                }
-            }
-
-            _solution.Remove(project);
-        }
-
         public void SelectItem(string itemName)
         {
             var dte = (DTE2)GetDTE();
@@ -1102,34 +913,6 @@ namespace Microsoft.VisualStudio.IntegrationTest.Utilities.InProcess
             solutionExplorer.Parent.Activate();
         }
 
-        public string[] GetChildrenOfItem(string itemName)
-        {
-            var dte = (DTE2)GetDTE();
-            var solutionExplorer = dte.ToolWindows.SolutionExplorer;
-
-            var item = FindFirstItemRecursively(solutionExplorer.UIHierarchyItems, itemName);
-            Contract.ThrowIfNull(item);
-
-            return item.UIHierarchyItems
-                .Cast<EnvDTE.UIHierarchyItem>()
-                .Select(i => i.Name)
-                .ToArray();
-        }
-
-        public string[] GetChildrenOfItemAtPath(params string[] path)
-        {
-            var dte = (DTE2)GetDTE();
-            var solutionExplorer = dte.ToolWindows.SolutionExplorer;
-
-            var item = FindItemAtPath(solutionExplorer.UIHierarchyItems, path);
-            Contract.ThrowIfNull(item);
-
-            return item.UIHierarchyItems
-                .Cast<EnvDTE.UIHierarchyItem>()
-                .Select(i => i.Name)
-                .ToArray();
-        }
-
         private static EnvDTE.UIHierarchyItem? FindItemAtPath(
             EnvDTE.UIHierarchyItems currentItems,
             string[] path)
