@@ -2,43 +2,54 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
+using System;
+using System.Globalization;
 using System.Threading;
 using System.Threading.Tasks;
 using Microsoft.CodeAnalysis.Editing;
 using Microsoft.CodeAnalysis.Formatting;
 using Microsoft.CodeAnalysis.Options;
+using Microsoft.CodeAnalysis.OrganizeImports;
 using Microsoft.CodeAnalysis.Shared.Extensions;
 using Microsoft.CodeAnalysis.Test.Utilities;
+using Microsoft.CodeAnalysis.UnitTests;
 using Roslyn.Test.Utilities;
 using Xunit;
 
 namespace Microsoft.CodeAnalysis.CSharp.Workspaces.UnitTests.OrganizeImports
 {
     [UseExportProvider]
+    [Trait(Traits.Feature, Traits.Features.Organizing)]
     public class OrganizeUsingsTests
     {
         protected static async Task CheckAsync(
             string initial, string final,
             bool placeSystemNamespaceFirst = false,
-            bool separateImportGroups = false)
+            bool separateImportGroups = false,
+            string? endOfLine = null)
         {
             using var workspace = new AdhocWorkspace();
             var project = workspace.CurrentSolution.AddProject("Project", "Project.dll", LanguageNames.CSharp);
-            var document = project.AddDocument("Document", initial.NormalizeLineEndings());
+            var document = project.AddDocument("Document", initial.ReplaceLineEndings(endOfLine ?? Environment.NewLine));
 
-            var newOptions = workspace.Options.WithChangedOption(new OptionKey(GenerationOptions.PlaceSystemNamespaceFirst, document.Project.Language), placeSystemNamespaceFirst);
-            newOptions = newOptions.WithChangedOption(new OptionKey(GenerationOptions.SeparateImportDirectiveGroups, document.Project.Language), separateImportGroups);
-            document = document.WithSolutionOptions(newOptions);
+            var options = new OrganizeImportsOptions()
+            {
+                PlaceSystemNamespaceFirst = placeSystemNamespaceFirst,
+                SeparateImportDirectiveGroups = separateImportGroups,
+                NewLine = endOfLine ?? OrganizeImportsOptions.Default.NewLine
+            };
 
-            var newRoot = await (await Formatter.OrganizeImportsAsync(document, CancellationToken.None)).GetRequiredSyntaxRootAsync(default);
-            Assert.Equal(final.NormalizeLineEndings(), newRoot.ToFullString());
+            var organizeImportsService = document.GetRequiredLanguageService<IOrganizeImportsService>();
+            var newDocument = await organizeImportsService.OrganizeImportsAsync(document, options, CancellationToken.None);
+            var newRoot = await newDocument.GetRequiredSyntaxRootAsync(default);
+            Assert.Equal(final.ReplaceLineEndings(endOfLine ?? Environment.NewLine), newRoot.ToFullString());
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task EmptyFile()
             => await CheckAsync(string.Empty, string.Empty);
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task SingleUsingStatement()
         {
             var initial = @"using A;";
@@ -46,7 +57,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Workspaces.UnitTests.OrganizeImports
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task AliasesAtBottom()
         {
             var initial =
@@ -65,7 +76,7 @@ using D = E;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task UsingStaticsBetweenUsingsAndAliases()
         {
             var initial =
@@ -90,7 +101,7 @@ using D = E;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task NestedStatements()
         {
             var initial =
@@ -112,7 +123,7 @@ namespace N
   {
     using H;
     using G;
-  } 
+  }
 }
 
 namespace N3
@@ -130,7 +141,7 @@ namespace N3
   {
     using N;
     using M;
-  } 
+  }
 }";
 
             var final =
@@ -152,7 +163,7 @@ namespace N
   {
     using G;
     using H;
-  } 
+  }
 }
 
 namespace N3
@@ -170,12 +181,12 @@ namespace N3
   {
     using M;
     using N;
-  } 
+  }
 }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task FileScopedNamespace()
         {
             var initial =
@@ -200,7 +211,7 @@ using D;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task SpecialCaseSystem()
         {
             var initial =
@@ -218,7 +229,7 @@ using M2;
             await CheckAsync(initial, final, placeSystemNamespaceFirst: true);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task SpecialCaseSystemWithUsingStatic()
         {
             var initial =
@@ -240,7 +251,7 @@ using static Microsoft.Win32.Registry;
             await CheckAsync(initial, final, placeSystemNamespaceFirst: true);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotSpecialCaseSystem()
         {
             var initial =
@@ -259,7 +270,7 @@ using System.Linq;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotSpecialCaseSystemWithUsingStatics()
         {
             var initial =
@@ -280,7 +291,7 @@ using static System.BitConverter;";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task IndentationAfterSorting()
         {
             var initial =
@@ -314,7 +325,7 @@ namespace X.Y.Z { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotTouchCommentsAtBeginningOfFile1()
         {
             var initial =
@@ -340,7 +351,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotTouchCommentsAtBeginningOfFile2()
         {
             var initial =
@@ -366,7 +377,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotTouchCommentsAtBeginningOfFile3()
         {
             var initial =
@@ -392,8 +403,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
-        [WorkItem(33251, "https://github.com/dotnet/roslyn/issues/33251")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/33251")]
         public async Task DoNotTouchCommentsAtBeginningOfFile4()
         {
             var initial =
@@ -419,8 +429,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
-        [WorkItem(33251, "https://github.com/dotnet/roslyn/issues/33251")]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/33251")]
         public async Task DoNotTouchCommentsAtBeginningOfFile5()
         {
             var initial =
@@ -448,8 +457,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [WorkItem(2480, "https://github.com/dotnet/roslyn/issues/2480")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/2480")]
         public async Task DoTouchCommentsAtBeginningOfFile1()
         {
             var initial =
@@ -473,8 +481,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [WorkItem(2480, "https://github.com/dotnet/roslyn/issues/2480")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/2480")]
         public async Task DoTouchCommentsAtBeginningOfFile2()
         {
             var initial =
@@ -498,8 +505,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [WorkItem(2480, "https://github.com/dotnet/roslyn/issues/2480")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/2480")]
         public async Task DoTouchCommentsAtBeginningOfFile3()
         {
             var initial =
@@ -523,8 +529,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [WorkItem(2480, "https://github.com/dotnet/roslyn/issues/2480")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/2480")]
         public async Task CommentsNotAtTheStartOfTheFile1()
         {
             var initial =
@@ -548,8 +553,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [WorkItem(2480, "https://github.com/dotnet/roslyn/issues/2480")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/2480")]
         public async Task CommentsNotAtTheStartOfTheFile2()
         {
             var initial =
@@ -575,7 +579,7 @@ namespace B { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotSortIfEndIfBlocks()
         {
             var initial =
@@ -596,7 +600,7 @@ namespace D { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task ExternAliases()
         {
             var initial =
@@ -676,7 +680,7 @@ namespace C
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DuplicateUsings()
         {
             var initial =
@@ -688,7 +692,7 @@ using A;";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task InlineComments()
         {
             var initial =
@@ -708,7 +712,7 @@ using A;";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task AllOnOneLine()
         {
             var initial =
@@ -722,7 +726,7 @@ using C; ";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task InsideRegionBlock()
         {
             var initial =
@@ -749,7 +753,7 @@ class Class1
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task NestedRegionBlock()
         {
             var initial =
@@ -764,7 +768,7 @@ using B;";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task MultipleRegionBlocks()
         {
             var initial =
@@ -781,7 +785,7 @@ using B;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task InterleavedNewlines()
         {
             var initial =
@@ -803,7 +807,7 @@ class D { }";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task InsideIfEndIfBlock()
         {
             var initial =
@@ -823,7 +827,7 @@ using C;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task IfEndIfBlockAbove()
         {
             var initial =
@@ -840,7 +844,7 @@ using E;";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task IfEndIfBlockMiddle()
         {
             var initial =
@@ -860,7 +864,7 @@ using G;";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task IfEndIfBlockBelow()
         {
             var initial =
@@ -877,7 +881,7 @@ using F;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task Korean()
         {
             var initial =
@@ -916,7 +920,7 @@ using 하;
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotSpecialCaseSystem1()
         {
             var initial =
@@ -945,7 +949,7 @@ using SystemZ;
             await CheckAsync(initial, final, placeSystemNamespaceFirst: false);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task DoNotSpecialCaseSystem2()
         {
             var initial =
@@ -988,7 +992,7 @@ using Z = System.Int32;
             await CheckAsync(initial, final, placeSystemNamespaceFirst: false);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task CaseSensitivity1()
         {
             var initial =
@@ -1030,13 +1034,44 @@ using bbb;
 using Bbb;
 using cc;
 using cC;
-using CC;
+using CC;";
 
-// If Kana is sensitive あ != ア, if Kana is insensitive あ == ア.
-// If Width is sensitiveア != ｱ, if Width is insensitive ア == ｱ.";
+            string sortedKana;
+            if (GlobalizationUtilities.ICUMode())
+            {
+                sortedKana =
+@"using あ;
+using ｱ;
+using ああ;
+using あｱ;
+using ｱあ;
+using ｱｱ;
+using あア;
+using ｱア;
+using ア;
+using アあ;
+using アｱ;
+using アア;";
+            }
+            else
+            {
+                sortedKana =
+@"using ア;
+using ｱ;
+using あ;
+using アア;
+using アｱ;
+using ｱア;
+using ｱｱ;
+using アあ;
+using ｱあ;
+using あア;
+using あｱ;
+using ああ;";
+            }
 
             var final =
-@"using a;
+@$"using a;
 using A;
 using aa;
 using aA;
@@ -1063,25 +1098,12 @@ using cC;
 using cC;
 using Cc;
 using CC;
-using ア;
-using ｱ;
-using あ;
-using アア;
-using アｱ;
-using ｱア;
-using ｱｱ;
-using アあ;
-using ｱあ;
-using あア;
-using あｱ;
-using ああ;
-
-// If Kana is sensitive あ != ア, if Kana is insensitive あ == ア.
-// If Width is sensitiveア != ｱ, if Width is insensitive ア == ｱ.";
+{sortedKana}
+";
             await CheckAsync(initial, final);
         }
 
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact]
         public async Task CaseSensitivity2()
         {
             var initial =
@@ -1098,7 +1120,26 @@ using ｱあ;
 using ｱア;
 using ｱｱ;";
 
-            var final =
+            if (GlobalizationUtilities.ICUMode())
+            {
+                await CheckAsync(initial,
+@"using あ;
+using ｱ;
+using ああ;
+using あｱ;
+using ｱあ;
+using ｱｱ;
+using あア;
+using ｱア;
+using ア;
+using アあ;
+using アｱ;
+using アア;
+");
+            }
+            else
+            {
+                await CheckAsync(initial,
 @"using ア;
 using ｱ;
 using あ;
@@ -1111,13 +1152,11 @@ using ｱあ;
 using あア;
 using あｱ;
 using ああ;
-";
-
-            await CheckAsync(initial, final);
+");
+            }
         }
 
-        [WorkItem(20988, "https://github.com/dotnet/roslyn/issues/20988")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/20988")]
         public async Task TestGrouping()
         {
             var initial =
@@ -1153,8 +1192,7 @@ using IntList = System.Collections.Generic.List<int>;
             await CheckAsync(initial, final, placeSystemNamespaceFirst: true, separateImportGroups: true);
         }
 
-        [WorkItem(20988, "https://github.com/dotnet/roslyn/issues/20988")]
-        [Fact, Trait(Traits.Feature, Traits.Features.Organizing)]
+        [Fact, WorkItem("https://github.com/dotnet/roslyn/issues/20988")]
         public async Task TestGrouping2()
         {
             // Make sure we don't insert extra newlines if they're already there.
@@ -1194,5 +1232,43 @@ using IntList = System.Collections.Generic.List<int>;
 
             await CheckAsync(initial, final, placeSystemNamespaceFirst: true, separateImportGroups: true);
         }
+
+        [Theory, WorkItem(20988, "https://github.com/dotnet/roslyn/issues/19306")]
+        [InlineData("\n")]
+        [InlineData("\r\n")]
+        public async Task TestGrouping3(string endOfLine)
+        {
+            var initial =
+@"// Banner
+
+using Microsoft.CodeAnalysis.CSharp.Extensions;
+using Microsoft.CodeAnalysis.CSharp.Syntax;
+
+using System.Collections.Generic;
+using System.Linq;
+using Microsoft.CodeAnalysis.Shared.Extensions;
+using Roslyn.Utilities;
+using IntList = System.Collections.Generic.List<int>;
+using static System.Console;";
+
+            var final =
+@"// Banner
+
+using System.Collections.Generic;
+using System.Linq;
+
+using Microsoft.CodeAnalysis.CSharp.Extensions;
+using Microsoft.CodeAnalysis.CSharp.Syntax;
+using Microsoft.CodeAnalysis.Shared.Extensions;
+
+using Roslyn.Utilities;
+
+using static System.Console;
+
+using IntList = System.Collections.Generic.List<int>;
+";
+
+            await CheckAsync(initial, final, placeSystemNamespaceFirst: true, separateImportGroups: true, endOfLine: endOfLine);
+        }
     }
 }
