@@ -22,8 +22,8 @@ namespace Microsoft.CodeAnalysis.Differencing
         protected override bool ItemsEqual(string oldSequence, int oldIndex, string newSequence, int newIndex)
             => oldSequence[oldIndex] == newSequence[newIndex];
 
-        public static double ComputeDistance(string oldValue, string newValue)
-            => s_instance.ComputeDistance(oldValue, oldValue.Length, newValue, newValue.Length);
+        public static double ComputePrefixDistance(string oldValue, int oldLength, string newValue, int newLength)
+            => s_instance.ComputeDistance(oldValue, oldLength, newValue, newLength);
 
         public static IEnumerable<SequenceEdit> GetEdits(string oldValue, string newValue)
             => s_instance.GetEdits(oldValue, oldValue.Length, newValue, newValue.Length);
