@@ -25,8 +25,8 @@ namespace Microsoft.CodeAnalysis.Differencing
         private readonly TNode _root1;
         private readonly TNode _root2;
 
-        private readonly Dictionary<TNode, TNode> _oneToTwo;
-        private readonly Dictionary<TNode, TNode> _twoToOne;
+        private readonly Dictionary<TNode, TNode> _oneToTwo = new();
+        private readonly Dictionary<TNode, TNode> _twoToOne = new();
 
         internal Match(TNode root1, TNode root2, TreeComparer<TNode> comparer, IEnumerable<KeyValuePair<TNode, TNode>> knownMatches)
         {
@@ -38,9 +38,6 @@ namespace Microsoft.CodeAnalysis.Differencing
             CategorizeNodesByLabels(comparer, root1, labelCount, out var nodes1, out _);
             CategorizeNodesByLabels(comparer, root2, labelCount, out var nodes2, out _);
 
-            _oneToTwo = new Dictionary<TNode, TNode>();
-            _twoToOne = new Dictionary<TNode, TNode>();
-
             // Root nodes always match. Add them before adding known matches to make sure we always have root mapping.
             TryAdd(root1, root2);
 
