@@ -6,12 +6,9 @@ using System;
 
 namespace Microsoft.CodeAnalysis.ExternalAccess.UnitTesting.Api
 {
-    internal readonly struct UnitTestingChecksumWrapper
+    internal readonly struct UnitTestingChecksumWrapper(Checksum underlyingObject)
     {
-        private Checksum UnderlyingObject { get; }
-
-        public UnitTestingChecksumWrapper(Checksum underlyingObject)
-            => UnderlyingObject = underlyingObject ?? throw new ArgumentNullException(nameof(underlyingObject));
+        private Checksum UnderlyingObject { get; } = underlyingObject ?? throw new ArgumentNullException(nameof(underlyingObject));
 
         public bool IsEqualTo(UnitTestingChecksumWrapper other)
             => other.UnderlyingObject == UnderlyingObject;
