@@ -13,7 +13,7 @@ namespace Microsoft.CodeAnalysis.FindSymbols
     public static partial class SymbolFinder
     {
         internal static async Task<ImmutableArray<ReferencedSymbol>> FindRenamableReferencesAsync(
-            ISymbol symbol,
+            ImmutableArray<ISymbol> symbols,
             Solution solution,
             CancellationToken cancellationToken)
         {
@@ -28,7 +28,7 @@ namespace Microsoft.CodeAnalysis.FindSymbols
                     streamingProgress,
                     FindReferencesSearchOptions.Default);
 
-                await engine.FindReferencesAsync(symbol, cancellationToken).ConfigureAwait(false);
+                await engine.FindReferencesAsync(symbols, cancellationToken).ConfigureAwait(false);
                 return streamingProgress.GetReferencedSymbols();
             }
         }
