@@ -11,9 +11,8 @@ namespace Microsoft.CodeAnalysis.Shared.Utilities
     internal static class IStreamingProgressTrackerExtensions
     {
         /// <summary>
-        /// Returns an <see cref="IAsyncDisposable"/> that will call <see
-        /// cref="IStreamingProgressTracker.ItemCompletedAsync"/> on <paramref
-        /// name="progressTracker"/> when it is disposed.
+        /// Returns an <see cref="IAsyncDisposable"/> that will call <see cref="ItemCompletedAsync"/> on
+        /// <paramref name="progressTracker"/> when it is disposed.
         /// </summary>
         public static async Task<IAsyncDisposable> AddSingleItemAsync(this IStreamingProgressTracker progressTracker, CancellationToken cancellationToken)
         {
@@ -21,19 +20,13 @@ namespace Microsoft.CodeAnalysis.Shared.Utilities
             return new StreamingProgressDisposer(progressTracker, cancellationToken);
         }
 
-        private class StreamingProgressDisposer : IAsyncDisposable
+        public static ValueTask ItemCompletedAsync(this IStreamingProgressTracker tracker, CancellationToken cancellationToken)
+            => tracker.ItemsCompletedAsync(1, cancellationToken);
+
+        private class StreamingProgressDisposer(IStreamingProgressTracker progressTracker, CancellationToken cancellationToken) : IAsyncDisposable
         {
-            private readonly IStreamingProgressTracker _progressTracker;
-            private readonly CancellationToken _cancellationToken;
-
-            public StreamingProgressDisposer(IStreamingProgressTracker progressTracker, CancellationToken cancellationToken)
-            {
-                _progressTracker = progressTracker;
-                _cancellationToken = cancellationToken;
-            }
-
             public async ValueTask DisposeAsync()
-                => await _progressTracker.ItemCompletedAsync(_cancellationToken).ConfigureAwait(false);
+                => await progressTracker.ItemCompletedAsync(cancellationToken).ConfigureAwait(false);
         }
     }
 }
