@@ -5,19 +5,19 @@
 #nullable disable
 
 using System.Threading;
-using Microsoft.CodeAnalysis.Options;
 using Microsoft.CodeAnalysis.Text;
 
 namespace Microsoft.CodeAnalysis.Simplification.Simplifiers
 {
-    internal abstract class AbstractSimplifier<TSyntax, TSimplifiedSyntax>
+    internal abstract class AbstractSimplifier<TSyntax, TSimplifiedSyntax, TSimplifierOptions>
         where TSyntax : SyntaxNode
         where TSimplifiedSyntax : SyntaxNode
+        where TSimplifierOptions : SimplifierOptions
     {
         public abstract bool TrySimplify(
             TSyntax syntax,
             SemanticModel semanticModel,
-            OptionSet optionSet,
+            TSimplifierOptions options,
             out TSimplifiedSyntax replacementNode,
             out TextSpan issueSpan,
             CancellationToken cancellationToken);
