@@ -3,6 +3,7 @@
 // See the LICENSE file in the project root for more information.
 
 using System;
+using System.Collections.Generic;
 using System.Diagnostics;
 using System.Diagnostics.CodeAnalysis;
 using System.Threading;
@@ -25,7 +26,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
         public void WhenAll_Null()
         {
 #pragma warning disable CA2012 // Use ValueTasks correctly (the instance is never created)
-            Assert.Throws<ArgumentNullException>(() => SpecializedTasks.WhenAll<int>(null!));
+            Assert.Throws<ArgumentNullException>(() => SpecializedTasks.WhenAll<int>((IEnumerable<ValueTask<int>>)null!));
 #pragma warning restore CA2012 // Use ValueTasks correctly
         }
 
@@ -211,7 +212,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 await Task.Yield();
                 gate.Wait(CancellationToken.None);
                 cts.Token.ThrowIfCancellationRequested();
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             };
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
             {
@@ -271,7 +272,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 await Task.Yield();
                 gate.Wait(CancellationToken.None);
                 unexpectedCts.Token.ThrowIfCancellationRequested();
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             };
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
             {
@@ -338,7 +339,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 await Task.Yield();
                 gate.Wait(CancellationToken.None);
                 unexpectedCts.Token.ThrowIfCancellationRequested();
-                throw ExceptionUtilities.Unreachable;
+                throw ExceptionUtilities.Unreachable();
             };
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
             {
@@ -362,7 +363,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
         {
             var executedTransform = false;
 
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var cancellationToken = new CancellationToken(canceled: false);
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = (_, _) => throw fault;
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
@@ -384,7 +385,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
         {
             var executedTransform = false;
 
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var cancellationToken = new CancellationToken(canceled: false);
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = (_, _) => new(Task.FromException<IntermediateType>(fault));
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
@@ -406,7 +407,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
         {
             var executedTransform = false;
 
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var cancellationToken = new CancellationToken(canceled: false);
             var gate = new ManualResetEventSlim();
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = async (_, _) =>
@@ -439,7 +440,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
 
             var executedTransform = false;
 
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var cancellationToken = cts.Token;
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = (_, _) => throw fault;
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
@@ -464,7 +465,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
 
             var executedTransform = false;
 
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var cancellationToken = cts.Token;
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = (_, _) => new(Task.FromException<IntermediateType>(fault));
             Func<IntermediateType, StateType, ResultType> transform = (_, _) =>
@@ -489,7 +490,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
 
             var executedTransform = false;
 
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var cancellationToken = cts.Token;
             var gate = new ManualResetEventSlim();
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = async (_, _) =>
@@ -518,7 +519,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
         [Fact]
         public void Transform_SyncCompletedFunction_FaultedTransform()
         {
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = (_, _) => new(new IntermediateType());
             Func<IntermediateType, StateType, ResultType> transform = (_, _) => throw fault;
             var arg = new StateType();
@@ -533,7 +534,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
         [Fact]
         public async Task Transform_AsyncCompletedFunction_FaultedTransform()
         {
-            var fault = ExceptionUtilities.Unreachable;
+            var fault = ExceptionUtilities.Unreachable();
             var gate = new ManualResetEventSlim();
             Func<StateType, CancellationToken, ValueTask<IntermediateType>> func = async (_, _) =>
             {
