@@ -104,6 +104,24 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 (@"CSharpProject\Properties\AssemblyInfo.cs", Resources.SourceFiles.CSharp.AssemblyInfo));
         }
 
+        protected static FileSet GetSimpleVisualBasicSolutionFiles()
+        {
+            return new FileSet(
+                (@"NuGet.Config", Resources.NuGet_Config),
+                (@"Directory.Build.props", Resources.Directory_Build_props),
+                (@"Directory.Build.targets", Resources.Directory_Build_targets),
+                (@"TestSolution.sln", Resources.SolutionFiles.VisualBasic),
+                (@"VisualBasicProject\VisualBasicProject.vbproj", Resources.ProjectFiles.VisualBasic.VisualBasicProject),
+                (@"VisualBasicProject\VisualBasicClass.vb", Resources.SourceFiles.VisualBasic.VisualBasicClass),
+                (@"VisualBasicProject\My Project\Application.Designer.vb", Resources.SourceFiles.VisualBasic.Application_Designer),
+                (@"VisualBasicProject\My Project\Application.myapp", Resources.SourceFiles.VisualBasic.Application),
+                (@"VisualBasicProject\My Project\AssemblyInfo.vb", Resources.SourceFiles.VisualBasic.AssemblyInfo),
+                (@"VisualBasicProject\My Project\Resources.Designer.vb", Resources.SourceFiles.VisualBasic.Resources_Designer),
+                (@"VisualBasicProject\My Project\Resources.resx", Resources.SourceFiles.VisualBasic.Resources),
+                (@"VisualBasicProject\My Project\Settings.Designer.vb", Resources.SourceFiles.VisualBasic.Settings_Designer),
+                (@"VisualBasicProject\My Project\Settings.settings", Resources.SourceFiles.VisualBasic.Settings));
+        }
+
         protected static FileSet GetSimpleCSharpSolutionWithAdditionaFile()
         {
             return new FileSet(
@@ -117,40 +135,40 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 (@"CSharpProject\ValidAdditionalFile.txt", Resources.SourceFiles.Text.ValidAdditionalFile));
         }
 
-        protected static FileSet GetNetCoreApp2Files()
+        protected static FileSet GetNetCoreAppFiles()
         {
             return new FileSet(
                 (@"NuGet.Config", Resources.NuGet_Config),
                 (@"Directory.Build.props", Resources.Directory_Build_props),
                 (@"Directory.Build.targets", Resources.Directory_Build_targets),
-                (@"Project.csproj", Resources.ProjectFiles.CSharp.NetCoreApp2_Project),
-                (@"Program.cs", Resources.SourceFiles.CSharp.NetCoreApp2_Program));
+                (@"Project.csproj", Resources.ProjectFiles.CSharp.NetCoreApp_Project),
+                (@"Program.cs", Resources.SourceFiles.CSharp.NetCoreApp_Program));
         }
 
-        protected static FileSet GetNetCoreApp2AndLibraryFiles()
+        protected static FileSet GetNetCoreAppAndLibraryFiles()
         {
             return new FileSet(
                 (@"NuGet.Config", Resources.NuGet_Config),
                 (@"Directory.Build.props", Resources.Directory_Build_props),
                 (@"Directory.Build.targets", Resources.Directory_Build_targets),
-                (@"Project\Project.csproj", Resources.ProjectFiles.CSharp.NetCoreApp2AndLibrary_Project),
-                (@"Project\Program.cs", Resources.SourceFiles.CSharp.NetCoreApp2AndLibrary_Program),
-                (@"Library\Library.csproj", Resources.ProjectFiles.CSharp.NetCoreApp2AndLibrary_Library),
-                (@"Library\Class1.cs", Resources.SourceFiles.CSharp.NetCoreApp2AndLibrary_Class1));
+                (@"Project\Project.csproj", Resources.ProjectFiles.CSharp.NetCoreAppAndLibrary_Project),
+                (@"Project\Program.cs", Resources.SourceFiles.CSharp.NetCoreAppAndLibrary_Program),
+                (@"Library\Library.csproj", Resources.ProjectFiles.CSharp.NetCoreAppAndLibrary_Library),
+                (@"Library\Class1.cs", Resources.SourceFiles.CSharp.NetCoreAppAndLibrary_Class1));
         }
 
-        protected static FileSet GetNetCoreApp2AndTwoLibrariesFiles()
+        protected static FileSet GetNetCoreAppAndTwoLibrariesFiles()
         {
             return new FileSet(
                 (@"NuGet.Config", Resources.NuGet_Config),
                 (@"Directory.Build.props", Resources.Directory_Build_props),
                 (@"Directory.Build.targets", Resources.Directory_Build_targets),
-                (@"Project\Project.csproj", Resources.ProjectFiles.CSharp.NetCoreApp2AndTwoLibraries_Project),
-                (@"Project\Program.cs", Resources.SourceFiles.CSharp.NetCoreApp2AndTwoLibraries_Program),
-                (@"Library1\Library1.csproj", Resources.ProjectFiles.CSharp.NetCoreApp2AndTwoLibraries_Library1),
-                (@"Library1\Class1.cs", Resources.SourceFiles.CSharp.NetCoreApp2AndTwoLibraries_Class1),
-                (@"Library2\Library2.csproj", Resources.ProjectFiles.CSharp.NetCoreApp2AndTwoLibraries_Library2),
-                (@"Library2\Class2.cs", Resources.SourceFiles.CSharp.NetCoreApp2AndTwoLibraries_Class2));
+                (@"Project\Project.csproj", Resources.ProjectFiles.CSharp.NetCoreAppAndTwoLibraries_Project),
+                (@"Project\Program.cs", Resources.SourceFiles.CSharp.NetCoreAppAndTwoLibraries_Program),
+                (@"Library1\Library1.csproj", Resources.ProjectFiles.CSharp.NetCoreAppAndTwoLibraries_Library1),
+                (@"Library1\Class1.cs", Resources.SourceFiles.CSharp.NetCoreAppAndTwoLibraries_Class1),
+                (@"Library2\Library2.csproj", Resources.ProjectFiles.CSharp.NetCoreAppAndTwoLibraries_Library2),
+                (@"Library2\Class2.cs", Resources.SourceFiles.CSharp.NetCoreAppAndTwoLibraries_Class2));
         }
 
         protected static FileSet GetNetCoreMultiTFMFiles()
@@ -160,7 +178,7 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 (@"Directory.Build.props", Resources.Directory_Build_props),
                 (@"Directory.Build.targets", Resources.Directory_Build_targets),
                 (@"Project.csproj", Resources.ProjectFiles.CSharp.NetCoreMultiTFM_Project),
-                (@"Program.cs", Resources.SourceFiles.CSharp.NetCoreApp2_Program));
+                (@"Program.cs", Resources.SourceFiles.CSharp.NetCoreApp_Program));
         }
 
         protected static FileSet GetNetCoreMultiTFMFiles_ExtensionWithConditionOnTFM()
@@ -292,5 +310,17 @@ namespace Microsoft.CodeAnalysis.UnitTests
                 (@"CircularCSharpProject1.csproj", Resources.ProjectFiles.CSharp.CircularProjectReferences_CircularCSharpProject1),
                 (@"CircularCSharpProject2.csproj", Resources.ProjectFiles.CSharp.CircularProjectReferences_CircularCSharpProject2));
         }
+
+        protected static FileSet GetVBNetCoreAppWithGlobalImportAndLibraryFiles()
+        {
+            return new FileSet(
+                (@"NuGet.Config", Resources.NuGet_Config),
+                (@"Directory.Build.props", Resources.Directory_Build_props),
+                (@"Directory.Build.targets", Resources.Directory_Build_targets),
+                (@"VBProject\VBProject.vbproj", Resources.ProjectFiles.VisualBasic.VBNetCoreAppWithGlobalImportAndLibrary_VBProject),
+                (@"VBProject\Program.vb", Resources.SourceFiles.VisualBasic.VBNetCoreAppWithGlobalImportAndLibrary_Program),
+                (@"Library\Library.csproj", Resources.ProjectFiles.CSharp.VBNetCoreAppWithGlobalImportAndLibrary_Library),
+                (@"Library\MyHelperClass.cs", Resources.SourceFiles.CSharp.VBNetCoreAppWithGlobalImportAndLibrary_MyHelperClass));
+        }
     }
 }
