@@ -2,8 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-using System.Collections.Immutable;
-using System.IO;
 using System.IO.Pipelines;
 using System.Threading;
 using System.Threading.Tasks;
@@ -18,6 +16,9 @@ namespace Microsoft.CodeAnalysis.Remote
         /// <summary>
         /// Streams serialized assets into the given stream.
         /// </summary>
-        ValueTask GetAssetsAsync(PipeWriter pipeWriter, int scopeId, Checksum[] checksums, CancellationToken cancellationToken);
+        /// <param name="pipeWriter">The writer to write the assets into.  Implementations of this method must call<see
+        /// cref="PipeWriter.Complete"/> on it (in the event of failure or success).  Failing to do so will lead to
+        /// hangs on the code that reads from the corresponding <see cref="PipeReader"/> side of this.</param>
+        ValueTask GetAssetsAsync(PipeWriter pipeWriter, Checksum solutionChecksum, Checksum[] checksums, CancellationToken cancellationToken);
     }
 }
