@@ -4,40 +4,39 @@
 
 using System;
 using System.Collections.Generic;
-using Microsoft.CodeAnalysis;
 using Microsoft.CodeAnalysis.CSharp.Extensions;
 using Microsoft.CodeAnalysis.CSharp.Syntax;
-using Microsoft.CodeAnalysis.Diagnostics;
+using Microsoft.CodeAnalysis.Formatting;
 using Microsoft.CodeAnalysis.Formatting.Rules;
-using Microsoft.CodeAnalysis.Options;
 using Roslyn.Utilities;
 
 namespace Microsoft.CodeAnalysis.CSharp.Formatting
 {
     internal sealed class SpacingFormattingRule : BaseFormattingRule
     {
-        private readonly CachedOptions _options;
+        private readonly CSharpSyntaxFormattingOptions _options;
 
         public SpacingFormattingRule()
-            : this(new CachedOptions(null))
+            : this(CSharpSyntaxFormattingOptions.Default)
         {
         }
 
-        private SpacingFormattingRule(CachedOptions options)
+        private SpacingFormattingRule(CSharpSyntaxFormattingOptions options)
         {
             _options = options;
         }
 
-        public override AbstractFormattingRule WithOptions(AnalyzerConfigOptions options)
+        public override AbstractFormattingRule WithOptions(SyntaxFormattingOptions options)
         {
-            var cachedOptions = new CachedOptions(options);
+            var newOptions = options as CSharpSyntaxFormattingOptions ?? CSharpSyntaxFormattingOptions.Default;
 
-            if (cachedOptions == _options)
+            if (_options.Spacing == newOptions.Spacing &&
+                _options.SpacingAroundBinaryOperator == newOptions.SpacingAroundBinaryOperator)
             {
                 return this;
             }
 
-            return new SpacingFormattingRule(cachedOptions);
+            return new SpacingFormattingRule(newOptions);
         }
 
         public override AdjustSpacesOperation? GetAdjustSpacesOperation(in SyntaxToken previousToken, in SyntaxToken currentToken, in NextGetAdjustSpacesOperation nextOperation)
@@ -58,7 +57,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
                     return CreateAdjustSpacesOperation(1, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
                 }
 
-                return AdjustSpacesOperationZeroOrOne(_options.SpacingAfterMethodDeclarationName);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterMethodDeclarationName));
             }
 
             // For Generic Method Declaration
@@ -72,7 +71,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
 
                 if (previousParentKind == SyntaxKind.TypeParameterList)
                 {
-                    return AdjustSpacesOperationZeroOrOne(_options.SpacingAfterMethodDeclarationName);
+                    return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterMethodDeclarationName));
                 }
             }
 
@@ -82,31 +81,31 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
             // Case: public static implicit operator int[](Program p) { return null; }
             // Case: public static implicit operator (int, int)(Program p) { return null; }
             // Case: public static implicit operator Action<int>(Program p) { return null; }
-            if ((previousToken.IsKeyword() || previousToken.IsKind(SyntaxKind.QuestionToken, SyntaxKind.AsteriskToken, SyntaxKind.CloseBracketToken, SyntaxKind.CloseParenToken, SyntaxKind.GreaterThanToken))
+            if ((previousToken.IsKeyword() || previousToken.Kind() is SyntaxKind.QuestionToken or SyntaxKind.AsteriskToken or SyntaxKind.CloseBracketToken or SyntaxKind.CloseParenToken or SyntaxKind.GreaterThanToken)
                 && currentToken.IsOpenParenInParameterListOfAConversionOperatorDeclaration())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpacingAfterMethodDeclarationName);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterMethodDeclarationName));
             }
 
             // Case: public static Program operator !(Program p) { return null; }
             if (previousToken.Parent.IsKind(SyntaxKind.OperatorDeclaration) && currentToken.IsOpenParenInParameterListOfAOperationDeclaration())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpacingAfterMethodDeclarationName);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterMethodDeclarationName));
             }
 
             if (previousToken.IsOpenParenInParameterList() && currentToken.IsCloseParenInParameterList())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBetweenEmptyMethodDeclarationParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BetweenEmptyMethodDeclarationParentheses));
             }
 
             if (previousToken.IsOpenParenInParameterList())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodDeclarationParenthesis);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodDeclarationParenthesis));
             }
 
             if (currentToken.IsCloseParenInParameterList())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodDeclarationParenthesis);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodDeclarationParenthesis));
             }
 
             // For Method Call
@@ -115,38 +114,38 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
             //   x is TypeName ( args )
             if (currentToken.IsOpenParenInArgumentListOrPositionalPattern())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterMethodCallName);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterMethodCallName));
             }
 
             if (previousToken.IsOpenParenInArgumentListOrPositionalPattern() && currentToken.IsCloseParenInArgumentListOrPositionalPattern())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBetweenEmptyMethodCallParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BetweenEmptyMethodCallParentheses));
             }
 
             if (previousToken.IsOpenParenInArgumentListOrPositionalPattern())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodCallParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodCallParentheses));
             }
 
             if (currentToken.IsCloseParenInArgumentListOrPositionalPattern())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodCallParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodCallParentheses));
             }
 
             // For spacing around: typeof, default, and sizeof; treat like a Method Call
             if (currentKind == SyntaxKind.OpenParenToken && IsFunctionLikeKeywordExpressionKind(currentParentKind))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterMethodCallName);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterMethodCallName));
             }
 
             if (previousKind == SyntaxKind.OpenParenToken && IsFunctionLikeKeywordExpressionKind(previousParentKind))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodCallParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodCallParentheses));
             }
 
             if (currentKind == SyntaxKind.CloseParenToken && IsFunctionLikeKeywordExpressionKind(currentParentKind))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodCallParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodCallParentheses));
             }
 
             // For Spacing b/n control flow keyword and paren. Parent check not needed.
@@ -156,21 +155,21 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
                 previousKind == SyntaxKind.UsingKeyword || previousKind == SyntaxKind.WhenKeyword || previousKind == SyntaxKind.LockKeyword ||
                 previousKind == SyntaxKind.FixedKeyword))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterControlFlowStatementKeyword);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterControlFlowStatementKeyword));
             }
 
             // For spacing between parenthesis and expression
-            if ((previousToken.Parent.IsKind(SyntaxKind.ParenthesizedExpression, SyntaxKind.ParenthesizedPattern) && previousKind == SyntaxKind.OpenParenToken) ||
-                (currentToken.Parent.IsKind(SyntaxKind.ParenthesizedExpression, SyntaxKind.ParenthesizedPattern) && currentKind == SyntaxKind.CloseParenToken))
+            if ((previousToken.Parent.Kind() is SyntaxKind.ParenthesizedExpression or SyntaxKind.ParenthesizedPattern && previousKind == SyntaxKind.OpenParenToken) ||
+                (currentToken.Parent.Kind() is SyntaxKind.ParenthesizedExpression or SyntaxKind.ParenthesizedPattern && currentKind == SyntaxKind.CloseParenToken))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinExpressionParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinExpressionParentheses));
             }
 
             // For spacing between the parenthesis and the cast expression
             if ((previousParentKind == SyntaxKind.CastExpression && previousKind == SyntaxKind.OpenParenToken) ||
                 (currentParentKind == SyntaxKind.CastExpression && currentKind == SyntaxKind.CloseParenToken))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinCastParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinCastParentheses));
             }
 
             // Semicolons in an empty for statement.  i.e.   for(;;)
@@ -179,35 +178,51 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
             {
                 if (currentKind == SyntaxKind.SemicolonToken
                     && (previousKind != SyntaxKind.SemicolonToken
-                        || _options.SpaceBeforeSemicolonsInForStatement))
+                        || _options.Spacing.HasFlag(SpacePlacement.BeforeSemicolonsInForStatement)))
                 {
-                    return AdjustSpacesOperationZeroOrOne(_options.SpaceBeforeSemicolonsInForStatement);
+                    return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BeforeSemicolonsInForStatement));
                 }
 
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterSemicolonsInForStatement);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterSemicolonsInForStatement));
             }
 
             // For spacing between the parenthesis and the expression inside the control flow expression
             if (previousKind == SyntaxKind.OpenParenToken && IsControlFlowLikeKeywordStatementKind(previousParentKind))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinOtherParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinOtherParentheses));
             }
 
             if (currentKind == SyntaxKind.CloseParenToken && IsControlFlowLikeKeywordStatementKind(currentParentKind))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinOtherParentheses);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinOtherParentheses));
             }
 
             // For spacing after the cast
             if (previousParentKind == SyntaxKind.CastExpression && previousKind == SyntaxKind.CloseParenToken)
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterCast);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterCast));
+            }
+
+            // List patterns
+            if (currentKind == SyntaxKind.OpenBracketToken && currentToken.Parent.Kind() is SyntaxKind.ListPattern or SyntaxKind.CollectionExpression)
+            {
+                // For the space after the middle comma in ([1, 2], [1, 2])
+                if (previousKind == SyntaxKind.CommaToken)
+                {
+                    return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterComma));
+                }
+
+                // For "is [", "and [", but not "([" or "[["
+                if (previousKind is not (SyntaxKind.OpenParenToken or SyntaxKind.OpenBracketToken))
+                {
+                    return CreateAdjustSpacesOperation(1, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
+                }
             }
 
             // For spacing Before Square Braces
             if (currentKind == SyntaxKind.OpenBracketToken && HasFormattableBracketParent(currentToken) && !previousToken.IsOpenBraceOrCommaOfObjectInitializer())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBeforeOpenSquareBracket);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BeforeOpenSquareBracket));
             }
 
             // For spacing empty square braces, also treat [,] as empty
@@ -215,85 +230,96 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
                 || currentKind == SyntaxKind.OmittedArraySizeExpressionToken)
                 && HasFormattableBracketParent(previousToken))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBetweenEmptySquareBrackets);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BetweenEmptySquareBrackets));
             }
 
             // For spacing square brackets within
             if (previousKind == SyntaxKind.OpenBracketToken && HasFormattableBracketParent(previousToken))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinSquareBrackets);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinSquareBrackets));
             }
 
             if (currentKind == SyntaxKind.CloseBracketToken && previousKind != SyntaxKind.OmittedArraySizeExpressionToken && HasFormattableBracketParent(currentToken))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinSquareBrackets);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinSquareBrackets));
             }
 
             // attribute case ] *
-            // Place a space between the attribute and the next member if they're on the same line.
             if (previousKind == SyntaxKind.CloseBracketToken && previousToken.Parent.IsKind(SyntaxKind.AttributeList))
             {
-                var attributeOwner = previousToken.Parent?.Parent;
-                if (attributeOwner is MemberDeclarationSyntax)
+                // [Attribute1]$$[Attribute2]
+                if (currentToken.IsKind(SyntaxKind.OpenBracketToken) &&
+                    currentToken.Parent.IsKind(SyntaxKind.AttributeList))
                 {
-                    return CreateAdjustSpacesOperation(1, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
+                    return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
                 }
+
+                // [Attribute1]$${EOF}
+                if (currentToken.IsKind(SyntaxKind.EndOfFileToken))
+                {
+                    return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
+                }
+
+                // [Attribute]$$ int Prop { ... }
+                return CreateAdjustSpacesOperation(1, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
             }
 
             // For spacing delimiters - after colon
             if (previousToken.IsColonInTypeBaseList())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterColonInBaseTypeDeclaration);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterColonInBaseTypeDeclaration));
             }
 
             // For spacing delimiters - before colon
             if (currentToken.IsColonInTypeBaseList())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBeforeColonInBaseTypeDeclaration);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BeforeColonInBaseTypeDeclaration));
             }
 
             // For spacing delimiters - after comma
             if ((previousToken.IsCommaInArgumentOrParameterList() && currentKind != SyntaxKind.OmittedTypeArgumentToken)
                 || previousToken.IsCommaInInitializerExpression()
+                || previousToken.IsCommaInCollectionExpression()
                 || (previousKind == SyntaxKind.CommaToken
                     && currentKind != SyntaxKind.OmittedArraySizeExpressionToken
                     && HasFormattableBracketParent(previousToken)))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterComma);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterComma));
             }
 
             // For spacing delimiters - before comma
             if ((currentToken.IsCommaInArgumentOrParameterList() && previousKind != SyntaxKind.OmittedTypeArgumentToken)
                 || currentToken.IsCommaInInitializerExpression()
+                || previousToken.IsCommaInCollectionExpression()
                 || (currentKind == SyntaxKind.CommaToken
                     && previousKind != SyntaxKind.OmittedArraySizeExpressionToken
                     && HasFormattableBracketParent(currentToken)))
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBeforeComma);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BeforeComma));
             }
 
             // For Spacing delimiters - after Dot
             if (previousToken.IsDotInMemberAccessOrQualifiedName())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterDot);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterDot));
             }
 
             // For spacing delimiters - before Dot
             if (currentToken.IsDotInMemberAccessOrQualifiedName())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBeforeDot);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BeforeDot));
             }
 
             // For spacing delimiters - after semicolon
             if (previousToken.IsSemicolonInForStatement() && currentKind != SyntaxKind.CloseParenToken)
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceAfterSemicolonsInForStatement);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.AfterSemicolonsInForStatement));
             }
 
             // For spacing delimiters - before semicolon
             if (currentToken.IsSemicolonInForStatement())
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceBeforeSemicolonsInForStatement);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.BeforeSemicolonsInForStatement));
             }
 
             // For spacing around the binary operators
@@ -301,8 +327,8 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
                 previousToken.Parent is BinaryExpressionSyntax ||
                 currentToken.Parent is AssignmentExpressionSyntax ||
                 previousToken.Parent is AssignmentExpressionSyntax ||
-                currentToken.Parent.IsKind(SyntaxKind.AndPattern, SyntaxKind.OrPattern, SyntaxKind.RelationalPattern) ||
-                previousToken.Parent.IsKind(SyntaxKind.AndPattern, SyntaxKind.OrPattern, SyntaxKind.RelationalPattern))
+                currentToken.Parent.Kind() is SyntaxKind.AndPattern or SyntaxKind.OrPattern or SyntaxKind.RelationalPattern ||
+                previousToken.Parent.Kind() is SyntaxKind.AndPattern or SyntaxKind.OrPattern or SyntaxKind.RelationalPattern)
             {
                 switch (_options.SpacingAroundBinaryOperator)
                 {
@@ -417,14 +443,14 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
             // delegate*<void
             if (previousKind == SyntaxKind.LessThanToken && previousParentKind == SyntaxKind.FunctionPointerParameterList)
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodDeclarationParenthesis);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodDeclarationParenthesis));
             }
 
             // Respect spacing setting for before the > in function pointer parameter lists
             // delegate*<void>
             if (currentKind == SyntaxKind.GreaterThanToken && currentParentKind == SyntaxKind.FunctionPointerParameterList)
             {
-                return AdjustSpacesOperationZeroOrOne(_options.SpaceWithinMethodDeclarationParenthesis);
+                return AdjustSpacesOperationZeroOrOne(_options.Spacing.HasFlag(SpacePlacement.WithinMethodDeclarationParenthesis));
             }
 
             // For spacing after the 'not' pattern operator
@@ -433,15 +459,25 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
                 return CreateAdjustSpacesOperation(1, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
             }
 
+            // Slice pattern:
+            // .. var x
+            if (previousKind == SyntaxKind.DotDotToken && previousParentKind == SyntaxKind.SlicePattern)
+            {
+                return CreateAdjustSpacesOperation(1, AdjustSpacesOption.ForceSpacesIfOnSingleLine);
+            }
+
             // No space after $" and $@" and @$" at the start of an interpolated string
             if (previousKind is SyntaxKind.InterpolatedStringStartToken or
-                SyntaxKind.InterpolatedVerbatimStringStartToken)
+                                SyntaxKind.InterpolatedVerbatimStringStartToken or
+                                SyntaxKind.InterpolatedSingleLineRawStringStartToken or
+                                SyntaxKind.InterpolatedMultiLineRawStringStartToken)
             {
                 return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpaces);
             }
 
             // No space before " at the end of an interpolated string
-            if (currentKind == SyntaxKind.InterpolatedStringEndToken)
+            if (currentKind is SyntaxKind.InterpolatedStringEndToken or
+                               SyntaxKind.InterpolatedRawStringEndToken)
             {
                 return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpaces);
             }
@@ -453,10 +489,20 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
                 return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpaces);
             }
 
-            // Preserve space after { or before } in interpolations (i.e. between the braces and the expression)
-            if ((previousKind == SyntaxKind.OpenBraceToken && previousToken.Parent is InterpolationSyntax) ||
-                (currentKind == SyntaxKind.CloseBraceToken && currentToken.Parent is InterpolationSyntax))
+            // No space after { in interpolations (i.e. between the braces and the expression)
+            if (previousKind == SyntaxKind.OpenBraceToken && previousToken.Parent is InterpolationSyntax)
             {
+                return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpaces);
+            }
+
+            // Handle space before } in interpolations (i.e. between the braces and the expression)
+            if (currentKind == SyntaxKind.CloseBraceToken && currentToken.Parent is InterpolationSyntax interpolation)
+            {
+                // If there is no format specifier (i.e. a colon) remove spaces
+                if (interpolation.FormatClause is null)
+                    return CreateAdjustSpacesOperation(0, AdjustSpacesOption.ForceSpaces);
+
+                // If there is a format specifier then whitespace is significant so preserve it
                 return CreateAdjustSpacesOperation(0, AdjustSpacesOption.PreserveSpaces);
             }
 
@@ -518,19 +564,22 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
             SuppressVariableDeclaration(list, node);
         }
 
-        private static bool IsEmptyForStatement(ForStatementSyntax forStatement) =>
-            forStatement.Initializers.Count == 0
+        private static bool IsEmptyForStatement(ForStatementSyntax forStatement)
+            => forStatement.Initializers.Count == 0
             && forStatement.Declaration == null
             && forStatement.Condition == null
             && forStatement.Incrementors.Count == 0;
 
         private void SuppressVariableDeclaration(List<SuppressOperation> list, SyntaxNode node)
         {
-            if (node.IsKind(SyntaxKind.FieldDeclaration) || node.IsKind(SyntaxKind.EventDeclaration) ||
-                node.IsKind(SyntaxKind.EventFieldDeclaration) || node.IsKind(SyntaxKind.LocalDeclarationStatement) ||
-                node.IsKind(SyntaxKind.EnumMemberDeclaration))
+            if (node.Kind()
+                    is SyntaxKind.FieldDeclaration
+                    or SyntaxKind.EventDeclaration
+                    or SyntaxKind.EventFieldDeclaration
+                    or SyntaxKind.LocalDeclarationStatement
+                    or SyntaxKind.EnumMemberDeclaration)
             {
-                if (_options.SpacesIgnoreAroundVariableDeclaration)
+                if (_options.Spacing.HasFlag(SpacePlacement.IgnoreAroundVariableDeclaration))
                 {
                     var firstToken = node.GetFirstToken(includeZeroWidth: true);
                     var lastToken = node.GetLastToken(includeZeroWidth: true);
@@ -553,149 +602,18 @@ namespace Microsoft.CodeAnalysis.CSharp.Formatting
         }
 
         private static bool HasFormattableBracketParent(SyntaxToken token)
-            => token.Parent.IsKind(SyntaxKind.ArrayRankSpecifier, SyntaxKind.BracketedArgumentList, SyntaxKind.BracketedParameterList, SyntaxKind.ImplicitArrayCreationExpression);
+            => token.Parent is (kind: SyntaxKind.ArrayRankSpecifier or SyntaxKind.BracketedArgumentList or SyntaxKind.BracketedParameterList or SyntaxKind.ImplicitArrayCreationExpression or SyntaxKind.ListPattern or SyntaxKind.CollectionExpression);
 
         private static bool IsFunctionLikeKeywordExpressionKind(SyntaxKind syntaxKind)
             => (syntaxKind is SyntaxKind.TypeOfExpression or SyntaxKind.DefaultExpression or SyntaxKind.SizeOfExpression);
 
         private static bool IsControlFlowLikeKeywordStatementKind(SyntaxKind syntaxKind)
         {
-            return (syntaxKind is SyntaxKind.IfStatement or SyntaxKind.WhileStatement or SyntaxKind.SwitchStatement or
+            return syntaxKind is SyntaxKind.IfStatement or SyntaxKind.WhileStatement or SyntaxKind.SwitchStatement or
                 SyntaxKind.ForStatement or SyntaxKind.ForEachStatement or SyntaxKind.ForEachVariableStatement or
                 SyntaxKind.DoStatement or
                 SyntaxKind.CatchDeclaration or SyntaxKind.UsingStatement or SyntaxKind.LockStatement or
-                SyntaxKind.FixedStatement or SyntaxKind.CatchFilterClause);
-        }
-
-        private readonly struct CachedOptions : IEquatable<CachedOptions>
-        {
-            public readonly bool SpacesIgnoreAroundVariableDeclaration;
-            public readonly bool SpacingAfterMethodDeclarationName;
-            public readonly bool SpaceBetweenEmptyMethodDeclarationParentheses;
-            public readonly bool SpaceWithinMethodDeclarationParenthesis;
-            public readonly bool SpaceAfterMethodCallName;
-            public readonly bool SpaceBetweenEmptyMethodCallParentheses;
-            public readonly bool SpaceWithinMethodCallParentheses;
-            public readonly bool SpaceAfterControlFlowStatementKeyword;
-            public readonly bool SpaceWithinExpressionParentheses;
-            public readonly bool SpaceWithinCastParentheses;
-            public readonly bool SpaceBeforeSemicolonsInForStatement;
-            public readonly bool SpaceAfterSemicolonsInForStatement;
-            public readonly bool SpaceWithinOtherParentheses;
-            public readonly bool SpaceAfterCast;
-            public readonly bool SpaceBeforeOpenSquareBracket;
-            public readonly bool SpaceBetweenEmptySquareBrackets;
-            public readonly bool SpaceWithinSquareBrackets;
-            public readonly bool SpaceAfterColonInBaseTypeDeclaration;
-            public readonly bool SpaceBeforeColonInBaseTypeDeclaration;
-            public readonly bool SpaceAfterComma;
-            public readonly bool SpaceBeforeComma;
-            public readonly bool SpaceAfterDot;
-            public readonly bool SpaceBeforeDot;
-            public readonly BinaryOperatorSpacingOptions SpacingAroundBinaryOperator;
-
-            public CachedOptions(AnalyzerConfigOptions? options)
-            {
-                SpacesIgnoreAroundVariableDeclaration = GetOptionOrDefault(options, CSharpFormattingOptions2.SpacesIgnoreAroundVariableDeclaration);
-                SpacingAfterMethodDeclarationName = GetOptionOrDefault(options, CSharpFormattingOptions2.SpacingAfterMethodDeclarationName);
-                SpaceBetweenEmptyMethodDeclarationParentheses = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBetweenEmptyMethodDeclarationParentheses);
-                SpaceWithinMethodDeclarationParenthesis = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceWithinMethodDeclarationParenthesis);
-                SpaceAfterMethodCallName = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterMethodCallName);
-                SpaceBetweenEmptyMethodCallParentheses = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBetweenEmptyMethodCallParentheses);
-                SpaceWithinMethodCallParentheses = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceWithinMethodCallParentheses);
-                SpaceAfterControlFlowStatementKeyword = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterControlFlowStatementKeyword);
-                SpaceWithinExpressionParentheses = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceWithinExpressionParentheses);
-                SpaceWithinCastParentheses = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceWithinCastParentheses);
-                SpaceBeforeSemicolonsInForStatement = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBeforeSemicolonsInForStatement);
-                SpaceAfterSemicolonsInForStatement = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterSemicolonsInForStatement);
-                SpaceWithinOtherParentheses = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceWithinOtherParentheses);
-                SpaceAfterCast = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterCast);
-                SpaceBeforeOpenSquareBracket = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBeforeOpenSquareBracket);
-                SpaceBetweenEmptySquareBrackets = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBetweenEmptySquareBrackets);
-                SpaceWithinSquareBrackets = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceWithinSquareBrackets);
-                SpaceAfterColonInBaseTypeDeclaration = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterColonInBaseTypeDeclaration);
-                SpaceBeforeColonInBaseTypeDeclaration = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBeforeColonInBaseTypeDeclaration);
-                SpaceAfterComma = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterComma);
-                SpaceBeforeComma = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBeforeComma);
-                SpaceAfterDot = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceAfterDot);
-                SpaceBeforeDot = GetOptionOrDefault(options, CSharpFormattingOptions2.SpaceBeforeDot);
-                SpacingAroundBinaryOperator = GetOptionOrDefault(options, CSharpFormattingOptions2.SpacingAroundBinaryOperator);
-            }
-
-            public static bool operator ==(CachedOptions left, CachedOptions right)
-                => left.Equals(right);
-
-            public static bool operator !=(CachedOptions left, CachedOptions right)
-                => !(left == right);
-
-            private static T GetOptionOrDefault<T>(AnalyzerConfigOptions? options, Option2<T> option)
-            {
-                if (options is null)
-                    return option.DefaultValue;
-
-                return options.GetOption(option);
-            }
-
-            public override bool Equals(object? obj)
-                => obj is CachedOptions options && Equals(options);
-
-            public bool Equals(CachedOptions other)
-            {
-                return SpacesIgnoreAroundVariableDeclaration == other.SpacesIgnoreAroundVariableDeclaration
-                    && SpacingAfterMethodDeclarationName == other.SpacingAfterMethodDeclarationName
-                    && SpaceBetweenEmptyMethodDeclarationParentheses == other.SpaceBetweenEmptyMethodDeclarationParentheses
-                    && SpaceWithinMethodDeclarationParenthesis == other.SpaceWithinMethodDeclarationParenthesis
-                    && SpaceAfterMethodCallName == other.SpaceAfterMethodCallName
-                    && SpaceBetweenEmptyMethodCallParentheses == other.SpaceBetweenEmptyMethodCallParentheses
-                    && SpaceWithinMethodCallParentheses == other.SpaceWithinMethodCallParentheses
-                    && SpaceAfterControlFlowStatementKeyword == other.SpaceAfterControlFlowStatementKeyword
-                    && SpaceWithinExpressionParentheses == other.SpaceWithinExpressionParentheses
-                    && SpaceWithinCastParentheses == other.SpaceWithinCastParentheses
-                    && SpaceBeforeSemicolonsInForStatement == other.SpaceBeforeSemicolonsInForStatement
-                    && SpaceAfterSemicolonsInForStatement == other.SpaceAfterSemicolonsInForStatement
-                    && SpaceWithinOtherParentheses == other.SpaceWithinOtherParentheses
-                    && SpaceAfterCast == other.SpaceAfterCast
-                    && SpaceBeforeOpenSquareBracket == other.SpaceBeforeOpenSquareBracket
-                    && SpaceBetweenEmptySquareBrackets == other.SpaceBetweenEmptySquareBrackets
-                    && SpaceWithinSquareBrackets == other.SpaceWithinSquareBrackets
-                    && SpaceAfterColonInBaseTypeDeclaration == other.SpaceAfterColonInBaseTypeDeclaration
-                    && SpaceBeforeColonInBaseTypeDeclaration == other.SpaceBeforeColonInBaseTypeDeclaration
-                    && SpaceAfterComma == other.SpaceAfterComma
-                    && SpaceBeforeComma == other.SpaceBeforeComma
-                    && SpaceAfterDot == other.SpaceAfterDot
-                    && SpaceBeforeDot == other.SpaceBeforeDot
-                    && SpacingAroundBinaryOperator == other.SpacingAroundBinaryOperator;
-            }
-
-            public override int GetHashCode()
-            {
-                var hashCode = 0;
-                hashCode = (hashCode << 1) + (SpacesIgnoreAroundVariableDeclaration ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpacingAfterMethodDeclarationName ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBetweenEmptyMethodDeclarationParentheses ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceWithinMethodDeclarationParenthesis ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterMethodCallName ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBetweenEmptyMethodCallParentheses ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceWithinMethodCallParentheses ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterControlFlowStatementKeyword ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceWithinExpressionParentheses ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceWithinCastParentheses ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBeforeSemicolonsInForStatement ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterSemicolonsInForStatement ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceWithinOtherParentheses ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterCast ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBeforeOpenSquareBracket ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBetweenEmptySquareBrackets ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceWithinSquareBrackets ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterColonInBaseTypeDeclaration ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBeforeColonInBaseTypeDeclaration ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterComma ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBeforeComma ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceAfterDot ? 1 : 0);
-                hashCode = (hashCode << 1) + (SpaceBeforeDot ? 1 : 0);
-                hashCode = (hashCode << 2) + (int)SpacingAroundBinaryOperator;
-                return hashCode;
-            }
+                SyntaxKind.FixedStatement or SyntaxKind.CatchFilterClause;
         }
     }
 }
