@@ -121,6 +121,18 @@ namespace Microsoft.CodeAnalysis.Formatting
         /// </summary>
         protected abstract bool IsEndOfLine(SyntaxTrivia trivia);
 
+        /// <summary>
+        /// true if previoustrivia is _ and nextTrivia is a Visual Basic comment
+        /// </summary>
+        protected abstract bool LineContinuationFollowedByWhitespaceComment(SyntaxTrivia previousTrivia, SyntaxTrivia nextTrivia);
+
+        /// <summary>
+        /// check whether given trivia is a Comment in VB or not
+        /// It is never reachable in C# since it follows a test for
+        /// LineContinuation Character.
+        /// </summary>
+        protected abstract bool IsVisualBasicComment(SyntaxTrivia trivia);
+
         /// <summary>
         /// check whether given string is either null or whitespace
         /// </summary>
@@ -165,7 +177,7 @@ namespace Microsoft.CodeAnalysis.Formatting
         /// <summary>
         /// return line column rule for the given two trivia
         /// </summary>
-        protected abstract LineColumnRule GetLineColumnRuleBetween(SyntaxTrivia trivia1, LineColumnDelta existingWhitespaceBetween, bool implicitLineBreak, SyntaxTrivia trivia2);
+        protected abstract LineColumnRule GetLineColumnRuleBetween(SyntaxTrivia trivia1, LineColumnDelta existingWhitespaceBetween, bool implicitLineBreak, SyntaxTrivia trivia2, CancellationToken cancellationToken);
 
         /// <summary>
         /// format the given trivia at the line column position and put result to the changes list
@@ -213,7 +225,7 @@ namespace Microsoft.CodeAnalysis.Formatting
             get { return this.Context.TreeData; }
         }
 
-        protected AnalyzerConfigOptions Options
+        protected SyntaxFormattingOptions Options
         {
             get { return this.Context.Options; }
         }
@@ -279,8 +291,13 @@ namespace Microsoft.CodeAnalysis.Formatting
             var implicitLineBreak = false;
 
             var list = new TriviaList(this.Token1.TrailingTrivia, this.Token2.LeadingTrivia);
-            foreach (var trivia in list)
+
+            // Holds last position before _ ' Comment so we can reset after processing comment
+            var previousLineColumn = LineColumn.Default;
+            SyntaxTrivia trivia;
+            for (var i = 0; i < list.Count; i++)
             {
+                trivia = list[i];
                 if (trivia.RawKind == 0)
                 {
                     continue;
@@ -288,18 +305,31 @@ namespace Microsoft.CodeAnalysis.Formatting
 
                 if (IsWhitespaceOrEndOfLine(trivia))
                 {
+                    existingWhitespaceDelta = existingWhitespaceDelta.With(
+                       GetLineColumnOfWhitespace(
+                           lineColumn,
+                           previousTrivia,
+                           previousWhitespaceTrivia,
+                           existingWhitespaceDelta,
+                           trivia));
+
                     if (IsEndOfLine(trivia))
                     {
                         implicitLineBreak = false;
+                        // If we are on a new line we don't want to continue
+                        // reseting indenting this handles the case of a NewLine
+                        // followed by whitespace and a comment
+                        previousLineColumn = LineColumn.Default;
+                    }
+                    else if (LineContinuationFollowedByWhitespaceComment(previousTrivia, (i + 1) < list.Count ? list[i + 1] : default))
+                    {
+                        // we have a comment following an underscore space the formatter
+                        // thinks this next line should be shifted to right by
+                        // indentation value. Since we know through the test above that
+                        // this is the special case of _ ' Comment we don't want the extra indent
+                        // so we set the LineColumn value back to where it was before the comment
+                        previousLineColumn = lineColumn;
                     }
-
-                    existingWhitespaceDelta = existingWhitespaceDelta.With(
-                        GetLineColumnOfWhitespace(
-                            lineColumn,
-                            previousTrivia,
-                            previousWhitespaceTrivia,
-                            existingWhitespaceDelta,
-                            trivia));
 
                     previousWhitespaceTrivia = trivia;
                     continue;
@@ -313,6 +343,16 @@ namespace Microsoft.CodeAnalysis.Formatting
                     formatter, whitespaceAdder,
                     changes, implicitLineBreak, cancellationToken);
 
+                if (previousLineColumn.Column != 0
+                    && previousLineColumn.Column < lineColumn.Column
+                    && IsVisualBasicComment(trivia))
+                {
+                    lineColumn = previousLineColumn;
+                    // When we see a NewLine we don't want any special handling
+                    // for _ ' Comment
+                    previousLineColumn = LineColumn.Default;
+                }
+
                 implicitLineBreak = implicitLineBreak || ContainsImplicitLineBreak(trivia);
                 existingWhitespaceDelta = LineColumnDelta.Default;
 
@@ -342,7 +382,7 @@ namespace Microsoft.CodeAnalysis.Formatting
             var lineColumnAfterTrivia1 = trivia1.RawKind == 0 ?
                     lineColumnBeforeTrivia1 : lineColumnBeforeTrivia1.With(format(lineColumnBeforeTrivia1, trivia1, changes, cancellationToken));
 
-            var rule = GetOverallLineColumnRuleBetween(trivia1, existingWhitespaceBetween, implicitLineBreak, trivia2);
+            var rule = GetOverallLineColumnRuleBetween(trivia1, existingWhitespaceBetween, implicitLineBreak, trivia2, cancellationToken);
             var whitespaceDelta = Apply(lineColumnBeforeTrivia1, trivia1, lineColumnAfterTrivia1, existingWhitespaceBetween, trivia2, rule);
 
             var span = GetTextSpan(trivia1, trivia2);
@@ -354,9 +394,9 @@ namespace Microsoft.CodeAnalysis.Formatting
         /// <summary>
         /// get line column rule between two trivia
         /// </summary>
-        private LineColumnRule GetOverallLineColumnRuleBetween(SyntaxTrivia trivia1, LineColumnDelta existingWhitespaceBetween, bool implicitLineBreak, SyntaxTrivia trivia2)
+        private LineColumnRule GetOverallLineColumnRuleBetween(SyntaxTrivia trivia1, LineColumnDelta existingWhitespaceBetween, bool implicitLineBreak, SyntaxTrivia trivia2, CancellationToken cancellationToken)
         {
-            var defaultRule = GetLineColumnRuleBetween(trivia1, existingWhitespaceBetween, implicitLineBreak, trivia2);
+            var defaultRule = GetLineColumnRuleBetween(trivia1, existingWhitespaceBetween, implicitLineBreak, trivia2, cancellationToken);
             GetTokensAtEdgeOfStructureTrivia(trivia1, trivia2, out var token1, out var token2);
 
             // if there are tokens, try formatting rules to see whether there is a user supplied one
@@ -418,7 +458,7 @@ namespace Microsoft.CodeAnalysis.Formatting
         }
 
         /// <summary>
-        /// if the given trivia is the very first or the last trivia between two normal tokens and 
+        /// if the given trivia is the very first or the last trivia between two normal tokens and
         /// if the trivia is structured trivia, get one token that belongs to the structured trivia and one belongs to the normal token stream
         /// </summary>
         private void GetTokensAtEdgeOfStructureTrivia(SyntaxTrivia trivia1, SyntaxTrivia trivia2, out SyntaxToken token1, out SyntaxToken token2)
@@ -475,7 +515,7 @@ namespace Microsoft.CodeAnalysis.Formatting
         /// </summary>
         private bool FirstLineBlank()
         {
-            // if we see elastic trivia as the first trivia in the trivia list, 
+            // if we see elastic trivia as the first trivia in the trivia list,
             // we consider it as blank line
             if (this.Token1.TrailingTrivia.Count > 0 &&
                 this.Token1.TrailingTrivia[0].IsElastic())
@@ -568,8 +608,8 @@ namespace Microsoft.CodeAnalysis.Formatting
                 return this.Spaces;
             }
 
-            var position = lastText.ConvertTabToSpace(this.Options.GetOption(FormattingOptions2.TabSize), initialColumn, index);
-            var tokenPosition = lastText.ConvertTabToSpace(this.Options.GetOption(FormattingOptions2.TabSize), initialColumn, lastText.Length);
+            var position = lastText.ConvertTabToSpace(Options.TabSize, initialColumn, index);
+            var tokenPosition = lastText.ConvertTabToSpace(Options.TabSize, initialColumn, lastText.Length);
 
             return this.Spaces - (tokenPosition - position);
         }
@@ -601,7 +641,7 @@ namespace Microsoft.CodeAnalysis.Formatting
 
         private int GetInsertionIndex(ArrayBuilder<SyntaxTrivia> changes)
         {
-            // first line is blank or there is no changes. 
+            // first line is blank or there is no changes.
             // just insert at the head
             if (_firstLineBlank ||
                 changes.Count == 0)
@@ -690,7 +730,7 @@ namespace Microsoft.CodeAnalysis.Formatting
 
         private TextSpan GetInsertionSpan(ArrayBuilder<TextChange> changes)
         {
-            // first line is blank or there is no changes. 
+            // first line is blank or there is no changes.
             // just insert at the head
             if (_firstLineBlank ||
                 changes.Count == 0)
@@ -742,13 +782,10 @@ namespace Microsoft.CodeAnalysis.Formatting
                 return;
             }
 
-            var useTabs = this.Options.GetOption(FormattingOptions2.UseTabs);
-            var tabSize = this.Options.GetOption(FormattingOptions2.TabSize);
-
             // space indicates indentation
             if (delta.Lines > 0 || lineColumn.Column == 0)
             {
-                changes.Add(CreateWhitespace(delta.Spaces.CreateIndentationString(useTabs, tabSize)));
+                changes.Add(CreateWhitespace(delta.Spaces.CreateIndentationString(Options.UseTabs, Options.TabSize)));
                 return;
             }
 
@@ -760,7 +797,7 @@ namespace Microsoft.CodeAnalysis.Formatting
         {
             var sb = StringBuilderPool.Allocate();
 
-            var newLine = this.Options.GetOption(FormattingOptions2.NewLine);
+            var newLine = Options.NewLine;
             for (var i = 0; i < delta.Lines; i++)
             {
                 sb.Append(newLine);
@@ -771,13 +808,10 @@ namespace Microsoft.CodeAnalysis.Formatting
                 return StringBuilderPool.ReturnAndFree(sb);
             }
 
-            var useTabs = this.Options.GetOption(FormattingOptions2.UseTabs);
-            var tabSize = this.Options.GetOption(FormattingOptions2.TabSize);
-
             // space indicates indentation
             if (delta.Lines > 0 || lineColumn.Column == 0)
             {
-                sb.AppendIndentationString(delta.Spaces, useTabs, tabSize);
+                sb.AppendIndentationString(delta.Spaces, Options.UseTabs, Options.TabSize);
                 return StringBuilderPool.ReturnAndFree(sb);
             }
 
@@ -853,7 +887,7 @@ namespace Microsoft.CodeAnalysis.Formatting
             var text = trivia2.ToFullString();
             return new LineColumnDelta(
                 lines: 0,
-                spaces: text.ConvertTabToSpace(this.Options.GetOption(FormattingOptions2.TabSize), lineColumn.With(whitespaceBetween).Column, text.Length),
+                spaces: text.ConvertTabToSpace(Options.TabSize, lineColumn.With(whitespaceBetween).Column, text.Length),
                 whitespaceOnly: true,
                 forceUpdate: false);
         }
@@ -889,20 +923,20 @@ namespace Microsoft.CodeAnalysis.Formatting
             {
                 return new LineColumnDelta(
                     lines: text.GetNumberOfLineBreaks(),
-                    spaces: lineText.GetColumnFromLineOffset(lineText.Length, this.Options.GetOption(FormattingOptions2.TabSize)),
+                    spaces: lineText.GetColumnFromLineOffset(lineText.Length, Options.TabSize),
                     whitespaceOnly: IsNullOrWhitespace(lineText));
             }
 
             return new LineColumnDelta(
                 lines: 0,
-                spaces: text.ConvertTabToSpace(this.Options.GetOption(FormattingOptions2.TabSize), initialColumn, text.Length),
+                spaces: text.ConvertTabToSpace(Options.TabSize, initialColumn, text.Length),
                 whitespaceOnly: IsNullOrWhitespace(lineText));
         }
 
         protected int GetExistingIndentation(SyntaxTrivia trivia)
         {
             var offset = trivia.FullSpan.Start - this.StartPosition;
-            var originalText = this.OriginalString.Substring(0, offset);
+            var originalText = this.OriginalString[..offset];
             var delta = GetLineColumnDelta(this.InitialLineColumn.Column, originalText);
 
             return this.InitialLineColumn.With(delta).Column;
