@@ -2,22 +2,22 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 // See the LICENSE file in the project root for more information.
 
-#nullable disable
-
 using System;
 using System.Collections.Immutable;
 using System.Threading;
+using Microsoft.CodeAnalysis.Text;
 
 namespace Microsoft.CodeAnalysis.RemoveUnnecessaryImports
 {
-    internal interface IUnnecessaryImportsProvider
+    internal interface IUnnecessaryImportsProvider<TSyntaxNode>
+        where TSyntaxNode : SyntaxNode
     {
-        ImmutableArray<SyntaxNode> GetUnnecessaryImports(SemanticModel model, CancellationToken cancellationToken);
+        ImmutableArray<TSyntaxNode> GetUnnecessaryImports(SemanticModel model, TextSpan? span, CancellationToken cancellationToken);
 
-        ImmutableArray<SyntaxNode> GetUnnecessaryImports(
+        ImmutableArray<TSyntaxNode> GetUnnecessaryImports(
             SemanticModel model,
-            SyntaxNode root,
-            Func<SyntaxNode, bool> predicate,
+            TextSpan? span,
+            Func<SyntaxNode, bool>? predicate,
             CancellationToken cancellationToken);
     }
 }
