@@ -111,11 +111,11 @@ namespace Microsoft.CodeAnalysis.Internal.Log
         Formatting_ContextInitialization = 85,
         Formatting_Format = 86,
         Formatting_ApplyResultToBuffer = 87,
-        Formatting_IterateNodes = 88,
-        Formatting_CollectIndentBlock = 89,
-        Formatting_CollectSuppressOperation = 90,
-        Formatting_CollectAlignOperation = 91,
-        Formatting_CollectAnchorOperation = 92,
+        // obsolete: Formatting_IterateNodes = 88,
+        // obsolete: Formatting_CollectIndentBlock = 89,
+        // obsolete: Formatting_CollectSuppressOperation = 90,
+        // obsolete: Formatting_CollectAlignOperation = 91,
+        // obsolete: Formatting_CollectAnchorOperation = 92,
         Formatting_CollectTokenOperation = 93,
         Formatting_BuildContext = 94,
         Formatting_ApplySpaceAndLine = 95,
@@ -189,7 +189,6 @@ namespace Microsoft.CodeAnalysis.Internal.Log
         Snippet_OnAfterInsertion = 155,
 
         Misc_NonReentrantLock_BlockingWait = 156,
-        Misc_VisualStudioWaitIndicator_Wait = 157,
         Misc_SaveEventsSink_OnBeforeSave = 158,
 
         TaskList_Refresh = 159,
@@ -279,7 +278,7 @@ namespace Microsoft.CodeAnalysis.Internal.Log
         WorkCoordinatorRegistrationService_Unregister = 224,
         WorkCoordinatorRegistrationService_Reanalyze = 225,
 
-        WorkCoordinator_SolutionCrawlerOption = 226,
+        // obsolete: WorkCoordinator_SolutionCrawlerOption = 226,
         WorkCoordinator_PersistentStorageAdded = 227,
         WorkCoordinator_PersistentStorageRemoved = 228,
         WorkCoordinator_Shutdown = 229,
@@ -329,7 +328,7 @@ namespace Microsoft.CodeAnalysis.Internal.Log
         VisualBasic_Interactive_Window = 268,
 
         NonFatalWatson = 269,
-        GlobalOperationRegistration = 270,
+        // GlobalOperationRegistration = 270, No longer fired.
         CommandHandler_FindAllReference = 271,
 
         CodefixInfobar_Enable = 272,
@@ -418,7 +417,7 @@ namespace Microsoft.CodeAnalysis.Internal.Log
         LiveTableDataSource_OnDiagnosticsUpdated = 333,
         Experiment_KeybindingsReset = 334,
         Diagnostics_GeneratePerformaceReport = 335,
-        Diagnostics_BadAnalyzer = 336,
+        // obsolete: Diagnostics_BadAnalyzer = 336,
         CodeAnalysisService_ReportAnalyzerPerformance = 337,
         PerformanceTrackerService_AddSnapshot = 338,
         // obsolete: AbstractProject_SetIntelliSenseBuild = 339,
@@ -495,7 +494,7 @@ namespace Microsoft.CodeAnalysis.Internal.Log
         DependentTypeFinder_FindAndCacheImplementingTypesAsync = 432,
 
         RemoteSemanticClassificationCacheService_ExceptionInCacheRead = 440,
-        FeatureNotAvailable = 441,
+        // obsolete: FeatureNotAvailable = 441,
 
         LSPCompletion_MissingLSPCompletionTriggerKind = 450,
         LSPCompletion_MissingLSPCompletionInvokeKind = 451,
@@ -532,6 +531,90 @@ namespace Microsoft.CodeAnalysis.Internal.Log
 
         SuggestedActions_GetSuggestedActionsAsync = 500,
 
+        NavigateTo_CacheItemsMiss = 510,
+
         AssetService_Perf = 520,
+
+        Inline_Hints_DoubleClick = 530,
+        NavigateToExternalSources = 531,
+
+        StackTraceToolWindow_ShowOnActivated = 540,
+
+        CodeModel_FileCodeModel_Create = 550,
+
+        Refactoring_FixAllOccurrencesSession = 560,
+        Refactoring_FixAllOccurrencesContext = 561,
+        Refactoring_FixAllOccurrencesComputation = 562,
+        Refactoring_FixAllOccurrencesPreviewChanges = 563,
+
+        LSP_UsedForkedSolution = 571,
+
+        DocumentOutline_WindowOpen = 572,
+        DocumentOutline_SortByName = 573,
+        DocumentOutline_SortByOrder = 574,
+        DocumentOutline_SortByType = 575,
+
+        ApplyChangesOperation_WorkspaceVersionMatch_ApplicationSucceeded = 580,
+        ApplyChangesOperation_WorkspaceVersionMatch_ApplicationFailed = 581,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationSucceeded = 582,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationFailed_IncompatibleSolutionChange = 583,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationFailed_IncompatibleProjectChange = 584,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationFailed_NoChangedDocument = 585,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationFailed_NoTextChange = 586,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationFailed_DocumentRemoved = 587,
+        ApplyChangesOperation_WorkspaceVersionMismatch_ApplicationFailed_TextChangeConflict = 588,
+
+        Completion_SemanticSnippets = 589,
+        // please leave the range up through 600 free in case we need to add more items to learn more about ApplyChangesOperation results.
+
+        SpellCheckFixer_CouldNotFindDocument = 600,
+        SpellCheckFixer_LanguageDoesNotSupportRename = 601,
+        SpellCheckFixer_LanguageCouldNotGetRenameInfo = 602,
+        SpellCheckFixer_RenameSpanNotWithinTokenSpan = 603,
+        SpellCheckFixer_ReplacementTextInvalid = 604,
+        SpellCheckFixer_TryApplyChangesFailure = 605,
+
+        SourceGenerator_SolutionStatistics = 620,
+        SourceGenerator_OtherWorkspaceSessionStatistics = 621,
+
+        // 630-650 for sqlite errors.
+        SQLite_SqlException = 630,
+        SQLite_StorageDisabled = 631,
+
+        // 650-660 for diagnostic/fix related ids.
+        //Diagnostics_AnalyzerPerformanceInfo = 651, - Deprecated due to high volume of events.
+        Diagnostics_AnalyzerPerformanceInfo2 = 652,
+
+        // 660-670 for semantic model reuse service.
+        SemanticModelReuseLanguageService_TryGetSpeculativeSemanticModelAsync_Equivalent = 660,
+
+        // 670-680 for newer rename ids
+        Rename_TryApplyRename_WorkspaceChanged = 670,
+        Rename_InlineSession_Cancel_NonDocumentChangedWorkspaceChange = 671,
+
+        // 680-690 LSP Initialization info ids.
+        LSP_Initialize = 680,
+
+        // 700-799 Performance logging. Typically pairs with Delay/Summary values.
+        //  Summary is a aggregation of all times collected.
+        //  Delay is used as a non-aggregated value for an unexpectedly poor performing scenario.
+        CodeFix_Delay = 700,
+        CodeFix_Summary = 701,
+
+        CodeRefactoring_Delay = 710,
+        CodeRefactoring_Summary = 711,
+
+        PerformAnalysis_Delay = 720,
+        PerformAnalysis_Summary = 721,
+
+        RequestDiagnostics_Delay = 730,
+        RequestDiagnostics_Summary = 731,
+
+        SuggestedAction_Delay = 740,
+        SuggestedAction_Summary = 741,
+        SuggestedAction_Application_Delay = 742,
+        SuggestedAction_Application_Summary = 743,
+        SuggestedAction_Preview_Delay = 744,
+        SuggestedAction_Preview_Summary = 745,
     }
 }
