@@ -4,10 +4,6 @@
 
 using System;
 
-#if CODE_STYLE
-using System.Collections.Immutable;
-#endif
-
 namespace Microsoft.CodeAnalysis.Options
 {
     /// <summary>
@@ -22,16 +18,11 @@ namespace Microsoft.CodeAnalysis.Options
         , IOption
 #endif
     {
-        OptionDefinition OptionDefinition { get; }
+        OptionDefinition Definition { get; }
+        IPublicOption? PublicOption { get; }
 
 #if CODE_STYLE
-        string Feature { get; }
-        string Name { get; }
-        Type Type { get; }
-        object? DefaultValue { get; }
         bool IsPerLanguage { get; }
-
-        ImmutableArray<OptionStorageLocation2> StorageLocations { get; }
 #endif
     }
 }
