@@ -6,7 +6,7 @@
 
 using System.Threading;
 
-namespace Microsoft.CodeAnalysis.LanguageServices
+namespace Microsoft.CodeAnalysis.LanguageService
 {
     internal interface IDocumentationCommentService
     {
