@@ -153,20 +153,21 @@ namespace Roslyn.Utilities
 
             Contract.ThrowIfNull(continuationFunction, nameof(continuationFunction));
 
-            TResult outerFunction(Task t)
+            static TResult outerFunction(Task t, object? state)
             {
                 try
                 {
+                    var continuationFunction = (Func<Task, TResult>)state!;
                     return continuationFunction(t);
                 }
                 catch (Exception e) when (FatalError.ReportAndPropagateUnlessCanceled(e))
                 {
-                    throw ExceptionUtilities.Unreachable;
+                    throw ExceptionUtilities.Unreachable();
                 }
             }
 
             // This is the only place in the code where we're allowed to call ContinueWith.
-            return task.ContinueWith(outerFunction, cancellationToken, continuationOptions | TaskContinuationOptions.LazyCancellation, scheduler);
+            return task.ContinueWith(outerFunction, continuationFunction, cancellationToken, continuationOptions | TaskContinuationOptions.LazyCancellation, scheduler);
         }
 
         [SuppressMessage("Style", "VSTHRD200:Use \"Async\" suffix for async methods", Justification = "This is a Task wrapper, not an asynchronous method.")]
@@ -356,5 +357,47 @@ namespace Roslyn.Utilities
 
             return task;
         }
+
+        public static Task ReportNonFatalErrorUnlessCancelledAsync(this Task task, CancellationToken cancellationToken)
+        {
+            task.ContinueWith(p => FatalError.ReportAndCatchUnlessCanceled(p.Exception!, cancellationToken),
+                CancellationToken.None,
+                TaskContinuationOptions.OnlyOnFaulted | TaskContinuationOptions.ExecuteSynchronously,
+                TaskScheduler.Default);
+
+            return task;
+        }
+
+        /// <summary>
+        /// Asserts the <see cref="Task"/> passed has already been completed.
+        /// </summary>
+        /// <remarks>
+        /// This is useful for a specific case: sometimes you might be calling an API that is "sometimes" async, and you're
+        /// calling it from a synchronous method where you know it should have completed synchronously. This is an easy
+        /// way to assert that while silencing any compiler complaints.
+        /// </remarks>
+        public static void VerifyCompleted(this Task task)
+        {
+            Contract.ThrowIfFalse(task.IsCompleted);
+
+            // Propagate any exceptions that may have been thrown.
+            task.GetAwaiter().GetResult();
+        }
+
+        /// <summary>
+        /// Asserts the <see cref="Task"/> passed has already been completed.
+        /// </summary>
+        /// <remarks>
+        /// This is useful for a specific case: sometimes you might be calling an API that is "sometimes" async, and you're
+        /// calling it from a synchronous method where you know it should have completed synchronously. This is an easy
+        /// way to assert that while silencing any compiler complaints.
+        /// </remarks>
+        public static TResult VerifyCompleted<TResult>(this Task<TResult> task)
+        {
+            Contract.ThrowIfFalse(task.IsCompleted);
+
+            // Propagate any exceptions that may have been thrown.
+            return task.GetAwaiter().GetResult();
+        }
     }
 }
