@@ -35,16 +35,19 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
             return false;
         }
 
-        public static bool IsStaticKeywordInUsingDirective(this SyntaxToken token)
+        public static bool IsStaticKeywordContextInUsingDirective(this SyntaxToken token)
         {
-            if (token.IsKind(SyntaxKind.StaticKeyword))
+            // using static |
+            if (token is { RawKind: (int)SyntaxKind.StaticKeyword, Parent: UsingDirectiveSyntax })
             {
-                var usingDirective = token.GetAncestor<UsingDirectiveSyntax>();
-                if (usingDirective != null &&
-                    usingDirective.StaticKeyword == token)
-                {
-                    return true;
-                }
+                return true;
+            }
+
+            // using static unsafe |
+            if (token.IsKind(SyntaxKind.UnsafeKeyword) &&
+                token.GetPreviousToken() is { RawKind: (int)SyntaxKind.StaticKeyword, Parent: UsingDirectiveSyntax })
+            {
+                return true;
             }
 
             return false;
@@ -129,17 +132,14 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
                             // There are two exceptions.
                             // try {}
                             // do {}
-                            if (!token.Parent.IsParentKind(SyntaxKind.TryStatement) &&
-                                !token.Parent.IsParentKind(SyntaxKind.DoStatement))
-                            {
+                            if (token.Parent.Parent.Kind() is not SyntaxKind.TryStatement and not SyntaxKind.DoStatement)
                                 return true;
-                            }
                         }
-                        else if (
-                            token.Parent.IsParentKind(SyntaxKind.ElseClause) ||
-                            token.Parent.IsParentKind(SyntaxKind.FinallyClause) ||
-                            token.Parent.IsParentKind(SyntaxKind.CatchClause) ||
-                            token.Parent.IsParentKind(SyntaxKind.SwitchSection))
+                        else if (token.Parent.Parent?.Kind()
+                                is SyntaxKind.ElseClause
+                                or SyntaxKind.FinallyClause
+                                or SyntaxKind.CatchClause
+                                or SyntaxKind.SwitchSection)
                         {
                             return true;
                         }
@@ -153,24 +153,22 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
                     return false;
 
                 case SyntaxKind.ColonToken:
-                    return token.Parent.IsKind(SyntaxKind.CaseSwitchLabel,
-                                               SyntaxKind.DefaultSwitchLabel,
-                                               SyntaxKind.CasePatternSwitchLabel,
-                                               SyntaxKind.LabeledStatement);
+                    return token.Parent is (kind: SyntaxKind.CaseSwitchLabel or SyntaxKind.DefaultSwitchLabel or SyntaxKind.CasePatternSwitchLabel or SyntaxKind.LabeledStatement);
 
                 case SyntaxKind.DoKeyword when token.Parent.IsKind(SyntaxKind.DoStatement):
                     return true;
 
                 case SyntaxKind.CloseParenToken:
                     var parent = token.Parent;
-                    return parent.IsKind(SyntaxKind.ForStatement) ||
-                           parent.IsKind(SyntaxKind.ForEachStatement) ||
-                           parent.IsKind(SyntaxKind.ForEachVariableStatement) ||
-                           parent.IsKind(SyntaxKind.WhileStatement) ||
-                           parent.IsKind(SyntaxKind.IfStatement) ||
-                           parent.IsKind(SyntaxKind.LockStatement) ||
-                           parent.IsKind(SyntaxKind.UsingStatement) ||
-                           parent.IsKind(SyntaxKind.FixedStatement);
+                    return parent?.Kind()
+                        is SyntaxKind.ForStatement
+                        or SyntaxKind.ForEachStatement
+                        or SyntaxKind.ForEachVariableStatement
+                        or SyntaxKind.WhileStatement
+                        or SyntaxKind.IfStatement
+                        or SyntaxKind.LockStatement
+                        or SyntaxKind.UsingStatement
+                        or SyntaxKind.FixedStatement;
 
                 case SyntaxKind.ElseKeyword:
                     return token.Parent.IsKind(SyntaxKind.ElseClause);
@@ -222,8 +220,10 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
                 if (globalStatement != null && globalStatement.GetLastToken(includeZeroWidth: true) == token)
                     return true;
 
+                // Need this check to check file scoped namespace declarations prior to a catch-all of 
+                // member declarations since otherwise it would return true.
                 if (token.Parent is FileScopedNamespaceDeclarationSyntax namespaceDeclaration && namespaceDeclaration.SemicolonToken == token)
-                    return true;
+                    return false;
 
                 var memberDeclaration = token.GetAncestor<MemberDeclarationSyntax>();
                 if (memberDeclaration != null && memberDeclaration.GetLastToken(includeZeroWidth: true) == token &&
@@ -276,7 +276,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
                     return true;
                 }
 
-                if (token.Parent.IsKind(SyntaxKind.ParenthesizedExpression, out ParenthesizedExpressionSyntax? parenExpr))
+                if (token.Parent is ParenthesizedExpressionSyntax parenExpr)
                 {
                     var expr = parenExpr.Expression;
 
@@ -360,7 +360,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
             //   orderby a, b |
             //   orderby a, b a|
 
-            if (!targetToken.IsKind(SyntaxKind.IdentifierToken, SyntaxKind.CloseParenToken, SyntaxKind.CloseBracketToken))
+            if (targetToken.Kind() is not (SyntaxKind.IdentifierToken or SyntaxKind.CloseParenToken or SyntaxKind.CloseBracketToken))
             {
                 return false;
             }
@@ -400,8 +400,7 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
 
             if (targetToken.Kind() == SyntaxKind.ColonToken)
             {
-                if (targetToken.Parent.IsKind(
-                        SyntaxKind.CaseSwitchLabel, SyntaxKind.DefaultSwitchLabel, SyntaxKind.CasePatternSwitchLabel))
+                if (targetToken.Parent is (kind: SyntaxKind.CaseSwitchLabel or SyntaxKind.DefaultSwitchLabel or SyntaxKind.CasePatternSwitchLabel))
                 {
                     return true;
                 }
@@ -428,8 +427,8 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
 
         public static bool IsXmlCrefParameterModifierContext(this SyntaxToken targetToken)
         {
-            return targetToken.IsKind(SyntaxKind.CommaToken, SyntaxKind.OpenParenToken)
-                && targetToken.Parent.IsKind(SyntaxKind.CrefBracketedParameterList, SyntaxKind.CrefParameterList);
+            return targetToken.Kind() is SyntaxKind.CommaToken or SyntaxKind.OpenParenToken &&
+                   targetToken.Parent is (kind: SyntaxKind.CrefBracketedParameterList or SyntaxKind.CrefParameterList);
         }
 
         public static bool IsConstructorOrMethodParameterArgumentContext(this SyntaxToken targetToken)
@@ -453,10 +452,11 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
                 targetToken.Parent.Parent.Parent.IsKind(SyntaxKind.ArgumentList))
             {
                 var owner = targetToken.Parent.Parent.Parent.Parent;
-                if (owner.IsKind(SyntaxKind.InvocationExpression) ||
-                    owner.IsKind(SyntaxKind.ObjectCreationExpression) ||
-                    owner.IsKind(SyntaxKind.BaseConstructorInitializer) ||
-                    owner.IsKind(SyntaxKind.ThisConstructorInitializer))
+                if (owner?.Kind()
+                        is SyntaxKind.InvocationExpression
+                        or SyntaxKind.ObjectCreationExpression
+                        or SyntaxKind.BaseConstructorInitializer
+                        or SyntaxKind.ThisConstructorInitializer)
                 {
                     return true;
                 }
@@ -467,9 +467,10 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
             {
                 if (targetToken.Parent.IsKind(SyntaxKind.ArgumentList))
                 {
-                    if (targetToken.Parent.IsParentKind(SyntaxKind.ObjectCreationExpression) ||
-                        targetToken.Parent.IsParentKind(SyntaxKind.BaseConstructorInitializer) ||
-                        targetToken.Parent.IsParentKind(SyntaxKind.ThisConstructorInitializer))
+                    if (targetToken.Parent?.Parent?.Kind()
+                            is SyntaxKind.ObjectCreationExpression
+                            or SyntaxKind.BaseConstructorInitializer
+                            or SyntaxKind.ThisConstructorInitializer)
                     {
                         return true;
                     }
@@ -502,7 +503,8 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
         {
             return
                 targetToken.GetAncestors<StatementSyntax>().Any(s => s.IsKind(SyntaxKind.UnsafeStatement)) ||
-                targetToken.GetAncestors<MemberDeclarationSyntax>().Any(m => m.GetModifiers().Any(SyntaxKind.UnsafeKeyword));
+                targetToken.GetAncestors<MemberDeclarationSyntax>().Any(m => m.GetModifiers().Any(SyntaxKind.UnsafeKeyword) ||
+                targetToken.GetAncestors<LocalFunctionStatementSyntax>().Any(f => f.GetModifiers().Any(SyntaxKind.UnsafeKeyword)));
         }
 
         public static bool IsAfterYieldKeyword(this SyntaxToken targetToken)
@@ -612,9 +614,9 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions.ContextQuery
         {
             if (node.IsKind(SyntaxKind.TypeParameterList))
             {
-                if (node.IsParentKind(SyntaxKind.InterfaceDeclaration, out TypeDeclarationSyntax? typeDecl))
+                if (node?.Parent is TypeDeclarationSyntax(SyntaxKind.InterfaceDeclaration) typeDecl)
                     return typeDecl.TypeParameterList == node;
-                else if (node.IsParentKind(SyntaxKind.DelegateDeclaration, out DelegateDeclarationSyntax? delegateDecl))
+                else if (node?.Parent is DelegateDeclarationSyntax delegateDecl)
                     return delegateDecl.TypeParameterList == node;
             }
 
