@@ -10,6 +10,6 @@ namespace Microsoft.CodeAnalysis.CSharp.Extensions
     internal static class SyntaxTokenListExtensions
     {
         public static IEnumerable<SyntaxToken> SkipKinds(this SyntaxTokenList tokenList, params SyntaxKind[] kinds)
-            => tokenList.SkipWhile(t => t.IsKind(kinds));
+            => tokenList.SkipWhile(t => kinds.Contains(t.Kind()));
     }
 }
