@@ -34,7 +34,7 @@ namespace System
             // create the hashtable that keeps track of inherited types
             Dictionary<Type, AttributeUsageAttribute> types = new Dictionary<Type, AttributeUsageAttribute>(11);
 
-            // create an array list to collect all the requested attibutes
+            // create an array list to collect all the requested attributes
             List<Attribute> attributeList = new List<Attribute>();
             CopyToAttributeList(attributeList, attributes, types);
             do
@@ -137,7 +137,7 @@ namespace System
             }
 
             // create the hashtable that keeps track of inherited types
-            // create an array list to collect all the requested attibutes
+            // create an array list to collect all the requested attributes
             Dictionary<Type, AttributeUsageAttribute> types = new Dictionary<Type, AttributeUsageAttribute>(11);
             List<Attribute> attributeList = new List<Attribute>();
             CopyToAttributeList(attributeList, attributes, types);
@@ -434,10 +434,8 @@ namespace System
                 SR.Format(SR.Format_AttributeUsage, type));
         }
 
-        private static Attribute[] CreateAttributeArrayHelper(Type elementType, int elementCount)
-        {
-            return (Attribute[])Array.CreateInstance(elementType, elementCount);
-        }
+        private static Attribute[] CreateAttributeArrayHelper(Type elementType, int elementCount) =>
+            elementType.ContainsGenericParameters ? new Attribute[elementCount] : (Attribute[])Array.CreateInstance(elementType, elementCount);
         #endregion
 
         #endregion
@@ -452,11 +450,8 @@ namespace System
 
         public static Attribute[] GetCustomAttributes(MemberInfo element, Type attributeType, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
-
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -465,7 +460,7 @@ namespace System
             {
                 MemberTypes.Property => InternalGetCustomAttributes((PropertyInfo)element, attributeType, inherit),
                 MemberTypes.Event => InternalGetCustomAttributes((EventInfo)element, attributeType, inherit),
-                _ => (element.GetCustomAttributes(attributeType, inherit) as Attribute[])!,
+                _ => (Attribute[])element.GetCustomAttributes(attributeType, inherit)
             };
         }
 
@@ -476,14 +471,13 @@ namespace System
 
         public static Attribute[] GetCustomAttributes(MemberInfo element, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
 
             return element.MemberType switch
             {
                 MemberTypes.Property => InternalGetCustomAttributes((PropertyInfo)element, typeof(Attribute), inherit),
                 MemberTypes.Event => InternalGetCustomAttributes((EventInfo)element, typeof(Attribute), inherit),
-                _ => (element.GetCustomAttributes(typeof(Attribute), inherit) as Attribute[])!,
+                _ => (Attribute[])element.GetCustomAttributes(typeof(Attribute), inherit)
             };
         }
 
@@ -494,13 +488,10 @@ namespace System
 
         public static bool IsDefined(MemberInfo element, Type attributeType, bool inherit)
         {
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
+
             // Returns true if a custom attribute subclass of attributeType class/interface with inheritance walk
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
-
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
-
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
 
@@ -524,10 +515,11 @@ namespace System
             if (attrib == null || attrib.Length == 0)
                 return null;
 
+            Attribute match = attrib[0];
             if (attrib.Length == 1)
-                return attrib[0];
+                return match;
 
-            throw new AmbiguousMatchException(SR.RFLCT_AmbigCust);
+            throw ThrowHelper.GetAmbiguousMatchException(match);
         }
 
         #endregion
@@ -545,11 +537,8 @@ namespace System
 
         public static Attribute[] GetCustomAttributes(ParameterInfo element, Type attributeType, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
-
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -557,28 +546,25 @@ namespace System
             if (element.Member == null)
                 throw new ArgumentException(SR.Argument_InvalidParameterInfo, nameof(element));
 
-
             MemberInfo member = element.Member;
             if (member.MemberType == MemberTypes.Method && inherit)
                 return InternalParamGetCustomAttributes(element, attributeType, inherit);
 
-            return (element.GetCustomAttributes(attributeType, inherit) as Attribute[])!;
+            return (Attribute[])element.GetCustomAttributes(attributeType, inherit);
         }
 
         public static Attribute[] GetCustomAttributes(ParameterInfo element, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
 
             if (element.Member == null)
                 throw new ArgumentException(SR.Argument_InvalidParameterInfo, nameof(element));
 
-
             MemberInfo member = element.Member;
             if (member.MemberType == MemberTypes.Method && inherit)
                 return InternalParamGetCustomAttributes(element, null, inherit);
 
-            return (element.GetCustomAttributes(typeof(Attribute), inherit) as Attribute[])!;
+            return (Attribute[])element.GetCustomAttributes(typeof(Attribute), inherit);
         }
 
         public static bool IsDefined(ParameterInfo element, Type attributeType)
@@ -588,12 +574,10 @@ namespace System
 
         public static bool IsDefined(ParameterInfo element, Type attributeType, bool inherit)
         {
-            // Returns true is a custom attribute subclass of attributeType class/interface with inheritance walk
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            // Returns true is a custom attribute subclass of attributeType class/interface with inheritance walk
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -624,20 +608,18 @@ namespace System
 
         public static Attribute? GetCustomAttribute(ParameterInfo element, Type attributeType, bool inherit)
         {
-            // Returns an Attribute of base class/inteface attributeType on the ParameterInfo or null if none exists.
+            // Returns an Attribute of base class/interface attributeType on the ParameterInfo or null if none exists.
             // throws an AmbiguousMatchException if there are more than one defined.
             Attribute[] attrib = GetCustomAttributes(element, attributeType, inherit);
 
             if (attrib == null || attrib.Length == 0)
                 return null;
 
-            if (attrib.Length == 0)
-                return null;
-
+            Attribute match = attrib[0];
             if (attrib.Length == 1)
-                return attrib[0];
+                return match;
 
-            throw new AmbiguousMatchException(SR.RFLCT_AmbigCust);
+            throw ThrowHelper.GetAmbiguousMatchException(match);
         }
 
         #endregion
@@ -655,19 +637,15 @@ namespace System
 
         public static Attribute[] GetCustomAttributes(Module element, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
 
             return (Attribute[])element.GetCustomAttributes(typeof(Attribute), inherit);
         }
 
         public static Attribute[] GetCustomAttributes(Module element, Type attributeType, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
-
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -682,12 +660,10 @@ namespace System
 
         public static bool IsDefined(Module element, Type attributeType, bool inherit)
         {
-            // Returns true is a custom attribute subclass of attributeType class/interface with no inheritance walk
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            // Returns true is a custom attribute subclass of attributeType class/interface with no inheritance walk
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -702,17 +678,18 @@ namespace System
 
         public static Attribute? GetCustomAttribute(Module element, Type attributeType, bool inherit)
         {
-            // Returns an Attribute of base class/inteface attributeType on the Module or null if none exists.
+            // Returns an Attribute of base class/interface attributeType on the Module or null if none exists.
             // throws an AmbiguousMatchException if there are more than one defined.
             Attribute[] attrib = GetCustomAttributes(element, attributeType, inherit);
 
             if (attrib == null || attrib.Length == 0)
                 return null;
 
+            Attribute match = attrib[0];
             if (attrib.Length == 1)
-                return attrib[0];
+                return match;
 
-            throw new AmbiguousMatchException(SR.RFLCT_AmbigCust);
+            throw ThrowHelper.GetAmbiguousMatchException(match);
         }
 
         #endregion
@@ -725,11 +702,8 @@ namespace System
 
         public static Attribute[] GetCustomAttributes(Assembly element, Type attributeType, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
-
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -744,8 +718,7 @@ namespace System
 
         public static Attribute[] GetCustomAttributes(Assembly element, bool inherit)
         {
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
 
             return (Attribute[])element.GetCustomAttributes(typeof(Attribute), inherit);
         }
@@ -757,12 +730,10 @@ namespace System
 
         public static bool IsDefined(Assembly element, Type attributeType, bool inherit)
         {
-            // Returns true is a custom attribute subclass of attributeType class/interface with no inheritance walk
-            if (element == null)
-                throw new ArgumentNullException(nameof(element));
+            ArgumentNullException.ThrowIfNull(element);
+            ArgumentNullException.ThrowIfNull(attributeType);
 
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            // Returns true is a custom attribute subclass of attributeType class/interface with no inheritance walk
 
             if (!attributeType.IsSubclassOf(typeof(Attribute)) && attributeType != typeof(Attribute))
                 throw new ArgumentException(SR.Argument_MustHaveAttributeBaseClass);
@@ -777,17 +748,18 @@ namespace System
 
         public static Attribute? GetCustomAttribute(Assembly element, Type attributeType, bool inherit)
         {
-            // Returns an Attribute of base class/inteface attributeType on the Assembly or null if none exists.
+            // Returns an Attribute of base class/interface attributeType on the Assembly or null if none exists.
             // throws an AmbiguousMatchException if there are more than one defined.
             Attribute[] attrib = GetCustomAttributes(element, attributeType, inherit);
 
             if (attrib == null || attrib.Length == 0)
                 return null;
 
+            Attribute match = attrib[0];
             if (attrib.Length == 1)
-                return attrib[0];
+                return match;
 
-            throw new AmbiguousMatchException(SR.RFLCT_AmbigCust);
+            throw ThrowHelper.GetAmbiguousMatchException(match);
         }
 
         #endregion
