@@ -8,10 +8,10 @@ using System.Diagnostics;
 
 namespace System.Reflection.Emit
 {
-    internal sealed class DynamicILGenerator : ILGenerator
+    internal sealed class DynamicILGenerator : RuntimeILGenerator
     {
         internal DynamicScope m_scope;
-        private int m_methodSigToken;
+        private readonly int m_methodSigToken;
 
         internal DynamicILGenerator(DynamicMethod method, byte[] methodSignature, int size)
             : base(method, size)
@@ -23,7 +23,7 @@ namespace System.Reflection.Emit
 
         internal void GetCallableMethod(RuntimeModule module, DynamicMethod dm)
         {
-            dm.m_methodHandle = ModuleHandle.GetDynamicMethod(dm,
+            dm._methodHandle = ModuleHandle.GetDynamicMethod(dm,
                                           module,
                                           m_methodBuilder.Name,
                                           (byte[])m_scope[m_methodSigToken]!,
@@ -34,16 +34,16 @@ namespace System.Reflection.Emit
 
         public override LocalBuilder DeclareLocal(Type localType, bool pinned)
         {
+            ArgumentNullException.ThrowIfNull(localType);
+
             LocalBuilder localBuilder;
-            if (localType == null)
-                throw new ArgumentNullException(nameof(localType));
 
             RuntimeType? rtType = localType as RuntimeType;
 
             if (rtType == null)
                 throw new ArgumentException(SR.Argument_MustBeRuntimeType);
 
-            localBuilder = new LocalBuilder(m_localCount, localType, m_methodBuilder);
+            localBuilder = new LocalBuilder(m_localCount, localType, m_methodBuilder, pinned);
             // add the localType to local signature
             m_localSignature.AddArgument(localType, pinned);
             m_localCount++;
@@ -57,11 +57,10 @@ namespace System.Reflection.Emit
         //
         public override void Emit(OpCode opcode, MethodInfo meth)
         {
-            if (meth == null)
-                throw new ArgumentNullException(nameof(meth));
+            ArgumentNullException.ThrowIfNull(meth);
 
             int stackchange = 0;
-            int token = 0;
+            int token;
             DynamicMethod? dynMeth = meth as DynamicMethod;
             if (dynMeth == null)
             {
@@ -106,14 +105,12 @@ namespace System.Reflection.Emit
             }
 
             UpdateStackSize(opcode, stackchange);
-
             PutInteger4(token);
         }
 
         public override void Emit(OpCode opcode, ConstructorInfo con)
         {
-            if (con == null)
-                throw new ArgumentNullException(nameof(con));
+            ArgumentNullException.ThrowIfNull(con);
 
             RuntimeConstructorInfo? rtConstructor = con as RuntimeConstructorInfo;
             if (rtConstructor == null)
@@ -133,14 +130,12 @@ namespace System.Reflection.Emit
 
             // need to sort out the stack size story
             UpdateStackSize(opcode, 1);
-
             PutInteger4(token);
         }
 
         public override void Emit(OpCode opcode, Type type)
         {
-            if (type == null)
-                throw new ArgumentNullException(nameof(type));
+            ArgumentNullException.ThrowIfNull(type);
 
             RuntimeType? rtType = type as RuntimeType;
 
@@ -155,8 +150,7 @@ namespace System.Reflection.Emit
 
         public override void Emit(OpCode opcode, FieldInfo field)
         {
-            if (field == null)
-                throw new ArgumentNullException(nameof(field));
+            ArgumentNullException.ThrowIfNull(field);
 
             RuntimeFieldInfo? runtimeField = field as RuntimeFieldInfo;
             if (runtimeField == null)
@@ -175,8 +169,7 @@ namespace System.Reflection.Emit
 
         public override void Emit(OpCode opcode, string str)
         {
-            if (str == null)
-                throw new ArgumentNullException(nameof(str));
+            ArgumentNullException.ThrowIfNull(str);
 
             int tempVal = GetTokenForString(str);
             EnsureCapacity(7);
@@ -196,15 +189,17 @@ namespace System.Reflection.Emit
                                        Type[]? optionalParameterTypes)
         {
             int stackchange = 0;
-            SignatureHelper sig;
             if (optionalParameterTypes != null)
                 if ((callingConvention & CallingConventions.VarArgs) == 0)
                     throw new InvalidOperationException(SR.InvalidOperation_NotAVarArgCallingConvention);
 
-            sig = GetMemberRefSignature(callingConvention,
-                                        returnType,
-                                        parameterTypes,
-                                        optionalParameterTypes);
+
+            SignatureHelper sig = GetMethodSigHelper(callingConvention,
+                                                     returnType,
+                                                     parameterTypes,
+                                                     null,
+                                                     null,
+                                                     optionalParameterTypes);
 
             EnsureCapacity(7);
             Emit(OpCodes.Calli);
@@ -233,17 +228,10 @@ namespace System.Reflection.Emit
         {
             int stackchange = 0;
             int cParams = 0;
-            int i;
-            SignatureHelper sig;
-
             if (parameterTypes != null)
                 cParams = parameterTypes.Length;
 
-            sig = SignatureHelper.GetMethodSigHelper(unmanagedCallConv, returnType);
-
-            if (parameterTypes != null)
-                for (i = 0; i < cParams; i++)
-                    sig.AddArgument(parameterTypes[i]);
+            SignatureHelper sig = GetMethodSigHelper(unmanagedCallConv, returnType, parameterTypes);
 
             // If there is a non-void return type, push one.
             if (returnType != typeof(void))
@@ -266,8 +254,7 @@ namespace System.Reflection.Emit
 
         public override void EmitCall(OpCode opcode, MethodInfo methodInfo, Type[]? optionalParameterTypes)
         {
-            if (methodInfo == null)
-                throw new ArgumentNullException(nameof(methodInfo));
+            ArgumentNullException.ThrowIfNull(methodInfo);
 
             if (!(opcode.Equals(OpCodes.Call) || opcode.Equals(OpCodes.Callvirt) || opcode.Equals(OpCodes.Newobj)))
                 throw new ArgumentException(SR.Argument_NotMethodCallOpcode, nameof(opcode));
@@ -305,8 +292,7 @@ namespace System.Reflection.Emit
 
         public override void Emit(OpCode opcode, SignatureHelper signature)
         {
-            if (signature == null)
-                throw new ArgumentNullException(nameof(signature));
+            ArgumentNullException.ThrowIfNull(signature);
 
             int stackchange = 0;
             EnsureCapacity(7);
@@ -353,7 +339,7 @@ namespace System.Reflection.Emit
             current.MarkFilterAddr(ILOffset);
         }
 
-        public override void BeginCatchBlock(Type exceptionType)
+        public override void BeginCatchBlock(Type? exceptionType)
         {
             if (CurrExcStackCount == 0)
                 throw new NotSupportedException(SR.Argument_NotInExceptionBlock);
@@ -376,8 +362,7 @@ namespace System.Reflection.Emit
             else
             {
                 // execute this branch if previous clause is Catch or Fault
-                if (exceptionType == null)
-                    throw new ArgumentNullException(nameof(exceptionType));
+                ArgumentNullException.ThrowIfNull(exceptionType);
 
                 if (rtType == null)
                     throw new ArgumentException(SR.Argument_MustBeRuntimeType);
@@ -416,7 +401,7 @@ namespace System.Reflection.Emit
             throw new NotSupportedException(SR.InvalidOperation_NotAllowedInDynamicMethod);
         }
 
-        private int GetMemberRefToken(MethodBase methodInfo, Type[]? optionalParameterTypes)
+        private int GetMemberRefToken(MethodInfo methodInfo, Type[]? optionalParameterTypes)
         {
             Type[]? parameterTypes;
             Type[][]? requiredCustomModifiers;
@@ -452,8 +437,8 @@ namespace System.Reflection.Emit
                 optionalCustomModifiers = null;
             }
 
-            SignatureHelper sig = GetMemberRefSignature(methodInfo.CallingConvention,
-                                                     MethodBuilder.GetMethodBaseReturnType(methodInfo),
+            SignatureHelper sig = GetMethodSigHelper(methodInfo.CallingConvention,
+                                                     RuntimeMethodBuilder.GetMethodBaseReturnType(methodInfo),
                                                      parameterTypes,
                                                      requiredCustomModifiers,
                                                      optionalCustomModifiers,
@@ -465,7 +450,17 @@ namespace System.Reflection.Emit
                 return GetTokenForVarArgMethod(dm!, sig);
         }
 
-        internal override SignatureHelper GetMemberRefSignature(
+        private SignatureHelper GetMethodSigHelper(
+                                                CallingConvention unmanagedCallConv,
+                                                Type? returnType,
+                                                Type[]? parameterTypes)
+        {
+            SignatureHelper sigHelp = SignatureHelper.GetMethodSigHelper(null, unmanagedCallConv, returnType);
+            AddParameters(sigHelp, parameterTypes, null, null);
+            return sigHelp;
+        }
+
+        private SignatureHelper GetMethodSigHelper(
                                                 CallingConventions call,
                                                 Type? returnType,
                                                 Type[]? parameterTypes,
@@ -473,8 +468,8 @@ namespace System.Reflection.Emit
                                                 Type[][]? optionalCustomModifiers,
                                                 Type[]? optionalParameterTypes)
         {
-            SignatureHelper sig = SignatureHelper.GetMethodSigHelper(null, call, returnType, null, null, parameterTypes, requiredCustomModifiers, optionalCustomModifiers);
-
+            SignatureHelper sig = SignatureHelper.GetMethodSigHelper(call, returnType);
+            AddParameters(sig, parameterTypes, requiredCustomModifiers, optionalCustomModifiers);
             if (optionalParameterTypes != null && optionalParameterTypes.Length != 0)
             {
                 sig.AddSentinel();
@@ -483,6 +478,23 @@ namespace System.Reflection.Emit
             return sig;
         }
 
+        private void AddParameters(SignatureHelper sigHelp, Type[]? parameterTypes, Type[][]? requiredCustomModifiers, Type[][]? optionalCustomModifiers)
+        {
+            if (requiredCustomModifiers != null && (parameterTypes == null || requiredCustomModifiers.Length != parameterTypes.Length))
+                throw new ArgumentException(SR.Format(SR.Argument_MismatchedArrays, nameof(requiredCustomModifiers), nameof(parameterTypes)));
+
+            if (optionalCustomModifiers != null && (parameterTypes == null || optionalCustomModifiers.Length != parameterTypes.Length))
+                throw new ArgumentException(SR.Format(SR.Argument_MismatchedArrays, nameof(optionalCustomModifiers), nameof(parameterTypes)));
+
+            if (parameterTypes != null)
+            {
+                for (int i = 0; i < parameterTypes.Length; i++)
+                {
+                    sigHelp.AddDynamicArgument(m_scope, parameterTypes[i], requiredCustomModifiers?[i], optionalCustomModifiers?[i]);
+                }
+            }
+        }
+
         internal override void RecordTokenFixup()
         {
             // DynamicMethod doesn't need fixup.
@@ -557,13 +569,13 @@ namespace System.Reflection.Emit
     internal sealed class DynamicResolver : Resolver
     {
         #region Private Data Members
-        private __ExceptionInfo[]? m_exceptions;
-        private byte[]? m_exceptionHeader;
-        private DynamicMethod m_method;
-        private byte[] m_code;
-        private byte[] m_localSignature;
-        private int m_stackSize;
-        private DynamicScope m_scope;
+        private readonly __ExceptionInfo[]? m_exceptions;
+        private readonly byte[]? m_exceptionHeader;
+        private readonly DynamicMethod m_method;
+        private readonly byte[] m_code;
+        private readonly byte[] m_localSignature;
+        private readonly int m_stackSize;
+        private readonly DynamicScope m_scope;
         #endregion
 
         #region Internal Methods
@@ -576,7 +588,7 @@ namespace System.Reflection.Emit
             m_scope = ilGenerator.m_scope;
 
             m_method = (DynamicMethod)ilGenerator.m_methodBuilder;
-            m_method.m_resolver = this;
+            m_method._resolver = this;
         }
 
         internal DynamicResolver(DynamicILInfo dynamicILInfo)
@@ -588,7 +600,7 @@ namespace System.Reflection.Emit
             m_scope = dynamicILInfo.DynamicScope;
 
             m_method = dynamicILInfo.DynamicMethod;
-            m_method.m_resolver = this;
+            m_method._resolver = this;
         }
 
         //
@@ -614,7 +626,7 @@ namespace System.Reflection.Emit
             if (method == null)
                 return;
 
-            if (method.m_methodHandle == null)
+            if (method._methodHandle == null)
                 return;
 
             DestroyScout scout;
@@ -631,7 +643,7 @@ namespace System.Reflection.Emit
 
             // We can never ever have two active destroy scouts for the same method. We need to initialize the scout
             // outside the try/reregister block to avoid possibility of reregistration for finalization with active scout.
-            scout.m_methodHandle = method.m_methodHandle.Value;
+            scout.m_methodHandle = method._methodHandle.Value;
         }
 
         private sealed class DestroyScout
@@ -673,12 +685,12 @@ namespace System.Reflection.Emit
 
             SecurityControlFlags flags = SecurityControlFlags.Default;
 
-            if (m_method.m_restrictedSkipVisibility)
+            if (m_method._restrictedSkipVisibility)
                 flags |= SecurityControlFlags.RestrictedSkipVisibilityChecks;
-            else if (m_method.m_skipVisibility)
+            else if (m_method._skipVisibility)
                 flags |= SecurityControlFlags.SkipVisibilityChecks;
 
-            typeOwner = m_method.m_typeOwner;
+            typeOwner = m_method._typeOwner;
 
             securityControlFlags = (int)flags;
 
@@ -820,7 +832,7 @@ namespace System.Reflection.Emit
                 if (vaMeth.m_dynamicMethod == null)
                 {
                     methodHandle = vaMeth.m_method!.MethodHandle.Value;
-                    typeHandle = vaMeth.m_method.GetDeclaringTypeInternal().GetTypeHandleInternal().Value;
+                    typeHandle = vaMeth.m_method.GetDeclaringTypeInternal().TypeHandle.Value;
                 }
                 else
                 {
@@ -836,23 +848,20 @@ namespace System.Reflection.Emit
             return m_scope.ResolveSignature(token, fromMethod);
         }
 
-        internal override MethodInfo GetDynamicMethod()
-        {
-            return m_method.GetMethodInfo();
-        }
+        internal override MethodInfo GetDynamicMethod() => m_method;
         #endregion
     }
 
     public class DynamicILInfo
     {
         #region Private Data Members
-        private DynamicMethod m_method;
-        private DynamicScope m_scope;
+        private readonly DynamicMethod m_method;
+        private readonly DynamicScope m_scope;
         private byte[] m_exceptions;
         private byte[] m_code;
         private byte[] m_localSignature;
         private int m_maxStackSize;
-        private int m_methodSignature;
+        private readonly int m_methodSignature;
         #endregion
 
         #region Constructor
@@ -870,7 +879,7 @@ namespace System.Reflection.Emit
         #region Internal Methods
         internal void GetCallableMethod(RuntimeModule module, DynamicMethod dm)
         {
-            dm.m_methodHandle = ModuleHandle.GetDynamicMethod(dm,
+            dm._methodHandle = ModuleHandle.GetDynamicMethod(dm,
                 module, m_method.Name, (byte[])m_scope[m_methodSignature]!, new DynamicResolver(this));
         }
 
@@ -893,10 +902,9 @@ namespace System.Reflection.Emit
         [CLSCompliant(false)]
         public unsafe void SetCode(byte* code, int codeSize, int maxStackSize)
         {
-            if (codeSize < 0)
-                throw new ArgumentOutOfRangeException(nameof(codeSize), SR.ArgumentOutOfRange_GenericPositive);
-            if (codeSize > 0 && code == null)
-                throw new ArgumentNullException(nameof(code));
+            ArgumentOutOfRangeException.ThrowIfNegative(codeSize);
+            if (codeSize > 0)
+                ArgumentNullException.ThrowIfNull(code);
 
             m_code = new Span<byte>(code, codeSize).ToArray();
             m_maxStackSize = maxStackSize;
@@ -910,11 +918,10 @@ namespace System.Reflection.Emit
         [CLSCompliant(false)]
         public unsafe void SetExceptions(byte* exceptions, int exceptionsSize)
         {
-            if (exceptionsSize < 0)
-                throw new ArgumentOutOfRangeException(nameof(exceptionsSize), SR.ArgumentOutOfRange_GenericPositive);
+            ArgumentOutOfRangeException.ThrowIfNegative(exceptionsSize);
 
-            if (exceptionsSize > 0 && exceptions == null)
-                throw new ArgumentNullException(nameof(exceptions));
+            if (exceptionsSize > 0)
+                ArgumentNullException.ThrowIfNull(exceptions);
 
             m_exceptions = new Span<byte>(exceptions, exceptionsSize).ToArray();
         }
@@ -927,11 +934,9 @@ namespace System.Reflection.Emit
         [CLSCompliant(false)]
         public unsafe void SetLocalSignature(byte* localSignature, int signatureSize)
         {
-            if (signatureSize < 0)
-                throw new ArgumentOutOfRangeException(nameof(signatureSize), SR.ArgumentOutOfRange_GenericPositive);
-
-            if (signatureSize > 0 && localSignature == null)
-                throw new ArgumentNullException(nameof(localSignature));
+            ArgumentOutOfRangeException.ThrowIfNegative(signatureSize);
+            if (signatureSize > 0)
+                ArgumentNullException.ThrowIfNull(localSignature);
 
             m_localSignature = new Span<byte>(localSignature, signatureSize).ToArray();
         }
@@ -1022,7 +1027,7 @@ namespace System.Reflection.Emit
                 if (!RuntimeMethodHandle.IsDynamicMethod(rmhi))
                 {
                     RuntimeType type = RuntimeMethodHandle.GetDeclaringType(rmhi);
-                    if ((type != null) && RuntimeTypeHandle.HasInstantiation(type))
+                    if (type.IsGenericType)
                     {
                         // Do we really need to retrieve this much info just to throw an exception?
                         MethodBase m = RuntimeType.GetMethodBase(methodReal)!;
