@@ -1,6 +1,7 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
+using System.ComponentModel;
 using System.Collections.Generic;
 using System.Diagnostics;
 using System.Diagnostics.CodeAnalysis;
@@ -13,10 +14,11 @@ using System.Runtime.InteropServices;
 using System.Runtime.CompilerServices;
 using System.Runtime.Serialization;
 using System.Threading;
+using System.Security;
 
 namespace System.Reflection
 {
-    internal class RuntimeAssembly : Assembly
+    internal sealed partial class RuntimeAssembly : Assembly
     {
         internal RuntimeAssembly() { throw new NotSupportedException(); }
 
@@ -36,12 +38,18 @@ namespace System.Reflection
 
         private sealed class ManifestResourceStream : UnmanagedMemoryStream
         {
-            private RuntimeAssembly _manifestAssembly;
+            // ensures the RuntimeAssembly is kept alive for as long as the stream lives
+            private readonly RuntimeAssembly _manifestAssembly;
 
             internal unsafe ManifestResourceStream(RuntimeAssembly manifestAssembly, byte* pointer, long length, long capacity, FileAccess access) : base(pointer, length, capacity, access)
             {
                 _manifestAssembly = manifestAssembly;
             }
+
+            // override Read(Span<byte>) because the base UnmanagedMemoryStream doesn't optimize it for derived types
+            public override int Read(Span<byte> buffer) => ReadCore(buffer);
+
+            // NOTE: no reason to override Write(Span<byte>), since a ManifestResourceStream is read-only.
         }
 
         internal object SyncRoot
@@ -68,8 +76,9 @@ namespace System.Reflection
             }
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern bool GetCodeBase(QCallAssembly assembly,
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetCodeBase")]
+        [return: MarshalAs(UnmanagedType.Bool)]
+        private static partial bool GetCodeBase(QCallAssembly assembly,
                                                StringHandleOnStack retString);
 
         internal string? GetCodeBase()
@@ -83,17 +92,30 @@ namespace System.Reflection
             return null;
         }
 
+        [Obsolete("Assembly.CodeBase and Assembly.EscapedCodeBase are only included for .NET Framework compatibility. Use Assembly.Location.", DiagnosticId = "SYSLIB0012", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
         [RequiresAssemblyFiles(ThrowingMessageInRAF)]
         public override string? CodeBase
         {
             get
             {
-                var codeBase = GetCodeBase();
+                if (IsDynamic)
+                {
+                    throw new NotSupportedException(SR.NotSupported_DynamicAssembly);
+                }
+
+                string? codeBase = GetCodeBase();
                 if (codeBase is null)
                 {
-                    // Not supported if the assembly was loaded from memory
+                    // Not supported if the assembly was loaded from single-file bundle.
                     throw new NotSupportedException(SR.NotSupported_CodeBase);
                 }
+                if (codeBase.Length == 0)
+                {
+                    // For backward compatibility, return CoreLib codebase for assemblies loaded from memory.
+#pragma warning disable SYSLIB0012
+                    codeBase = typeof(object).Assembly.CodeBase;
+#pragma warning restore SYSLIB0012
+                }
                 return codeBase;
             }
         }
@@ -105,17 +127,23 @@ namespace System.Reflection
         // is returned.
         public override AssemblyName GetName(bool copiedName)
         {
-            string? codeBase = GetCodeBase();
+            var an = new AssemblyName();
+            an.Name = GetSimpleName();
+            an.Version = GetVersion();
+            an.CultureInfo = GetLocale();
 
-            var an = new AssemblyName(GetSimpleName(),
-                    GetPublicKey(),
-                    null, // public key token
-                    GetVersion(),
-                    GetLocale(),
-                    GetHashAlgorithm(),
-                    AssemblyVersionCompatibility.SameMachine,
-                    codeBase,
-                    GetFlags() | AssemblyNameFlags.PublicKey);
+            an.SetPublicKey(GetPublicKey());
+
+            an.RawFlags = GetFlags() | AssemblyNameFlags.PublicKey;
+
+#pragma warning disable IL3000, SYSLIB0044 // System.Reflection.AssemblyName.CodeBase' always returns an empty string for assemblies embedded in a single-file app.
+                                           // AssemblyName.CodeBase and AssemblyName.EscapedCodeBase are obsolete. Using them for loading an assembly is not supported.
+            an.CodeBase = GetCodeBase();
+#pragma warning restore IL3000, SYSLIB0044
+
+#pragma warning disable SYSLIB0037 // AssemblyName.HashAlgorithm is obsolete
+            an.HashAlgorithm = GetHashAlgorithm();
+#pragma warning restore SYSLIB0037
 
             Module manifestModule = ManifestModule;
             if (manifestModule.MDStreamVersion > 0x10000)
@@ -126,8 +154,8 @@ namespace System.Reflection
             return an;
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetFullName(QCallAssembly assembly, StringHandleOnStack retString);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetFullName")]
+        private static partial void GetFullName(QCallAssembly assembly, StringHandleOnStack retString);
 
         public override string? FullName
         {
@@ -146,8 +174,8 @@ namespace System.Reflection
             }
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetEntryPoint(QCallAssembly assembly, ObjectHandleOnStack retMethod);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetEntryPoint")]
+        private static partial void GetEntryPoint(QCallAssembly assembly, ObjectHandleOnStack retMethod);
 
         public override MethodInfo? EntryPoint
         {
@@ -164,41 +192,69 @@ namespace System.Reflection
             }
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetType(QCallAssembly assembly,
-                                            string name,
-                                            bool throwOnError,
-                                            bool ignoreCase,
-                                            ObjectHandleOnStack type,
-                                            ObjectHandleOnStack keepAlive,
-                                            ObjectHandleOnStack assemblyLoadContext);
+        // For case-sensitive lookups, marshal the strings directly to Utf8 to avoid unnecessary string copies.
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetTypeCore", StringMarshalling = StringMarshalling.Utf8)]
+        private static partial void GetTypeCore(QCallAssembly assembly,
+                                            string typeName,
+                                            ReadOnlySpan<string> nestedTypeNames,
+                                            int nestedTypeNamesLength,
+                                            ObjectHandleOnStack retType);
 
-        [RequiresUnreferencedCode("Types might be removed")]
-        public override Type? GetType(string name, bool throwOnError, bool ignoreCase)
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetTypeCoreIgnoreCase", StringMarshalling = StringMarshalling.Utf16)]
+        private static partial void GetTypeCoreIgnoreCase(QCallAssembly assembly,
+                                            string typeName,
+                                            ReadOnlySpan<string> nestedTypeNames,
+                                            int nestedTypeNamesLength,
+                                            ObjectHandleOnStack retType);
+
+        internal Type? GetTypeCore(string typeName, ReadOnlySpan<string> nestedTypeNames, bool throwOnError, bool ignoreCase)
         {
-            // throw on null strings regardless of the value of "throwOnError"
-            if (name == null)
-                throw new ArgumentNullException(nameof(name));
-
-            RuntimeType? type = null;
-            object? keepAlive = null;
-            AssemblyLoadContext? assemblyLoadContextStack = AssemblyLoadContext.CurrentContextualReflectionContext;
-
             RuntimeAssembly runtimeAssembly = this;
-            GetType(new QCallAssembly(ref runtimeAssembly),
-                    name,
-                    throwOnError,
-                    ignoreCase,
-                    ObjectHandleOnStack.Create(ref type),
-                    ObjectHandleOnStack.Create(ref keepAlive),
-                    ObjectHandleOnStack.Create(ref assemblyLoadContextStack));
-            GC.KeepAlive(keepAlive);
+            Type? type = null;
+
+            try
+            {
+                if (ignoreCase)
+                {
+                    GetTypeCoreIgnoreCase(new QCallAssembly(ref runtimeAssembly),
+                        typeName,
+                        nestedTypeNames,
+                        nestedTypeNames.Length,
+                        ObjectHandleOnStack.Create(ref type));
+                }
+                else
+                {
+                    GetTypeCore(new QCallAssembly(ref runtimeAssembly),
+                        typeName,
+                        nestedTypeNames,
+                        nestedTypeNames.Length,
+                        ObjectHandleOnStack.Create(ref type));
+                }
+            }
+            catch (FileNotFoundException) when (!throwOnError)
+            {
+                return null;
+            }
+
+            if (type == null && throwOnError)
+                throw new TypeLoadException(SR.Format(SR.ClassLoad_General /* TypeLoad_TypeNotFoundInAssembly */, typeName, FullName));
 
             return type;
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetExportedTypes(QCallAssembly assembly, ObjectHandleOnStack retTypes);
+        [RequiresUnreferencedCode("Types might be removed")]
+        public override Type? GetType(
+            string name, // throw on null strings regardless of the value of "throwOnError"
+            bool throwOnError, bool ignoreCase)
+        {
+            ArgumentException.ThrowIfNullOrEmpty(name);
+
+            return TypeNameParser.GetType(name, topLevelAssembly: this,
+                throwOnError: throwOnError, ignoreCase: ignoreCase);
+        }
+
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetExportedTypes")]
+        private static partial void GetExportedTypes(QCallAssembly assembly, ObjectHandleOnStack retTypes);
 
         [RequiresUnreferencedCode("Types might be removed")]
         public override Type[] GetExportedTypes()
@@ -231,8 +287,8 @@ namespace System.Reflection
             }
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        internal static extern Interop.BOOL GetIsCollectible(QCallAssembly assembly);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetIsCollectible")]
+        internal static partial Interop.BOOL GetIsCollectible(QCallAssembly assembly);
 
         public override bool IsCollectible
         {
@@ -244,22 +300,22 @@ namespace System.Reflection
         }
 
         // GetResource will return a pointer to the resources in memory.
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern unsafe byte* GetResource(QCallAssembly assembly,
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetResource", StringMarshalling = StringMarshalling.Utf16)]
+        private static unsafe partial byte* GetResource(QCallAssembly assembly,
                                                        string resourceName,
                                                        out uint length);
 
         // Load a resource based on the NameSpace of the type.
         public override Stream? GetManifestResourceStream(Type type, string name)
         {
-            if (type == null && name == null)
-                throw new ArgumentNullException(nameof(type));
+            if (name == null)
+                ArgumentNullException.ThrowIfNull(type);
 
             string? nameSpace = type?.Namespace;
 
             char c = Type.Delimiter;
             string resourceName = nameSpace != null && name != null ?
-                string.Concat(nameSpace, new ReadOnlySpan<char>(ref c, 1), name) :
+                string.Concat(nameSpace, new ReadOnlySpan<char>(in c), name) :
                 string.Concat(nameSpace, name);
 
             return GetManifestResourceStream(resourceName);
@@ -279,6 +335,8 @@ namespace System.Reflection
         }
 
         // ISerializable implementation
+        [Obsolete(Obsoletions.LegacyFormatterImplMessage, DiagnosticId = Obsoletions.LegacyFormatterImplDiagId, UrlFormat = Obsoletions.SharedUrlFormat)]
+        [EditorBrowsable(EditorBrowsableState.Never)]
         public override void GetObjectData(SerializationInfo info, StreamingContext context)
         {
             throw new PlatformNotSupportedException();
@@ -287,7 +345,7 @@ namespace System.Reflection
         public override Module ManifestModule =>
             // We don't need to return the "external" ModuleBuilder because
             // it is meant to be read-only
-            RuntimeAssembly.GetManifestModule(GetNativeHandle());
+            GetManifestModule(GetNativeHandle());
 
         public override object[] GetCustomAttributes(bool inherit)
         {
@@ -296,12 +354,9 @@ namespace System.Reflection
 
         public override object[] GetCustomAttributes(Type attributeType, bool inherit)
         {
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            ArgumentNullException.ThrowIfNull(attributeType);
 
-            RuntimeType? attributeRuntimeType = attributeType.UnderlyingSystemType as RuntimeType;
-
-            if (attributeRuntimeType == null)
+            if (attributeType.UnderlyingSystemType is not RuntimeType attributeRuntimeType)
                 throw new ArgumentException(SR.Arg_MustBeType, nameof(attributeType));
 
             return CustomAttribute.GetCustomAttributes(this, attributeRuntimeType);
@@ -309,12 +364,9 @@ namespace System.Reflection
 
         public override bool IsDefined(Type attributeType, bool inherit)
         {
-            if (attributeType == null)
-                throw new ArgumentNullException(nameof(attributeType));
+            ArgumentNullException.ThrowIfNull(attributeType);
 
-            RuntimeType? attributeRuntimeType = attributeType.UnderlyingSystemType as RuntimeType;
-
-            if (attributeRuntimeType == null)
+            if (attributeType.UnderlyingSystemType is not RuntimeType attributeRuntimeType)
                 throw new ArgumentException(SR.Arg_MustBeType, nameof(attributeType));
 
             return CustomAttribute.IsDefined(this, attributeRuntimeType);
@@ -328,30 +380,59 @@ namespace System.Reflection
         internal static RuntimeAssembly InternalLoad(string assemblyName, ref StackCrawlMark stackMark, AssemblyLoadContext? assemblyLoadContext = null)
             => InternalLoad(new AssemblyName(assemblyName), ref stackMark, assemblyLoadContext);
 
-        internal static RuntimeAssembly InternalLoad(AssemblyName assemblyName, ref StackCrawlMark stackMark, AssemblyLoadContext? assemblyLoadContext = null)
-            => InternalLoad(assemblyName, requestingAssembly: null, ref stackMark, throwOnFileNotFound: true, assemblyLoadContext);
-
-        internal static RuntimeAssembly InternalLoad(AssemblyName assemblyName,
-                                                     RuntimeAssembly? requestingAssembly,
-                                                     ref StackCrawlMark stackMark,
-                                                     bool throwOnFileNotFound,
-                                                     AssemblyLoadContext? assemblyLoadContext = null)
+        internal static unsafe RuntimeAssembly InternalLoad(AssemblyName assemblyName,
+                                                            ref StackCrawlMark stackMark,
+                                                            AssemblyLoadContext? assemblyLoadContext = null,
+                                                            RuntimeAssembly? requestingAssembly = null,
+                                                            bool throwOnFileNotFound = true)
         {
             RuntimeAssembly? retAssembly = null;
-            InternalLoad(ObjectHandleOnStack.Create(ref assemblyName),
-                         ObjectHandleOnStack.Create(ref requestingAssembly),
-                         new StackCrawlMarkHandle(ref stackMark),
-                         throwOnFileNotFound,
-                         ObjectHandleOnStack.Create(ref assemblyLoadContext),
-                         ObjectHandleOnStack.Create(ref retAssembly));
+
+            AssemblyNameFlags flags = assemblyName.RawFlags;
+
+            // Note that we prefer to take a public key token if present,
+            // even if flags indicate a full public key
+            byte[]? publicKeyOrToken;
+            if ((publicKeyOrToken = assemblyName.RawPublicKeyToken) != null)
+            {
+                flags &= ~AssemblyNameFlags.PublicKey;
+            }
+            else if ((publicKeyOrToken = assemblyName.RawPublicKey) != null)
+            {
+                flags |= AssemblyNameFlags.PublicKey;
+            }
+
+            fixed (char* pName = assemblyName.Name)
+            fixed (char* pCultureName = assemblyName.CultureName)
+            fixed (byte* pPublicKeyOrToken = publicKeyOrToken)
+            {
+                NativeAssemblyNameParts nameParts = default;
+
+                nameParts._flags = flags;
+                nameParts._pName = pName;
+                nameParts._pCultureName = pCultureName;
+
+                nameParts._pPublicKeyOrToken = pPublicKeyOrToken;
+                nameParts._cbPublicKeyOrToken = (publicKeyOrToken != null) ? publicKeyOrToken.Length : 0;
+
+                nameParts.SetVersion(assemblyName.Version, defaultValue: ushort.MaxValue);
+
+                InternalLoad(&nameParts,
+                             ObjectHandleOnStack.Create(ref requestingAssembly),
+                             new StackCrawlMarkHandle(ref stackMark),
+                             throwOnFileNotFound,
+                             ObjectHandleOnStack.Create(ref assemblyLoadContext),
+                             ObjectHandleOnStack.Create(ref retAssembly));
+            }
+
             return retAssembly!;
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void InternalLoad(ObjectHandleOnStack assemblyName,
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_InternalLoad")]
+        private static unsafe partial void InternalLoad(NativeAssemblyNameParts* pAssemblyNameParts,
                                                 ObjectHandleOnStack requestingAssembly,
                                                 StackCrawlMarkHandle stackMark,
-                                                bool throwOnFileNotFound,
+                                                [MarshalAs(UnmanagedType.Bool)] bool throwOnFileNotFound,
                                                 ObjectHandleOnStack assemblyLoadContext,
                                                 ObjectHandleOnStack retAssembly);
 
@@ -359,8 +440,8 @@ namespace System.Reflection
 
         // Returns the module in this assembly with name 'name'
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetModule(QCallAssembly assembly, string name, ObjectHandleOnStack retModule);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetModule", StringMarshalling = StringMarshalling.Utf16)]
+        private static partial void GetModule(QCallAssembly assembly, string name, ObjectHandleOnStack retModule);
 
         public override Module? GetModule(string name)
         {
@@ -432,8 +513,8 @@ namespace System.Reflection
             return GetReferencedAssemblies(GetNativeHandle());
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern int GetManifestResourceInfo(QCallAssembly assembly,
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetManifestResourceInfo", StringMarshalling = StringMarshalling.Utf16)]
+        private static partial int GetManifestResourceInfo(QCallAssembly assembly,
                                                           string resourceName,
                                                           ObjectHandleOnStack assemblyRef,
                                                           StringHandleOnStack retFileName);
@@ -454,8 +535,8 @@ namespace System.Reflection
                                                 (ResourceLocation)location);
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetLocation(QCallAssembly assembly, StringHandleOnStack retString);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetLocation")]
+        private static partial void GetLocation(QCallAssembly assembly, StringHandleOnStack retString);
 
         public override string Location
         {
@@ -470,8 +551,8 @@ namespace System.Reflection
             }
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetImageRuntimeVersion(QCallAssembly assembly, StringHandleOnStack retString);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetImageRuntimeVersion")]
+        private static partial void GetImageRuntimeVersion(QCallAssembly assembly, StringHandleOnStack retString);
 
         public override string ImageRuntimeVersion
         {
@@ -489,24 +570,24 @@ namespace System.Reflection
 
         public override long HostContext => 0;
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetVersion(QCallAssembly assembly,
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetVersion")]
+        private static partial void GetVersion(QCallAssembly assembly,
                                               out int majVer,
                                               out int minVer,
                                               out int buildNum,
                                               out int revNum);
 
-        internal Version GetVersion()
+        private Version GetVersion()
         {
             RuntimeAssembly runtimeAssembly = this;
             GetVersion(new QCallAssembly(ref runtimeAssembly), out int majorVer, out int minorVer, out int build, out int revision);
             return new Version(majorVer, minorVer, build, revision);
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetLocale(QCallAssembly assembly, StringHandleOnStack retString);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetLocale")]
+        private static partial void GetLocale(QCallAssembly assembly, StringHandleOnStack retString);
 
-        internal CultureInfo GetLocale()
+        private CultureInfo GetLocale()
         {
             string? locale = null;
 
@@ -524,8 +605,8 @@ namespace System.Reflection
 
         public override bool IsDynamic => FCallIsDynamic(GetNativeHandle());
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetSimpleName(QCallAssembly assembly, StringHandleOnStack retSimpleName);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetSimpleName")]
+        private static partial void GetSimpleName(QCallAssembly assembly, StringHandleOnStack retSimpleName);
 
         internal string? GetSimpleName()
         {
@@ -535,8 +616,8 @@ namespace System.Reflection
             return name;
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern AssemblyHashAlgorithm GetHashAlgorithm(QCallAssembly assembly);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetHashAlgorithm")]
+        private static partial AssemblyHashAlgorithm GetHashAlgorithm(QCallAssembly assembly);
 
         private AssemblyHashAlgorithm GetHashAlgorithm()
         {
@@ -544,8 +625,8 @@ namespace System.Reflection
             return GetHashAlgorithm(new QCallAssembly(ref runtimeAssembly));
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern AssemblyNameFlags GetFlags(QCallAssembly assembly);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetFlags")]
+        private static partial AssemblyNameFlags GetFlags(QCallAssembly assembly);
 
         private AssemblyNameFlags GetFlags()
         {
@@ -553,8 +634,8 @@ namespace System.Reflection
             return GetFlags(new QCallAssembly(ref runtimeAssembly));
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetPublicKey(QCallAssembly assembly, ObjectHandleOnStack retPublicKey);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetPublicKey")]
+        private static partial void GetPublicKey(QCallAssembly assembly, ObjectHandleOnStack retPublicKey);
 
         internal byte[]? GetPublicKey()
         {
@@ -572,13 +653,12 @@ namespace System.Reflection
         // Useful for binding to a very specific version of a satellite assembly
         public override Assembly GetSatelliteAssembly(CultureInfo culture, Version? version)
         {
-            if (culture == null)
-                throw new ArgumentNullException(nameof(culture));
+            ArgumentNullException.ThrowIfNull(culture);
 
             return InternalGetSatelliteAssembly(culture, version, throwOnFileNotFound: true)!;
         }
 
-        [System.Security.DynamicSecurityMethod] // Methods containing StackCrawlMark local var has to be marked DynamicSecurityMethod
+        [DynamicSecurityMethod] // Methods containing StackCrawlMark local var has to be marked DynamicSecurityMethod
         internal Assembly? InternalGetSatelliteAssembly(CultureInfo culture,
                                                        Version? version,
                                                        bool throwOnFileNotFound)
@@ -593,7 +673,7 @@ namespace System.Reflection
             // This stack crawl mark is never used because the requesting assembly is explicitly specified,
             // so the value could be anything.
             StackCrawlMark unused = default;
-            RuntimeAssembly? retAssembly = InternalLoad(an, this, ref unused, throwOnFileNotFound);
+            RuntimeAssembly? retAssembly = InternalLoad(an, ref unused, requestingAssembly: this, throwOnFileNotFound: throwOnFileNotFound);
 
             if (retAssembly == this)
             {
@@ -608,10 +688,10 @@ namespace System.Reflection
             return retAssembly;
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetModules(QCallAssembly assembly,
-                                              bool loadIfNotFound,
-                                              bool getResourceModules,
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetModules")]
+        private static partial void GetModules(QCallAssembly assembly,
+                                              [MarshalAs(UnmanagedType.Bool)] bool loadIfNotFound,
+                                              [MarshalAs(UnmanagedType.Bool)] bool getResourceModules,
                                               ObjectHandleOnStack retModuleHandles);
 
         private RuntimeModule[] GetModulesInternal(bool loadIfNotFound,
@@ -713,7 +793,7 @@ namespace System.Reflection
             }
         }
 
-        [DllImport(RuntimeHelpers.QCall, CharSet = CharSet.Unicode)]
-        private static extern void GetForwardedType(QCallAssembly assembly, MetadataToken mdtExternalType, ObjectHandleOnStack type);
+        [LibraryImport(RuntimeHelpers.QCall, EntryPoint = "AssemblyNative_GetForwardedType")]
+        private static partial void GetForwardedType(QCallAssembly assembly, MetadataToken mdtExternalType, ObjectHandleOnStack type);
     }
 }
