@@ -1,21 +1,19 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
+
 //*****************************************************************************
 // File: RsMain.cpp
 //
-
-// Random RS utility stuff, plus root ICorCordbug implementation
+// Random RS utility stuff, plus root ICorDebug implementation
 //
 //*****************************************************************************
+
 #include "stdafx.h"
 #include "primitives.h"
 #include "safewrap.h"
 
 #include "check.h"
 
-#include <tlhelp32.h>
-#include "wtsapi32.h"
-
 #ifndef SM_REMOTESESSION
 #define SM_REMOTESESSION 0x1000
 #endif
@@ -595,7 +593,7 @@ namespace
 
         COM_METHOD CreateConnection(ICorDebugProcess *pProcess,
                                     CONNID dwConnectionId,
-                                    __in_z WCHAR* pConnectionName);
+                                    _In_z_ WCHAR* pConnectionName);
         COM_METHOD ChangeConnection(ICorDebugProcess *pProcess, CONNID dwConnectionId);
         COM_METHOD DestroyConnection(ICorDebugProcess *pProcess, CONNID dwConnectionId);
 
@@ -725,7 +723,7 @@ namespace
     HRESULT
     DefaultManagedCallback2::CreateConnection(ICorDebugProcess *pProcess,
                                               CONNID dwConnectionId,
-                                              __in_z WCHAR* pConnectionName)
+                                              _In_z_ WCHAR* pConnectionName)
     {
         _ASSERTE(!"DefaultManagedCallback2::CreateConnection not implemented");
         return E_NOTIMPL;
@@ -966,11 +964,8 @@ Cordb::Cordb(CorDebugInterfaceVersion iDebuggerVersion, const ProcessDescriptor&
     m_initialized(false),
     m_debuggerSpecifiedVersion(iDebuggerVersion),
     m_pd(pd),
-    m_dacModulePath(dacModulePath)
-#ifdef FEATURE_CORESYSTEM
-    ,
+    m_dacModulePath(dacModulePath),
     m_targetCLR(0)
-#endif
 {
     g_pRSDebuggingInfo->m_Cordb = this;
 
@@ -1421,10 +1416,7 @@ HRESULT Cordb::SetTargetCLR(HMODULE hmodTargetCLR)
     if (m_initialized)
         return E_FAIL;
 
-#ifdef FEATURE_CORESYSTEM
     m_targetCLR = hmodTargetCLR;
-#endif
-
     return S_OK;
 }
 
@@ -1529,16 +1521,6 @@ bool Cordb::IsInteropDebuggingSupported()
     // We explicitly refrain from checking the unmanaged callback. See comment in
     // ICorDebug::SetUnmanagedHandler for details.
 #ifdef FEATURE_INTEROP_DEBUGGING
-
-#if !defined(FEATURE_CORESYSTEM)
-    // Interop debugging is only supported internally on CoreCLR.
-    // Check if the special reg key is set.  If not, then we don't allow interop debugging.
-    if (CLRConfig::GetConfigValue(CLRConfig::INTERNAL_DbgEnableMixedModeDebugging) == 0)
-    {
-        return false;
-    }
-#endif // FEATURE_CORESYSTEM
-
     return true;
 #else
     return false;
@@ -1573,7 +1555,7 @@ bool Cordb::IsInteropDebuggingSupported()
 //
 //---------------------------------------------------------------------------------------
 HRESULT Cordb::CreateProcess(LPCWSTR lpApplicationName,
-                             __in_z LPWSTR lpCommandLine,
+                             _In_z_ LPWSTR lpCommandLine,
                              LPSECURITY_ATTRIBUTES lpProcessAttributes,
                              LPSECURITY_ATTRIBUTES lpThreadAttributes,
                              BOOL bInheritHandles,
@@ -1602,7 +1584,7 @@ HRESULT Cordb::CreateProcess(LPCWSTR lpApplicationName,
 
 HRESULT Cordb::CreateProcessCommon(ICorDebugRemoteTarget * pRemoteTarget,
                                    LPCWSTR lpApplicationName,
-                                   __in_z LPWSTR lpCommandLine,
+                                   _In_z_ LPWSTR lpCommandLine,
                                    LPSECURITY_ATTRIBUTES lpProcessAttributes,
                                    LPSECURITY_ATTRIBUTES lpThreadAttributes,
                                    BOOL bInheritHandles,
@@ -1736,7 +1718,7 @@ HRESULT Cordb::CreateProcessCommon(ICorDebugRemoteTarget * pRemoteTarget,
 
 HRESULT Cordb::CreateProcessEx(ICorDebugRemoteTarget * pRemoteTarget,
                                LPCWSTR lpApplicationName,
-                               __in_z LPWSTR lpCommandLine,
+                               _In_z_ LPWSTR lpCommandLine,
                                LPSECURITY_ATTRIBUTES lpProcessAttributes,
                                LPSECURITY_ATTRIBUTES lpThreadAttributes,
                                BOOL bInheritHandles,
