@@ -280,7 +280,7 @@ DebuggerJitInfo::DebuggerJitInfo(DebuggerMethodInfo *minfo, NativeCodeVersion na
     _ASSERTE(minfo);
     m_encVersion = minfo->GetCurrentEnCVersion();
     _ASSERTE(m_encVersion >= CorDB_DEFAULT_ENC_FUNCTION_VERSION);
-    LOG((LF_CORDB,LL_EVERYTHING, "DJI::DJI : created at 0x%x\n", this));
+    LOG((LF_CORDB,LL_EVERYTHING, "DJI::DJI: created at %p\n", this));
 
     // Debugger doesn't track LightWeight codegen methods.
     // We should never even be creating a DJI for one.
@@ -383,8 +383,8 @@ DebuggerJitInfo::NativeOffset DebuggerJitInfo::MapILOffsetToNative(DebuggerJitIn
     {
 #endif // FEATURE_EH_FUNCLETS
         PREFIX_ASSUME( map != NULL );
-        LOG((LF_CORDB, LL_INFO10000, "DJI::MILOTN: ilOff 0x%x to nat 0x%x exact:0x%x (Entry IL Off:0x%x)\n",
-             ilOffset.m_ilOffset, map->nativeStartOffset, resultOffset.m_fExact, map->ilOffset));
+        LOG((LF_CORDB, LL_INFO10000, "DJI::MILOTN: ilOffset 0x%zx to nat 0x%x exact:%s (Entry IL Off:0x%x)\n",
+             ilOffset.m_ilOffset, map->nativeStartOffset, (resultOffset.m_fExact ? "true" : "false"), map->ilOffset));
 
         resultOffset.m_nativeOffset = map->nativeStartOffset;
 
@@ -654,7 +654,7 @@ SIZE_T DebuggerJitInfo::MapILOffsetToNativeForSetIP(SIZE_T offsetILTo, int funcl
 // calls MapILOffsetToNative for the startOffset (putting the
 // result into start), and the endOffset (putting the result into end).
 // SIZE_T startOffset:  IL offset from beginning of function.
-// SIZE_T endOffset:  IL offset from beginngin of function,
+// SIZE_T endOffset:  IL offset from beginning of function,
 // or zero to indicate that the end of the function should be used.
 // DebuggerILToNativeMap **start:  Contains start & end
 // native offsets that correspond to startOffset.  Set to NULL if
@@ -892,17 +892,13 @@ void DebuggerJitInfo::LazyInitBounds()
         PRECONDITION(!g_pDebugger->HasDebuggerDataLock());
     } CONTRACTL_END;
 
-    LOG((LF_CORDB, LL_EVERYTHING, "DJI::LazyInitBounds: this=0x%x m_fAttemptInit %s\n", this, m_fAttemptInit == true ? "true": "false"));
-
     // Only attempt lazy-init once
     if (m_fAttemptInit)
-    {
         return;
-    }
 
     EX_TRY
     {
-        LOG((LF_CORDB, LL_EVERYTHING, "DJI::LazyInitBounds: this=0x%x Initing\n", this));
+        LOG((LF_CORDB, LL_EVERYTHING, "DJI::LazyInitBounds: this=0x%p Initing\n", this));
 
         // Should have already been jitted
         _ASSERTE(this->m_jitComplete);
@@ -927,7 +923,8 @@ void DebuggerJitInfo::LazyInitBounds()
             &cMap, &pMap,
             &cVars, &pVars);
 
-        LOG((LF_CORDB,LL_EVERYTHING, "DJI::LazyInitBounds: this=0x%x GetBoundariesAndVars success=0x%x\n", this, fSuccess));
+        LOG((LF_CORDB,LL_EVERYTHING, "DJI::LazyInitBounds: this=%p GetBoundariesAndVars success=%s\n",
+            this, (fSuccess ? "true" : "false")));
 
         // SetBoundaries uses the CodeVersionManager, need to take it now for lock ordering reasons
         CodeVersionManager::LockHolder codeVersioningLockHolder;
@@ -951,8 +948,8 @@ void DebuggerJitInfo::LazyInitBounds()
     }
     EX_CATCH
     {
-        LOG((LF_CORDB,LL_WARNING, "DJI::LazyInitBounds: this=0x%x Exception was thrown and caught\n", this));
-        // Just catch the exception. The DJI maps may or may-not be intialized,
+        LOG((LF_CORDB,LL_WARNING, "DJI::LazyInitBounds: this=%p Exception was thrown and caught\n", this));
+        // Just catch the exception. The DJI maps may or may-not be initialized,
         // but they should still be in a consistent state, so we should be ok.
     }
     EX_END_CATCH(SwallowAllExceptions)
@@ -1016,7 +1013,7 @@ void DebuggerJitInfo::SetBoundaries(ULONG32 cMap, ICorDebugInfo::OffsetMapping *
     }
     CONTRACTL_END;
 
-    LOG((LF_CORDB,LL_EVERYTHING, "DJI::SetBoundaries: this=0x%x cMap=0x%x pMap=0x%x\n", this, cMap, pMap));
+    LOG((LF_CORDB,LL_EVERYTHING, "DJI::sB: this=%p cMap=%u pMap=%p\n", this, cMap, pMap));
     _ASSERTE((cMap == 0) == (pMap == NULL));
     _ASSERTE(m_sequenceMap == NULL);
 
@@ -1043,7 +1040,7 @@ void DebuggerJitInfo::SetBoundaries(ULONG32 cMap, ICorDebugInfo::OffsetMapping *
     // like the DebuggerJitInfo's.</TODO>
     //
     m_sequenceMap = (DebuggerILToNativeMap *)new (interopsafe) DebuggerILToNativeMap[cMap];
-    LOG((LF_CORDB,LL_EVERYTHING, "DJI::SetBoundaries: this=0x%x m_sequenceMap=0x%x\n", this, m_sequenceMap));
+    LOG((LF_CORDB,LL_EVERYTHING, "DJI::sB: this=%p m_sequenceMap=%p\n", this, m_sequenceMap));
     _ASSERTE(m_sequenceMap != NULL); // we'll throw on null
 
     m_sequenceMapCount = cMap;
@@ -1186,42 +1183,46 @@ void DebuggerJitInfo::SetBoundaries(ULONG32 cMap, ICorDebugInfo::OffsetMapping *
     m_callsiteMap = m_sequenceMap + m_sequenceMapCount;
     m_callsiteMapCount -= m_sequenceMapCount;
 
-    LOG((LF_CORDB, LL_INFO100000, "DJI::SetBoundaries: this=0x%x boundary count is %d (%d callsites)\n",
+    LOG((LF_CORDB, LL_INFO100000, "DJI::sB: this=%p boundary count is %u (%u callsites)\n",
          this, m_sequenceMapCount, m_callsiteMapCount));
 
 #ifdef LOGGING
-    for (unsigned int count = 0; count < m_sequenceMapCount + m_callsiteMapCount; count++)
+    for (unsigned count = 0; count < m_sequenceMapCount + m_callsiteMapCount; count++)
     {
-        if( m_sequenceMap[count].ilOffset ==
-            (ULONG) ICorDebugInfo::PROLOG )
-            LOG((LF_CORDB, LL_INFO1000000,
-                 "D::sB: PROLOG               --> 0x%08x -- 0x%08x",
-                 m_sequenceMap[count].nativeStartOffset,
-                 m_sequenceMap[count].nativeEndOffset));
-        else if ( m_sequenceMap[count].ilOffset ==
-                  (ULONG) ICorDebugInfo::EPILOG )
-            LOG((LF_CORDB, LL_INFO1000000,
-                 "D::sB: EPILOG              --> 0x%08x -- 0x%08x",
-                 m_sequenceMap[count].nativeStartOffset,
-                 m_sequenceMap[count].nativeEndOffset));
-        else if ( m_sequenceMap[count].ilOffset ==
-                  (ULONG) ICorDebugInfo::NO_MAPPING )
-            LOG((LF_CORDB, LL_INFO1000000,
-                 "D::sB: NO MAP              --> 0x%08x -- 0x%08x",
-                 m_sequenceMap[count].nativeStartOffset,
-                 m_sequenceMap[count].nativeEndOffset));
-        else
-            LOG((LF_CORDB, LL_INFO1000000,
-                 "D::sB: 0x%04x (Real:0x%04x) --> 0x%08x -- 0x%08x",
-                 m_sequenceMap[count].ilOffset,
-                 m_methodInfo->TranslateToInstIL(&mapping,
-                                                 m_sequenceMap[count].ilOffset,
-                                                 bOriginalToInstrumented),
-                 m_sequenceMap[count].nativeStartOffset,
-                 m_sequenceMap[count].nativeEndOffset));
-
-        LOG((LF_CORDB, LL_INFO1000000, " Src:0x%x\n", m_sequenceMap[count].source));
+        const DebuggerILToNativeMap& entry = m_sequenceMap[count];
+        switch (entry.ilOffset)
+        {
+            case (ULONG) ICorDebugInfo::PROLOG:
+                LOG((LF_CORDB, LL_INFO1000000,
+                    "DJI::sB: PROLOG               --> 0x%08x -- 0x%08x",
+                    entry.nativeStartOffset,
+                    entry.nativeEndOffset));
+                break;
+            case (ULONG) ICorDebugInfo::EPILOG:
+                LOG((LF_CORDB, LL_INFO1000000,
+                    "DJI::sB: EPILOG               --> 0x%08x -- 0x%08x",
+                    entry.nativeStartOffset,
+                    entry.nativeEndOffset));
+                break;
+            case (ULONG) ICorDebugInfo::NO_MAPPING:
+                LOG((LF_CORDB, LL_INFO1000000,
+                    "DJI::sB: NO MAP               --> 0x%08x -- 0x%08x",
+                    entry.nativeStartOffset,
+                    entry.nativeEndOffset));
+                break;
+            default:
+                LOG((LF_CORDB, LL_INFO1000000,
+                    "DJI::sB: 0x%04x (Real:0x%04x) --> 0x%08x -- 0x%08x",
+                    entry.ilOffset,
+                    m_methodInfo->TranslateToInstIL(&mapping,
+                                                    entry.ilOffset,
+                                                    bOriginalToInstrumented),
+                    entry.nativeStartOffset,
+                    entry.nativeEndOffset));
+                break;
+        }
 
+        LOG((LF_CORDB, LL_INFO1000000, " Src:0x%x\n", entry.source));
     }
 #endif //LOGGING
 }
@@ -1230,7 +1231,7 @@ void DebuggerJitInfo::SetBoundaries(ULONG32 cMap, ICorDebugInfo::OffsetMapping *
 // Init a DJI after it's jitted.
 void DebuggerJitInfo::Init(TADDR newAddress)
 {
-    // Shouldn't initialize while holding the lock b/c intialzing may call functions that lock,
+    // Shouldn't initialize while holding the lock b/c initializing may call functions that lock,
     // and thus we'd have a locking violation.
     _ASSERTE(!g_pDebugger->HasDebuggerDataLock());
 
@@ -1246,17 +1247,18 @@ void DebuggerJitInfo::Init(TADDR newAddress)
     this->InitFuncletAddress();
 #endif // FEATURE_EH_FUNCLETS
 
-    LOG((LF_CORDB,LL_INFO10000,"De::JITCo:Got DJI 0x%p(V %d),"
-         "Hot section from 0x%p to 0x%p "
-         "Cold section from 0x%p to 0x%p "
-         "varCount=%d  seqCount=%d\n",
+    LOG((LF_CORDB,LL_INFO10000,"De::JITCo:Got DJI %p (encVersion: %zx),"
+         "Hot section from %p to %p "
+         "Cold section from %p to %p "
+         "Code from %p to %p "
+         "varCount=%u  seqCount=%u\n",
          this, this->m_encVersion,
          this->m_codeRegionInfo.getAddrOfHotCode(),
          this->m_codeRegionInfo.getAddrOfHotCode() + this->m_codeRegionInfo.getSizeOfHotCode(),
          this->m_codeRegionInfo.getAddrOfColdCode(),
          this->m_codeRegionInfo.getAddrOfColdCode() + this->m_codeRegionInfo.getSizeOfColdCode(),
-         (ULONG)this->m_addrOfCode,
-         (ULONG)this->m_addrOfCode+(ULONG)this->m_sizeOfCode,
+         this->m_addrOfCode,
+         this->m_addrOfCode+(ULONG)this->m_sizeOfCode,
          this->GetVarNativeInfoCount(),
          this->GetSequenceMapCount()));
 
@@ -1421,7 +1423,7 @@ DebuggerMethodInfo::DebuggerMethodInfo(Module *module, mdMethodDef token) :
     }
     CONTRACTL_END;
 
-    LOG((LF_CORDB,LL_EVERYTHING, "DMI::DMI : created at 0x%p\n", this));
+    LOG((LF_CORDB,LL_EVERYTHING, "DMI::DMI: created at %p\n", this));
 
     _ASSERTE(g_pDebugger->HasDebuggerDataLock());
 
@@ -1518,6 +1520,9 @@ DebuggerJitInfo * DebuggerMethodInfo::FindJitInfo(MethodDesc * pMD,
     }
     CONTRACTL_END;
 
+#ifdef TARGET_ARM
+    addrNativeStartAddr = addrNativeStartAddr|THUMB_CODE;
+#endif
 
     DebuggerJitInfo * pCheck = m_latestJitInfo;
     while (pCheck != NULL)
@@ -1560,9 +1565,7 @@ DebuggerJitInfo *DebuggerMethodInfo::FindOrCreateInitAndAddJitInfo(MethodDesc* f
         GC_NOTRIGGER;
     }
     CONTRACTL_END;
-
     _ASSERTE(fd != NULL);
-
     // The debugger doesn't track Lightweight-codegen methods b/c they have no metadata.
     if (fd->IsDynamicMethod())
     {
@@ -1571,10 +1574,10 @@ DebuggerJitInfo *DebuggerMethodInfo::FindOrCreateInitAndAddJitInfo(MethodDesc* f
 
     if (startAddr == NULL)
     {
-        // This will grab the start address for the current code version.
         startAddr = g_pEEInterface->GetFunctionAddress(fd);
         if (startAddr == NULL)
         {
+            //The only case this should happen is if we are trying to get the DJI for a method that has not been jitted yet.
             return NULL;
         }
     }
@@ -1691,7 +1694,7 @@ DebuggerJitInfo *DebuggerMethodInfo::CreateInitAndAddJitInfo(NativeCodeVersion n
         // We know it's not in the table. Go add it!
         DebuggerJitInfo *djiPrev = m_latestJitInfo;
 
-        LOG((LF_CORDB,LL_INFO10000,"DMI:CAAJI: current head of dji list:0x%08x\n", djiPrev));
+        LOG((LF_CORDB,LL_INFO10000,"DMI:CAAJI: current head of dji list:0x%p\n", djiPrev));
 
         if (djiPrev != NULL)
         {
@@ -1714,7 +1717,7 @@ DebuggerJitInfo *DebuggerMethodInfo::CreateInitAndAddJitInfo(NativeCodeVersion n
     // We've now added a new DJI into the table and released the lock. Thus any other thread
     // can come and use our DJI. Good thing we inited the DJI _before_ adding it to the table.
 
-    LOG((LF_CORDB,LL_INFO10000,"DMI:CAAJI: new head of dji list:0x%08x\n", m_latestJitInfo));
+    LOG((LF_CORDB,LL_INFO10000,"DMI:CAAJI: new head of dji list: %p\n", m_latestJitInfo));
 
     return dji;
 }
@@ -1980,7 +1983,7 @@ void DebuggerMethodInfo::IterateAllDJIs(AppDomain * pAppDomain, Module * pLoader
     _ASSERTE(pEnum != NULL);
     _ASSERTE(pAppDomain != NULL || pMethodDescFilter != NULL);
 
-    // Esnure we have DJIs for everything.
+    // Ensure we have DJIs for everything.
     CreateDJIsForNativeBlobs(pAppDomain, pLoaderModuleFilter, pMethodDescFilter);
 
     pEnum->m_pCurrent = m_latestJitInfo;
@@ -2073,6 +2076,7 @@ void DebuggerMethodInfo::CreateDJIsForMethodDesc(MethodDesc * pMethodDesc)
     }
     CONTRACTL_END;
 
+   LOG((LF_CORDB, LL_INFO10000, "DMI::CDJIFMD pMethodDesc:%p\n", pMethodDesc));
 
     // The debugger doesn't track Lightweight-codegen methods b/c they have no metadata.
     if (pMethodDesc->IsDynamicMethod())
@@ -2087,19 +2091,25 @@ void DebuggerMethodInfo::CreateDJIsForMethodDesc(MethodDesc * pMethodDesc)
         CodeVersionManager::LockHolder codeVersioningLockHolder;
         NativeCodeVersionCollection nativeCodeVersions = pCodeVersionManager->GetNativeCodeVersions(pMethodDesc);
 
+#ifdef LOGGING
+        int count = 0;
+#endif // LOGGING
         for (NativeCodeVersionIterator itr = nativeCodeVersions.Begin(), end = nativeCodeVersions.End(); itr != end; itr++)
         {
             // Some versions may not be compiled yet - skip those for now
             // if they compile later the JitCompiled callback will add a DJI to our cache at that time
             PCODE codeAddr = itr->GetNativeCode();
+            LOG((LF_CORDB, LL_INFO10000, "DMI::CDJIFMD (%d) Native code for DJI - %p\n", ++count, codeAddr));
             if (codeAddr)
             {
                 // The DJI may already be populated in the cache, if so CreateInitAndAdd is
                 // a no-op and that is fine.
                 BOOL unusedDjiWasCreated;
                 CreateInitAndAddJitInfo(*itr, codeAddr, &unusedDjiWasCreated);
+                LOG((LF_CORDB, LL_INFO10000, "DMI::CDJIFMD Was DJI created? %s\n", (unusedDjiWasCreated ? "true" : "false")));
             }
         }
+        LOG((LF_CORDB, LL_INFO10000, "DMI::CDJIFMD NativeCodeVersion total %d for md=%p\n", count, pMethodDesc));
     }
 #else
     // We just ask for the DJI to ensure that it's lazily created.
@@ -2169,8 +2179,8 @@ HRESULT DebuggerMethodInfoTable::AddMethodInfo(Module *pModule,
     }
     CONTRACTL_END;
 
-   LOG((LF_CORDB, LL_INFO1000, "DMIT::AMI Adding dmi:0x%x Mod:0x%x tok:"
-        "0x%x nVer:0x%x\n", mi, pModule, token, mi->GetCurrentEnCVersion()));
+   LOG((LF_CORDB, LL_INFO1000, "DMIT::AMI: Adding dmi:%p Mod:%p tok:0x%08x nVer:0x%zx\n",
+        mi, pModule, token, mi->GetCurrentEnCVersion()));
 
    _ASSERTE(mi != NULL);
 
@@ -2187,18 +2197,18 @@ HRESULT DebuggerMethodInfoTable::AddMethodInfo(Module *pModule,
     DebuggerMethodInfoEntry *dmie =
         (DebuggerMethodInfoEntry *) Add(HASH(&dmik));
 
-    if (dmie != NULL)
+    if (dmie == NULL)
     {
-        dmie->key.pModule = pModule;
-        dmie->key.token = token;
-        dmie->mi = mi;
-
-        LOG((LF_CORDB, LL_INFO1000, "DMIT::AJI: mod:0x%x tok:0%x ",
-            pModule, token));
-        return S_OK;
+        ThrowOutOfMemory();
+        return E_OUTOFMEMORY;
     }
 
-    ThrowOutOfMemory();
+    dmie->key.pModule = pModule;
+    dmie->key.token = token;
+    dmie->mi = mi;
+
+    LOG((LF_CORDB, LL_INFO1000, "DMIT::AMI: mod:%p tok:0x%08x\n",
+        pModule, token));
     return S_OK;
 }
 
@@ -2217,8 +2227,8 @@ HRESULT DebuggerMethodInfoTable::OverwriteMethodInfo(Module *pModule,
     }
     CONTRACTL_END;
 
-    LOG((LF_CORDB, LL_INFO1000, "DMIT::OJI: dmi:0x%x mod:0x%x tok:0x%x\n", mi,
-        pModule, token));
+    LOG((LF_CORDB, LL_INFO1000, "DMIT::OMI: dmi:%p mod:%p tok:0x%08x\n",
+        mi, pModule, token));
 
     _ASSERTE(g_pDebugger->HasDebuggerDataLock());
 
@@ -2230,14 +2240,13 @@ HRESULT DebuggerMethodInfoTable::OverwriteMethodInfo(Module *pModule,
       = (DebuggerMethodInfoEntry *) Find(HASH(&dmik), KEY(&dmik));
     if (entry != NULL)
     {
-        if ( (fOnlyIfNull &&
-              entry->mi == NULL) ||
-             !fOnlyIfNull)
+        if ( (fOnlyIfNull && entry->mi == NULL)
+            || !fOnlyIfNull)
         {
             entry->mi = mi;
 
-            LOG((LF_CORDB, LL_INFO1000, "DMIT::OJI: mod:0x%x tok:0x%x remap"
-                "nVer:0x%x\n", pModule, token, entry->nVersionLastRemapped));
+            LOG((LF_CORDB, LL_INFO1000, "DMIT::OMI: mod:%p tok:0x%08x remap nVer:0x%zx\n",
+                pModule, token, entry->nVersionLastRemapped));
             return S_OK;
         }
     }
@@ -2256,7 +2265,7 @@ void DebuggerMethodInfoTable::ClearMethodsOfModule(Module *pModule)
 
     _ASSERTE(g_pDebugger->HasDebuggerDataLock());
 
-    LOG((LF_CORDB, LL_INFO1000000, "CMOM:mod:0x%x (%S)\n", pModule
+    LOG((LF_CORDB, LL_INFO1000000, "CMOM:mod:0x%x (%s)\n", pModule
         ,pModule->GetDebugName()));
 
     HASHFIND info;
@@ -2326,43 +2335,6 @@ void DebuggerMethodInfoTable::DeleteEntryDMI(DebuggerMethodInfoEntry *entry)
 
 #endif // #ifndef DACCESS_COMPILE
 
-DebuggerJitInfo *DebuggerJitInfo::GetJitInfoByAddress(const BYTE *pbAddr )
-{
-    CONTRACTL
-    {
-        NOTHROW;
-        GC_NOTRIGGER;
-    }
-    CONTRACTL_END;
-
-    DebuggerJitInfo *dji = this;
-
-#ifdef LOGGING
-    LOG((LF_CORDB,LL_INFO10000,"DJI:GJIBA finding DJI "
-            "corresponding to addr 0x%p, starting with 0x%p\n", pbAddr, dji));
-#endif //LOGGING
-
-    // If it's not NULL, but not in the range m_addrOfCode to end of function,
-    //  then get the previous one.
-    while( dji != NULL &&
-           !CodeRegionInfo::GetCodeRegionInfo(dji).IsMethodAddress(pbAddr))
-    {
-        LOG((LF_CORDB,LL_INFO10000,"DJI:GJIBA: pbAddr 0x%p is not in code "
-            "0x%p (size:0x%p)\n", pbAddr, dji->m_addrOfCode,
-            dji->m_sizeOfCode));
-        dji = dji->m_prevJitInfo;
-    }
-
-#ifdef LOGGING
-    if (dji == NULL)
-    {
-        LOG((LF_CORDB,LL_INFO10000,"DJI:GJIBA couldn't find a DJI "
-            "corresponding to addr 0x%p\n", pbAddr));
-    }
-#endif //LOGGING
-    return dji;
-}
-
 PTR_DebuggerJitInfo DebuggerMethodInfo::GetLatestJitInfo(MethodDesc *mdesc)
 {
     // dac checks ngen'ed image content first, so
@@ -2407,7 +2379,7 @@ DebuggerMethodInfo *DebuggerMethodInfoTable::GetMethodInfo(Module *pModule, mdMe
     }
     else
     {
-        LOG((LF_CORDB, LL_INFO1000, "DMI::GMI: for methodDef 0x%x, got 0x%x prev:0x%x\n",
+        LOG((LF_CORDB, LL_INFO1000, "DMI::GMI: for methodDef 0x%x, DMI=%p prev DMI=%p\n",
             token, entry->mi, (entry->mi?entry->mi->m_prevMethodInfo:0)));
         return entry->mi;
     }
@@ -2483,9 +2455,7 @@ DebuggerMethodInfoEntry::EnumMemoryRegions(CLRDataEnumMemoryFlags flags)
 
     // For a MiniDumpNormal, what is needed for modules is already enumerated elsewhere.
     // Don't waste time doing it here an extra time. Also, this will add many MB extra into the dump.
-    if ((key.pModule.IsValid()) &&
-        CLRDATA_ENUM_MEM_MINI != flags
-        && CLRDATA_ENUM_MEM_TRIAGE != flags)
+    if ((key.pModule.IsValid()) && CLRDATA_ENUM_MEM_MINI != flags && CLRDATA_ENUM_MEM_TRIAGE != flags && CLRDATA_ENUM_MEM_HEAP2 != flags)
     {
         key.pModule->EnumMemoryRegions(flags, true);
     }
@@ -2503,7 +2473,7 @@ DebuggerMethodInfo::EnumMemoryRegions(CLRDataEnumMemoryFlags flags)
     DAC_ENUM_DTHIS();
     SUPPORTS_DAC;
 
-    if (flags != CLRDATA_ENUM_MEM_MINI && flags != CLRDATA_ENUM_MEM_TRIAGE)
+    if (flags != CLRDATA_ENUM_MEM_MINI && flags != CLRDATA_ENUM_MEM_TRIAGE && flags != CLRDATA_ENUM_MEM_HEAP2)
     {
         // Modules are enumerated already for minidumps, save the empty calls.
         if (m_module.IsValid())
@@ -2532,7 +2502,7 @@ DebuggerJitInfo::EnumMemoryRegions(CLRDataEnumMemoryFlags flags)
         m_methodInfo->EnumMemoryRegions(flags);
     }
 
-    if (flags != CLRDATA_ENUM_MEM_MINI && flags != CLRDATA_ENUM_MEM_TRIAGE)
+    if (flags != CLRDATA_ENUM_MEM_MINI && flags != CLRDATA_ENUM_MEM_TRIAGE && flags != CLRDATA_ENUM_MEM_HEAP2)
     {
         if (m_nativeCodeVersion.GetMethodDesc().IsValid())
         {
