@@ -24,8 +24,6 @@
 // Resource strings for MDA descriptions.
 //-----------------------------------------------------------------------------
 
-#define MDARC_DEBUGGER_FIBER_MODE_NOT_SUPPORTED 0x1934
-
 #define IDS_RTL                                 0x01F5
 
 #define IDS_DS_ACTIVESESSIONS                   0x1701
@@ -48,7 +46,6 @@
 #define IDS_EE_LOAD_BAD_MAIN_SIG                0x1712
 #define IDS_EE_COM_UNSUPPORTED_TYPE             0x1713
 
-#define IDS_EE_NOTNDIRECT                       0x1719
 #define IDS_EE_RETHROW_NOT_ALLOWED              0x171d
 #define IDS_EE_INVALID_OLE_VARIANT              0x171e
 
@@ -126,7 +123,6 @@
 #define IDS_CLASSLOAD_BADFORMAT                 0x1774
 #define IDS_CLASSLOAD_BYREFARRAY                0x1775
 #define IDS_CLASSLOAD_BYREFLIKEARRAY            0x1776
-#define IDS_CLASSLOAD_MISSINGMETHOD             0x1777
 #define IDS_CLASSLOAD_STATICVIRTUAL             0x1778
 #define IDS_CLASSLOAD_REDUCEACCESS              0x1779
 #define IDS_CLASSLOAD_BADPINVOKE                0x177a
@@ -170,20 +166,20 @@
 #define IDS_CLASSLOAD_MI_BADRETURNTYPE          0x17a8
 #define IDS_CLASSLOAD_STATICVIRTUAL_NOTIMPL     0x17a9
 
+#define IDS_INVALID_RECURSIVE_GENERIC_FIELD_LOAD 0x17aa
 #define IDS_CLASSLOAD_TOOMANYGENERICARGS        0x17ab
-#define IDS_ERROR                               0x17b0
-#define IDS_DEBUG_SERVICE_CAPTION               0x17b4
+
+#define IDS_CLASSLOAD_INLINE_ARRAY_FIELD_COUNT  0x17ac
+#define IDS_CLASSLOAD_INLINE_ARRAY_LENGTH       0x17ad
+#define IDS_CLASSLOAD_INLINE_ARRAY_EXPLICIT     0x17ae
+
 #define IDS_DEBUG_USERBREAKPOINT                0x17b6
-#define IDS_DEBUG_UNHANDLEDEXCEPTION            0x17b7
-#define IDS_DEBUG_UNHANDLEDEXCEPTION_IPC        0x17b8
+
 #define IDS_PERFORMANCEMON_FUNCNOTFOUND         0x17bb
 #define IDS_PERFORMANCEMON_FUNCNOTFOUND_TITLE   0x17bc
 #define IDS_PERFORMANCEMON_PSAPINOTFOUND        0x17bd
 #define IDS_PERFORMANCEMON_PSAPINOTFOUND_TITLE  0x17be
 
-#define IDS_DEBUG_UNHANDLED_EXCEPTION_MSG       0x17c0
-#define IDS_DEBUG_USER_BREAKPOINT_MSG           0x17c1
-
 #define IDS_INVALID_REDIM                       0x17c3
 #define IDS_INVALID_PINVOKE_CALLCONV            0x17c4
 #define IDS_CLASSLOAD_NSTRUCT_EXPLICIT_OFFSET   0x17c7
@@ -201,11 +197,8 @@
 #define IDS_EE_INVOKE_NEW_ENUM_INVALID_RETURN   0x17d7
 #define IDS_EE_COM_OBJECT_RELEASE_RACE          0x17d8
 #define IDS_EE_COM_OBJECT_NO_LONGER_HAS_WRAPPER 0x17d9
-#define IDS_EE_CALLBACK_NOT_CALLED_FROM_CCTOR   0x17da
-#define IDS_EE_CALLBACK_ALREADY_REGISTERED      0x17de
 #define IDS_EE_NDIRECT_BADNATL_CALLCONV         0x17df
 #define IDS_EE_CANNOTCAST                       0x17e0
-#define IDS_EE_NOTISOMORPHIC                    0x17e1
 
 #define IDS_EE_NOCUSTOMMARSHALER                0x17e7
 #define IDS_EE_SIZECONTROLOUTOFRANGE            0x17e8
@@ -214,9 +207,6 @@
 #define IDS_EE_INVALID_VT_FOR_CUSTOM_MARHALER   0x17ec
 #define IDS_EE_BAD_COMEXTENDS_CLASS             0x17ed
 
-#define IDS_EE_ERRORTITLE                       0x17f0
-#define IDS_EE_ERRORMESSAGETEMPLATE             0x17f1
-
 #define IDS_EE_LOCAL_COGETCLASSOBJECT_FAILED    0x17f5
 
 #define IDS_EE_MISSING_FIELD                    0x17f7
@@ -230,27 +220,18 @@
 #define IDS_CLASSLOAD_EQUIVALENTSTRUCTMETHODS   0x17fe
 #define IDS_CLASSLOAD_EQUIVALENTSTRUCTFIELDS    0x17ff
 
-#define IDS_EE_NO_IDISPATCH                     0x1a02
-
-
 #define IDS_EE_SIGTOOCOMPLEX                    0x1a03
 #define IDS_EE_STRUCTTOOCOMPLEX                 0x1a04
 #define IDS_EE_STRUCTARRAYTOOLARGE              0x1a05
 #define IDS_EE_BADMARSHALFIELD_NOSTRINGBUILDER  0x1a06
-#define IDS_EE_NAME_UNKNOWN                     0x1a07
-#define IDS_EE_THREAD_NOTSTARTED                0x1a0a
+
 #define IDS_EE_NO_BACKING_CLASS_FACTORY         0x1a0b
-#define IDS_EE_NAME_UNKNOWN_UNQ                 0x1a0c
 #define IDS_EE_STRING_TOOLONG                   0x1a0d
 #define IDS_EE_VARARG_NOT_SUPPORTED             0x1a0f
 
 #define IDS_EE_INVALID_CA                       0x1a10
 
-#define IDS_EE_THREADSTART_STATE                0x1a12
-
 #define IDS_EE_THREAD_CANNOT_GET                0x1a15
-#define IDS_EE_THREAD_DEAD_PRIORITY             0x1a19
-#define IDS_EE_THREAD_DEAD_STATE                0x1a1a
 #define IDS_EE_THREAD_BAD_STATE                 0x1a1b
 #define IDS_EE_THREAD_ABORT_WHILE_SUSPEND       0x1a1c
 
@@ -294,8 +275,6 @@
 #define IDS_CLASSLOAD_TYPEWRONGNUMGENERICARGS   0x1a4b
 #define IDS_CLASSLOAD_NSTRUCT_NEGATIVE_OFFSET   0x1a4d
 
-#define IDS_EE_THREAD_PRIORITY_FAIL             0x1a4e
-
 #define IDS_CLASSLOAD_INVALIDINSTANTIATION      0x1a59
 
 #define IDS_EE_CLASSLOAD_INVALIDINSTANTIATION      0x1a59
@@ -305,6 +284,7 @@
 #define IDS_EE_BADMARSHAL_ABSTRACTOUTCRITICALHANDLE 0x1a63
 #define IDS_EE_BADMARSHAL_RETURNCHCOMTONATIVE       0x1a64
 #define IDS_EE_BADMARSHAL_CRITICALHANDLE            0x1a65
+#define IDS_EE_BADMARSHAL_INT128_RESTRICTION        0x1a66
 
 #define IDS_EE_BADMARSHAL_ABSTRACTRETCRITICALHANDLE 0x1a6a
 #define IDS_EE_CH_IN_VARIANT_NOT_SUPPORTED          0x1a6b
@@ -327,9 +307,7 @@
 
 #define IDS_CLASSLOAD_OVERLAPPING_INTERFACES 0x1a80
 #define IDS_CLASSLOAD_32BITCLRLOADING64BITASSEMBLY 0x1a81
-#define IDS_EE_ASSEMBLY_GETTYPE_CANNONT_HAVE_ASSEMBLY_SPEC     0x1a84
 
-#define IDS_EE_CANNOT_HAVE_ASSEMBLY_SPEC        0x1a86
 #define IDS_EE_NEEDS_ASSEMBLY_SPEC              0x1a87
 
 #define IDS_EE_FILELOAD_ERROR_GENERIC           0x1a88
@@ -376,12 +354,7 @@
 #define IDS_CLASSLOAD_EQUIVALENTBADTYPE         0x1ab7
 #define IDS_EE_CODEEXECUTION_CONTAINSGENERICVAR 0x1abb
 #define IDS_CLASSLOAD_WRONGCPU                  0x1abc
-#define IDS_EE_CREATEINSTANCEFROMAPP_FAILED     0x1abd
 
-#define IDS_EE_SIMD_NGEN_DISALLOWED             0x1ac3
-#define IDS_IBC_MISSING_EXTERNAL_TYPE           0x1ac5
-#define IDS_IBC_MISSING_EXTERNAL_METHOD         0x1ac6
-#define IDS_EE_HWINTRINSIC_NGEN_DISALLOWED      0x1ac7
 #define IDS_CLASSLOAD_MI_FINAL_IMPL             0x1ac8
 #define IDS_CLASSLOAD_AMBIGUOUS_OVERRIDE        0x1ac9
 #define IDS_CLASSLOAD_UNSUPPORTED_DISPATCH      0x1aca
@@ -450,16 +423,15 @@
 #define BFA_BAD_SIGNATURE                       0x2044
 #define BFA_TYPEREG_NAME_TOO_LONG               0x2045
 #define BFA_BAD_TYPEREF_TOKEN                   0x2046
-#define BFA_BAD_CLASS_INT_CA                    0x2047
 #define BFA_BAD_CLASS_INT_CA_FORMAT             0x2048
 #define BFA_BAD_COMPLUS_SIG                     0x2049
 #define BFA_BAD_ELEM_IN_SIZEOF                  0x204b
 #define BFA_IJW_IN_COLLECTIBLE_ALC              0x204c
+#define BFA_INVALID_UNSAFEACCESSOR              0x204d
 
 #define IDS_CLASSLOAD_INTERFACE_NO_ACCESS       0x204f
 
 #define BFA_BAD_CA_HEADER                       0x2050
-#define BFA_BAD_STRING_TOKEN                    0x2052
 #define BFA_BAD_STRING_TOKEN_RANGE              0x2053
 #define BFA_FIXUP_WRONG_PLATFORM                0x2054
 #define BFA_UNEXPECTED_GENERIC_TOKENTYPE        0x2055
@@ -469,7 +441,6 @@
 #define BFA_UNEXPECTED_ARRAY_TYPE               0x2059
 #define BFA_BAD_VISIBILITY                      0x205a
 #define BFA_FAMILY_ON_GLOBAL                    0x205b
-#define BFA_NOFIND_EXPORTED_TYPE                0x205c
 #define BFA_NOT_AN_ARRAY                        0x205d
 #define BFA_EXPECTED_METHODDEF_OR_MEMBERREF     0x205e
 
@@ -510,8 +481,8 @@
 #define IDS_ER_STACK_OVERFLOW                   0x208a
 #define IDS_ER_STACK                            0x208b
 #define IDS_ER_WORDAT                           0x208c
-#define IDS_ER_UNMANAGEDFAILFASTMSG             0x208d
-#define IDS_ER_UNHANDLEDEXCEPTIONINFO           0x208e
+
+
 #define IDS_ER_MESSAGE_TRUNCATE                 0x208f
 
 #define IDS_EE_OBJECT_TO_VARIANT_NOT_SUPPORTED  0x2090
@@ -520,8 +491,6 @@
 #define IDS_EE_BADMARSHALFIELD_DECIMAL          0x2099
 
 #define IDS_EE_CANNOTCASTSAME                   0x209a
-#define IDS_EE_CANNOTCAST_HELPER_BYTE           0x209b
-#define IDS_EE_CANNOTCAST_HELPER_PATH           0x209c
 
 // For ForwardInteropStubAttribute
 #ifdef FEATURE_COMINTEROP
@@ -532,8 +501,6 @@
 #define IDS_EE_INTEROP_STUB_CA_NO_ACCESS_TO_STUB_METHOD             0x2111
 #endif
 
-#define IDS_EE_INTEROP_CODE_SIZE_COMMENT        0x2112
-
 #define BFA_REFERENCE_ASSEMBLY                  0x2113
 
 #define IDS_E_FIELDACCESS                       0x2114
@@ -576,19 +543,16 @@
 
 #define IDS_HOST_ASSEMBLY_RESOLVER_ASSEMBLY_ALREADY_LOADED_IN_CONTEXT                  0x2636
 #define IDS_HOST_ASSEMBLY_RESOLVER_DYNAMICALLY_EMITTED_ASSEMBLIES_UNSUPPORTED          0x2637
-#define IDS_HOST_ASSEMBLY_RESOLVER_INCOMPATIBLE_BINDING_CONTEXT                        0x2638
 
 #define IDS_NATIVE_IMAGE_CANNOT_BE_LOADED_MULTIPLE_TIMES                               0x263a
 
-#define IDS_CLASSLOAD_BYREFLIKE_STATICFIELD        0x263b
-#define IDS_CLASSLOAD_BYREFLIKE_INSTANCEFIELD      0x263c
+#define IDS_CLASSLOAD_BYREF_OR_BYREFLIKE_STATICFIELD    0x263b
+#define IDS_CLASSLOAD_BYREF_OR_BYREFLIKE_INSTANCEFIELD  0x263c
 #define IDS_EE_NDIRECT_LOADLIB_LINUX               0x263e
 #define IDS_EE_NDIRECT_LOADLIB_MAC                 0x263f
 #define IDS_EE_NDIRECT_GETPROCADDRESS_UNIX         0x2640
 #define IDS_EE_ERROR_COM                           0x2641
 
-#define IDS_INVOKE_NULLREF_RETURNED             0x2642
-
 #define IDS_EE_CANNOT_SET_INITONLY_STATIC_FIELD    0x2643
 
 #define IDS_EE_NDIRECT_GETPROCADDR_WIN_DLL         0x2644
@@ -598,3 +562,8 @@
 #define IDS_EE_BADMARSHAL_DELEGATE_TLB_INTERFACE   0x2649
 #define IDS_EE_THREAD_APARTMENT_NOT_SUPPORTED      0x264A
 #define IDS_EE_NO_IINSPECTABLE                     0x264B
+#define IDS_EE_BADMARSHAL_MARSHAL_DISABLED         0x264C
+#define IDS_EE_NDIRECT_DISABLEDMARSHAL_SETLASTERROR 0x264D
+#define IDS_EE_NDIRECT_DISABLEDMARSHAL_LCID        0x264E
+#define IDS_EE_NDIRECT_DISABLEDMARSHAL_PRESERVESIG 0x264F
+#define IDS_EE_NDIRECT_DISABLEDMARSHAL_VARARGS     0x2650
