@@ -1,5 +1,6 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
+
 #ifndef __GCENV_H__
 #define __GCENV_H__
 
@@ -42,11 +43,11 @@
 
 #define STRESS_LOG
 #ifdef STRESS_LOG
-/*  STRESS_LOG_VA was added to allow sendign GC trace output to the stress log. msg must be enclosed
-      in ()'s and contain a format string followed by 0 - 4 arguments.  The arguments must be numbers or
-      string literals.  LogMsgOL is overloaded so that all of the possible sets of parameters are covered.
-      This was done becasue GC Trace uses dprintf which dosen't contain info on how many arguments are
-      getting passed in and using va_args would require parsing the format string during the GC
+/*  STRESS_LOG_VA was added to allow sending GC trace output to the stress log. msg must be enclosed
+    in ()'s and contain a format string followed by 0 to 12 arguments. The arguments must be numbers
+     or string literals. This was done because GC Trace uses dprintf which doesn't contain info on
+    how many arguments are getting passed in and using va_args would require parsing the format
+    string during the GC
 */
 #define STRESS_LOG_VA(level, msg) do {                                        \
             if (StressLog::LogOn(LF_GC, LL_ALWAYS))                           \
@@ -139,12 +140,12 @@ enum LogLevel
     LL_FATALERROR,
     LL_ERROR,
     LL_WARNING,
-    LL_INFO10,       // can be expected to generate 10 logs per small but not trival run
-    LL_INFO100,      // can be expected to generate 100 logs per small but not trival run
-    LL_INFO1000,     // can be expected to generate 1,000 logs per small but not trival run
-    LL_INFO10000,    // can be expected to generate 10,000 logs per small but not trival run
-    LL_INFO100000,   // can be expected to generate 100,000 logs per small but not trival run
-    LL_INFO1000000,  // can be expected to generate 1,000,000 logs per small but not trival run
+    LL_INFO10,       // can be expected to generate 10 logs per small but not trivial run
+    LL_INFO100,      // can be expected to generate 100 logs per small but not trivial run
+    LL_INFO1000,     // can be expected to generate 1,000 logs per small but not trivial run
+    LL_INFO10000,    // can be expected to generate 10,000 logs per small but not trivial run
+    LL_INFO100000,   // can be expected to generate 100,000 logs per small but not trivial run
+    LL_INFO1000000,  // can be expected to generate 1,000,000 logs per small but not trivial run
     LL_EVERYTHING,
 };
 
