@@ -417,7 +417,7 @@ public:
     // Remarks:
     //  If a process runs with a restricted memory limit, it returns the limit. If there's no limit
     //  specified, it returns amount of actual physical memory.
-    static uint64_t GetPhysicalMemoryLimit(bool* is_restricted=NULL);
+    static uint64_t GetPhysicalMemoryLimit(bool* is_restricted=NULL, bool refresh=false);
 
     // Get memory status
     // Parameters:
