@@ -7,18 +7,11 @@
 #define OEMRESOURCE
 #define INITGUID
 
-#include <windows.h>
 #include <cor.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include <winuser.h>
-#include <commctrl.h>
-#include <commdlg.h>
-#include <richedit.h>
-#include <shellapi.h>
-#include <htmlhelp.h>
 #include <conio.h>
 
 #ifndef Debug_ReportError
