@@ -63,24 +63,29 @@
 //                                              (Note that the Null- and Deleted-related functions below
 //                                              are not affected by this and must always be NOTHROW.)
 //
-// static element_t Null()                      Return the Null sentinal value.  May be inherited from
+// static element_t Null()                      Return the Null sentinel value.  May be inherited from
 //                                              default traits if it can be assigned from 0.
-// static element_t Deleted()                   Return the Deleted sentinal value.  May be inherited from the
+// static element_t Deleted()                   Return the Deleted sentinel value.  May be inherited from the
 //                                              default traits if it can be assigned from -1.
-// static const bool IsNull(const ELEMENT &e)   Compare element with Null sentinal value. May be inherited from
+// static const bool IsNull(const ELEMENT &e)   Compare element with Null sentinel value. May be inherited from
 //                                              default traits if it can be assigned from 0.
-// static const bool IsDeleted(const ELEMENT &e) Compare element with Deleted sentinal value. May be inherited from the
+// static const bool IsDeleted(const ELEMENT &e) Compare element with Deleted sentinel value. May be inherited from the
 //                                              default traits if it can be assigned from -1.
+// static bool ShouldDelete(const ELEMENT &e)   Called in addition to IsDeleted() when s_supports_autoremove is true, see more
+//                                              information there.
 //
 // static void OnDestructPerEntryCleanupAction(ELEMENT& e) Called on every element when in hashtable destructor.
 //                                              s_DestructPerEntryCleanupAction must be set to true if implemented.
+// static void OnRemovePerEntryCleanupAction(ELEMENT& e) Called when an element is removed from the hashtable, including when
+//                                              the hashtable is destructed. s_RemovePerEntryCleanupAction must be set to true
+//                                              if implemented.
 //
 // s_growth_factor_numerator
 // s_growth_factor_denominator                  Factor to grow allocation (numerator/denominator).
 //                                              Typically inherited from default traits (3/2)
 //
 // s_density_factor_numerator
-// s_density_factor_denominator                 Maxium occupied density of table before growth
+// s_density_factor_denominator                 Maximum occupied density of table before growth
 //                                              occurs (num/denom).  Typically inherited (3/4).
 //
 // s_minimum_allocation                         Minimum table allocation count (size on first growth.)  It is
@@ -92,7 +97,19 @@
 //                                              in that there may be more copies of the template instantiated through
 //                                              the system as different variants are used.
 //
+// s_supports_autoremove                        When autoremove is supported, ShouldDelete() is called in addition to
+//                                              IsDeleted() in any situation that involves walking the table's elements, to
+//                                              determine if an element should be deleted. It enables the hash table to
+//                                              self-clean elements whose underlying lifetime may be controlled externally. Note
+//                                              that since some lookup/iteration operations are const (can operate on a
+//                                              "const SHash"), when autoremove is supported, any such const operation may still
+//                                              modify the hash table. If this is set to true, s_supports_remove must also be
+//                                              true.
+//
 // s_DestructPerEntryCleanupAction              Set to true if OnDestructPerEntryCleanupAction has non-empty implementation.
+// s_RemovePerEntryCleanupAction                Set to true if OnRemovePerEntryCleanupAction has non-empty implementation.
+//                                              Only one of s_DestructPerEntryCleanupAction and s_RemovePerEntryCleanupAction
+//                                              may be set to true.
 //
 // DefaultHashTraits provides defaults for seldomly customized values in traits classes.
 
@@ -115,15 +132,20 @@ class DefaultSHashTraits
     static const COUNT_T s_minimum_allocation = 7;
 
     static const bool s_supports_remove = true;
+    static const bool s_supports_autoremove = false;
 
     static ELEMENT Null() { return (ELEMENT)(TADDR)0; }
     static ELEMENT Deleted() { return (ELEMENT)(TADDR)-1; }
     static bool IsNull(const ELEMENT &e) { return e == (ELEMENT)(TADDR)0; }
     static bool IsDeleted(const ELEMENT &e) { return e == (ELEMENT)(TADDR)-1; }
+    static bool ShouldDelete(const ELEMENT &e) { return false; }
 
     static inline void OnDestructPerEntryCleanupAction(const ELEMENT& e) { /* Do nothing */ }
     static const bool s_DestructPerEntryCleanupAction = false;
 
+    static void OnRemovePerEntryCleanupAction(const ELEMENT &e) {}
+    static const bool s_RemovePerEntryCleanupAction = false;
+
     static const bool s_NoThrow = true;
 
     // No defaults - must specify:
@@ -177,6 +199,10 @@ class EMPTY_BASES_DECL SHash : public TRAITS
 
     const element_t* LookupPtr(key_t key) const;
 
+    // Pointer-based flavor to replace an existing element (allows efficient access to tables of structures)
+
+    void ReplacePtr(const element_t *elementPtr, const element_t &newElement, bool invokeCleanupAction = true);
+
     // Add an element to the hash table.  This will never replace an element; multiple
     // elements may be stored with the same key.
 
@@ -264,11 +290,6 @@ class EMPTY_BASES_DECL SHash : public TRAITS
     // NoThrow version of CheckGrowth function. Returns FALSE on failure.
     BOOL CheckGrowthNoThrow();
 
-    // See if it is OK to grow the hash table by one element. If not, allocate new
-    // hash table and return it together with its size *pcNewSize (used by code:AddPhases).
-    // Returns NULL if there already is space for one element.
-    element_t * CheckGrowth_OnlyAllocateNewTable(count_t * pcNewSize);
-
     // Allocates new resized hash table for growth. Does not update the hash table on the object.
     // The new size is computed based on the current population, growth factor, and maximum density factor.
     element_t * Grow_OnlyAllocateNewTable(count_t * pcNewSize);
@@ -278,7 +299,7 @@ class EMPTY_BASES_DECL SHash : public TRAITS
 
     // Utility function to allocate new table (does not copy the values into it yet). Returns the size of new table in
     // *pcNewTableSize (finds next prime).
-    // Phase 1 of code:Reallocate - it is split to support code:AddPhases.
+    // Phase 1 of code:Reallocate
     element_t * AllocateNewTable(count_t requestedSize, count_t * pcNewTableSize);
 
     // NoThrow version of AllocateNewTable utility function. Returns NULL on failure.
@@ -287,11 +308,11 @@ class EMPTY_BASES_DECL SHash : public TRAITS
     // Utility function to replace old table with newly allocated table (as allocated by
     // code:AllocateNewTable). Copies all 'old' values into the new table first.
     // Returns the old table. Caller is expected to delete it (via code:DeleteOldTable).
-    // Phase 2 of code:Reallocate - it is split to support code:AddPhases.
+    // Phase 2 of code:Reallocate
     element_t * ReplaceTable(element_t * newTable, count_t newTableSize);
 
     // Utility function to delete old table (as returned by code:ReplaceTable).
-    // Phase 3 of code:Reallocate - it is split to support code:AddPhases.
+    // Phase 3 of code:Reallocate
     void DeleteOldTable(element_t * oldTable);
 
     // Utility function that does not call code:CheckGrowth.
@@ -303,7 +324,7 @@ class EMPTY_BASES_DECL SHash : public TRAITS
     // it is perfectly fine for the element to be a duplicate - if so it
     // is added an additional time. Returns TRUE if a new empty spot was used;
     // FALSE if an existing deleted slot.
-    static BOOL Add(element_t *table, count_t tableSize, const element_t &element);
+    BOOL Add(element_t *table, count_t tableSize, const element_t &element);
 
     // Utility function to add a new element to the hash table, if no element with the same key
     // is already there. Otherwise, it will replace the existing element. This has the effect of
@@ -313,7 +334,7 @@ class EMPTY_BASES_DECL SHash : public TRAITS
     // Utility function to find the first element with the given key in
     // the hash table.
 
-    static const element_t* Lookup(PTR_element_t table, count_t tableSize, key_t key);
+    const element_t* Lookup(PTR_element_t table, count_t tableSize, key_t key) const;
 
     // Utility function to remove the first element with the given key
     // in the hash table.
@@ -339,13 +360,17 @@ class EMPTY_BASES_DECL SHash : public TRAITS
         // Some compilers won't compile the separate implementation in shash.inl
       protected:
 
+        SHash *m_hash;
         PTR_element_t m_table;
         count_t m_tableSize;
         count_t m_index;
 
-
+        // Iteration may modify the hash table if s_supports_autoremove is true. Since it typically does not modify the hash
+        // table, it should be possible to iterate over a "const SHash". So this takes a "const SHash *" and casts away the
+        // const to support autoremove.
         Index(const SHash *hash, BOOL begin)
-        : m_table(hash->m_table),
+        : m_hash(const_cast<SHash *>(hash)),
+            m_table(hash->m_table),
             m_tableSize(hash->m_tableSize),
             m_index(begin ? 0 : m_tableSize)
         {
@@ -363,9 +388,24 @@ class EMPTY_BASES_DECL SHash : public TRAITS
         {
             LIMITED_METHOD_CONTRACT;
 
-            if (m_index < m_tableSize)
-                if (TRAITS::IsNull(m_table[m_index]) || TRAITS::IsDeleted(m_table[m_index]))
-                    Next();
+            if (m_index >= m_tableSize)
+            {
+                return;
+            }
+
+            if (!TRAITS::IsNull(m_table[m_index]) && !TRAITS::IsDeleted(m_table[m_index]))
+            {
+                if (TRAITS::s_supports_autoremove && TRAITS::ShouldDelete(m_table[m_index]))
+                {
+                    m_hash->RemoveElement(m_table, m_tableSize, &m_table[m_index]);
+                }
+                else
+                {
+                    return;
+                }
+            }
+
+            Next();
         }
 
         void Next()
@@ -381,7 +421,16 @@ class EMPTY_BASES_DECL SHash : public TRAITS
                 if (m_index >= m_tableSize)
                     break;
                 if (!TRAITS::IsNull(m_table[m_index]) && !TRAITS::IsDeleted(m_table[m_index]))
-                    break;
+                {
+                    if (TRAITS::s_supports_autoremove && TRAITS::ShouldDelete(m_table[m_index]))
+                    {
+                        m_hash->RemoveElement(m_table, m_tableSize, &m_table[m_index]);
+                    }
+                    else
+                    {
+                        break;
+                    }
+                }
             }
         }
 
@@ -447,11 +496,26 @@ class EMPTY_BASES_DECL SHash : public TRAITS
                 m_increment = (hash % (this->m_tableSize-1)) + 1;
 
                 // Find first valid element
+
                 if (TRAITS::IsNull(this->m_table[this->m_index]))
+                {
                     this->m_index = this->m_tableSize;
-                else if (TRAITS::IsDeleted(this->m_table[this->m_index])
-                        || !TRAITS::Equals(m_key, TRAITS::GetKey(this->m_table[this->m_index])))
-                    Next();
+                    return;
+                }
+
+                if (!TRAITS::IsDeleted(this->m_table[this->m_index]))
+                {
+                    if (TRAITS::s_supports_autoremove && TRAITS::ShouldDelete(this->m_table[this->m_index]))
+                    {
+                        this->m_hash->RemoveElement(this->m_table, this->m_tableSize, &this->m_table[this->m_index]);
+                    }
+                    else if (TRAITS::Equals(m_key, TRAITS::GetKey(this->m_table[this->m_index])))
+                    {
+                        return;
+                    }
+                }
+
+                Next();
             }
         }
 
@@ -471,8 +535,18 @@ class EMPTY_BASES_DECL SHash : public TRAITS
                     break;
                 }
 
-                if (!TRAITS::IsDeleted(this->m_table[this->m_index])
-                        && TRAITS::Equals(m_key, TRAITS::GetKey(this->m_table[this->m_index])))
+                if (TRAITS::IsDeleted(this->m_table[this->m_index]))
+                {
+                    continue;
+                }
+
+                if (TRAITS::s_supports_autoremove && TRAITS::ShouldDelete(this->m_table[this->m_index]))
+                {
+                    this->m_hash->RemoveElement(this->m_table, this->m_tableSize, &this->m_table[this->m_index]);
+                    continue;
+                }
+
+                if (TRAITS::Equals(m_key, TRAITS::GetKey(this->m_table[this->m_index])))
                 {
                     break;
                 }
@@ -502,92 +576,6 @@ class EMPTY_BASES_DECL SHash : public TRAITS
         }
     };
 
-    // Wrapper and holder for adding an element to the hash table. Useful for Add operations that have to happen
-    // under a rare lock that does not allow call out into host.
-    // There are 3 phases:
-    //    1. code:PreallocateForAdd ... Can allocate memory (calls into host).
-    //    2. code:Add ... Adds one element (does NOT call into host).
-    //       or code:AddNothing_PublishPreallocatedTable ... Publishes the pre-allocated memory from step #1 (if any).
-    //    3. code:DeleteOldTable (or destructor) ... Can delete the old memory (calls into host).
-    // Example:
-    //      CrstHolder lockAdd(&crstLockForAdd); // Serialize all Add operations.
-    //      HostAssemblyMap::AddPhases addCall;
-    //      addCall.PreallocateForAdd(&shash); // 1. Allocates memory for one Add call (if needed). addCall serves as holder for the allocated memory.
-    //      {
-    //          // We cannot call out into host from ForbidSuspend region (i.e. no allocations/deallocations).
-    //          ForbidSuspendThreadHolder suspend; // Required by the 'special' read-lock
-    //          {
-    //              CrstHolder lock(&crstLock);
-    //              if (some_condition)
-    //              {   // 2a. Add item. This may replace SHash inner table with the one pre-allocated in step 1.
-    //                  addCall.Add(shashItem);
-    //              }
-    //              else
-    //              {   // 2b. Skip adding item. This may replace SHash inner table with the one pre-allocated in step 1.
-    //                  addCall.AddNothing_PublishPreallocatedTable();
-    //              }
-    //          }
-    //      }
-    //      addCall.DeleteOldTable(); // 3. Cleanup old table memory from shash (if it was replaced by pre-allocated table in step 2).
-    //                                // Note: addCall destructor would take care of deleting the memory as well.
-    class AddPhases
-    {
-    public:
-        AddPhases();
-        ~AddPhases();
-
-        // Prepares object for one call to code:Add. Pre-allocates new table memory if needed, does not publish
-        // the table yet (it is kept ready only in this holder for call to code:Add).
-        // Calls out into host.
-        void PreallocateForAdd(SHash * pHash);
-
-        // Add an element to the hash table. This will never replace an element; multiple elements may be stored
-        // with the same key.
-        // Will use/publish pre-allocated memory from code:PreallocateForAdd.
-        // Does not call out into host.
-        // Only one Add* method can be called once per object! (Create a new object for each call)
-        void Add(const element_t & element);
-
-        // Element will not be added to the hash table.
-        // Will use/publish pre-allocated memory from code:PreallocateForAdd.
-        // Does not call out into host.
-        // Only one Add* method can be called once per object! (Create a new object for each call)
-        void AddNothing_PublishPreallocatedTable();
-
-        // Deletes old table if it was replaced by call to code:Add or code:AddNothing_PublishPreallocatedTable.
-        // Calls out into host.
-        void DeleteOldTable();
-
-    private:
-        SHash *     m_pHash;
-        element_t * m_newTable;
-        count_t     m_newTableSize;
-        element_t * m_oldTable;
-
-    #ifdef _DEBUG
-        PTR_element_t dbg_m_table;
-        count_t       dbg_m_tableSize;
-        count_t       dbg_m_tableCount;
-        count_t       dbg_m_tableOccupied;
-        count_t       dbg_m_tableMax;
-        BOOL          dbg_m_fAddCalled;
-    #endif //_DEBUG
-    };  // class SHash::AddPhases
-
-    // Adds an entry to the hash table according to the guidelines above for
-    // avoiding a callout to the host while the read lock is held.
-    // Returns true if elem was added to the map, otherwise false.
-    // When elem was not added (false is returned), and if ppStoredElem is non-null,
-    // then it is set to point to the value in the map.
-    template <typename LockHolderT,
-              typename AddLockHolderT,
-              typename LockT,
-              typename AddLockT>
-    bool CheckAddInPhases(
-        element_t const & elem,
-        LockT & lock,
-        AddLockT & addLock,
-        IUnknown * addRefObject = nullptr);
 
   private:
 
@@ -704,7 +692,7 @@ private:
 
     static size_t _strcmp(WCHAR const *left, WCHAR const *right)
     {
-        return ::wcscmp(left, right);
+        return ::u16_strcmp(left, right);
     }
 
     static size_t _hash(CHAR const *str)
