@@ -25,7 +25,7 @@
 //                                              Typically inherited from default traits (3/2)
 //
 // s_density_factor_numerator
-// s_density_factor_denominator                 Maxium occupied density of table before growth
+// s_density_factor_denominator                 Maximum occupied density of table before growth
 //                                              occurs (num/denom).  Typically inherited (3/4).
 //
 // s_minimum_allocation                         Minimum table allocation count (size on first growth.)  It is
@@ -94,7 +94,21 @@ public:
 extern const JitPrimeInfo jitPrimeInfo[27];
 
 // Hash table class definition
-
+//
+// Several iterators are defined that work with range-based `for`:
+//   KeyIteration: yields just the hash table keys
+//   ValueIteration: yields just the hash table values
+//   KeyValueIteration: yields just the hash table <key,value> pairs
+//
+// For example:
+//
+//    for (const unsigned int lclNum : LclVarRefCounts::KeyIteration(&defsInBlock))
+//
+//    for (ValueNumToAssertsMap::Node* const iter : ValueNumToAssertsMap::KeyValueIteration(optValueNumToAsserts))
+//    {
+//        // use iter->GetKey(), iter->GetValue()
+//    }
+//
 template <typename Key,
           typename KeyFuncs,
           typename Value,
@@ -103,7 +117,44 @@ template <typename Key,
 class JitHashTable
 {
 public:
-    class KeyIterator;
+    // The node type.
+    // The only reason this class is public is to support the NodeIterator and KeyValueIteration. Only GetKey()
+    // and GetValue() need to be public methods.
+    class Node
+    {
+        friend class JitHashTable;
+
+        Node* m_next; // Assume that the alignment requirement of Key and Value are no greater than Node*,
+                      // so put m_next first to avoid unnecessary padding.
+        Key   m_key;
+        Value m_val;
+
+        template <class... Args>
+        Node(Node* next, Key k, Args&&... args) : m_next(next), m_key(k), m_val(std::forward<Args>(args)...)
+        {
+        }
+
+        void* operator new(size_t sz, Allocator alloc)
+        {
+            return alloc.template allocate<unsigned char>(sz);
+        }
+
+        void operator delete(void* p, Allocator alloc)
+        {
+            alloc.deallocate(p);
+        }
+
+    public:
+        Key GetKey() const
+        {
+            return m_key;
+        }
+
+        Value GetValue() const
+        {
+            return m_val;
+        }
+    };
 
     //------------------------------------------------------------------------
     // JitHashTable: Construct an empty JitHashTable object.
@@ -344,22 +395,205 @@ public:
         m_tableSizeInfo = JitPrimeInfo();
         m_tableCount    = 0;
         m_tableMax      = 0;
-
-        return;
     }
 
-    // Get an iterator to the first key in the table.
-    KeyIterator Begin() const
+    //
+    // Iteration support
+    //
+
+    class NodeIterator
     {
-        KeyIterator i(this, true);
-        return i;
-    }
+    protected:
+        Node**   m_table;
+        Node*    m_node;
+        unsigned m_tableSize;
+        unsigned m_index;
 
-    // Get an iterator following the last key in the table.
-    KeyIterator End() const
+        //------------------------------------------------------------------------
+        // NodeIterator: Construct an iterator for the specified JitHashTable.
+        //
+        // Arguments:
+        //    hash  - the hashtable
+        //    begin - `true` to construct an "begin" iterator,
+        //            `false` to construct an "end" iterator
+        //
+        NodeIterator(const JitHashTable* hash, bool begin)
+            : m_table(hash->m_table)
+            , m_node(nullptr)
+            , m_tableSize(hash->m_tableSizeInfo.prime)
+            , m_index(begin ? 0 : m_tableSize)
+        {
+            if (begin && (hash->m_tableCount > 0))
+            {
+                assert(m_table != nullptr);
+                while ((m_index < m_tableSize) && (m_table[m_index] == nullptr))
+                {
+                    m_index++;
+                }
+
+                if (m_index < m_tableSize)
+                {
+                    m_node = m_table[m_index];
+                    assert(m_node != nullptr);
+                }
+            }
+        }
+
+        //------------------------------------------------------------------------
+        // Next: Advance the iterator to the next node.
+        //
+        // Notes:
+        //    Advancing the end iterator has no effect.
+        //
+        void Next()
+        {
+            if (m_node != nullptr)
+            {
+                m_node = m_node->m_next;
+                if (m_node != nullptr)
+                {
+                    return;
+                }
+
+                // Otherwise...
+                m_index++;
+            }
+            while ((m_index < m_tableSize) && (m_table[m_index] == nullptr))
+            {
+                m_index++;
+            }
+
+            if (m_index < m_tableSize)
+            {
+                m_node = m_table[m_index];
+                assert(m_node != nullptr);
+            }
+            else
+            {
+                m_node = nullptr;
+            }
+        }
+
+    public:
+        // Advance the iterator to the next node
+        NodeIterator& operator++()
+        {
+            Next();
+            return *this;
+        }
+
+        bool operator!=(const NodeIterator& i) const
+        {
+            return i.m_node != m_node;
+        }
+    };
+
+    // KeyIterator: an iterator which yields only the hash table keys.
+    class KeyIterator : public NodeIterator
     {
-        return KeyIterator(this, false);
-    }
+    public:
+        KeyIterator(const JitHashTable* hash, bool begin) : NodeIterator(hash, begin)
+        {
+        }
+
+        Key operator*() const
+        {
+            return this->m_node->GetKey();
+        }
+    };
+
+    // ValueIterator: an iterator which yields only the hash table values.
+    class ValueIterator : public NodeIterator
+    {
+    public:
+        ValueIterator(const JitHashTable* hash, bool begin) : NodeIterator(hash, begin)
+        {
+        }
+
+        Value operator*() const
+        {
+            return this->m_node->GetValue();
+        }
+    };
+
+    // KeyValueIterator: an iterator which yields the hash table <key,value> pairs. It exposes a bit of the
+    // hash table implementation by returning a `Node*` that contains the <key,value> data.
+    class KeyValueIterator : public NodeIterator
+    {
+    public:
+        KeyValueIterator(const JitHashTable* hash, bool begin) : NodeIterator(hash, begin)
+        {
+        }
+
+        // We could return a new <key,value> struct, but why bother copying data?
+        Node* operator*() const
+        {
+            return this->m_node;
+        }
+    };
+
+    // KeyIteration: an adaptor to use for range-based `for` iteration over the hash table keys.
+    class KeyIteration
+    {
+        const JitHashTable* const m_hash;
+
+    public:
+        KeyIteration(const JitHashTable* hash) : m_hash(hash)
+        {
+        }
+
+        KeyIterator begin() const
+        {
+            return KeyIterator(m_hash, true);
+        }
+
+        KeyIterator end() const
+        {
+            return KeyIterator(m_hash, false);
+        }
+    };
+
+    // ValueIteration: an adaptor to use for range-based `for` iteration over the hash table values.
+    class ValueIteration
+    {
+        const JitHashTable* const m_hash;
+
+    public:
+        ValueIteration(const JitHashTable* hash) : m_hash(hash)
+        {
+        }
+
+        ValueIterator begin() const
+        {
+            return ValueIterator(m_hash, true);
+        }
+
+        ValueIterator end() const
+        {
+            return ValueIterator(m_hash, false);
+        }
+    };
+
+    // KeyValueIteration: an adaptor to use for range-based `for` iteration over the hash table <key,value> pairs.
+    class KeyValueIteration
+    {
+        const JitHashTable* const m_hash;
+
+    public:
+        KeyValueIteration(const JitHashTable* hash) : m_hash(hash)
+        {
+        }
+
+        KeyValueIterator begin() const
+        {
+            return KeyValueIterator(m_hash, true);
+        }
+
+        KeyValueIterator end() const
+        {
+            return KeyValueIterator(m_hash, false);
+        }
+    };
 
     // Get the number of keys currently stored in the table.
     unsigned GetCount() const
@@ -374,8 +608,6 @@ public:
     }
 
 private:
-    struct Node;
-
     //------------------------------------------------------------------------
     // GetIndexForKey: Get the bucket index for the specified key.
     //
@@ -525,165 +757,6 @@ public:
             (unsigned)(newTableSize * Behavior::s_density_factor_numerator / Behavior::s_density_factor_denominator);
     }
 
-    // For iteration, we use a pattern similar to the STL "forward
-    // iterator" pattern.  It basically consists of wrapping an
-    // "iteration variable" in an object, and providing pointer-like
-    // operators on the iterator. Example usage:
-    //
-    // for (JitHashTable::KeyIterator iter = foo->Begin(), end = foo->End(); !iter.Equal(end); iter++)
-    // {
-    //      // use foo, iter.
-    // }
-    // iter.Get() will yield (a reference to) the
-    // current key.  It will assert the equivalent of "iter != end."
-    class KeyIterator
-    {
-    private:
-        friend class JitHashTable;
-
-        Node**   m_table;
-        Node*    m_node;
-        unsigned m_tableSize;
-        unsigned m_index;
-
-    public:
-        //------------------------------------------------------------------------
-        // KeyIterator: Construct an iterator for the specified JitHashTable.
-        //
-        // Arguments:
-        //    hash  - the hashtable
-        //    begin - `true` to construct an "begin" iterator,
-        //            `false` to construct an "end" iterator
-        //
-        KeyIterator(const JitHashTable* hash, bool begin)
-            : m_table(hash->m_table)
-            , m_node(nullptr)
-            , m_tableSize(hash->m_tableSizeInfo.prime)
-            , m_index(begin ? 0 : m_tableSize)
-        {
-            if (begin && (hash->m_tableCount > 0))
-            {
-                assert(m_table != nullptr);
-                while ((m_index < m_tableSize) && (m_table[m_index] == nullptr))
-                {
-                    m_index++;
-                }
-
-                if (m_index >= m_tableSize)
-                {
-                    return;
-                }
-                else
-                {
-                    m_node = m_table[m_index];
-                }
-                assert(m_node != nullptr);
-            }
-        }
-
-        //------------------------------------------------------------------------
-        // Get: Get a reference to this iterator's key.
-        //
-        // Return Value:
-        //    A reference to this iterator's key.
-        //
-        // Assumptions:
-        //    This must not be the "end" iterator.
-        //
-        const Key& Get() const
-        {
-            assert(m_node != nullptr);
-
-            return m_node->m_key;
-        }
-
-        //------------------------------------------------------------------------
-        // GetValue: Get a reference to this iterator's value.
-        //
-        // Return Value:
-        //    A reference to this iterator's value.
-        //
-        // Assumptions:
-        //    This must not be the "end" iterator.
-        //
-        Value& GetValue() const
-        {
-            assert(m_node != nullptr);
-
-            return m_node->m_val;
-        }
-
-        //------------------------------------------------------------------------
-        // SetValue: Assign a new value to this iterator's key
-        //
-        // Arguments:
-        //    value - the value to assign
-        //
-        // Assumptions:
-        //    This must not be the "end" iterator.
-        //
-        void SetValue(const Value& value) const
-        {
-            assert(m_node != nullptr);
-
-            m_node->m_val = value;
-        }
-
-        //------------------------------------------------------------------------
-        // Next: Advance the iterator to the next node.
-        //
-        // Notes:
-        //    Advancing the end iterator has no effect.
-        //
-        void Next()
-        {
-            if (m_node != nullptr)
-            {
-                m_node = m_node->m_next;
-                if (m_node != nullptr)
-                {
-                    return;
-                }
-
-                // Otherwise...
-                m_index++;
-            }
-            while ((m_index < m_tableSize) && (m_table[m_index] == nullptr))
-            {
-                m_index++;
-            }
-
-            if (m_index >= m_tableSize)
-            {
-                m_node = nullptr;
-                return;
-            }
-            else
-            {
-                m_node = m_table[m_index];
-            }
-            assert(m_node != nullptr);
-        }
-
-        // Return `true` if the specified iterator has the same position as this iterator
-        bool Equal(const KeyIterator& i) const
-        {
-            return i.m_node == m_node;
-        }
-
-        // Advance the iterator to the next node
-        void operator++()
-        {
-            Next();
-        }
-
-        // Advance the iterator to the next node
-        void operator++(int)
-        {
-            Next();
-        }
-    };
-
     //------------------------------------------------------------------------
     // operator[]: Get a reference to the value associated with the specified key.
     //
@@ -715,7 +788,7 @@ private:
     //
     static JitPrimeInfo NextPrime(unsigned number)
     {
-        for (int i = 0; i < (int)(_countof(jitPrimeInfo)); i++)
+        for (int i = 0; i < (int)(ArrLen(jitPrimeInfo)); i++)
         {
             if (jitPrimeInfo[i].prime >= number)
             {
@@ -727,30 +800,6 @@ private:
         Behavior::NoMemory();
     }
 
-    // The node type.
-    struct Node
-    {
-        Node* m_next; // Assume that the alignment requirement of Key and Value are no greater than Node*,
-                      // so put m_next first to avoid unnecessary padding.
-        Key   m_key;
-        Value m_val;
-
-        template <class... Args>
-        Node(Node* next, Key k, Args&&... args) : m_next(next), m_key(k), m_val(std::forward<Args>(args)...)
-        {
-        }
-
-        void* operator new(size_t sz, Allocator alloc)
-        {
-            return alloc.template allocate<unsigned char>(sz);
-        }
-
-        void operator delete(void* p, Allocator alloc)
-        {
-            alloc.deallocate(p);
-        }
-    };
-
     // Instance members
     Allocator    m_alloc;         // Allocator to use in this table.
     Node**       m_table;         // pointer to table
