@@ -5,13 +5,47 @@
 #ifndef TARGET_H_
 #define TARGET_H_
 
-// Native Varargs are not supported on Unix (all architectures) and Windows ARM
-#if defined(TARGET_WINDOWS) && !defined(TARGET_ARM)
-#define FEATURE_VARARG 1
-#else
-#define FEATURE_VARARG 0
+#ifdef TARGET_UNIX_POSSIBLY_SUPPORTED
+#define FEATURE_CFI_SUPPORT
 #endif
 
+// Undefine all of the target OS macros
+// Within the JIT codebase we use the TargetOS features
+#ifdef TARGET_UNIX
+#undef TARGET_UNIX
+#endif
+
+#ifdef TARGET_OSX
+#undef TARGET_OSX
+#endif
+
+#ifdef TARGET_APPLE
+#undef TARGET_APPLE
+#endif
+
+#ifdef TARGET_WINDOWS
+#undef TARGET_WINDOWS
+#endif
+
+// Native Varargs are not supported on Unix (all architectures) and Windows ARM
+inline bool compFeatureVarArg()
+{
+    return TargetOS::IsWindows && !TargetArchitecture::IsArm32;
+}
+inline bool compMacOsArm64Abi()
+{
+    return TargetArchitecture::IsArm64 && TargetOS::IsMacOS;
+}
+inline bool compFeatureArgSplit()
+{
+    return TargetArchitecture::IsLoongArch64 || TargetArchitecture::IsArm32 || TargetArchitecture::IsRiscv64 ||
+           (TargetOS::IsWindows && TargetArchitecture::IsArm64);
+}
+inline bool compUnixX86Abi()
+{
+    return TargetArchitecture::IsX86 && TargetOS::IsUnix;
+}
+
 /*****************************************************************************/
 // The following are human readable names for the target architectures
 #if defined(TARGET_X86)
@@ -22,6 +56,10 @@
 #define TARGET_READABLE_NAME "ARM"
 #elif defined(TARGET_ARM64)
 #define TARGET_READABLE_NAME "ARM64"
+#elif defined(TARGET_LOONGARCH64)
+#define TARGET_READABLE_NAME "LOONGARCH64"
+#elif defined(TARGET_RISCV64)
+#define TARGET_READABLE_NAME "RISCV64"
 #else
 #error Unsupported or unset target architecture
 #endif
@@ -29,7 +67,11 @@
 /*****************************************************************************/
 // The following are intended to capture only those #defines that cannot be replaced
 // with static const members of Target
-#if defined(TARGET_XARCH)
+#if defined(TARGET_AMD64)
+#define REGMASK_BITS 64
+#define CSE_CONST_SHARED_LOW_BITS 16
+
+#elif defined(TARGET_X86)
 #define REGMASK_BITS 32
 #define CSE_CONST_SHARED_LOW_BITS 16
 
@@ -41,6 +83,14 @@
 #define REGMASK_BITS 64
 #define CSE_CONST_SHARED_LOW_BITS 12
 
+#elif defined(TARGET_LOONGARCH64)
+#define REGMASK_BITS 64
+#define CSE_CONST_SHARED_LOW_BITS 12
+
+#elif defined(TARGET_RISCV64)
+#define REGMASK_BITS 64
+#define CSE_CONST_SHARED_LOW_BITS 12
+
 #else
 #error Unsupported or unset target architecture
 #endif
@@ -56,7 +106,7 @@
 //                       be assigned during register allocation.
 //    REG_NA           - Used to indicate that a register is either not yet assigned or not required.
 //
-#if defined(TARGET_ARM)
+#if defined(TARGET_ARM) || defined(TARGET_LOONGARCH64) || defined(TARGET_RISCV64)
 enum _regNumber_enum : unsigned
 {
 #define REGDEF(name, rnum, mask, sname) REG_##name = rnum,
@@ -110,13 +160,14 @@ enum _regNumber_enum : unsigned
     ACTUAL_REG_COUNT = REG_COUNT - 1 // everything but REG_STK (only real regs)
 };
 
-enum _regMask_enum : unsigned
+enum _regMask_enum : uint64_t
 {
     RBM_NONE = 0,
 
 #define REGDEF(name, rnum, mask, sname) RBM_##name = mask,
 #define REGALIAS(alias, realname) RBM_##alias = RBM_##realname,
 #include "register.h"
+
 };
 
 #elif defined(TARGET_X86)
@@ -145,6 +196,8 @@ enum _regMask_enum : unsigned
 #error Unsupported target architecture
 #endif
 
+#define AVAILABLE_REG_COUNT get_AVAILABLE_REG_COUNT()
+
 /*****************************************************************************/
 
 // TODO-Cleanup: The types defined below are mildly confusing: why are there both?
@@ -156,7 +209,7 @@ enum _regMask_enum : unsigned
 // In any case, we believe that is OK to freely cast between these types; no information will
 // be lost.
 
-#ifdef TARGET_ARMARCH
+#if defined(TARGET_AMD64) || defined(TARGET_ARMARCH) || defined(TARGET_LOONGARCH64) || defined(TARGET_RISCV64)
 typedef unsigned __int64 regMaskTP;
 #else
 typedef unsigned       regMaskTP;
@@ -185,11 +238,6 @@ typedef unsigned char   regNumberSmall;
 
 /*****************************************************************************/
 
-#define LEA_AVAILABLE 1
-#define SCALED_ADDR_MODES 1
-
-/*****************************************************************************/
-
 #ifdef DEBUG
 #define DSP_SRC_OPER_LEFT 0
 #define DSP_SRC_OPER_RIGHT 1
@@ -213,6 +261,10 @@ typedef unsigned char   regNumberSmall;
 #include "targetarm.h"
 #elif defined(TARGET_ARM64)
 #include "targetarm64.h"
+#elif defined(TARGET_LOONGARCH64)
+#include "targetloongarch64.h"
+#elif defined(TARGET_RISCV64)
+#include "targetriscv64.h"
 #else
   #error Unsupported or unset target architecture
 #endif
@@ -249,7 +301,6 @@ C_ASSERT((FEATURE_TAILCALL_OPT == 0) || (FEATURE_FASTTAILCALL == 1));
 /*****************************************************************************/
 
 #define BITS_PER_BYTE              8
-#define RBM_ALL(type) (varTypeUsesFloatReg(type) ? RBM_ALLFLOAT : RBM_ALLINT)
 
 /*****************************************************************************/
 
@@ -268,7 +319,10 @@ class Target
 {
 public:
     static const char* g_tgtCPUName;
-    static const char* g_tgtPlatformName;
+    static const char* g_tgtPlatformName()
+    {
+        return TargetOS::IsWindows ? "Windows" : "Unix";
+    };
 
     enum ArgOrder
     {
@@ -279,11 +333,8 @@ public:
     static const enum ArgOrder g_tgtUnmanagedArgOrder;
 };
 
-#if defined(DEBUG) || defined(LATE_DISASM) || DUMP_GC_TABLES
-const char* getRegName(unsigned reg, bool isFloat = false); // this is for gcencode.cpp and disasm.cpp that don't use
-                                                            // the regNumber type
-const char* getRegName(regNumber reg, bool isFloat = false);
-#endif // defined(DEBUG) || defined(LATE_DISASM) || DUMP_GC_TABLES
+const char* getRegName(unsigned reg); // this is for gcencode.cpp and disasm.cpp that don't use the regNumber type
+const char* getRegName(regNumber reg);
 
 #ifdef DEBUG
 const char* getRegNameFloat(regNumber reg, var_types type);
@@ -303,7 +354,7 @@ inline bool isByteReg(regNumber reg)
 #endif
 
 inline regMaskTP genRegMask(regNumber reg);
-inline regMaskTP genRegMaskFloat(regNumber reg, var_types type = TYP_DOUBLE);
+inline regMaskTP genRegMaskFloat(regNumber reg ARM_ARG(var_types type = TYP_DOUBLE));
 
 /*****************************************************************************
  * Return true if the register number is valid
@@ -322,6 +373,18 @@ inline bool genIsValidIntReg(regNumber reg)
     return reg >= REG_INT_FIRST && reg <= REG_INT_LAST;
 }
 
+/*****************************************************************************
+ * Return true if the register is a valid integer or fake register
+ */
+inline bool genIsValidIntOrFakeReg(regNumber reg)
+{
+#if defined(TARGET_ARM64)
+    return genIsValidIntReg(reg) || (reg == REG_SP);
+#else
+    return genIsValidIntReg(reg);
+#endif
+}
+
 /*****************************************************************************
  * Return true if the register is a valid floating point register
  */
@@ -330,6 +393,16 @@ inline bool genIsValidFloatReg(regNumber reg)
     return reg >= REG_FP_FIRST && reg <= REG_FP_LAST;
 }
 
+#if defined(TARGET_XARCH)
+/*****************************************************************************
+ * Return true if the register is a valid mask register
+ */
+inline bool genIsValidMaskReg(regNumber reg)
+{
+    return reg >= REG_MASK_FIRST && reg <= REG_MASK_LAST;
+}
+#endif // TARGET_XARCH
+
 #ifdef TARGET_ARM
 
 /*****************************************************************************
@@ -495,7 +568,7 @@ inline regMaskTP genRegMask(regNumber reg)
     // (L1 latency on sandy bridge is 4 cycles for [base] and 5 for [base + index*c] )
     // the reason this is AMD-only is because the x86 BE will try to get reg masks for REG_STK
     // and the result needs to be zero.
-    regMaskTP result = 1 << reg;
+    regMaskTP result = 1ULL << reg;
     assert(result == regMasks[reg]);
     return result;
 #else
@@ -508,9 +581,10 @@ inline regMaskTP genRegMask(regNumber reg)
  *  Map a register number to a floating-point register mask.
  */
 
-inline regMaskTP genRegMaskFloat(regNumber reg, var_types type /* = TYP_DOUBLE */)
+inline regMaskTP genRegMaskFloat(regNumber reg ARM_ARG(var_types type /* = TYP_DOUBLE */))
 {
-#if defined(TARGET_AMD64) || defined(TARGET_ARM64) || defined(TARGET_X86)
+#if defined(TARGET_AMD64) || defined(TARGET_ARM64) || defined(TARGET_X86) || defined(TARGET_LOONGARCH64) ||            \
+    defined(TARGET_RISCV64)
     assert(genIsValidFloatReg(reg));
     assert((unsigned)reg < ArrLen(regMasks));
     return regMasks[reg];
@@ -552,20 +626,22 @@ inline regMaskTP genRegMaskFloat(regNumber reg, var_types type /* = TYP_DOUBLE *
 //
 inline regMaskTP genRegMask(regNumber regNum, var_types type)
 {
-#ifndef TARGET_ARM
-    return genRegMask(regNum);
-#else
+#if defined(TARGET_ARM)
     regMaskTP regMask = RBM_NONE;
 
-    if (varTypeUsesFloatReg(type))
-    {
-        regMask = genRegMaskFloat(regNum, type);
-    }
-    else
+    if (varTypeUsesIntReg(type))
     {
         regMask = genRegMask(regNum);
     }
+    else
+    {
+        assert(varTypeUsesFloatReg(type));
+        regMask = genRegMaskFloat(regNum, type);
+    }
+
     return regMask;
+#else
+    return genRegMask(regNum);
 #endif
 }
 
@@ -605,16 +681,27 @@ inline regNumber regNextOfType(regNumber reg, var_types type)
     regReturn = REG_NEXT(reg);
 #endif
 
-    if (varTypeUsesFloatReg(type))
+    if (varTypeUsesIntReg(type))
     {
-        if (regReturn > REG_FP_LAST)
+        if (regReturn > REG_INT_LAST)
         {
             regReturn = REG_NA;
         }
     }
+#if defined(TARGET_XARCH)
+    else if (varTypeUsesMaskReg(type))
+    {
+        if (regReturn > REG_MASK_LAST)
+        {
+            regReturn = REG_NA;
+        }
+    }
+#endif // TARGET_XARCH
     else
     {
-        if (regReturn > REG_INT_LAST)
+        assert(varTypeUsesFloatReg(type));
+
+        if (regReturn > REG_FP_LAST)
         {
             regReturn = REG_NA;
         }
@@ -671,7 +758,7 @@ C_ASSERT(sizeof(target_ssize_t) == TARGET_POINTER_SIZE);
 #if defined(TARGET_X86)
 // instrDescCns holds constant values for the emitter. The X86 compiler is unique in that it
 // may represent relocated pointer values with these constants. On the 64bit to 32 bit
-// cross-targetting jit, the the constant value must be represented as a 64bit value in order
+// cross-targeting jit, the constant value must be represented as a 64bit value in order
 // to represent these pointers.
 typedef ssize_t cnsval_ssize_t;
 typedef size_t  cnsval_size_t;
