@@ -10,7 +10,7 @@ XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 */
 
-#ifdef TARGET_ARMARCH
+#if defined(TARGET_ARMARCH) || defined(TARGET_LOONGARCH64) || defined(TARGET_RISCV64)
 
 // Windows no longer imposes a maximum prolog size. However, we still have an
 // assert here just to inform us if we increase the size of the prolog
@@ -34,7 +34,24 @@ const unsigned MAX_EPILOG_SIZE_BYTES = 100;
 #define UW_MAX_FRAGMENT_SIZE_BYTES (1U << 20)
 #define UW_MAX_CODE_WORDS_COUNT 31
 #define UW_MAX_EPILOG_START_INDEX 0x3FFU
-#endif // TARGET_ARM64
+#elif defined(TARGET_LOONGARCH64)
+const unsigned MAX_PROLOG_SIZE_BYTES = 200;
+const unsigned MAX_EPILOG_SIZE_BYTES = 200;
+#define UWC_END 0xE4   // "end" unwind code
+#define UWC_END_C 0xE5 // "end_c" unwind code
+#define UW_MAX_FRAGMENT_SIZE_BYTES (1U << 20)
+#define UW_MAX_CODE_WORDS_COUNT 31
+#define UW_MAX_EPILOG_START_INDEX 0x3FFU
+#elif defined(TARGET_RISCV64)
+const unsigned MAX_PROLOG_SIZE_BYTES = 200;
+const unsigned MAX_EPILOG_SIZE_BYTES = 200;
+#define UWC_END 0xE4   // "end" unwind code
+#define UWC_END_C 0xE5 // "end_c" unwind code
+#define UW_MAX_FRAGMENT_SIZE_BYTES (1U << 20)
+#define UW_MAX_CODE_WORDS_COUNT 31
+#define UW_MAX_EPILOG_START_INDEX 0x3FFU
+
+#endif // TARGET_RISCV64
 
 #define UW_MAX_EPILOG_COUNT 31                 // Max number that can be encoded in the "Epilog count" field
                                                // of the .pdata record
@@ -62,7 +79,7 @@ class UnwindEpilogInfo;
 class UnwindFragmentInfo;
 class UnwindInfo;
 
-// UnwindBase: A base class shared by the the unwind classes that require
+// UnwindBase: A base class shared by the unwind classes that require
 // a Compiler* for memory allocation.
 
 class UnwindBase
@@ -79,36 +96,10 @@ protected:
     {
     }
 
-// TODO: How do we get the ability to access uwiComp without error on Clang?
-#if defined(DEBUG) && !defined(__GNUC__)
-
-    template <typename T>
-    T dspPtr(T p)
-    {
-        return uwiComp->dspPtr(p);
-    }
-
-    template <typename T>
-    T dspOffset(T o)
-    {
-        return uwiComp->dspOffset(o);
-    }
-
-    static const char* dspBool(bool b)
-    {
-        return (b) ? "true" : "false";
-    }
-
-#endif // DEBUG
-
-    //
-    // Data
-    //
-
     Compiler* uwiComp;
 };
 
-// UnwindCodesBase: A base class shared by the the classes used to represent the prolog
+// UnwindCodesBase: A base class shared by the classes used to represent the prolog
 // and epilog unwind codes.
 
 class UnwindCodesBase
@@ -129,9 +120,9 @@ public:
     {
 #if defined(TARGET_ARM)
         return b >= 0xFD;
-#elif defined(TARGET_ARM64)
+#elif defined(TARGET_ARM64) || defined(TARGET_LOONGARCH64) || defined(TARGET_RISCV64)
         return (b == UWC_END); // TODO-ARM64-Bug?: what about the "end_c" code?
-#endif // TARGET_ARM64
+#endif // TARGET_ARM64 || TARGET_LOONGARCH64 || TARGET_RISCV64
     }
 
 #ifdef DEBUG
@@ -427,10 +418,12 @@ public:
 
         uecFinalized = true; // With the "end" code in place, now we're done
 
+#ifndef TARGET_RISCV64 // TODO COMMENTED OUT BECAUSE s_UnwindSize is not set
 #ifdef DEBUG
         unsigned codeSize = GetCodeSizeFromUnwindCodes(false);
         assert(codeSize <= MAX_EPILOG_SIZE_BYTES);
 #endif // DEBUG
+#endif // !TARGET_RISCV64
     }
 
     UnwindEpilogCodes()
@@ -578,7 +571,7 @@ private:
 
 // UnwindFragmentInfo: represents all the unwind information for a single fragment of a function or funclet.
 // A fragment is a section with a code size less than the maximum unwind code size: either 512K bytes, or
-// that specified by COMPlus_JitSplitFunctionSize. In most cases, there will be exactly one fragment.
+// that specified by DOTNET_JitSplitFunctionSize. In most cases, there will be exactly one fragment.
 
 class UnwindFragmentInfo : public UnwindBase
 {
@@ -813,7 +806,7 @@ public:
     // Given the first byte of the unwind code, check that its opsize matches
     // the last instruction added in the emitter.
     void CheckOpsize(BYTE b1);
-#elif defined(TARGET_ARM64)
+#elif defined(TARGET_ARM64) || defined(TARGET_LOONGARCH64) || defined(TARGET_RISCV64)
     void CheckOpsize(BYTE b1)
     {
     } // nothing to do; all instructions are 4 bytes
@@ -864,4 +857,4 @@ void DumpUnwindInfo(Compiler*         comp,
 
 #endif // DEBUG
 
-#endif // TARGET_ARMARCH
+#endif // TARGET_ARMARCH || TARGET_LOONGARCH64 || TARGET_RISCV64
