@@ -449,13 +449,13 @@ SharedMemoryId::SharedMemoryId(LPCSTR name)
     if (strncmp(name, "Global\\", 7) == 0)
     {
         m_isSessionScope = false;
-        name += _countof("Global\\") - 1;
+        name += STRING_LENGTH("Global\\");
     }
     else
     {
         if (strncmp(name, "Local\\", 6) == 0)
         {
-            name += _countof("Local\\") - 1;
+            name += STRING_LENGTH("Local\\");
         }
         m_isSessionScope = true;
     }
