@@ -5,15 +5,13 @@ using System;
 using System.Collections;
 using System.Collections.Generic;
 using System.Diagnostics;
-using System.Text.RegularExpressions;
-using ILCompiler;
 
 namespace Internal.Pgo
 {
     public enum PgoInstrumentationKind
     {
         // This must be kept in sync with PgoInstrumentationKind in corjit.h
-        // New InstrumentationKinds should recieve corresponding merging logic in
+        // New InstrumentationKinds should receive corresponding merging logic in
         // PgoSchemeMergeComparer and the MergeInSchemaElem functions below
 
         // Schema data types
@@ -21,6 +19,7 @@ namespace Internal.Pgo
         FourByte = 1,
         EightByte = 2,
         TypeHandle = 3,
+        MethodHandle = 4,
 
         // Mask of all schema data types
         MarshalMask = 0xF,
@@ -39,24 +38,28 @@ namespace Internal.Pgo
         Done = None, // All instrumentation schemas must end with a record which is "Done"
         BasicBlockIntCount = (DescriptorMin * 1) | FourByte, // basic block counter using unsigned 4 byte int
         BasicBlockLongCount = (DescriptorMin * 1) | EightByte, // basic block counter using unsigned 8 byte int
-        TypeHandleHistogramIntCount = (DescriptorMin * 2) | FourByte | AlignPointer, // 4 byte counter that is part of a type histogram. Aligned to match ClassProfile32's alignment.
-        TypeHandleHistogramLongCount = (DescriptorMin * 2) | EightByte, // 8 byte counter that is part of a type histogram
-        TypeHandleHistogramTypeHandle = (DescriptorMin * 3) | TypeHandle, // TypeHandle that is part of a type histogram
+        HandleHistogramIntCount = (DescriptorMin * 2) | FourByte | AlignPointer, // 4 byte counter that is part of a type histogram. Aligned to match ClassProfile32's alignment.
+        HandleHistogramLongCount = (DescriptorMin * 2) | EightByte, // 8 byte counter that is part of a type histogram
+        HandleHistogramTypes = (DescriptorMin * 3) | TypeHandle, // TypeHandle that is part of a type histogram
+        HandleHistogramMethods = (DescriptorMin * 3) | MethodHandle, // TypeHandle that is part of a type histogram
         Version = (DescriptorMin * 4) | None, // Version is encoded in the Other field of the schema
         NumRuns = (DescriptorMin * 5) | None, // Number of runs is encoded in the Other field of the schema
         EdgeIntCount = (DescriptorMin * 6) | FourByte, // edge counter using unsigned 4 byte int
         EdgeLongCount = (DescriptorMin * 6) | EightByte, // edge counter using unsigned 8 byte int
         GetLikelyClass = (DescriptorMin * 7) | TypeHandle, // Compressed get likely class data
+        GetLikelyMethod = (DescriptorMin * 7) | MethodHandle, // Compressed get likely method data
     }
 
-    public interface IPgoSchemaDataLoader<TType>
+    public interface IPgoSchemaDataLoader<TType, TMethod>
     {
         TType TypeFromLong(long input);
+        TMethod MethodFromLong(long input);
     }
 
-    public interface IPgoEncodedValueEmitter<TType>
+    public interface IPgoEncodedValueEmitter<TType, TMethod>
     {
         void EmitType(TType type, TType previousValue);
+        void EmitMethod(TMethod method, TMethod previousValue);
         void EmitLong(long value, long previousValue);
         bool EmitDone();
     }
@@ -77,6 +80,8 @@ namespace Internal.Pgo
         public bool DataHeldInDataLong => (Count == 1 &&
                             (((InstrumentationKind & PgoInstrumentationKind.MarshalMask) == PgoInstrumentationKind.FourByte) ||
                             ((InstrumentationKind & PgoInstrumentationKind.MarshalMask) == PgoInstrumentationKind.EightByte)));
+
+        public override string ToString() => $"{InstrumentationKind} @ {ILOffset} Count = {Count} DataLong = {DataLong}";
     }
 
     // Flags stored in 'Other' field of TypeHandleHistogram*Count entries.
@@ -106,8 +111,8 @@ namespace Internal.Pgo
 
         public class PgoEncodedCompressedIntParser : IEnumerable<long>, IEnumerator<long>
         {
-            long _current;
-            byte[] _bytes;
+            private long _current;
+            private byte[] _bytes;
 
             public PgoEncodedCompressedIntParser(byte[] bytes, int startOffset)
             {
@@ -230,13 +235,14 @@ namespace Internal.Pgo
             }
         }
 
-        public static IEnumerable<PgoSchemaElem> ParsePgoData<TType>(IPgoSchemaDataLoader<TType> dataProvider, IEnumerable<long> inputDataStream, bool longsAreCompressed)
+        public static IEnumerable<PgoSchemaElem> ParsePgoData<TType, TMethod>(IPgoSchemaDataLoader<TType, TMethod> dataProvider, IEnumerable<long> inputDataStream, bool longsAreCompressed)
         {
             int dataCountToRead = 0;
             PgoSchemaElem curSchema = default(PgoSchemaElem);
             InstrumentationDataProcessingState processingState = InstrumentationDataProcessingState.UpdateProcessMaskFlag;
             long lastDataValue = 0;
             long lastTypeValue = 0;
+            long lastMethodValue = 0;
 
             foreach (long value in inputDataStream)
             {
@@ -278,6 +284,14 @@ namespace Internal.Pgo
                                     lastTypeValue = value;
                                 ((TType[])curSchema.DataObject)[dataIndex] = dataProvider.TypeFromLong(lastTypeValue);
                                 break;
+
+                            case PgoInstrumentationKind.MethodHandle:
+                                if (longsAreCompressed)
+                                    lastMethodValue += value;
+                                else
+                                    lastMethodValue = value;
+                                ((TMethod[])curSchema.DataObject)[dataIndex] = dataProvider.MethodFromLong(lastMethodValue);
+                                break;
                         }
                     }
                     dataCountToRead--;
@@ -303,7 +317,7 @@ namespace Internal.Pgo
                     else
                         curSchema.ILOffset = checked((int)value);
 
-                    processingState = processingState & ~InstrumentationDataProcessingState.ILOffset;
+                    processingState &= ~InstrumentationDataProcessingState.ILOffset;
                 }
                 else if ((processingState & InstrumentationDataProcessingState.Type) == InstrumentationDataProcessingState.Type)
                 {
@@ -312,7 +326,7 @@ namespace Internal.Pgo
                     else
                         curSchema.InstrumentationKind = (PgoInstrumentationKind)value;
 
-                    processingState = processingState & ~InstrumentationDataProcessingState.Type;
+                    processingState &= ~InstrumentationDataProcessingState.Type;
                 }
                 else if ((processingState & InstrumentationDataProcessingState.Count) == InstrumentationDataProcessingState.Count)
                 {
@@ -320,7 +334,7 @@ namespace Internal.Pgo
                         curSchema.Count = checked((int)(value + (long)curSchema.Count));
                     else
                         curSchema.Count = checked((int)value);
-                    processingState = processingState & ~InstrumentationDataProcessingState.Count;
+                    processingState &= ~InstrumentationDataProcessingState.Count;
                 }
                 else if ((processingState & InstrumentationDataProcessingState.Other) == InstrumentationDataProcessingState.Other)
                 {
@@ -328,7 +342,7 @@ namespace Internal.Pgo
                         curSchema.Other = checked((int)(value + (long)curSchema.Other));
                     else
                         curSchema.Other = checked((int)value);
-                    processingState = processingState & ~InstrumentationDataProcessingState.Other;
+                    processingState &= ~InstrumentationDataProcessingState.Other;
                 }
 
                 if (processingState == InstrumentationDataProcessingState.Done)
@@ -364,6 +378,10 @@ namespace Internal.Pgo
                             curSchema.DataObject = new TType[curSchema.Count];
                             dataCountToRead = curSchema.Count;
                             break;
+                        case PgoInstrumentationKind.MethodHandle:
+                            curSchema.DataObject = new TMethod[curSchema.Count];
+                            dataCountToRead = curSchema.Count;
+                            break;
                         default:
                             throw new Exception("Unknown Type");
                     }
@@ -373,10 +391,11 @@ namespace Internal.Pgo
             throw new Exception("Partial Instrumentation Data");
         }
 
-        public static void EncodePgoData<TType>(IEnumerable<PgoSchemaElem> schemas, IPgoEncodedValueEmitter<TType> valueEmitter, bool emitAllElementsUnconditionally)
+        public static void EncodePgoData<TType, TMethod>(IEnumerable<PgoSchemaElem> schemas, IPgoEncodedValueEmitter<TType, TMethod> valueEmitter, bool emitAllElementsUnconditionally)
         {
             PgoSchemaElem prevSchema = default(PgoSchemaElem);
             TType prevEmittedType = default(TType);
+            TMethod prevEmittedMethod = default(TMethod);
             long prevEmittedIntData = 0;
 
             foreach (PgoSchemaElem schema in schemas)
@@ -391,13 +410,13 @@ namespace Internal.Pgo
                 if (!emitAllElementsUnconditionally)
                 {
                     if (ilOffsetDiff != 0)
-                        modifyMask = modifyMask | InstrumentationDataProcessingState.ILOffset;
+                        modifyMask |= InstrumentationDataProcessingState.ILOffset;
                     if (TypeDiff != 0)
-                        modifyMask = modifyMask | InstrumentationDataProcessingState.Type;
+                        modifyMask |= InstrumentationDataProcessingState.Type;
                     if (CountDiff != 0)
-                        modifyMask = modifyMask | InstrumentationDataProcessingState.Count;
+                        modifyMask |= InstrumentationDataProcessingState.Count;
                     if (OtherDiff != 0)
-                        modifyMask = modifyMask | InstrumentationDataProcessingState.Other;
+                        modifyMask |= InstrumentationDataProcessingState.Other;
                 }
                 else
                 {
@@ -422,7 +441,8 @@ namespace Internal.Pgo
 
                 for (int i = 0; i < schema.Count; i++)
                 {
-                    switch (schema.InstrumentationKind & PgoInstrumentationKind.MarshalMask)
+                    PgoInstrumentationKind marshal = schema.InstrumentationKind & PgoInstrumentationKind.MarshalMask;
+                    switch (marshal)
                     {
                         case PgoInstrumentationKind.None:
                             break;
@@ -463,6 +483,15 @@ namespace Internal.Pgo
                                 prevEmittedType = typeToEmit;
                                 break;
                             }
+                        case PgoInstrumentationKind.MethodHandle:
+                            {
+                                TMethod methodToEmit = ((TMethod[])schema.DataObject)[i];
+                                valueEmitter.EmitMethod(methodToEmit, prevEmittedMethod);
+                                prevEmittedMethod = methodToEmit;
+                                break;
+                            }
+                        default:
+                            throw new ArgumentException("Unknown schema marshal " + marshal);
                     }
                 }
 
@@ -480,7 +509,7 @@ namespace Internal.Pgo
         }
 
 
-        private class PgoSchemaMergeComparer : IComparer<PgoSchemaElem>, IEqualityComparer<PgoSchemaElem>
+        private sealed class PgoSchemaMergeComparer : IComparer<PgoSchemaElem>, IEqualityComparer<PgoSchemaElem>
         {
             public static PgoSchemaMergeComparer Singleton = new PgoSchemaMergeComparer();
 
@@ -512,7 +541,7 @@ namespace Internal.Pgo
             int IEqualityComparer<PgoSchemaElem>.GetHashCode(PgoSchemaElem obj) => obj.ILOffset ^ ((int)(obj.InstrumentationKind & PgoInstrumentationKind.DescriptorMask) << 20);
         }
 
-        public static PgoSchemaElem[] Merge<TType>(ReadOnlySpan<PgoSchemaElem[]> schemasToMerge)
+        public static PgoSchemaElem[] Merge<TType, TMethod>(ReadOnlySpan<PgoSchemaElem[]> schemasToMerge)
         {
             {
                 // The merging algorithm will sort the schema data by iloffset, then by InstrumentationKind
@@ -532,7 +561,7 @@ namespace Internal.Pgo
 
                     if (!foundNumRuns)
                     {
-                        PgoSchemaElem oneRunSchema = new PgoSchemaElem();
+                        PgoSchemaElem oneRunSchema = default(PgoSchemaElem);
                         oneRunSchema.InstrumentationKind = PgoInstrumentationKind.NumRuns;
                         oneRunSchema.ILOffset = 0;
                         oneRunSchema.Other = 1;
@@ -547,7 +576,7 @@ namespace Internal.Pgo
                 return result;
             }
 
-            void MergeInSchemaElem(Dictionary<PgoSchemaElem, PgoSchemaElem> dataMerger, PgoSchemaElem schema)
+            static void MergeInSchemaElem(Dictionary<PgoSchemaElem, PgoSchemaElem> dataMerger, PgoSchemaElem schema)
             {
                 if (dataMerger.TryGetValue(schema, out var existingSchemaItem))
                 {
@@ -560,8 +589,8 @@ namespace Internal.Pgo
                         case PgoInstrumentationKind.BasicBlockLongCount:
                         case PgoInstrumentationKind.EdgeIntCount:
                         case PgoInstrumentationKind.EdgeLongCount:
-                        case PgoInstrumentationKind.TypeHandleHistogramIntCount:
-                        case PgoInstrumentationKind.TypeHandleHistogramLongCount:
+                        case PgoInstrumentationKind.HandleHistogramIntCount:
+                        case PgoInstrumentationKind.HandleHistogramLongCount:
                             if ((existingSchemaItem.Count != 1) || (schema.Count != 1))
                             {
                                 throw new Exception("Unable to merge pgo data. Invalid format");
@@ -569,7 +598,7 @@ namespace Internal.Pgo
                             mergedElem.DataLong = existingSchemaItem.DataLong + schema.DataLong;
                             break;
 
-                        case PgoInstrumentationKind.TypeHandleHistogramTypeHandle:
+                        case PgoInstrumentationKind.HandleHistogramTypes:
                             {
                                 mergedElem.Count = existingSchemaItem.Count + schema.Count;
                                 TType[] newMergedTypeArray = new TType[mergedElem.Count];
@@ -586,6 +615,23 @@ namespace Internal.Pgo
                                 break;
                             }
 
+                        case PgoInstrumentationKind.HandleHistogramMethods:
+                            {
+                                mergedElem.Count = existingSchemaItem.Count + schema.Count;
+                                TMethod[] newMergedMethodArray = new TMethod[mergedElem.Count];
+                                mergedElem.DataObject = newMergedMethodArray;
+                                int i = 0;
+                                foreach (TMethod meth in (TMethod[])existingSchemaItem.DataObject)
+                                {
+                                    newMergedMethodArray[i++] = meth;
+                                }
+                                foreach (TMethod meth in (TMethod[])schema.DataObject)
+                                {
+                                    newMergedMethodArray[i++] = meth;
+                                }
+                                break;
+                            }
+
                         case PgoInstrumentationKind.Version:
                             {
                                 mergedElem.Other = Math.Max(existingSchemaItem.Other, schema.Other);
@@ -600,7 +646,7 @@ namespace Internal.Pgo
                     }
 
                     Debug.Assert(PgoSchemaMergeComparer.Singleton.Compare(schema, mergedElem) == 0);
-                    Debug.Assert(PgoSchemaMergeComparer.Singleton.Equals(schema, mergedElem) == true);
+                    Debug.Assert(PgoSchemaMergeComparer.Singleton.Equals(schema, mergedElem));
                     dataMerger[mergedElem] = mergedElem;
                 }
                 else
