@@ -1,12 +1,10 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-
 namespace Internal.TypeSystem
 {
     // Holds code for canonicalizing a function pointer type
-    partial class FunctionPointerType
+    public partial class FunctionPointerType
     {
         public override bool IsCanonicalSubtype(CanonicalFormKind policy)
         {
