@@ -1,13 +1,12 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
 using Debug = System.Diagnostics.Debug;
 
 namespace Internal.TypeSystem
 {
     /// <summary>
-    /// RuntimeInterfaces algorithm for for array types which are similar to a generic type
+    /// RuntimeInterfaces algorithm for array types which are similar to a generic type
     /// </summary>
     public sealed class ArrayOfTRuntimeInterfacesAlgorithm : RuntimeInterfacesAlgorithm
     {
@@ -17,7 +16,7 @@ namespace Internal.TypeSystem
         private MetadataType _arrayOfTType;
 
         /// <summary>
-        /// RuntimeInterfaces algorithm for for array types which are similar to a generic type
+        /// RuntimeInterfaces algorithm for array types which are similar to a generic type
         /// </summary>
         /// <param name="arrayOfTType">Open type to instantiate to get the interfaces associated with an array.</param>
         public ArrayOfTRuntimeInterfacesAlgorithm(MetadataType arrayOfTType)
