@@ -1,8 +1,6 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-
 using Debug = System.Diagnostics.Debug;
 
 namespace Internal.TypeSystem
@@ -99,6 +97,11 @@ namespace Internal.TypeSystem
         /// Most of the checks are performed by the CanCastTo, but some cases are pre-filtered out.
         /// </summary>
         public static bool IsCompatibleWith(this TypeDesc thisType, TypeDesc otherType)
+        {
+            return thisType.IsCompatibleWith(otherType, null);
+        }
+
+        internal  static bool IsCompatibleWith(this TypeDesc thisType, TypeDesc otherType, StackOverflowProtect visited)
         {
             // Structs can be cast to the interfaces they implement, but they are not compatible according to ECMA I.8.7.1
             bool isCastFromValueTypeToReferenceType = otherType.IsValueType && !thisType.IsValueType;
@@ -135,7 +138,7 @@ namespace Internal.TypeSystem
             }
 
             // Nullable<T> can be cast to T, but this is not compatible according to ECMA I.8.7.1
-            bool isCastFromNullableOfTtoT = thisType.IsNullable && otherType.IsEquivalentTo(thisType.Instantiation[0]);
+            bool isCastFromNullableOfTtoT = thisType.IsNullable && otherType.IsEquivalentTo(thisType.Instantiation[0], visited);
             if (isCastFromNullableOfTtoT)
             {
                 return false;
@@ -144,12 +147,25 @@ namespace Internal.TypeSystem
             return otherType.CanCastTo(thisType);
         }
 
-        private static bool IsEquivalentTo(this TypeDesc thisType, TypeDesc otherType)
+        public static bool IsEquivalentTo(this TypeDesc thisType, TypeDesc otherType)
         {
-            // TODO: Once type equivalence is implemented, this implementation needs to be enhanced to honor it.
-            return thisType == otherType;
+            bool isEquivalentTo = thisType == otherType;
+            if (!isEquivalentTo)
+                thisType.IsEquivalentTo(otherType, (StackOverflowProtect)null, ref isEquivalentTo);
+
+            return isEquivalentTo;
         }
 
+        internal static bool IsEquivalentTo(this TypeDesc thisType, TypeDesc otherType, StackOverflowProtect visited)
+        {
+            bool isEquivalentTo = thisType == otherType;
+            if (!isEquivalentTo)
+                thisType.IsEquivalentTo(otherType, visited, ref isEquivalentTo);
+
+            return isEquivalentTo;
+        }
+
+        static partial void IsEquivalentTo(this TypeDesc thisType, TypeDesc otherType, StackOverflowProtect visited, ref bool isEquivalentTo);
         private static bool CanCastToInternal(this TypeDesc thisType, TypeDesc otherType, StackOverflowProtect protect)
         {
             if (thisType == otherType)
@@ -197,9 +213,21 @@ namespace Internal.TypeSystem
                 return true;
             }
 
+            Instantiation typeInstantiation;
+            Instantiation methodInstantiation = default(Instantiation);
+            if (thisType.AssociatedTypeOrMethod is MethodDesc method)
+            {
+                typeInstantiation = method.OwningType.Instantiation;
+                methodInstantiation = method.Instantiation;
+            }
+            else
+            {
+                typeInstantiation = ((TypeDesc)thisType.AssociatedTypeOrMethod).Instantiation;
+            }
             foreach (var typeConstraint in thisType.TypeConstraints)
             {
-                if (typeConstraint.CanCastToInternal(otherType, protect))
+                TypeDesc instantiatedConstraint = typeConstraint.InstantiateSignature(typeInstantiation, methodInstantiation);
+                if (instantiatedConstraint.CanCastToInternal(otherType, protect))
                 {
                     return true;
                 }
@@ -247,7 +275,7 @@ namespace Internal.TypeSystem
             // unboxed versions do not.  Parameterized types have the
             // unboxed version, thus, if the from type parameter is value
             // class then only an exact match/equivalence works.
-            if (thisType.ParameterType == paramType)
+            if (thisType.ParameterType.IsEquivalentTo(paramType))
             {
                 return true;
             }
@@ -263,7 +291,7 @@ namespace Internal.TypeSystem
             else if (curTypesParm.IsGenericParameter)
             {
                 var genericVariableFromParam = (GenericParameterDesc)curTypesParm;
-                if (genericVariableFromParam.HasReferenceTypeConstraint)
+                if (genericVariableFromParam.HasReferenceTypeConstraint || IsConstrainedAsGCPointer(genericVariableFromParam))
                 {
                     return genericVariableFromParam.CanCastToInternal(paramType, protect);
                 }
@@ -281,14 +309,38 @@ namespace Internal.TypeSystem
             return false;
         }
 
+        private static bool IsConstrainedAsGCPointer(GenericParameterDesc type)
+        {
+            foreach (var typeConstraint in type.TypeConstraints)
+            {
+                if (typeConstraint.IsGenericParameter)
+                {
+                    if (IsConstrainedAsGCPointer((GenericParameterDesc)typeConstraint))
+                        return true;
+                }
+
+                if (!typeConstraint.IsInterface && typeConstraint.IsGCPointer)
+                {
+                    // Object, ValueType, and Enum are GCPointers but they do not constrain the type to GCPointer!
+                    if (!typeConstraint.IsWellKnownType(WellKnownType.Object) &&
+                        !typeConstraint.IsWellKnownType(WellKnownType.ValueType) &&
+                        !typeConstraint.IsWellKnownType(WellKnownType.Enum))
+                    {
+                        return true;
+                    }
+                }
+            }
+            return false;
+        }
+
         private static TypeFlags GetNormalizedIntegralArrayElementType(TypeDesc type)
         {
             Debug.Assert(!type.IsEnum);
 
             // Primitive types such as E_T_I4 and E_T_U4 are interchangeable
-            // Enums with interchangeable underlying types are interchangable
+            // Enums with interchangeable underlying types are interchangeable
             // BOOL is NOT interchangeable with I1/U1, neither CHAR -- with I2/U2
-            // Float and double are not interchangable here.
+            // Float and double are not interchangeable here.
 
             TypeFlags elementType = type.Category;
             switch (elementType)
@@ -341,7 +393,7 @@ namespace Internal.TypeSystem
         {
             if (!otherType.HasVariance)
             {
-                return thisType.CanCastToNonVariantInterface(otherType, protect);
+                return thisType.CanCastToNonVariantInterface(otherType);
             }
             else
             {
@@ -362,16 +414,16 @@ namespace Internal.TypeSystem
             return false;
         }
 
-        private static bool CanCastToNonVariantInterface(this TypeDesc thisType, TypeDesc otherType, StackOverflowProtect protect)
+        private static bool CanCastToNonVariantInterface(this TypeDesc thisType, TypeDesc otherType)
         {
-            if (otherType == thisType)
+            if (otherType.IsEquivalentTo(thisType))
             {
                 return true;
             }
 
             foreach (var interfaceType in thisType.RuntimeInterfaces)
             {
-                if (interfaceType == otherType)
+                if (interfaceType.IsEquivalentTo(otherType))
                 {
                     return true;
                 }
@@ -382,6 +434,11 @@ namespace Internal.TypeSystem
 
         private static bool CanCastByVarianceToInterfaceOrDelegate(this TypeDesc thisType, TypeDesc otherType, StackOverflowProtect protectInput)
         {
+            if (thisType == otherType)
+            {
+                return true;
+            }
+
             if (!thisType.HasSameTypeDefinition(otherType))
             {
                 return false;
@@ -408,7 +465,7 @@ namespace Internal.TypeSystem
                 TypeDesc arg = instantiationThis[i];
                 TypeDesc targetArg = instantiationTarget[i];
 
-                if (arg != targetArg)
+                if (!arg.IsEquivalentTo(targetArg))
                 {
                     GenericParameterDesc openArgType = (GenericParameterDesc)instantiationOpen[i];
 
@@ -450,7 +507,7 @@ namespace Internal.TypeSystem
                 // First chase inheritance hierarchy until we hit a class that only differs in its instantiation
                 do
                 {
-                    if (curType == otherType)
+                    if (curType.IsEquivalentTo(otherType))
                     {
                         return true;
                     }
@@ -481,7 +538,7 @@ namespace Internal.TypeSystem
 
                 do
                 {
-                    if (curType == otherType)
+                    if (curType.IsEquivalentTo(otherType))
                         return true;
 
                     curType = curType.BaseType;
@@ -502,7 +559,7 @@ namespace Internal.TypeSystem
             else if (thisType.IsGenericParameter)
             {
                 var genericVariableFromParam = (GenericParameterDesc)thisType;
-                if (genericVariableFromParam.HasReferenceTypeConstraint)
+                if (genericVariableFromParam.HasReferenceTypeConstraint || IsConstrainedAsGCPointer(genericVariableFromParam))
                 {
                     return genericVariableFromParam.CanCastToInternal(otherType, protect);
                 }
@@ -510,39 +567,47 @@ namespace Internal.TypeSystem
 
             return false;
         }
+    }
 
-        private class StackOverflowProtect
+    internal sealed class StackOverflowProtect
+    {
+        private CastingPair _value;
+        private StackOverflowProtect _previous;
+
+        public StackOverflowProtect(CastingPair value, StackOverflowProtect previous)
         {
-            private CastingPair _value;
-            private StackOverflowProtect _previous;
-
-            public StackOverflowProtect(CastingPair value, StackOverflowProtect previous)
-            {
-                _value = value;
-                _previous = previous;
-            }
-
-            public bool Contains(CastingPair value)
-            {
-                for (var current = this; current != null; current = current._previous)
-                    if (current._value.Equals(value))
-                        return true;
-                return false;
-            }
+            _value = value;
+            _previous = previous;
         }
 
-        private struct CastingPair
+        public static StackOverflowProtect GetTypeEquivalentForbiddenScope(StackOverflowProtect previous)
         {
-            public readonly TypeDesc FromType;
-            public readonly TypeDesc ToType;
+            var protect = new StackOverflowProtect(default(CastingPair), previous);
+            return protect;
+        }
 
-            public CastingPair(TypeDesc fromType, TypeDesc toType)
+        public bool Contains(CastingPair value)
+        {
+            for (var current = this; current != null; current = current._previous)
             {
-                FromType = fromType;
-                ToType = toType;
+                if (current._value.Equals(value))
+                    return true;
             }
-
-            public bool Equals(CastingPair other) => FromType == other.FromType && ToType == other.ToType;
+            return false;
         }
     }
+
+    internal struct CastingPair
+    {
+        public readonly TypeDesc FromType;
+        public readonly TypeDesc ToType;
+
+        public CastingPair(TypeDesc fromType, TypeDesc toType)
+        {
+            FromType = fromType;
+            ToType = toType;
+        }
+
+        public bool Equals(CastingPair other) => FromType == other.FromType && ToType == other.ToType;
+    }
 }
