@@ -1,9 +1,6 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-using System;
-using Debug = System.Diagnostics.Debug;
-
 namespace Internal.TypeSystem
 {
     /// <summary>
@@ -17,5 +14,6 @@ namespace Internal.TypeSystem
         X64,
         X86,
         Wasm32,
+        LoongArch64,
     }
 }
