@@ -233,6 +233,7 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
         public int m_byteStackIndex;     // Stack offset in bytes (or -1)
         public int m_byteStackSize;      // Stack size in bytes
 
+        public uint m_floatFlags;        // struct with two-fields can be passed by registers.
         // Initialize to represent a non-placed argument (no register or stack slots referenced).
         public void Init()
         {
@@ -242,13 +243,14 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
             m_cGenReg = 0;
             m_byteStackIndex = -1;
             m_byteStackSize = 0;
+            m_floatFlags = 0;
 
             m_fRequires64BitAlignment = false;
         }
     };
 
     // The ArgDestination class represents a destination location of an argument.
-    internal class ArgDestination
+    internal readonly struct ArgDestination
     {
         /// <summary>
         /// Transition block context.
@@ -309,8 +311,6 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
         //  fieldBytes - size of the structure
         internal void ReportPointersFromStructInRegisters(TypeDesc type, int delta, CORCOMPILE_GCREFMAP_TOKENS[] frame)
         {
-            // SPAN-TODO: GC reporting - https://github.com/dotnet/runtime/issues/7103
-
             Debug.Assert(IsStructPassedInRegs());
 
             int genRegDest = GetStructGenRegDestinationAddress();
@@ -617,6 +617,13 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                             return ((_argSize > _transitionBlock.EnregisteredParamTypeMaxSize) && (!_argTypeHandle.IsHomogeneousAggregate() || IsVarArg));
                         }
                         return false;
+                    case TargetArchitecture.LoongArch64:
+                        if (_argType == CorElementType.ELEMENT_TYPE_VALUETYPE)
+                        {
+                            Debug.Assert(!_argTypeHandle.IsNull());
+                            return ((_argSize > _transitionBlock.EnregisteredParamTypeMaxSize) || _transitionBlock.IsArgPassedByRef(_argTypeHandle));
+                        }
+                        return false;
                     default:
                         throw new NotImplementedException();
                 }
@@ -730,37 +737,18 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
             {
                 int numRegistersUsed = 0;
 
-#if PROJECTN || FEATURE_INTERPRETER
-                int initialArgOffset = 0;
-#endif
                 if (HasThis)
                     numRegistersUsed++;
 
                 if (HasRetBuffArg() && _transitionBlock.IsRetBuffPassedAsFirstArg)
                 {
-#if PROJECTN
-                    if (!_transitionBlock.IsX86)
-#endif
-                    {
-                        numRegistersUsed++;
-                    }
-#if PROJECTN
-                    else
-                    {
-                        // DESKTOP BEHAVIOR is to do nothing here, as ret buf is never reached by the scan algorithm that walks backwards
-                        // but in .NET Native, the x86 argument scan is a forward scan, so we need to skip the ret buf arg (which is always
-                        // on the stack)
-                        initialArgOffset = _transitionBlock.PointerSize;
-                    }
-#endif
+                    numRegistersUsed++;
                 }
 
                 Debug.Assert(!IsVarArg || !HasParamType);
 
-#if !PROJECTN
                 // DESKTOP BEHAVIOR - This block is disabled for x86 as the param arg is the last argument on .NET Framework x86.
                 if (!_transitionBlock.IsX86)
-#endif
                 {
                     if (HasParamType)
                     {
@@ -802,11 +790,7 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         }
 #endif
                         _x86NumRegistersUsed = numRegistersUsed;
-#if PROJECTN
-                        _x86OfsStack = (int)(_transitionBlock.OffsetOfArgs + initialArgOffset);
-#else
                         _x86OfsStack = (int)(_transitionBlock.OffsetOfArgs + SizeOfArgStack());
-#endif
                         break;
 
                     case TargetArchitecture.X64:
@@ -836,6 +820,13 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         _arm64IdxFPReg = 0;
                         break;
 
+                    case TargetArchitecture.LoongArch64:
+                        _loongarch64IdxGenReg = numRegistersUsed;
+                        _loongarch64OfsStack = 0;
+
+                        _loongarch64IdxFPReg = 0;
+                        break;
+
                     default:
                         throw new NotImplementedException();
                 }
@@ -880,13 +871,9 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         return _transitionBlock.OffsetOfArgumentRegisters + (_transitionBlock.NumArgumentRegisters - _x86NumRegistersUsed) * _transitionBlock.PointerSize;
                     }
 
-#if PROJECTN
-                    argOfs = _x86OfsStack;
-                    _x86OfsStack += _transitionBlock.StackElemSize(argSize);
-#else
                     _x86OfsStack -= _transitionBlock.StackElemSize(argSize);
                     argOfs = _x86OfsStack;
-#endif
+
                     Debug.Assert(argOfs >= _transitionBlock.OffsetOfArgs);
                     return argOfs;
 
@@ -1325,6 +1312,128 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         return argOfs;
                     }
 
+                case TargetArchitecture.LoongArch64:
+                    {
+                        int cFPRegs = 0;
+                        uint floatFieldFlags = (uint)StructFloatFieldInfoFlags.STRUCT_NO_FLOAT_FIELD;
+                        _hasArgLocDescForStructInRegs = false;
+
+                        switch (argType)
+                        {
+                            case CorElementType.ELEMENT_TYPE_R4:
+                                // 32-bit floating point argument.
+                                cFPRegs = 1;
+                                break;
+
+                            case CorElementType.ELEMENT_TYPE_R8:
+                                // 64-bit floating point argument.
+                                cFPRegs = 1;
+                                break;
+
+                            case CorElementType.ELEMENT_TYPE_VALUETYPE:
+                                {
+                                    // Composite greater than 16 bytes should be passed by reference
+                                    if (argSize > _transitionBlock.EnregisteredParamTypeMaxSize)
+                                    {
+                                        argSize = _transitionBlock.PointerSize;
+                                    }
+                                    else
+                                    {
+                                        floatFieldFlags = LoongArch64PassStructInRegister.GetLoongArch64PassStructInRegisterFlags(_argTypeHandle.GetRuntimeTypeHandle());
+                                        if ((floatFieldFlags & (uint)StructFloatFieldInfoFlags.STRUCT_FLOAT_FIELD_ONLY_TWO) != 0)
+                                        {
+                                            cFPRegs = 2;
+                                        }
+                                        else if ((floatFieldFlags & (uint)StructFloatFieldInfoFlags.STRUCT_HAS_FLOAT_FIELDS_MASK) != 0)
+                                        {
+                                            cFPRegs = 1;
+                                        }
+                                    }
+
+                                    break;
+                                }
+
+                            default:
+                                break;
+                        }
+
+                        bool isValueType = (argType == CorElementType.ELEMENT_TYPE_VALUETYPE);
+                        int cbArg = _transitionBlock.StackElemSize(argSize, isValueType, false);
+
+                        if (cFPRegs > 0 && !IsVarArg)
+                        {
+                            if (isValueType && ((floatFieldFlags & (uint)StructFloatFieldInfoFlags.STRUCT_HAS_ONE_FLOAT_MASK) != 0))
+                            {
+                                Debug.Assert(cFPRegs == 1);
+                                if ((_loongarch64IdxFPReg < 8) && (_loongarch64IdxGenReg < 8))
+                                {
+                                    _argLocDescForStructInRegs = new ArgLocDesc();
+                                    _argLocDescForStructInRegs.m_idxFloatReg = _loongarch64IdxFPReg;
+                                    _argLocDescForStructInRegs.m_cFloatReg = 1;
+
+                                    _argLocDescForStructInRegs.m_idxGenReg = _loongarch64IdxGenReg;
+                                    _argLocDescForStructInRegs.m_cGenReg = 1;
+
+                                    _hasArgLocDescForStructInRegs = true;
+                                    _argLocDescForStructInRegs.m_floatFlags = floatFieldFlags;
+
+                                    int argOfsInner = _transitionBlock.OffsetOfFloatArgumentRegisters + _loongarch64IdxFPReg * 8;
+                                    _loongarch64IdxFPReg++;
+                                    _loongarch64IdxGenReg++;
+                                    return argOfsInner;
+                                }
+                            }
+                            else if (cFPRegs + _loongarch64IdxFPReg <= 8)
+                            {
+                                // Each floating point register in the argument area is 8 bytes.
+                                int argOfsInner = _transitionBlock.OffsetOfFloatArgumentRegisters + _loongarch64IdxFPReg * 8;
+                                if (floatFieldFlags == (uint)StructFloatFieldInfoFlags.STRUCT_FLOAT_FIELD_ONLY_TWO)
+                                {
+                                    // struct with two single-float fields.
+                                    _argLocDescForStructInRegs = new ArgLocDesc();
+                                    _argLocDescForStructInRegs.m_idxFloatReg = _loongarch64IdxFPReg;
+                                    _argLocDescForStructInRegs.m_cFloatReg = 2;
+                                    Debug.Assert(cFPRegs == 2);
+                                    Debug.Assert(argSize == 8);
+
+                                    _hasArgLocDescForStructInRegs = true;
+                                    _argLocDescForStructInRegs.m_floatFlags = (uint)StructFloatFieldInfoFlags.STRUCT_FLOAT_FIELD_ONLY_TWO;
+                                }
+                                _loongarch64IdxFPReg += cFPRegs;
+                                return argOfsInner;
+                            }
+                            else
+                            {
+                                _loongarch64IdxFPReg = 8;
+                            }
+                        }
+
+                        {
+                            Debug.Assert((cbArg % _transitionBlock.PointerSize) == 0);
+
+                            int regSlots = ALIGN_UP(cbArg, _transitionBlock.PointerSize) / _transitionBlock.PointerSize;
+                            // Only R4-R11 are valid argument registers.
+                            if (_loongarch64IdxGenReg + regSlots <= 8)
+                            {
+                                // The entirety of the arg fits in the register slots.
+                                int argOfsInner = _transitionBlock.OffsetOfArgumentRegisters + _loongarch64IdxGenReg * 8;
+                                _loongarch64IdxGenReg += regSlots;
+                                return argOfsInner;
+                            }
+                            else if (_loongarch64IdxGenReg < 8)
+                            {
+                                int argOfsInner = _transitionBlock.OffsetOfArgumentRegisters + _loongarch64IdxGenReg * 8;
+                                _loongarch64IdxGenReg = 8;
+                                _loongarch64OfsStack += 8;
+                                return argOfsInner;
+                            }
+                        }
+
+                        argOfs = _transitionBlock.OffsetOfArgs + _loongarch64OfsStack;
+                        _loongarch64OfsStack += cbArg;
+                        return argOfs;
+                    }
+
                 default:
                     throw new NotImplementedException();
             }
@@ -1384,25 +1493,9 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
 
                 if (HasRetBuffArg() && _transitionBlock.IsRetBuffPassedAsFirstArg)
                 {
-#if PROJECTN
-                    // On ProjectN ret buff arg is passed on the call stack as the top stack arg
-                    nSizeOfArgStack += _transitionBlock.PointerSize;
-#else
                     numRegistersUsed++;
-#endif
                 }
 
-#if PROJECTN
-                // DESKTOP BEHAVIOR - This block is disabled for x86 as the param arg is the last argument on .NET Framework x86.
-                if (HasParamType)
-                {
-                    numRegistersUsed++;
-                    _paramTypeLoc = (numRegistersUsed == 1) ?
-                        ParamTypeLocation.Ecx : ParamTypeLocation.Edx;
-                    Debug.Assert(numRegistersUsed <= 2);
-                }
-#endif
-
                 if (IsVarArg)
                 {
                     nSizeOfArgStack += _transitionBlock.PointerSize;
@@ -1448,7 +1541,6 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                     }
                 }
 
-#if !PROJECTN
                 if (HasParamType)
                 {
                     if (numRegistersUsed < _transitionBlock.NumArgumentRegisters)
@@ -1463,7 +1555,6 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         _paramTypeLoc = ParamTypeLocation.Stack;
                     }
                 }
-#endif
             }
             else
             {
@@ -1606,6 +1697,48 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
 
                         int byteArgSize = GetArgSize();
 
+                        // On ARM64 some composites are implicitly passed by reference.
+                        if (IsArgPassedByRef())
+                        {
+                            byteArgSize = _transitionBlock.PointerSize;
+                        }
+
+                        if (!_transitionBlock.IsStackArgumentOffset(argOffset))
+                        {
+                            pLoc.m_idxGenReg = _transitionBlock.GetArgumentIndexFromOffset(argOffset);
+                            pLoc.m_cGenReg = (short)(ALIGN_UP(byteArgSize, _transitionBlock.PointerSize) / _transitionBlock.PointerSize);
+                        }
+                        else
+                        {
+                            pLoc.m_byteStackIndex = _transitionBlock.GetStackArgumentByteIndexFromOffset(argOffset);
+                            pLoc.m_byteStackSize = _transitionBlock.StackElemSize(byteArgSize, IsValueType(), IsFloatHfa());
+                        }
+                        return pLoc;
+                    }
+
+                case TargetArchitecture.LoongArch64:
+                    {
+                        if (_hasArgLocDescForStructInRegs)
+                        {
+                            return _argLocDescForStructInRegs;
+                        }
+
+                        //        LIMITED_METHOD_CONTRACT;
+
+                        ArgLocDesc pLoc = new ArgLocDesc();
+
+                        if (_transitionBlock.IsFloatArgumentRegisterOffset(argOffset))
+                        {
+                            int floatRegOfsInBytes = argOffset - _transitionBlock.OffsetOfFloatArgumentRegisters;
+                            Debug.Assert((floatRegOfsInBytes % _transitionBlock.FloatRegisterSize) == 0);
+                            pLoc.m_idxFloatReg = floatRegOfsInBytes / _transitionBlock.FloatRegisterSize;
+                            pLoc.m_cFloatReg = 1;
+
+                            return pLoc;
+                        }
+
+                        int byteArgSize = GetArgSize();
+
                         // Composites greater than 16bytes are passed by reference
                         TypeHandle dummy;
                         if (GetArgType(out dummy) == CorElementType.ELEMENT_TYPE_VALUETYPE && GetArgSize() > _transitionBlock.EnregisteredParamTypeMaxSize)
@@ -1616,7 +1749,14 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         if (!_transitionBlock.IsStackArgumentOffset(argOffset))
                         {
                             pLoc.m_idxGenReg = _transitionBlock.GetArgumentIndexFromOffset(argOffset);
-                            pLoc.m_cGenReg = (short)(ALIGN_UP(byteArgSize, _transitionBlock.PointerSize) / _transitionBlock.PointerSize);
+                            if ((pLoc.m_idxGenReg == 7) && (byteArgSize > _transitionBlock.PointerSize))
+                            {
+                                pLoc.m_cGenReg = 1;
+                                pLoc.m_byteStackIndex = 0;
+                                pLoc.m_byteStackSize = 8;
+                            }
+                            else
+                                pLoc.m_cGenReg = (short)(ALIGN_UP(byteArgSize, _transitionBlock.PointerSize) / _transitionBlock.PointerSize);
                         }
                         else
                         {
@@ -1711,6 +1851,10 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
         private int _arm64OfsStack;         // Offset of next stack location to be assigned a value
         private int _arm64IdxFPReg;         // Next FP register to be assigned a value
 
+        private int _loongarch64IdxGenReg;  // Next general register to be assigned a value
+        private int _loongarch64OfsStack;   // Offset of next stack location to be assigned a value
+        private int _loongarch64IdxFPReg;   // Next FP register to be assigned a value
+
         // These are enum flags in CallingConventions.h, but that's really ugly in C#, so I've changed them to bools.
         private bool _ITERATION_STARTED; // Started iterating over arguments
         private bool _SIZE_OF_ARG_STACK_COMPUTED;
@@ -1739,7 +1883,6 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
         //        METHOD_INVOKE_NEEDS_ACTIVATION  = 0x0040,   // Flag used by ArgIteratorForMethodInvoke
 
         //        RETURN_FP_SIZE_SHIFT            = 8,        // The rest of the flags is cached value of GetFPReturnSize
-        //    };
 
         private void ComputeReturnFlags()
         {
