@@ -29,7 +29,7 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         X64UnixTransitionBlock.Instance;
 
                 case TargetArchitecture.ARM:
-                    if (target.Abi == TargetAbi.CoreRTArmel)
+                    if (target.Abi == TargetAbi.NativeAotArmel)
                     {
                         return Arm32ElTransitionBlock.Instance;
                     }
@@ -43,6 +43,9 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                         AppleArm64TransitionBlock.Instance :
                         Arm64TransitionBlock.Instance;
 
+                case TargetArchitecture.LoongArch64:
+                    return LoongArch64TransitionBlock.Instance;
+
                 default:
                     throw new NotImplementedException(target.Architecture.ToString());
             }
@@ -60,6 +63,7 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
         public bool IsX64 => Architecture == TargetArchitecture.X64;
         public bool IsARM => Architecture == TargetArchitecture.ARM;
         public bool IsARM64 => Architecture == TargetArchitecture.ARM64;
+        public bool IsLoongArch64 => Architecture == TargetArchitecture.LoongArch64;
 
         /// <summary>
         /// This property is only overridden in AMD64 Unix variant of the transition block.
@@ -205,22 +209,6 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                     case CorElementType.ELEMENT_TYPE_SZARRAY:
                         pNumRegistersUsed++;
                         return true;
-#if PROJECTN
-                    case CorElementType.ELEMENT_TYPE_VALUETYPE:
-                        {
-                            // On ProjectN valuetypes of integral size are passed enregistered
-                            int structSize = TypeHandle.GetElemSize(typ, thArgType);
-                            switch (structSize)
-                            {
-                                case 1:
-                                case 2:
-                                case 4:
-                                    pNumRegistersUsed++;
-                                    return true;
-                            }
-                            break;
-                        }
-#elif READYTORUN
                     case CorElementType.ELEMENT_TYPE_VALUETYPE:
                         if (IsTrivialPointerSizedStruct(thArgType))
                         {
@@ -228,7 +216,6 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                             return true;
                         }
                         break;
-#endif
                 }
             }
 
@@ -396,7 +383,12 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                             }
 
                             if (size <= EnregisteredReturnTypeIntegerMaxSize)
+                            {
+                                if (IsLoongArch64)
+                                    fpReturnSize = LoongArch64PassStructInRegister.GetLoongArch64PassStructInRegisterFlags(thRetType.GetRuntimeTypeHandle()) & 0xff;
                                 break;
+                            }
+
                         }
                     }
 
@@ -462,11 +454,7 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
             /// </summary>
             public override int GetRetBuffArgOffset(bool hasThis)
             {
-#if PROJECTN
-                return OffsetOfArgs;
-#else
                 return hasThis ? X86Constants.OffsetOfEdx : X86Constants.OffsetOfEcx;
-#endif
             }
 
             public override int StackElemSize(int parmSize, bool isValueType = false, bool isFloatHfa = false)
@@ -650,5 +638,50 @@ namespace ILCompiler.DependencyAnalysis.ReadyToRun
                 return base.StackElemSize(parmSize, isValueType, isFloatHfa);
             }
         }
+
+        private class LoongArch64TransitionBlock : TransitionBlock
+        {
+            public static TransitionBlock Instance = new LoongArch64TransitionBlock();
+            public override TargetArchitecture Architecture => TargetArchitecture.LoongArch64;
+            public override int PointerSize => 8;
+            public override int FloatRegisterSize => 8; // TODO: for SIMD.
+            // R4(=A0) .. R11(=A7)
+            public override int NumArgumentRegisters => 8;
+            // fp=R22,ra=R1,s0-s8(R23-R31),tp=R2
+            public override int NumCalleeSavedRegisters => 12;
+            // Callee-saves, argument registers
+            public override int SizeOfTransitionBlock => SizeOfCalleeSavedRegisters + SizeOfArgumentRegisters;
+            public override int OffsetOfFirstGCRefMapSlot => SizeOfCalleeSavedRegisters;
+            public override int OffsetOfArgumentRegisters => OffsetOfFirstGCRefMapSlot;
+
+            // F0..F7
+            public override int OffsetOfFloatArgumentRegisters => 8 * sizeof(double);
+            public override int EnregisteredParamTypeMaxSize => 16;
+            public override int EnregisteredReturnTypeIntegerMaxSize => 16;
+
+            public override bool IsArgPassedByRef(TypeHandle th)
+            {
+                Debug.Assert(!th.IsNull());
+                Debug.Assert(th.IsValueType());
+
+                // Composites greater than 16 bytes are passed by reference
+                if (th.GetSize() > EnregisteredParamTypeMaxSize)
+                {
+                    return true;
+                }
+                else
+                {
+                    return false;
+                }
+            }
+
+            public sealed override int GetRetBuffArgOffset(bool hasThis) => OffsetOfFirstGCRefMapSlot + (hasThis ? 8 : 0);
+
+            public override int StackElemSize(int parmSize, bool isValueType = false, bool isFloatHfa = false)
+            {
+                int stackSlotSize = 8;
+                return ALIGN_UP(parmSize, stackSlotSize);
+            }
+        }
     }
 }
