@@ -105,7 +105,7 @@ namespace ILCompiler.Reflection.ReadyToRun
         /// <summary>
         /// The relative virtual address to the end of the code block
         /// </summary>
-        public int EndAddress { get;  }
+        public int EndAddress { get; }
 
         /// <summary>
         /// The size of the code block in bytes
@@ -225,9 +225,13 @@ namespace ILCompiler.Reflection.ReadyToRun
             {
                 return (int)arm64Info.FunctionLength;
             }
+            else if (UnwindInfo is LoongArch64.UnwindInfo loongarch64Info)
+            {
+                return (int)loongarch64Info.FunctionLength;
+            }
             else if (Method.GcInfo != null)
             {
-                return Method.GcInfo.CodeLength; 
+                return Method.GcInfo.CodeLength;
             }
             else
             {
@@ -306,6 +310,7 @@ namespace ILCompiler.Reflection.ReadyToRun
             {
                 this._runtimeFunctions = new List<RuntimeFunction>();
                 this.ParseRuntimeFunctions(false);
+                _readyToRunReader.ValidateRuntimeFunctions(_runtimeFunctions);
             }
         }
 
@@ -313,7 +318,7 @@ namespace ILCompiler.Reflection.ReadyToRun
         /// The id of the entrypoint runtime function
         /// </summary>
         public int EntryPointRuntimeFunctionId { get; set; }
-
+        public int ColdRuntimeFunctionId { get; set; }
         public int GcInfoRva { get; set; }
 
         public BaseGcInfo GcInfo
@@ -357,6 +362,7 @@ namespace ILCompiler.Reflection.ReadyToRun
         public string[] InstanceArgs { get; set; }
 
         public int RuntimeFunctionCount { get; set; }
+        public int ColdRuntimeFunctionCount { get; set; }
 
         /// <summary>
         /// Extracts the method signature from the metadata by rid
@@ -465,7 +471,7 @@ namespace ILCompiler.Reflection.ReadyToRun
                 {
                     sb.Append(", ");
                 }
-                sb.AppendFormat($"{Signature.ParameterTypes[i]}");
+                sb.Append($"{Signature.ParameterTypes[i]}");
             }
             sb.Append(")");
 
@@ -486,7 +492,7 @@ namespace ILCompiler.Reflection.ReadyToRun
                     }
                     else
                     {
-                        // Arm and Arm64 use the same GcInfo format as Amd64
+                        // Arm, Arm64 and LoongArch64 use the same GcInfo format as Amd64
                         _gcInfo = new Amd64.GcInfo(_readyToRunReader.Image, gcInfoOffset, _readyToRunReader.Machine, _readyToRunReader.ReadyToRunHeader.MajorVersion);
                     }
                 }
@@ -554,13 +560,22 @@ namespace ILCompiler.Reflection.ReadyToRun
         private void ParseRuntimeFunctions(bool partial)
         {
             int runtimeFunctionId = EntryPointRuntimeFunctionId;
+            int coldRuntimeFunctionId = ColdRuntimeFunctionId;
             int runtimeFunctionSize = _readyToRunReader.CalculateRuntimeFunctionSize();
             int runtimeFunctionOffset = _readyToRunReader.CompositeReader.GetOffset(_readyToRunReader.ReadyToRunHeader.Sections[ReadyToRunSectionType.RuntimeFunctions].RelativeVirtualAddress);
             int curOffset = runtimeFunctionOffset + runtimeFunctionId * runtimeFunctionSize;
+            int coldOffset = runtimeFunctionOffset + coldRuntimeFunctionId * runtimeFunctionSize;
             int codeOffset = 0;
 
+            int hotRuntimeFunctionCount = RuntimeFunctionCount - ColdRuntimeFunctionCount;
+
             for (int i = 0; i < RuntimeFunctionCount; i++)
             {
+                if (i == hotRuntimeFunctionCount)
+                {
+                    curOffset = coldOffset;
+                    runtimeFunctionId = coldRuntimeFunctionId;
+                }
                 int startRva = NativeReader.ReadInt32(_readyToRunReader.Image, ref curOffset);
                 if (_readyToRunReader.Machine == Machine.ArmThumb2)
                 {
@@ -593,6 +608,10 @@ namespace ILCompiler.Reflection.ReadyToRun
                 {
                     unwindInfo = new Arm64.UnwindInfo(_readyToRunReader.Image, unwindOffset);
                 }
+                else if (_readyToRunReader.Machine == Machine.LoongArch64)
+                {
+                    unwindInfo = new LoongArch64.UnwindInfo(_readyToRunReader.Image, unwindOffset);
+                }
 
                 if (i == 0 && unwindInfo != null)
                 {
@@ -625,7 +644,6 @@ namespace ILCompiler.Reflection.ReadyToRun
                 runtimeFunctionId++;
                 codeOffset += rtf.Size;
             }
-
             _size = codeOffset;
         }
     }
