@@ -9,12 +9,7 @@
 #include "sstring.h"
 #include "ex.h"
 
-// Config prefixes
-#define COMPLUS_PREFIX W("COMPlus_")
-#define LEN_OF_COMPLUS_PREFIX StrLen(COMPLUS_PREFIX)
-
-#define DOTNET_PREFIX W("DOTNET_")
-#define LEN_OF_DOTNET_PREFIX StrLen(DOTNET_PREFIX)
+#include "clrconfignocache.h"
 
 using ConfigDWORDInfo = CLRConfig::ConfigDWORDInfo;
 using ConfigStringInfo = CLRConfig::ConfigStringInfo;
@@ -148,12 +143,12 @@ namespace
 
         WCHAR buff[64];
         const WCHAR* fallbackPrefix = NULL;
-        const size_t namelen = wcslen(name);
+        const size_t namelen = u16_strlen(name);
 
         bool noPrefix = CheckLookupOption(options, LookupOptions::DontPrependPrefix);
         if (noPrefix)
         {
-            if (namelen >= _countof(buff))
+            if (namelen >= ARRAY_SIZE(buff))
             {
                 _ASSERTE(!"Environment variable name too long.");
                 return NULL;
@@ -163,8 +158,8 @@ namespace
         }
         else
         {
-            bool dotnetValid = namelen < (size_t)(_countof(buff) - 1 - LEN_OF_DOTNET_PREFIX);
-            bool complusValid = namelen < (size_t)(_countof(buff) - 1 - LEN_OF_COMPLUS_PREFIX);
+            bool dotnetValid = namelen < (size_t)(STRING_LENGTH(buff) - LEN_OF_DOTNET_PREFIX);
+            bool complusValid = namelen < (size_t)(STRING_LENGTH(buff) - LEN_OF_COMPLUS_PREFIX);
             if(!dotnetValid || !complusValid)
             {
                 _ASSERTE(!"Environment variable name too long.");
@@ -176,11 +171,11 @@ namespace
                 return NULL;
 
             // Priority order is DOTNET_ and then COMPlus_.
-            wcscpy_s(buff, _countof(buff), DOTNET_PREFIX);
+            wcscpy_s(buff, ARRAY_SIZE(buff), DOTNET_PREFIX);
             fallbackPrefix = COMPLUS_PREFIX;
         }
 
-        wcscat_s(buff, _countof(buff), name);
+        wcscat_s(buff, ARRAY_SIZE(buff), name);
 
         FAULT_NOT_FATAL(); // We don't report OOM errors here, we return a default value.
 
@@ -193,13 +188,25 @@ namespace
             DWORD len = WszGetEnvironmentVariable(buff, temp);
             if (len == 0 && fallbackPrefix != NULL)
             {
-                wcscpy_s(buff, _countof(buff), fallbackPrefix);
-                wcscat_s(buff, _countof(buff), name);
+                wcscpy_s(buff, ARRAY_SIZE(buff), fallbackPrefix);
+                wcscat_s(buff, ARRAY_SIZE(buff), name);
                 len = WszGetEnvironmentVariable(buff, temp);
             }
 
             if (len != 0)
+            {
                 ret = temp.GetCopyOfUnicodeString();
+
+#if defined(DEBUG) && !defined(SELF_NO_HOST)
+                // Validate the cache and no-cache logic result in the same answer
+                SString nameToConvert(name);
+
+                CLRConfigNoCache nonCache = CLRConfigNoCache::Get(nameToConvert.GetUTF8(), noPrefix);
+                LPCSTR valueNoCache = nonCache.AsString();
+
+                _ASSERTE(SString::_stricmp(valueNoCache, temp.GetUTF8()) == 0);
+#endif // defined(DEBUG) && !defined(SELF_NO_HOST)
+            }
         }
         EX_CATCH_HRESULT(hr);
 
@@ -217,7 +224,7 @@ namespace
     HRESULT GetConfigDWORD(
         LPCWSTR name,
         DWORD defValue,
-        __out DWORD *result,
+        _Out_ DWORD *result,
         LookupOptions options)
     {
         CONTRACTL
@@ -242,7 +249,7 @@ namespace
         {
             errno = 0;
             LPWSTR endPtr;
-            DWORD configMaybe = wcstoul(val, &endPtr, radix);
+            DWORD configMaybe = u16_strtoul(val, &endPtr, radix);
             BOOL fSuccess = ((errno != ERANGE) && (endPtr != val));
             if (fSuccess)
             {
@@ -301,7 +308,7 @@ namespace
     // Return Value:
     //     HRESULT indicating success or failure.
     //
-    HRESULT TrimWhiteSpace(LPCWSTR wszOrig, __deref_out_z LPWSTR * pwszTrimmed)
+    HRESULT TrimWhiteSpace(LPCWSTR wszOrig, _Outptr_result_z_ LPWSTR * pwszTrimmed)
     {
         CONTRACTL
         {
@@ -317,7 +324,7 @@ namespace
         *pwszTrimmed = NULL;
 
         // Get pointers into internal string that show where to do the trimming.
-        size_t cchOrig = wcslen(wszOrig);
+        size_t cchOrig = u16_strlen(wszOrig);
         if (!FitsIn<DWORD>(cchOrig))
             return COR_E_OVERFLOW;
         DWORD cchAfterTrim = (DWORD) cchOrig;
@@ -363,13 +370,13 @@ namespace
 // Creating structs using the macro table in CLRConfigValues.h
 //
 
-// These macros intialize ConfigDWORDInfo structs.
+// These macros initialize ConfigDWORDInfo structs.
 #define RETAIL_CONFIG_DWORD_INFO(symbol, name, defaultValue, description) \
     const CLRConfig::ConfigDWORDInfo CLRConfig::symbol = {name, defaultValue, CLRConfig::LookupOptions::Default};
 #define RETAIL_CONFIG_DWORD_INFO_EX(symbol, name, defaultValue, description, lookupOptions) \
     const CLRConfig::ConfigDWORDInfo CLRConfig::symbol = {name, defaultValue, lookupOptions};
 
-// These macros intialize ConfigStringInfo structs.
+// These macros initialize ConfigStringInfo structs.
 #define RETAIL_CONFIG_STRING_INFO(symbol, name, description) \
     const CLRConfig::ConfigStringInfo CLRConfig::symbol = {name, CLRConfig::LookupOptions::Default};
 #define RETAIL_CONFIG_STRING_INFO_EX(symbol, name, description, lookupOptions) \
@@ -405,6 +412,13 @@ namespace
 #undef CONFIG_DWORD_INFO_EX
 #undef CONFIG_STRING_INFO_EX
 
+BOOL CLRConfig::IsConfigEnabled(const ConfigDWORDInfo & info)
+{
+    WRAPPER_NO_CONTRACT;
+
+    return IsConfigOptionSpecified(info.name);
+}
+
 //
 // Look up a DWORD config value.
 //
@@ -469,6 +483,8 @@ DWORD CLRConfig::GetConfigValue(const ConfigDWORDInfo & info, DWORD defaultValue
 // static
 DWORD CLRConfig::GetConfigValue(const ConfigDWORDInfo & info)
 {
+    WRAPPER_NO_CONTRACT;
+
     bool unused;
     return GetConfigValue(info, &unused);
 }
@@ -517,7 +533,7 @@ LPWSTR CLRConfig::GetConfigValue(const ConfigStringInfo & info)
 //         not found.
 //
 // static
-HRESULT CLRConfig::GetConfigValue(const ConfigStringInfo & info, __deref_out_z LPWSTR * outVal)
+HRESULT CLRConfig::GetConfigValue(const ConfigStringInfo & info, _Outptr_result_z_ LPWSTR * outVal)
 {
     CONTRACT(HRESULT) {
         NOTHROW;
@@ -589,7 +605,7 @@ BOOL CLRConfig::IsConfigOptionSpecified(LPCWSTR name)
 // Deallocation function for code:CLRConfig::FreeConfigString
 //
 // static
-void CLRConfig::FreeConfigString(__in_z LPWSTR str)
+void CLRConfig::FreeConfigString(_In_z_ LPWSTR str)
 {
     LIMITED_METHOD_CONTRACT;
 
@@ -614,8 +630,8 @@ void CLRConfig::Initialize()
     if (CLRConfig::GetConfigValue(CLRConfig::EXTERNAL_DisableConfigCache) != 0)
         return;
 
-    const WCHAR prefixC = towlower(COMPLUS_PREFIX[0]);
-    const WCHAR prefixD = towlower(DOTNET_PREFIX[0]);
+    const WCHAR prefixC = (WCHAR)towlower(COMPLUS_PREFIX[0]);
+    const WCHAR prefixD = (WCHAR)towlower(DOTNET_PREFIX[0]);
 
     // Create a cache of environment variables
     WCHAR* wszStrings = GetEnvironmentStringsW();
@@ -625,7 +641,7 @@ void CLRConfig::Initialize()
         // null terminated strings
         for(WCHAR *wszCurr = wszStrings; *wszCurr; wszCurr++)
         {
-            WCHAR wch = towlower(*wszCurr);
+            WCHAR wch = (WCHAR)towlower(*wszCurr);
 
             // Lets only cache env variables with targeted prefixes
             bool matchC = wch == prefixC;
