@@ -8,9 +8,6 @@
 
 #ifdef DECLARE_DATA
 #include "asmconstants.h"
-#ifdef FEATURE_PREJIT
-#include "compile.h"
-#endif
 #endif
 
 //#define STUB_LOGGING
@@ -96,8 +93,8 @@ struct DispatchHolder;
 Monomorphic and mostly monomorphic call sites eventually point to DispatchStubs.
 A dispatch stub has an expected type (expectedMT), target address (target) and fail address (failure).
 If the calling frame does in fact have the <this> type be of the expected type, then
-control is transfered to the target address, the method implementation.  If not,
-then control is transfered to the fail address, a fail stub (see below) where a polymorphic
+control is transferred to the target address, the method implementation.  If not,
+then control is transferred to the fail address, a fail stub (see below) where a polymorphic
 lookup is done to find the correct address to go to.
 
 implementation note: Order, choice of instructions, and branch directions
@@ -106,7 +103,7 @@ attention needs to be paid to the effects on the BTB and branch prediction, both
 and in the large, i.e. it needs to run well in the face of BTB overflow--using static predictions.
 Note that since this stub is only used for mostly monomorphic callsites (ones that are not, get patched
 to something else), therefore the conditional jump "jne failure" is mostly not taken, and hence it is important
-that the branch prediction staticly predict this, which means it must be a forward jump.  The alternative
+that the branch prediction statically predict this, which means it must be a forward jump.  The alternative
 is to reverse the order of the jumps and make sure that the resulting conditional jump "je implTarget"
 is statically predicted as taken, i.e a backward jump. The current choice was taken since it was easier
 to control the placement of the stubs than control the placement of the jitted code and the stubs. */
@@ -201,7 +198,7 @@ transfers to the resolve piece (see ResolveStub).  The failEntryPoint decrements
 every time it is entered.  The ee at various times will add a large chunk to the counter.
 
 ResolveEntry - does a lookup via in a cache by hashing the actual type of the calling frame s
-<this> and the token identifying the (contract,method) pair desired.  If found, control is transfered
+<this> and the token identifying the (contract,method) pair desired.  If found, control is transferred
 to the method implementation.  If not found in the cache, the token is pushed and the ee is entered via
 the ResolveWorkerStub to do a full lookup and eventual transfer to the correct method implementation.  Since
 there is a different resolve stub for every token, the token can be inlined and the token can be pre-hashed.
@@ -427,7 +424,6 @@ void VTableCallHolder::Initialize(unsigned slot)
 {
     unsigned offsetOfIndirection = MethodTable::GetVtableOffset() + MethodTable::GetIndexOfVtableIndirection(slot) * TARGET_POINTER_SIZE;
     unsigned offsetAfterIndirection = MethodTable::GetIndexAfterVtableIndirection(slot) * TARGET_POINTER_SIZE;
-    _ASSERTE(MethodTable::VTableIndir_t::isRelative == false /* TODO: NYI */);
 
     VTableCallStub* pStub = stub();
     BYTE* p = (BYTE*)(pStub->entryPoint() & ~THUMB_CODE);
@@ -487,67 +483,6 @@ void VTableCallHolder::Initialize(unsigned slot)
 
 #endif // DACCESS_COMPILE
 
-VirtualCallStubManager::StubKind VirtualCallStubManager::predictStubKind(PCODE stubStartAddress)
-{
-    SUPPORTS_DAC;
-#ifdef DACCESS_COMPILE
-
-    return SK_BREAKPOINT;  // Dac always uses the slower lookup
-
-#else
-
-    StubKind stubKind = SK_UNKNOWN;
-    TADDR pInstr = PCODEToPINSTR(stubStartAddress);
-
-    EX_TRY
-    {
-        // If stubStartAddress is completely bogus, then this might AV,
-        // so we protect it with SEH. An AV here is OK.
-        AVInRuntimeImplOkayHolder AVOkay;
-
-        WORD firstWord = *((WORD*) pInstr);
-
-        if (*((UINT32*)pInstr) == 0xc000f8d0)
-        {
-            // Confirm the thrid word belongs to the vtable stub pattern
-            WORD thirdWord = ((WORD*)pInstr)[2];
-            if (thirdWord == 0xf84d /* Part of str r0, [sp, #-4] */  ||
-                thirdWord == 0xf8dc /* Part of ldr r12, [r12 + offset] */)
-                stubKind = SK_VTABLECALL;
-        }
-
-        if (stubKind == SK_UNKNOWN)
-        {
-            //Assuming that RESOLVE_STUB_FIRST_WORD & DISPATCH_STUB_FIRST_WORD have same values
-            if (firstWord == DISPATCH_STUB_FIRST_WORD)
-            {
-                WORD thirdWord = ((WORD*)pInstr)[2];
-                if (thirdWord == 0xf84d)
-                {
-                    stubKind = SK_DISPATCH;
-                }
-                else if (thirdWord == 0xb460)
-                {
-                    stubKind = SK_RESOLVE;
-                }
-            }
-            else if (firstWord == 0xf8df)
-            {
-                stubKind = SK_LOOKUP;
-            }
-        }
-    }
-    EX_CATCH
-    {
-        stubKind = SK_UNKNOWN;
-    }
-    EX_END_CATCH(SwallowAllExceptions);
-
-    return stubKind;
-
-#endif // DACCESS_COMPILE
-}
-
 #endif //DECLARE_DATA
 
 #endif // _VIRTUAL_CALL_STUB_ARM_H
