@@ -8,6 +8,7 @@
 
 #include "binder.h"
 #include "ecall.h"
+#include "qcall.h"
 
 #include "field.h"
 #include "excep.h"
@@ -16,15 +17,10 @@
 #include "customattribute.h"
 #include "debugdebugger.h"
 #include "dllimport.h"
-#include "nativeoverlapped.h"
 #include "clrvarargs.h"
 #include "sigbuilder.h"
 #include "olevariant.h"
 
-#ifdef FEATURE_PREJIT
-#include "compile.h"
-#endif
-
 //
 // Retrieve structures from ID.
 //
@@ -65,7 +61,34 @@ PTR_MethodTable CoreLibBinder::LookupClassLocal(BinderClassID id)
 
     const CoreLibClassDescription *d = m_classDescriptions + (int)id;
 
-    pMT = ClassLoader::LoadTypeByNameThrowing(GetModule()->GetAssembly(), d->nameSpace, d->name).AsMethodTable();
+    LPCUTF8 nameSpace = d->nameSpace;
+    LPCUTF8 name = d->name;
+
+    LPCUTF8 nestedTypeMaybe = strchr(name, '+');
+    if (nestedTypeMaybe == NULL)
+    {
+        NameHandle nameHandle = NameHandle(nameSpace, name);
+        pMT = ClassLoader::LoadTypeByNameThrowing(GetModule()->GetAssembly(), &nameHandle).AsMethodTable();
+    }
+    else
+    {
+        // Handle the nested type scenario.
+        // The same NameHandle must be used to retain the scope to look for the nested type.
+        NameHandle nameHandle(GetModule(), mdtBaseType);
+
+        SString splitName(SString::Utf8, name, (COUNT_T)(nestedTypeMaybe - name));
+        nameHandle.SetName(nameSpace, splitName.GetUTF8());
+
+        // The side-effect of updating the scope in the NameHandle is the point of the call.
+        (void)ClassLoader::LoadTypeByNameThrowing(GetModule()->GetAssembly(), &nameHandle);
+
+        // Now load the nested type.
+        nameHandle.SetName(NULL, nestedTypeMaybe + 1);
+
+        // We don't support nested types in nested types.
+        _ASSERTE(strchr(nameHandle.GetName(), '+') == NULL);
+        pMT = ClassLoader::LoadTypeByNameThrowing(GetModule()->GetAssembly(), &nameHandle).AsMethodTable();
+    }
 
     _ASSERTE(pMT->GetModule() == GetModule());
 
@@ -259,11 +282,7 @@ Signature CoreLibBinder::GetTargetSignature(LPHARDCODEDMETASIG pHardcodedSig)
     }
     CONTRACTL_END
 
-#ifdef CROSSGEN_COMPILE
-    return GetModule()->m_pBinder->GetSignatureLocal(pHardcodedSig);
-#else
     return (&g_CoreLib)->GetSignatureLocal(pHardcodedSig);
-#endif
 }
 
 // Get the metasig, do a one-time conversion if necessary
@@ -387,7 +406,7 @@ Again:
 
 //------------------------------------------------------------------
 // Resolve type references in the hardcoded metasig.
-// Returns a new signature with type refences resolved.
+// Returns a new signature with type references resolved.
 //------------------------------------------------------------------
 void CoreLibBinder::BuildConvertedSignature(const BYTE* pSig, SigBuilder * pSigBuilder)
 {
@@ -414,7 +433,7 @@ void CoreLibBinder::BuildConvertedSignature(const BYTE* pSig, SigBuilder * pSigB
     }
     else {
         if ((callConv & IMAGE_CEE_CS_CALLCONV_MASK) != IMAGE_CEE_CS_CALLCONV_FIELD)
-            THROW_BAD_FORMAT(BFA_BAD_SIGNATURE, (Module*)NULL);
+            THROW_BAD_FORMAT(BFA_BAD_SIGNATURE, (ModuleBase*)NULL);
         argCount = 0;
     }
 
@@ -482,7 +501,7 @@ void CoreLibBinder::TriggerGCUnderStress()
 #ifndef DACCESS_COMPILE
     _ASSERTE (GetThreadNULLOk());
     TRIGGERSGC ();
-    // Force a GC here because GetClass could trigger GC nondeterminsticly
+    // Force a GC here because GetClass could trigger GC nondeterministicly
     if (g_pConfig->GetGCStressLevel() != 0)
     {
         DEBUG_ONLY_REGION();
@@ -516,7 +535,7 @@ void CoreLibBinder::Startup()
     s_SigConvertCrst.Init(CrstSigConvert);
 }
 
-#if defined(_DEBUG) && !defined(CROSSGEN_COMPILE)
+#if defined(_DEBUG)
 
 // NoClass is used to suppress check for unmanaged and managed size match
 #define NoClass char[USHRT_MAX]
@@ -540,7 +559,7 @@ void CoreLibBinder::Check()
 
     MethodTable * pMT = NULL;
 
-    for (unsigned i = 0; i < NumItems(OffsetsAndSizes); i++)
+    for (unsigned i = 0; i < ARRAY_SIZE(OffsetsAndSizes); i++)
     {
         const OffsetAndSizeCheck * p = OffsetsAndSizes + i;
 
@@ -566,7 +585,7 @@ void CoreLibBinder::Check()
         else
         if (p->fieldName != NULL)
         {
-            // This assert will fire if there is DEFINE_FIELD_U macro without preceeding DEFINE_CLASS_U macro in corelib.h
+            // This assert will fire if there is DEFINE_FIELD_U macro without preceding DEFINE_CLASS_U macro in corelib.h
             _ASSERTE(pMT != NULL);
 
             FieldDesc * pFD = MemberLoader::FindField(pMT, p->fieldName, NULL, 0, NULL);
@@ -808,8 +827,7 @@ static void FCallCheckSignature(MethodDesc* pMD, PCODE pImpl)
             size_t len = pUnmanagedTypeEnd - pUnmanagedArg;
             // generate the unmanaged argument signature to show them in the error message if possible
             StackSString ssUnmanagedType(SString::Ascii, pUnmanagedArg, (COUNT_T)len);
-            StackScratchBuffer buffer;
-            const char * pUnManagedType = ssUnmanagedType.GetANSI(buffer);
+            const char * pUnManagedType = ssUnmanagedType.GetUTF8();
 
             if (expectedType != NULL)
             {
@@ -829,11 +847,11 @@ static void FCallCheckSignature(MethodDesc* pMD, PCODE pImpl)
                 }
                 else if (argType == ELEMENT_TYPE_I4)
                 {
-                    bSigError = !IsStrInArray(pUnmanagedArg, len, aInt32Type, NumItems(aInt32Type));
+                    bSigError = !IsStrInArray(pUnmanagedArg, len, aInt32Type, ARRAY_SIZE(aInt32Type));
                 }
                 else if (argType == ELEMENT_TYPE_U4)
                 {
-                    bSigError = !IsStrInArray(pUnmanagedArg, len, aUInt32Type, NumItems(aUInt32Type));
+                    bSigError = !IsStrInArray(pUnmanagedArg, len, aUInt32Type, ARRAY_SIZE(aUInt32Type));
                 }
                 else if (argType == ELEMENT_TYPE_VALUETYPE)
                 {
@@ -842,7 +860,7 @@ static void FCallCheckSignature(MethodDesc* pMD, PCODE pImpl)
                 }
                 else
                 {
-                    bSigError = IsStrInArray(pUnmanagedArg, len, aType, NumItems(aType));
+                    bSigError = IsStrInArray(pUnmanagedArg, len, aType, ARRAY_SIZE(aType));
                 }
                 if (bSigError)
                 {
@@ -1038,19 +1056,25 @@ void CoreLibBinder::CheckExtended()
         else
         if (pMD->IsNDirect())
         {
-            NDirect::PopulateNDirectMethodDesc((NDirectMethodDesc *)pMD);
+            NDirectMethodDesc* pNMD = (NDirectMethodDesc*)pMD;
+            NDirect::PopulateNDirectMethodDesc(pNMD);
 
-            if (pMD->IsQCall())
+            if (pNMD->IsQCall() && QCallResolveDllImport(pNMD->GetEntrypointName()) == nullptr)
             {
-                id = ((NDirectMethodDesc *)pMD)->GetECallID();
-                if (id == 0) {
-                    id = ECall::GetIDForMethod(pMD);
+                LPCUTF8 pszClassName;
+                LPCUTF8 pszNameSpace;
+                if (FAILED(pInternalImport->GetNameOfTypeDef(tdClass, &pszClassName, &pszNameSpace)))
+                {
+                    pszClassName = pszNameSpace = "Invalid TypeDef record";
                 }
+                LPCUTF8 pszName;
+                if (FAILED(pInternalImport->GetNameOfMethodDef(td, &pszName)))
+                {
+                    pszName = "Invalid method name";
+                }
+                printf("CheckExtended: Unable to find qcall implementation: %s.%s::%s (EntryPoint name: %s)\n", pszNameSpace, pszClassName, pszName, pNMD->GetEntrypointName());
             }
-            else
-            {
-                continue;
-            }
+            continue;
         }
         else
         {
@@ -1106,7 +1130,7 @@ ErrExit:
     _ASSERTE(SUCCEEDED(hr));
 }
 
-#endif // _DEBUG && !CROSSGEN_COMPILE
+#endif // _DEBUG
 
 extern const CoreLibClassDescription c_rgCoreLibClassDescriptions[];
 extern const USHORT c_nCoreLibClassDescriptions;
@@ -1117,19 +1141,6 @@ extern const USHORT c_nCoreLibMethodDescriptions;
 extern const CoreLibFieldDescription c_rgCoreLibFieldDescriptions[];
 extern const USHORT c_nCoreLibFieldDescriptions;
 
-#ifdef CROSSGEN_COMPILE
-namespace CrossGenCoreLib
-{
-    extern const CoreLibClassDescription c_rgCoreLibClassDescriptions[];
-    extern const USHORT c_nCoreLibClassDescriptions;
-
-    extern const CoreLibMethodDescription c_rgCoreLibMethodDescriptions[];
-    extern const USHORT c_nCoreLibMethodDescriptions;
-
-    extern const CoreLibFieldDescription c_rgCoreLibFieldDescriptions[];
-    extern const USHORT c_nCoreLibFieldDescriptions;
-};
-#endif
 
 void CoreLibBinder::AttachModule(Module * pModule)
 {
@@ -1142,40 +1153,9 @@ void CoreLibBinder::AttachModule(Module * pModule)
         c_rgCoreLibMethodDescriptions, c_nCoreLibMethodDescriptions,
         c_rgCoreLibFieldDescriptions,  c_nCoreLibFieldDescriptions);
 
-#if defined(FEATURE_PREJIT) && !defined(CROSSGEN_COMPILE)
-    CoreLibBinder * pPersistedBinder = pModule->m_pBinder;
-
-    if (pPersistedBinder != NULL
-        // Do not use persisted binder for profiling native images. See comment in code:CoreLibBinder::Fixup.
-        && !(pModule->GetNativeImage()->GetNativeVersionInfo()->wConfigFlags  & CORCOMPILE_CONFIG_PROFILING))
-    {
-        pGlobalBinder->m_pClasses = pPersistedBinder->m_pClasses;
-        pGlobalBinder->m_pMethods = pPersistedBinder->m_pMethods;
-        pGlobalBinder->m_pFields = pPersistedBinder->m_pFields;
-
-        pModule->m_pBinder = pGlobalBinder;
-        return;
-    }
-#endif // FEATURE_PREJIT && CROSSGEN_COMPILE
-
     pGlobalBinder->AllocateTables();
 
-#ifdef CROSSGEN_COMPILE
-    CoreLibBinder * pTargetBinder = (CoreLibBinder *)(void *)
-        pModule->GetAssembly()->GetLowFrequencyHeap()
-            ->AllocMem(S_SIZE_T(sizeof(CoreLibBinder)));
-
-    pTargetBinder->SetDescriptions(pModule,
-        CrossGenCoreLib::c_rgCoreLibClassDescriptions,  CrossGenCoreLib::c_nCoreLibClassDescriptions,
-        CrossGenCoreLib::c_rgCoreLibMethodDescriptions, CrossGenCoreLib::c_nCoreLibMethodDescriptions,
-        CrossGenCoreLib::c_rgCoreLibFieldDescriptions,  CrossGenCoreLib::c_nCoreLibFieldDescriptions);
-
-    pTargetBinder->AllocateTables();
-
-    pModule->m_pBinder = pTargetBinder;
-#else
     pModule->m_pBinder = pGlobalBinder;
-#endif
 }
 
 void CoreLibBinder::SetDescriptions(Module * pModule,
@@ -1240,95 +1220,6 @@ PTR_MethodTable CoreLibBinder::LoadPrimitiveType(CorElementType et)
     return pMT;
 }
 
-#ifdef FEATURE_NATIVE_IMAGE_GENERATION
-void CoreLibBinder::BindAll()
-{
-    STANDARD_VM_CONTRACT;
-
-    for (BinderClassID cID = (BinderClassID) 1; cID < m_cClasses; cID = (BinderClassID) (cID + 1))
-    {
-        if (m_classDescriptions[cID].name != NULL) // Allow for CorSigElement entries with no classes
-            GetClassLocal(cID);
-    }
-
-    for (BinderMethodID mID = (BinderMethodID) 1; mID < m_cMethods; mID = (BinderMethodID) (mID + 1))
-        GetMethodLocal(mID);
-
-    for (BinderFieldID fID = (BinderFieldID) 1; fID < m_cFields; fID = (BinderFieldID) (fID + 1))
-        GetFieldLocal(fID);
-}
-
-void CoreLibBinder::Save(DataImage *image)
-{
-    STANDARD_VM_CONTRACT;
-
-    image->StoreStructure(this, sizeof(CoreLibBinder),
-                          DataImage::ITEM_BINDER);
-
-    image->StoreStructure(m_pClasses, m_cClasses * sizeof(*m_pClasses),
-                          DataImage::ITEM_BINDER_ITEMS);
-
-    image->StoreStructure(m_pMethods, m_cMethods * sizeof(*m_pMethods),
-                          DataImage::ITEM_BINDER_ITEMS);
-
-    image->StoreStructure(m_pFields, m_cFields * sizeof(*m_pFields),
-                          DataImage::ITEM_BINDER_ITEMS);
-}
-
-void CoreLibBinder::Fixup(DataImage *image)
-{
-    STANDARD_VM_CONTRACT;
-
-    image->FixupPointerField(this, offsetof(CoreLibBinder, m_pModule));
-
-    int i;
-
-    image->FixupPointerField(this, offsetof(CoreLibBinder, m_pClasses));
-    for (i = 1; i < m_cClasses; i++)
-    {
-#if _DEBUG
-        //
-        // We do not want to check for restore at runtime for performance reasons.
-        // If there is ever a case that requires restore, it should be special
-        // cased here and restored explicitly by GetClass/GetField/GetMethod caller.
-        //
-        // Profiling NGen images force restore for all types. We are still going to save
-        // the binder for nidump, but we are not going to use it at runtime.
-        //
-        if (m_pClasses[i] != NULL && !GetAppDomain()->ToCompilationDomain()->m_fForceProfiling)
-        {
-            _ASSERTE(!m_pClasses[i]->NeedsRestore(image));
-        }
-#endif
-        image->FixupPointerField(m_pClasses, i * sizeof(m_pClasses[0]));
-    }
-
-    image->FixupPointerField(this, offsetof(CoreLibBinder, m_pMethods));
-    for (i = 1; i < m_cMethods; i++)
-    {
-#if _DEBUG
-        // See comment above.
-        if (m_pMethods[i] != NULL && !GetAppDomain()->ToCompilationDomain()->m_fForceProfiling)
-        {
-            _ASSERTE(!m_pMethods[i]->NeedsRestore(image));
-        }
-#endif
-
-        image->FixupPointerField(m_pMethods, i * sizeof(m_pMethods[0]));
-    }
-
-    image->FixupPointerField(this, offsetof(CoreLibBinder, m_pFields));
-    for (i = 1; i < m_cFields; i++)
-    {
-        image->FixupPointerField(m_pFields, i * sizeof(m_pFields[0]));
-    }
-
-    image->ZeroPointerField(this, offsetof(CoreLibBinder, m_classDescriptions));
-    image->ZeroPointerField(this, offsetof(CoreLibBinder, m_methodDescriptions));
-    image->ZeroPointerField(this, offsetof(CoreLibBinder, m_fieldDescriptions));
-}
-#endif // FEATURE_NATIVE_IMAGE_GENERATION
-
 #endif // #ifndef DACCESS_COMPILE
 
 #ifdef DACCESS_COMPILE
