@@ -21,10 +21,9 @@
 // wszFullName is escaped (TYPE_NAME_RESERVED_CHAR). It should not be byref or contain enclosing type name,
 // assembly name, and generic argument list.
 //**************************************************
-mdTypeRef QCALLTYPE COMModule::GetTypeRef(QCall::ModuleHandle pModule,
+extern "C" mdTypeRef QCALLTYPE ModuleBuilder_GetTypeRef(QCall::ModuleHandle pModule,
                                           LPCWSTR wszFullName,
                                           QCall::ModuleHandle pRefedModule,
-                                          LPCWSTR wszRefedModuleFileName,
                                           INT32 tkResolutionArg)
 {
     QCALL_CONTRACT;
@@ -85,35 +84,25 @@ mdTypeRef QCALLTYPE COMModule::GetTypeRef(QCall::ModuleHandle pModule,
         else
         {
             // reference to top level type
-            if ( pThisAssembly != pRefedAssembly )
+
+            SafeComHolderPreemp<IMetaDataAssemblyEmit> pAssemblyEmit;
+
+            // Generate AssemblyRef
+            IfFailThrow( pEmit->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
+            tkResolution = pThisAssembly->AddAssemblyRef(pRefedAssembly, pAssemblyEmit);
+
+            // Add the assembly ref token and the manifest module it is referring to this module's rid map.
+            // This is needed regardless of whether the dynamic assembly has run access. Even in Save-only
+            // or Refleciton-only mode, CreateType() of the referencing type may still need the referenced
+            // type to be resolved and loaded, e.g. if the referencing type is a subclass of the referenced type.
+            //
+            // Don't cache if there is assembly associated with the token already. The assembly ref resolution
+            // can be ambiguous because of reflection emit does not require unique assembly names.
+            // We always let the first association win. Ideally, we would disallow this situation by throwing
+            // exception, but that would be a breaking change.
+            if(pModule->LookupAssemblyRef(tkResolution) == NULL)
             {
-                SafeComHolderPreemp<IMetaDataAssemblyEmit> pAssemblyEmit;
-
-                // Generate AssemblyRef
-                IfFailThrow( pEmit->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
-                tkResolution = pThisAssembly->AddAssemblyRef(pRefedAssembly, pAssemblyEmit);
-
-                // Add the assembly ref token and the manifest module it is referring to this module's rid map.
-                // This is needed regardless of whether the dynamic assembly has run access. Even in Save-only
-                // or Refleciton-only mode, CreateType() of the referencing type may still need the referenced
-                // type to be resolved and loaded, e.g. if the referencing type is a subclass of the referenced type.
-                //
-                // Don't cache if there is assembly associated with the token already. The assembly ref resolution
-                // can be ambiguous because of reflection emit does not require unique assembly names.
-                // We always let the first association win. Ideally, we would disallow this situation by throwing
-                // exception, but that would be a breaking change.
-                if(pModule->LookupAssemblyRef(tkResolution) == NULL)
-                {
-                    pModule->ForceStoreAssemblyRef(tkResolution, pRefedAssembly);
-                }
-            }
-            else
-            {
-                _ASSERTE(pModule != pRefedModule);
-                _ASSERTE(wszRefedModuleFileName != NULL);
-
-                // Generate ModuleRef
-                IfFailThrow(pEmit->DefineModuleRef(wszRefedModuleFileName, &tkResolution));
+                pModule->ForceStoreAssemblyRef(tkResolution, pRefedAssembly);
             }
         }
 
@@ -135,7 +124,7 @@ mdTypeRef QCALLTYPE COMModule::GetTypeRef(QCall::ModuleHandle pModule,
 **           int            tkTypeSpec
 **Exceptions:
 ==============================================================================*/
-INT32 QCALLTYPE COMModule::GetArrayMethodToken(QCall::ModuleHandle pModule,
+extern "C" INT32 QCALLTYPE ModuleBuilder_GetArrayMethodToken(QCall::ModuleHandle pModule,
                                                INT32 tkTypeSpec,
                                                LPCWSTR wszMethodName,
                                                LPCBYTE pSignature,
@@ -167,6 +156,46 @@ INT32 QCALLTYPE COMModule::GetArrayMethodToken(QCall::ModuleHandle pModule,
     return (INT32)memberRefE;
 }
 
+namespace
+{
+    //******************************************************************************
+    //
+    // Return a TypeRef token given a TypeDef token from the same emit scope
+    //
+    //******************************************************************************
+    void DefineTypeRefHelper(
+        IMetaDataEmit       *pEmit,         // given emit scope
+        mdTypeDef           td,             // given typedef in the emit scope
+        mdTypeRef           *ptr)           // return typeref
+    {
+        CONTRACTL  {
+            STANDARD_VM_CHECK;
+
+            PRECONDITION(CheckPointer(pEmit));
+            PRECONDITION(CheckPointer(ptr));
+        }
+        CONTRACTL_END;
+
+        CQuickBytes qb;
+        WCHAR* szTypeDef = (WCHAR*) qb.AllocThrows((MAX_CLASSNAME_LENGTH+1) * sizeof(WCHAR));
+        mdToken             rs;             // resolution scope
+        DWORD               dwFlags;
+
+        SafeComHolder<IMetaDataImport> pImport;
+        IfFailThrow( pEmit->QueryInterface(IID_IMetaDataImport, (void **)&pImport) );
+        IfFailThrow( pImport->GetTypeDefProps(td, szTypeDef, MAX_CLASSNAME_LENGTH, NULL, &dwFlags, NULL) );
+        if ( IsTdNested(dwFlags) )
+        {
+            mdToken         tdNested;
+            IfFailThrow( pImport->GetNestedClassProps(td, &tdNested) );
+            DefineTypeRefHelper( pEmit, tdNested, &rs);
+        }
+        else
+            rs = TokenFromRid( 1, mdtModule );
+
+        IfFailThrow( pEmit->DefineTypeRefByName( rs, szTypeDef, ptr) );
+    }   // DefineTypeRefHelper
+}
 
 //******************************************************************************
 //
@@ -174,7 +203,7 @@ INT32 QCALLTYPE COMModule::GetArrayMethodToken(QCall::ModuleHandle pModule,
 // This function will return a MemberRef token given a MethodDef token and the module where the MethodDef/FieldDef is defined.
 //
 //******************************************************************************
-INT32 QCALLTYPE COMModule::GetMemberRef(QCall::ModuleHandle pModule, QCall::ModuleHandle pRefedModule, INT32 tr, INT32 token)
+extern "C" INT32 QCALLTYPE ModuleBuilder_GetMemberRef(QCall::ModuleHandle pModule, QCall::ModuleHandle pRefedModule, INT32 tr, INT32 token)
 {
     QCALL_CONTRACT;
 
@@ -216,13 +245,13 @@ INT32 QCALLTYPE COMModule::GetMemberRef(QCall::ModuleHandle pModule, QCall::Modu
     }
 
     SafeComHolderPreemp<IMetaDataAssemblyEmit> pAssemblyEmit;
-    IfFailThrow( pRefingAssembly->GetManifestModule()->GetEmitter()->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
+    IfFailThrow( pRefingAssembly->GetModule()->GetEmitter()->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
 
     CQuickBytes             qbNewSig;
     ULONG                   cbNewSig;
 
     IfFailThrow( pRefedModule->GetMDImport()->TranslateSigWithScope(
-        pRefedAssembly->GetManifestImport(),
+        pRefedAssembly->GetMDImport(),
         NULL, 0,        // hash value
         pvComSig,
         cbComSig,
@@ -250,52 +279,12 @@ INT32 QCALLTYPE COMModule::GetMemberRef(QCall::ModuleHandle pModule, QCall::Modu
     return (INT32)memberRefE;
 }
 
-
-//******************************************************************************
-//
-// Return a TypeRef token given a TypeDef token from the same emit scope
-//
-//******************************************************************************
-void COMModule::DefineTypeRefHelper(
-    IMetaDataEmit       *pEmit,         // given emit scope
-    mdTypeDef           td,             // given typedef in the emit scope
-    mdTypeRef           *ptr)           // return typeref
-{
-    CONTRACTL  {
-        STANDARD_VM_CHECK;
-
-        PRECONDITION(CheckPointer(pEmit));
-        PRECONDITION(CheckPointer(ptr));
-    }
-    CONTRACTL_END;
-
-    CQuickBytes qb;
-    WCHAR* szTypeDef = (WCHAR*) qb.AllocThrows((MAX_CLASSNAME_LENGTH+1) * sizeof(WCHAR));
-    mdToken             rs;             // resolution scope
-    DWORD               dwFlags;
-
-    SafeComHolder<IMetaDataImport> pImport;
-    IfFailThrow( pEmit->QueryInterface(IID_IMetaDataImport, (void **)&pImport) );
-    IfFailThrow( pImport->GetTypeDefProps(td, szTypeDef, MAX_CLASSNAME_LENGTH, NULL, &dwFlags, NULL) );
-    if ( IsTdNested(dwFlags) )
-    {
-        mdToken         tdNested;
-        IfFailThrow( pImport->GetNestedClassProps(td, &tdNested) );
-        DefineTypeRefHelper( pEmit, tdNested, &rs);
-    }
-    else
-        rs = TokenFromRid( 1, mdtModule );
-
-    IfFailThrow( pEmit->DefineTypeRefByName( rs, szTypeDef, ptr) );
-}   // DefineTypeRefHelper
-
-
 //******************************************************************************
 //
 // Return a MemberRef token given a RuntimeMethodInfo
 //
 //******************************************************************************
-INT32 QCALLTYPE COMModule::GetMemberRefOfMethodInfo(QCall::ModuleHandle pModule, INT32 tr, MethodDesc * pMeth)
+extern "C" INT32 QCALLTYPE ModuleBuilder_GetMemberRefOfMethodInfo(QCall::ModuleHandle pModule, INT32 tr, MethodDesc * pMeth)
 {
     QCALL_CONTRACT;
 
@@ -335,7 +324,7 @@ INT32 QCALLTYPE COMModule::GetMemberRefOfMethodInfo(QCall::ModuleHandle pModule,
         Assembly * pRefingAssembly = pModule->GetAssembly();
 
         SafeComHolderPreemp<IMetaDataAssemblyEmit> pAssemblyEmit;
-        IfFailThrow( pRefingAssembly->GetManifestModule()->GetEmitter()->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
+        IfFailThrow( pRefingAssembly->GetModule()->GetEmitter()->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
 
         CQuickBytes     qbNewSig;
         ULONG           cbNewSig;
@@ -349,7 +338,7 @@ INT32 QCALLTYPE COMModule::GetMemberRefOfMethodInfo(QCall::ModuleHandle pModule,
         }
 
         IfFailThrow( pMeth->GetMDImport()->TranslateSigWithScope(
-            pRefedAssembly->GetManifestImport(),
+            pRefedAssembly->GetMDImport(),
             NULL, 0,        // hash blob value
             pvComSig,
             cbComSig,
@@ -376,7 +365,7 @@ INT32 QCALLTYPE COMModule::GetMemberRefOfMethodInfo(QCall::ModuleHandle pModule,
 // Return a MemberRef token given a RuntimeFieldInfo
 //
 //******************************************************************************
-mdMemberRef QCALLTYPE COMModule::GetMemberRefOfFieldInfo(QCall::ModuleHandle pModule, mdTypeDef tr, QCall::TypeHandle th, mdFieldDef tkField)
+extern "C" mdMemberRef QCALLTYPE ModuleBuilder_GetMemberRefOfFieldInfo(QCall::ModuleHandle pModule, mdTypeDef tr, QCall::TypeHandle th, mdFieldDef tkField)
 {
     QCALL_CONTRACT;
 
@@ -421,14 +410,14 @@ mdMemberRef QCALLTYPE COMModule::GetMemberRefOfFieldInfo(QCall::ModuleHandle pMo
                 COMPlusThrow(kNotSupportedException, W("NotSupported_CollectibleBoundNonCollectible"));
         }
         SafeComHolderPreemp<IMetaDataAssemblyEmit> pAssemblyEmit;
-        IfFailThrow( pRefingAssembly->GetManifestModule()->GetEmitter()->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
+        IfFailThrow( pRefingAssembly->GetModule()->GetEmitter()->QueryInterface(IID_IMetaDataAssemblyEmit, (void **) &pAssemblyEmit) );
 
         // Translate the field signature this scope
         CQuickBytes     qbNewSig;
         ULONG           cbNewSig;
 
         IfFailThrow( pRefedMDImport->TranslateSigWithScope(
-        pRefedAssembly->GetManifestImport(),
+        pRefedAssembly->GetMDImport(),
         NULL, 0,            // hash value
         pvComSig,
         cbComSig,
@@ -450,7 +439,7 @@ mdMemberRef QCALLTYPE COMModule::GetMemberRefOfFieldInfo(QCall::ModuleHandle pMo
 // Return a MemberRef token given a Signature
 //
 //******************************************************************************
-INT32 QCALLTYPE COMModule::GetMemberRefFromSignature(QCall::ModuleHandle pModule,
+extern "C" INT32 QCALLTYPE ModuleBuilder_GetMemberRefFromSignature(QCall::ModuleHandle pModule,
                                                      INT32 tr,
                                                      LPCWSTR wszMemberName,
                                                      LPCBYTE pSignature,
@@ -482,7 +471,7 @@ INT32 QCALLTYPE COMModule::GetMemberRefFromSignature(QCall::ModuleHandle pModule
 // This function is used to set the FieldRVA with the content data
 //
 //******************************************************************************
-void QCALLTYPE COMModule::SetFieldRVAContent(QCall::ModuleHandle pModule, INT32 tkField, LPCBYTE pContent, INT32 length)
+extern "C" void QCALLTYPE ModuleBuilder_SetFieldRVAContent(QCall::ModuleHandle pModule, INT32 tkField, LPCBYTE pContent, INT32 length)
 {
     QCALL_CONTRACT;
 
@@ -499,14 +488,20 @@ void QCALLTYPE COMModule::SetFieldRVAContent(QCall::ModuleHandle pModule, INT32
     if (pReflectionModule->m_sdataSection == 0)
         IfFailThrow( pGen->GetSectionCreate (".sdata", sdReadWrite, &pReflectionModule->m_sdataSection) );
 
+    // Define the alignment that the rva will be set to. Since the CoreCLR runtime only has hard alignment requirements
+    // up to 8 bytes, the highest alignment we may need is 8 byte alignment. This hard alignment requirement is only needed
+    // by Runtime.Helpers.CreateSpan<T>. Since the previous alignment was 4 bytes before CreateSpan was implemented, if the
+    // data isn't itself of size divisible by 8, just align to 4 to the memory cost of excess alignment.
+    DWORD alignment = (length % 8 == 0) ? 8 : 4;
+
     // Get the size of current .sdata section. This will be the RVA for this field within the section
     DWORD dwRVA = 0;
     IfFailThrow( pGen->GetSectionDataLen(pReflectionModule->m_sdataSection, &dwRVA) );
-    dwRVA = (dwRVA + sizeof(DWORD)-1) & ~(sizeof(DWORD)-1);
+    dwRVA = (dwRVA + alignment-1) & ~(alignment-1);
 
     // allocate the space in .sdata section
     void * pvBlob;
-    IfFailThrow( pGen->GetSectionBlock(pReflectionModule->m_sdataSection, length, sizeof(DWORD), (void**) &pvBlob) );
+    IfFailThrow( pGen->GetSectionBlock(pReflectionModule->m_sdataSection, length, alignment, (void**) &pvBlob) );
 
     // copy over the initialized data if specified
     if (pContent != NULL)
@@ -532,7 +527,7 @@ void QCALLTYPE COMModule::SetFieldRVAContent(QCall::ModuleHandle pModule, INT32
 //  string constant or return the token of an existing constant.
 //
 //******************************************************************************
-mdString QCALLTYPE COMModule::GetStringConstant(QCall::ModuleHandle pModule, LPCWSTR pwzValue, INT32 iLength)
+extern "C" mdString QCALLTYPE ModuleBuilder_GetStringConstant(QCall::ModuleHandle pModule, LPCWSTR pwzValue, INT32 iLength)
 {
     QCALL_CONTRACT;
 
@@ -559,7 +554,7 @@ mdString QCALLTYPE COMModule::GetStringConstant(QCall::ModuleHandle pModule, LPC
 /*=============================SetModuleName====================================
 // SetModuleName
 ==============================================================================*/
-void QCALLTYPE COMModule::SetModuleName(QCall::ModuleHandle pModule, LPCWSTR wszModuleName)
+extern "C" void QCALLTYPE ModuleBuilder_SetModuleName(QCall::ModuleHandle pModule, LPCWSTR wszModuleName)
 {
     QCALL_CONTRACT;
 
@@ -578,7 +573,7 @@ void QCALLTYPE COMModule::SetModuleName(QCall::ModuleHandle pModule, LPCWSTR wsz
 // Return a type spec token given a byte array
 //
 //******************************************************************************
-mdTypeSpec QCALLTYPE COMModule::GetTokenFromTypeSpec(QCall::ModuleHandle pModule, LPCBYTE pSignature, INT32 sigLength)
+extern "C" mdTypeSpec QCALLTYPE ModuleBuilder_GetTokenFromTypeSpec(QCall::ModuleHandle pModule, LPCBYTE pSignature, INT32 sigLength)
 {
     QCALL_CONTRACT;
 
@@ -597,80 +592,21 @@ mdTypeSpec QCALLTYPE COMModule::GetTokenFromTypeSpec(QCall::ModuleHandle pModule
 }
 
 
-// GetType
-// Given a class name, this method will look for that class
-//  with in the module.
-void QCALLTYPE COMModule::GetType(QCall::ModuleHandle pModule, LPCWSTR wszName, BOOL bThrowOnError, BOOL bIgnoreCase, QCall::ObjectHandleOnStack retType, QCall::ObjectHandleOnStack keepAlive)
-{
-    CONTRACTL
-    {
-        QCALL_CHECK;
-        PRECONDITION(CheckPointer(wszName));
-    }
-    CONTRACTL_END;
-
-    TypeHandle retTypeHandle;
-
-    BEGIN_QCALL;
-
-    DomainAssembly *pAssembly = pModule->GetDomainAssembly();
-    _ASSERTE(pAssembly);
-
-    BOOL prohibitAsmQualifiedName = TRUE;
-
-    // Load the class from this assembly (fail if it is in a different one).
-    retTypeHandle = TypeName::GetTypeManaged(wszName, pAssembly, bThrowOnError, bIgnoreCase, prohibitAsmQualifiedName, NULL, (OBJECTREF*)keepAlive.m_ppObject);
-
-    // Verify that it's in 'this' module
-    // But, if it's in a different assembly than expected, that's okay, because
-    // it just means that it's been type forwarded.
-    if (!retTypeHandle.IsNull())
-    {
-        if ( (retTypeHandle.GetModule() != pModule) &&
-             (retTypeHandle.GetModule()->GetAssembly() == pModule->GetAssembly()) )
-            retTypeHandle = TypeHandle();
-    }
-
-    if (!retTypeHandle.IsNull())
-    {
-        GCX_COOP();
-        retType.Set(retTypeHandle.GetManagedClassObject());
-    }
-
-    END_QCALL;
-
-    return;
-}
-
-
 // GetName
 // This routine will return the name of the module as a String
-void QCALLTYPE COMModule::GetScopeName(QCall::ModuleHandle pModule, QCall::StringHandleOnStack retString)
+extern "C" void QCALLTYPE RuntimeModule_GetScopeName(QCall::ModuleHandle pModule, QCall::StringHandleOnStack retString)
 {
     QCALL_CONTRACT;
 
     BEGIN_QCALL;
 
+    if (!pModule->GetMDImport()->IsValidToken(pModule->GetMDImport()->GetModuleFromScope()))
+    {
+        ThrowHR(COR_E_BADIMAGEFORMAT);
+    }
+
     LPCSTR    szName = NULL;
-
-    if (pModule->IsResource())
-    {
-        IfFailThrow(pModule->GetAssembly()->GetManifestImport()->GetFileProps(
-            pModule->GetModuleRef(),
-            &szName,
-            NULL,
-            NULL,
-            NULL));
-    }
-    else
-    {
-        if (!pModule->GetMDImport()->IsValidToken(pModule->GetMDImport()->GetModuleFromScope()))
-        {
-            ThrowHR(COR_E_BADIMAGEFORMAT);
-        }
-        IfFailThrow(pModule->GetMDImport()->GetScopeProps(&szName, 0));
-    }
-
+    IfFailThrow(pModule->GetMDImport()->GetScopeProps(&szName, 0));
     retString.Set(szName);
 
     END_QCALL;
@@ -682,7 +618,7 @@ static void ReplaceNiExtension(SString& fileName, PCWSTR pwzOldSuffix, PCWSTR pw
 
     if (fileName.EndsWithCaseInsensitive(pwzOldSuffix))
     {
-        COUNT_T oldSuffixLen = (COUNT_T)wcslen(pwzOldSuffix);
+        COUNT_T oldSuffixLen = (COUNT_T)u16_strlen(pwzOldSuffix);
         fileName.Replace(fileName.End() - oldSuffixLen, oldSuffixLen, pwzNewSuffix);
     }
 }
@@ -693,7 +629,7 @@ static void ReplaceNiExtension(SString& fileName, PCWSTR pwzOldSuffix, PCWSTR pw
 **Arguments:
 **Exceptions:
 ==============================================================================*/
-void QCALLTYPE COMModule::GetFullyQualifiedName(QCall::ModuleHandle pModule, QCall::StringHandleOnStack retString)
+extern "C" void QCALLTYPE RuntimeModule_GetFullyQualifiedName(QCall::ModuleHandle pModule, QCall::StringHandleOnStack retString)
 {
     QCALL_CONTRACT;
 
@@ -724,7 +660,7 @@ void QCALLTYPE COMModule::GetFullyQualifiedName(QCall::ModuleHandle pModule, QCa
 **Arguments: refThis
 **Exceptions: None.
 ==============================================================================*/
-HINSTANCE QCALLTYPE COMModule::GetHINSTANCE(QCall::ModuleHandle pModule)
+extern "C" HINSTANCE QCALLTYPE MarshalNative_GetHINSTANCE(QCall::ModuleHandle pModule)
 {
     QCALL_CONTRACT;
 
@@ -734,10 +670,10 @@ HINSTANCE QCALLTYPE COMModule::GetHINSTANCE(QCall::ModuleHandle pModule)
 
     // This returns the base address
     // Other modules should have zero base
-    PEFile *pPEFile = pModule->GetFile();
-    if (!pPEFile->IsDynamic() && !pPEFile->IsResource())
+    PEAssembly *pPEAssembly = pModule->GetPEAssembly();
+    if (!pPEAssembly->IsDynamic())
     {
-        hMod = (HMODULE) pModule->GetFile()->GetManagedFileContents();
+        hMod = (HMODULE) pModule->GetPEAssembly()->GetManagedFileContents();
     }
 
     //If we don't have an hMod, set it to -1 so that they know that there's none
@@ -801,12 +737,6 @@ Object* GetTypesInner(Module* pModule)
     int             AllocSize = 0;
     MethodTable* pMT = NULL;
 
-    if (pModule->IsResource())
-    {
-        refArrClasses = (PTRARRAYREF) AllocateObjectArray(0, CoreLibBinder::GetClass(CLASS__CLASS));
-        RETURN(OBJECTREFToObject(refArrClasses));
-    }
-
     GCPROTECT_BEGIN(refArrClasses);
     GCPROTECT_BEGIN(xcept);
 
@@ -890,50 +820,3 @@ Object* GetTypesInner(Module* pModule)
 
     RETURN(OBJECTREFToObject(refArrClasses));
 }
-
-
-FCIMPL1(FC_BOOL_RET, COMModule::IsResource, ReflectModuleBaseObject* pModuleUNSAFE)
-{
-    FCALL_CONTRACT;
-
-    if (pModuleUNSAFE == NULL)
-        FCThrowRes(kArgumentNullException, W("Arg_InvalidHandle"));
-
-    FC_RETURN_BOOL(pModuleUNSAFE->GetModule()->IsResource());
-}
-FCIMPLEND
-
-
-//---------------------------------------------------------------------
-// Helper code for PunkSafeHandle class. This does the Release in the
-// safehandle's critical finalizer.
-//---------------------------------------------------------------------
-static VOID __stdcall DReleaseTarget(IUnknown *punk)
-{
-    CONTRACTL
-    {
-        NOTHROW;
-        GC_TRIGGERS;
-        MODE_PREEMPTIVE;
-    }
-    CONTRACTL_END;
-
-    if (punk)
-    {
-        punk->Release();
-    }
-}
-
-
-//---------------------------------------------------------------------
-// Helper code for PunkSafeHandle class. This returns the function that performs
-// the Release() for the safehandle's critical finalizer.
-//---------------------------------------------------------------------
-FCIMPL0(void*, COMPunkSafeHandle::nGetDReleaseTarget)
-{
-    FCALL_CONTRACT;
-
-    return (void*)DReleaseTarget;
-}
-FCIMPLEND
-
