@@ -16,11 +16,10 @@
 #include <cordbpriv.h>
 #include "field.h"
 #include <msodwwrap.h>
-#include <shlobj.h>
+#include <werapi.h>
 #include "dbginterface.h"
 #include <winver.h>
 #include "dlwrap.h"
-#include "eemessagebox.h"
 #include "eventreporter.h"
 #include "utilcode.h"
 #include "../dlls/mscorrc/resource.h"   // for resource ids
@@ -57,14 +56,6 @@ public:
     operator HMODULE() { return hModule; }
 };
 
-#ifndef FEATURE_CORESYSTEM
-#define WER_MODULE_NAME_W WINDOWS_KERNEL32_DLLNAME_W
-typedef SimpleModuleHolder<WszGetModuleHandle, false> WerModuleHolder;
-#else
-#define WER_MODULE_NAME_W W("api-ms-win-core-windowserrorreporting-l1-1-0.dll")
-typedef SimpleModuleHolder<CLRLoadLibrary, true> WerModuleHolder;
-#endif
-
 //------------------------------------------------------------------------------
 // Description
 //  Indicate if Watson is enabled
@@ -107,47 +98,17 @@ BOOL RegisterOutOfProcessWatsonCallbacks()
     CONTRACTL_END;
 
     WCHAR wszDACName[] = MAIN_DAC_MODULE_NAME_W W(".dll");
-    WerModuleHolder hWerModule(WER_MODULE_NAME_W);
-
-#ifdef FEATURE_CORESYSTEM
-    if ((hWerModule == NULL) && !RunningOnWin8())
-    {
-        // If we are built for CoreSystemServer, but are running on Windows 7, we need to look elsewhere
-        hWerModule = WerModuleHolder(W("Kernel32.dll"));
-    }
-#endif
-
-    if (hWerModule == NULL)
-    {
-        _ASSERTE(!"failed to get WER module handle");
-        return FALSE;
-    }
-
-    typedef HRESULT (WINAPI * WerRegisterRuntimeExceptionModuleFnPtr)(PCWSTR, PDWORD);
-    WerRegisterRuntimeExceptionModuleFnPtr pFnWerRegisterRuntimeExceptionModule;
-
-    pFnWerRegisterRuntimeExceptionModule = (WerRegisterRuntimeExceptionModuleFnPtr)
-                                        GetProcAddress(hWerModule, "WerRegisterRuntimeExceptionModule");
-
-    _ASSERTE(pFnWerRegisterRuntimeExceptionModule != NULL);
-    if (pFnWerRegisterRuntimeExceptionModule == NULL)
-    {
-       return FALSE;
-    }
     HRESULT hr = S_OK;
 
     EX_TRY
     {
         PathString wszDACPath;
-        if (SUCCEEDED(::GetClrModuleDirectory(wszDACPath)))
+        hr = ::GetClrModuleDirectory(wszDACPath);
+        if (SUCCEEDED(hr))
         {
             wszDACPath.Append(wszDACName);
-            hr = (*pFnWerRegisterRuntimeExceptionModule)(wszDACPath, (PDWORD)GetClrModuleBase());
+            hr = WerRegisterRuntimeExceptionModule(wszDACPath, (PDWORD)GetClrModuleBase());
         }
-        else {
-            hr = E_FAIL;
-        }
-
     }
     EX_CATCH_HRESULT(hr);
 
@@ -156,16 +117,7 @@ BOOL RegisterOutOfProcessWatsonCallbacks()
         STRESS_LOG0(LF_STARTUP,
                     LL_ERROR,
                     "WATSON support: failed to register DAC dll with WerRegisterRuntimeExceptionModule");
-
-#ifdef FEATURE_CORESYSTEM
-        // For CoreSys we *could* be running on a platform that doesn't have Watson proper
-        // (the APIs might exist but they just fail).
-        // WerRegisterRuntimeExceptionModule may return E_NOIMPL.
         return TRUE;
-#else // FEATURE_CORESYSTEM
-       _ASSERTE(! "WATSON support: failed to register DAC dll with WerRegisterRuntimeExceptionModule");
-        return FALSE;
-#endif // FEATURE_CORESYSTEM
     }
 
     STRESS_LOG0(LF_STARTUP,
@@ -179,7 +131,7 @@ BOOL RegisterOutOfProcessWatsonCallbacks()
 //------------------------------------------------------------------------------
 // Description
 HRESULT DwGetFileVersionInfo(
-    __in_z LPCWSTR wszFilePath,
+    _In_z_ LPCWSTR wszFilePath,
     USHORT& major,
     USHORT& minor,
     USHORT& build,
@@ -222,7 +174,7 @@ HRESULT DwGetFileVersionInfo(
 //   None
 //------------------------------------------------------------------------------
 int DwGetAppDescription(                // Number of characters written.
-    __in_z LPCWSTR wszFilePath,          // Path to the executable.
+    _In_z_ LPCWSTR wszFilePath,          // Path to the executable.
     SString& pBuf // Put description here.
     )                     // Size of buf, wide chars.
 {
@@ -297,7 +249,7 @@ int DwGetAppDescription(                // Number of characters written.
 
     // Build the query key for the language-specific file description resource.
     WCHAR buf[64];                 //----+----1----+----2----+----3----+----4----+
-    _snwprintf_s(buf, NumItems(buf), _TRUNCATE, W("\\StringFileInfo\\%04x%04x\\FileDescription"),
+    _snwprintf_s(buf, ARRAY_SIZE(buf), _TRUNCATE, W("\\StringFileInfo\\%04x%04x\\FileDescription"),
                translation->language, translation->codePage);
 
     // Get the file description.
@@ -317,7 +269,7 @@ int DwGetAppDescription(                // Number of characters written.
     }
 
     // If the description is a single space, ignore it.
-    if (wcscmp(fileDescription, W(" ")) == 0)
+    if (u16_strcmp(fileDescription, W(" ")) == 0)
     {
         return 0;
     }
@@ -354,8 +306,8 @@ int DwGetAppDescription(                // Number of characters written.
 //   None
 //------------------------------------------------------------------------------
 int DwGetAssemblyVersion(               // Number of characters written.
-    __in_z LPCWSTR  wszFilePath,         // Path to the executable.
-    __inout_ecount(cchBuf) WCHAR *pBuf, // Put description here.
+    _In_z_ LPCWSTR  wszFilePath,         // Path to the executable.
+    _Inout_updates_(cchBuf) WCHAR *pBuf, // Put description here.
     int cchBuf)                     // Size of buf, wide chars.
 {
     CONTRACTL
@@ -429,7 +381,7 @@ int DwGetAssemblyVersion(               // Number of characters written.
 
     // Build the query key for the language-specific assembly version resource.
     WCHAR buf[64];                                                 //----+----1----+----2----+----3----+----4----+
-    _snwprintf_s(buf, NumItems(buf), _TRUNCATE, W("\\StringFileInfo\\%04x%04x\\Assembly Version"),
+    _snwprintf_s(buf, ARRAY_SIZE(buf), _TRUNCATE, W("\\StringFileInfo\\%04x%04x\\Assembly Version"),
                translation->language, translation->codePage);
 
     // Get the assembly version.
@@ -449,7 +401,7 @@ int DwGetAssemblyVersion(               // Number of characters written.
     }
 
     // If the assembly version is a single space, ignore it.
-    if (wcscmp(assemblyVersion, W(" ")) == 0)
+    if (u16_strcmp(assemblyVersion, W(" ")) == 0)
     {
         return 0;
     }
@@ -551,7 +503,7 @@ UINT_PTR GetIPOfThrowSite(
 //   false   -- otherwise.
 //
 //------------------------------------------------------------------------------
-BOOL ContainsUnicodeChars(__in_z LPCWSTR wsz)
+BOOL ContainsUnicodeChars(_In_z_ LPCWSTR wsz)
 {
     LIMITED_METHOD_CONTRACT;
 
@@ -888,19 +840,19 @@ HRESULT GetBucketParametersForCurrentException(
     if (hr == S_OK)
     {
         // Event type name.
-        wcsncpy_s(pParams->pszEventTypeName, COUNTOF(pParams->pszEventTypeName), gmb.wzEventTypeName, _TRUNCATE);
+        wcsncpy_s(pParams->pszEventTypeName, ARRAY_SIZE(pParams->pszEventTypeName), gmb.wzEventTypeName, _TRUNCATE);
 
         // Buckets.  Mind the 1-based vs 0-based.
-        wcsncpy_s(pParams->pszParams[0], COUNTOF(pParams->pszParams[0]), gmb.wzP1, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[1], COUNTOF(pParams->pszParams[1]), gmb.wzP2, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[2], COUNTOF(pParams->pszParams[2]), gmb.wzP3, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[3], COUNTOF(pParams->pszParams[3]), gmb.wzP4, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[4], COUNTOF(pParams->pszParams[4]), gmb.wzP5, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[5], COUNTOF(pParams->pszParams[5]), gmb.wzP6, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[6], COUNTOF(pParams->pszParams[6]), gmb.wzP7, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[7], COUNTOF(pParams->pszParams[7]), gmb.wzP8, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[8], COUNTOF(pParams->pszParams[8]), gmb.wzP9, _TRUNCATE);
-        wcsncpy_s(pParams->pszParams[9], COUNTOF(pParams->pszParams[9]), gmb.wzP10, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[0], ARRAY_SIZE(pParams->pszParams[0]), gmb.wzP1, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[1], ARRAY_SIZE(pParams->pszParams[1]), gmb.wzP2, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[2], ARRAY_SIZE(pParams->pszParams[2]), gmb.wzP3, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[3], ARRAY_SIZE(pParams->pszParams[3]), gmb.wzP4, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[4], ARRAY_SIZE(pParams->pszParams[4]), gmb.wzP5, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[5], ARRAY_SIZE(pParams->pszParams[5]), gmb.wzP6, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[6], ARRAY_SIZE(pParams->pszParams[6]), gmb.wzP7, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[7], ARRAY_SIZE(pParams->pszParams[7]), gmb.wzP8, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[8], ARRAY_SIZE(pParams->pszParams[8]), gmb.wzP9, _TRUNCATE);
+        wcsncpy_s(pParams->pszParams[9], ARRAY_SIZE(pParams->pszParams[9]), gmb.wzP10, _TRUNCATE);
 
         // All good.
         pParams->fInited = TRUE;
@@ -1000,10 +952,8 @@ void ResetWatsonBucketsFavorWorker(void * pParam)
     return;
 }
 
-
 //----------------------------------------------------------------------------
-// CreateThread() callback to invoke native Watson or put up our fake Watson
-// dialog depending on m_fDoReportFault value.
+// CreateThread() callback to invoke native Watson
 //
 // The output is a FaultReport* value communicated by setting
 // pFaultReportInfo->m_result. The DWORD function return value
@@ -1024,32 +974,7 @@ static DWORD WINAPI DoFaultReportCreateThreadCallback(LPVOID pFaultReportInfoAsV
 
     FaultReportInfo *pFaultReportInfo = (FaultReportInfo*)pFaultReportInfoAsVoid;
     EXCEPTION_POINTERS *pExceptionInfo = pFaultReportInfo->m_pExceptionInfo;
-
-    if (pFaultReportInfo->m_fDoReportFault)
-    {
-        pFaultReportInfo->m_faultRepRetValResult = DoReportFault(pExceptionInfo);
-    }
-    else
-    {
-        int res = EEMessageBoxCatastrophicWithCustomizedStyle(
-                               IDS_DEBUG_UNHANDLEDEXCEPTION,
-                               IDS_DEBUG_SERVICE_CAPTION,
-                               MB_OKCANCEL | MB_ICONEXCLAMATION,
-                               TRUE,
-                               GetCurrentProcessId(),
-                               GetCurrentProcessId(),
-                               pFaultReportInfo->m_threadid,
-                               pFaultReportInfo->m_threadid
-                              );
-        if (res == IDOK)
-        {
-            pFaultReportInfo->m_faultReportResult = FaultReportResultQuit;
-        }
-        else
-        {
-            pFaultReportInfo->m_faultReportResult = FaultReportResultDebug;
-        }
-    }
+    pFaultReportInfo->m_faultRepRetValResult = DoReportFault(pExceptionInfo);
 
     return 0;
 }
@@ -1136,7 +1061,7 @@ ContractFailureKind GetContractFailureKind(OBJECTREF obj)
                 // getting closer, now look for all three fields on ContractException
                 const int requiredFieldMatches = 3;
 
-                PTR_EEClass pEEClass = pMT->GetClass_NoLogging();
+                PTR_EEClass pEEClass = pMT->GetClass();
 
                 PTR_FieldDesc pFD = pEEClass->GetFieldDescList();
                 PTR_FieldDesc pFDEnd = pFD + numFields;
