@@ -35,7 +35,7 @@ public:
     TypeNamesList();
     ~TypeNamesList();
 
-    HRESULT Init(__in_z LPCWSTR str);
+    HRESULT Init(_In_z_ LPCWSTR str);
     bool IsInList(LPCUTF8 typeName);
 };
 #endif
@@ -76,21 +76,29 @@ public:
     unsigned int  GenOptimizeType(void)                     const {LIMITED_METHOD_CONTRACT;  return iJitOptimizeType; }
     bool          JitFramed(void)                           const {LIMITED_METHOD_CONTRACT;  return fJitFramed; }
     bool          JitMinOpts(void)                          const {LIMITED_METHOD_CONTRACT;  return fJitMinOpts; }
-
-    void          DisableDefaultCodeVersioning()            { fDisableDefaultCodeVersioning = TRUE; }
-    bool          DefaultCodeVersioningDisabled()           const {LIMITED_METHOD_CONTRACT;  return fDisableDefaultCodeVersioning; }
+    bool          JitEnableOptionalRelocs(void)             const {LIMITED_METHOD_CONTRACT;  return fJitEnableOptionalRelocs; }
+    bool          DisableOptimizedThreadStaticAccess(void)  const {LIMITED_METHOD_CONTRACT;  return fDisableOptimizedThreadStaticAccess; }
 
     // Tiered Compilation config
 #if defined(FEATURE_TIERED_COMPILATION)
-    bool          TieredCompilation(void)           const { LIMITED_METHOD_CONTRACT;  return fTieredCompilation && !fDisableDefaultCodeVersioning; }
-    bool          TieredCompilation_QuickJit() const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_QuickJit && !fDisableDefaultCodeVersioning; }
-    bool          TieredCompilation_QuickJitForLoops() const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_QuickJitForLoops && !fDisableDefaultCodeVersioning; }
-    DWORD         TieredCompilation_BackgroundWorkerTimeoutMs() const { LIMITED_METHOD_CONTRACT; return fDisableDefaultCodeVersioning ? 0 : tieredCompilation_BackgroundWorkerTimeoutMs; }
-    bool          TieredCompilation_CallCounting()  const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_CallCounting && !fDisableDefaultCodeVersioning; }
-    UINT16        TieredCompilation_CallCountThreshold() const { LIMITED_METHOD_CONTRACT; return fDisableDefaultCodeVersioning ? 1 : tieredCompilation_CallCountThreshold; }
-    DWORD         TieredCompilation_CallCountingDelayMs() const { LIMITED_METHOD_CONTRACT; return fDisableDefaultCodeVersioning ? 0 : tieredCompilation_CallCountingDelayMs; }
-    bool          TieredCompilation_UseCallCountingStubs() const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_UseCallCountingStubs && !fDisableDefaultCodeVersioning; }
-    DWORD         TieredCompilation_DeleteCallCountingStubsAfter() const { LIMITED_METHOD_CONTRACT; return fDisableDefaultCodeVersioning ? 0 : tieredCompilation_DeleteCallCountingStubsAfter; }
+    bool          TieredCompilation(void)           const { LIMITED_METHOD_CONTRACT;  return fTieredCompilation; }
+    bool          TieredCompilation_QuickJit() const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_QuickJit; }
+    bool          TieredCompilation_QuickJitForLoops() const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_QuickJitForLoops; }
+    DWORD         TieredCompilation_BackgroundWorkerTimeoutMs() const { LIMITED_METHOD_CONTRACT; return tieredCompilation_BackgroundWorkerTimeoutMs; }
+    bool          TieredCompilation_CallCounting()  const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_CallCounting; }
+    UINT16        TieredCompilation_CallCountThreshold() const { LIMITED_METHOD_CONTRACT; return tieredCompilation_CallCountThreshold; }
+    DWORD         TieredCompilation_CallCountingDelayMs() const { LIMITED_METHOD_CONTRACT; return tieredCompilation_CallCountingDelayMs; }
+    bool          TieredCompilation_UseCallCountingStubs() const { LIMITED_METHOD_CONTRACT; return fTieredCompilation_UseCallCountingStubs; }
+    DWORD         TieredCompilation_DeleteCallCountingStubsAfter() const { LIMITED_METHOD_CONTRACT; return tieredCompilation_DeleteCallCountingStubsAfter; }
+#endif
+
+#if defined(FEATURE_PGO)
+    bool          TieredPGO(void) const { LIMITED_METHOD_CONTRACT;  return fTieredPGO; }
+    bool          TieredPGO_InstrumentOnlyHotCode(void) const { LIMITED_METHOD_CONTRACT;  return tieredPGO_InstrumentOnlyHotCode; }
+#endif
+
+#if defined(FEATURE_READYTORUN)
+    bool          ReadyToRun(void) const { LIMITED_METHOD_CONTRACT;  return fReadyToRun; }
 #endif
 
 #if defined(FEATURE_ON_STACK_REPLACEMENT)
@@ -104,9 +112,7 @@ public:
     DWORD         OSR_HighId() const { LIMITED_METHOD_CONTRACT; return dwOSR_HighId; }
 #endif
 
-#ifndef CROSSGEN_COMPILE
     bool          BackpatchEntryPointSlots() const { LIMITED_METHOD_CONTRACT; return backpatchEntryPointSlots; }
-#endif
 
 #if defined(FEATURE_GDBJIT) && defined(_DEBUG)
     inline bool ShouldDumpElfOnMethod(LPCUTF8 methodName) const
@@ -141,9 +147,6 @@ public:
 #ifdef _DEBUG
     bool GenDebuggableCode(void)                    const {LIMITED_METHOD_CONTRACT;  return fDebuggable; }
 
-    bool ShouldExposeExceptionsInCOMToConsole()     const {LIMITED_METHOD_CONTRACT;  return (iExposeExceptionsInCOM & 1) != 0; }
-    bool ShouldExposeExceptionsInCOMToMsgBox()      const {LIMITED_METHOD_CONTRACT;  return (iExposeExceptionsInCOM & 2) != 0; }
-
     static bool RegexOrExactMatch(LPCUTF8 regex, LPCUTF8 input);
 
     inline bool ShouldPrestubHalt(MethodDesc* pMethodInfo) const
@@ -249,7 +252,7 @@ public:
         } CONTRACTL_END
         return RegexOrExactMatch(pszBreakOnStructMarshalSetup, className);
     }
-    static HRESULT ParseTypeList(__in_z LPWSTR str, TypeNamesList** out);
+    static HRESULT ParseTypeList(_In_z_ LPWSTR str, TypeNamesList** out);
     static void DestroyTypeList(TypeNamesList* list);
 
     inline bool ShouldGcCoverageOnMethod(LPCUTF8 methodName) const
@@ -421,24 +424,8 @@ public:
 #endif
 
     // Loader
-
-    enum RequireZapsType
-    {
-        REQUIRE_ZAPS_NONE,      // Dont care if native image is used or not
-        REQUIRE_ZAPS_ALL,       // All assemblies must have native images
-        REQUIRE_ZAPS_ALL_JIT_OK,// All assemblies must have native images, but its OK if the JIT-compiler also gets used (if some function was not ngenned)
-
-        REQUIRE_ZAPS_COUNT
-    };
-    RequireZapsType RequireZaps()           const {LIMITED_METHOD_CONTRACT;  return iRequireZaps; }
-    bool    RequireZap(LPCUTF8 assemblyName) const;
-#ifdef _DEBUG
-    bool    ForbidZap(LPCUTF8 assemblyName) const;
-#endif
     bool    ExcludeReadyToRun(LPCUTF8 assemblyName) const;
 
-    LPCWSTR ZapSet()                        const { LIMITED_METHOD_CONTRACT; return pZapSet; }
-
     bool    NgenBindOptimizeNonGac()        const { LIMITED_METHOD_CONTRACT; return fNgenBindOptimizeNonGac; }
 
     LPUTF8  GetZapBBInstr()                 const { LIMITED_METHOD_CONTRACT; return szZapBBInstr; }
@@ -483,9 +470,6 @@ public:
 
 #endif
 
-    int ThreadPoolThreadTimeoutMs() const { LIMITED_METHOD_CONTRACT; return threadPoolThreadTimeoutMs; }
-    int ThreadPoolThreadsToKeepAlive() const { LIMITED_METHOD_CONTRACT; return threadPoolThreadsToKeepAlive; }
-
 private: //----------------------------------------------------------------
 
     bool fInited;                   // have we synced to the registry at least once?
@@ -496,6 +480,8 @@ private: //----------------------------------------------------------------
     bool fTrackDynamicMethodDebugInfo; //  Enable/Disable tracking dynamic method debug info
     bool fJitFramed;                   // Enable/Disable EBP based frames
     bool fJitMinOpts;                  // Enable MinOpts for all jitted methods
+    bool fJitEnableOptionalRelocs;     // Allow optional relocs
+    bool fDisableOptimizedThreadStaticAccess; // Disable OptimizedThreadStatic access
 
     unsigned iJitOptimizeType; // 0=Blended,1=SmallCode,2=FastCode,              default is 0=Blended
 
@@ -515,7 +501,7 @@ private: //----------------------------------------------------------------
     bool   m_fInteropLogArguments; // Log all pinned arguments passed to an interop call
 
 #ifdef _DEBUG
-    static HRESULT ParseMethList(__in_z LPWSTR str, MethodNamesList* * out);
+    static HRESULT ParseMethList(_In_z_ LPWSTR str, MethodNamesList* * out);
     static void DestroyMethList(MethodNamesList* list);
     static bool IsInMethList(MethodNamesList* list, MethodDesc* pMD);
 
@@ -541,8 +527,6 @@ private: //----------------------------------------------------------------
     bool   fConditionalContracts;       // Conditional contracts (off inside asserts)
     bool   fSuppressChecks;             // Disable checks (including contracts)
 
-    DWORD  iExposeExceptionsInCOM;      // Should we exposed exceptions that will be transformed into HRs?
-
     unsigned m_SuspendThreadDeadlockTimeoutMs;  // Used in Thread::SuspendThread()
     unsigned m_SuspendDeadlockTimeout; // Used in Thread::SuspendRuntime.
 
@@ -621,28 +605,9 @@ private: //----------------------------------------------------------------
     AssemblyNamesList *pSkipGCCoverageList;
 #endif
 
-    RequireZapsType iRequireZaps;
-    // Assemblies which need to have native images.
-    // This is only used if iRequireZaps!=REQUIRE_ZAPS_NONE
-    // This can be used to enforce that ngen images are used only selectively for some assemblies
-    AssemblyNamesList * pRequireZapsList;
-    // assemblies which need NOT have native images.
-    // This is only used if iRequireZaps!=REQUIRE_ZAPS_NONE
-    // This overrides pRequireZapsList.
-    AssemblyNamesList * pRequireZapsExcludeList;
-
     // Assemblies which cannot use Ready to Run images.
     AssemblyNamesList * pReadyToRunExcludeList;
 
-#ifdef _DEBUG
-    // Exact opposite of require zaps
-    BOOL iForbidZaps;
-    AssemblyNamesList * pForbidZapsList;
-    AssemblyNamesList * pForbidZapsExcludeList;
-#endif
-
-    LPCWSTR pZapSet;
-
     bool fNgenBindOptimizeNonGac;
 
     bool fStressLog;
@@ -661,9 +626,6 @@ private: //----------------------------------------------------------------
     int       m_TraceWrapper;
 #endif
 
-    // Flag to keep track of memory
-    int     m_fFreepZapSet;
-
 #ifdef _DEBUG
     // GC Alloc perf flags
     int iPerfNumAllocsThreshold;        // Start logging after this many allocations are made
@@ -683,11 +645,6 @@ private: //----------------------------------------------------------------
     DWORD testThreadAbort;
 #endif
 
-    int threadPoolThreadTimeoutMs;
-    int threadPoolThreadsToKeepAlive;
-
-    bool fDisableDefaultCodeVersioning;
-
 #if defined(FEATURE_TIERED_COMPILATION)
     bool fTieredCompilation;
     bool fTieredCompilation_QuickJit;
@@ -700,6 +657,15 @@ private: //----------------------------------------------------------------
     DWORD tieredCompilation_DeleteCallCountingStubsAfter;
 #endif
 
+#if defined(FEATURE_PGO)
+    bool fTieredPGO;
+    bool tieredPGO_InstrumentOnlyHotCode;
+#endif
+
+#if defined(FEATURE_READYTORUN)
+    bool fReadyToRun;
+#endif
+
 #if defined(FEATURE_ON_STACK_REPLACEMENT)
     DWORD dwOSR_HitLimit;
     DWORD dwOSR_CounterBump;
@@ -710,9 +676,7 @@ private: //----------------------------------------------------------------
     DWORD dwOSR_HighId;
 #endif
 
-#ifndef CROSSGEN_COMPILE
     bool backpatchEntryPointSlots;
-#endif
 
 #if defined(FEATURE_GDBJIT) && defined(_DEBUG)
     LPCUTF8 pszGDBJitElfDump;
@@ -819,16 +783,4 @@ public:
 
 #endif
 
-// NGENImagesAllowed is the safe way to determine if NGEN Images are allowed to be loaded. (Defined as
-// a macro instead of an inlined function to avoid compilation errors due to dependent
-// definitions not being available to this header.)
-#ifdef PROFILING_SUPPORTED
-#define NGENImagesAllowed()                                                                                     \
-    (g_fAllowNativeImages &&                /* No one disabled use of native images */                          \
-    !(CORProfilerDisableAllNGenImages()))   /* Profiler didn't explicitly refuse NGEN images */
-#else
-#define NGENImagesAllowed()                                                                                     \
-    (g_fAllowNativeImages)
-#endif
-
 #endif // EECONFIG_H
