@@ -5,6 +5,7 @@
 
 // Runtime headers
 #include "common.h"
+#include "simplerwlock.hpp"
 #include "rcwrefcache.h"
 #ifdef FEATURE_COMINTEROP_APARTMENT_SUPPORT
 #include "olecontexthelpers.h"
@@ -236,7 +237,7 @@ namespace
                 ExtObjCxtCache* instMaybe = new ExtObjCxtCache();
 
                 // Attempt to set the global instance.
-                if (NULL != FastInterlockCompareExchangePointer(&g_Instance, instMaybe, NULL))
+                if (NULL != InterlockedCompareExchangeT(&g_Instance, instMaybe, NULL))
                     delete instMaybe;
             }
 
@@ -257,32 +258,36 @@ namespace
         using Element = SHash<Traits>::element_t;
         using Iterator = SHash<Traits>::Iterator;
 
-        class LockHolder : public CrstHolder
+        class ReaderLock final
         {
+            SimpleReadLockHolder _lock;
         public:
-            LockHolder(_In_ ExtObjCxtCache *cache)
-                : CrstHolder(&cache->_lock)
-            {
-                // This cache must be locked in Cooperative mode
-                // since releases of wrappers can occur during a GC.
-                CONTRACTL
-                {
-                    NOTHROW;
-                    GC_NOTRIGGER;
-                    MODE_COOPERATIVE;
-                }
-                CONTRACTL_END;
-            }
+            ReaderLock(_In_ ExtObjCxtCache* cache)
+                : _lock{ &cache->_lock }
+            { }
+
+            ~ReaderLock() = default;
+        };
+
+        class WriterLock final
+        {
+            SimpleWriteLockHolder _lock;
+        public:
+            WriterLock(_In_ ExtObjCxtCache* cache)
+                : _lock{ &cache->_lock }
+            { }
+
+            ~WriterLock() = default;
         };
 
     private:
         friend struct InteropLibImports::RuntimeCallContext;
         SHash<Traits> _hashMap;
-        Crst _lock;
+        SimpleRWLock _lock;
         ExtObjCxtRefCache* _refCache;
 
         ExtObjCxtCache()
-            : _lock(CrstExternalObjectContextCache, CRST_UNSAFE_COOPGC)
+            : _lock(COOPERATIVE, LOCK_TYPE_DEFAULT)
             , _refCache(GetAppDomain()->GetRCWRefCache())
         { }
         ~ExtObjCxtCache() = default;
@@ -292,7 +297,7 @@ namespace
         bool IsLockHeld()
         {
             WRAPPER_NO_CONTRACT;
-            return (_lock.OwnedByCurrentThread() != FALSE);
+            return (_lock.LockTaken() != FALSE);
         }
 #endif // _DEBUG
 
@@ -326,7 +331,8 @@ namespace
                 PTRARRAYREF arrRefTmp;
                 PTRARRAYREF arrRef;
             } gc;
-            ::ZeroMemory(&gc, sizeof(gc));
+            gc.arrRefTmp = NULL;
+            gc.arrRef = NULL;
             GCPROTECT_BEGIN(gc);
 
             // Only add objects that are in the correct thread
@@ -342,7 +348,7 @@ namespace
             // Determine the count of objects to return.
             SIZE_T objCountMax = 0;
             {
-                LockHolder lock(this);
+                ReaderLock lock(this);
                 Iterator end = _hashMap.End();
                 for (Iterator curr = _hashMap.Begin(); curr != end; ++curr)
                 {
@@ -364,7 +370,7 @@ namespace
             SIZE_T objCount = 0;
             if (0 < objCountMax)
             {
-                LockHolder lock(this);
+                ReaderLock lock(this);
                 Iterator end = _hashMap.End();
                 for (Iterator curr = _hashMap.Begin(); curr != end; ++curr)
                 {
@@ -413,7 +419,7 @@ namespace
                 // Separate the wrapper from the tracker runtime prior to
                 // passing them onto the caller. This call is okay to make
                 // even if the instance isn't from the tracker runtime.
-                // We switch to Preemptive mode since seperating a wrapper
+                // We switch to Preemptive mode since separating a wrapper
                 // requires us to call out to non-runtime code which could
                 // call back into the runtime and/or trigger a GC.
                 GCX_PREEMP();
@@ -695,11 +701,9 @@ namespace
             OBJECTREF implRef;
             OBJECTREF instRef;
         } gc;
-        ::ZeroMemory(&gc, sizeof(gc));
-        GCPROTECT_BEGIN(gc);
-
         gc.implRef = impl;
         gc.instRef = instance;
+        GCPROTECT_BEGIN(gc);
 
         // Check the object's SyncBlock for a managed object wrapper.
         SyncBlock* syncBlock = gc.instRef->GetSyncBlock();
@@ -807,14 +811,13 @@ namespace
             OBJECTREF wrapperMaybeRef;
             OBJECTREF objRefMaybe;
         } gc;
-        ::ZeroMemory(&gc, sizeof(gc));
+        gc.implRef = impl;
+        gc.wrapperMaybeRef = wrapperMaybe;
+        gc.objRefMaybe = NULL;
         GCPROTECT_BEGIN(gc);
 
         STRESS_LOG4(LF_INTEROP, LL_INFO1000, "Get or Create EOC: (Identity: 0x%p) (Flags: %x) (Maybe: 0x%p) (ID: %lld)\n", identity, flags, OBJECTREFToObject(wrapperMaybe), wrapperId);
 
-        gc.implRef = impl;
-        gc.wrapperMaybeRef = wrapperMaybe;
-
         ExtObjCxtCache* cache = ExtObjCxtCache::GetInstance();
         InteropLib::OBJECTHANDLE handle = NULL;
 
@@ -824,16 +827,43 @@ namespace
         bool uniqueInstance = !!(flags & CreateObjectFlags::CreateObjectFlags_UniqueInstance);
         if (!uniqueInstance)
         {
-            // Query the external object cache
-            ExtObjCxtCache::LockHolder lock(cache);
-            extObjCxt = cache->Find(cacheKey);
+            bool objectFound = false;
+            bool tryRemove = false;
+            {
+                // Perform a quick look up to determine if we know of the object and if
+                // we need to perform a more expensive cleanup operation below.
+                ExtObjCxtCache::ReaderLock lock(cache);
+                extObjCxt = cache->Find(cacheKey);
+                objectFound = extObjCxt != NULL;
+                tryRemove = objectFound && extObjCxt->IsSet(ExternalObjectContext::Flags_Detached);
+            }
+
+            if (tryRemove)
+            {
+                // Perform the slower cleanup operation that may be appropriate
+                // if the object still exists and has been detached.
+                ExtObjCxtCache::WriterLock lock(cache);
+                extObjCxt = cache->Find(cacheKey);
+                objectFound = extObjCxt != NULL;
+                if (objectFound && extObjCxt->IsSet(ExternalObjectContext::Flags_Detached))
+                {
+                    // If an EOC has been found but is marked detached, then we will remove it from the
+                    // cache here instead of letting the GC do it later and pretend like it wasn't found.
+                    STRESS_LOG1(LF_INTEROP, LL_INFO10, "Detached EOC requested: 0x%p\n", extObjCxt);
+                    cache->Remove(extObjCxt);
+                    extObjCxt->MarkNotInCache();
+                    extObjCxt = NULL;
+                }
+            }
 
             // If is no object found in the cache, check if the object COM instance is actually the CCW
             // representing a managed object. If the user passed the Unwrap flag, COM instances that are
             // actually CCWs should be unwrapped to the original managed object to allow for round
             // tripping object -> COM instance -> object.
-            if (extObjCxt == NULL && (flags & CreateObjectFlags::CreateObjectFlags_Unwrap))
+            if (!objectFound && (flags & CreateObjectFlags::CreateObjectFlags_Unwrap))
             {
+                GCX_PREEMP();
+
                 // If the COM instance is a CCW that is not COM-activated, use the object of that wrapper object.
                 InteropLib::OBJECTHANDLE handleLocal;
                 if (InteropLib::Com::GetObjectForWrapper(identity, &handleLocal) ==  S_OK
@@ -842,15 +872,6 @@ namespace
                     handle = handleLocal;
                 }
             }
-            else if (extObjCxt != NULL && extObjCxt->IsSet(ExternalObjectContext::Flags_Detached))
-            {
-                // If an EOC has been found but is marked detached, then we will remove it from the
-                // cache here instead of letting the GC do it later and pretend like it wasn't found.
-                STRESS_LOG1(LF_INTEROP, LL_INFO10, "Detached EOC requested: 0x%p\n", extObjCxt);
-                cache->Remove(extObjCxt);
-                extObjCxt->MarkNotInCache();
-                extObjCxt = NULL;
-            }
         }
 
         STRESS_LOG2(LF_INTEROP, LL_INFO1000, "EOC: 0x%p or Handle: 0x%p\n", extObjCxt, handle);
@@ -861,12 +882,42 @@ namespace
         }
         else if (handle != NULL)
         {
-            // We have an object handle from the COM instance which is a CCW. Use that object.
-            // This allows for the round-trip from object -> COM instance -> object.
+            // We have an object handle from the COM instance which is a CCW.
             ::OBJECTHANDLE objectHandle = static_cast<::OBJECTHANDLE>(handle);
-            gc.objRefMaybe = ObjectFromHandle(objectHandle);
+
+            // Now we need to check if this object is a CCW from the same ComWrappers instance
+            // as the one creating the EOC. If it is not, we need to create a new EOC for it.
+            // Otherwise, use it. This allows for the round-trip from object -> COM instance -> object.
+            OBJECTREF objRef = NULL;
+            GCPROTECT_BEGIN(objRef);
+            objRef = ObjectFromHandle(objectHandle);
+
+            SyncBlock* syncBlock = objRef->GetSyncBlock();
+            InteropSyncBlockInfo* interopInfo = syncBlock->GetInteropInfo();
+
+            // If we found a managed object wrapper in this ComWrappers instance
+            // and it's the same identity pointer as the one we're creating an EOC for,
+            // unwrap it. We don't AddRef the wrapper as we don't take a reference to it.
+            //
+            // A managed object can have multiple managed object wrappers, with a max of one per context.
+            // Let's say we have a managed object A and ComWrappers instances C1 and C2. Let B1 and B2 be the
+            // managed object wrappers for A created with C1 and C2 respectively.
+            // If we are asked to create an EOC for B1 with the unwrap flag on the C2 ComWrappers instance,
+            // we will create a new wrapper. In this scenario, we'll only unwrap B2.
+            void* wrapperRawMaybe = NULL;
+            if (interopInfo->TryGetManagedObjectComWrapper(wrapperId, &wrapperRawMaybe)
+                && wrapperRawMaybe == identity)
+            {
+                gc.objRefMaybe = objRef;
+            }
+            else
+            {
+                STRESS_LOG2(LF_INTEROP, LL_INFO1000, "Not unwrapping handle (0x%p) because the object's MOW in this ComWrappers instance (if any) (0x%p) is not the provided identity\n", handle, wrapperRawMaybe);
+            }
+            GCPROTECT_END();
         }
-        else
+
+        if (gc.objRefMaybe == NULL)
         {
             // Create context instance for the possibly new external object.
             ExternalWrapperResultHolder resultHolder;
@@ -923,7 +974,7 @@ namespace
                 else
                 {
                     // Attempt to insert the new context into the cache.
-                    ExtObjCxtCache::LockHolder lock(cache);
+                    ExtObjCxtCache::WriterLock lock(cache);
                     extObjCxt = cache->FindOrAdd(cacheKey, resultHolder.GetContext());
                 }
 
@@ -945,7 +996,7 @@ namespace
                     {
                         // Failed to set the context; one must already exist.
                         // Remove from the cache above as well.
-                        ExtObjCxtCache::LockHolder lock(cache);
+                        ExtObjCxtCache::WriterLock lock(cache);
                         cache->Remove(resultHolder.GetContext());
 
                         COMPlusThrow(kNotSupportedException);
@@ -1121,10 +1172,9 @@ namespace InteropLibImports
                 OBJECTREF implRef;
                 OBJECTREF objsEnumRef;
             } gc;
-            ::ZeroMemory(&gc, sizeof(gc));
-            GCPROTECT_BEGIN(gc);
-
             gc.implRef = NULL; // Use the globally registered implementation.
+            gc.objsEnumRef = NULL;
+            GCPROTECT_BEGIN(gc);
 
             // Pass the objects along to get released.
             ExtObjCxtCache* cache = ExtObjCxtCache::GetInstanceNoThrow();
@@ -1166,16 +1216,11 @@ namespace InteropLibImports
         }
         CONTRACTL_END;
 
-        bool isValid = false;
         ::OBJECTHANDLE objectHandle = static_cast<::OBJECTHANDLE>(handle);
 
-        {
-            // Switch to cooperative mode so the handle can be safely inspected.
-            GCX_COOP_THREAD_EXISTS(GET_THREAD());
-            isValid = ObjectFromHandle(objectHandle) != NULL;
-        }
-
-        return isValid;
+        // A valid target is one that is not null.
+        bool isNotNull = ObjectHandleIsNull(objectHandle) == FALSE;
+        return isNotNull;
     }
 
     bool GetGlobalPeggingState() noexcept
@@ -1233,11 +1278,10 @@ namespace InteropLibImports
                 OBJECTREF wrapperMaybeRef;
                 OBJECTREF objRef;
             } gc;
-            ::ZeroMemory(&gc, sizeof(gc));
-            GCPROTECT_BEGIN(gc);
-
             gc.implRef = NULL; // Use the globally registered implementation.
             gc.wrapperMaybeRef = NULL; // No supplied wrapper here.
+            gc.objRef = NULL;
+            GCPROTECT_BEGIN(gc);
 
             // Get wrapper for external object
             bool success = TryGetOrCreateObjectForComInstanceInternal(
@@ -1320,7 +1364,7 @@ namespace InteropLibImports
             {
                 OBJECTREF objRef;
             } gc;
-            ::ZeroMemory(&gc, sizeof(gc));
+            gc.objRef = NULL;
             GCPROTECT_BEGIN(gc);
 
             // Get the target of the external object's reference.
@@ -1425,7 +1469,7 @@ namespace InteropLibImports
     }
 }
 
-BOOL QCALLTYPE ComWrappersNative::TryGetOrCreateComInterfaceForObject(
+extern "C" BOOL QCALLTYPE ComWrappers_TryGetOrCreateComInterfaceForObject(
     _In_ QCall::ObjectHandleOnStack comWrappersImpl,
     _In_ INT64 wrapperId,
     _In_ QCall::ObjectHandleOnStack instance,
@@ -1434,7 +1478,7 @@ BOOL QCALLTYPE ComWrappersNative::TryGetOrCreateComInterfaceForObject(
 {
     QCALL_CONTRACT;
 
-    bool success;
+    bool success = false;
 
     BEGIN_QCALL;
 
@@ -1456,7 +1500,7 @@ BOOL QCALLTYPE ComWrappersNative::TryGetOrCreateComInterfaceForObject(
     return (success ? TRUE : FALSE);
 }
 
-BOOL QCALLTYPE ComWrappersNative::TryGetOrCreateObjectForComInstance(
+extern "C" BOOL QCALLTYPE ComWrappers_TryGetOrCreateObjectForComInstance(
     _In_ QCall::ObjectHandleOnStack comWrappersImpl,
     _In_ INT64 wrapperId,
     _In_ void* ext,
@@ -1469,7 +1513,7 @@ BOOL QCALLTYPE ComWrappersNative::TryGetOrCreateObjectForComInstance(
 
     _ASSERTE(ext != NULL);
 
-    bool success;
+    bool success = false;
 
     BEGIN_QCALL;
 
@@ -1512,7 +1556,7 @@ BOOL QCALLTYPE ComWrappersNative::TryGetOrCreateObjectForComInstance(
     return (success ? TRUE : FALSE);
 }
 
-void QCALLTYPE ComWrappersNative::GetIUnknownImpl(
+extern "C" void QCALLTYPE ComWrappers_GetIUnknownImpl(
         _Out_ void** fpQueryInterface,
         _Out_ void** fpAddRef,
         _Out_ void** fpRelease)
@@ -1530,6 +1574,81 @@ void QCALLTYPE ComWrappersNative::GetIUnknownImpl(
     END_QCALL;
 }
 
+extern "C" BOOL QCALLTYPE ComWrappers_TryGetComInstance(
+    _In_ QCall::ObjectHandleOnStack wrapperMaybe,
+    _Out_ void** externalComObject)
+{
+    QCALL_CONTRACT;
+
+    _ASSERTE(externalComObject != NULL);
+
+    bool success = false;
+
+    BEGIN_QCALL;
+
+    // Switch to Cooperative mode since object references
+    // are being manipulated.
+    {
+        GCX_COOP();
+
+        SyncBlock* syncBlock = ObjectToOBJECTREF(*wrapperMaybe.m_ppObject)->PassiveGetSyncBlock();
+        if (syncBlock != nullptr)
+        {
+            InteropSyncBlockInfo* interopInfo = syncBlock->GetInteropInfoNoCreate();
+            if (interopInfo != nullptr)
+            {
+                void* contextMaybe;
+                if (interopInfo->TryGetExternalComObjectContext(&contextMaybe))
+                {
+                    ExternalObjectContext* context = reinterpret_cast<ExternalObjectContext*>(contextMaybe);
+                    IUnknown* identity = reinterpret_cast<IUnknown*>(context->Identity);
+                    GCX_PREEMP();
+                    success = SUCCEEDED(identity->QueryInterface(IID_IUnknown, externalComObject));
+                }
+            }
+        }
+    }
+
+    END_QCALL;
+
+    return (success ? TRUE : FALSE);
+}
+
+extern "C" BOOL QCALLTYPE ComWrappers_TryGetObject(
+    _In_ void* wrapperMaybe,
+    _Inout_ QCall::ObjectHandleOnStack instance)
+{
+    QCALL_CONTRACT;
+
+    _ASSERTE(wrapperMaybe != NULL);
+
+    bool success = false;
+
+    BEGIN_QCALL;
+
+    // Determine the true identity of the object
+    SafeComHolder<IUnknown> identity;
+    HRESULT hr = ((IUnknown*)wrapperMaybe)->QueryInterface(IID_IUnknown, &identity);
+    _ASSERTE(hr == S_OK);
+
+    InteropLib::OBJECTHANDLE handle;
+    if (InteropLib::Com::GetObjectForWrapper(identity, &handle) == S_OK)
+    {
+        // Switch to Cooperative mode since object references
+        // are being manipulated.
+        GCX_COOP();
+
+        // We have an object handle from the COM instance which is a CCW.
+        ::OBJECTHANDLE objectHandle = static_cast<::OBJECTHANDLE>(handle);
+        instance.Set(ObjectFromHandle(objectHandle));
+        success = true;
+    }
+
+    END_QCALL;
+
+    return (success ? TRUE : FALSE);
+}
+
 void ComWrappersNative::DestroyManagedObjectComWrapper(_In_ void* wrapper)
 {
     CONTRACTL
@@ -1604,18 +1723,22 @@ void ComWrappersNative::MarkWrapperAsComActivated(_In_ IUnknown* wrapperMaybe)
 {
     CONTRACTL
     {
-        NOTHROW;
-        MODE_ANY;
+        THROWS;
+        GC_TRIGGERS;
+        MODE_COOPERATIVE;
         PRECONDITION(wrapperMaybe != NULL);
     }
     CONTRACTL_END;
 
-    // The IUnknown may or may not represent a wrapper, so E_INVALIDARG is okay here.
-    HRESULT hr = InteropLib::Com::MarkComActivated(wrapperMaybe);
-    _ASSERTE(SUCCEEDED(hr) || hr == E_INVALIDARG);
+    {
+        GCX_PREEMP();
+        // The IUnknown may or may not represent a wrapper, so E_INVALIDARG is okay here.
+        HRESULT hr = InteropLib::Com::MarkComActivated(wrapperMaybe);
+        _ASSERTE(SUCCEEDED(hr) || hr == E_INVALIDARG);
+    }
 }
 
-void QCALLTYPE GlobalComWrappersForMarshalling::SetGlobalInstanceRegisteredForMarshalling(INT64 id)
+extern "C" void QCALLTYPE ComWrappers_SetGlobalInstanceRegisteredForMarshalling(INT64 id)
 {
     QCALL_CONTRACT_NO_GC_TRANSITION;
 
@@ -1708,7 +1831,7 @@ bool GlobalComWrappersForMarshalling::TryGetOrCreateObjectForComInstance(
     }
 }
 
-void QCALLTYPE GlobalComWrappersForTrackerSupport::SetGlobalInstanceRegisteredForTrackerSupport(INT64 id)
+extern "C" void QCALLTYPE ComWrappers_SetGlobalInstanceRegisteredForTrackerSupport(INT64 id)
 {
     QCALL_CONTRACT_NO_GC_TRANSITION;
 
