@@ -24,11 +24,10 @@
 #include "runtimehandles.h"
 #include "argdestination.h"
 
-#ifndef CROSSGEN_COMPILE
 
 // The Attributes Table
 //  20 bits for built in types and 12 bits for Properties
-//  The properties are followed by the widening mask.  All types widen to them selves.
+//  The properties are followed by the widening mask.  All types widen to themselves.
 const DWORD InvokeUtil::PrimitiveAttributes[PRIMITIVE_TABLE_SIZE] = {
     0x00,                     // ELEMENT_TYPE_END
     0x00,                     // ELEMENT_TYPE_VOID
@@ -125,61 +124,48 @@ void *InvokeUtil::GetIntPtrValue(OBJECTREF pObj) {
     RETURN *(void **)((pObj)->UnBox());
 }
 
-void InvokeUtil::CopyArg(TypeHandle th, OBJECTREF *pObjUNSAFE, ArgDestination *argDest) {
+void InvokeUtil::CopyArg(TypeHandle th, PVOID argRef, ArgDestination *argDest) {
     CONTRACTL {
         THROWS;
         GC_NOTRIGGER; // Caller does not protect object references
         MODE_COOPERATIVE;
         PRECONDITION(!th.IsNull());
-        PRECONDITION(CheckPointer(pObjUNSAFE));
         INJECT_FAULT(COMPlusThrowOM());
     }
     CONTRACTL_END;
 
     void *pArgDst = argDest->GetDestinationAddress();
+    CorElementType type = th.GetVerifierCorElementType();
 
-    OBJECTREF rObj = *pObjUNSAFE;
-    MethodTable* pMT;
-    CorElementType oType;
-    CorElementType type;
-
-    if (rObj != 0) {
-        pMT = rObj->GetMethodTable();
-        oType = pMT->GetInternalCorElementType();
-    }
-    else {
-        pMT = 0;
-        oType = ELEMENT_TYPE_OBJECT;
-    }
-    type = th.GetVerifierCorElementType();
-
-    // This basically maps the Signature type our type and calls the CreatePrimitiveValue
-    //  method.  We can omit this if we get alignment on these types.
     switch (type) {
     case ELEMENT_TYPE_BOOLEAN:
-    case ELEMENT_TYPE_I1:
     case ELEMENT_TYPE_U1:
-    case ELEMENT_TYPE_I2:
-    case ELEMENT_TYPE_U2:
-    case ELEMENT_TYPE_CHAR:
-    case ELEMENT_TYPE_I4:
-    case ELEMENT_TYPE_U4:
-    case ELEMENT_TYPE_R4:
-    IN_TARGET_32BIT(case ELEMENT_TYPE_I:)
-    IN_TARGET_32BIT(case ELEMENT_TYPE_U:)
+    case ELEMENT_TYPE_I1:
     {
-        // If we got the univeral zero...Then assign it and exit.
-        if (rObj == 0)
-            *(PVOID *)pArgDst = 0;
-        else
-        {
-            ARG_SLOT slot;
-            CreatePrimitiveValue(type, oType, rObj, &slot);
-            *(PVOID *)pArgDst = (PVOID)slot;
-        }
+        _ASSERTE(argRef != NULL);
+        *(INT8 *)pArgDst = *(INT8 *)argRef;
         break;
     }
 
+    case ELEMENT_TYPE_I2:
+    case ELEMENT_TYPE_U2:
+    case ELEMENT_TYPE_CHAR:
+    {
+        _ASSERTE(argRef != NULL);
+        *(INT16 *)pArgDst = *(INT16 *)argRef;
+        break;
+    }
+
+    case ELEMENT_TYPE_I4:
+    case ELEMENT_TYPE_U4:
+    case ELEMENT_TYPE_R4:
+    IN_TARGET_32BIT(case ELEMENT_TYPE_U:)
+    IN_TARGET_32BIT(case ELEMENT_TYPE_I:)
+    {
+        _ASSERTE(argRef != NULL);
+        *(INT32 *)pArgDst = *(INT32 *)argRef;
+        break;
+    }
 
     case ELEMENT_TYPE_I8:
     case ELEMENT_TYPE_U8:
@@ -187,129 +173,44 @@ void InvokeUtil::CopyArg(TypeHandle th, OBJECTREF *pObjUNSAFE, ArgDestination *a
     IN_TARGET_64BIT(case ELEMENT_TYPE_I:)
     IN_TARGET_64BIT(case ELEMENT_TYPE_U:)
     {
-        // If we got the univeral zero...Then assign it and exit.
-        if (rObj == 0)
-            *(INT64 *)pArgDst = 0;
-        else
-        {
-            ARG_SLOT slot;
-            CreatePrimitiveValue(type, oType, rObj, &slot);
-            *(INT64 *)pArgDst = (INT64)slot;
-        }
+        _ASSERTE(argRef != NULL);
+        *(INT64 *)pArgDst = *(INT64 *)argRef;
         break;
     }
 
     case ELEMENT_TYPE_VALUETYPE:
     {
-        // If we got the universal zero...Then assign it and exit.
-        if (rObj == 0) {
-            InitValueClassArg(argDest, th.AsMethodTable());
-         }
-        else {
-            if (!th.AsMethodTable()->UnBoxIntoArg(argDest, rObj))
-                COMPlusThrow(kArgumentException, W("Arg_ObjObj"));
-        }
+        MethodTable* pMT = th.GetMethodTable();
+        CopyValueClassArg(argDest, argRef, pMT, 0);
         break;
     }
 
+    case ELEMENT_TYPE_STRING:           // System.String
+    case ELEMENT_TYPE_CLASS:            // Class
+    case ELEMENT_TYPE_OBJECT:           // System.Object
     case ELEMENT_TYPE_SZARRAY:          // Single Dim
     case ELEMENT_TYPE_ARRAY:            // General Array
-    case ELEMENT_TYPE_CLASS:            // Class
-    case ELEMENT_TYPE_OBJECT:
-    case ELEMENT_TYPE_STRING:           // System.String
     case ELEMENT_TYPE_VAR:
     {
-        if (rObj == 0)
+        if (argRef == NULL)
             *(PVOID *)pArgDst = 0;
         else
-            *(PVOID *)pArgDst = OBJECTREFToObject(rObj);
+            *(PVOID *)pArgDst = OBJECTREFToObject((OBJECTREF)(Object*)*(PVOID*)argRef);
         break;
     }
 
     case ELEMENT_TYPE_BYREF:
     {
-       //
-       //     (obj is the parameter passed to MethodInfo.Invoke, by the caller)
-       //     if argument is a primitive
-       //     {
-       //         if incoming argument, obj, is null
-       //             Allocate a boxed object and place ref to it in 'obj'
-       //         Unbox 'obj' and pass it to callee
-       //     }
-       //     if argument is a value class
-       //     {
-       //         if incoming argument, obj, is null
-       //             Allocate an object of that valueclass, and place ref to it in 'obj'
-       //         Unbox 'obj' and pass it to callee
-       //     }
-       //     if argument is an objectref
-       //     {
-       //         pass obj to callee
-       //     }
-       //
-        TypeHandle thBaseType = th.AsTypeDesc()->GetTypeParam();
-
-        // We should never get here for nullable types.  Instead invoke
-        // heads these off and morphs the type handle to not be byref anymore
-        _ASSERTE(!Nullable::IsNullableType(thBaseType));
-
-        TypeHandle srcTH = TypeHandle();
-        if (rObj == 0)
-            oType = thBaseType.GetSignatureCorElementType();
-        else
-            srcTH = rObj->GetTypeHandle();
-
-        //CreateByRef only triggers GC in throw path, so it's OK to use the raw unsafe pointer
-        *(PVOID *)pArgDst = CreateByRef(thBaseType, oType, srcTH, rObj, pObjUNSAFE);
-        break;
-    }
-
-    case ELEMENT_TYPE_TYPEDBYREF:
-    {
-        TypedByRef* ptr = (TypedByRef*) pArgDst;
-        TypeHandle srcTH;
-        BOOL bIsZero = FALSE;
-
-        // If we got the univeral zero...Then assign it and exit.
-        if (rObj== 0) {
-            bIsZero = TRUE;
-            ptr->data = 0;
-            ptr->type = TypeHandle();
-        }
-        else {
-            bIsZero = FALSE;
-            srcTH = rObj->GetTypeHandle();
-            ptr->type = rObj->GetTypeHandle();
-        }
-
-        if (!bIsZero)
-        {
-            //CreateByRef only triggers GC in throw path
-            ptr->data = CreateByRef(srcTH, oType, srcTH, rObj, pObjUNSAFE);
-        }
-
+        *(PVOID *)pArgDst = argRef;
         break;
     }
 
     case ELEMENT_TYPE_PTR:
     case ELEMENT_TYPE_FNPTR:
     {
-        // If we got the univeral zero...Then assign it and exit.
-        if (rObj == 0) {
-            *(PVOID *)pArgDst = 0;
-        }
-        else {
-            if (rObj->GetMethodTable() == CoreLibBinder::GetClassIfExist(CLASS__POINTER) && type == ELEMENT_TYPE_PTR)
-                *(PVOID *)pArgDst = GetPointerValue(rObj);
-            else if (rObj->GetTypeHandle().AsMethodTable() == CoreLibBinder::GetElementType(ELEMENT_TYPE_I))
-            {
-                ARG_SLOT slot;
-                CreatePrimitiveValue(oType, oType, rObj, &slot);
-                *(PVOID *)pArgDst = (PVOID)slot;
-            }
-            else
-                COMPlusThrow(kArgumentException,W("Arg_ObjObj"));
-        }
+        _ASSERTE(argRef != NULL);
+        MethodTable* pMT = th.GetMethodTable();
+        CopyValueClassArg(argDest, argRef, pMT, 0);
         break;
     }
 
@@ -343,10 +244,12 @@ void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,
     CreatePrimitiveValue(dstType, srcType, srcObj->UnBox(), srcObj->GetMethodTable(), pDst);
 }
 
-void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,CorElementType srcType,
-    void *pSrc, MethodTable *pSrcMT, ARG_SLOT* pDst)
+void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,
+                                      CorElementType srcType,
+                                      void *pSrc,
+                                      MethodTable *pSrcMT,
+                                      ARG_SLOT* pDst)
 {
-
     CONTRACTL {
         THROWS;
         GC_NOTRIGGER;
@@ -435,10 +338,10 @@ void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,CorElementType srcT
             *pDst = data;
             break;
         case ELEMENT_TYPE_R4:
-            *pDst = (I8)(*(R4*)pSrc);
+            *pDst = (CLR_I8)(*(CLR_R4*)pSrc);
             break;
         case ELEMENT_TYPE_R8:
-            *pDst = (I8)(*(R8*)pSrc);
+            *pDst = (CLR_I8)(*(CLR_R8*)pSrc);
             break;
         default:
             _ASSERTE(!"Unknown conversion");
@@ -449,35 +352,35 @@ void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,CorElementType srcT
     case ELEMENT_TYPE_R4:
     case ELEMENT_TYPE_R8:
         {
-        R8 r8 = 0;
+        CLR_R8 r8 = 0;
         switch (srcType) {
         case ELEMENT_TYPE_BOOLEAN:
         case ELEMENT_TYPE_I1:
         case ELEMENT_TYPE_I2:
         case ELEMENT_TYPE_I4:
         IN_TARGET_32BIT(case ELEMENT_TYPE_I:)
-            r8 = (R8)((INT32)data);
+            r8 = (CLR_R8)((INT32)data);
             break;
         case ELEMENT_TYPE_U1:
         case ELEMENT_TYPE_CHAR:
         case ELEMENT_TYPE_U2:
         case ELEMENT_TYPE_U4:
         IN_TARGET_32BIT(case ELEMENT_TYPE_U:)
-            r8 = (R8)((UINT32)data);
+            r8 = (CLR_R8)((UINT32)data);
             break;
         case ELEMENT_TYPE_U8:
         IN_TARGET_64BIT(case ELEMENT_TYPE_U:)
-            r8 = (R8)((UINT64)data);
+            r8 = (CLR_R8)((UINT64)data);
             break;
         case ELEMENT_TYPE_I8:
         IN_TARGET_64BIT(case ELEMENT_TYPE_I:)
-            r8 = (R8)((INT64)data);
+            r8 = (CLR_R8)((INT64)data);
             break;
         case ELEMENT_TYPE_R4:
-            r8 = *(R4*)pSrc;
+            r8 = *(CLR_R4*)pSrc;
             break;
         case ELEMENT_TYPE_R8:
-            r8 = *(R8*)pSrc;
+            r8 = *(CLR_R8*)pSrc;
             break;
         default:
             _ASSERTE(!"Unknown R4 or R8 conversion");
@@ -486,7 +389,7 @@ void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,CorElementType srcT
         }
 
         if (dstType == ELEMENT_TYPE_R4) {
-            R4 r4 = (R4)r8;
+            CLR_R4 r4 = (CLR_R4)r8;
             *pDst = (UINT32&)r4;
         }
         else {
@@ -500,72 +403,6 @@ void InvokeUtil::CreatePrimitiveValue(CorElementType dstType,CorElementType srcT
     }
 }
 
-void* InvokeUtil::CreateByRef(TypeHandle dstTh,
-                              CorElementType srcType,
-                              TypeHandle srcTH,
-                              OBJECTREF srcObj,
-                              OBJECTREF *pIncomingObj) {
-    CONTRACTL {
-        THROWS;
-        GC_NOTRIGGER;
-        MODE_COOPERATIVE;
-        PRECONDITION(!dstTh.IsNull());
-        PRECONDITION(CheckPointer(pIncomingObj));
-
-        INJECT_FAULT(COMPlusThrowOM());
-    }
-    CONTRACTL_END;
-
-    CorElementType dstType = dstTh.GetSignatureCorElementType();
-    if (IsPrimitiveType(srcType) && IsPrimitiveType(dstType)) {
-        if (dstType != srcType)
-        {
-            CONTRACT_VIOLATION (GCViolation);
-            COMPlusThrow(kArgumentException,W("Arg_PrimWiden"));
-        }
-
-        return srcObj->UnBox();
-    }
-
-    if (srcTH.IsNull()) {
-        return pIncomingObj;
-    }
-
-    _ASSERTE(srcObj != NULL);
-
-    if (dstType == ELEMENT_TYPE_VALUETYPE) {
-        return srcObj->UnBox();
-    }
-    else
-        return pIncomingObj;
-}
-
-// GetBoxedObject
-// Given an address of a primitve type, this will box that data...
-// <TODO>@TODO: We need to handle all value classes?</TODO>
-OBJECTREF InvokeUtil::GetBoxedObject(TypeHandle th, void* pData) {
-    CONTRACTL {
-        THROWS;
-        GC_TRIGGERS;
-        MODE_COOPERATIVE;
-        PRECONDITION(!th.IsNull());
-        PRECONDITION(CheckPointer(pData));
-
-        INJECT_FAULT(COMPlusThrowOM());
-    }
-    CONTRACTL_END;
-
-    MethodTable *pMethTable = th.GetMethodTable();
-    PREFIX_ASSUME(pMethTable != NULL);
-    // Save off the data.  We are going to create and object
-    //  which may cause GC to occur.
-    int size = pMethTable->GetNumInstanceFieldBytes();
-    void *p = _alloca(size);
-    memcpy(p, pData, size);
-    OBJECTREF retO = pMethTable->Box(p);
-    return retO;
-}
-
 //ValidField
 // This method checks that the object can be widened to the proper type
 void InvokeUtil::ValidField(TypeHandle th, OBJECTREF* value)
@@ -640,7 +477,6 @@ void InvokeUtil::ValidField(TypeHandle th, OBJECTREF* value)
         return;
     }
 
-
     if (!IsPrimitiveType(oType))
         COMPlusThrow(kArgumentException,W("Arg_ObjObj"));
     // Now make sure we can widen into the proper type -- CanWiden may run GC...
@@ -734,7 +570,8 @@ OBJECTREF InvokeUtil::CreateClassLoadExcept(OBJECTREF* classes, OBJECTREF* excep
         OBJECTREF o;
         STRINGREF str;
     } gc;
-    ZeroMemory(&gc, sizeof(gc));
+    gc.o = NULL;
+    gc.str = NULL;
 
     MethodTable *pVMClassLoadExcept = CoreLibBinder::GetException(kReflectionTypeLoadException);
     gc.o = AllocateObject(pVMClassLoadExcept);
@@ -824,60 +661,6 @@ OBJECTREF InvokeUtil::CreateTargetExcept(OBJECTREF* except) {
     RETURN oRet;
 }
 
-// ChangeType
-// This method will invoke the Binder change type method on the object
-//  binder -- The Binder object
-//  srcObj -- The source object to be changed
-//  th -- The TypeHandel of the target type
-//  locale -- The locale passed to the class.
-OBJECTREF InvokeUtil::ChangeType(OBJECTREF binder, OBJECTREF srcObj, TypeHandle th, OBJECTREF locale) {
-    CONTRACTL {
-        THROWS;
-        GC_TRIGGERS;
-        MODE_COOPERATIVE;
-        PRECONDITION(binder != NULL);
-        PRECONDITION(srcObj != NULL);
-
-        INJECT_FAULT(COMPlusThrowOM());
-    }
-    CONTRACTL_END;
-
-    OBJECTREF typeClass = NULL;
-    OBJECTREF o;
-
-    struct _gc {
-        OBJECTREF binder;
-        OBJECTREF srcObj;
-        OBJECTREF locale;
-        OBJECTREF typeClass;
-    } gc;
-
-    gc.binder = binder;
-    gc.srcObj = srcObj;
-    gc.locale = locale;
-    gc.typeClass = NULL;
-
-    GCPROTECT_BEGIN(gc);
-
-    MethodDescCallSite changeType(METHOD__BINDER__CHANGE_TYPE, &gc.binder);
-
-    // Now call this method on this object.
-    typeClass = th.GetManagedClassObject();
-
-    ARG_SLOT pNewArgs[] = {
-            ObjToArgSlot(gc.binder),
-            ObjToArgSlot(gc.srcObj),
-            ObjToArgSlot(gc.typeClass),
-            ObjToArgSlot(gc.locale),
-    };
-
-    o = changeType.Call_RetOBJECTREF(pNewArgs);
-
-    GCPROTECT_END();
-
-    return o;
-}
-
 // Ensure that the field is declared on the type or subtype of the type to which the typed reference refers.
 // Note that a typed reference is a reference to an object and is not a field on that object (as in C# ref).
 // Ensure that if the field is an instance field that the typed reference is not null.
@@ -898,7 +681,7 @@ void InvokeUtil::ValidateObjectTarget(FieldDesc *pField, TypeHandle enclosingTyp
         return;
 
     if (!pField->IsStatic() && !*target)
-        COMPlusThrow(kTargetException,W("RFLCT.Targ_StatFldReqTarg"));
+        COMPlusThrow(kTargetException,W("RFLCT_Targ_StatFldReqTarg"));
 
     // Verify that the object is of the proper type...
     TypeHandle ty = (*target)->GetTypeHandle();
@@ -1123,9 +906,13 @@ void InvokeUtil::SetValidField(CorElementType fldType,
         {
             void* pFieldData;
             if (pField->IsStatic())
+            {
                 pFieldData = pField->GetCurrentStaticAddress();
+            }
             else
-                pFieldData = (*((BYTE**)target)) + pField->GetOffset() + sizeof(Object);
+            {
+                pFieldData = pField->GetInstanceAddress(*target);
+            }
 
             if (*valueObj == NULL)
                 InitValueClass(pFieldData, pMT);
@@ -1266,9 +1053,12 @@ OBJECTREF InvokeUtil::GetFieldValue(FieldDesc* pField, TypeHandle fieldType, OBJ
         GCPROTECT_BEGIN(obj);
         // calculate the offset to the field...
         if (pField->IsStatic())
+        {
             p = pField->GetCurrentStaticAddress();
-        else {
-                p = (*((BYTE**)target)) + pField->GetOffset() + sizeof(Object);
+        }
+        else
+        {
+            p = pField->GetInstanceAddress(*target);
         }
         GCPROTECT_END();
 
@@ -1278,8 +1068,8 @@ OBJECTREF InvokeUtil::GetFieldValue(FieldDesc* pField, TypeHandle fieldType, OBJ
             CopyValueClass(obj->GetData(), p, fieldType.AsMethodTable());
         }
 
-            // If it is a Nullable<T>, box it using Nullable<T> conventions.
-            // TODO: this double allocates on constructions which is wastefull
+        // If it is a Nullable<T>, box it using Nullable<T> conventions.
+        // TODO: this double allocates on constructions which is wastefull
         obj = Nullable::NormalizeBox(obj);
         break;
     }
@@ -1317,6 +1107,3 @@ OBJECTREF InvokeUtil::GetFieldValue(FieldDesc* pField, TypeHandle fieldType, OBJ
 
     return obj;
 }
-
-
-#endif // CROSSGEN_COMPILE
