@@ -96,7 +96,7 @@ StringLiteralMap::~StringLiteralMap()
 
     // We do need to take the globalstringliteralmap lock because we are manipulating
     // StringLiteralEntry objects that belong to it.
-    // Note that we remember the current entry and relaese it only when the
+    // Note that we remember the current entry and release it only when the
     // enumerator has advanced to the next entry so that we don't endup deleteing the
     // current entry itself and killing the enumerator.
 
@@ -143,7 +143,7 @@ StringLiteralMap::~StringLiteralMap()
 
 
 
-STRINGREF *StringLiteralMap::GetStringLiteral(EEStringData *pStringData, BOOL bAddIfNotFound, BOOL bAppDomainWontUnload)
+STRINGREF *StringLiteralMap::GetStringLiteral(EEStringData *pStringData, BOOL bAddIfNotFound, BOOL bIsCollectible, void** ppPinnedString)
 {
     CONTRACTL
     {
@@ -165,7 +165,9 @@ STRINGREF *StringLiteralMap::GetStringLiteral(EEStringData *pStringData, BOOL bA
     // someone beat us to inserting it. (m_StringToEntryHashTable->GetValue(pStringData, &Data))
     // (Rather than waiting until after we look the string up in the global map)
 
-    StringLiteralEntryHolder pEntry(SystemDomain::GetGlobalStringLiteralMap()->GetStringLiteral(pStringData, dwHash, bAddIfNotFound));
+    // Don't use FOH for collectible modules to avoid potential memory leaks
+    const bool preferFrozenObjectHeap = !bIsCollectible;
+    StringLiteralEntryHolder pEntry(SystemDomain::GetGlobalStringLiteralMap()->GetStringLiteral(pStringData, dwHash, bAddIfNotFound, preferFrozenObjectHeap));
 
     _ASSERTE(pEntry || !bAddIfNotFound);
 
@@ -177,7 +179,7 @@ STRINGREF *StringLiteralMap::GetStringLiteral(EEStringData *pStringData, BOOL bA
         // TODO: except that by not inserting into our local table we always take the global map lock
         // and come into this path, when we could succeed at a lock free lookup above.
 
-        if (!bAppDomainWontUnload)
+        if (bIsCollectible)
         {
             // Make sure some other thread has not already added it.
             if (!m_StringToEntryHashTable->GetValue(pStringData, &Data))
@@ -201,6 +203,12 @@ STRINGREF *StringLiteralMap::GetStringLiteral(EEStringData *pStringData, BOOL bA
         // Retrieve the string objectref from the string literal entry.
         pStrObj = pEntry->GetStringObject();
         _ASSERTE(!bAddIfNotFound || pStrObj);
+
+
+        if (pStrObj != nullptr && ppPinnedString != nullptr && preferFrozenObjectHeap && pEntry->IsStringFrozen())
+        {
+            *ppPinnedString = *reinterpret_cast<void**>(pStrObj);
+        }
         return pStrObj;
     }
     // If the bAddIfNotFound flag is set then we better have a string
@@ -209,7 +217,7 @@ STRINGREF *StringLiteralMap::GetStringLiteral(EEStringData *pStringData, BOOL bA
     return NULL;
 }
 
-STRINGREF *StringLiteralMap::GetInternedString(STRINGREF *pString, BOOL bAddIfNotFound, BOOL bAppDomainWontUnload)
+STRINGREF *StringLiteralMap::GetInternedString(STRINGREF *pString, BOOL bAddIfNotFound, BOOL bIsCollectible)
 {
     CONTRACTL
     {
@@ -242,6 +250,7 @@ STRINGREF *StringLiteralMap::GetInternedString(STRINGREF *pString, BOOL bAddIfNo
         // (Rather than waiting until after we look the string up in the global map)
 
         // Retrieve the string literal from the global string literal map.
+
         StringLiteralEntryHolder pEntry(SystemDomain::GetGlobalStringLiteralMap()->GetInternedString(pString, dwHash, bAddIfNotFound));
 
         _ASSERTE(pEntry || !bAddIfNotFound);
@@ -254,7 +263,7 @@ STRINGREF *StringLiteralMap::GetInternedString(STRINGREF *pString, BOOL bAddIfNo
             // TODO: except that by not inserting into our local table we always take the global map lock
             // and come into this path, when we could succeed at a lock free lookup above.
 
-            if (!bAppDomainWontUnload)
+            if (bIsCollectible)
             {
                 // Since GlobalStringLiteralMap::GetInternedString() could have caused a GC,
                 // we need to recreate the string data.
@@ -297,10 +306,6 @@ GlobalStringLiteralMap::GlobalStringLiteralMap()
         GC_TRIGGERS;
     }
     CONTRACTL_END;
-
-#ifdef _DEBUG
-    m_PinnedHeapHandleTable.RegisterCrstDebug(&m_HashTableCrstGlobal);
-#endif
 }
 
 GlobalStringLiteralMap::~GlobalStringLiteralMap()
@@ -361,7 +366,7 @@ void GlobalStringLiteralMap::Init()
         ThrowOutOfMemory();
 }
 
-StringLiteralEntry *GlobalStringLiteralMap::GetStringLiteral(EEStringData *pStringData, DWORD dwHash, BOOL bAddIfNotFound)
+StringLiteralEntry *GlobalStringLiteralMap::GetStringLiteral(EEStringData *pStringData, DWORD dwHash, BOOL bAddIfNotFound, BOOL bPreferFrozenObjectHeap)
 {
     CONTRACTL
     {
@@ -387,7 +392,9 @@ StringLiteralEntry *GlobalStringLiteralMap::GetStringLiteral(EEStringData *pStri
     else
     {
         if (bAddIfNotFound)
-            pEntry = AddStringLiteral(pStringData);
+        {
+            pEntry = AddStringLiteral(pStringData, bPreferFrozenObjectHeap);
+        }
     }
 
     return pEntry;
@@ -428,22 +435,28 @@ StringLiteralEntry *GlobalStringLiteralMap::GetInternedString(STRINGREF *pString
 }
 
 #ifdef LOGGING
-static void LogStringLiteral(__in_z const char* action, EEStringData *pStringData)
+static void LogStringLiteral(_In_z_ const char* action, EEStringData *pStringData)
 {
     STATIC_CONTRACT_NOTHROW;
     STATIC_CONTRACT_GC_NOTRIGGER;
     STATIC_CONTRACT_FORBID_FAULT;
 
-    int length = pStringData->GetCharCount();
-    length = min(length, 100);
+    ULONG length = pStringData->GetCharCount();
+    length = min(length, 128);
     WCHAR *szString = (WCHAR *)_alloca((length + 1) * sizeof(WCHAR));
     memcpyNoGCRefs((void*)szString, (void*)pStringData->GetStringBuffer(), length * sizeof(WCHAR));
     szString[length] = '\0';
-    LOG((LF_APPDOMAIN, LL_INFO10000, "String literal \"%S\" %s to Global map, size %d bytes\n", szString, action, pStringData->GetCharCount()));
+
+    LPCUTF8 strUtf8 = "<Failed to convert to UTF8>";
+    MAKE_UTF8PTR_FROMWIDE_NOTHROW(cvtStr, szString);
+    if (cvtStr != NULL)
+        strUtf8 = cvtStr;
+
+    LOG((LF_APPDOMAIN, LL_INFO10000, "String literal \"%s\" %s to Global map, %u characters\n", strUtf8, action, length));
 }
 #endif
 
-STRINGREF AllocateStringObject(EEStringData *pStringData)
+STRINGREF AllocateStringObject(EEStringData *pStringData, bool preferFrozenObjHeap, bool* pIsFrozen)
 {
     CONTRACTL
     {
@@ -456,7 +469,7 @@ STRINGREF AllocateStringObject(EEStringData *pStringData)
     // Create the COM+ string object.
     DWORD cCount = pStringData->GetCharCount();
 
-    STRINGREF strObj = AllocateString(cCount);
+    STRINGREF strObj = AllocateString(cCount, preferFrozenObjHeap, pIsFrozen);
 
     GCPROTECT_BEGIN(strObj)
     {
@@ -472,7 +485,7 @@ STRINGREF AllocateStringObject(EEStringData *pStringData)
 
     return strObj;
 }
-StringLiteralEntry *GlobalStringLiteralMap::AddStringLiteral(EEStringData *pStringData)
+StringLiteralEntry *GlobalStringLiteralMap::AddStringLiteral(EEStringData *pStringData, bool preferFrozenObjHeap)
 {
     CONTRACTL
     {
@@ -486,27 +499,41 @@ StringLiteralEntry *GlobalStringLiteralMap::AddStringLiteral(EEStringData *pStri
 
     StringLiteralEntry *pRet;
 
+    bool isFrozen = false;
+    STRINGREF strObj = AllocateStringObject(pStringData, preferFrozenObjHeap, &isFrozen);
+    if (isFrozen)
     {
-    PinnedHeapHandleBlockHolder pStrObj(&m_PinnedHeapHandleTable,1);
-    // Create the COM+ string object.
-    STRINGREF strObj = AllocateStringObject(pStringData);
+        _ASSERT(preferFrozenObjHeap);
+        StringLiteralEntryHolder pEntry(StringLiteralEntry::AllocateFrozenEntry(pStringData, strObj));
+        m_StringToEntryHashTable->InsertValue(pStringData, pEntry, FALSE);
+        pEntry.SuppressRelease();
+        pRet = pEntry;
+        _ASSERT(pRet->IsStringFrozen());
+    }
+    else
+    {
+        GCPROTECT_BEGIN(strObj)
+        {
+            // If it's not frozen we need to gc allocate a pinned handle to the non-pinned string object
+            PinnedHeapHandleBlockHolder pStrObj(&m_PinnedHeapHandleTable,1);
 
-    // Allocate a handle for the string.
-    SetObjectReference(pStrObj[0], (OBJECTREF) strObj);
+            // Allocate a handle for the string.
+            SetObjectReference(pStrObj[0], strObj);
 
-
-    // Allocate the StringLiteralEntry.
-    StringLiteralEntryHolder pEntry(StringLiteralEntry::AllocateEntry(pStringData, (STRINGREF*)pStrObj[0]));
-    pStrObj.SuppressRelease();
-    // Insert the handle to the string into the hash table.
-    m_StringToEntryHashTable->InsertValue(pStringData, (LPVOID)pEntry, FALSE);
-    pEntry.SuppressRelease();
-    pRet = pEntry;
+            // Allocate the StringLiteralEntry.
+            StringLiteralEntryHolder pEntry(StringLiteralEntry::AllocateEntry(pStringData, (STRINGREF*)pStrObj[0]));
+            pStrObj.SuppressRelease();
+            // Insert the handle to the string into the hash table.
+            m_StringToEntryHashTable->InsertValue(pStringData, pEntry, FALSE);
+            pEntry.SuppressRelease();
+            pRet = pEntry;
+        }
+        GCPROTECT_END();
+    }
 
 #ifdef LOGGING
     LogStringLiteral("added", pStringData);
 #endif
-    }
 
     return pRet;
 }
@@ -524,24 +551,26 @@ StringLiteralEntry *GlobalStringLiteralMap::AddInternedString(STRINGREF *pString
     }
     CONTRACTL_END;
 
-    EEStringData StringData = EEStringData((*pString)->GetStringLength(), (*pString)->GetBuffer());
-    StringLiteralEntry *pRet;
+    StringLiteralEntry* pRet;
 
     {
-    PinnedHeapHandleBlockHolder pStrObj(&m_PinnedHeapHandleTable,1);
-    SetObjectReference(pStrObj[0], (OBJECTREF) *pString);
+        // All frozen strings are expected to be registered in m_StringToEntryHashTable, we might relax this assert
+        // in future if we start allocating frozen strings for non-literals
+        _ASSERT(!GCHeapUtilities::GetGCHeap()->IsInFrozenSegment(STRINGREFToObject(*pString)));
 
-    // Since the allocation might have caused a GC we need to re-get the
-    // string data.
-    StringData = EEStringData((*pString)->GetStringLength(), (*pString)->GetBuffer());
+        PinnedHeapHandleBlockHolder pStrObj(&m_PinnedHeapHandleTable, 1);
+        SetObjectReference(pStrObj[0], (OBJECTREF)*pString);
 
-    StringLiteralEntryHolder pEntry(StringLiteralEntry::AllocateEntry(&StringData, (STRINGREF*)pStrObj[0]));
-    pStrObj.SuppressRelease();
+        // Since the allocation might have caused a GC we need to get the string data after it
+        EEStringData StringData = EEStringData((*pString)->GetStringLength(), (*pString)->GetBuffer());
 
-    // Insert the handle to the string into the hash table.
-    m_StringToEntryHashTable->InsertValue(&StringData, (LPVOID)pEntry, FALSE);
-    pEntry.SuppressRelease();
-    pRet = pEntry;
+        StringLiteralEntryHolder pEntry(StringLiteralEntry::AllocateEntry(&StringData, (STRINGREF*)pStrObj[0]));
+        pStrObj.SuppressRelease();
+
+        // Insert the handle to the string into the hash table.
+        m_StringToEntryHashTable->InsertValue(&StringData, (LPVOID)pEntry, FALSE);
+        pEntry.SuppressRelease();
+        pRet = pEntry;
     }
 
     return pRet;
@@ -568,7 +597,7 @@ void GlobalStringLiteralMap::RemoveStringLiteralEntry(StringLiteralEntry *pEntry
 
         BOOL bSuccess;
         bSuccess = m_StringToEntryHashTable->DeleteValue(&StringData);
-        // this assert is comented out to accomodate case when StringLiteralEntryHolder
+        // this assert is comented out to accommodate case when StringLiteralEntryHolder
         // releases this object after failed insertion into hash
         //_ASSERTE(bSuccess);
 
@@ -581,16 +610,19 @@ void GlobalStringLiteralMap::RemoveStringLiteralEntry(StringLiteralEntry *pEntry
         }
 #endif
 
-        // Release the object handle that the entry was using.
-        STRINGREF *pObjRef = pEntry->GetStringObject();
-        m_PinnedHeapHandleTable.ReleaseHandles((OBJECTREF*)pObjRef, 1);
+        if (!pEntry->IsStringFrozen())
+        {
+            // Release the object handle that the entry was using.
+            STRINGREF *pObjRef = pEntry->GetStringObject();
+            m_PinnedHeapHandleTable.ReleaseHandles((OBJECTREF*)pObjRef, 1);
+        }
     }
 
     // We do not delete the StringLiteralEntry itself that will be done in the
     // release method of the StringLiteralEntry.
 }
 
-StringLiteralEntry *StringLiteralEntry::AllocateEntry(EEStringData *pStringData, STRINGREF *pStringObj)
+void* StringLiteralEntry::AllocateEntryInternal()
 {
    CONTRACTL
     {
@@ -622,7 +654,17 @@ StringLiteralEntry *StringLiteralEntry::AllocateEntry(EEStringData *pStringData,
     }
     _ASSERTE (pMem && "Unable to allocate String literal Entry");
 
-    return new (pMem) StringLiteralEntry (pStringData, pStringObj);
+    return pMem;
+}
+
+StringLiteralEntry* StringLiteralEntry::AllocateEntry(EEStringData* pStringData, STRINGREF* pStringObj)
+{
+    return new (AllocateEntryInternal()) StringLiteralEntry(pStringData, pStringObj);
+}
+
+StringLiteralEntry* StringLiteralEntry::AllocateFrozenEntry(EEStringData* pStringData, STRINGREF pFrozenStringObj)
+{
+    return new (AllocateEntryInternal()) StringLiteralEntry(pStringData, pFrozenStringObj);
 }
 
 void StringLiteralEntry::DeleteEntry (StringLiteralEntry *pEntry)
@@ -635,13 +677,9 @@ void StringLiteralEntry::DeleteEntry (StringLiteralEntry *pEntry)
     }
     CONTRACTL_END;
 
-    _ASSERTE (VolatileLoad(&pEntry->m_dwRefCount) == 0);
-
-#ifdef _DEBUG
-    memset (pEntry, 0xc, sizeof(StringLiteralEntry));
-#endif
-
+    _ASSERTE (pEntry->IsStringFrozen() || pEntry->GetRefCount() == 0);
 #ifdef _DEBUG
+    memset (&pEntry->m_pStringObj, 0xc, sizeof(pEntry->m_pStringObj));
     pEntry->m_bDeleted = TRUE;
 #endif
 
