@@ -10,7 +10,6 @@
 #include "gcrefmap.h"
 #include "threads.h"
 
-#ifndef CROSSGEN_COMPILE
 
 FCIMPL2(void*, TailCallHelp::AllocTailCallArgBuffer, INT32 size, void* gcDesc)
 {
@@ -43,7 +42,6 @@ FCIMPL2(void*, TailCallHelp::GetTailCallInfo, void** retAddrSlot, void** retAddr
 }
 FCIMPLEND
 
-#endif
 
 struct ArgBufferValue
 {
@@ -138,7 +136,11 @@ void TailCallHelp::CreateTailCallHelperStubs(
     LOG((LF_STUBS, LL_INFO1000, "TAILCALLHELP: Incoming sig %s\n", incSig.GetCString()));
 #endif
 
-    *storeArgsNeedsTarget = pCalleeMD == NULL || pCalleeMD->IsSharedByGenericInstantiations();
+    // GVMs are not strictly "needs target" for the unshared case, but the JIT
+    // is going to resolve the target anyway so we may as well take it in the
+    // stub to avoid computing it in both places.
+    *storeArgsNeedsTarget = pCalleeMD == NULL || pCalleeMD->IsSharedByGenericInstantiations() ||
+        (pCalleeMD->IsVirtual() && !pCalleeMD->IsStatic() && pCalleeMD->HasMethodInstantiation());
 
     // The tailcall helper stubs are always allocated together with the caller.
     // If we ever wish to share these stubs they should be allocated with the
@@ -177,8 +179,7 @@ void TailCallHelp::LayOutArgBuffer(
         bool thisParamByRef = (calleeMD != NULL) ? calleeMD->GetMethodTable()->IsValueType() : thisArgByRef;
         if (thisParamByRef)
         {
-            thisHnd = TypeHandle(CoreLibBinder::GetElementType(ELEMENT_TYPE_U1))
-                      .MakeByRef();
+            thisHnd = TypeHandle(CoreLibBinder::GetElementType(ELEMENT_TYPE_U1)).MakeByRef();
         }
         else
         {
@@ -465,7 +466,7 @@ MethodDesc* TailCallHelp::CreateCallTargetStub(const TailCallInfo& info)
 
     ILCodeStream* pCode = sl.NewCodeStream(ILStubLinker::kDispatch);
 
-    // void CallTarget(void* argBuffer, void* retVal, PortableTailCallFrame* pFrame)
+    // void CallTarget(void* argBuffer, ref byte retVal, PortableTailCallFrame* pFrame)
     const int ARG_ARG_BUFFER = 0;
     const int ARG_RET_VAL = 1;
     const int ARG_PTR_FRAME = 2;
@@ -617,7 +618,8 @@ void TailCallHelp::CreateCallTargetStubSig(const TailCallInfo& info, SigBuilder*
     sig->AppendElementType(ELEMENT_TYPE_I);
 
     // Return value
-    sig->AppendElementType(ELEMENT_TYPE_I);
+    sig->AppendElementType(ELEMENT_TYPE_BYREF);
+    sig->AppendElementType(ELEMENT_TYPE_U1);
 
     // Pointer to tail call frame
     sig->AppendElementType(ELEMENT_TYPE_I);
@@ -632,49 +634,28 @@ void TailCallHelp::CreateCallTargetStubSig(const TailCallInfo& info, SigBuilder*
 #endif // _DEBUG
 }
 
-// Get TypeHandle for ByReference<System.Byte>
-static TypeHandle GetByReferenceOfByteType()
+static TypeHandle GetByReferenceType()
 {
-    TypeHandle byteTH(CoreLibBinder::GetElementType(ELEMENT_TYPE_U1));
-    Instantiation byteInst(&byteTH, 1);
-    TypeHandle th = TypeHandle(CoreLibBinder::GetClass(CLASS__BYREFERENCE)).Instantiate(byteInst);
+    TypeHandle th = TypeHandle(CoreLibBinder::GetClass(CLASS__BYREFERENCE));
     return th;
 }
 
-// Get MethodDesc* for ByReference<System.Byte>::get_Value
-static MethodDesc* GetByReferenceOfByteValueGetter()
+static FieldDesc* GetByReferenceValueField()
 {
-    MethodDesc* getter = CoreLibBinder::GetMethod(METHOD__BYREFERENCE__GET_VALUE);
-    getter =
-        MethodDesc::FindOrCreateAssociatedMethodDesc(
-                getter,
-                GetByReferenceOfByteType().GetMethodTable(),
-                false,
-                Instantiation(),
-                TRUE);
-
-    return getter;
+    FieldDesc* pFD = CoreLibBinder::GetField(FIELD__BYREFERENCE__VALUE);
+    return pFD;
 }
 
-// Get MethodDesc* for ByReference<System.Byte>::.ctor
-static MethodDesc* GetByReferenceOfByteCtor()
+static MethodDesc* GetByReferenceOfCtor()
 {
-    MethodDesc* ctor = CoreLibBinder::GetMethod(METHOD__BYREFERENCE__CTOR);
-    ctor =
-        MethodDesc::FindOrCreateAssociatedMethodDesc(
-                ctor,
-                GetByReferenceOfByteType().GetMethodTable(),
-                false,
-                Instantiation(),
-                TRUE);
-
-    return ctor;
+    MethodDesc* pMD = CoreLibBinder::GetMethod(METHOD__BYREFERENCE__CTOR);
+    return pMD;
 }
 
 void TailCallHelp::EmitLoadTyHnd(ILCodeStream* stream, TypeHandle tyHnd)
 {
     if (tyHnd.IsByRef())
-        stream->EmitCALL(stream->GetToken(GetByReferenceOfByteValueGetter()), 1, 1);
+        stream->EmitLDFLD(stream->GetToken(GetByReferenceValueField()));
     else
         stream->EmitLDOBJ(stream->GetToken(tyHnd));
 }
@@ -683,8 +664,8 @@ void TailCallHelp::EmitStoreTyHnd(ILCodeStream* stream, TypeHandle tyHnd)
 {
     if (tyHnd.IsByRef())
     {
-        stream->EmitNEWOBJ(stream->GetToken(GetByReferenceOfByteCtor()), 1);
-        stream->EmitSTOBJ(stream->GetToken(GetByReferenceOfByteType()));
+        stream->EmitNEWOBJ(stream->GetToken(GetByReferenceOfCtor()), 1);
+        stream->EmitSTOBJ(stream->GetToken(GetByReferenceType()));
     }
     else
     {
