@@ -1,12 +1,8 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-#nullable enable
-
 using System;
-using System.Diagnostics.CodeAnalysis;
 using System.Reflection;
-using System.Runtime.Serialization;
 
 namespace Microsoft.Extensions.Internal
 {
@@ -29,4 +25,4 @@ namespace Microsoft.Extensions.Internal
             }
         }
     }
-}
\ No newline at end of file
+}
