@@ -10,16 +10,16 @@ namespace Microsoft.Extensions.Logging
 {
     internal static class ProviderAliasUtilities
     {
-        private const string AliasAttibuteTypeFullName = "Microsoft.Extensions.Logging.ProviderAliasAttribute";
+        private const string AliasAttributeTypeFullName = "Microsoft.Extensions.Logging.ProviderAliasAttribute";
 
-        internal static string GetAlias(Type providerType)
+        internal static string? GetAlias(Type providerType)
         {
             IList<CustomAttributeData> attributes = CustomAttributeData.GetCustomAttributes(providerType);
 
             for (int i = 0; i < attributes.Count; i++)
             {
                 CustomAttributeData attributeData = attributes[i];
-                if (attributeData.AttributeType.FullName == AliasAttibuteTypeFullName &&
+                if (attributeData.AttributeType.FullName == AliasAttributeTypeFullName &&
                     attributeData.ConstructorArguments.Count > 0)
                 {
                     CustomAttributeTypedArgument arg = attributeData.ConstructorArguments[0];
