@@ -17,7 +17,10 @@ internal static partial class Interop
             internal ushort YPixel;
         };
 
-        [DllImport(Libraries.SystemNative, EntryPoint = "SystemNative_GetWindowSize", SetLastError = true)]
-        internal static extern int GetWindowSize(out WinSize winSize);
+        [LibraryImport(Libraries.SystemNative, EntryPoint = "SystemNative_GetWindowSize", SetLastError = true)]
+        internal static partial int GetWindowSize(out WinSize winSize);
+
+        [LibraryImport(Libraries.SystemNative, EntryPoint = "SystemNative_SetWindowSize", SetLastError = true)]
+        internal static partial int SetWindowSize(in WinSize winSize);
     }
 }
