@@ -8,11 +8,12 @@ internal static partial class Interop
 {
     internal static partial class Advapi32
     {
-        [DllImport(Libraries.Advapi32, CharSet = CharSet.Unicode, SetLastError = true)]
-        public static extern bool CryptEncrypt(
-            SafeKeyHandle hKey,
+        [LibraryImport(Libraries.Advapi32, SetLastError = true)]
+        [return: MarshalAs(UnmanagedType.Bool)]
+        public static partial bool CryptEncrypt(
+            SafeCapiKeyHandle hKey,
             SafeHashHandle hHash,
-            bool Final,
+            [MarshalAs(UnmanagedType.Bool)] bool Final,
             int dwFlags,
             byte[]? pbData,
             ref int pdwDataLen,
