@@ -8,8 +8,8 @@ internal static partial class Interop
 {
     internal static partial class Gdi32
     {
-        [DllImport(Libraries.Gdi32, ExactSpelling = true, SetLastError = true)]
-        public static extern int BitBlt(IntPtr hdc, int x, int y, int cx, int cy,
+        [LibraryImport(Libraries.Gdi32, SetLastError = true)]
+        public static partial int BitBlt(IntPtr hdc, int x, int y, int cx, int cy,
                                         IntPtr hdcSrc, int x1, int y1, RasterOp rop);
 
         public static int BitBlt(HandleRef hdc, int x, int y, int cx, int cy,
