@@ -8,8 +8,8 @@ internal static partial class Interop
 {
     internal static partial class Gdi32
     {
-        [DllImport(Libraries.Gdi32, SetLastError = true, ExactSpelling = true)]
-        public static extern RegionType SelectClipRgn(IntPtr hdc, IntPtr hrgn);
+        [LibraryImport(Libraries.Gdi32, SetLastError = true)]
+        public static partial RegionType SelectClipRgn(IntPtr hdc, IntPtr hrgn);
 
         public static RegionType SelectClipRgn(HandleRef hdc, HandleRef hrgn)
         {
