@@ -9,8 +9,8 @@ internal static partial class Interop
 {
     internal static partial class Kernel32
     {
-        [DllImport(Libraries.Kernel32, SetLastError = true)]
+        [LibraryImport(Libraries.Kernel32, SetLastError = true)]
         [return: MarshalAs(UnmanagedType.Bool)]
-        internal static extern bool DisconnectNamedPipe(SafePipeHandle hNamedPipe);
+        internal static partial bool DisconnectNamedPipe(SafePipeHandle hNamedPipe);
     }
 }
