@@ -8,10 +8,10 @@ internal static partial class Interop
 {
     internal static partial class Kernel32
     {
-        [DllImport(Libraries.Kernel32)]
+        [LibraryImport(Libraries.Kernel32)]
 #if !NO_SUPPRESS_GC_TRANSITION
         [SuppressGCTransition]
 #endif
-        internal static extern IntPtr GetStdHandle(int nStdHandle);  // param is NOT a handle, but it returns one!
+        internal static partial IntPtr GetStdHandle(int nStdHandle);  // param is NOT a handle, but it returns one!
     }
 }
