@@ -7,8 +7,8 @@ internal static partial class Interop
 {
     internal static partial class Kernel32
     {
-        [DllImport(Libraries.Kernel32)]
-        internal static extern unsafe int MultiByteToWideChar(
+        [LibraryImport(Libraries.Kernel32)]
+        internal static unsafe partial int MultiByteToWideChar(
             uint CodePage, uint dwFlags,
             byte* lpMultiByteStr, int cbMultiByte,
             char* lpWideCharStr, int cchWideChar);
