@@ -24,14 +24,9 @@ internal static partial class Interop
             internal short wProcessorRevision;
         }
 
-        internal enum ProcessorArchitecture : ushort
-        {
-            Processor_Architecture_INTEL = 0,
-            Processor_Architecture_ARM = 5,
-            Processor_Architecture_IA64 = 6,
-            Processor_Architecture_AMD64 = 9,
-            Processor_Architecture_ARM64 = 12,
-            Processor_Architecture_UNKNOWN = 0xFFFF
-        }
+        internal const int PROCESSOR_ARCHITECTURE_INTEL = 0;
+        internal const int PROCESSOR_ARCHITECTURE_ARM = 5;
+        internal const int PROCESSOR_ARCHITECTURE_AMD64 = 9;
+        internal const int PROCESSOR_ARCHITECTURE_ARM64 = 12;
     }
 }
