@@ -7,7 +7,8 @@ internal static partial class Interop
 {
     internal static partial class Kernel32
     {
-        [DllImport(Libraries.Kernel32, CharSet = CharSet.Unicode, SetLastError = true, EntryPoint = "SetConsoleTitleW")]
-        internal static extern bool SetConsoleTitle(string title);
+        [LibraryImport(Libraries.Kernel32, EntryPoint = "SetConsoleTitleW",  SetLastError = true, StringMarshalling = StringMarshalling.Utf16)]
+        [return: MarshalAs(UnmanagedType.Bool)]
+        internal static partial bool SetConsoleTitle(string title);
     }
 }
