@@ -9,7 +9,7 @@ internal static partial class Interop
     internal static partial class NtDll
     {
         // https://msdn.microsoft.com/en-us/library/windows/desktop/ms680600(v=vs.85).aspx
-        [DllImport(Libraries.NtDll, ExactSpelling = true)]
-        public static extern uint RtlNtStatusToDosError(int Status);
+        [LibraryImport(Libraries.NtDll)]
+        public static partial uint RtlNtStatusToDosError(int Status);
     }
 }
