@@ -8,7 +8,7 @@ internal static partial class Interop
 {
     internal static partial class SspiCli
     {
-        [DllImport(Interop.Libraries.SspiCli)]
-        internal static extern int LsaDeregisterLogonProcess(IntPtr LsaHandle);
+        [LibraryImport(Interop.Libraries.SspiCli)]
+        internal static partial int LsaDeregisterLogonProcess(IntPtr LsaHandle);
     }
 }
