@@ -163,7 +163,7 @@ namespace System.Net
             return errorCode;
         }
 
-        internal static int CompleteAuthToken(ISSPIInterface secModule, ref SafeDeleteSslContext? context, in SecurityBuffer inputBuffer)
+        internal static int CompleteAuthToken(ISSPIInterface secModule, ref SafeDeleteSslContext? context, in InputSecurityBuffer inputBuffer)
         {
             int errorCode = secModule.CompleteAuthToken(ref context, in inputBuffer);
 
@@ -172,7 +172,7 @@ namespace System.Net
             return errorCode;
         }
 
-        internal static int ApplyControlToken(ISSPIInterface secModule, ref SafeDeleteContext? context, in SecurityBuffer inputBuffer)
+        internal static int ApplyControlToken(ISSPIInterface secModule, ref SafeDeleteSslContext? context, in SecurityBuffer inputBuffer)
         {
             int errorCode = secModule.ApplyControlToken(ref context, in inputBuffer);
 
@@ -186,153 +186,13 @@ namespace System.Net
             return secModule.QuerySecurityContextToken(context, out token);
         }
 
-        public static int EncryptMessage(ISSPIInterface secModule, SafeDeleteContext context, Span<SecurityBuffer> input, uint sequenceNumber)
-        {
-            return EncryptDecryptHelper(OP.Encrypt, secModule, context, input, sequenceNumber);
-        }
-
-        public static int DecryptMessage(ISSPIInterface secModule, SafeDeleteContext context, Span<SecurityBuffer> input, uint sequenceNumber)
-        {
-            return EncryptDecryptHelper(OP.Decrypt, secModule, context, input, sequenceNumber);
-        }
-
-        internal static int MakeSignature(ISSPIInterface secModule, SafeDeleteContext context, Span<SecurityBuffer> input, uint sequenceNumber)
-        {
-            return EncryptDecryptHelper(OP.MakeSignature, secModule, context, input, sequenceNumber);
-        }
-
-        public static int VerifySignature(ISSPIInterface secModule, SafeDeleteContext context, Span<SecurityBuffer> input, uint sequenceNumber)
-        {
-            return EncryptDecryptHelper(OP.VerifySignature, secModule, context, input, sequenceNumber);
-        }
-
-        private enum OP
-        {
-            Encrypt = 1,
-            Decrypt,
-            MakeSignature,
-            VerifySignature
-        }
-
-        [StructLayout(LayoutKind.Sequential)]
-        private ref struct ThreeByteArrays
-        {
-            public const int NumItems = 3;
-            internal byte[] _item0;
-            private byte[] _item1;
-            private byte[] _item2;
-        }
-
-        private static unsafe int EncryptDecryptHelper(OP op, ISSPIInterface secModule, SafeDeleteContext context, Span<SecurityBuffer> input, uint sequenceNumber)
-        {
-            Debug.Assert(Enum.IsDefined<OP>(op), $"Unknown op: {op}");
-            Debug.Assert(input.Length <= 3, "The below logic only works for 3 or fewer buffers.");
-
-            Interop.SspiCli.SecBufferDesc sdcInOut = new Interop.SspiCli.SecBufferDesc(input.Length);
-            Span<Interop.SspiCli.SecBuffer> unmanagedBuffer = stackalloc Interop.SspiCli.SecBuffer[input.Length];
-            unmanagedBuffer.Clear();
-
-            fixed (Interop.SspiCli.SecBuffer* unmanagedBufferPtr = unmanagedBuffer)
-            fixed (byte* pinnedBuffer0 = input.Length > 0 ? input[0].token : null)
-            fixed (byte* pinnedBuffer1 = input.Length > 1 ? input[1].token : null)
-            fixed (byte* pinnedBuffer2 = input.Length > 2 ? input[2].token : null)
-            {
-                sdcInOut.pBuffers = unmanagedBufferPtr;
-
-                ThreeByteArrays byteArrayStruct = default;
-                Span<byte[]> buffers = MemoryMarshal.CreateSpan(ref byteArrayStruct._item0!, ThreeByteArrays.NumItems).Slice(0, input.Length);
-
-                for (int i = 0; i < input.Length; i++)
-                {
-                    ref readonly SecurityBuffer iBuffer = ref input[i];
-                    unmanagedBuffer[i].cbBuffer = iBuffer.size;
-                    unmanagedBuffer[i].BufferType = iBuffer.type;
-                    if (iBuffer.token == null || iBuffer.token.Length == 0)
-                    {
-                        unmanagedBuffer[i].pvBuffer = IntPtr.Zero;
-                    }
-                    else
-                    {
-                        unmanagedBuffer[i].pvBuffer = Marshal.UnsafeAddrOfPinnedArrayElement(iBuffer.token, iBuffer.offset);
-                        buffers[i] = iBuffer.token;
-                    }
-                }
-
-                // The result is written in the input Buffer passed as type=BufferType.Data.
-                int errorCode = op switch
-                {
-                    OP.Encrypt => secModule.EncryptMessage(context, ref sdcInOut, sequenceNumber),
-                    OP.Decrypt => secModule.DecryptMessage(context, ref sdcInOut, sequenceNumber),
-                    OP.MakeSignature => secModule.MakeSignature(context, ref sdcInOut, sequenceNumber),
-                    _ /* OP.VerifySignature */ => secModule.VerifySignature(context, ref sdcInOut, sequenceNumber),
-                };
-
-                // Marshalling back returned sizes / data.
-                for (int i = 0; i < input.Length; i++)
-                {
-                    ref SecurityBuffer iBuffer = ref input[i];
-                    iBuffer.size = unmanagedBuffer[i].cbBuffer;
-                    iBuffer.type = unmanagedBuffer[i].BufferType;
-
-                    if (iBuffer.size == 0)
-                    {
-                        iBuffer.offset = 0;
-                        iBuffer.token = null;
-                    }
-                    else
-                    {
-
-                        // Find the buffer this is inside of.  Usually they all point inside buffer 0.
-                        int j;
-                        for (j = 0; j < input.Length; j++)
-                        {
-                            if (buffers[j] != null)
-                            {
-                                checked
-                                {
-                                    byte* bufferAddress = (byte*)Marshal.UnsafeAddrOfPinnedArrayElement(buffers[j], 0);
-                                    if ((byte*)unmanagedBuffer[i].pvBuffer >= bufferAddress &&
-                                        (byte*)unmanagedBuffer[i].pvBuffer + iBuffer.size <= bufferAddress + buffers[j].Length)
-                                    {
-                                        iBuffer.offset = (int)((byte*)unmanagedBuffer[i].pvBuffer - bufferAddress);
-                                        iBuffer.token = buffers[j];
-                                        break;
-                                    }
-                                }
-                            }
-                        }
-
-                        if (j >= input.Length)
-                        {
-                            Debug.Fail("Output buffer out of range.");
-                            iBuffer.size = 0;
-                            iBuffer.offset = 0;
-                            iBuffer.token = null;
-                        }
-                    }
-
-                    // Backup validate the new sizes.
-                    Debug.Assert(iBuffer.offset >= 0 && iBuffer.offset <= (iBuffer.token == null ? 0 : iBuffer.token.Length), $"'offset' out of range.  [{iBuffer.offset}]");
-                    Debug.Assert(iBuffer.size >= 0 && iBuffer.size <= (iBuffer.token == null ? 0 : iBuffer.token.Length - iBuffer.offset), $"'size' out of range.  [{iBuffer.size}]");
-                }
-
-                if (NetEventSource.Log.IsEnabled() && errorCode != 0)
-                {
-                    NetEventSource.Error(null, errorCode == Interop.SspiCli.SEC_I_RENEGOTIATE ?
-                        SR.Format(SR.event_OperationReturnedSomething, op, "SEC_I_RENEGOTIATE") :
-                        SR.Format(SR.net_log_operation_failed_with_error, op, $"0x{0:X}"));
-                }
-
-                return errorCode;
-            }
-        }
-
         public static SafeFreeContextBufferChannelBinding? QueryContextChannelBinding(ISSPIInterface secModule, SafeDeleteContext securityContext, Interop.SspiCli.ContextAttribute contextAttribute)
         {
             SafeFreeContextBufferChannelBinding result;
             int errorCode = secModule.QueryContextChannelBinding(securityContext, contextAttribute, out result);
             if (errorCode != 0)
             {
+                result.Dispose();
                 if (NetEventSource.Log.IsEnabled()) NetEventSource.Error(null, $"ERROR = {ErrorDescription(errorCode)}");
                 return null;
             }
@@ -342,20 +202,12 @@ namespace System.Net
 
         public static bool QueryBlittableContextAttributes<T>(ISSPIInterface secModule, SafeDeleteContext securityContext, Interop.SspiCli.ContextAttribute contextAttribute, ref T attribute) where T : unmanaged
         {
-            Span<T> span =
-#if NETSTANDARD2_0
-                stackalloc T[1] { attribute };
-#else
-                MemoryMarshal.CreateSpan(ref attribute, 1);
-#endif
+            Span<T> span = new Span<T>(ref attribute);
             int errorCode = secModule.QueryContextAttributes(
                 securityContext, contextAttribute,
                 MemoryMarshal.AsBytes(span),
                 null,
                 out SafeHandle? sspiHandle);
-#if NETSTANDARD2_0
-            attribute = span[0];
-#endif
 
             using (sspiHandle)
             {
@@ -371,20 +223,12 @@ namespace System.Net
 
         public static bool QueryBlittableContextAttributes<T>(ISSPIInterface secModule, SafeDeleteContext securityContext, Interop.SspiCli.ContextAttribute contextAttribute, Type safeHandleType, out SafeHandle? sspiHandle, ref T attribute) where T : unmanaged
         {
-            Span<T> span =
-#if NETSTANDARD2_0
-                stackalloc T[1] { attribute };
-#else
-                MemoryMarshal.CreateSpan(ref attribute, 1);
-#endif
+            Span<T> span = new Span<T>(ref attribute);
             int errorCode = secModule.QueryContextAttributes(
                 securityContext, contextAttribute,
                 MemoryMarshal.AsBytes(span),
                 safeHandleType,
                 out sspiHandle);
-#if NETSTANDARD2_0
-            attribute = span[0];
-#endif
 
             if (errorCode != 0)
             {
@@ -460,24 +304,15 @@ namespace System.Net
         public static bool QueryContextAttributes_SECPKG_ATTR_REMOTE_CERT_CHAIN(ISSPIInterface secModule, SafeDeleteContext securityContext, out SafeFreeCertContext? certContext)
             => QueryCertContextAttribute(secModule, securityContext, Interop.SspiCli.ContextAttribute.SECPKG_ATTR_REMOTE_CERT_CHAIN, out certContext);
 
-
         public static bool QueryContextAttributes_SECPKG_ATTR_ISSUER_LIST_EX(ISSPIInterface secModule, SafeDeleteContext securityContext, ref Interop.SspiCli.SecPkgContext_IssuerListInfoEx ctx, out SafeHandle? sspiHandle)
         {
-            Span<Interop.SspiCli.SecPkgContext_IssuerListInfoEx> buffer =
-#if NETSTANDARD2_0
-                stackalloc Interop.SspiCli.SecPkgContext_IssuerListInfoEx[1] { ctx };
-#else
-                MemoryMarshal.CreateSpan(ref ctx, 1);
-#endif
+            Span<Interop.SspiCli.SecPkgContext_IssuerListInfoEx> buffer = new Span<Interop.SspiCli.SecPkgContext_IssuerListInfoEx>(ref ctx);
             int errorCode = secModule.QueryContextAttributes(
                 securityContext,
                 Interop.SspiCli.ContextAttribute.SECPKG_ATTR_ISSUER_LIST_EX,
                 MemoryMarshal.AsBytes(buffer),
                 typeof(SafeFreeContextBuffer),
                 out sspiHandle);
-#if NETSTANDARD2_0
-            ctx = buffer[0];
-#endif
 
             if (errorCode != 0)
             {
