@@ -8,7 +8,7 @@ internal static partial class Interop
 {
     internal static partial class User32
     {
-        [DllImport(Libraries.User32, CharSet = CharSet.Unicode, ExactSpelling = true)]
-        public static extern void PostQuitMessage(int exitCode);
+        [LibraryImport(Libraries.User32)]
+        public static partial void PostQuitMessage(int exitCode);
     }
 }
