@@ -8,8 +8,8 @@ internal static partial class Interop
 {
     internal static partial class Winsock
     {
-        [DllImport(Interop.Libraries.Ws2_32, SetLastError = true)]
-        internal static extern unsafe SocketError getsockname(
+        [LibraryImport(Interop.Libraries.Ws2_32, SetLastError = true)]
+        internal static unsafe partial SocketError getsockname(
             SafeSocketHandle socketHandle,
             byte* socketAddress,
             int* socketAddressSize);
