@@ -8,58 +8,100 @@ using System.Diagnostics;
 
 namespace System.Security.Cryptography
 {
-    internal sealed class RsaPaddingProcessor
+    internal static class RsaPaddingProcessor
     {
         // DigestInfo header values taken from https://tools.ietf.org/html/rfc3447#section-9.2, Note 1.
-        private static readonly byte[] s_digestInfoMD5 =
+        private static ReadOnlySpan<byte> DigestInfoMD5 => new byte[]
             {
                 0x30, 0x20, 0x30, 0x0C, 0x06, 0x08, 0x2A, 0x86,
                 0x48, 0x86, 0xF7, 0x0D, 0x02, 0x05, 0x05, 0x00,
                 0x04, 0x10,
             };
 
-        private static readonly byte[] s_digestInfoSha1 =
+        private static ReadOnlySpan<byte> DigestInfoSha1 => new byte[]
             {
                 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2B, 0x0E, 0x03,
                 0x02, 0x1A, 0x05, 0x00, 0x04, 0x14,
             };
 
-        private static readonly byte[] s_digestInfoSha256 =
+        private static ReadOnlySpan<byte> DigestInfoSha256 => new byte[]
             {
                 0x30, 0x31, 0x30, 0x0D, 0x06, 0x09, 0x60, 0x86, 0x48,
                 0x01, 0x65, 0x03, 0x04, 0x02, 0x01, 0x05, 0x00, 0x04,
                 0x20,
             };
 
-        private static readonly byte[] s_digestInfoSha384 =
+        private static ReadOnlySpan<byte> DigestInfoSha384 => new byte[]
             {
                 0x30, 0x41, 0x30, 0x0D, 0x06, 0x09, 0x60, 0x86, 0x48,
                 0x01, 0x65, 0x03, 0x04, 0x02, 0x02, 0x05, 0x00, 0x04,
                 0x30,
             };
 
-        private static readonly byte[] s_digestInfoSha512 =
+        private static ReadOnlySpan<byte> DigestInfoSha512 => new byte[]
             {
                 0x30, 0x51, 0x30, 0x0D, 0x06, 0x09, 0x60, 0x86, 0x48,
                 0x01, 0x65, 0x03, 0x04, 0x02, 0x03, 0x05, 0x00, 0x04,
                 0x40,
             };
 
-        private static readonly ConcurrentDictionary<HashAlgorithmName, RsaPaddingProcessor> s_lookup =
-            new ConcurrentDictionary<HashAlgorithmName, RsaPaddingProcessor>();
+        private static ReadOnlySpan<byte> DigestInfoSha3_256 => new byte[]
+            {
+                0x30, 0x31, 0x30, 0x0D, 0x06, 0x09, 0x60, 0x86, 0x48,
+                0x01, 0x65, 0x03, 0x04, 0x02, 0x08, 0x05, 0x00, 0x04,
+                0x20,
+            };
 
-        private readonly HashAlgorithmName _hashAlgorithmName;
-        private readonly int _hLen;
-        private readonly ReadOnlyMemory<byte> _digestInfoPrefix;
+        private static ReadOnlySpan<byte> DigestInfoSha3_384 => new byte[]
+            {
+                0x30, 0x41, 0x30, 0x0D, 0x06, 0x09, 0x60, 0x86, 0x48,
+                0x01, 0x65, 0x03, 0x04, 0x02, 0x09, 0x05, 0x00, 0x04,
+                0x30,
+            };
 
-        private RsaPaddingProcessor(
+        private static ReadOnlySpan<byte> DigestInfoSha3_512 => new byte[]
+            {
+                0x30, 0x51, 0x30, 0x0D, 0x06, 0x09, 0x60, 0x86, 0x48,
+                0x01, 0x65, 0x03, 0x04, 0x02, 0x0A, 0x05, 0x00, 0x04,
+                0x40,
+            };
+
+        private static ReadOnlySpan<byte> EightZeros => new byte[8];
+
+        private static ReadOnlySpan<byte> GetDigestInfoForAlgorithm(
             HashAlgorithmName hashAlgorithmName,
-            int hLen,
-            ReadOnlyMemory<byte> digestInfoPrefix)
+            out int digestLengthInBytes)
         {
-            _hashAlgorithmName = hashAlgorithmName;
-            _hLen = hLen;
-            _digestInfoPrefix = digestInfoPrefix;
+            switch (hashAlgorithmName.Name)
+            {
+                case HashAlgorithmNames.MD5:
+                    digestLengthInBytes = MD5.HashSizeInBytes;
+                    return DigestInfoMD5;
+                case HashAlgorithmNames.SHA1:
+                    digestLengthInBytes = SHA1.HashSizeInBytes;
+                    return DigestInfoSha1;
+                case HashAlgorithmNames.SHA256:
+                    digestLengthInBytes = SHA256.HashSizeInBytes;
+                    return DigestInfoSha256;
+                case HashAlgorithmNames.SHA384:
+                    digestLengthInBytes = SHA384.HashSizeInBytes;
+                    return DigestInfoSha384;
+                case HashAlgorithmNames.SHA512:
+                    digestLengthInBytes = SHA512.HashSizeInBytes;
+                    return DigestInfoSha512;
+                case HashAlgorithmNames.SHA3_256:
+                    digestLengthInBytes = SHA3_256.HashSizeInBytes;
+                    return DigestInfoSha3_256;
+                case HashAlgorithmNames.SHA3_384:
+                    digestLengthInBytes = SHA3_384.HashSizeInBytes;
+                    return DigestInfoSha3_384;
+                case HashAlgorithmNames.SHA3_512:
+                    digestLengthInBytes = SHA3_512.HashSizeInBytes;
+                    return DigestInfoSha3_512;
+                default:
+                    Debug.Fail("Unknown digest algorithm");
+                    throw new CryptographicException();
+            }
         }
 
         internal static int BytesRequiredForBitCount(int keySizeInBits)
@@ -67,57 +109,10 @@ namespace System.Security.Cryptography
             return (int)(((uint)keySizeInBits + 7) / 8);
         }
 
-        private static ReadOnlySpan<byte> EightZeros => new byte[8]; // rely on C# compiler optimization to eliminate allocation
-
-        internal int HashLength => _hLen;
-
-        internal static RsaPaddingProcessor OpenProcessor(HashAlgorithmName hashAlgorithmName)
+        internal static int HashLength(HashAlgorithmName hashAlgorithmName)
         {
-            return s_lookup.GetOrAdd(
-                hashAlgorithmName,
-                static hashAlgorithmName =>
-                {
-                    using (IncrementalHash hasher = IncrementalHash.CreateHash(hashAlgorithmName))
-                    {
-                        // SHA-2-512 is the biggest we expect
-                        Span<byte> stackDest = stackalloc byte[512 / 8];
-                        ReadOnlyMemory<byte> digestInfoPrefix;
-
-                        if (hashAlgorithmName == HashAlgorithmName.MD5)
-                        {
-                            digestInfoPrefix = s_digestInfoMD5;
-                        }
-                        else if (hashAlgorithmName == HashAlgorithmName.SHA1)
-                        {
-                            digestInfoPrefix = s_digestInfoSha1;
-                        }
-                        else if (hashAlgorithmName == HashAlgorithmName.SHA256)
-                        {
-                            digestInfoPrefix = s_digestInfoSha256;
-                        }
-                        else if (hashAlgorithmName == HashAlgorithmName.SHA384)
-                        {
-                            digestInfoPrefix = s_digestInfoSha384;
-                        }
-                        else if (hashAlgorithmName == HashAlgorithmName.SHA512)
-                        {
-                            digestInfoPrefix = s_digestInfoSha512;
-                        }
-                        else
-                        {
-                            Debug.Fail("Unknown digest algorithm");
-                            throw new CryptographicException();
-                        }
-
-                        if (hasher.TryGetHashAndReset(stackDest, out int bytesWritten))
-                        {
-                            return new RsaPaddingProcessor(hashAlgorithmName, bytesWritten, digestInfoPrefix);
-                        }
-
-                        byte[] big = hasher.GetHashAndReset();
-                        return new RsaPaddingProcessor(hashAlgorithmName, big.Length, digestInfoPrefix);
-                    }
-                });
+            GetDigestInfoForAlgorithm(hashAlgorithmName, out int hLen);
+            return hLen;
         }
 
         internal static void PadPkcs1Encryption(
@@ -143,7 +138,7 @@ namespace System.Security.Cryptography
             destination[ps.Length + 2] = 0;
 
             // 2(a). Fill PS with random data from a CSPRNG, but no zero-values.
-            FillNonZeroBytes(ps);
+            RandomNumberGeneratorImplementation.FillNonZeroBytes(ps);
 
             source.CopyTo(mInEM);
         }
@@ -251,7 +246,8 @@ namespace System.Security.Cryptography
             return SignStretch(mask);
         }
 
-        internal void PadPkcs1Signature(
+        internal static void PadPkcs1Signature(
+            HashAlgorithmName hashAlgorithmName,
             ReadOnlySpan<byte> source,
             Span<byte> destination)
         {
@@ -261,7 +257,7 @@ namespace System.Security.Cryptography
             // Done by the caller.
 
             // 2. Encode the DigestInfo value
-            ReadOnlySpan<byte> digestInfoPrefix = _digestInfoPrefix.Span;
+            ReadOnlySpan<byte> digestInfoPrefix = GetDigestInfoForAlgorithm(hashAlgorithmName, out _);
             int expectedLength = digestInfoPrefix[^1];
 
             if (source.Length != expectedLength)
@@ -289,12 +285,14 @@ namespace System.Security.Cryptography
             source.CopyTo(destination.Slice(paddingLength + 3 + digestInfoPrefix.Length));
         }
 
-        internal void PadOaep(
+        internal static void PadOaep(
+            HashAlgorithmName hashAlgorithmName,
             ReadOnlySpan<byte> source,
             Span<byte> destination)
         {
             // https://tools.ietf.org/html/rfc3447#section-7.1.1
 
+            int hLen = HashLength(hashAlgorithmName);
             byte[]? dbMask = null;
             Span<byte> dbMaskSpan = Span<byte>.Empty;
 
@@ -302,7 +300,7 @@ namespace System.Security.Cryptography
             {
                 // Since the biggest known _hLen is 512/8 (64) and destination.Length is 0 or more,
                 // this shouldn't underflow without something having severely gone wrong.
-                int maxInput = checked(destination.Length - _hLen - _hLen - 2);
+                int maxInput = checked(destination.Length - hLen - hLen - 2);
 
                 // 1(a) does not apply, we do not allow custom label values.
 
@@ -315,19 +313,21 @@ namespace System.Security.Cryptography
 
                 // The final message (step 2(i)) will be
                 // 0x00 || maskedSeed (hLen long) || maskedDB (rest of the buffer)
-                Span<byte> seed = destination.Slice(1, _hLen);
-                Span<byte> db = destination.Slice(1 + _hLen);
+                Span<byte> seed = destination.Slice(1, hLen);
+                Span<byte> db = destination.Slice(1 + hLen);
 
-                using (IncrementalHash hasher = IncrementalHash.CreateHash(_hashAlgorithmName))
+                using (IncrementalHash hasher = IncrementalHash.CreateHash(hashAlgorithmName))
                 {
+                    Debug.Assert(hasher.HashLengthInBytes == hLen);
+
                     // DB = lHash || PS || 0x01 || M
-                    Span<byte> lHash = db.Slice(0, _hLen);
+                    Span<byte> lHash = db.Slice(0, hLen);
                     Span<byte> mDest = db.Slice(db.Length - source.Length);
-                    Span<byte> ps = db.Slice(_hLen, db.Length - _hLen - 1 - mDest.Length);
-                    Span<byte> psEnd = db.Slice(_hLen + ps.Length, 1);
+                    Span<byte> ps = db.Slice(hLen, db.Length - hLen - 1 - mDest.Length);
+                    Span<byte> psEnd = db.Slice(hLen + ps.Length, 1);
 
                     // 2(a) lHash = Hash(L), where L is the empty string.
-                    if (!hasher.TryGetHashAndReset(lHash, out int hLen2) || hLen2 != _hLen)
+                    if (!hasher.TryGetHashAndReset(lHash, out int hLen2) || hLen2 != hLen)
                     {
                         Debug.Fail("TryGetHashAndReset failed with exact-size destination");
                         throw new CryptographicException();
@@ -354,7 +354,7 @@ namespace System.Security.Cryptography
                     Xor(db, dbMaskSpan);
 
                     // 2(g)
-                    Span<byte> seedMask = stackalloc byte[_hLen];
+                    Span<byte> seedMask = stackalloc byte[hLen];
                     Mgf1(hasher, db, seedMask);
 
                     // 2(h)
@@ -379,119 +379,27 @@ namespace System.Security.Cryptography
             }
         }
 
-        internal bool DepadOaep(
-            ReadOnlySpan<byte> source,
-            Span<byte> destination,
-            out int bytesWritten)
+        internal static void EncodePss(HashAlgorithmName hashAlgorithmName, ReadOnlySpan<byte> mHash, Span<byte> destination, int keySize)
         {
-            // https://tools.ietf.org/html/rfc3447#section-7.1.2
-            using (IncrementalHash hasher = IncrementalHash.CreateHash(_hashAlgorithmName))
-            {
-                Span<byte> lHash = stackalloc byte[_hLen];
+            int hLen = HashLength(hashAlgorithmName);
 
-                if (!hasher.TryGetHashAndReset(lHash, out int hLen2) || hLen2 != _hLen)
-                {
-                    Debug.Fail("TryGetHashAndReset failed with exact-size destination");
-                    throw new CryptographicException();
-                }
-
-                int y = source[0];
-                ReadOnlySpan<byte> maskedSeed = source.Slice(1, _hLen);
-                ReadOnlySpan<byte> maskedDB = source.Slice(1 + _hLen);
-
-                Span<byte> seed = stackalloc byte[_hLen];
-                // seedMask = MGF(maskedDB, hLen)
-                Mgf1(hasher, maskedDB, seed);
-
-                // seed = seedMask XOR maskedSeed
-                Xor(seed, maskedSeed);
-
-                byte[] tmp = CryptoPool.Rent(source.Length);
-
-                try
-                {
-                    Span<byte> dbMask = new Span<byte>(tmp, 0, maskedDB.Length);
-                    // dbMask = MGF(seed, k - hLen - 1)
-                    Mgf1(hasher, seed, dbMask);
-
-                    // DB = dbMask XOR maskedDB
-                    Xor(dbMask, maskedDB);
-
-                    ReadOnlySpan<byte> lHashPrime = dbMask.Slice(0, _hLen);
-
-                    int separatorPos = int.MaxValue;
-
-                    for (int i = dbMask.Length - 1; i >= _hLen; i--)
-                    {
-                        // if dbMask[i] is 1, val is 0. otherwise val is [01,FF]
-                        byte dbMinus1 = (byte)(dbMask[i] - 1);
-                        int val = dbMinus1;
-
-                        // if val is 0: FFFFFFFF & FFFFFFFF => FFFFFFFF
-                        // if val is any other byte value, val-1 will be in the range 00000000 to 000000FE,
-                        // and so the high bit will not be set.
-                        val = (~val & (val - 1)) >> 31;
-
-                        // if val is 0: separator = (0 & i) | (~0 & separator) => separator
-                        // else: separator = (~0 & i) | (0 & separator) => i
-                        //
-                        // Net result: non-branching "if (dbMask[i] == 1) separatorPos = i;"
-                        separatorPos = (val & i) | (~val & separatorPos);
-                    }
-
-                    bool lHashMatches = CryptographicOperations.FixedTimeEquals(lHash, lHashPrime);
-                    bool yIsZero = y == 0;
-                    bool separatorMadeSense = separatorPos < dbMask.Length;
-
-                    // This intentionally uses non-short-circuiting operations to hide the timing
-                    // differential between the three failure cases
-                    bool shouldContinue = lHashMatches & yIsZero & separatorMadeSense;
-
-                    if (!shouldContinue)
-                    {
-                        throw new CryptographicException(SR.Cryptography_OAEP_Decryption_Failed);
-                    }
-
-                    Span<byte> message = dbMask.Slice(separatorPos + 1);
-
-                    if (message.Length <= destination.Length)
-                    {
-                        message.CopyTo(destination);
-                        bytesWritten = message.Length;
-                        return true;
-                    }
-                    else
-                    {
-                        bytesWritten = 0;
-                        return false;
-                    }
-                }
-                finally
-                {
-                    CryptoPool.Return(tmp, source.Length);
-                }
-            }
-        }
-
-        internal void EncodePss(ReadOnlySpan<byte> mHash, Span<byte> destination, int keySize)
-        {
             // https://tools.ietf.org/html/rfc3447#section-9.1.1
             int emBits = keySize - 1;
             int emLen = BytesRequiredForBitCount(emBits);
 
-            if (mHash.Length != _hLen)
+            if (mHash.Length != hLen)
             {
                 throw new CryptographicException(SR.Cryptography_SignHash_WrongSize);
             }
 
             // In this implementation, sLen is restricted to the length of the input hash.
-            int sLen = _hLen;
+            int sLen = hLen;
 
             // 3.  if emLen < hLen + sLen + 2, encoding error.
             //
             // sLen = hLen in this implementation.
 
-            if (emLen < 2 + _hLen + sLen)
+            if (emLen < 2 + hLen + sLen)
             {
                 throw new CryptographicException(SR.Cryptography_KeyTooSmall);
             }
@@ -503,17 +411,18 @@ namespace System.Security.Cryptography
             // 12. Let EM = maskedDB || H || 0xbc (H has length hLen)
             Span<byte> em = destination.Slice(destination.Length - emLen, emLen);
 
-            int dbLen = emLen - _hLen - 1;
+            int dbLen = emLen - hLen - 1;
 
             Span<byte> db = em.Slice(0, dbLen);
-            Span<byte> hDest = em.Slice(dbLen, _hLen);
+            Span<byte> hDest = em.Slice(dbLen, hLen);
             em[emLen - 1] = 0xBC;
 
             byte[] dbMaskRented = CryptoPool.Rent(dbLen);
             Span<byte> dbMask = new Span<byte>(dbMaskRented, 0, dbLen);
 
-            using (IncrementalHash hasher = IncrementalHash.CreateHash(_hashAlgorithmName))
+            using (IncrementalHash hasher = IncrementalHash.CreateHash(hashAlgorithmName))
             {
+                Debug.Assert(hasher.HashLengthInBytes == hLen);
                 // 4. Generate a random salt of length sLen
                 Span<byte> salt = stackalloc byte[sLen];
                 RandomNumberGenerator.Fill(salt);
@@ -525,7 +434,7 @@ namespace System.Security.Cryptography
                 hasher.AppendData(mHash);
                 hasher.AppendData(salt);
 
-                if (!hasher.TryGetHashAndReset(hDest, out int hLen2) || hLen2 != _hLen)
+                if (!hasher.TryGetHashAndReset(hDest, out int hLen2) || hLen2 != hLen)
                 {
                     Debug.Fail("TryGetHashAndReset failed with exact-size destination");
                     throw new CryptographicException();
@@ -533,7 +442,7 @@ namespace System.Security.Cryptography
 
                 // 7. Generate PS as zero-valued bytes of length emLen - sLen - hLen - 2.
                 // 8. Let DB = PS || 0x01 || salt
-                int psLen = emLen - sLen - _hLen - 2;
+                int psLen = emLen - sLen - hLen - 2;
                 db.Slice(0, psLen).Clear();
                 db[psLen] = 0x01;
                 salt.CopyTo(db.Slice(psLen + 1));
@@ -558,14 +467,16 @@ namespace System.Security.Cryptography
             CryptoPool.Return(dbMaskRented, clearSize: 0);
         }
 
-        internal bool VerifyPss(ReadOnlySpan<byte> mHash, ReadOnlySpan<byte> em, int keySize)
+        internal static bool VerifyPss(HashAlgorithmName hashAlgorithmName, ReadOnlySpan<byte> mHash, ReadOnlySpan<byte> em, int keySize)
         {
+            int hLen = HashLength(hashAlgorithmName);
+
             // https://tools.ietf.org/html/rfc3447#section-9.1.2
 
             int emBits = keySize - 1;
             int emLen = BytesRequiredForBitCount(emBits);
 
-            if (mHash.Length != _hLen)
+            if (mHash.Length != hLen)
             {
                 return false;
             }
@@ -573,10 +484,10 @@ namespace System.Security.Cryptography
             Debug.Assert(em.Length >= emLen);
 
             // In this implementation, sLen is restricted to hLen.
-            int sLen = _hLen;
+            int sLen = hLen;
 
             // 3. If emLen < hLen + sLen + 2, output "inconsistent" and stop.
-            if (emLen < _hLen + sLen + 2)
+            if (emLen < hLen + sLen + 2)
             {
                 return false;
             }
@@ -588,10 +499,10 @@ namespace System.Security.Cryptography
             }
 
             // 5. maskedDB is the leftmost emLen - hLen -1 bytes, H is the next hLen bytes.
-            int dbLen = emLen - _hLen - 1;
+            int dbLen = emLen - hLen - 1;
 
             ReadOnlySpan<byte> maskedDb = em.Slice(0, dbLen);
-            ReadOnlySpan<byte> h = em.Slice(dbLen, _hLen);
+            ReadOnlySpan<byte> h = em.Slice(dbLen, hLen);
 
             // 6. If the unused bits aren't zero, output "inconsistent" and stop.
             int unusedBits = 8 * emLen - emBits;
@@ -608,8 +519,10 @@ namespace System.Security.Cryptography
 
             try
             {
-                using (IncrementalHash hasher = IncrementalHash.CreateHash(_hashAlgorithmName))
+                using (IncrementalHash hasher = IncrementalHash.CreateHash(hashAlgorithmName))
                 {
+                    Debug.Assert(hasher.HashLengthInBytes == hLen);
+
                     Mgf1(hasher, h, dbMask);
 
                     // 8. DB = maskedDB XOR dbMask
@@ -623,7 +536,7 @@ namespace System.Security.Cryptography
                     //
                     // Since signature verification is a public key operation there's no need to
                     // use fixed time equality checking here.
-                    for (int i = emLen - _hLen - sLen - 2 - 1; i >= 0; --i)
+                    for (int i = emLen - hLen - sLen - 2 - 1; i >= 0; --i)
                     {
                         if (dbMask[i] != 0)
                         {
@@ -633,7 +546,7 @@ namespace System.Security.Cryptography
 
                     // 10 ("b") If the octet at position emLen - hLen - sLen - 1 (under a 1-indexed scheme)
                     // is not 0x01, output "inconsistent" and stop.
-                    if (dbMask[emLen - _hLen - sLen - 2] != 0x01)
+                    if (dbMask[emLen - hLen - sLen - 2] != 0x01)
                     {
                         return false;
                     }
@@ -646,9 +559,9 @@ namespace System.Security.Cryptography
                     hasher.AppendData(mHash);
                     hasher.AppendData(salt);
 
-                    Span<byte> hPrime = stackalloc byte[_hLen];
+                    Span<byte> hPrime = stackalloc byte[hLen];
 
-                    if (!hasher.TryGetHashAndReset(hPrime, out int hLen2) || hLen2 != _hLen)
+                    if (!hasher.TryGetHashAndReset(hPrime, out int hLen2) || hLen2 != hLen)
                     {
                         Debug.Fail("TryGetHashAndReset failed with exact-size destination");
                         throw new CryptographicException();
@@ -669,8 +582,9 @@ namespace System.Security.Cryptography
         }
 
         // https://tools.ietf.org/html/rfc3447#appendix-B.2.1
-        private void Mgf1(IncrementalHash hasher, ReadOnlySpan<byte> mgfSeed, Span<byte> mask)
+        private static void Mgf1(IncrementalHash hasher, ReadOnlySpan<byte> mgfSeed, Span<byte> mask)
         {
+            int hLen = hasher.HashLengthInBytes;
             Span<byte> writePtr = mask;
             int count = 0;
             Span<byte> bigEndianCount = stackalloc byte[sizeof(int)];
@@ -681,7 +595,7 @@ namespace System.Security.Cryptography
                 BinaryPrimitives.WriteInt32BigEndian(bigEndianCount, count);
                 hasher.AppendData(bigEndianCount);
 
-                if (writePtr.Length >= _hLen)
+                if (writePtr.Length >= hLen)
                 {
                     if (!hasher.TryGetHashAndReset(writePtr, out int bytesWritten))
                     {
@@ -689,12 +603,12 @@ namespace System.Security.Cryptography
                         throw new CryptographicException();
                     }
 
-                    Debug.Assert(bytesWritten == _hLen);
+                    Debug.Assert(bytesWritten == hLen);
                     writePtr = writePtr.Slice(bytesWritten);
                 }
                 else
                 {
-                    Span<byte> tmp = stackalloc byte[_hLen];
+                    Span<byte> tmp = stackalloc byte[hLen];
 
                     if (!hasher.TryGetHashAndReset(tmp, out int bytesWritten))
                     {
@@ -702,7 +616,7 @@ namespace System.Security.Cryptography
                         throw new CryptographicException();
                     }
 
-                    Debug.Assert(bytesWritten == _hLen);
+                    Debug.Assert(bytesWritten == hLen);
                     tmp.Slice(0, writePtr.Length).CopyTo(writePtr);
                     break;
                 }
@@ -711,41 +625,6 @@ namespace System.Security.Cryptography
             }
         }
 
-        // This is a copy of RandomNumberGeneratorImplementation.GetNonZeroBytes, but adapted
-        // to the object-less RandomNumberGenerator.Fill.
-        private static void FillNonZeroBytes(Span<byte> data)
-        {
-            while (data.Length > 0)
-            {
-                // Fill the remaining portion of the span with random bytes.
-                RandomNumberGenerator.Fill(data);
-
-                // Find the first zero in the remaining portion.
-                int indexOfFirst0Byte = data.Length;
-                for (int i = 0; i < data.Length; i++)
-                {
-                    if (data[i] == 0)
-                    {
-                        indexOfFirst0Byte = i;
-                        break;
-                    }
-                }
-
-                // If there were any zeros, shift down all non-zeros.
-                for (int i = indexOfFirst0Byte + 1; i < data.Length; i++)
-                {
-                    if (data[i] != 0)
-                    {
-                        data[indexOfFirst0Byte++] = data[i];
-                    }
-                }
-
-                // Request new random bytes if necessary; dont re-use
-                // existing bytes since they were shifted down.
-                data = data.Slice(indexOfFirst0Byte);
-            }
-        }
-
         /// <summary>
         /// Bitwise XOR of <paramref name="b"/> into <paramref name="a"/>.
         /// </summary>
