@@ -1,7 +1,6 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-#nullable enable
 using System.Diagnostics;
 using System.Diagnostics.CodeAnalysis;
 using System.Runtime.CompilerServices;
@@ -160,14 +159,14 @@ namespace System.IO
             }
 
             public override IAsyncResult BeginRead(byte[] buffer, int offset, int count, AsyncCallback? callback, object? state) =>
-                TaskToApm.Begin(ReadAsync(buffer, offset, count), callback, state);
+                TaskToAsyncResult.Begin(ReadAsync(buffer, offset, count), callback, state);
 
             public override int EndRead(IAsyncResult asyncResult)
             {
                 ThrowIfDisposed();
                 ThrowIfReadingNotSupported();
 
-                return TaskToApm.End<int>(asyncResult);
+                return TaskToAsyncResult.End<int>(asyncResult);
             }
 
             public override Task<int> ReadAsync(byte[] buffer, int offset, int count, CancellationToken cancellationToken)
@@ -230,14 +229,14 @@ namespace System.IO
             }
 
             public override IAsyncResult BeginWrite(byte[] buffer, int offset, int count, AsyncCallback? callback, object? state) =>
-                TaskToApm.Begin(WriteAsync(buffer, offset, count), callback, state);
+                TaskToAsyncResult.Begin(WriteAsync(buffer, offset, count), callback, state);
 
             public override void EndWrite(IAsyncResult asyncResult)
             {
                 ThrowIfDisposed();
                 ThrowIfWritingNotSupported();
 
-                TaskToApm.End(asyncResult);
+                TaskToAsyncResult.End(asyncResult);
             }
 
             public override long Length => throw new NotSupportedException();
@@ -349,12 +348,12 @@ namespace System.IO
             }
 
             public override IAsyncResult BeginRead(byte[] buffer, int offset, int count, AsyncCallback? callback, object? state) =>
-                TaskToApm.Begin(ReadAsync(buffer, offset, count), callback, state);
+                TaskToAsyncResult.Begin(ReadAsync(buffer, offset, count), callback, state);
 
             public override int EndRead(IAsyncResult asyncResult)
             {
                 ThrowIfDisposed();
-                return TaskToApm.End<int>(asyncResult);
+                return TaskToAsyncResult.End<int>(asyncResult);
             }
 
             public override Task<int> ReadAsync(byte[] buffer, int offset, int count, CancellationToken cancellationToken)
@@ -403,12 +402,12 @@ namespace System.IO
             }
 
             public override IAsyncResult BeginWrite(byte[] buffer, int offset, int count, AsyncCallback? callback, object? state) =>
-                TaskToApm.Begin(WriteAsync(buffer, offset, count), callback, state);
+                TaskToAsyncResult.Begin(WriteAsync(buffer, offset, count), callback, state);
 
             public override void EndWrite(IAsyncResult asyncResult)
             {
                 ThrowIfDisposed();
-                TaskToApm.End(asyncResult);
+                TaskToAsyncResult.End(asyncResult);
             }
 
             public override long Length => throw new NotSupportedException();
