@@ -11,6 +11,9 @@ namespace System.Security.Cryptography.Rsa.Tests
         bool SupportsLargeExponent { get; }
         bool SupportsSha2Oaep { get; }
         bool SupportsPss { get; }
+        bool SupportsSha1Signatures { get; }
+        bool SupportsMd5Signatures { get; }
+        bool SupportsSha3 { get; }
     }
 
     public static partial class RSAFactory
@@ -39,5 +42,11 @@ namespace System.Security.Cryptography.Rsa.Tests
         public static bool SupportsSha2Oaep => s_provider.SupportsSha2Oaep;
 
         public static bool SupportsPss => s_provider.SupportsPss;
+
+        public static bool SupportsSha1Signatures => s_provider.SupportsSha1Signatures;
+        public static bool SupportsMd5Signatures => s_provider.SupportsMd5Signatures;
+
+        public static bool SupportsSha3 => s_provider.SupportsSha3;
+        public static bool NoSupportsSha3 => !SupportsSha3;
     }
 }
