@@ -8,7 +8,7 @@ namespace Microsoft.Extensions.Options
     /// <summary>
     /// Used to fetch <see cref="IChangeToken"/> used for tracking options changes.
     /// </summary>
-    /// <typeparam name="TOptions">Options type.</typeparam>
+    /// <typeparam name="TOptions">The options type being changed.</typeparam>
     public interface IOptionsChangeTokenSource<out TOptions>
     {
         /// <summary>
@@ -20,6 +20,6 @@ namespace Microsoft.Extensions.Options
         /// <summary>
         /// The name of the option instance being changed.
         /// </summary>
-        string Name { get; }
+        string? Name { get; }
     }
 }
