@@ -92,9 +92,9 @@ namespace System.Collections.Immutable
                     Requires.NotNull(value, nameof(value));
                     if (value != this.KeyComparer)
                     {
-                        var comparers = Comparers.Get(value, this.ValueComparer);
+                        Comparers comparers = Comparers.Get(value, this.ValueComparer);
                         var input = new MutationInput(SortedInt32KeyNode<HashBucket>.EmptyNode, comparers);
-                        var result = ImmutableDictionary<TKey, TValue>.AddRange(this, input);
+                        ImmutableDictionary<TKey, TValue>.MutationResult result = ImmutableDictionary<TKey, TValue>.AddRange(this, input);
 
                         _immutable = null;
                         _comparers = comparers;
@@ -346,7 +346,7 @@ namespace System.Collections.Immutable
                 Requires.Range(arrayIndex >= 0, nameof(arrayIndex));
                 Requires.Range(array.Length >= arrayIndex + this.Count, nameof(arrayIndex));
 
-                foreach (var item in this)
+                foreach (KeyValuePair<TKey, TValue> item in this)
                 {
                     array.SetValue(new DictionaryEntry(item.Key, item.Value), arrayIndex++);
                 }
@@ -419,7 +419,7 @@ namespace System.Collections.Immutable
 
                 set
                 {
-                    var result = ImmutableDictionary<TKey, TValue>.Add(key, value, KeyCollisionBehavior.SetValue, this.Origin);
+                    ImmutableDictionary<TKey, TValue>.MutationResult result = ImmutableDictionary<TKey, TValue>.Add(key, value, KeyCollisionBehavior.SetValue, this.Origin);
                     this.Apply(result);
                 }
             }
@@ -432,7 +432,7 @@ namespace System.Collections.Immutable
             /// <param name="items">The items.</param>
             public void AddRange(IEnumerable<KeyValuePair<TKey, TValue>> items)
             {
-                var result = ImmutableDictionary<TKey, TValue>.AddRange(items, this.Origin);
+                ImmutableDictionary<TKey, TValue>.MutationResult result = ImmutableDictionary<TKey, TValue>.AddRange(items, this.Origin);
                 this.Apply(result);
             }
 
@@ -444,7 +444,7 @@ namespace System.Collections.Immutable
             {
                 Requires.NotNull(keys, nameof(keys));
 
-                foreach (var key in keys)
+                foreach (TKey key in keys)
                 {
                     this.Remove(key);
                 }
@@ -505,12 +505,7 @@ namespace System.Collections.Immutable
                 // Creating an instance of ImmutableSortedMap<T> with our root node automatically freezes our tree,
                 // ensuring that the returned instance is immutable.  Any further mutations made to this builder
                 // will clone (and unfreeze) the spine of modified nodes until the next time this method is invoked.
-                if (_immutable == null)
-                {
-                    _immutable = ImmutableDictionary<TKey, TValue>.Wrap(_root, _comparers, _count);
-                }
-
-                return _immutable;
+                return _immutable ??= ImmutableDictionary<TKey, TValue>.Wrap(_root, _comparers, _count);
             }
 
             #endregion
@@ -527,7 +522,7 @@ namespace System.Collections.Immutable
             /// <exception cref="NotSupportedException">The <see cref="IDictionary{TKey, TValue}"/> is read-only.</exception>
             public void Add(TKey key, TValue value)
             {
-                var result = ImmutableDictionary<TKey, TValue>.Add(key, value, KeyCollisionBehavior.ThrowIfValueDifferent, this.Origin);
+                ImmutableDictionary<TKey, TValue>.MutationResult result = ImmutableDictionary<TKey, TValue>.Add(key, value, KeyCollisionBehavior.ThrowIfValueDifferent, this.Origin);
                 this.Apply(result);
             }
 
@@ -580,7 +575,7 @@ namespace System.Collections.Immutable
             /// <exception cref="NotSupportedException">The <see cref="IDictionary{TKey, TValue}"/> is read-only.</exception>
             public bool Remove(TKey key)
             {
-                var result = ImmutableDictionary<TKey, TValue>.Remove(key, this.Origin);
+                ImmutableDictionary<TKey, TValue>.MutationResult result = ImmutableDictionary<TKey, TValue>.Remove(key, this.Origin);
                 return this.Apply(result);
             }
 
@@ -645,7 +640,7 @@ namespace System.Collections.Immutable
             {
                 Requires.NotNull(array, nameof(array));
 
-                foreach (var item in this)
+                foreach (KeyValuePair<TKey, TValue> item in this)
                 {
                     array[arrayIndex++] = item;
                 }
@@ -744,17 +739,6 @@ namespace System.Collections.Immutable
         /// Gets a simple debugger-viewable collection.
         /// </summary>
         [DebuggerBrowsable(DebuggerBrowsableState.RootHidden)]
-        public KeyValuePair<TKey, TValue>[] Contents
-        {
-            get
-            {
-                if (_contents == null)
-                {
-                    _contents = _map.ToArray(_map.Count);
-                }
-
-                return _contents;
-            }
-        }
+        public KeyValuePair<TKey, TValue>[] Contents => _contents ??= _map.ToArray(_map.Count);
     }
 }
