@@ -13,10 +13,10 @@ namespace System.Collections.Immutable.Tests
         [Fact]
         public void DoesNotCacheContents()
         {
-            var set = ImmutableSortedSet<int>.Empty.Add(1);
-            var builder = set.ToBuilder();
+            ImmutableSortedSet<int> set = ImmutableSortedSet<int>.Empty.Add(1);
+            ImmutableSortedSet<int>.Builder builder = set.ToBuilder();
             var debuggerProxy = new ImmutableSortedSetBuilderDebuggerProxy<int>(builder);
-            var contents = debuggerProxy.Contents; // view the contents to trigger caching
+            _ = debuggerProxy.Contents; // view the contents to trigger caching
             builder.Add(2);
             Assert.Equal(builder.ToArray(), debuggerProxy.Contents);
         }
