@@ -269,21 +269,11 @@ namespace System.ComponentModel.Composition.Primitives
         ///         properties.
         ///     </para>
         /// </remarks>
-        public override Expression<Func<ExportDefinition, bool>> Constraint
-        {
-            get
-            {
-                if (_constraint == null)
-                {
-                    _constraint = ConstraintServices.CreateConstraint(ContractName, RequiredTypeIdentity, RequiredMetadata, RequiredCreationPolicy);
-                }
-
-                return _constraint;
-            }
-        }
+        public override Expression<Func<ExportDefinition, bool>> Constraint =>
+            _constraint ??= ConstraintServices.CreateConstraint(ContractName, RequiredTypeIdentity, RequiredMetadata, RequiredCreationPolicy);
 
         /// <summary>
-        ///     Executes an optimized version of the contraint given by the <see cref="Constraint"/> property
+        ///     Executes an optimized version of the constraint given by the <see cref="Constraint"/> property
         /// </summary>
         /// <param name="exportDefinition">
         ///     A definition for a <see cref="Export"/> used to determine if it satisfies the
