@@ -110,7 +110,7 @@ namespace System.Data
 
         private void RowErrorChanged()
         {
-            // We don't know wich record was used by view index. try to use both.
+            // We don't know which record was used by view index. try to use both.
             if (_oldRecord != -1)
             {
                 _table.RecordChanged(_oldRecord);
@@ -228,7 +228,7 @@ namespace System.Data
             DataRelation[] nestedParentRelations = _table.NestedParentRelations;
             foreach (DataRelation rel in nestedParentRelations)
             {
-                if (rel == null) // don't like this but done for backward code compatability
+                if (rel == null) // don't like this but done for backward code compatibility
                 {
                     continue;
                 }
@@ -414,7 +414,7 @@ namespace System.Data
         /// <remarks>
         /// Arrays returned from this property will never contain <see langword="null" />.
         /// When setting this property, <see langword="null" /> indicates that the current column value
-        /// should be left as-is. <see cref="DBNull.Value" /> can be used to set the the value to null.
+        /// should be left as-is. <see cref="DBNull.Value" /> can be used to set the value to null.
         /// </remarks>
         public object?[] ItemArray
         {
@@ -682,7 +682,7 @@ namespace System.Data
 
             if (_newRecord == -1)
             {
-                return; // this is meaningless, detatched row case
+                return; // this is meaningless, detached row case
             }
 
             if (_tempRecord != -1)
@@ -732,7 +732,7 @@ namespace System.Data
             long logScopeId = DataCommonEventSource.Log.EnterScope("<ds.DataRow.SetColumnError|API> {0}, column={1}, error='{2}'", _objectID, column.ObjectID, error);
             try
             {
-                if (_error == null) _error = new DataError();
+                _error ??= new DataError();
                 if (GetColumnError(column) != error)
                 {
                     _error.SetColumnError(column, error);
@@ -761,7 +761,7 @@ namespace System.Data
         public string GetColumnError(DataColumn column)
         {
             CheckColumn(column);
-            if (_error == null) _error = new DataError();
+            _error ??= new DataError();
             return _error.GetColumnError(column);
         }
 
@@ -886,7 +886,7 @@ namespace System.Data
             DataRelation[] nestedParentRelations = _table.NestedParentRelations;
             foreach (DataRelation rel in nestedParentRelations)
             {
-                if (rel == null) // don't like this but done for backward code compatability
+                if (rel == null) // don't like this but done for backward code compatibility
                 {
                     continue;
                 }
@@ -1204,7 +1204,7 @@ namespace System.Data
                     }
                     foreach (DataColumn dc in _columns.ColumnsImplementingIChangeTracking)
                     {
-                        object? value = null;
+                        object? value;
                         if (RowState != DataRowState.Deleted)
                             value = this[dc];
                         else
@@ -1426,7 +1426,6 @@ namespace System.Data
                     _columns[i].CopyValueIntoStore(_tempRecord, storeList[i]!, (BitArray)nullbitList[i]!, storeIndex);
                 }
                 recordCount++;
-                storeIndex++;
             }
 
             return recordCount;
