@@ -250,17 +250,15 @@ namespace System.Data.OleDb
                     {
                         value[0] = DBNull.Value;
 
-                        object recordsAffected;
                         object nextresult;
-                        OleDbHResult hr = ((UnsafeNativeMethods.Recordset15)adodb).NextRecordset(out recordsAffected, out nextresult);
+                        OleDbHResult hr = ((UnsafeNativeMethods.Recordset15)adodb).NextRecordset(out _, out nextresult);
 
                         if (0 > hr)
                         {
                             // Current provider does not support returning multiple recordsets from a single execution.
                             if (ODB.ADODB_NextResultError != (int)hr)
                             {
-                                UnsafeNativeMethods.IErrorInfo? errorInfo = null;
-                                UnsafeNativeMethods.GetErrorInfo(0, out errorInfo);
+                                SafeNativeMethods.Wrapper.ClearErrorInfo();
 
                                 string message = string.Empty;
                                 throw new COMException(message, (int)hr);
@@ -311,7 +309,7 @@ namespace System.Data.OleDb
             incrementResultCount = false;
 
             IntPtr chapter; /*ODB.DB_NULL_HCHAPTER*/
-            object? result = null;
+            object? result;
             try
             {
                 result = recordset.get_Rowset();
@@ -336,7 +334,7 @@ namespace System.Data.OleDb
                 OleDbDataReader? dataReader = null;
                 try
                 {
-                    // intialized with chapter only since we don't want ReleaseChapter called for this chapter handle
+                    // initialized with chapter only since we don't want ReleaseChapter called for this chapter handle
                     ChapterHandle chapterHandle = ChapterHandle.CreateChapterHandle(chapter);
 
                     dataReader = new OleDbDataReader(null, null, 0, behavior);
@@ -358,10 +356,7 @@ namespace System.Data.OleDb
                 }
                 finally
                 {
-                    if (null != dataReader)
-                    {
-                        dataReader.Close();
-                    }
+                    dataReader?.Close();
                 }
             }
             return 0;
@@ -369,7 +364,7 @@ namespace System.Data.OleDb
 
         private int FillFromRecord(object data, UnsafeNativeMethods.ADORecordConstruction record, string srcTable)
         {
-            object? result = null;
+            object? result;
             try
             {
                 result = record.get_Row();
@@ -408,16 +403,13 @@ namespace System.Data.OleDb
                 }
                 finally
                 {
-                    if (null != dataReader)
-                    {
-                        dataReader.Close();
-                    }
+                    dataReader?.Close();
                 }
             }
             return 0;
         }
 
-        private void FillClose(bool isrecordset, object value)
+        private static void FillClose(bool isrecordset, object value)
         {
             OleDbHResult hr;
             if (isrecordset)
@@ -430,8 +422,7 @@ namespace System.Data.OleDb
             }
             if ((0 < (int)hr) && (ODB.ADODB_AlreadyClosedError != (int)hr))
             {
-                UnsafeNativeMethods.IErrorInfo? errorInfo = null;
-                UnsafeNativeMethods.GetErrorInfo(0, out errorInfo);
+                SafeNativeMethods.Wrapper.ClearErrorInfo();
                 string message = string.Empty;
                 throw new COMException(message, (int)hr);
             }
