@@ -1,5 +1,6 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
+using System.Collections;
 using System.Collections.Concurrent;
 using System.Collections.Generic;
 using System.Diagnostics.CodeAnalysis;
@@ -233,10 +234,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Events from DiagnosticSource can be forwarded to EventSource using this event.
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(2, Keywords = Keywords.Events)]
         private void Event(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>>? Arguments)
         {
@@ -256,10 +256,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Used to mark the beginning of an activity
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(4, Keywords = Keywords.Events)]
         private void Activity1Start(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>> Arguments)
         {
@@ -269,10 +268,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Used to mark the end of an activity
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(5, Keywords = Keywords.Events)]
         private void Activity1Stop(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>> Arguments)
         {
@@ -282,10 +280,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Used to mark the beginning of an activity
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(6, Keywords = Keywords.Events)]
         private void Activity2Start(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>> Arguments)
         {
@@ -295,10 +292,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Used to mark the end of an activity that can be recursive.
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(7, Keywords = Keywords.Events)]
         private void Activity2Stop(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>> Arguments)
         {
@@ -308,10 +304,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Used to mark the beginning of an activity
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(8, Keywords = Keywords.Events, ActivityOptions = EventActivityOptions.Recursive)]
         private void RecursiveActivity1Start(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>> Arguments)
         {
@@ -321,10 +316,9 @@ namespace System.Diagnostics
         /// <summary>
         /// Used to mark the end of an activity that can be recursive.
         /// </summary>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(9, Keywords = Keywords.Events, ActivityOptions = EventActivityOptions.Recursive)]
         private void RecursiveActivity1Stop(string SourceName, string EventName, IEnumerable<KeyValuePair<string, string?>> Arguments)
         {
@@ -347,10 +341,9 @@ namespace System.Diagnostics
         /// <param name="SourceName">The ActivitySource name</param>
         /// <param name="ActivityName">The Activity name</param>
         /// <param name="Arguments">Name and value pairs of the Activity properties</param>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(11, Keywords = Keywords.Events, ActivityOptions = EventActivityOptions.Recursive)]
         private void ActivityStart(string SourceName, string ActivityName, IEnumerable<KeyValuePair<string, string?>> Arguments) =>
             WriteEvent(11, SourceName, ActivityName, Arguments);
@@ -361,10 +354,9 @@ namespace System.Diagnostics
         /// <param name="SourceName">The ActivitySource name</param>
         /// <param name="ActivityName">The Activity name</param>
         /// <param name="Arguments">Name and value pairs of the Activity properties</param>
-#if !ES_BUILD_STANDALONE
         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-            Justification = "Arguments parameter is trimmer safe")]
-#endif
+            Justification = "Arguments parameter is preserved by DynamicDependency")]
+        [DynamicDependency(DynamicallyAccessedMemberTypes.PublicProperties, typeof(KeyValuePair<,>))]
         [Event(12, Keywords = Keywords.Events, ActivityOptions = EventActivityOptions.Recursive)]
         private void ActivityStop(string SourceName, string ActivityName, IEnumerable<KeyValuePair<string, string?>> Arguments) =>
             WriteEvent(12, SourceName, ActivityName, Arguments);
@@ -483,8 +475,7 @@ namespace System.Diagnostics
             public static void CreateFilterAndTransformList(ref FilterAndTransform? specList, string? filterAndPayloadSpecs, DiagnosticSourceEventSource eventSource)
             {
                 DestroyFilterAndTransformList(ref specList, eventSource);        // Stop anything that was on before.
-                if (filterAndPayloadSpecs == null)
-                    filterAndPayloadSpecs = "";
+                filterAndPayloadSpecs ??= "";
 
                 // Points just beyond the last point in the string that has yet to be parsed. Thus we start with the whole string.
                 int endIdx = filterAndPayloadSpecs.Length;
@@ -642,10 +633,7 @@ namespace System.Diagnostics
                         _eventSource.Message("DiagnosticSource: Could not find Event to log Activity " + activityName);
                 }
 
-                if (writeEvent == null)
-                {
-                    writeEvent = _eventSource.Event;
-                }
+                writeEvent ??= _eventSource.Event;
 
                 // Set up a subscription that watches for the given Diagnostic Sources and events which will call back
                 // to the EventSource.
@@ -660,6 +648,9 @@ namespace System.Diagnostics
 
                         [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
                             Justification = "DiagnosticSource.Write is marked with RequiresUnreferencedCode.")]
+                        [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2119",
+                            Justification = "DAM on EventSource references this compiler-generated local function which calls a " +
+                                            "method that requires unreferenced code. EventSource will not access this local function.")]
                         void OnEventWritten(KeyValuePair<string, object?> evnt)
                         {
                             // The filter given to the DiagnosticSource may not work if users don't is 'IsEnabled' as expected.
@@ -755,13 +746,13 @@ namespace System.Diagnostics
                 {
                     activitySourceName = entry.Slice(0, eventNameIndex).Trim();
 
-                    ReadOnlySpan<char> suffixPart = entry.Slice(eventNameIndex + 1, entry.Length - eventNameIndex - 1).Trim();
+                    ReadOnlySpan<char> suffixPart = entry.Slice(eventNameIndex + 1).Trim();
                     int samplingResultIndex = suffixPart.IndexOf('-');
                     if (samplingResultIndex >= 0)
                     {
                         // We have the format "[AS]SourceName/[EventName]-[SamplingResult]
                         eventName = suffixPart.Slice(0, samplingResultIndex).Trim();
-                        suffixPart = suffixPart.Slice(samplingResultIndex + 1, suffixPart.Length - samplingResultIndex - 1).Trim();
+                        suffixPart = suffixPart.Slice(samplingResultIndex + 1).Trim();
 
                         if (suffixPart.Length > 0)
                         {
@@ -817,7 +808,7 @@ namespace System.Diagnostics
                     activitySourceName = activitySourceName.Slice(0, plusSignIndex).Trim();
                 }
 
-                var transform = new FilterAndTransform(filterAndPayloadSpec, endIdx, colonIdx, activitySourceName.ToString(), activityName, supportedEvent, samplingResult, eventSource);
+                new FilterAndTransform(filterAndPayloadSpec, endIdx, colonIdx, activitySourceName.ToString(), activityName, supportedEvent, samplingResult, eventSource);
             }
 
             // Check if we are interested to listen to such ActivitySource
@@ -1066,11 +1057,7 @@ namespace System.Diagnostics
                                 Interlocked.CompareExchange(ref _implicitTransformsTable,
                                     new ConcurrentDictionary<Type, TransformSpec?>(1, 8), null);
                             }
-                            implicitTransforms = _implicitTransformsTable.GetOrAdd(argType, type => MakeImplicitTransformsWrapper(type));
-
-                            [UnconditionalSuppressMessage("ReflectionAnalysis", "IL2026:RequiresUnreferencedCode",
-                                Justification = "The Morph method has RequiresUnreferencedCode, but the trimmer can't see through lamdba calls.")]
-                            static TransformSpec? MakeImplicitTransformsWrapper(Type transformType) => MakeImplicitTransforms(transformType);
+                            implicitTransforms = _implicitTransformsTable.GetOrAdd(argType, MakeImplicitTransforms);
                         }
 
                         // implicitTransformas now fetched from cache or constructed, use it to Fetch all the implicit fields.
@@ -1190,12 +1177,11 @@ namespace System.Diagnostics
                     if (0 <= dotIdx)
                         idIdx = dotIdx + 1;
 
-                    string propertName = transformSpec.Substring(idIdx, endIdx - idIdx);
-                    _fetches = new PropertySpec(propertName, _fetches);
+                    string propertyName = transformSpec.Substring(idIdx, endIdx - idIdx);
+                    _fetches = new PropertySpec(propertyName, _fetches);
 
                     // If the user did not explicitly set a name, it is the last one (first to be processed from the end).
-                    if (_outputName == null)
-                        _outputName = propertName;
+                    _outputName ??= propertyName;
 
                     endIdx = dotIdx;    // This works even when LastIndexOf return -1.
                 }
@@ -1333,10 +1319,7 @@ namespace System.Diagnostics
                                     continue;
                                 }
 
-                                Type elemType = iFaceTypeInfo.GetGenericArguments()[0];
-                                Type instantiatedTypedPropertyFetcher = typeof(EnumeratePropertyFetch<>)
-                                    .GetTypeInfo().MakeGenericType(elemType);
-                                return (PropertyFetch)Activator.CreateInstance(instantiatedTypedPropertyFetcher, type)!;
+                                return CreateEnumeratePropertyFetch(type, iFaceTypeInfo);
                             }
 
                             // no implementation of IEnumerable<T> found, return a null fetcher
@@ -1369,20 +1352,50 @@ namespace System.Diagnostics
                                 Log.Message($"Property {propertyName} is static.");
                                 return new PropertyFetch(type);
                             }
-                            Type typedPropertyFetcher = typeInfo.IsValueType ?
-                                typeof(ValueTypedFetchProperty<,>) : typeof(RefTypedFetchProperty<,>);
-                            Type instantiatedTypedPropertyFetcher = typedPropertyFetcher.GetTypeInfo().MakeGenericType(
-                                propertyInfo.DeclaringType!, propertyInfo.PropertyType);
-                            return (PropertyFetch)Activator.CreateInstance(instantiatedTypedPropertyFetcher, type, propertyInfo)!;
+
+                            return CreatePropertyFetch(typeInfo, propertyInfo);
                         }
                     }
 
+                    [UnconditionalSuppressMessage("AOT", "IL3050:RequiresDynamicCode",
+                        Justification = "MakeGenericType is only called when IsDynamicCodeSupported is true or only with ref types.")]
+                    private static PropertyFetch CreateEnumeratePropertyFetch(Type type, TypeInfo enumerableOfTType)
+                    {
+                        Type elemType = enumerableOfTType.GetGenericArguments()[0];
+#if NETCOREAPP
+                        if (!RuntimeFeature.IsDynamicCodeSupported && elemType.IsValueType)
+                        {
+                            return new EnumeratePropertyFetch(type);
+                        }
+#endif
+                        Type instantiatedTypedPropertyFetcher = typeof(EnumeratePropertyFetch<>)
+                            .GetTypeInfo().MakeGenericType(elemType);
+                        return (PropertyFetch)Activator.CreateInstance(instantiatedTypedPropertyFetcher, type)!;
+                    }
+
+                    [UnconditionalSuppressMessage("AOT", "IL3050:RequiresDynamicCode",
+                        Justification = "MakeGenericType is only called when IsDynamicCodeSupported is true or only with ref types.")]
+                    private static PropertyFetch CreatePropertyFetch(Type type, PropertyInfo propertyInfo)
+                    {
+#if NETCOREAPP
+                        if (!RuntimeFeature.IsDynamicCodeSupported && (propertyInfo.DeclaringType!.IsValueType || propertyInfo.PropertyType.IsValueType))
+                        {
+                            return new ReflectionPropertyFetch(type, propertyInfo);
+                        }
+#endif
+                        Type typedPropertyFetcher = type.IsValueType ?
+                            typeof(ValueTypedFetchProperty<,>) : typeof(RefTypedFetchProperty<,>);
+                        Type instantiatedTypedPropertyFetcher = typedPropertyFetcher.GetTypeInfo().MakeGenericType(
+                            propertyInfo.DeclaringType!, propertyInfo.PropertyType);
+                        return (PropertyFetch)Activator.CreateInstance(instantiatedTypedPropertyFetcher, type, propertyInfo)!;
+                    }
+
                     /// <summary>
                     /// Given an object, fetch the property that this propertyFech represents.
                     /// </summary>
                     public virtual object? Fetch(object? obj) { return null; }
 
-                    #region private
+#region private
 
                     private sealed class RefTypedFetchProperty<TObject, TProperty> : PropertyFetch
                     {
@@ -1421,6 +1434,74 @@ namespace System.Diagnostics
                         private readonly StructFunc<TStruct, TProperty> _propertyFetch;
                     }
 
+#if NETCOREAPP
+                    /// <summary>
+                    /// A fetcher that can be used when MakeGenericType isn't available.
+                    /// </summary>
+                    private sealed class ReflectionPropertyFetch : PropertyFetch
+                    {
+                        private readonly PropertyInfo _property;
+                        public ReflectionPropertyFetch(Type type, PropertyInfo property) : base(type)
+                        {
+                            _property = property;
+                        }
+
+                        public override object? Fetch(object? obj) => _property.GetValue(obj);
+                    }
+
+                    /// <summary>
+                    /// A fetcher that enumerates and formats an IEnumerable when MakeGenericType isn't available.
+                    /// </summary>
+                    private sealed class EnumeratePropertyFetch : PropertyFetch
+                    {
+                        public EnumeratePropertyFetch(Type type) : base(type) { }
+
+                        public override object? Fetch(object? obj)
+                        {
+                            IEnumerable? enumerable = obj as IEnumerable;
+                            Debug.Assert(enumerable is not null);
+
+                            // string.Join for a non-generic IEnumerable
+                            IEnumerator en = enumerable.GetEnumerator();
+                            using (IDisposable? disposable = en as IDisposable)
+                            {
+                                if (!en.MoveNext())
+                                {
+                                    return string.Empty;
+                                }
+
+                                object? currentValue = en.Current;
+                                string? firstString = currentValue?.ToString();
+
+                                // If there's only 1 item, simply return the ToString of that
+                                if (!en.MoveNext())
+                                {
+                                    // Only one value available
+                                    return firstString ?? string.Empty;
+                                }
+
+                                var result = new ValueStringBuilder(stackalloc char[256]);
+
+                                result.Append(firstString);
+
+                                do
+                                {
+                                    currentValue = en.Current;
+
+                                    result.Append(",");
+                                    if (currentValue != null)
+                                    {
+                                        result.Append(currentValue.ToString());
+                                    }
+                                }
+                                while (en.MoveNext());
+
+                                return result.ToString();
+                            }
+                        }
+                    }
+#endif
+
                     /// <summary>
                     /// A fetcher that returns the result of Activity.Current
                     /// </summary>
@@ -1445,17 +1526,17 @@ namespace System.Diagnostics
                             return string.Join(",", (IEnumerable<ElementType>)obj);
                         }
                     }
-                    #endregion
+#endregion
                 }
 
                 private readonly string _propertyName;
                 private volatile PropertyFetch? _fetchForExpectedType;
-                #endregion
+#endregion
             }
 
             private readonly string _outputName = null!;
             private readonly PropertySpec? _fetches;
-            #endregion
+#endregion
         }
 
         /// <summary>
@@ -1468,13 +1549,13 @@ namespace System.Diagnostics
         {
             public CallbackObserver(Action<T> callback) { _callback = callback; }
 
-            #region private
+#region private
             public void OnCompleted() { }
             public void OnError(Exception error) { }
             public void OnNext(T value) { _callback(value); }
 
             private readonly Action<T> _callback;
-            #endregion
+#endregion
         }
 
         // A linked list of IObservable subscriptions (which are IDisposable).
@@ -1491,11 +1572,11 @@ namespace System.Diagnostics
             public Subscriptions? Next;
         }
 
-        #endregion
+#endregion
 
         private FilterAndTransform? _specs;                 // Transformation specifications that indicate which sources/events are forwarded.
         private FilterAndTransform? _activitySourceSpecs;   // ActivitySource Transformation specifications that indicate which sources/events are forwarded.
         private ActivityListener? _activityListener;
-        #endregion
+#endregion
     }
 }
