@@ -24,9 +24,6 @@ namespace System.Diagnostics
     /// Using more than eight tags will cause allocating memory to store the tags.
     /// Public static (Shared in Visual Basic) members of this type are thread safe. Any instance members are not guaranteed to be thread safe.
     /// </remarks>
-#if ALLOW_PARTIALLY_TRUSTED_CALLERS
-    [System.Security.SecuritySafeCriticalAttribute]
-#endif
     [StructLayout(LayoutKind.Sequential)]
     public struct TagList : IList<KeyValuePair<string, object?>>, IReadOnlyList<KeyValuePair<string, object?>>
     {
@@ -234,7 +231,7 @@ namespace System.Diagnostics
 
             if (_overflowTags is not null)
             {
-                _overflowTags.AsSpan().Slice(0, _tagsCount).CopyTo(tags);
+                _overflowTags.AsSpan(0, _tagsCount).CopyTo(tags);
                 return;
             }
 
@@ -273,7 +270,7 @@ namespace System.Diagnostics
                 throw new ArgumentOutOfRangeException(nameof(arrayIndex));
             }
 
-            CopyTo(array.AsSpan().Slice(arrayIndex));
+            CopyTo(array.AsSpan(arrayIndex));
         }
 
         /// <summary>
@@ -553,4 +550,4 @@ namespace System.Diagnostics
             public void Reset() => _index = -1;
         }
     }
-}
\ No newline at end of file
+}
