@@ -17,9 +17,6 @@ namespace System.Diagnostics
     public partial class Process : IDisposable
     {
         private static volatile bool s_initialized;
-        private static uint s_euid;
-        private static uint s_egid;
-        private static uint[]? s_groups;
         private static readonly object s_initializedGate = new object();
         private static readonly ReaderWriterLockSlim s_processStartLock = new ReaderWriterLockSlim();
 
@@ -68,9 +65,9 @@ namespace System.Diagnostics
 
             EnsureState(State.HaveId);
 
-            // Check if we know the process has exited. This avoids us targetting another
+            // Check if we know the process has exited. This avoids us targeting another
             // process that has a recycled PID. This only checks our internal state, the Kill call below
-            // activly checks if the process is still alive.
+            // actively checks if the process is still alive.
             if (GetHasExited(refresh: false))
             {
                 return;
@@ -124,7 +121,7 @@ namespace System.Diagnostics
                 return;
             }
 
-            IReadOnlyList<Process> children = GetChildProcesses();
+            List<Process> children = GetChildProcesses();
 
             int killResult = Interop.Sys.Kill(_processId, Interop.Sys.Signals.SIGKILL);
             if (killResult != 0)
@@ -145,10 +142,7 @@ namespace System.Diagnostics
         }
 
         /// <summary>Discards any information about the associated process.</summary>
-        private void RefreshCore()
-        {
-            // Nop.  No additional state to reset.
-        }
+        partial void RefreshCore();
 
         /// <summary>Additional logic invoked when the Process is closed.</summary>
         private void CloseCore()
@@ -252,7 +246,7 @@ namespace System.Diagnostics
         /// should be temporarily boosted by the operating system when the main window
         /// has focus.
         /// </summary>
-        private bool PriorityBoostEnabledCore
+        private static bool PriorityBoostEnabledCore
         {
             get { return false; } //Nop
             set { } // Nop
@@ -269,8 +263,7 @@ namespace System.Diagnostics
             {
                 EnsureState(State.HaveNonExitedId);
 
-                int pri = 0;
-                int errno = Interop.Sys.GetPriority(Interop.Sys.PriorityWhich.PRIO_PROCESS, _processId, out pri);
+                int errno = Interop.Sys.GetPriority(Interop.Sys.PriorityWhich.PRIO_PROCESS, _processId, out int pri);
                 if (errno != 0) // Interop.Sys.GetPriority returns GetLastWin32Error()
                 {
                     throw new Win32Exception(errno); // match Windows exception
@@ -770,29 +763,50 @@ namespace System.Diagnostics
                 return false;
             }
 
-            Interop.Sys.Permissions permissions = (Interop.Sys.Permissions)fileinfo.Mode;
+            const UnixFileMode AllExecute = UnixFileMode.UserExecute | UnixFileMode.GroupExecute | UnixFileMode.OtherExecute;
 
-            if (s_euid == 0)
+            UnixFileMode permissions = ((UnixFileMode)fileinfo.Mode) & AllExecute;
+
+            // Avoid checking user/group when permission.
+            if (permissions == AllExecute)
             {
-                // We're root.
-                return (permissions & Interop.Sys.Permissions.S_IXUGO) != 0;
+                return true;
+            }
+            else if (permissions == 0)
+            {
+                return false;
             }
 
-            if (s_euid == fileinfo.Uid)
+            uint euid = Interop.Sys.GetEUid();
+
+            if (euid == 0)
+            {
+                return true; // We're root.
+            }
+
+            if (euid == fileinfo.Uid)
             {
                 // We own the file.
-                return (permissions & Interop.Sys.Permissions.S_IXUSR) != 0;
+                return (permissions & UnixFileMode.UserExecute) != 0;
             }
 
-            if (s_egid == fileinfo.Gid ||
-                (s_groups != null && Array.BinarySearch(s_groups, fileinfo.Gid) >= 0))
+            bool groupCanExecute = (permissions & UnixFileMode.GroupExecute) != 0;
+            bool otherCanExecute = (permissions & UnixFileMode.OtherExecute) != 0;
+
+            // Avoid group check when group and other have same permissions.
+            if (groupCanExecute == otherCanExecute)
             {
-                // A group we're a member of owns the file.
-                return (permissions & Interop.Sys.Permissions.S_IXGRP) != 0;
+                return groupCanExecute;
             }
 
-            // Other.
-            return (permissions & Interop.Sys.Permissions.S_IXOTH) != 0;
+            if (Interop.Sys.IsMemberOfGroup(fileinfo.Gid))
+            {
+                return groupCanExecute;
+            }
+            else
+            {
+                return otherCanExecute;
+            }
         }
 
         private static long s_ticksPerSecond;
@@ -823,7 +837,7 @@ namespace System.Diagnostics
         /// <param name="fd">The file descriptor.</param>
         /// <param name="direction">The pipe direction.</param>
         /// <returns>The opened stream.</returns>
-        private static Stream OpenStream(int fd, PipeDirection direction)
+        private static AnonymousPipeClientStream OpenStream(int fd, PipeDirection direction)
         {
             Debug.Assert(fd >= 0);
             return new AnonymousPipeClientStream(direction, new SafePipeHandle((IntPtr)fd, ownsHandle: true));
@@ -1040,13 +1054,13 @@ namespace System.Diagnostics
 
         public IntPtr MainWindowHandle => IntPtr.Zero;
 
-        private bool CloseMainWindowCore() => false;
+        private static bool CloseMainWindowCore() => false;
 
         public string MainWindowTitle => string.Empty;
 
         public bool Responding => true;
 
-        private bool WaitForInputIdleCore(int milliseconds) => throw new InvalidOperationException(SR.InputIdleUnkownError);
+        private static bool WaitForInputIdleCore(int _ /*milliseconds*/) => throw new InvalidOperationException(SR.InputIdleUnknownError);
 
         private static unsafe void EnsureInitialized()
         {
@@ -1064,14 +1078,6 @@ namespace System.Diagnostics
                         throw new Win32Exception();
                     }
 
-                    s_euid = Interop.Sys.GetEUid();
-                    s_egid = Interop.Sys.GetEGid();
-                    s_groups = Interop.Sys.GetGroups();
-                    if (s_groups != null)
-                    {
-                        Array.Sort(s_groups);
-                    }
-
                     // Register our callback.
                     Interop.Sys.RegisterForSigChld(&OnSigChild);
                     SetDelayedSigChildConsoleConfigurationHandler();
@@ -1107,6 +1113,16 @@ namespace System.Diagnostics
             }
         }
 
+        /// <summary>Gets the friendly name of the process.</summary>
+        public string ProcessName
+        {
+            get
+            {
+                EnsureState(State.HaveProcessInfo);
+                return _processInfo!.ProcessName;
+            }
+        }
+
         private static bool PlatformDoesNotSupportProcessStartAndKill
             => (OperatingSystem.IsIOS() && !OperatingSystem.IsMacCatalyst()) || OperatingSystem.IsTvOS();
     }
