@@ -44,7 +44,7 @@ namespace System.DirectoryServices.AccountManagement
                                     "ADDNLinkedAttrSet",
                                     "ADDNLinkedAttrSet: groupDN={0}, primaryGroupDN={1}, recursive={2}, PG queryFilter={3}, PG queryBase={4}",
                                     groupDN,
-                                    (primaryGroupDN != null ? primaryGroupDN : "NULL"),
+                                    primaryGroupDN ?? "NULL",
                                     recursive,
                                     (primaryGroupMembersSearcher != null ? primaryGroupMembersSearcher.Filter : "NULL"),
                                     (primaryGroupMembersSearcher != null ? primaryGroupMembersSearcher.SearchRoot.Path : "NULL"));
@@ -89,7 +89,7 @@ namespace System.DirectoryServices.AccountManagement
                                     "ADDNLinkedAttrSet",
                                     "ADDNLinkedAttrSet: groupDN={0}, primaryGroupDN={1}, recursive={2}, M queryFilter={3}, M queryBase={4}, PG queryFilter={5}, PG queryBase={6}",
                                     groupDN,
-                                    (primaryGroupDN != null ? primaryGroupDN : "NULL"),
+                                    primaryGroupDN ?? "NULL",
                                     recursive,
                                     (membersSearcher != null ? membersSearcher[0].Filter : "NULL"),
                                     (membersSearcher != null ? membersSearcher[0].SearchRoot.Path : "NULL"),
@@ -257,7 +257,7 @@ namespace System.DirectoryServices.AccountManagement
                 memberFound = _memberSearchResultsEnumerator.MoveNext();
 
                 // The search is complete.
-                // Dipose the searcher and search results.
+                // Dispose the searcher and search results.
                 if (!memberFound)
                 {
                     _currentMembersSearcher.Dispose();
@@ -354,7 +354,7 @@ namespace System.DirectoryServices.AccountManagement
                     // We reached the end of this group's membership.  If we're not processing recursively,
                     // we're done.  Otherwise, go on to the next group to visit.
                     // First create a DE that points to the group we want to expand,  Using that as a search root run
-                    // an ASQ search against  member and start enumerting those results.
+                    // an ASQ search against  member and start enumerating those results.
                     if (_recursive)
                     {
                         GlobalDebug.WriteLineIf(GlobalDebug.Info,
@@ -417,16 +417,8 @@ namespace System.DirectoryServices.AccountManagement
 
                 if (!memberFound)
                 {
-                    IDisposable disposableMembers = _members as IDisposable;
-                    if (disposableMembers != null)
-                    {
-                        disposableMembers.Dispose();
-                    }
-                    IDisposable disposableMembersEnum = _membersEnum as IDisposable;
-                    if (disposableMembersEnum != null)
-                    {
-                        disposableMembersEnum.Dispose();
-                    }
+                    (_members as IDisposable)?.Dispose();
+                    (_membersEnum as IDisposable)?.Dispose();
                     _members = null;
                     _membersEnum = null;
                 }
@@ -590,7 +582,7 @@ namespace System.DirectoryServices.AccountManagement
 
             List<byte[]> sidList = new List<byte[]>(_foreignMembersCurrentGroup.Count);
 
-            // Foreach foreign principal retrive the sid.
+            // Foreach foreign principal retrieve the sid.
             // If the SID is for a fake object we have to track it separately.  If we were attempt to translate it
             // it would fail and not be returned and we would lose it.
             // Once we have a list of sids then translate them against the target store in one call.
@@ -675,11 +667,10 @@ namespace System.DirectoryServices.AccountManagement
                     if (null == foreignSid.sidIssuerName)
                     {
                         // create and return the unknown principal if it is not yet present in usersVisited
-                        if (!_usersVisited.ContainsKey(foreignSid.name))
+                        if (_usersVisited.TryAdd(foreignSid.name, true))
                         {
                             byte[] sid = Utils.ConvertNativeSidToByteArray(foreignSid.pSid);
                             UnknownPrincipal unknownPrincipal = UnknownPrincipal.CreateUnknownPrincipal(_storeCtx.OwningContext, sid, foreignSid.name);
-                            _usersVisited.Add(foreignSid.name, true);
                             this.current = null;
                             _currentForeignDE = null;
                             _currentForeignPrincipal = unknownPrincipal;
@@ -708,18 +699,7 @@ namespace System.DirectoryServices.AccountManagement
                     {
                         ContextOptions remoteOptions = DefaultContextOptions.ADDefaultContextOption;
 
-#if USE_CTX_CACHE
                         PrincipalContext remoteCtx = SDSCache.Domain.GetContext(foreignSid.sidIssuerName, _storeCtx.Credentials, remoteOptions);
-#else
-                        PrincipalContext remoteCtx = new PrincipalContext(
-                                        ContextType.Domain,
-                                        foreignSid.sidIssuerName,
-                                        null,
-                                        (this.storeCtx.Credentials != null ? this.storeCtx.Credentials.UserName : null),
-                                        (this.storeCtx.Credentials != null ? storeCtx.storeCtx.Credentials.Password : null),
-                                        remoteOptions);
-
-#endif
                         foreignStoreCtx = remoteCtx.QueryCtx;
                     }
 
@@ -827,7 +807,7 @@ namespace System.DirectoryServices.AccountManagement
 
         private bool ExpandForeignGroupEnumerator()
         {
-            Debug.Assert(_recursive == true);
+            Debug.Assert(_recursive);
             GlobalDebug.WriteLineIf(GlobalDebug.Info,
                                     "ADDNLinkedAttrSet",
                                     "ExpandForeignGroupEnumerator: there are {0} foreignGroups",
@@ -857,7 +837,7 @@ namespace System.DirectoryServices.AccountManagement
 
         private bool ExpandForeignGroupSearcher()
         {
-            Debug.Assert(_recursive == true);
+            Debug.Assert(_recursive);
             GlobalDebug.WriteLineIf(GlobalDebug.Info,
                                     "ADDNLinkedAttrSet",
                                     "ExpandForeignGroupSearcher: there are {0} foreignGroups",
@@ -972,8 +952,7 @@ namespace System.DirectoryServices.AccountManagement
                 _foreignMembersCurrentGroup.Clear();
                 _fakePrincipalMembers.Clear();
 
-                if (null != _foreignMembersToReturn)
-                    _foreignMembersToReturn.Clear();
+                _foreignMembersToReturn?.Clear();
 
                 _currentForeignPrincipal = null;
                 _currentForeignDE = null;
@@ -1160,8 +1139,7 @@ namespace System.DirectoryServices.AccountManagement
             _currentForeignPrincipal = adBookmark.currentForeignPrincipal;
             _currentForeignDE = adBookmark.currentForeignDE;
             _foreignGroups = adBookmark.foreignGroups;
-            if (_queryMembersResults != null)
-                _queryMembersResults.Dispose();
+            _queryMembersResults?.Dispose();
             _queryMembersResults = adBookmark.queryMembersResults;
             _queryMembersResultEnumerator = adBookmark.queryMembersResultEnumerator;
             _memberSearchResults = adBookmark.memberSearchResults;
@@ -1266,11 +1244,7 @@ namespace System.DirectoryServices.AccountManagement
                         GlobalDebug.WriteLineIf(GlobalDebug.Info, "ADDNLinkedAttrSet", "Dispose: disposing membersQueue");
                         foreach (IEnumerable enumerable in _membersQueue)
                         {
-                            IDisposable disposableEnum = enumerable as IDisposable;
-                            if (disposableEnum != null)
-                            {
-                                disposableEnum.Dispose();
-                            }
+                            (enumerable as IDisposable)?.Dispose();
                         }
                     }
                     if (_foreignGroups != null)
