@@ -44,7 +44,7 @@ namespace System.DirectoryServices.AccountManagement
         public const string LDAP_CAP_ACTIVE_DIRECTORY_V61_OID = "1.2.840.113556.1.4.1935";
     }
 
-    internal sealed class CredentialValidator
+    internal sealed class CredentialValidator : IDisposable
     {
         private enum AuthMethod
         {
@@ -85,10 +85,7 @@ namespace System.DirectoryServices.AccountManagement
 
         private bool BindSam(string target, string userName, string password)
         {
-            StringBuilder adsPath = new StringBuilder();
-            adsPath.Append("WinNT://");
-            adsPath.Append(_serverName);
-            adsPath.Append(",computer");
+            string adsPath = $"WinNT://{_serverName},computer";
             Guid g = new Guid("fd8256d0-fd15-11ce-abc4-02608c9e7553"); // IID_IUnknown
             object value = null;
             // always attempt secure auth..
@@ -110,7 +107,7 @@ namespace System.DirectoryServices.AccountManagement
                     }
                 }
 
-                int hr = UnsafeNativeMethods.ADsOpenObject(adsPath.ToString(), userName, password, (int)authenticationType, ref g, out value);
+                int hr = UnsafeNativeMethods.ADsOpenObject(adsPath, userName, password, (int)authenticationType, ref g, out value);
 
                 if (hr != 0)
                 {
@@ -341,6 +338,14 @@ namespace System.DirectoryServices.AccountManagement
                 return (BindSam(_serverName, userName, password));
             }
         }
+
+        public void Dispose()
+        {
+            foreach (LdapConnection connection in _connCache.Values)
+            {
+                connection.Dispose();
+            }
+        }
     }
     // ********************************************
     public class PrincipalContext : IDisposable
@@ -620,10 +625,7 @@ namespace System.DirectoryServices.AccountManagement
 
             try
             {
-                string hostname = _name;
-
-                if (hostname == null)
-                    hostname = Utils.GetComputerFlatName();
+                string hostname = _name ?? Utils.GetComputerFlatName();
 
                 GlobalDebug.WriteLineIf(GlobalDebug.Info, "PrincipalContext", "DoMachineInit: hostname is " + hostname);
 
@@ -656,8 +658,7 @@ namespace System.DirectoryServices.AccountManagement
                                                    " and message " + e.Message);
 
                 // Cleanup the DE on failure
-                if (de != null)
-                    de.Dispose();
+                de?.Dispose();
 
                 throw;
             }
@@ -763,8 +764,7 @@ namespace System.DirectoryServices.AccountManagement
             finally
             {
                 // Cleanup the DE on failure
-                if (de != null)
-                    de.Dispose();
+                de?.Dispose();
             }
         }
 
@@ -820,8 +820,7 @@ namespace System.DirectoryServices.AccountManagement
             finally
             {
                 // Don't allow the DE to leak
-                if (deRootDse != null)
-                    deRootDse.Dispose();
+                deRootDse?.Dispose();
             }
 
             try
@@ -885,7 +884,7 @@ namespace System.DirectoryServices.AccountManagement
                 // The Users container will also be used as the default for Groups.
                 // The reason there are different contexts for groups, users and computers is so that
                 // when a principal is created it will go into the appropriate default container.  This is so users don't
-                // be default create principals in the root of their directory.  When a search happens the base context is used so that
+                // by default create principals in the root of their directory.  When a search happens the base context is used so that
                 // the whole directory will be covered.
                 //
                 deUserGroupOrg = new DirectoryEntry(adsPathUserGroupOrg, _username, _password, authTypes);
@@ -917,23 +916,12 @@ namespace System.DirectoryServices.AccountManagement
 
                 // Cleanup on failure.  Once a DE has been successfully handed off to a ADStoreCtx,
                 // that ADStoreCtx will handle Dispose()'ing it
-                if (deUserGroupOrg != null)
-                    deUserGroupOrg.Dispose();
-
-                if (deComputer != null)
-                    deComputer.Dispose();
-
-                if (deBase != null)
-                    deBase.Dispose();
-
-                if (storeCtxUserGroupOrg != null)
-                    storeCtxUserGroupOrg.Dispose();
-
-                if (storeCtxComputer != null)
-                    storeCtxComputer.Dispose();
-
-                if (storeCtxBase != null)
-                    storeCtxBase.Dispose();
+                deUserGroupOrg?.Dispose();
+                deComputer?.Dispose();
+                deBase?.Dispose();
+                storeCtxUserGroupOrg?.Dispose();
+                storeCtxComputer?.Dispose();
+                storeCtxBase?.Dispose();
 
                 throw;
             }
@@ -1002,17 +990,12 @@ namespace System.DirectoryServices.AccountManagement
                 // This is okay, since StoreCtxs allow multiple Dispose() calls, and ignore
                 // all but the first call.
 
-                if (_userCtx != null)
-                    _userCtx.Dispose();
+                _userCtx?.Dispose();
+                _groupCtx?.Dispose();
+                _computerCtx?.Dispose();
+                _queryCtx?.Dispose();
 
-                if (_groupCtx != null)
-                    _groupCtx.Dispose();
-
-                if (_computerCtx != null)
-                    _computerCtx.Dispose();
-
-                if (_queryCtx != null)
-                    _queryCtx.Dispose();
+                _credValidate.Dispose();
 
                 _disposed = true;
                 GC.SuppressFinalize(this);
@@ -1107,7 +1090,7 @@ namespace System.DirectoryServices.AccountManagement
 
                 ldapConnection.AutoBind = false;
                 // If SSL was enabled on the initial connection then turn it on for the search.
-                // This is requried bc the appended port number will be SSL and we don't know what port LDAP is running on.
+                // This is required bc the appended port number will be SSL and we don't know what port LDAP is running on.
                 ldapConnection.SessionOptions.SecureSocketLayer = useSSL;
 
                 string baseDN = null; // specify base as null for RootDSE search
@@ -1147,7 +1130,7 @@ namespace System.DirectoryServices.AccountManagement
                     }
                 }
 
-                // If we can't determine the OS vesion so we must fall back to lowest level of functionality
+                // If we can't determine the OS version so we must fall back to lowest level of functionality
                 if (searchResponse.Entries[0].Attributes.Contains("domainControllerFunctionality"))
                 {
                     properties.OsVersion = (DomainControllerMode)Convert.ToInt32(searchResponse.Entries[0].Attributes["domainControllerFunctionality"][0], CultureInfo.InvariantCulture);
@@ -1177,10 +1160,7 @@ namespace System.DirectoryServices.AccountManagement
             }
             finally
             {
-                if (ldapConnection != null)
-                {
-                    ldapConnection.Dispose();
-                }
+                ldapConnection?.Dispose();
             }
         }
 
