@@ -11,7 +11,7 @@ using System.Collections;
 
 namespace System.DirectoryServices.AccountManagement
 {
-    [System.Diagnostics.DebuggerDisplay("Name ( {Name} )")]
+    [System.Diagnostics.DebuggerDisplay("Name = {Name}")]
     public abstract class Principal : IDisposable
     {
         //
@@ -32,7 +32,7 @@ namespace System.DirectoryServices.AccountManagement
                 CheckDisposedOrDeleted();
 
                 // The only way we can't have a PrincipalContext is if we're unpersisted
-                Debug.Assert(_ctx != null || this.unpersisted == true);
+                Debug.Assert(_ctx != null || this.unpersisted);
 
                 return _ctx;
             }
@@ -47,7 +47,7 @@ namespace System.DirectoryServices.AccountManagement
                 CheckDisposedOrDeleted();
 
                 // The only way we can't have a PrincipalContext is if we're unpersisted
-                Debug.Assert(_ctx != null || this.unpersisted == true);
+                Debug.Assert(_ctx != null || this.unpersisted);
 
                 if (_ctx == null)
                     throw new InvalidOperationException(SR.PrincipalMustSetContextForProperty);
@@ -303,7 +303,7 @@ namespace System.DirectoryServices.AccountManagement
             // and they never set a PrincipalContext.
             if (_ctx == null)
             {
-                Debug.Assert(this.unpersisted == true);
+                Debug.Assert(this.unpersisted);
                 throw new InvalidOperationException(SR.PrincipalMustSetContextForSave);
             }
 
@@ -340,7 +340,7 @@ namespace System.DirectoryServices.AccountManagement
             // and they never set a PrincipalContext.
             if (context == null)
             {
-                Debug.Assert(this.unpersisted == true);
+                Debug.Assert(this.unpersisted);
                 throw new InvalidOperationException(SR.NullArguments);
             }
 
@@ -360,7 +360,7 @@ namespace System.DirectoryServices.AccountManagement
             // context is of the same type.
             if (context.ContextType != _ctx.ContextType)
             {
-                Debug.Assert(this.unpersisted == true);
+                Debug.Assert(this.unpersisted);
                 throw new InvalidOperationException(SR.SaveToMustHaveSamecontextType);
             }
 
@@ -766,8 +766,7 @@ namespace System.DirectoryServices.AccountManagement
             set
             {
                 // Verify that the passed context is not disposed.
-                if (value != null)
-                    value.CheckDisposed();
+                value?.CheckDisposed();
                 _ctx = value;
             }
         }
@@ -802,7 +801,7 @@ namespace System.DirectoryServices.AccountManagement
         {
             if (_ctx == null)
             {
-                Debug.Assert(this.unpersisted == true);
+                Debug.Assert(this.unpersisted);
                 return null;
             }
 
@@ -893,7 +892,7 @@ namespace System.DirectoryServices.AccountManagement
             if (_isDeleted)
             {
                 GlobalDebug.WriteLineIf(GlobalDebug.Warn, "Principal", "CheckDisposedOrDeleted: accessing deleted object");
-                throw new InvalidOperationException(SR.PrincipalDeleted);
+                throw new InvalidOperationException(SR.PrincipalAccessedAfterBeingDeleted);
             }
         }
 
@@ -1028,7 +1027,7 @@ namespace System.DirectoryServices.AccountManagement
             // Make sure we're not disposed or deleted.
             CheckDisposedOrDeleted();
 
-            // Check that we actually support this propery in our store
+            // Check that we actually support this property in our store
             //CheckSupportedProperty(name);
 
             if (state == LoadState.NotSet)
@@ -1048,7 +1047,7 @@ namespace System.DirectoryServices.AccountManagement
             // Make sure we're not disposed or deleted.
             CheckDisposedOrDeleted();
 
-            // Check that we actually support this propery in our store
+            // Check that we actually support this property in our store
             //CheckSupportedProperty(name);
 
             // Need to do this now so that newly-set value doesn't get overwritten by later load
