@@ -164,8 +164,7 @@ namespace System.DirectoryServices.AccountManagement
                 _endReached = false;
                 _current = null;
 
-                if (_enumerator != null)
-                    _enumerator.Reset();
+                _enumerator?.Reset();
 
                 _resultsReturned = 0;
             }
@@ -339,9 +338,7 @@ namespace System.DirectoryServices.AccountManagement
                 filter.Extra = regex;
             }
 
-            Match match = regex.Match(property);
-
-            return match.Success;
+            return regex.IsMatch(property);
         }
         // returns true if specified WinNT property's value matches filter.Value
         private delegate bool MatcherDelegate(FilterBase filter, string winNTPropertyName, DirectoryEntry de);
@@ -864,5 +861,3 @@ namespace System.DirectoryServices.AccountManagement
         }
     }
 }
-
-//#endif  // PAPI_REGSAM
