@@ -354,7 +354,7 @@ namespace System.DirectoryServices.AccountManagement
                 {
                     pSid = Utils.ConvertByteArrayToIntPtr(sid);
 
-                    if (UnsafeNativeMethods.IsValidSid(pSid) && (Utils.ClassifySID(pSid) == SidType.FakeObject))
+                    if (Interop.Advapi32.IsValidSid(pSid) && (Utils.ClassifySID(pSid) == SidType.FakeObject))
                     {
                         GlobalDebug.WriteLineIf(GlobalDebug.Info,
                                                 "SAMStoreCtx",
@@ -419,7 +419,7 @@ namespace System.DirectoryServices.AccountManagement
                         {
                             pSid = Utils.ConvertByteArrayToIntPtr(sid);
 
-                            if (UnsafeNativeMethods.IsValidSid(pSid) && (Utils.ClassifySID(pSid) == SidType.FakeObject))
+                            if (Interop.Advapi32.IsValidSid(pSid) && (Utils.ClassifySID(pSid) == SidType.FakeObject))
                             {
                                 GlobalDebug.WriteLineIf(GlobalDebug.Info,
                                                         "SAMStoreCtx",
@@ -853,7 +853,7 @@ namespace System.DirectoryServices.AccountManagement
         private static void ElapsedTimeFromWinNTConverter(DirectoryEntry de, string suggestedWinNTProperty, Principal p, string propertyName)
         {
             // These properties are expressed as "seconds passed since the event of interest".  So to convert
-            // to a DateTime, we substract them from DateTime.UtcNow.
+            // to a DateTime, we subtract them from DateTime.UtcNow.
 
             PropertyValueCollection values = de.Properties[suggestedWinNTProperty];
 
@@ -1175,5 +1175,3 @@ namespace System.DirectoryServices.AccountManagement
         }
     }
 }
-
-// #endif   // PAPI_REGSAM
