@@ -19,7 +19,7 @@ namespace System.DirectoryServices.ActiveDirectory
         private TopLevelNameStatus _status;
         internal readonly LARGE_INTEGER time;
 
-        internal TopLevelName(int flag, LSA_UNICODE_STRING val, LARGE_INTEGER time)
+        internal TopLevelName(int flag, global::Interop.UNICODE_STRING val, LARGE_INTEGER time)
         {
             _status = (TopLevelNameStatus)flag;
             Name = Marshal.PtrToStringUni(val.Buffer, val.Length / 2);
