@@ -19,18 +19,5 @@ namespace System.IO.IsolatedStorage
                 Assert.Equal(randomPath, Helper.GetExistingRandomDirectory(temp.Path));
             }
         }
-
-        [Theory,
-            InlineData(IsolatedStorageScope.User),
-            InlineData(IsolatedStorageScope.Machine),
-            ]
-        public void GetRandomDirectory(IsolatedStorageScope scope)
-        {
-            using (var temp = new TempDirectory())
-            {
-                string randomDir = Helper.GetRandomDirectory(temp.Path, scope);
-                Assert.True(Directory.Exists(randomDir));
-            }
-        }
     }
 }
