@@ -23,6 +23,9 @@ namespace System.Net.Http
         private const int ColonSpaceLength = 2;
         private const int CommaSpaceLength = 2;
 
+        private static readonly SearchValues<char> s_allowedBoundaryChars =
+            SearchValues.Create(" '()+,-./0123456789:=?ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz");
+
         private readonly List<HttpContent> _nestedContent;
         private readonly string _boundary;
 
@@ -40,10 +43,7 @@ namespace System.Net.Http
 
         public MultipartContent(string subtype, string boundary)
         {
-            if (string.IsNullOrWhiteSpace(subtype))
-            {
-                throw new ArgumentException(SR.net_http_argument_empty_string, nameof(subtype));
-            }
+            ArgumentException.ThrowIfNullOrWhiteSpace(subtype);
             ValidateBoundary(boundary);
 
             _boundary = boundary;
@@ -65,10 +65,7 @@ namespace System.Net.Http
         {
             // NameValueHeaderValue is too restrictive for boundary.
             // Instead validate it ourselves and then quote it.
-            if (string.IsNullOrWhiteSpace(boundary))
-            {
-                throw new ArgumentException(SR.net_http_argument_empty_string, nameof(boundary));
-            }
+            ArgumentException.ThrowIfNullOrWhiteSpace(boundary);
 
             // RFC 2046 Section 5.1.1
             // boundary := 0*69<bchars> bcharsnospace
@@ -85,21 +82,9 @@ namespace System.Net.Http
                 throw new ArgumentException(SR.Format(System.Globalization.CultureInfo.InvariantCulture, SR.net_http_headers_invalid_value, boundary), nameof(boundary));
             }
 
-            const string AllowedMarks = @"'()+_,-./:=? ";
-
-            foreach (char ch in boundary)
+            if (boundary.AsSpan().ContainsAnyExcept(s_allowedBoundaryChars))
             {
-                if (('0' <= ch && ch <= '9') || // Digit.
-                    ('a' <= ch && ch <= 'z') || // alpha.
-                    ('A' <= ch && ch <= 'Z') || // ALPHA.
-                    (AllowedMarks.Contains(ch))) // Marks.
-                {
-                    // Valid.
-                }
-                else
-                {
-                    throw new ArgumentException(SR.Format(System.Globalization.CultureInfo.InvariantCulture, SR.net_http_headers_invalid_value, boundary), nameof(boundary));
-                }
+                throw new ArgumentException(SR.Format(System.Globalization.CultureInfo.InvariantCulture, SR.net_http_headers_invalid_value, boundary), nameof(boundary));
             }
         }
 
@@ -110,10 +95,7 @@ namespace System.Net.Http
 
         public virtual void Add(HttpContent content)
         {
-            if (content == null)
-            {
-                throw new ArgumentNullException(nameof(content));
-            }
+            ArgumentNullException.ThrowIfNull(content);
 
             _nestedContent.Add(content);
         }
@@ -358,12 +340,12 @@ namespace System.Net.Http
             return stream.WriteAsync(new ReadOnlyMemory<byte>(buffer), cancellationToken);
         }
 
-        private static Stream EncodeStringToNewStream(string input)
+        private static MemoryStream EncodeStringToNewStream(string input)
         {
             return new MemoryStream(HttpRuleParser.DefaultHttpEncoding.GetBytes(input), writable: false);
         }
 
-        private Stream EncodeHeadersToNewStream(HttpContent content, bool writeDivider)
+        private MemoryStream EncodeHeadersToNewStream(HttpContent content, bool writeDivider)
         {
             var stream = new MemoryStream();
             SerializeHeadersToStream(stream, content, writeDivider);
@@ -499,6 +481,12 @@ namespace System.Net.Http
                 }
             }
 
+            public override int ReadByte()
+            {
+                byte b = 0;
+                return Read(new Span<byte>(ref b)) == 1 ? b : -1;
+            }
+
             public override int Read(Span<byte> buffer)
             {
                 if (buffer.Length == 0)
@@ -539,10 +527,10 @@ namespace System.Net.Http
                 ReadAsyncPrivate(buffer, cancellationToken);
 
             public override IAsyncResult BeginRead(byte[] array, int offset, int count, AsyncCallback? asyncCallback, object? asyncState) =>
-                TaskToApm.Begin(ReadAsync(array, offset, count, CancellationToken.None), asyncCallback, asyncState);
+                TaskToAsyncResult.Begin(ReadAsync(array, offset, count, CancellationToken.None), asyncCallback, asyncState);
 
             public override int EndRead(IAsyncResult asyncResult) =>
-                TaskToApm.End<int>(asyncResult);
+                TaskToAsyncResult.End<int>(asyncResult);
 
             public async ValueTask<int> ReadAsyncPrivate(Memory<byte> buffer, CancellationToken cancellationToken)
             {
@@ -579,10 +567,7 @@ namespace System.Net.Http
                 get { return _position; }
                 set
                 {
-                    if (value < 0)
-                    {
-                        throw new ArgumentOutOfRangeException(nameof(value));
-                    }
+                    ArgumentOutOfRangeException.ThrowIfNegative(value);
 
                     long previousStreamsLength = 0;
                     for (int i = 0; i < _streams.Length; i++)
@@ -641,6 +626,8 @@ namespace System.Net.Http
             public override long Length => _length;
 
             public override void Flush() { }
+            public override Task FlushAsync(CancellationToken cancellationToken) => Task.CompletedTask;
+
             public override void SetLength(long value) { throw new NotSupportedException(); }
             public override void Write(byte[] buffer, int offset, int count) { throw new NotSupportedException(); }
             public override void Write(ReadOnlySpan<byte> buffer) { throw new NotSupportedException(); }
