@@ -44,17 +44,8 @@ namespace System.Net.Mail
 
         internal Message(string from, string to) : this()
         {
-            if (from == null)
-                throw new ArgumentNullException(nameof(from));
-
-            if (to == null)
-                throw new ArgumentNullException(nameof(to));
-
-            if (from.Length == 0)
-                throw new ArgumentException(SR.Format(SR.net_emptystringcall, nameof(from)), nameof(from));
-
-            if (to.Length == 0)
-                throw new ArgumentException(SR.Format(SR.net_emptystringcall, nameof(to)), nameof(to));
+            ArgumentException.ThrowIfNullOrEmpty(from);
+            ArgumentException.ThrowIfNullOrEmpty(to);
 
             _from = new MailAddress(from);
             MailAddressCollection collection = new MailAddressCollection();
@@ -94,10 +85,7 @@ namespace System.Net.Mail
             }
             set
             {
-                if (value == null)
-                {
-                    throw new ArgumentNullException(nameof(value));
-                }
+                ArgumentNullException.ThrowIfNull(value);
                 _from = value;
             }
         }
@@ -159,7 +147,7 @@ namespace System.Net.Mail
                     {
                         // Store the decoded value, we'll re-encode before sending
                         value = MimeBasePart.DecodeHeaderValue(value);
-                        _subjectEncoding = _subjectEncoding ?? inputEncoding;
+                        _subjectEncoding ??= inputEncoding;
                     }
                     // Failed to decode, just pass it through as ascii (legacy)
                     catch (FormatException) { }
@@ -243,11 +231,7 @@ namespace System.Net.Mail
             }
             set
             {
-                if (value == null)
-                {
-                    throw new ArgumentNullException(nameof(value));
-                }
-
+                ArgumentNullException.ThrowIfNull(value);
                 _content = value;
             }
         }
@@ -268,7 +252,7 @@ namespace System.Net.Mail
             EmptySendContext context = (EmptySendContext)result.AsyncState!;
             try
             {
-                context._writer.EndGetContentStream(result).Close();
+                BaseWriter.EndGetContentStream(result).Close();
             }
             catch (Exception ex)
             {
@@ -289,10 +273,10 @@ namespace System.Net.Mail
             internal BaseWriter _writer;
         }
 
-        internal IAsyncResult BeginSend(BaseWriter writer, bool sendEnvelope, bool allowUnicode,
+        internal IAsyncResult BeginSend(BaseWriter writer, bool allowUnicode,
             AsyncCallback? callback, object? state)
         {
-            PrepareHeaders(sendEnvelope, allowUnicode);
+            PrepareHeaders(allowUnicode);
             writer.WriteHeaders(Headers, allowUnicode);
 
             if (Content != null)
@@ -305,7 +289,7 @@ namespace System.Net.Mail
                 IAsyncResult newResult = writer.BeginGetContentStream(EmptySendCallback, new EmptySendContext(writer, result));
                 if (newResult.CompletedSynchronously)
                 {
-                    writer.EndGetContentStream(newResult).Close();
+                    BaseWriter.EndGetContentStream(newResult).Close();
                     result.InvokeCallback();
                 }
                 return result;
@@ -314,10 +298,7 @@ namespace System.Net.Mail
 
         internal void EndSend(IAsyncResult asyncResult)
         {
-            if (asyncResult == null)
-            {
-                throw new ArgumentNullException(nameof(asyncResult));
-            }
+            ArgumentNullException.ThrowIfNull(asyncResult);
 
             if (Content != null)
             {
@@ -350,11 +331,11 @@ namespace System.Net.Mail
         {
             if (sendEnvelope)
             {
-                PrepareEnvelopeHeaders(sendEnvelope, allowUnicode);
+                PrepareEnvelopeHeaders(allowUnicode);
                 writer.WriteHeaders(EnvelopeHeaders, allowUnicode);
             }
 
-            PrepareHeaders(sendEnvelope, allowUnicode);
+            PrepareHeaders(allowUnicode);
             writer.WriteHeaders(Headers, allowUnicode);
 
             if (Content != null)
@@ -367,12 +348,9 @@ namespace System.Net.Mail
             }
         }
 
-        internal void PrepareEnvelopeHeaders(bool sendEnvelope, bool allowUnicode)
+        internal void PrepareEnvelopeHeaders(bool allowUnicode)
         {
-            if (_headersEncoding == null)
-            {
-                _headersEncoding = Encoding.GetEncoding(MimeBasePart.DefaultCharSet);
-            }
+            _headersEncoding ??= Encoding.GetEncoding(MimeBasePart.DefaultCharSet);
 
             EncodeHeaders(EnvelopeHeaders, allowUnicode);
 
@@ -401,14 +379,9 @@ namespace System.Net.Mail
             }
         }
 
-        internal void PrepareHeaders(bool sendEnvelope, bool allowUnicode)
+        internal void PrepareHeaders(bool allowUnicode)
         {
-            string headerName;
-
-            if (_headersEncoding == null)
-            {
-                _headersEncoding = Encoding.GetEncoding(MimeBasePart.DefaultCharSet);
-            }
+            _headersEncoding ??= Encoding.GetEncoding(MimeBasePart.DefaultCharSet);
 
             //ContentType is written directly to the stream so remove potential user duplicate
             Headers.Remove(MailHeaderInfo.GetString(MailHeaderID.ContentType)!);
@@ -417,7 +390,7 @@ namespace System.Net.Mail
 
             // add sender to headers first so that it is written first to allow the IIS smtp svc to
             // send MAIL FROM with the sender if both sender and from are present
-            headerName = MailHeaderInfo.GetString(MailHeaderID.Sender)!;
+            string headerName = MailHeaderInfo.GetString(MailHeaderID.Sender)!;
             if (Sender != null)
             {
                 Headers.InternalAdd(headerName, Sender.Encode(headerName.Length, allowUnicode));
@@ -532,7 +505,7 @@ namespace System.Net.Mail
                 }
 
                 string[] values = headers.GetValues(headerName)!;
-                string encodedValue = string.Empty;
+                string encodedValue;
                 for (int j = 0; j < values.Length; j++)
                 {
                     //encode if we need to
