@@ -5,8 +5,12 @@ using System.Buffers.Binary;
 using System.Diagnostics;
 using System.Diagnostics.CodeAnalysis;
 using System.Net.Sockets;
+using System.Numerics;
 using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
+using System.Runtime.Intrinsics;
+
+#pragma warning disable SA1648 // TODO: https://github.com/DotNetAnalyzers/StyleCopAnalyzers/issues/3595
 
 namespace System.Net
 {
@@ -15,7 +19,7 @@ namespace System.Net
     ///     Provides an Internet Protocol (IP) address.
     ///   </para>
     /// </devdoc>
-    public class IPAddress
+    public class IPAddress : ISpanFormattable, ISpanParsable<IPAddress>, IUtf8SpanFormattable
     {
         public static readonly IPAddress Any = new ReadOnlyIPAddress(new byte[] { 0, 0, 0, 0 });
         public static readonly IPAddress Loopback = new ReadOnlyIPAddress(new byte[] { 127, 0, 0, 1 });
@@ -24,11 +28,11 @@ namespace System.Net
 
         internal const uint LoopbackMaskHostOrder = 0xFF000000;
 
-        public static readonly IPAddress IPv6Any = new IPAddress((ReadOnlySpan<byte>) new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, 0);
-        public static readonly IPAddress IPv6Loopback = new IPAddress((ReadOnlySpan<byte>) new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 }, 0);
+        public static readonly IPAddress IPv6Any = new IPAddress((ReadOnlySpan<byte>)new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, 0);
+        public static readonly IPAddress IPv6Loopback = new IPAddress((ReadOnlySpan<byte>)new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 }, 0);
         public static readonly IPAddress IPv6None = IPv6Any;
 
-        private static readonly IPAddress s_loopbackMappedToIPv6 = new IPAddress((ReadOnlySpan<byte>) new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 255, 255, 127, 0, 0, 1 }, 0);
+        private static readonly IPAddress s_loopbackMappedToIPv6 = new IPAddress((ReadOnlySpan<byte>)new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 255, 255, 127, 0, 0, 1 }, 0);
 
         /// <summary>
         /// For IPv4 addresses, this field stores the Address.
@@ -54,24 +58,26 @@ namespace System.Net
 
         internal const int NumberOfLabels = IPAddressParserStatics.IPv6AddressBytes / 2;
 
+        [MemberNotNullWhen(false, nameof(_numbers))]
         private bool IsIPv4
         {
             get { return _numbers == null; }
         }
 
+        [MemberNotNullWhen(true, nameof(_numbers))]
         private bool IsIPv6
         {
             get { return _numbers != null; }
         }
 
-        private uint PrivateAddress
+        internal uint PrivateAddress
         {
             get
             {
                 Debug.Assert(IsIPv4);
                 return _addressOrScopeId;
             }
-            set
+            private set
             {
                 Debug.Assert(IsIPv4);
                 _toString = null;
@@ -104,10 +110,7 @@ namespace System.Net
         /// </devdoc>
         public IPAddress(long newAddress)
         {
-            if (newAddress < 0 || newAddress > 0x00000000FFFFFFFF)
-            {
-                throw new ArgumentOutOfRangeException(nameof(newAddress));
-            }
+            ArgumentOutOfRangeException.ThrowIfGreaterThan((ulong)newAddress, 0x00000000FFFFFFFF, nameof(newAddress));
 
             PrivateAddress = (uint)newAddress;
         }
@@ -131,18 +134,9 @@ namespace System.Net
 
             // Consider: Since scope is only valid for link-local and site-local
             //           addresses we could implement some more robust checking here
-            if (scopeid < 0 || scopeid > 0x00000000FFFFFFFF)
-            {
-                throw new ArgumentOutOfRangeException(nameof(scopeid));
-            }
-
-            _numbers = new ushort[NumberOfLabels];
-
-            for (int i = 0; i < NumberOfLabels; i++)
-            {
-                _numbers[i] = (ushort)(address[i * 2] * 256 + address[i * 2 + 1]);
-            }
+            ArgumentOutOfRangeException.ThrowIfGreaterThan((ulong)scopeid, 0x00000000FFFFFFFF, nameof(scopeid));
 
+            _numbers = ReadUInt16NumbersFromBytes(address);
             PrivateScopeId = (uint)scopeid;
         }
 
@@ -151,13 +145,7 @@ namespace System.Net
             Debug.Assert(numbers != null);
             Debug.Assert(numbers.Length == NumberOfLabels);
 
-            var arr = new ushort[NumberOfLabels];
-            for (int i = 0; i < arr.Length; i++)
-            {
-                arr[i] = numbers[i];
-            }
-
-            _numbers = arr;
+            _numbers = numbers.ToArray();
             PrivateScopeId = scopeid;
         }
 
@@ -188,12 +176,7 @@ namespace System.Net
             }
             else if (address.Length == IPAddressParserStatics.IPv6AddressBytes)
             {
-                _numbers = new ushort[NumberOfLabels];
-
-                for (int i = 0; i < NumberOfLabels; i++)
-                {
-                    _numbers[i] = (ushort)(address[i * 2] * 256 + address[i * 2 + 1]);
-                }
+                _numbers = ReadUInt16NumbersFromBytes(address);
             }
             else
             {
@@ -201,6 +184,31 @@ namespace System.Net
             }
         }
 
+        [MethodImpl(MethodImplOptions.AggressiveInlining)]
+        private static ushort[] ReadUInt16NumbersFromBytes(ReadOnlySpan<byte> address)
+        {
+            ushort[] numbers = new ushort[NumberOfLabels];
+            if (Vector128.IsHardwareAccelerated)
+            {
+                Vector128<ushort> ushorts = Vector128.LoadUnsafe(ref MemoryMarshal.GetReference(address)).AsUInt16();
+                if (BitConverter.IsLittleEndian)
+                {
+                    // Reverse endianness of each ushort
+                    ushorts = Vector128.ShiftLeft(ushorts, 8) | Vector128.ShiftRightLogical(ushorts, 8);
+                }
+                ushorts.StoreUnsafe(ref MemoryMarshal.GetArrayDataReference(numbers));
+            }
+            else
+            {
+                for (int i = 0; i < numbers.Length; i++)
+                {
+                    numbers[i] = BinaryPrimitives.ReadUInt16BigEndian(address.Slice(i * 2));
+                }
+            }
+
+            return numbers;
+        }
+
         // We need this internally since we need to interface with winsock,
         // and winsock only understands Int32.
         internal IPAddress(int newAddress)
@@ -231,12 +239,19 @@ namespace System.Net
             return (address != null);
         }
 
+        /// <inheritdoc/>
+        static bool IParsable<IPAddress>.TryParse([NotNullWhen(true)] string? s, IFormatProvider? provider, [NotNullWhen(true)] out IPAddress? result) =>
+            // provider is explicitly ignored
+            TryParse(s, out result);
+
+        /// <inheritdoc/>
+        static bool ISpanParsable<IPAddress>.TryParse(ReadOnlySpan<char> s, IFormatProvider? provider, [NotNullWhen(true)] out IPAddress? result) =>
+            // provider is explicitly ignored
+            TryParse(s, out result);
+
         public static IPAddress Parse(string ipString)
         {
-            if (ipString == null)
-            {
-                throw new ArgumentNullException(nameof(ipString));
-            }
+            ArgumentNullException.ThrowIfNull(ipString);
 
             return IPAddressParser.Parse(ipString.AsSpan(), tryParse: false)!;
         }
@@ -246,6 +261,16 @@ namespace System.Net
             return IPAddressParser.Parse(ipSpan, tryParse: false)!;
         }
 
+        /// <inheritdoc/>
+        static IPAddress ISpanParsable<IPAddress>.Parse(ReadOnlySpan<char> s, IFormatProvider? provider) =>
+            // provider is explicitly ignored
+            Parse(s);
+
+        /// <inheritdoc/>
+        static IPAddress IParsable<IPAddress>.Parse(string s, IFormatProvider? provider) =>
+            // provider is explicitly ignored
+            Parse(s);
+
         public bool TryWriteBytes(Span<byte> destination, out int bytesWritten)
         {
             if (IsIPv6)
@@ -277,12 +302,28 @@ namespace System.Net
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
         private void WriteIPv6Bytes(Span<byte> destination)
         {
-            Debug.Assert(_numbers != null && _numbers.Length == NumberOfLabels);
-            int j = 0;
-            for (int i = 0; i < NumberOfLabels; i++)
+            ushort[]? numbers = _numbers;
+            Debug.Assert(numbers != null && numbers.Length == NumberOfLabels);
+
+            if (BitConverter.IsLittleEndian)
             {
-                destination[j++] = (byte)((_numbers[i] >> 8) & 0xFF);
-                destination[j++] = (byte)((_numbers[i]) & 0xFF);
+                if (Vector128.IsHardwareAccelerated)
+                {
+                    Vector128<ushort> ushorts = Vector128.LoadUnsafe(ref MemoryMarshal.GetArrayDataReference(numbers));
+                    ushorts = Vector128.ShiftLeft(ushorts, 8) | Vector128.ShiftRightLogical(ushorts, 8);
+                    ushorts.AsByte().StoreUnsafe(ref MemoryMarshal.GetReference(destination));
+                }
+                else
+                {
+                    for (int i = 0; i < numbers.Length; i++)
+                    {
+                        BinaryPrimitives.WriteUInt16BigEndian(destination.Slice(i * 2), numbers[i]);
+                    }
+                }
+            }
+            else
+            {
+                MemoryMarshal.AsBytes<ushort>(numbers).CopyTo(destination);
             }
         }
 
@@ -290,7 +331,7 @@ namespace System.Net
         private void WriteIPv4Bytes(Span<byte> destination)
         {
             uint address = PrivateAddress;
-            MemoryMarshal.Write(destination, ref address);
+            MemoryMarshal.Write(destination, in address);
         }
 
         /// <devdoc>
@@ -335,7 +376,7 @@ namespace System.Net
                 // Not valid for IPv4 addresses
                 if (IsIPv4)
                 {
-                    throw new SocketException(SocketError.OperationNotSupported);
+                    ThrowSocketOperationNotSupported();
                 }
 
                 return PrivateScopeId;
@@ -345,15 +386,13 @@ namespace System.Net
                 // Not valid for IPv4 addresses
                 if (IsIPv4)
                 {
-                    throw new SocketException(SocketError.OperationNotSupported);
+                    ThrowSocketOperationNotSupported();
                 }
 
                 // Consider: Since scope is only valid for link-local and site-local
                 //           addresses we could implement some more robust checking here
-                if (value < 0 || value > 0x00000000FFFFFFFF)
-                {
-                    throw new ArgumentOutOfRangeException(nameof(value));
-                }
+                ArgumentOutOfRangeException.ThrowIfNegative(value);
+                ArgumentOutOfRangeException.ThrowIfGreaterThan(value, 0x00000000FFFFFFFF);
 
                 PrivateScopeId = (uint)value;
             }
@@ -367,21 +406,78 @@ namespace System.Net
         /// </devdoc>
         public override string ToString()
         {
-            if (_toString == null)
+            string? toString = _toString;
+            if (toString is null)
             {
-                _toString = IsIPv4 ?
-                    IPAddressParser.IPv4AddressToString(PrivateAddress) :
-                    IPAddressParser.IPv6AddressToString(_numbers!, PrivateScopeId);
+                Span<char> span = stackalloc char[IPAddressParser.MaxIPv6StringLength];
+                int length = IsIPv4 ?
+                    IPAddressParser.FormatIPv4Address(_addressOrScopeId, span) :
+                    IPAddressParser.FormatIPv6Address(_numbers, _addressOrScopeId, span);
+                _toString = toString = new string(span.Slice(0, length));
             }
 
-            return _toString;
+            return toString;
         }
 
-        public bool TryFormat(Span<char> destination, out int charsWritten)
+        /// <inheritdoc/>
+        string IFormattable.ToString(string? format, IFormatProvider? formatProvider) =>
+            // format and provider are explicitly ignored
+            ToString();
+
+        public bool TryFormat(Span<char> destination, out int charsWritten) =>
+            TryFormatCore(destination, out charsWritten);
+
+        /// <summary>Tries to format the current IP address into the provided span.</summary>
+        /// <param name="utf8Destination">When this method returns, the IP address as a span of UTF-8 bytes.</param>
+        /// <param name="bytesWritten">When this method returns, the number of bytes written into the <paramref name="utf8Destination"/>.</param>
+        /// <returns><see langword="true" /> if the formatting was successful; otherwise, <see langword="false" />.</returns>
+        public bool TryFormat(Span<byte> utf8Destination, out int bytesWritten) =>
+            TryFormatCore(utf8Destination, out bytesWritten);
+
+        /// <inheritdoc/>
+        bool ISpanFormattable.TryFormat(Span<char> destination, out int charsWritten, ReadOnlySpan<char> format, IFormatProvider? provider) =>
+            // format and provider are explicitly ignored
+            TryFormatCore(destination, out charsWritten);
+
+        /// <inheritdoc/>
+        bool IUtf8SpanFormattable.TryFormat(Span<byte> utf8Destination, out int bytesWritten, ReadOnlySpan<char> format, IFormatProvider? provider) =>
+            // format and provider are explicitly ignored
+            TryFormatCore(utf8Destination, out bytesWritten);
+
+        private bool TryFormatCore<TChar>(Span<TChar> destination, out int charsWritten) where TChar : unmanaged, IBinaryInteger<TChar>
         {
-            return IsIPv4 ?
-                IPAddressParser.IPv4AddressToString(PrivateAddress, destination, out charsWritten) :
-                IPAddressParser.IPv6AddressToString(_numbers!, PrivateScopeId, destination, out charsWritten);
+            if (IsIPv4)
+            {
+                if (destination.Length >= IPAddressParser.MaxIPv4StringLength)
+                {
+                    charsWritten = IPAddressParser.FormatIPv4Address(_addressOrScopeId, destination);
+                    return true;
+                }
+            }
+            else
+            {
+                if (destination.Length >= IPAddressParser.MaxIPv6StringLength)
+                {
+                    charsWritten = IPAddressParser.FormatIPv6Address(_numbers, _addressOrScopeId, destination);
+                    return true;
+                }
+            }
+
+            Span<TChar> tmpDestination = stackalloc TChar[IPAddressParser.MaxIPv6StringLength];
+            Debug.Assert(tmpDestination.Length >= IPAddressParser.MaxIPv4StringLength);
+
+            int written = IsIPv4 ?
+                IPAddressParser.FormatIPv4Address(PrivateAddress, tmpDestination) :
+                IPAddressParser.FormatIPv6Address(_numbers, PrivateScopeId, tmpDestination);
+
+            if (tmpDestination.Slice(0, written).TryCopyTo(destination))
+            {
+                charsWritten = written;
+                return true;
+            }
+
+            charsWritten = 0;
+            return false;
         }
 
         public static long HostToNetworkOrder(long host)
@@ -416,10 +512,7 @@ namespace System.Net
 
         public static bool IsLoopback(IPAddress address)
         {
-            if (address == null)
-            {
-                ThrowAddressNullException();
-            }
+            ArgumentNullException.ThrowIfNull(address);
 
             if (address.IsIPv6)
             {
@@ -442,7 +535,7 @@ namespace System.Net
         {
             get
             {
-                return IsIPv6 && ((_numbers![0] & 0xFF00) == 0xFF00);
+                return IsIPv6 && ((_numbers[0] & 0xFF00) == 0xFF00);
             }
         }
 
@@ -455,7 +548,7 @@ namespace System.Net
         {
             get
             {
-                return IsIPv6 && ((_numbers![0] & 0xFFC0) == 0xFE80);
+                return IsIPv6 && ((_numbers[0] & 0xFFC0) == 0xFE80);
             }
         }
 
@@ -468,7 +561,7 @@ namespace System.Net
         {
             get
             {
-                return IsIPv6 && ((_numbers![0] & 0xFFC0) == 0xFEC0);
+                return IsIPv6 && ((_numbers[0] & 0xFFC0) == 0xFEC0);
             }
         }
 
@@ -477,8 +570,8 @@ namespace System.Net
             get
             {
                 return IsIPv6 &&
-                       (_numbers![0] == 0x2001) &&
-                       (_numbers![1] == 0);
+                       (_numbers[0] == 0x2001) &&
+                       (_numbers[1] == 0);
             }
         }
 
@@ -487,7 +580,7 @@ namespace System.Net
         {
             get
             {
-                return IsIPv6 && ((_numbers![0] & 0xFE00) == 0xFC00);
+                return IsIPv6 && ((_numbers[0] & 0xFE00) == 0xFC00);
             }
         }
 
@@ -500,14 +593,11 @@ namespace System.Net
                 {
                     return false;
                 }
-                for (int i = 0; i < 5; i++)
-                {
-                    if (_numbers![i] != 0)
-                    {
-                        return false;
-                    }
-                }
-                return (_numbers![5] == 0xFFFF);
+
+                ReadOnlySpan<byte> numbers = MemoryMarshal.AsBytes(new ReadOnlySpan<ushort>(_numbers));
+                return
+                    MemoryMarshal.Read<ulong>(numbers) == 0 &&
+                    BinaryPrimitives.ReadUInt32LittleEndian(numbers.Slice(8)) == 0xFFFF0000;
             }
         }
 
@@ -516,37 +606,28 @@ namespace System.Net
         {
             get
             {
-                //
-                // IPv6 Changes: Can't do this for IPv6, so throw an exception.
-                //
-                //
                 if (AddressFamily == AddressFamily.InterNetworkV6)
                 {
-                    throw new SocketException(SocketError.OperationNotSupported);
-                }
-                else
-                {
-                    return PrivateAddress;
+                    ThrowSocketOperationNotSupported();
                 }
+
+                return PrivateAddress;
             }
             set
             {
-                //
-                // IPv6 Changes: Can't do this for IPv6 addresses
                 if (AddressFamily == AddressFamily.InterNetworkV6)
                 {
-                    throw new SocketException(SocketError.OperationNotSupported);
+                    ThrowSocketOperationNotSupported();
                 }
-                else
+
+                if (PrivateAddress != value)
                 {
-                    if (PrivateAddress != value)
+                    if (this is ReadOnlyIPAddress)
                     {
-                        if (this is ReadOnlyIPAddress)
-                        {
-                            throw new SocketException(SocketError.OperationNotSupported);
-                        }
-                        PrivateAddress = unchecked((uint)value);
+                        ThrowSocketOperationNotSupported();
                     }
+
+                    PrivateAddress = unchecked((uint)value);
                 }
             }
         }
@@ -635,13 +716,16 @@ namespace System.Net
                 return this;
             }
 
-            uint address = (uint)_numbers![6] << 16 | (uint)_numbers[7];
+            uint address = (uint)_numbers[6] << 16 | (uint)_numbers[7];
             return new IPAddress((uint)HostToNetworkOrder(unchecked((int)address)));
         }
 
         [DoesNotReturn]
         private static byte[] ThrowAddressNullException() => throw new ArgumentNullException("address");
 
+        [DoesNotReturn]
+        private static void ThrowSocketOperationNotSupported() => throw new SocketException(SocketError.OperationNotSupported);
+
         private sealed class ReadOnlyIPAddress : IPAddress
         {
             public ReadOnlyIPAddress(ReadOnlySpan<byte> newAddress) : base(newAddress)
