@@ -62,7 +62,6 @@ namespace System.Net.Sockets.Tests
         [Theory]
         [InlineData(false)]
         [InlineData(true)]
-        [PlatformSpecific(TestPlatforms.Windows)]
         public async Task UdpConnection_ThrowsException(bool usePreAndPostbufferOverload)
         {
             // Create file to send
@@ -77,16 +76,14 @@ namespace System.Net.Sockets.Tests
 
             client.Connect(listener.LocalEndPoint);
 
-            SocketException ex;
             if (usePreAndPostbufferOverload)
             {
-                ex = await Assert.ThrowsAsync<SocketException>(() => SendFileAsync(client, tempFile.Path, Array.Empty<byte>(), Array.Empty<byte>(), TransmitFileOptions.UseDefaultWorkerThread));
+                await Assert.ThrowsAsync<NotSupportedException>(() => SendFileAsync(client, tempFile.Path, Array.Empty<byte>(), Array.Empty<byte>(), TransmitFileOptions.UseDefaultWorkerThread));
             }
             else
             {
-                ex = await Assert.ThrowsAsync<SocketException>(() => SendFileAsync(client, tempFile.Path));
+                await Assert.ThrowsAsync<NotSupportedException>(() => SendFileAsync(client, tempFile.Path));
             }
-            Assert.Equal(SocketError.NotConnected, ex.SocketErrorCode);
         }
 
         public static IEnumerable<object[]> SendFile_MemberData()
@@ -235,16 +232,27 @@ namespace System.Net.Sockets.Tests
             }
         }
 
-        [Fact]
-        public async Task SendFileGetsCanceledByDispose()
+        [Theory]
+        [InlineData(true)]
+        [InlineData(false)]
+        [ActiveIssue("https://github.com/dotnet/runtime/issues/73536", TestPlatforms.iOS | TestPlatforms.tvOS)]
+        [ActiveIssue("https://github.com/dotnet/runtime/issues/80169", typeof(PlatformDetection), nameof(PlatformDetection.IsOSXLike))]
+        public async Task SendFileGetsCanceledByDispose(bool owning)
         {
+            // Aborting sync operations for non-owning handles is not supported on Unix.
+            if (!owning && UsesSync && !PlatformDetection.IsWindows)
+            {
+                return;
+            }
+
             // We try this a couple of times to deal with a timing race: if the Dispose happens
             // before the operation is started, the peer won't see a ConnectionReset SocketException and we won't
             // see a SocketException either.
-            int msDelay = 100;
             await RetryHelper.ExecuteAsync(async () =>
             {
                 (Socket socket1, Socket socket2) = SocketTestExtensions.CreateConnectedSocketPair();
+                using SafeSocketHandle? owner = ReplaceWithNonOwning(ref socket1, owning);
+
                 using (socket2)
                 {
                     Task socketOperation = Task.Run(async () =>
@@ -259,16 +267,17 @@ namespace System.Net.Sockets.Tests
                         await SendFileAsync(socket1, tempFile.Path);
                     });
 
-                    // Wait a little so the operation is started.
-                    await Task.Delay(msDelay);
-                    msDelay *= 2;
+                    // read one byte to make sure SendFileAsync started
+                    byte[] buffer = new byte[1];
+                    socket2.Receive(buffer);
+
                     Task disposeTask = Task.Run(() => socket1.Dispose());
 
                     await Task.WhenAny(disposeTask, socketOperation).WaitAsync(TimeSpan.FromSeconds(30));
                     await disposeTask;
 
                     SocketError? localSocketError = null;
-                    bool disposedException = false;
+
                     try
                     {
                         await socketOperation;
@@ -277,17 +286,8 @@ namespace System.Net.Sockets.Tests
                     {
                         localSocketError = se.SocketErrorCode;
                     }
-                    catch (ObjectDisposedException)
-                    {
-                        disposedException = true;
-                    }
 
-                    if (UsesApm)
-                    {
-                        Assert.Null(localSocketError);
-                        Assert.True(disposedException);
-                    }
-                    else if (UsesSync)
+                    if (UsesSync)
                     {
                         Assert.Equal(SocketError.ConnectionAborted, localSocketError);
                     }
@@ -296,6 +296,8 @@ namespace System.Net.Sockets.Tests
                         Assert.Equal(SocketError.OperationAborted, localSocketError);
                     }
 
+                    owner?.Dispose();
+
                     // On OSX, we're unable to unblock the on-going socket operations and
                     // perform an abortive close.
                     if (!(UsesSync && PlatformDetection.IsOSXLike))
@@ -454,7 +456,6 @@ namespace System.Net.Sockets.Tests
             s.Dispose();
             Assert.Throws<ObjectDisposedException>(() => s.BeginSendFile(null, null, null));
             Assert.Throws<ObjectDisposedException>(() => s.BeginSendFile(null, null, null, TransmitFileOptions.UseDefaultWorkerThread, null, null));
-            Assert.Throws<ObjectDisposedException>(() => s.EndSendFile(null));
         }
 
         [Fact]
@@ -467,7 +468,7 @@ namespace System.Net.Sockets.Tests
 
     // Running all cases of GreaterThan2GBFile_SendsAllBytes in parallel may attempt to allocate Min(ProcessorCount, Subclass_Count) * 2GB of disk space
     // in extreme cases. Some CI machines may run out of disk space if this happens.
-    [Collection(nameof(NoParallelTests))]
+    [Collection(nameof(DisableParallelization))]
     public abstract class SendFile_NonParallel<T> : SocketTestHelperBase<T> where T : SocketHelperBase, new()
     {
         protected SendFile_NonParallel(ITestOutputHelper output) : base(output)
