@@ -1,15 +1,6 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-/*============================================================
-**
-** This file exists to contain miscellaneous module-level attributes
-** and other miscellaneous stuff.
-**
-**
-**
-===========================================================*/
-
 using System;
 using System.Reflection;
 using System.Runtime.InteropServices;
@@ -21,7 +12,6 @@ using System.Resources;
 [assembly: DefaultDllImportSearchPaths(DllImportSearchPath.AssemblyDirectory | DllImportSearchPath.System32)]
 
 [assembly: AssemblyMetadata("Serviceable", "True")]
-[assembly: AssemblyMetadata(".NETFrameworkAssembly", "")]
 [assembly: AssemblyMetadata("IsTrimmable", "True")]
 
 [assembly: NeutralResourcesLanguage("en-US")]
