@@ -7,6 +7,7 @@ using System.Collections.Generic;
 using System.Diagnostics;
 using System.Diagnostics.CodeAnalysis;
 using System.IO;
+using System.Runtime.InteropServices;
 using System.Security;
 
 using Internal.Win32.SafeHandles;
@@ -37,10 +38,7 @@ namespace Internal.Win32
 
         void IDisposable.Dispose()
         {
-            if (_hkey != null)
-            {
-                _hkey.Dispose();
-            }
+            _hkey?.Dispose();
         }
 
         public void DeleteValue(string name, bool throwOnMissingValue)
@@ -98,6 +96,8 @@ namespace Internal.Win32
                 return new RegistryKey(result);
             }
 
+            result.Dispose();
+
             // Return null if we didn't find the key.
             if (ret == Interop.Errors.ERROR_ACCESS_DENIED || ret == Interop.Errors.ERROR_BAD_IMPERSONATION_LEVEL)
             {
@@ -112,40 +112,33 @@ namespace Internal.Win32
         public string[] GetSubKeyNames()
         {
             var names = new List<string>();
-            char[] name = ArrayPool<char>.Shared.Rent(MaxKeyLength + 1);
+            Span<char> name = stackalloc char[MaxKeyLength + 1];
 
-            try
+            int result;
+            int nameLength = name.Length;
+
+            while ((result = Interop.Advapi32.RegEnumKeyEx(
+                _hkey,
+                names.Count,
+                ref MemoryMarshal.GetReference(name),
+                ref nameLength,
+                null,
+                null,
+                null,
+                null)) != Interop.Errors.ERROR_NO_MORE_ITEMS)
             {
-                int result;
-                int nameLength = name.Length;
-
-                while ((result = Interop.Advapi32.RegEnumKeyEx(
-                    _hkey,
-                    names.Count,
-                    name,
-                    ref nameLength,
-                    null,
-                    null,
-                    null,
-                    null)) != Interop.Errors.ERROR_NO_MORE_ITEMS)
+                switch (result)
                 {
-                    switch (result)
-                    {
-                        case Interop.Errors.ERROR_SUCCESS:
-                            names.Add(new string(name, 0, nameLength));
-                            nameLength = name.Length;
-                            break;
-                        default:
-                            // Throw the error
-                            Win32Error(result, null);
-                            break;
-                    }
+                    case Interop.Errors.ERROR_SUCCESS:
+                        names.Add(new string(name.Slice(0, nameLength)));
+                        nameLength = name.Length;
+                        break;
+                    default:
+                        // Throw the error
+                        Win32Error(result, null);
+                        break;
                 }
             }
-            finally
-            {
-                ArrayPool<char>.Shared.Return(name);
-            }
 
             return names.ToArray();
         }
@@ -219,226 +212,195 @@ namespace Internal.Win32
             return GetValue(name, null);
         }
 
-        [return: NotNullIfNotNull("defaultValue")]
-        public object? GetValue(string name, object? defaultValue)
+        [return: NotNullIfNotNull(nameof(defaultValue))]
+        public unsafe object? GetValue(string name, object? defaultValue)
         {
-            object? data = defaultValue;
-            int type = 0;
-            int datasize = 0;
+            // Create an initial stack buffer large enough to satisfy many reg keys.  We need to call RegQueryValueEx
+            // in order to determine the type of the value, and we can avoid further retries if all of the data can be
+            // retrieved in that single call with this buffer.  If we do need to grow, we grow into a pooled array. The
+            // caller is always handed back a copy of this data, either in an array, a string, or a boxed integer.
+            Span<byte> span = stackalloc byte[512];
+            byte[]? pooledArray = null;
 
-            int ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, (byte[]?)null, ref datasize);
-
-            if (ret != 0)
+            try
             {
-                // For stuff like ERROR_FILE_NOT_FOUND, we want to return null (data).
-                // Some OS's returned ERROR_MORE_DATA even in success cases, so we
-                // want to continue on through the function.
-                if (ret != Interop.Errors.ERROR_MORE_DATA)
-                    return data;
-            }
+                // Loop in case we need to try again with a larger buffer size.
+                while (true)
+                {
+                    int type = 0;
+                    int result;
+                    int dataLength = span.Length;
 
-            if (datasize < 0)
-            {
-                // unexpected code path
-                Debug.Fail("[InternalGetValue] RegQueryValue returned ERROR_SUCCESS but gave a negative datasize");
-                datasize = 0;
-            }
-
-
-            switch (type)
-            {
-                case Interop.Advapi32.RegistryValues.REG_NONE:
-                case Interop.Advapi32.RegistryValues.REG_DWORD_BIG_ENDIAN:
-                case Interop.Advapi32.RegistryValues.REG_BINARY:
+                    fixed (byte* lpData = &MemoryMarshal.GetReference(span))
                     {
-                        byte[] blob = new byte[datasize];
-                        ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, blob, ref datasize);
-                        data = blob;
-                    }
-                    break;
-                case Interop.Advapi32.RegistryValues.REG_QWORD:
-                    {    // also REG_QWORD_LITTLE_ENDIAN
-                        if (datasize > 8)
+                        result = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, &type, lpData, (uint*)&dataLength);
+                        if (dataLength < 0)
                         {
-                            // prevent an AV in the edge case that datasize is larger than sizeof(long)
-                            goto case Interop.Advapi32.RegistryValues.REG_BINARY;
+                            // Greater than 2GB values aren't supported.
+                            throw new IOException(SR.Arg_RegValueTooLarge);
                         }
-                        long blob = 0;
-                        Debug.Assert(datasize == 8, "datasize==8");
-                        // Here, datasize must be 8 when calling this
-                        ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, ref blob, ref datasize);
-
-                        data = blob;
                     }
-                    break;
-                case Interop.Advapi32.RegistryValues.REG_DWORD:
-                    {    // also REG_DWORD_LITTLE_ENDIAN
-                        if (datasize > 4)
-                        {
-                            // prevent an AV in the edge case that datasize is larger than sizeof(int)
-                            goto case Interop.Advapi32.RegistryValues.REG_QWORD;
-                        }
-                        int blob = 0;
-                        Debug.Assert(datasize == 4, "datasize==4");
-                        // Here, datasize must be four when calling this
-                        ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, ref blob, ref datasize);
 
-                        data = blob;
-                    }
-                    break;
-
-                case Interop.Advapi32.RegistryValues.REG_SZ:
+                    // If RegQueryValueEx told us we need a larger buffer, get one and then loop around to try again.
+                    if (result == Interop.Errors.ERROR_MORE_DATA)
                     {
-                        if (datasize % 2 == 1)
+                        if (pooledArray is not null)
                         {
-                            // handle the case where the registry contains an odd-byte length (corrupt data?)
-                            try
-                            {
-                                datasize = checked(datasize + 1);
-                            }
-                            catch (OverflowException e)
-                            {
-                                throw new IOException(SR.Arg_RegGetOverflowBug, e);
-                            }
+                            // This should only happen if the registry key was changed concurrently, such that
+                            // we called RegQueryValueEx with our initial buffer size that was too small, we then
+                            // rented a buffer of the reportedly right size and called RegQueryValueEx again, but
+                            // it still came back with ERROR_MORE_DATA again.
+                            byte[] toReturn = pooledArray;
+                            pooledArray = null;
+                            ArrayPool<byte>.Shared.Return(toReturn);
                         }
-                        char[] blob = new char[datasize / 2];
 
-                        ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, blob, ref datasize);
-                        if (blob.Length > 0 && blob[^1] == (char)0)
+                        // Greater than 2GB values aren't supported.
+                        if (dataLength < 0)
                         {
-                            data = new string(blob, 0, blob.Length - 1);
-                        }
-                        else
-                        {
-                            // in the very unlikely case the data is missing null termination,
-                            // pass in the whole char[] to prevent truncating a character
-                            data = new string(blob);
+                            throw new IOException(SR.Arg_RegValueTooLarge);
                         }
+
+                        span = pooledArray = ArrayPool<byte>.Shared.Rent(dataLength);
+
+                        continue;
                     }
-                    break;
 
-                case Interop.Advapi32.RegistryValues.REG_EXPAND_SZ:
+                    // For any other error, return the default value.  This might be ERROR_FILE_NOT_FOUND if the reg key
+                    // wasn't found, or any other system error value for unspecified reasons. For compat, an exception
+                    // is thrown for perf keys rather than returning the default value.
+                    if (result != Interop.Errors.ERROR_SUCCESS)
                     {
-                        if (datasize % 2 == 1)
-                        {
-                            // handle the case where the registry contains an odd-byte length (corrupt data?)
-                            try
-                            {
-                                datasize = checked(datasize + 1);
-                            }
-                            catch (OverflowException e)
-                            {
-                                throw new IOException(SR.Arg_RegGetOverflowBug, e);
-                            }
-                        }
-                        char[] blob = new char[datasize / 2];
-
-                        ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, blob, ref datasize);
-
-                        if (blob.Length > 0 && blob[^1] == (char)0)
-                        {
-                            data = new string(blob, 0, blob.Length - 1);
-                        }
-                        else
-                        {
-                            // in the very unlikely case the data is missing null termination,
-                            // pass in the whole char[] to prevent truncating a character
-                            data = new string(blob);
-                        }
-
-                        data = Environment.ExpandEnvironmentVariables((string)data);
+                        return defaultValue;
                     }
-                    break;
-                case Interop.Advapi32.RegistryValues.REG_MULTI_SZ:
+
+                    // We only get here for a successful query of the data. Process and return the results.
+                    Debug.Assert((uint)dataLength <= span.Length, $"Expected {dataLength} <= {span.Length}");
+                    switch (type)
                     {
-                        if (datasize % 2 == 1)
-                        {
-                            // handle the case where the registry contains an odd-byte length (corrupt data?)
-                            try
+                        case Interop.Advapi32.RegistryValues.REG_NONE:
+                        case Interop.Advapi32.RegistryValues.REG_BINARY:
+                        case Interop.Advapi32.RegistryValues.REG_DWORD_BIG_ENDIAN:
+                            return span.Slice(0, dataLength).ToArray();
+
+                        case Interop.Advapi32.RegistryValues.REG_DWORD:
+                        case Interop.Advapi32.RegistryValues.REG_QWORD:
+                            return dataLength switch
                             {
-                                datasize = checked(datasize + 1);
-                            }
-                            catch (OverflowException e)
+                                4 => MemoryMarshal.Read<int>(span),
+                                8 => MemoryMarshal.Read<long>(span),
+                                _ => span.Slice(0, dataLength).ToArray(), // This shouldn't happen, but the previous implementation included it defensively.
+                            };
+
+                        case Interop.Advapi32.RegistryValues.REG_SZ:
+                        case Interop.Advapi32.RegistryValues.REG_EXPAND_SZ:
+                        case Interop.Advapi32.RegistryValues.REG_MULTI_SZ:
                             {
-                                throw new IOException(SR.Arg_RegGetOverflowBug, e);
-                            }
-                        }
-                        char[] blob = new char[datasize / 2];
-
-                        ret = Interop.Advapi32.RegQueryValueEx(_hkey, name, null, ref type, blob, ref datasize);
-
-                        // make sure the string is null terminated before processing the data
-                        if (blob.Length > 0 && blob[^1] != (char)0)
-                        {
-                            Array.Resize(ref blob, blob.Length + 1);
-                        }
-
-                        string[] strings = Array.Empty<string>();
-                        int stringsCount = 0;
-
-                        int cur = 0;
-                        int len = blob.Length;
-
-                        while (ret == 0 && cur < len)
-                        {
-                            int nextNull = cur;
-                            while (nextNull < len && blob[nextNull] != (char)0)
-                            {
-                                nextNull++;
-                            }
-
-                            string? toAdd = null;
-                            if (nextNull < len)
-                            {
-                                Debug.Assert(blob[nextNull] == (char)0, "blob[nextNull] should be 0");
-                                if (nextNull - cur > 0)
+                                // Handle the case where the registry contains an odd-byte length (corrupt data?)
+                                // by increasing the data by a single zero byte.
+                                if (dataLength % 2 == 1)
                                 {
-                                    toAdd = new string(blob, cur, nextNull - cur);
+                                    if (dataLength == int.MaxValue)
+                                    {
+                                        throw new IOException(SR.Arg_RegValueTooLarge);
+                                    }
+
+                                    if (dataLength >= span.Length)
+                                    {
+                                        byte[] newPooled = ArrayPool<byte>.Shared.Rent(dataLength + 1);
+                                        span.CopyTo(newPooled);
+                                        if (pooledArray is not null)
+                                        {
+                                            byte[] toReturn = pooledArray;
+                                            pooledArray = null;
+                                            ArrayPool<byte>.Shared.Return(toReturn);
+                                        }
+                                        span = pooledArray = newPooled;
+                                    }
+
+                                    span[dataLength++] = 0;
+                                }
+
+                                // From here on, we interpret the read bytes as chars; span and dataLength should no longer be used.
+                                ReadOnlySpan<char> chars = MemoryMarshal.Cast<byte, char>(span.Slice(0, dataLength));
+
+                                if (type == Interop.Advapi32.RegistryValues.REG_MULTI_SZ)
+                                {
+                                    string[] strings = Array.Empty<string>();
+                                    int count = 0;
+
+                                    while (chars.Length > 1 || (chars.Length == 1 && chars[0] != '\0'))
+                                    {
+                                        int nullPos = chars.IndexOf('\0');
+                                        string toAdd;
+                                        if (nullPos < 0)
+                                        {
+                                            toAdd = chars.ToString();
+                                            chars = default;
+                                        }
+                                        else
+                                        {
+                                            toAdd = chars.Slice(0, nullPos).ToString();
+                                            chars = chars.Slice(nullPos + 1);
+                                        }
+
+                                        if (count == strings.Length)
+                                        {
+                                            Array.Resize(ref strings, count == 0 ? 4 : count * 2);
+                                        }
+                                        strings[count++] = toAdd;
+                                    }
+
+                                    if (count != 0)
+                                    {
+                                        Array.Resize(ref strings, count);
+                                    }
+
+                                    return strings;
                                 }
                                 else
                                 {
-                                    // we found an empty string.  But if we're at the end of the data,
-                                    // it's just the extra null terminator.
-                                    if (nextNull != len - 1)
+                                    if (chars.Length == 0)
                                     {
-                                        toAdd = string.Empty;
+                                        return string.Empty;
                                     }
+
+                                    // Remove null termination if it exists.
+                                    if (chars[^1] == 0)
+                                    {
+                                        chars = chars[0..^1];
+                                    }
+
+                                    // Get the resulting string from the bytes.
+                                    string str = chars.ToString();
+                                    if (type == Interop.Advapi32.RegistryValues.REG_EXPAND_SZ)
+                                    {
+                                        str = Environment.ExpandEnvironmentVariables(str);
+                                    }
+
+                                    return str;
                                 }
                             }
-                            else
-                            {
-                                toAdd = new string(blob, cur, len - cur);
-                            }
-                            cur = nextNull + 1;
 
-                            if (toAdd != null)
-                            {
-                                if (strings.Length == stringsCount)
-                                {
-                                    Array.Resize(ref strings, stringsCount > 0 ? stringsCount * 2 : 4);
-                                }
-                                strings[stringsCount++] = toAdd;
-                            }
-                        }
-
-                        Array.Resize(ref strings, stringsCount);
-                        data = strings;
+                        default:
+                            return defaultValue;
                     }
-                    break;
-                case Interop.Advapi32.RegistryValues.REG_LINK:
-                default:
-                    break;
+                }
+            }
+            finally
+            {
+                if (pooledArray is not null)
+                {
+                    ArrayPool<byte>.Shared.Return(pooledArray);
+                }
             }
-
-            return data;
         }
 
         // The actual api is SetValue(string name, object value) but we only need to set Strings
         // so this is a cut-down version that supports on that.
         internal void SetValue(string name, string value)
         {
-            if (value == null)
-                throw new ArgumentNullException(nameof(value));
+            ArgumentNullException.ThrowIfNull(value);
 
             if (name != null && name.Length > MaxValueLength)
                 throw new ArgumentException(SR.Arg_RegValStrLenBug, nameof(name));
