@@ -11,6 +11,29 @@ namespace Microsoft.Win32.SafeHandles
 {
     public sealed partial class SafeFileHandle : SafeHandleZeroOrMinusOneIsInvalid
     {
+        private const UnixFileMode PermissionMask =
+            UnixFileMode.UserRead |
+            UnixFileMode.UserWrite |
+            UnixFileMode.UserExecute |
+            UnixFileMode.GroupRead |
+            UnixFileMode.GroupWrite |
+            UnixFileMode.GroupExecute |
+            UnixFileMode.OtherRead |
+            UnixFileMode.OtherWrite |
+            UnixFileMode.OtherExecute;
+
+        // If the file gets created a new, we'll select the permissions for it.  Most Unix utilities by default use 666 (read and
+        // write for all), so we do the same (even though this doesn't match Windows, where by default it's possible to write out
+        // a file and then execute it). No matter what we choose, it'll be subject to the umask applied by the system, such that the
+        // actual permissions will typically be less than what we select here.
+        internal const UnixFileMode DefaultCreateMode =
+            UnixFileMode.UserRead |
+            UnixFileMode.UserWrite |
+            UnixFileMode.GroupRead |
+            UnixFileMode.GroupWrite |
+            UnixFileMode.OtherRead |
+            UnixFileMode.OtherWrite;
+
         internal static bool DisableFileLocking { get; } = OperatingSystem.IsBrowser() // #40065: Emscripten does not support file locking
             || AppContextConfigHelper.GetBooleanConfig("System.IO.DisableFileLocking", "DOTNET_SYSTEM_IO_DISABLEFILELOCKING", defaultValue: false);
 
@@ -53,17 +76,22 @@ namespace Microsoft.Win32.SafeHandles
             }
         }
 
+#pragma warning disable CA1822
         internal ThreadPoolBoundHandle? ThreadPoolBinding => null;
 
         internal void EnsureThreadPoolBindingInitialized() { /* nop */ }
 
-        /// <summary>Opens the specified file with the requested flags and mode.</summary>
-        /// <param name="path">The path to the file.</param>
-        /// <param name="flags">The flags with which to open the file.</param>
-        /// <param name="mode">The mode for opening the file.</param>
-        /// <returns>A SafeFileHandle for the opened file.</returns>
-        private static SafeFileHandle Open(string path, Interop.Sys.OpenFlags flags, int mode)
+        internal bool TryGetCachedLength(out long cachedLength)
         {
+            cachedLength = -1;
+            return false;
+        }
+#pragma warning restore CA1822
+
+        private static SafeFileHandle Open(string path, Interop.Sys.OpenFlags flags, int mode, bool failForSymlink, out bool wasSymlink,
+                                           Func<Interop.ErrorInfo, Interop.Sys.OpenFlags, string, Exception?>? createOpenException)
+        {
+            wasSymlink = false;
             Debug.Assert(path != null);
             SafeFileHandle handle = Interop.Sys.Open(path, flags, mode);
             handle._path = path;
@@ -73,68 +101,28 @@ namespace Microsoft.Win32.SafeHandles
                 Interop.ErrorInfo error = Interop.Sys.GetLastErrorInfo();
                 handle.Dispose();
 
-                // If we fail to open the file due to a path not existing, we need to know whether to blame
-                // the file itself or its directory.  If we're creating the file, then we blame the directory,
-                // otherwise we blame the file.
-                //
-                // When opening, we need to align with Windows, which considers a missing path to be
-                // FileNotFound only if the containing directory exists.
-
-                bool isDirectory = (error.Error == Interop.Error.ENOENT) &&
-                    ((flags & Interop.Sys.OpenFlags.O_CREAT) != 0
-                    || !DirectoryExists(System.IO.Path.GetDirectoryName(System.IO.Path.TrimEndingDirectorySeparator(path!))!));
-
-                Interop.CheckIo(
-                    error.Error,
-                    path,
-                    isDirectory,
-                    errorRewriter: e => (e.Error == Interop.Error.EISDIR) ? Interop.Error.EACCES.Info() : e);
-            }
-
-            // Make sure it's not a directory; we do this after opening it once we have a file descriptor
-            // to avoid race conditions.
-            //
-            // We can omit the check when write access is requested. open will have failed with EISDIR.
-            if ((flags & (Interop.Sys.OpenFlags.O_WRONLY | Interop.Sys.OpenFlags.O_RDWR)) == 0)
-            {
-                Interop.Sys.FileStatus status;
-                if (Interop.Sys.FStat(handle, out status) != 0)
+                if (failForSymlink && error.Error == Interop.Error.ELOOP)
                 {
-                    Interop.ErrorInfo error = Interop.Sys.GetLastErrorInfo();
-                    handle.Dispose();
-                    throw Interop.GetExceptionForIoErrno(error, path);
-                }
-                if ((status.Mode & Interop.Sys.FileTypes.S_IFMT) == Interop.Sys.FileTypes.S_IFDIR)
-                {
-                    handle.Dispose();
-                    throw Interop.GetExceptionForIoErrno(Interop.Error.EACCES.Info(), path, isDirectory: true);
+                    wasSymlink = true;
+                    return handle;
                 }
 
-                if ((status.Mode & Interop.Sys.FileTypes.S_IFMT) == Interop.Sys.FileTypes.S_IFREG)
+                if (createOpenException?.Invoke(error, flags, path) is Exception ex)
                 {
-                    // we take advantage of the information provided by the fstat syscall
-                    // and for regular files (most common case)
-                    // avoid one extra sys call for determining whether file can be seeked
-                    handle._canSeek = NullableBool.True;
-                    Debug.Assert(Interop.Sys.LSeek(handle, 0, Interop.Sys.SeekWhence.SEEK_CUR) >= 0);
+                    throw ex;
                 }
+
+                if (error.Error == Interop.Error.EISDIR)
+                {
+                    error = Interop.Error.EACCES.Info();
+                }
+
+                Interop.CheckIo(error.Error, path);
             }
 
             return handle;
         }
 
-        private static bool DirectoryExists(string fullPath)
-        {
-            Interop.Sys.FileStatus fileinfo;
-
-            if (Interop.Sys.Stat(fullPath, out fileinfo) < 0)
-            {
-                return false;
-            }
-
-            return ((fileinfo.Mode & Interop.Sys.FileTypes.S_IFMT) == Interop.Sys.FileTypes.S_IFDIR);
-        }
-
         // Each thread will have its own copy. This prevents race conditions if the handle had the last error.
         [ThreadStatic]
         internal static Interop.ErrorInfo? t_lastCloseErrorInfo;
@@ -143,6 +131,7 @@ namespace Microsoft.Win32.SafeHandles
         {
             // If DeleteOnClose was requested when constructed, delete the file now.
             // (Unix doesn't directly support DeleteOnClose, so we mimic it here.)
+            // We delete the file before releasing the lock to detect the removal in Init.
             if (_deleteOnClose)
             {
                 // Since we still have the file open, this will end up deleting
@@ -183,30 +172,63 @@ namespace Microsoft.Win32.SafeHandles
             }
         }
 
-        internal static SafeFileHandle Open(string fullPath, FileMode mode, FileAccess access, FileShare share, FileOptions options, long preallocationSize)
+        // Specialized Open that returns the file length and permissions of the opened file.
+        // This information is retrieved from the 'stat' syscall that must be performed to ensure the path is not a directory.
+        internal static SafeFileHandle OpenReadOnly(string fullPath, FileOptions options, out long fileLength, out UnixFileMode filePermissions)
+        {
+            SafeFileHandle handle = Open(fullPath, FileMode.Open, FileAccess.Read, FileShare.Read, options, preallocationSize: 0, DefaultCreateMode, out fileLength, out filePermissions, false, out _, null);
+            Debug.Assert(fileLength >= 0);
+            return handle;
+        }
+
+        internal static SafeFileHandle Open(string fullPath, FileMode mode, FileAccess access, FileShare share, FileOptions options, long preallocationSize, UnixFileMode? unixCreateMode = null,
+                                            Func<Interop.ErrorInfo, Interop.Sys.OpenFlags, string, Exception?>? createOpenException = null)
+        {
+            return Open(fullPath, mode, access, share, options, preallocationSize, unixCreateMode ?? DefaultCreateMode, out _, out _, false, out _, createOpenException);
+        }
+
+        internal static SafeFileHandle? OpenNoFollowSymlink(string fullPath, FileMode mode, FileAccess access, FileShare share, FileOptions options, long preallocationSize, out bool wasSymlink, UnixFileMode? unixCreateMode = null,
+                                            Func<Interop.ErrorInfo, Interop.Sys.OpenFlags, string, Exception?>? createOpenException = null)
+        {
+            return Open(fullPath, mode, access, share, options, preallocationSize, unixCreateMode ?? DefaultCreateMode, out _, out _, true, out wasSymlink, createOpenException);
+        }
+
+        private static SafeFileHandle Open(string fullPath, FileMode mode, FileAccess access, FileShare share, FileOptions options, long preallocationSize, UnixFileMode openPermissions,
+                                            out long fileLength, out UnixFileMode filePermissions, bool failForSymlink, out bool wasSymlink,
+                                            Func<Interop.ErrorInfo, Interop.Sys.OpenFlags, string, Exception?>? createOpenException = null)
         {
             // Translate the arguments into arguments for an open call.
-            Interop.Sys.OpenFlags openFlags = PreOpenConfigurationFromOptions(mode, access, share, options);
+            Interop.Sys.OpenFlags openFlags = PreOpenConfigurationFromOptions(mode, access, share, options, failForSymlink);
 
-            // If the file gets created a new, we'll select the permissions for it.  Most Unix utilities by default use 666 (read and
-            // write for all), so we do the same (even though this doesn't match Windows, where by default it's possible to write out
-            // a file and then execute it). No matter what we choose, it'll be subject to the umask applied by the system, such that the
-            // actual permissions will typically be less than what we select here.
-            const Interop.Sys.Permissions OpenPermissions =
-                Interop.Sys.Permissions.S_IRUSR | Interop.Sys.Permissions.S_IWUSR |
-                Interop.Sys.Permissions.S_IRGRP | Interop.Sys.Permissions.S_IWGRP |
-                Interop.Sys.Permissions.S_IROTH | Interop.Sys.Permissions.S_IWOTH;
-
-            SafeFileHandle safeFileHandle = Open(fullPath, openFlags, (int)OpenPermissions);
+            SafeFileHandle? safeFileHandle = null;
             try
             {
-                safeFileHandle.Init(fullPath, mode, access, share, options, preallocationSize);
+                while (true)
+                {
+                    safeFileHandle = Open(fullPath, openFlags, (int)openPermissions, failForSymlink, out wasSymlink, createOpenException);
 
-                return safeFileHandle;
+                    if (failForSymlink && wasSymlink)
+                    {
+                        fileLength = default;
+                        filePermissions = default;
+                        return safeFileHandle;
+                    }
+
+                    // When Init return false, the path has changed to another file entry, and
+                    // we need to re-open the path to reflect that.
+                    if (safeFileHandle.Init(fullPath, mode, access, share, options, preallocationSize, out fileLength, out filePermissions))
+                    {
+                        return safeFileHandle;
+                    }
+                    else
+                    {
+                        safeFileHandle.Dispose();
+                    }
+                }
             }
             catch (Exception)
             {
-                safeFileHandle.Dispose();
+                safeFileHandle?.Dispose();
 
                 throw;
             }
@@ -217,11 +239,16 @@ namespace Microsoft.Win32.SafeHandles
         /// <param name="access">The FileAccess provided to the stream's constructor</param>
         /// <param name="share">The FileShare provided to the stream's constructor</param>
         /// <param name="options">The FileOptions provided to the stream's constructor</param>
+        /// <param name="failForSymlink">Whether to cause ELOOP error when opening a symlink</param>
         /// <returns>The flags value to be passed to the open system call.</returns>
-        private static Interop.Sys.OpenFlags PreOpenConfigurationFromOptions(FileMode mode, FileAccess access, FileShare share, FileOptions options)
+        private static Interop.Sys.OpenFlags PreOpenConfigurationFromOptions(FileMode mode, FileAccess access, FileShare share, FileOptions options, bool failForSymlink)
         {
             // Translate FileMode.  Most of the values map cleanly to one or more options for open.
             Interop.Sys.OpenFlags flags = default;
+            if (failForSymlink)
+            {
+                flags |= Interop.Sys.OpenFlags.O_NOFOLLOW;
+            }
             switch (mode)
             {
                 default:
@@ -291,8 +318,39 @@ namespace Microsoft.Win32.SafeHandles
             return flags;
         }
 
-        private void Init(string path, FileMode mode, FileAccess access, FileShare share, FileOptions options, long preallocationSize)
+        private bool Init(string path, FileMode mode, FileAccess access, FileShare share, FileOptions options, long preallocationSize,
+                          out long fileLength, out UnixFileMode filePermissions)
         {
+            Interop.Sys.FileStatus status = default;
+            bool statusHasValue = false;
+            fileLength = -1;
+            filePermissions = 0;
+
+            // Make sure our handle is not a directory.
+            // We can omit the check when write access is requested. open will have failed with EISDIR.
+            if ((access & FileAccess.Write) == 0)
+            {
+                // Stat the file descriptor to avoid race conditions.
+                FStatCheckIO(path, ref status, ref statusHasValue);
+
+                if ((status.Mode & Interop.Sys.FileTypes.S_IFMT) == Interop.Sys.FileTypes.S_IFDIR)
+                {
+                    throw Interop.GetExceptionForIoErrno(Interop.Error.EACCES.Info(), path);
+                }
+
+                if ((status.Mode & Interop.Sys.FileTypes.S_IFMT) == Interop.Sys.FileTypes.S_IFREG)
+                {
+                    // we take advantage of the information provided by the fstat syscall
+                    // and for regular files (most common case)
+                    // avoid one extra sys call for determining whether file can be seeked
+                    _canSeek = NullableBool.True;
+                    Debug.Assert(Interop.Sys.LSeek(this, 0, Interop.Sys.SeekWhence.SEEK_CUR) >= 0);
+                }
+
+                fileLength = status.Size;
+                filePermissions = ((UnixFileMode)status.Mode) & PermissionMask;
+            }
+
             IsAsync = (options & FileOptions.Asynchronous) != 0;
 
             // Lock the file if requested via FileShare.  This is only advisory locking. FileShare.None implies an exclusive
@@ -308,11 +366,45 @@ namespace Microsoft.Win32.SafeHandles
                 Interop.ErrorInfo errorInfo = Interop.Sys.GetLastErrorInfo();
                 if (errorInfo.Error == Interop.Error.EWOULDBLOCK)
                 {
-                    throw Interop.GetExceptionForIoErrno(errorInfo, path, isDirectory: false);
+                    throw Interop.GetExceptionForIoErrno(errorInfo, path);
                 }
             }
 
-            // Enable DeleteOnClose when we've succesfully locked the file.
+            // On Windows, DeleteOnClose happens when all kernel handles to the file are closed.
+            // Unix kernels don't have this feature, and .NET deletes the file when the Handle gets disposed.
+            // When the file is opened with an exclusive lock, we can use it to check the file at the path
+            // still matches the file we've opened.
+            // When the delete is performed by another .NET Handle, it holds the lock during the delete.
+            // Since we've just obtained the lock, the file will already be removed/replaced.
+            // We limit performing this check to cases where our file was opened with DeleteOnClose with
+            // a mode of OpenOrCreate.
+            if (_isLocked && ((options & FileOptions.DeleteOnClose) != 0) &&
+                share == FileShare.None && mode == FileMode.OpenOrCreate)
+            {
+                FStatCheckIO(path, ref status, ref statusHasValue);
+
+                Interop.Sys.FileStatus pathStatus;
+                if (Interop.Sys.Stat(path, out pathStatus) < 0)
+                {
+                    // If the file was removed, re-open.
+                    // Otherwise throw the error 'stat' gave us (assuming this is the
+                    // error 'open' will give us if we'd call it now).
+                    Interop.ErrorInfo error = Interop.Sys.GetLastErrorInfo();
+
+                    if (error.Error == Interop.Error.ENOENT)
+                    {
+                        return false;
+                    }
+
+                    throw Interop.GetExceptionForIoErrno(error, path);
+                }
+                if (pathStatus.Ino != status.Ino || pathStatus.Dev != status.Dev)
+                {
+                    // The file was replaced, re-open
+                    return false;
+                }
+            }
+            // Enable DeleteOnClose when we've successfully locked the file.
             // On Windows, the locking happens atomically as part of opening the file.
             _deleteOnClose = (options & FileOptions.DeleteOnClose) != 0;
 
@@ -341,7 +433,7 @@ namespace Microsoft.Win32.SafeHandles
                         // We know the file descriptor is valid and we know the size argument to FTruncate is correct,
                         // so if EBADF or EINVAL is returned, it means we're dealing with a special file that can't be
                         // truncated.  Ignore the error in such cases; in all others, throw.
-                        throw Interop.GetExceptionForIoErrno(errorInfo, path, isDirectory: false);
+                        throw Interop.GetExceptionForIoErrno(errorInfo, path);
                     }
                 }
             }
@@ -366,6 +458,8 @@ namespace Microsoft.Win32.SafeHandles
                                             path, preallocationSize));
                 }
             }
+
+            return true;
         }
 
         private bool CanLockTheFile(Interop.Sys.LockOperations lockOperation, FileAccess access)
@@ -402,6 +496,20 @@ namespace Microsoft.Win32.SafeHandles
             }
         }
 
+        private void FStatCheckIO(string path, ref Interop.Sys.FileStatus status, ref bool statusHasValue)
+        {
+            if (!statusHasValue)
+            {
+                if (Interop.Sys.FStat(this, out status) != 0)
+                {
+                    Interop.ErrorInfo error = Interop.Sys.GetLastErrorInfo();
+                    throw Interop.GetExceptionForIoErrno(error, path);
+                }
+
+                statusHasValue = true;
+            }
+        }
+
         private bool GetCanSeek()
         {
             Debug.Assert(!IsClosed);
@@ -416,6 +524,13 @@ namespace Microsoft.Win32.SafeHandles
             return canSeek == NullableBool.True;
         }
 
+        internal long GetFileLength()
+        {
+            int result = Interop.Sys.FStat(this, out Interop.Sys.FileStatus status);
+            FileStreamHelpers.CheckFileCall(result, Path);
+            return status.Size;
+        }
+
         private enum NullableBool
         {
             Undefined = 0,
