@@ -80,13 +80,10 @@ namespace System
 
         public string ApplyPolicy(string assemblyName)
         {
-            if (assemblyName == null)
+            ArgumentException.ThrowIfNullOrEmpty(assemblyName);
+            if (assemblyName[0] == '\0')
             {
-                throw new ArgumentNullException(nameof(assemblyName));
-            }
-            if (assemblyName.Length == 0 || assemblyName[0] == '\0')
-            {
-                throw new ArgumentException(SR.Argument_StringZeroLength, nameof(assemblyName));
+                throw new ArgumentException(SR.Argument_EmptyString, nameof(assemblyName));
             }
 
             return assemblyName;
@@ -95,7 +92,8 @@ namespace System
         [Obsolete(Obsoletions.AppDomainCreateUnloadMessage, DiagnosticId = Obsoletions.AppDomainCreateUnloadDiagId, UrlFormat = Obsoletions.SharedUrlFormat)]
         public static AppDomain CreateDomain(string friendlyName)
         {
-            if (friendlyName == null) throw new ArgumentNullException(nameof(friendlyName));
+            ArgumentNullException.ThrowIfNull(friendlyName);
+
             throw new PlatformNotSupportedException(SR.PlatformNotSupported_AppDomains);
         }
 
@@ -105,10 +103,8 @@ namespace System
         [RequiresUnreferencedCode("Types and members the loaded assembly depends on might be removed")]
         public int ExecuteAssembly(string assemblyFile, string?[]? args)
         {
-            if (assemblyFile == null)
-            {
-                throw new ArgumentNullException(nameof(assemblyFile));
-            }
+            ArgumentNullException.ThrowIfNull(assemblyFile);
+
             string fullPath = Path.GetFullPath(assemblyFile);
             Assembly assembly = Assembly.LoadFile(fullPath);
             return ExecuteAssembly(assembly, args);
@@ -167,10 +163,8 @@ namespace System
         [Obsolete(Obsoletions.AppDomainCreateUnloadMessage, DiagnosticId = Obsoletions.AppDomainCreateUnloadDiagId, UrlFormat = Obsoletions.SharedUrlFormat)]
         public static void Unload(AppDomain domain)
         {
-            if (domain == null)
-            {
-                throw new ArgumentNullException(nameof(domain));
-            }
+            ArgumentNullException.ThrowIfNull(domain);
+
             throw new CannotUnloadAppDomainException(SR.Arg_PlatformNotSupported);
         }
 
@@ -269,10 +263,7 @@ namespace System
 
         public void SetThreadPrincipal(IPrincipal principal)
         {
-            if (principal == null)
-            {
-                throw new ArgumentNullException(nameof(principal));
-            }
+            ArgumentNullException.ThrowIfNull(principal);
 
             // Set the principal while checking it has not been set previously.
             if (Interlocked.CompareExchange(ref _defaultPrincipal, principal, null) is not null)
@@ -284,21 +275,15 @@ namespace System
         [RequiresUnreferencedCode("Type and its constructor could be removed")]
         public ObjectHandle? CreateInstance(string assemblyName, string typeName)
         {
-            if (assemblyName == null)
-            {
-                throw new ArgumentNullException(nameof(assemblyName));
-            }
+            ArgumentNullException.ThrowIfNull(assemblyName);
 
             return Activator.CreateInstance(assemblyName, typeName);
         }
 
         [RequiresUnreferencedCode("Type and its constructor could be removed")]
-        public ObjectHandle? CreateInstance(string assemblyName, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, System.Globalization.CultureInfo? culture, object?[]? activationAttributes)
+        public ObjectHandle? CreateInstance(string assemblyName, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, Globalization.CultureInfo? culture, object?[]? activationAttributes)
         {
-            if (assemblyName == null)
-            {
-                throw new ArgumentNullException(nameof(assemblyName));
-            }
+            ArgumentNullException.ThrowIfNull(assemblyName);
 
             return Activator.CreateInstance(assemblyName,
                                             typeName,
@@ -313,10 +298,7 @@ namespace System
         [RequiresUnreferencedCode("Type and its constructor could be removed")]
         public ObjectHandle? CreateInstance(string assemblyName, string typeName, object?[]? activationAttributes)
         {
-            if (assemblyName == null)
-            {
-                throw new ArgumentNullException(nameof(assemblyName));
-            }
+            ArgumentNullException.ThrowIfNull(assemblyName);
 
             return Activator.CreateInstance(assemblyName, typeName, activationAttributes);
         }
@@ -329,7 +311,7 @@ namespace System
         }
 
         [RequiresUnreferencedCode("Type and its constructor could be removed")]
-        public object? CreateInstanceAndUnwrap(string assemblyName, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, System.Globalization.CultureInfo? culture, object?[]? activationAttributes)
+        public object? CreateInstanceAndUnwrap(string assemblyName, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, Globalization.CultureInfo? culture, object?[]? activationAttributes)
         {
             ObjectHandle? oh = CreateInstance(assemblyName,
                                              typeName,
@@ -356,7 +338,7 @@ namespace System
         }
 
         [RequiresUnreferencedCode("Type and its constructor could be removed")]
-        public ObjectHandle? CreateInstanceFrom(string assemblyFile, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, System.Globalization.CultureInfo? culture, object?[]? activationAttributes)
+        public ObjectHandle? CreateInstanceFrom(string assemblyFile, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, Globalization.CultureInfo? culture, object?[]? activationAttributes)
         {
             return Activator.CreateInstanceFrom(assemblyFile,
                                                 typeName,
@@ -382,7 +364,7 @@ namespace System
         }
 
         [RequiresUnreferencedCode("Type and its constructor could be removed")]
-        public object? CreateInstanceFromAndUnwrap(string assemblyFile, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, System.Globalization.CultureInfo? culture, object?[]? activationAttributes)
+        public object? CreateInstanceFromAndUnwrap(string assemblyFile, string typeName, bool ignoreCase, BindingFlags bindingAttr, Binder? binder, object?[]? args, Globalization.CultureInfo? culture, object?[]? activationAttributes)
         {
             ObjectHandle? oh = CreateInstanceFrom(assemblyFile,
                                                  typeName,
