@@ -5,7 +5,6 @@ using System.Buffers;
 using System.Diagnostics;
 using System.Globalization;
 using System.Runtime.InteropServices;
-using Internal.Runtime.CompilerServices;
 
 namespace System.Runtime.CompilerServices
 {
@@ -134,65 +133,6 @@ namespace System.Runtime.CompilerServices
         /// <param name="value">The string to write.</param>
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
         public void AppendLiteral(string value)
-        {
-            // AppendLiteral is expected to always be called by compiler-generated code with a literal string.
-            // By inlining it, the method body is exposed to the constant length of that literal, allowing the JIT to
-            // prune away the irrelevant cases.  This effectively enables multiple implementations of AppendLiteral,
-            // special-cased on and optimized for the literal's length.  We special-case lengths 1 and 2 because
-            // they're very common, e.g.
-            //     1: ' ', '.', '-', '\t', etc.
-            //     2: ", ", "0x", "=>", ": ", etc.
-            // but we refrain from adding more because, in the rare case where AppendLiteral is called with a non-literal,
-            // there is a lot of code here to be inlined.
-
-            // TODO: https://github.com/dotnet/runtime/issues/41692#issuecomment-685192193
-            // What we really want here is to be able to add a bunch of additional special-cases based on length,
-            // e.g. a switch with a case for each length <= 8, not mark the method as AggressiveInlining, and have
-            // it inlined when provided with a string literal such that all the other cases evaporate but not inlined
-            // if called directly with something that doesn't enable pruning.  Even better, if "literal".TryCopyTo
-            // could be unrolled based on the literal, ala https://github.com/dotnet/runtime/pull/46392, we might
-            // be able to remove all special-casing here.
-
-            if (value.Length == 1)
-            {
-                Span<char> chars = _chars;
-                int pos = _pos;
-                if ((uint)pos < (uint)chars.Length)
-                {
-                    chars[pos] = value[0];
-                    _pos = pos + 1;
-                }
-                else
-                {
-                    GrowThenCopyString(value);
-                }
-                return;
-            }
-
-            if (value.Length == 2)
-            {
-                Span<char> chars = _chars;
-                int pos = _pos;
-                if ((uint)pos < chars.Length - 1)
-                {
-                    Unsafe.WriteUnaligned(
-                        ref Unsafe.As<char, byte>(ref Unsafe.Add(ref MemoryMarshal.GetReference(chars), pos)),
-                        Unsafe.ReadUnaligned<int>(ref Unsafe.As<char, byte>(ref value.GetRawStringData())));
-                    _pos = pos + 2;
-                }
-                else
-                {
-                    GrowThenCopyString(value);
-                }
-                return;
-            }
-
-            AppendStringDirect(value);
-        }
-
-        /// <summary>Writes the specified string to the handler.</summary>
-        /// <param name="value">The string to write.</param>
-        private void AppendStringDirect(string value)
         {
             if (value.TryCopyTo(_chars.Slice(_pos)))
             {
@@ -282,6 +222,7 @@ namespace System.Runtime.CompilerServices
         #region AppendFormatted T
         /// <summary>Writes the specified value to the handler.</summary>
         /// <param name="value">The value to write.</param>
+        /// <typeparam name="T">The type of the value to write.</typeparam>
         public void AppendFormatted<T>(T value)
         {
             // This method could delegate to AppendFormatted with a null format, but explicitly passing
@@ -306,6 +247,19 @@ namespace System.Runtime.CompilerServices
             if (value is IFormattable)
             {
                 // If the value can format itself directly into our buffer, do so.
+
+                if (typeof(T).IsEnum)
+                {
+                    int charsWritten;
+                    while (!Enum.TryFormatUnconstrained(value, _chars.Slice(_pos), out charsWritten))
+                    {
+                        Grow();
+                    }
+
+                    _pos += charsWritten;
+                    return;
+                }
+
                 if (value is ISpanFormattable)
                 {
                     int charsWritten;
@@ -327,12 +281,14 @@ namespace System.Runtime.CompilerServices
 
             if (s is not null)
             {
-                AppendStringDirect(s);
+                AppendLiteral(s);
             }
         }
+
         /// <summary>Writes the specified value to the handler.</summary>
         /// <param name="value">The value to write.</param>
         /// <param name="format">The format string.</param>
+        /// <typeparam name="T">The type of the value to write.</typeparam>
         public void AppendFormatted<T>(T value, string? format)
         {
             // If there's a custom formatter, always use it.
@@ -353,6 +309,19 @@ namespace System.Runtime.CompilerServices
             if (value is IFormattable)
             {
                 // If the value can format itself directly into our buffer, do so.
+
+                if (typeof(T).IsEnum)
+                {
+                    int charsWritten;
+                    while (!Enum.TryFormatUnconstrained(value, _chars.Slice(_pos), out charsWritten, format))
+                    {
+                        Grow();
+                    }
+
+                    _pos += charsWritten;
+                    return;
+                }
+
                 if (value is ISpanFormattable)
                 {
                     int charsWritten;
@@ -374,13 +343,14 @@ namespace System.Runtime.CompilerServices
 
             if (s is not null)
             {
-                AppendStringDirect(s);
+                AppendLiteral(s);
             }
         }
 
         /// <summary>Writes the specified value to the handler.</summary>
         /// <param name="value">The value to write.</param>
         /// <param name="alignment">Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.</param>
+        /// <typeparam name="T">The type of the value to write.</typeparam>
         public void AppendFormatted<T>(T value, int alignment)
         {
             int startingPos = _pos;
@@ -395,6 +365,7 @@ namespace System.Runtime.CompilerServices
         /// <param name="value">The value to write.</param>
         /// <param name="format">The format string.</param>
         /// <param name="alignment">Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.</param>
+        /// <typeparam name="T">The type of the value to write.</typeparam>
         public void AppendFormatted<T>(T value, int alignment, string? format)
         {
             int startingPos = _pos;
@@ -409,7 +380,7 @@ namespace System.Runtime.CompilerServices
         #region AppendFormatted ReadOnlySpan<char>
         /// <summary>Writes the specified character span to the handler.</summary>
         /// <param name="value">The span to write.</param>
-        public void AppendFormatted(ReadOnlySpan<char> value)
+        public void AppendFormatted(scoped ReadOnlySpan<char> value)
         {
             // Fast path for when the value fits in the current buffer
             if (value.TryCopyTo(_chars.Slice(_pos)))
@@ -426,7 +397,7 @@ namespace System.Runtime.CompilerServices
         /// <param name="value">The span to write.</param>
         /// <param name="alignment">Minimum number of characters that should be written for this value.  If the value is negative, it indicates left-aligned and the required minimum is the absolute value.</param>
         /// <param name="format">The format string.</param>
-        public void AppendFormatted(ReadOnlySpan<char> value, int alignment = 0, string? format = null)
+        public void AppendFormatted(scoped ReadOnlySpan<char> value, int alignment = 0, string? format = null)
         {
             bool leftAlign = false;
             if (alignment < 0)
@@ -540,6 +511,7 @@ namespace System.Runtime.CompilerServices
         /// <summary>Formats the value using the custom formatter from the provider.</summary>
         /// <param name="value">The value to write.</param>
         /// <param name="format">The format string.</param>
+        /// <typeparam name="T">The type of the value to write.</typeparam>
         [MethodImpl(MethodImplOptions.NoInlining)]
         private void AppendCustomFormatter<T>(T value, string? format)
         {
@@ -555,7 +527,7 @@ namespace System.Runtime.CompilerServices
 
             if (formatter is not null && formatter.Format(format, value, _provider) is string customFormatted)
             {
-                AppendStringDirect(customFormatted);
+                AppendLiteral(customFormatted);
             }
         }
 
@@ -605,7 +577,7 @@ namespace System.Runtime.CompilerServices
             }
         }
 
-        /// <summary>Fallback for fast path in <see cref="AppendStringDirect"/> when there's not enough space in the destination.</summary>
+        /// <summary>Fallback for fast path in <see cref="AppendLiteral(string)"/> when there's not enough space in the destination.</summary>
         /// <param name="value">The string to write.</param>
         [MethodImpl(MethodImplOptions.NoInlining)]
         private void GrowThenCopyString(string value)
@@ -618,7 +590,7 @@ namespace System.Runtime.CompilerServices
         /// <summary>Fallback for <see cref="AppendFormatted(ReadOnlySpan{char})"/> for when not enough space exists in the current buffer.</summary>
         /// <param name="value">The span to write.</param>
         [MethodImpl(MethodImplOptions.NoInlining)]
-        private void GrowThenCopySpan(ReadOnlySpan<char> value)
+        private void GrowThenCopySpan(scoped ReadOnlySpan<char> value)
         {
             Grow(value.Length);
             value.CopyTo(_chars.Slice(_pos));
