@@ -22,48 +22,42 @@ namespace System.Runtime.Intrinsics
     // This ensures we get good codegen for the "fast-path" and allows the JIT to
     // determine inline profitability of the other paths as it would normally.
 
-    /// <summary>Represents a 64-bit vector of a specified numeric type that is suitable for low-level optimization of parallel algorithms.</summary>
+
+    /// <summary>Represents a 128-bit vector of a specified numeric type that is suitable for low-level optimization of parallel algorithms.</summary>
     /// <typeparam name="T">The type of the elements in the vector.</typeparam>
     [Intrinsic]
     [DebuggerDisplay("{DisplayString,nq}")]
-    [DebuggerTypeProxy(typeof(Vector64DebugView<>))]
-    [StructLayout(LayoutKind.Sequential, Size = Vector64.Size)]
-    public readonly struct Vector64<T> : IEquatable<Vector64<T>>
+    [DebuggerTypeProxy(typeof(Vector128DebugView<>))]
+    [StructLayout(LayoutKind.Sequential, Size = Vector128.Size)]
+    public readonly struct Vector128<T> : IEquatable<Vector128<T>>
     {
-        // This field allows the debug view to work https://github.com/dotnet/runtime/issues/9495)
-        internal readonly ulong _00;
+        internal readonly Vector64<T> _lower;
+        internal readonly Vector64<T> _upper;
 
-        /// <summary>Gets a new <see cref="Vector64{T}" /> with all bits set to 1.</summary>
+        /// <summary>Gets a new <see cref="Vector128{T}" /> with all bits set to 1.</summary>
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
-        public static Vector64<T> AllBitsSet
+        public static Vector128<T> AllBitsSet
         {
             [Intrinsic]
             [MethodImpl(MethodImplOptions.AggressiveInlining)]
             get
             {
-                ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
-
-                Unsafe.SkipInit(out Vector64<T> result);
-                Unsafe.AsRef(in result._00) = ulong.MaxValue;
-
-                return result;
+                Vector64<T> vector = Vector64<T>.AllBitsSet;
+                return Vector128.Create(vector, vector);
             }
         }
 
-#pragma warning disable CS8500 // This takes the address of, gets the size of, or declares a pointer to a managed type ('T')
-        /// <summary>Gets the number of <typeparamref name="T" /> that are in a <see cref="Vector64{T}" />.</summary>
+        /// <summary>Gets the number of <typeparamref name="T" /> that are in a <see cref="Vector128{T}" />.</summary>
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
-        public static unsafe int Count
+        public static int Count
         {
             [Intrinsic]
             [MethodImpl(MethodImplOptions.AggressiveInlining)]
             get
             {
-                ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
-                return Vector64.Size / sizeof(T);
+                return Vector64<T>.Count * 2;
             }
         }
-#pragma warning restore CS8500 // This takes the address of, gets the size of, or declares a pointer to a managed type ('T')
 
         /// <summary>Gets <c>true</c> if <typeparamref name="T" /> is supported; otherwise, <c>false</c>.</summary>
         /// <returns><c>true</c> if <typeparamref name="T" /> is supported; otherwise, <c>false</c>.</returns>
@@ -88,28 +82,28 @@ namespace System.Runtime.Intrinsics
             }
         }
 
-        /// <summary>Gets a new <see cref="Vector64{T}" /> with all elements initialized to one.</summary>
+        /// <summary>Gets a new <see cref="Vector128{T}" /> with all elements initialized to one.</summary>
         /// <exception cref="NotSupportedException">The type of the current instance (<typeparamref name="T" />) is not supported.</exception>
-        public static Vector64<T> One
+        public static Vector128<T> One
         {
             [Intrinsic]
             [MethodImpl(MethodImplOptions.AggressiveInlining)]
             get
             {
-                T scalar = Scalar<T>.One;
-                return Vector64.Create(scalar);
+                Vector64<T> vector = Vector64<T>.One;
+                return Vector128.Create(vector, vector);
             }
         }
 
-        /// <summary>Gets a new <see cref="Vector64{T}" /> with all elements initialized to zero.</summary>
+        /// <summary>Gets a new <see cref="Vector128{T}" /> with all elements initialized to zero.</summary>
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
-        public static Vector64<T> Zero
+        public static Vector128<T> Zero
         {
             [Intrinsic]
             [MethodImpl(MethodImplOptions.AggressiveInlining)]
             get
             {
-                ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
+                ThrowHelper.ThrowForUnsupportedIntrinsicsVector128BaseType<T>();
                 return default;
             }
         }
@@ -143,17 +137,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator +(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator +(Vector128<T> left, Vector128<T> right)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T value = Scalar<T>.Add(left.GetElementUnsafe(index), right.GetElementUnsafe(index));
-                result.SetElementUnsafe(index, value);
-            }
-
-            return result;
+            return Vector128.Create(
+                left._lower + right._lower,
+                left._upper + right._upper
+            );
         }
 
         /// <summary>Computes the bitwise-and of two vectors.</summary>
@@ -163,14 +152,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator &(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator &(Vector128<T> left, Vector128<T> right)
         {
-            ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
-
-            Unsafe.SkipInit(out Vector64<T> result);
-            Unsafe.AsRef(in result._00) = left._00 & right._00;
-
-            return result;
+            return Vector128.Create(
+                left._lower & right._lower,
+                left._upper & right._upper
+            );
         }
 
         /// <summary>Computes the bitwise-or of two vectors.</summary>
@@ -180,14 +167,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator |(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator |(Vector128<T> left, Vector128<T> right)
         {
-            ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
-
-            Unsafe.SkipInit(out Vector64<T> result);
-            Unsafe.AsRef(in result._00) = left._00 | right._00;
-
-            return result;
+            return Vector128.Create(
+                left._lower | right._lower,
+                left._upper | right._upper
+            );
         }
 
         /// <summary>Divides two vectors to compute their quotient.</summary>
@@ -197,17 +182,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator /(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator /(Vector128<T> left, Vector128<T> right)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T value = Scalar<T>.Divide(left.GetElementUnsafe(index), right.GetElementUnsafe(index));
-                result.SetElementUnsafe(index, value);
-            }
-
-            return result;
+            return Vector128.Create(
+                left._lower / right._lower,
+                left._upper / right._upper
+            );
         }
 
         /// <summary>Divides a vector by a scalar to compute the per-element quotient.</summary>
@@ -216,17 +196,12 @@ namespace System.Runtime.Intrinsics
         /// <returns>The quotient of <paramref name="left" /> divided by <paramref name="right" />.</returns>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator /(Vector64<T> left, T right)
+        public static Vector128<T> operator /(Vector128<T> left, T right)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T value = Scalar<T>.Divide(left.GetElementUnsafe(index), right);
-                result.SetElementUnsafe(index, value);
-            }
-
-            return result;
+            return Vector128.Create(
+                left._lower / right,
+                left._upper / right
+            );
         }
 
         /// <summary>Compares two vectors to determine if all elements are equal.</summary>
@@ -236,16 +211,10 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static bool operator ==(Vector64<T> left, Vector64<T> right)
+        public static bool operator ==(Vector128<T> left, Vector128<T> right)
         {
-            for (int index = 0; index < Count; index++)
-            {
-                if (!Scalar<T>.Equals(left.GetElementUnsafe(index), right.GetElementUnsafe(index)))
-                {
-                    return false;
-                }
-            }
-            return true;
+            return (left._lower == right._lower)
+                && (left._upper == right._upper);
         }
 
         /// <summary>Computes the exclusive-or of two vectors.</summary>
@@ -255,14 +224,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator ^(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator ^(Vector128<T> left, Vector128<T> right)
         {
-            ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
-
-            Unsafe.SkipInit(out Vector64<T> result);
-            Unsafe.AsRef(in result._00) = left._00 ^ right._00;
-
-            return result;
+            return Vector128.Create(
+                left._lower ^ right._lower,
+                left._upper ^ right._upper
+            );
         }
 
         /// <summary>Compares two vectors to determine if any elements are not equal.</summary>
@@ -272,16 +239,10 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static bool operator !=(Vector64<T> left, Vector64<T> right)
+        public static bool operator !=(Vector128<T> left, Vector128<T> right)
         {
-            for (int index = 0; index < Count; index++)
-            {
-                if (!Scalar<T>.Equals(left.GetElementUnsafe(index), right.GetElementUnsafe(index)))
-                {
-                    return true;
-                }
-            }
-            return false;
+            return (left._lower != right._lower)
+                || (left._upper != right._upper);
         }
 
         /// <summary>Shifts each element of a vector left by the specified amount.</summary>
@@ -290,17 +251,12 @@ namespace System.Runtime.Intrinsics
         /// <returns>A vector whose elements where shifted left by <paramref name="shiftCount" />.</returns>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator <<(Vector64<T> value, int shiftCount)
+        public static Vector128<T> operator <<(Vector128<T> value, int shiftCount)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T element = Scalar<T>.ShiftLeft(value.GetElementUnsafe(index), shiftCount);
-                result.SetElementUnsafe(index, element);
-            }
-
-            return result;
+            return Vector128.Create(
+                value._lower << shiftCount,
+                value._upper << shiftCount
+            );
         }
 
         /// <summary>Multiplies two vectors to compute their element-wise product.</summary>
@@ -310,17 +266,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator *(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator *(Vector128<T> left, Vector128<T> right)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T value = Scalar<T>.Multiply(left.GetElementUnsafe(index), right.GetElementUnsafe(index));
-                result.SetElementUnsafe(index, value);
-            }
-
-            return result;
+            return Vector128.Create(
+                left._lower * right._lower,
+                left._upper * right._upper
+            );
         }
 
         /// <summary>Multiplies a vector by a scalar to compute their product.</summary>
@@ -330,17 +281,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator *(Vector64<T> left, T right)
+        public static Vector128<T> operator *(Vector128<T> left, T right)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T value = Scalar<T>.Multiply(left.GetElementUnsafe(index), right);
-                result.SetElementUnsafe(index, value);
-            }
-
-            return result;
+            return Vector128.Create(
+                left._lower * right,
+                left._upper * right
+            );
         }
 
         /// <summary>Multiplies a vector by a scalar to compute their product.</summary>
@@ -350,7 +296,7 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator *(T left, Vector64<T> right) => right * left;
+        public static Vector128<T> operator *(T left, Vector128<T> right) => right * left;
 
         /// <summary>Computes the ones-complement of a vector.</summary>
         /// <param name="vector">The vector whose ones-complement is to be computed.</param>
@@ -358,14 +304,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator ~(Vector64<T> vector)
+        public static Vector128<T> operator ~(Vector128<T> vector)
         {
-            ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
-
-            Unsafe.SkipInit(out Vector64<T> result);
-            Unsafe.AsRef(in result._00) = ~vector._00;
-
-            return result;
+            return Vector128.Create(
+                ~vector._lower,
+                ~vector._upper
+            );
         }
 
         /// <summary>Shifts (signed) each element of a vector right by the specified amount.</summary>
@@ -374,17 +318,12 @@ namespace System.Runtime.Intrinsics
         /// <returns>A vector whose elements where shifted right by <paramref name="shiftCount" />.</returns>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator >>(Vector64<T> value, int shiftCount)
+        public static Vector128<T> operator >>(Vector128<T> value, int shiftCount)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T element = Scalar<T>.ShiftRightArithmetic(value.GetElementUnsafe(index), shiftCount);
-                result.SetElementUnsafe(index, element);
-            }
-
-            return result;
+            return Vector128.Create(
+                value._lower >> shiftCount,
+                value._upper >> shiftCount
+            );
         }
 
         /// <summary>Subtracts two vectors to compute their difference.</summary>
@@ -394,17 +333,12 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator -(Vector64<T> left, Vector64<T> right)
+        public static Vector128<T> operator -(Vector128<T> left, Vector128<T> right)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T value = Scalar<T>.Subtract(left.GetElementUnsafe(index), right.GetElementUnsafe(index));
-                result.SetElementUnsafe(index, value);
-            }
-
-            return result;
+            return Vector128.Create(
+                left._lower - right._lower,
+                left._upper - right._upper
+            );
         }
 
         /// <summary>Computes the unary negation of a vector.</summary>
@@ -413,7 +347,13 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator -(Vector64<T> vector) => Zero - vector;
+        public static Vector128<T> operator -(Vector128<T> vector)
+        {
+            return Vector128.Create(
+                -vector._lower,
+                -vector._upper
+            );
+        }
 
         /// <summary>Returns a given vector unchanged.</summary>
         /// <param name="value">The vector.</param>
@@ -421,9 +361,9 @@ namespace System.Runtime.Intrinsics
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator +(Vector64<T> value)
+        public static Vector128<T> operator +(Vector128<T> value)
         {
-            ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
+            ThrowHelper.ThrowForUnsupportedIntrinsicsVector128BaseType<T>();
             return value;
         }
 
@@ -433,60 +373,55 @@ namespace System.Runtime.Intrinsics
         /// <returns>A vector whose elements where shifted right by <paramref name="shiftCount" />.</returns>
         [Intrinsic]
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public static Vector64<T> operator >>>(Vector64<T> value, int shiftCount)
+        public static Vector128<T> operator >>>(Vector128<T> value, int shiftCount)
         {
-            Unsafe.SkipInit(out Vector64<T> result);
-
-            for (int index = 0; index < Count; index++)
-            {
-                T element = Scalar<T>.ShiftRightLogical(value.GetElementUnsafe(index), shiftCount);
-                result.SetElementUnsafe(index, element);
-            }
-
-            return result;
+            return Vector128.Create(
+                value._lower >>> shiftCount,
+                value._upper >>> shiftCount
+            );
         }
 
         /// <summary>Determines whether the specified object is equal to the current instance.</summary>
         /// <param name="obj">The object to compare with the current instance.</param>
-        /// <returns><c>true</c> if <paramref name="obj" /> is a <see cref="Vector64{T}" /> and is equal to the current instance; otherwise, <c>false</c>.</returns>
-        [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public override bool Equals([NotNullWhen(true)] object? obj) => (obj is Vector64<T> other) && Equals(other);
-
-        /// <summary>Determines whether the specified <see cref="Vector64{T}" /> is equal to the current instance.</summary>
-        /// <param name="other">The <see cref="Vector64{T}" /> to compare with the current instance.</param>
-        /// <returns><c>true</c> if <paramref name="other" /> is equal to the current instance; otherwise, <c>false</c>.</returns>
+        /// <returns><c>true</c> if <paramref name="obj" /> is a <see cref="Vector128{T}" /> and is equal to the current instance; otherwise, <c>false</c>.</returns>
         /// <exception cref="NotSupportedException">The type of the vector (<typeparamref name="T" />) is not supported.</exception>
         [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        public bool Equals(Vector64<T> other)
+        public override bool Equals([NotNullWhen(true)] object? obj) => (obj is Vector128<T> other) && Equals(other);
+
+        // Account for floating-point equality around NaN
+        // This is in a separate method so it can be optimized by the mono interpreter/jiterpreter
+        [Intrinsic]
+        [MethodImpl(MethodImplOptions.AggressiveInlining)]
+        internal static bool EqualsFloatingPoint (Vector128<T> lhs, Vector128<T> rhs)
+        {
+            Vector128<T> result = Vector128.Equals(lhs, rhs) | ~(Vector128.Equals(lhs, lhs) | Vector128.Equals(rhs, rhs));
+            return result.AsInt32() == Vector128<int>.AllBitsSet;
+        }
+
+        /// <summary>Determines whether the specified <see cref="Vector128{T}" /> is equal to the current instance.</summary>
+        /// <param name="other">The <see cref="Vector128{T}" /> to compare with the current instance.</param>
+        /// <returns><c>true</c> if <paramref name="other" /> is equal to the current instance; otherwise, <c>false</c>.</returns>
+        [MethodImpl(MethodImplOptions.AggressiveInlining)]
+        public bool Equals(Vector128<T> other)
         {
             // This function needs to account for floating-point equality around NaN
             // and so must behave equivalently to the underlying float/double.Equals
 
-            if (Vector64.IsHardwareAccelerated)
+            if (Vector128.IsHardwareAccelerated)
             {
                 if ((typeof(T) == typeof(double)) || (typeof(T) == typeof(float)))
                 {
-                    Vector64<T> result = Vector64.Equals(this, other) | ~(Vector64.Equals(this, this) | Vector64.Equals(other, other));
-                    return result.AsInt32() == Vector64<int>.AllBitsSet;
+                    return EqualsFloatingPoint(this, other);
                 }
                 else
                 {
                     return this == other;
                 }
             }
-
-            return SoftwareFallback(in this, other);
-
-            static bool SoftwareFallback(in Vector64<T> self, Vector64<T> other)
+            else
             {
-                for (int index = 0; index < Count; index++)
-                {
-                    if (!Scalar<T>.ObjectEquals(self.GetElementUnsafe(index), other.GetElementUnsafe(index)))
-                    {
-                        return false;
-                    }
-                }
-                return true;
+                return _lower.Equals(other._lower)
+                    && _upper.Equals(other._upper);
             }
         }
 
@@ -514,7 +449,7 @@ namespace System.Runtime.Intrinsics
 
         private string ToString([StringSyntax(StringSyntaxAttribute.NumericFormat)] string? format, IFormatProvider? formatProvider)
         {
-            ThrowHelper.ThrowForUnsupportedIntrinsicsVector64BaseType<T>();
+            ThrowHelper.ThrowForUnsupportedIntrinsicsVector128BaseType<T>();
 
             var sb = new ValueStringBuilder(stackalloc char[64]);
             string separator = NumberFormatInfo.GetInstance(formatProvider).NumberGroupSeparator;
