@@ -7,10 +7,7 @@ using System.Runtime.Intrinsics;
 using System.Runtime.Intrinsics.Arm;
 using System.Runtime.Intrinsics.X86;
 using System.Numerics;
-
-#if SYSTEM_PRIVATE_CORELIB
-using Internal.Runtime.CompilerServices;
-#endif
+using System.Buffers.Text;
 
 namespace System.Text.Unicode
 {
@@ -32,7 +29,7 @@ namespace System.Text.Unicode
             // First, we'll handle the common case of all-ASCII. If this is able to
             // consume the entire buffer, we'll skip the remainder of this method's logic.
 
-            int numAsciiCharsConsumedJustNow = (int)ASCIIUtility.GetIndexOfFirstNonAsciiChar(pInputBuffer, (uint)inputLength);
+            int numAsciiCharsConsumedJustNow = (int)Ascii.GetIndexOfFirstNonAsciiChar(pInputBuffer, (uint)inputLength);
             Debug.Assert(0 <= numAsciiCharsConsumedJustNow && numAsciiCharsConsumedJustNow <= inputLength);
 
             pInputBuffer += (uint)numAsciiCharsConsumedJustNow;
@@ -66,7 +63,7 @@ namespace System.Text.Unicode
             // ARM64-intrinsicified code paths. ARM64 platforms may still use the vectorized
             // non-intrinsicified 'else' block below.
 
-            if (/* (AdvSimd.Arm64.IsSupported && BitConverter.IsLittleEndian) || */ Sse41.IsSupported)
+            if (/* (AdvSimd.Arm64.IsSupported && BitConverter.IsLittleEndian) || */ Sse2.IsSupported)
             {
                 if (inputLength >= Vector128<ushort>.Count)
                 {
@@ -74,75 +71,36 @@ namespace System.Text.Unicode
                     Vector128<ushort> vector7800 = Vector128.Create((ushort)0x7800);
                     Vector128<ushort> vectorA000 = Vector128.Create((ushort)0xA000);
 
-                    Vector128<byte> bitMask128 = BitConverter.IsLittleEndian ?
-                        Vector128.Create(0x80402010_08040201).AsByte() :
-                        Vector128.Create(0x01020408_10204080).AsByte();
-
                     char* pHighestAddressWhereCanReadOneVector = pEndOfInputBuffer - Vector128<ushort>.Count;
                     Debug.Assert(pHighestAddressWhereCanReadOneVector >= pInputBuffer);
 
                     do
                     {
-                        Vector128<ushort> utf16Data;
-                        if (AdvSimd.Arm64.IsSupported)
-                        {
-                            utf16Data = AdvSimd.LoadVector128((ushort*)pInputBuffer); // unaligned
-                        }
-                        else
-                        {
-                            utf16Data = Sse2.LoadVector128((ushort*)pInputBuffer); // unaligned
-                        }
+                        Vector128<ushort> utf16Data = Vector128.Load((ushort*)pInputBuffer);
 
                         pInputBuffer += Vector128<ushort>.Count; // eagerly bump this now in preparation for next loop, will adjust later if necessary
-                        Vector128<ushort> charIsNonAscii;
 
                         // Sets the 0x0080 bit of each element in 'charIsNonAscii' if the corresponding
                         // input was 0x0080 <= [value]. (i.e., [value] is non-ASCII.)
 
-                        if (AdvSimd.Arm64.IsSupported)
-                        {
-                            charIsNonAscii = AdvSimd.Min(utf16Data, vector0080);
-                        }
-                        else
-                        {
-                            charIsNonAscii = Sse41.Min(utf16Data, vector0080);
-                        }
+                        Vector128<ushort> charIsNonAscii = Vector128.Min(utf16Data, vector0080);
 
 #if DEBUG
                         // Quick check to ensure we didn't accidentally set the 0x8000 bit of any element.
-                        uint debugMask;
-                        if (AdvSimd.Arm64.IsSupported)
-                        {
-                            debugMask = GetNonAsciiBytes(charIsNonAscii.AsByte(), bitMask128);
-                        }
-                        else
-                        {
-                            debugMask = (uint)Sse2.MoveMask(charIsNonAscii.AsByte());
-                        }
+                        uint debugMask = charIsNonAscii.AsByte().ExtractMostSignificantBits();
                         Debug.Assert((debugMask & 0b_1010_1010_1010_1010) == 0, "Shouldn't have set the 0x8000 bit of any element in 'charIsNonAscii'.");
 #endif // DEBUG
 
                         // Sets the 0x8080 bits of each element in 'charIsNonAscii' if the corresponding
                         // input was 0x0800 <= [value]. This also handles the missing range a few lines above.
 
-                        Vector128<ushort> charIsThreeByteUtf8Encoded;
-                        uint mask;
-
                         // Since 3-byte elements have a value >= 0x0800, we'll perform a saturating add of 0x7800 in order to
                         // get all 3-byte elements to have their 0x8000 bits set. A saturating add will not set the 0x8000
                         // bit for 1-byte or 2-byte elements. The 0x0080 bit will already have been set for non-ASCII (2-byte
                         // and 3-byte) elements.
 
-                        if (AdvSimd.IsSupported)
-                        {
-                            charIsThreeByteUtf8Encoded = AdvSimd.AddSaturate(utf16Data, vector7800);
-                            mask = GetNonAsciiBytes(AdvSimd.Or(charIsNonAscii, charIsThreeByteUtf8Encoded).AsByte(), bitMask128);
-                        }
-                        else
-                        {
-                            charIsThreeByteUtf8Encoded = Sse2.AddSaturate(utf16Data, vector7800);
-                            mask = (uint)Sse2.MoveMask(Sse2.Or(charIsNonAscii, charIsThreeByteUtf8Encoded).AsByte());
-                        }
+                        Vector128<ushort> charIsThreeByteUtf8Encoded = Vector128.AddSaturate(utf16Data, vector7800);
+                        uint mask = (charIsNonAscii | charIsThreeByteUtf8Encoded).AsByte().ExtractMostSignificantBits();
 
                         // Each even bit of mask will be 1 only if the char was >= 0x0080,
                         // and each odd bit of mask will be 1 only if the char was >= 0x0800.
@@ -176,14 +134,8 @@ namespace System.Text.Unicode
                         //
                         // Since surrogate code points are [D800..DFFF], adding {A000} to each element moves surrogate
                         // code points to [7800..7FFF], which allows performing a single signed comparison.
-                        if (AdvSimd.Arm64.IsSupported)
-                        {
-                            mask = GetNonAsciiBytes(AdvSimd.CompareLessThan(AdvSimd.Add(utf16Data, vectorA000).AsInt16(), vector7800.AsInt16()).AsByte(), bitMask128);
-                        }
-                        else
-                        {
-                            mask = (uint)Sse2.MoveMask(Sse2.CompareLessThan(Sse2.Add(utf16Data, vectorA000).AsInt16(), vector7800.AsInt16()).AsByte());
-                        }
+
+                        mask = Vector128.LessThan((utf16Data + vectorA000).AsInt16(), vector7800.AsInt16()).AsByte().ExtractMostSignificantBits();
 
                     FinishIteration:
 
@@ -224,15 +176,7 @@ namespace System.Text.Unicode
                             //   Since 'mask' already has 00 in these positions (since the corresponding char
                             //   wasn't a surrogate), "mask AND mask2 == 00" holds for these positions.
 
-                            uint mask2;
-                            if (AdvSimd.Arm64.IsSupported)
-                            {
-                                mask2 = GetNonAsciiBytes(AdvSimd.ShiftRightLogical(utf16Data, 3).AsByte(), bitMask128);
-                            }
-                            else
-                            {
-                                mask2 = (uint)Sse2.MoveMask(Sse2.ShiftRightLogical(utf16Data, 3).AsByte());
-                            }
+                            uint mask2 = Vector128.ShiftRightLogical(utf16Data, 3).AsByte().ExtractMostSignificantBits();
 
                             // 'lowSurrogatesMask' has its bits occur in pairs:
                             // - 01 if the corresponding char was a low surrogate char,
@@ -276,7 +220,7 @@ namespace System.Text.Unicode
 
                             // If we're 64-bit, we can perform the zero-extension of the surrogate pairs count for
                             // free right now, saving the extension step a few lines below. If we're 32-bit, the
-                            // convertion to nuint immediately below is a no-op, and we'll pay the cost of the real
+                            // conversion to nuint immediately below is a no-op, and we'll pay the cost of the real
                             // 64 -bit extension a few lines below.
                             nuint surrogatePairsCountNuint = (uint)BitOperations.PopCount(highSurrogatesMask);
 
@@ -314,16 +258,16 @@ namespace System.Text.Unicode
                     pInputBuffer -= Vector128<ushort>.Count;
                 }
             }
-            else if (Vector.IsHardwareAccelerated)
+            else if (Vector128.IsHardwareAccelerated)
             {
-                if (inputLength >= Vector<ushort>.Count)
+                if (inputLength >= Vector128<ushort>.Count)
                 {
-                    Vector<ushort> vector0080 = new Vector<ushort>(0x0080);
-                    Vector<ushort> vector0400 = new Vector<ushort>(0x0400);
-                    Vector<ushort> vector0800 = new Vector<ushort>(0x0800);
-                    Vector<ushort> vectorD800 = new Vector<ushort>(0xD800);
+                    Vector128<ushort> vector0080 = Vector128.Create<ushort>(0x0080);
+                    Vector128<ushort> vector0400 = Vector128.Create<ushort>(0x0400);
+                    Vector128<ushort> vector0800 = Vector128.Create<ushort>(0x0800);
+                    Vector128<ushort> vectorD800 = Vector128.Create<ushort>(0xD800);
 
-                    char* pHighestAddressWhereCanReadOneVector = pEndOfInputBuffer - Vector<ushort>.Count;
+                    char* pHighestAddressWhereCanReadOneVector = pEndOfInputBuffer - Vector128<ushort>.Count;
                     Debug.Assert(pHighestAddressWhereCanReadOneVector >= pInputBuffer);
 
                     do
@@ -343,16 +287,16 @@ namespace System.Text.Unicode
                         // performed by the SSE2 code path. This will overcount surrogates, but we'll
                         // handle that shortly.
 
-                        Vector<ushort> utf16Data = Unsafe.ReadUnaligned<Vector<ushort>>(pInputBuffer);
-                        Vector<ushort> twoOrMoreUtf8Bytes = Vector.GreaterThanOrEqual(utf16Data, vector0080);
-                        Vector<ushort> threeOrMoreUtf8Bytes = Vector.GreaterThanOrEqual(utf16Data, vector0800);
-                        Vector<nuint> sumVector = (Vector<nuint>)(Vector<ushort>.Zero - twoOrMoreUtf8Bytes - threeOrMoreUtf8Bytes);
+                        Vector128<ushort> utf16Data = Vector128.Load((ushort*)pInputBuffer);
+                        Vector128<ushort> twoOrMoreUtf8Bytes = Vector128.GreaterThanOrEqual(utf16Data, vector0080);
+                        Vector128<ushort> threeOrMoreUtf8Bytes = Vector128.GreaterThanOrEqual(utf16Data, vector0800);
+                        Vector128<nuint> sumVector = (Vector128<ushort>.Zero - twoOrMoreUtf8Bytes - threeOrMoreUtf8Bytes).AsNUInt();
 
                         // We'll try summing by a natural word (rather than a 16-bit word) at a time,
                         // which should halve the number of operations we must perform.
 
                         nuint popcnt = 0;
-                        for (int i = 0; i < Vector<nuint>.Count; i++)
+                        for (int i = 0; i < Vector128<nuint>.Count; i++)
                         {
                             popcnt += (nuint)sumVector[i];
                         }
@@ -371,16 +315,16 @@ namespace System.Text.Unicode
                         // Now check for surrogates.
 
                         utf16Data -= vectorD800;
-                        Vector<ushort> surrogateChars = Vector.LessThan(utf16Data, vector0800);
-                        if (surrogateChars != Vector<ushort>.Zero)
+                        Vector128<ushort> surrogateChars = Vector128.LessThan(utf16Data, vector0800);
+                        if (surrogateChars != Vector128<ushort>.Zero)
                         {
                             // There's at least one surrogate (high or low) UTF-16 code unit in
                             // the vector. We'll build up additional vectors: 'highSurrogateChars'
                             // and 'lowSurrogateChars', where the elements are 0xFFFF iff the original
                             // UTF-16 code unit was a high or low surrogate, respectively.
 
-                            Vector<ushort> highSurrogateChars = Vector.LessThan(utf16Data, vector0400);
-                            Vector<ushort> lowSurrogateChars = Vector.AndNot(surrogateChars, highSurrogateChars);
+                            Vector128<ushort> highSurrogateChars = Vector128.LessThan(utf16Data, vector0400);
+                            Vector128<ushort> lowSurrogateChars = Vector128.AndNot(surrogateChars, highSurrogateChars);
 
                             // We want to make sure that each high surrogate code unit is followed by
                             // a low surrogate code unit and each low surrogate code unit follows a
@@ -395,7 +339,7 @@ namespace System.Text.Unicode
                             }
 
                             ushort surrogatePairsCount = 0;
-                            for (int i = 0; i < Vector<ushort>.Count - 1; i++)
+                            for (int i = 0; i < Vector128<ushort>.Count - 1; i++)
                             {
                                 surrogatePairsCount -= highSurrogateChars[i]; // turns into +1 or +0
                                 if (highSurrogateChars[i] != lowSurrogateChars[i + 1])
@@ -404,7 +348,7 @@ namespace System.Text.Unicode
                                 }
                             }
 
-                            if (highSurrogateChars[Vector<ushort>.Count - 1] != 0)
+                            if (highSurrogateChars[Vector128<ushort>.Count - 1] != 0)
                             {
                                 // There was a standalone high surrogate at the end of the vector.
                                 // We'll adjust our counters so that we don't consider this char consumed.
@@ -429,7 +373,7 @@ namespace System.Text.Unicode
                         }
 
                         tempUtf8CodeUnitCountAdjustment += popcnt32;
-                        pInputBuffer += Vector<ushort>.Count;
+                        pInputBuffer += Vector128<ushort>.Count;
                     } while (pInputBuffer <= pHighestAddressWhereCanReadOneVector);
                 }
             }
@@ -490,20 +434,5 @@ namespace System.Text.Unicode
             scalarCountAdjustment = tempScalarCountAdjustment;
             return pInputBuffer;
         }
-
-        [MethodImpl(MethodImplOptions.AggressiveInlining)]
-        private static uint GetNonAsciiBytes(Vector128<byte> value, Vector128<byte> bitMask128)
-        {
-            Debug.Assert(AdvSimd.Arm64.IsSupported);
-
-            Vector128<byte> mostSignificantBitIsSet = AdvSimd.ShiftRightArithmetic(value.AsSByte(), 7).AsByte();
-            Vector128<byte> extractedBits = AdvSimd.And(mostSignificantBitIsSet, bitMask128);
-
-            // self-pairwise add until all flags have moved to the first two bytes of the vector
-            extractedBits = AdvSimd.Arm64.AddPairwise(extractedBits, extractedBits);
-            extractedBits = AdvSimd.Arm64.AddPairwise(extractedBits, extractedBits);
-            extractedBits = AdvSimd.Arm64.AddPairwise(extractedBits, extractedBits);
-            return extractedBits.AsUInt16().ToScalar();
-        }
     }
 }
