@@ -1,18 +1,18 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
+using System.Collections;
+using System.Collections.Specialized;
+using System.Text;
+using System.IO;
+using System.Diagnostics;
+using System.Xml.Schema;
+using System.Xml.XPath;
+using System.Runtime.Versioning;
+using System.Diagnostics.CodeAnalysis;
+
 namespace System.Xml.Schema
 {
-    using System.Collections;
-    using System.Collections.Specialized;
-    using System.Text;
-    using System.IO;
-    using System.Diagnostics;
-    using System.Xml.Schema;
-    using System.Xml.XPath;
-    using System.Runtime.Versioning;
-    using System.Diagnostics.CodeAnalysis;
-
 #pragma warning disable 618
     internal sealed class XsdValidator : BaseValidator
     {
@@ -142,14 +142,14 @@ namespace System.Xml.Schema
             { // Done
                 _inlineSchemaParser.FinishParsing();
                 XmlSchema? schema = _inlineSchemaParser.XmlSchema;
-                string? inlineNS = null;
+                string? inlineNS;
                 if (schema != null && schema.ErrorCount == 0)
                 {
                     try
                     {
                         SchemaInfo inlineSchemaInfo = new SchemaInfo();
                         inlineSchemaInfo.SchemaType = SchemaType.XSD;
-                        inlineNS = schema.TargetNamespace == null ? string.Empty : schema.TargetNamespace;
+                        inlineNS = schema.TargetNamespace ?? string.Empty;
                         if (!SchemaInfo!.TargetNamespaces.ContainsKey(inlineNS))
                         {
                             if (SchemaCollection!.Add(inlineNS, inlineSchemaInfo, schema, true) != null)
@@ -187,7 +187,7 @@ namespace System.Xml.Schema
         private object? ValidateChildElement()
         {
             object? particle = null;
-            int errorCode = 0;
+            int errorCode;
             if (context!.NeedValidateChildren)
             {
                 if (context.IsNill)
@@ -379,10 +379,7 @@ namespace System.Xml.Schema
 
         private SchemaElementDecl? ThoroughGetElementDecl(SchemaElementDecl? elementDecl, XmlQualifiedName xsiType, string? xsiNil)
         {
-            if (elementDecl == null)
-            {
-                elementDecl = schemaInfo!.GetElementDecl(elementName);
-            }
+            elementDecl ??= schemaInfo!.GetElementDecl(elementName);
             if (elementDecl != null)
             {
                 if (xsiType.IsEmpty)
@@ -572,7 +569,7 @@ namespace System.Xml.Schema
 
         private void LoadSchemaFromLocation(string uri, string url)
         {
-            XmlReader? reader = null;
+            XmlTextReader? reader = null;
             SchemaInfo? schemaInfo = null;
 
             try
@@ -613,10 +610,7 @@ namespace System.Xml.Schema
             }
             finally
             {
-                if (reader != null)
-                {
-                    reader.Close();
-                }
+                reader?.Close();
             }
         }
 
@@ -638,7 +632,7 @@ namespace System.Xml.Schema
             {
                 if (schemaInfo.SchemaType != SchemaType.XSD)
                 {
-                    throw new XmlException(SR.Xml_MultipleValidaitonTypes, string.Empty, this.PositionInfo.LineNumber, this.PositionInfo.LinePosition);
+                    throw new XmlException(SR.Xml_MultipleValidationTypes, string.Empty, this.PositionInfo.LineNumber, this.PositionInfo.LinePosition);
                 }
                 SchemaInfo.Add(schemaInfo, EventHandler);
                 return;
@@ -762,17 +756,14 @@ namespace System.Xml.Schema
             // Note: It used to be true that we only called this if _fValidate was true,
             // but due to the fact that you can now dynamically type somethign as an ID
             // that is no longer true.
-            if (_IDs == null)
-            {
-                _IDs = new Hashtable();
-            }
+            _IDs ??= new Hashtable();
 
             _IDs.Add(name, node);
         }
 
         public override object? FindId(string name)
         {
-            return _IDs == null ? null : _IDs[name];
+            return _IDs?[name];
         }
 
         public bool IsXSDRoot(string localName, string ns)
@@ -864,7 +855,7 @@ namespace System.Xml.Schema
             } // foreach constraint /constraintstruct
 
             // added on June 19, make connections between new keyref tables with key/unique tables in stack
-            // i can't put it in the above loop, coz there will be key on the same level
+            // i can't put it in the above loop, because there will be key on the same level
             for (int i = 0; i < context.Constr.Length; ++i)
             {
                 if (context.Constr[i].constraint.Role == CompiledIdentityConstraint.ConstraintRole.Keyref)
@@ -933,7 +924,7 @@ namespace System.Xml.Schema
                     {
                         // selector selects new node, activate a new set of fields
                         Debug.WriteLine("Selector Match!");
-                        Debug.WriteLine("Name: " + reader.LocalName + "\t|\tURI: " + reader.NamespaceURI + "\n");
+                        Debug.WriteLine($"Name: {reader.LocalName}\t|\tURI: {reader.NamespaceURI}\n");
                         // in which axisFields got updated
                         constraints[j].axisSelector.PushKS(PositionInfo.LineNumber, PositionInfo.LinePosition);
                     }
@@ -995,7 +986,7 @@ namespace System.Xml.Schema
                             //attribute is only simpletype, so needn't checking...
                             // can fill value here, yeah!!
                             Debug.WriteLine("Attribute Field Filling Value!");
-                            Debug.WriteLine("Name: " + name + "\t|\tURI: " + ns + "\t|\tValue: " + obj + "\n");
+                            Debug.WriteLine($"Name: {name}\t|\tURI: {ns}\t|\tValue: {obj}\n");
                             if (laxis.Ks[laxis.Column] != null)
                             {
                                 // should be evaluated to either an empty node-set or a node-set with exactly one member
@@ -1012,7 +1003,7 @@ namespace System.Xml.Schema
             }
         }
 
-        private object? UnWrapUnion(object? typedValue)
+        private static object? UnWrapUnion(object? typedValue)
         {
             XsdSimpleValue? simpleValue = typedValue as XsdSimpleValue;
             if (simpleValue != null)
@@ -1047,7 +1038,7 @@ namespace System.Xml.Schema
                         if (laxis.isMatched)
                         {
                             Debug.WriteLine("Element Field Filling Value!");
-                            Debug.WriteLine("Name: " + reader.LocalName + "\t|\tURI: " + reader.NamespaceURI + "\t|\tValue: " + reader.TypedValueObject + "\n");
+                            Debug.WriteLine($"Name: {reader.LocalName}\t|\tURI: {reader.NamespaceURI}\t|\tValue: {reader.TypedValueObject}\n");
                             // fill value
                             laxis.isMatched = false;
                             if (laxis.Ks[laxis.Column] != null)
@@ -1087,7 +1078,7 @@ namespace System.Xml.Schema
                                 else if (constraints[i].qualifiedTable!.Contains(ks))
                                 {
                                     // unique or key checking value confliction
-                                    // for redundant key, reporting both occurings
+                                    // for redundant key, reporting both occurrings
                                     // doesn't work... how can i retrieve value out??
                                     SendValidationEvent(new XmlSchemaException(SR.Sch_DuplicateKey,
                                         new string[2] { ks.ToString(), constraints[i].constraint.name.ToString() },
