@@ -1,11 +1,11 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
+using System;
+using System.Diagnostics.CodeAnalysis;
+
 namespace System.Xml.Serialization
 {
-    using System;
-    using System.Diagnostics.CodeAnalysis;
-
     [AttributeUsage(AttributeTargets.Field)]
     public class SoapEnumAttribute : System.Attribute
     {
@@ -23,7 +23,7 @@ namespace System.Xml.Serialization
         [AllowNull]
         public string Name
         {
-            get { return _name == null ? string.Empty : _name; }
+            get { return _name ?? string.Empty; }
             set { _name = value; }
         }
     }
