@@ -16,7 +16,7 @@ namespace System.Reflection.TypeLoading
         public abstract override int LocalSignatureMetadataToken { get; }
 
         // Unlike most apis, this one does not copy the byte array.
-        public sealed override byte[]? GetILAsByteArray() => _lazyIL ?? (_lazyIL = ComputeIL());
+        public sealed override byte[]? GetILAsByteArray() => _lazyIL ??= ComputeIL();
         protected abstract byte[]? ComputeIL();
         private volatile byte[]? _lazyIL;
 
