@@ -7,10 +7,12 @@ using System.Diagnostics;
 using System.Globalization;
 using System.IO;
 using System.Linq;
+using System.Reflection.Emit;
 using System.Reflection.Tests;
 using System.Runtime.CompilerServices;
 using System.Security;
 using System.Text;
+using Microsoft.DotNet.RemoteExecutor;
 using Xunit;
 
 [assembly:
@@ -21,7 +23,6 @@ StringAttr("hello", name = "StringAttrSimple"),
 EnumAttr(PublicEnum.Case1, name = "EnumAttrSimple"),
 TypeAttr(typeof(object), name = "TypeAttrSimple")]
 [assembly: CompilationRelaxations(8)]
-[assembly: Debuggable((DebuggableAttribute.DebuggingModes)263)]
 [assembly: CLSCompliant(false)]
 [assembly: TypeForwardedTo(typeof(string))]
 [assembly: TypeForwardedTo(typeof(TypeInForwardedAssembly))]
@@ -38,12 +39,15 @@ namespace System.Reflection.Tests
 
         public AssemblyTests()
         {
-            // Assembly.Location does not return the file path for single-file deployment targets.
-            DestTestAssemblyPath = Path.Combine(base.TestDirectory, s_sourceTestAssemblyName);
-            LoadFromTestPath = Path.Combine(base.TestDirectory, "System.Reflection.Tests.dll");
-            File.Copy(SourceTestAssemblyPath, DestTestAssemblyPath);
-            string currAssemblyPath = Path.Combine(Environment.CurrentDirectory, "System.Reflection.Tests.dll");
-            File.Copy(currAssemblyPath, LoadFromTestPath, true);
+            if (PlatformDetection.IsAssemblyLoadingSupported)
+            {
+                // Assembly.Location does not return the file path for single-file deployment targets.
+                DestTestAssemblyPath = Path.Combine(base.TestDirectory, s_sourceTestAssemblyName);
+                LoadFromTestPath = Path.Combine(base.TestDirectory, "System.Reflection.Tests.dll");
+                File.Copy(SourceTestAssemblyPath, DestTestAssemblyPath);
+                string currAssemblyPath = Path.Combine(Environment.CurrentDirectory, "System.Reflection.Tests.dll");
+                File.Copy(currAssemblyPath, LoadFromTestPath, true);
+            }
         }
 
         [Theory]
@@ -57,7 +61,6 @@ namespace System.Reflection.Tests
         [InlineData(typeof(AssemblyDescriptionAttribute))]
         [InlineData(typeof(AssemblyCompanyAttribute))]
         [InlineData(typeof(CLSCompliantAttribute))]
-        [InlineData(typeof(DebuggableAttribute))]
         [InlineData(typeof(Attr))]
         public void CustomAttributes(Type type)
         {
@@ -144,16 +147,66 @@ namespace System.Reflection.Tests
         }
 
         [Fact]
-        [SkipOnPlatform(TestPlatforms.Browser, "entry assembly won't be xunit.console on browser")]
-        [ActiveIssue("https://github.com/dotnet/runtime/issues/36892", TestPlatforms.iOS | TestPlatforms.tvOS | TestPlatforms.MacCatalyst)]
         public void GetEntryAssembly()
         {
             Assert.NotNull(Assembly.GetEntryAssembly());
             string assembly = Assembly.GetEntryAssembly().ToString();
-            bool correct = assembly.IndexOf("xunit.console", StringComparison.OrdinalIgnoreCase) != -1;
+
+            bool correct;
+            if (PlatformDetection.IsNativeAot)
+            {
+                // The single file test runner is not 'xunit.console'.
+                correct = assembly.IndexOf("System.Reflection.Tests", StringComparison.OrdinalIgnoreCase) != -1;
+            }
+            else if (PlatformDetection.IsiOS || PlatformDetection.IstvOS)
+            {
+                // The iOS/tvOS test runner is not 'xunit.console'.
+                correct = assembly.IndexOf("AppleTestRunner", StringComparison.OrdinalIgnoreCase) != -1;
+            }
+            else if (PlatformDetection.IsAndroid)
+            {
+                // The Android test runner is not 'xunit.console'.
+                correct = assembly.IndexOf("AndroidTestRunner", StringComparison.OrdinalIgnoreCase) != -1;
+            }
+            else if (PlatformDetection.IsBrowser)
+            {
+                // The browser test runner is not 'xunit.console'.
+                correct = assembly.IndexOf("WasmTestRunner", StringComparison.OrdinalIgnoreCase) != -1;
+            }
+            else
+            {
+                // Under Visual Studio, the runner is 'testhost', otherwise it is 'xunit.console'.
+                correct = assembly.IndexOf("xunit.console", StringComparison.OrdinalIgnoreCase) != -1 ||
+                          assembly.IndexOf("testhost", StringComparison.OrdinalIgnoreCase) != -1;
+            }
+
             Assert.True(correct, $"Unexpected assembly name {assembly}");
         }
 
+        [ConditionalFact(typeof(RemoteExecutor), nameof(RemoteExecutor.IsSupported))]
+        public void TestSetEntryAssembly()
+        {
+            Assert.NotNull(Assembly.GetEntryAssembly());
+
+            RemoteExecutor.Invoke(() =>
+            {
+                SetEntryAssembly(null);
+                Assert.Null(Assembly.GetEntryAssembly());
+                
+                Assembly testAssembly = typeof(AssemblyTests).Assembly;
+                
+                SetEntryAssembly(testAssembly);
+                Assert.Equal(Assembly.GetEntryAssembly(), testAssembly);
+            }).Dispose();
+        }
+
+        private static void SetEntryAssembly(Assembly assembly)
+        {
+            typeof(Assembly)
+                .GetMethod("SetEntryAssembly", BindingFlags.Public | BindingFlags.Static)
+                .Invoke(null, new object[]{ assembly });
+        }
+
         [Fact]
         public void GetFile()
         {
@@ -161,7 +214,7 @@ namespace System.Reflection.Tests
             if (asm.Location.Length > 0)
             {
                 Assert.Throws<ArgumentNullException>(() => asm.GetFile(null));
-                AssertExtensions.Throws<ArgumentException>(null, () => asm.GetFile(""));
+                Assert.Throws<ArgumentException>(() => asm.GetFile(""));
                 Assert.Null(asm.GetFile("NonExistentfile.dll"));
                 Assert.NotNull(asm.GetFile("System.Reflection.Tests.dll"));
 
@@ -174,7 +227,7 @@ namespace System.Reflection.Tests
             }
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void GetFile_InMemory()
         {
             var inMemBlob = File.ReadAllBytes(SourceTestAssemblyPath);
@@ -186,7 +239,7 @@ namespace System.Reflection.Tests
             Assert.Throws<FileNotFoundException>(() => asm.GetFiles(getResourceModules: false));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void CodeBaseInMemory()
         {
             var inMemBlob = File.ReadAllBytes(SourceTestAssemblyPath);
@@ -337,7 +390,7 @@ namespace System.Reflection.Tests
             Assert.Throws<FileNotFoundException>(() => Assembly.Load(new AssemblyName("no such assembly"))); // No such assembly
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFile()
         {
             Assembly currentAssembly = typeof(AssemblyTests).Assembly;
@@ -361,20 +414,20 @@ namespace System.Reflection.Tests
             Assert.Equal(loadedAssembly1, loadedAssembly2);
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFile_NullPath_ThrowsArgumentNullException()
         {
             AssertExtensions.Throws<ArgumentNullException>("path", () => Assembly.LoadFile(null));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFile_NoSuchPath_ThrowsFileNotFoundException()
         {
             string rootedPath = Path.GetFullPath(Guid.NewGuid().ToString("N"));
             AssertExtensions.ThrowsContains<FileNotFoundException>(() => Assembly.LoadFile(rootedPath), rootedPath);
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFile_PartiallyQualifiedPath_ThrowsArgumentException()
         {
             string path = "System.Runtime.Tests.dll";
@@ -384,18 +437,18 @@ namespace System.Reflection.Tests
 
         // This test should apply equally to Unix, but this reliably hits a particular one of the
         // myriad ways that assembly load can fail
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         [PlatformSpecific(TestPlatforms.Windows)]
         public void LoadFile_ValidPEBadIL_ThrowsBadImageFormatExceptionWithPath()
         {
-            string path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.System), "kernelbase.dll");
+            string path = Path.Combine(Environment.SystemDirectory, "kernelbase.dll");
             if (!File.Exists(path))
                 return;
 
             AssertExtensions.ThrowsContains<BadImageFormatException>(() => Assembly.LoadFile(path), path);
         }
 
-        [Theory]
+        [ConditionalTheory(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         [InlineData(0)]
         [InlineData(5)]
         [InlineData(50)]
@@ -415,13 +468,13 @@ namespace System.Reflection.Tests
             AssertExtensions.ThrowsContains<BadImageFormatException>(() => Assembly.LoadFile(path), path);
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFromUsingHashValue()
         {
             Assert.Throws<NotSupportedException>(() => Assembly.LoadFrom("abc", null, System.Configuration.Assemblies.AssemblyHashAlgorithm.SHA1));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFrom_SamePath_ReturnsEqualAssemblies()
         {
             Assembly assembly1 = Assembly.LoadFrom(DestTestAssemblyPath);
@@ -429,7 +482,7 @@ namespace System.Reflection.Tests
             Assert.Equal(assembly1, assembly2);
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFrom_SameIdentityAsAssemblyWithDifferentPath_ReturnsEqualAssemblies()
         {
             Assembly assembly1 = Assembly.LoadFrom(AssemblyPathHelper.GetAssemblyLocation(typeof(AssemblyTests).Assembly));
@@ -440,28 +493,28 @@ namespace System.Reflection.Tests
             Assert.Equal(assembly1, assembly2);
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFrom_NullAssemblyFile_ThrowsArgumentNullException()
         {
             AssertExtensions.Throws<ArgumentNullException>("assemblyFile", () => Assembly.LoadFrom(null));
             AssertExtensions.Throws<ArgumentNullException>("assemblyFile", () => Assembly.UnsafeLoadFrom(null));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFrom_EmptyAssemblyFile_ThrowsArgumentException()
         {
             AssertExtensions.Throws<ArgumentException>("path", null, (() => Assembly.LoadFrom("")));
             AssertExtensions.Throws<ArgumentException>("path", null, (() => Assembly.UnsafeLoadFrom("")));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFrom_NoSuchFile_ThrowsFileNotFoundException()
         {
             Assert.Throws<FileNotFoundException>(() => Assembly.LoadFrom("NoSuchPath"));
             Assert.Throws<FileNotFoundException>(() => Assembly.UnsafeLoadFrom("NoSuchPath"));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void UnsafeLoadFrom_SamePath_ReturnsEqualAssemblies()
         {
             Assembly assembly1 = Assembly.UnsafeLoadFrom(DestTestAssemblyPath);
@@ -469,13 +522,13 @@ namespace System.Reflection.Tests
             Assert.Equal(assembly1, assembly2);
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadFrom_WithHashValue_ThrowsNotSupportedException()
         {
             Assert.Throws<NotSupportedException>(() => Assembly.LoadFrom(DestTestAssemblyPath, new byte[0], Configuration.Assemblies.AssemblyHashAlgorithm.None));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void LoadModule()
         {
             Assembly assembly = typeof(AssemblyTests).Assembly;
@@ -513,7 +566,14 @@ namespace System.Reflection.Tests
         [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsNotBrowser))] // single file
         public void CodeBase()
         {
-            Assert.NotEmpty(Helpers.ExecutingAssembly.CodeBase);
+            if (PlatformDetection.IsNativeAot)
+            {
+                Assert.Throws<NotSupportedException>(() => _ = Helpers.ExecutingAssembly.CodeBase);
+            }
+            else
+            {
+                Assert.NotEmpty(Helpers.ExecutingAssembly.CodeBase);
+            }
         }
 #pragma warning restore SYSLIB0012
 
@@ -589,9 +649,16 @@ namespace System.Reflection.Tests
         [Fact]
         public void GetReferencedAssemblies()
         {
-            // It is too brittle to depend on the assembly references so we just call the method and check that it does not throw.
-            AssemblyName[] assemblies = Helpers.ExecutingAssembly.GetReferencedAssemblies();
-            Assert.NotEmpty(assemblies);
+            if (PlatformDetection.IsNativeAot)
+            {
+                Assert.Throws<PlatformNotSupportedException>(() => Helpers.ExecutingAssembly.GetReferencedAssemblies());
+            }
+            else
+            {
+                // It is too brittle to depend on the assembly references so we just call the method and check that it does not throw.
+                AssemblyName[] assemblies = Helpers.ExecutingAssembly.GetReferencedAssemblies();
+                Assert.NotEmpty(assemblies);
+            }
         }
 
         public static IEnumerable<object[]> Modules_TestData()
@@ -654,6 +721,7 @@ namespace System.Reflection.Tests
 
         [Theory]
         [ActiveIssue("https://github.com/dotnet/runtime/issues/51673", typeof(PlatformDetection), nameof(PlatformDetection.IsBrowser), nameof(PlatformDetection.IsMonoAOT))]
+        [ActiveIssue("https://github.com/dotnet/runtime/issues/69919", typeof(PlatformDetection), nameof(PlatformDetection.IsNativeAot))]
         [MemberData(nameof(GetCallingAssembly_TestData))]
         public void GetCallingAssembly(Assembly assembly1, Assembly assembly2, bool expected)
         {
@@ -667,12 +735,35 @@ namespace System.Reflection.Tests
         }
 
         [Fact]
+        [ActiveIssue("https://github.com/dotnet/runtime/issues/69919", typeof(PlatformDetection), nameof(PlatformDetection.IsNativeAot))]
         public void GetSatelliteAssemblyNeg()
         {
             Assert.Throws<ArgumentNullException>(() => (typeof(AssemblyTests).Assembly.GetSatelliteAssembly(null)));
             Assert.Throws<System.IO.FileNotFoundException>(() => (typeof(AssemblyTests).Assembly.GetSatelliteAssembly(CultureInfo.InvariantCulture)));
         }
 
+        [Fact]
+        public void AssemblyLoadWithPublicKey()
+        {
+            AssemblyName an = new AssemblyName("System.Runtime");
+
+            Assembly a = Assembly.Load(an);
+
+            byte[] publicKey = a.GetName().GetPublicKey();
+            Assert.True(publicKey.Length > 0);
+            an.SetPublicKey(publicKey);
+
+            Assembly a1 = Assembly.Load(an);
+            Assert.Equal(a, a1);
+
+            // Force the public key token to be created
+            Assert.True(an.GetPublicKeyToken().Length > 0);
+
+            // Verify that we can still load the assembly
+            Assembly a2 = Assembly.Load(an);
+            Assert.Equal(a, a2);
+        }
+
         [Fact]
         public void AssemblyLoadFromString()
         {
@@ -693,15 +784,15 @@ namespace System.Reflection.Tests
         public void AssemblyLoadFromStringNeg()
         {
             Assert.Throws<ArgumentNullException>(() => Assembly.Load((string)null));
-            AssertExtensions.Throws<ArgumentException>(null, () => Assembly.Load(string.Empty));
+            AssertExtensions.Throws<ArgumentException>("assemblyName", () => Assembly.Load(string.Empty));
 
             string emptyCName = new string('\0', 1);
-            AssertExtensions.Throws<ArgumentException>(null, () => Assembly.Load(emptyCName));
+            Assert.Throws<ArgumentException>(() => Assembly.Load(emptyCName));
 
             Assert.Throws<FileNotFoundException>(() => Assembly.Load("no such assembly")); // No such assembly
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void AssemblyLoadFromBytes()
         {
             Assembly assembly = typeof(AssemblyTests).Assembly;
@@ -710,6 +801,12 @@ namespace System.Reflection.Tests
             Assembly loadedAssembly = Assembly.Load(aBytes);
             Assert.NotNull(loadedAssembly);
             Assert.Equal(assembly.FullName, loadedAssembly.FullName);
+
+            System.Runtime.Loader.AssemblyLoadContext alc = System.Runtime.Loader.AssemblyLoadContext.GetLoadContext(loadedAssembly);
+            string expectedName = "Assembly.Load(byte[], ...)";
+            Assert.Equal(expectedName, alc.Name);
+            Assert.Contains(expectedName, alc.ToString());
+            Assert.Contains("System.Runtime.Loader.IndividualAssemblyLoadContext", alc.ToString());
         }
 
         [Fact]
@@ -719,8 +816,8 @@ namespace System.Reflection.Tests
             Assert.Throws<BadImageFormatException>(() => Assembly.Load(new byte[0]));
         }
 
-        [Fact]
-        [ActiveIssue("https://github.com/dotnet/runtime/issues/36892", TestPlatforms.iOS | TestPlatforms.tvOS | TestPlatforms.MacCatalyst)]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
+        [SkipOnPlatform(TestPlatforms.iOS | TestPlatforms.tvOS | TestPlatforms.MacCatalyst, "Symbols are in a different location on iOS/tvOS/MacCatalyst")]
         public void AssemblyLoadFromBytesWithSymbols()
         {
             Assembly assembly = typeof(AssemblyTests).Assembly;
@@ -740,7 +837,7 @@ namespace System.Reflection.Tests
             Assert.Throws<PlatformNotSupportedException>(() => Assembly.ReflectionOnlyLoad(an.FullName));
         }
 
-        [Fact]
+        [ConditionalFact(typeof(PlatformDetection), nameof(PlatformDetection.IsAssemblyLoadingSupported))]
         public void AssemblyReflectionOnlyLoadFromBytes()
         {
             Assembly assembly = typeof(AssemblyTests).Assembly;
@@ -797,7 +894,6 @@ namespace System.Reflection.Tests
         [InlineData(typeof(AssemblyDescriptionAttribute))]
         [InlineData(typeof(AssemblyCompanyAttribute))]
         [InlineData(typeof(CLSCompliantAttribute))]
-        [InlineData(typeof(DebuggableAttribute))]
         [InlineData(typeof(Attr))]
         public void GetCustomAttributesData(Type attrType)
         {
@@ -820,6 +916,8 @@ namespace System.Reflection.Tests
         }
 
         [Fact]
+        [ActiveIssue("https://github.com/dotnet/runtime/issues/69919", typeof(PlatformDetection), nameof(PlatformDetection.IsNativeAot))]
+        [ActiveIssue("https://github.com/dotnet/runtime/issues/77821", TestPlatforms.Android)]
         public static void AssemblyGetForwardedTypesLoadFailure()
         {
             Assembly a = typeof(TypeInForwardedAssembly).Assembly;
