@@ -18,6 +18,8 @@ namespace System.Resources.Extensions.Tests
 {
     public class PreserializedResourceWriterTests
     {
+        public static bool AllowsCustomResourceTypes => AppContext.TryGetSwitch("System.Resources.ResourceManager.AllowCustomResourceTypes", out bool isEnabled) ? isEnabled : true;
+
         [Fact]
         public static void ExceptionforNullStream()
         {
@@ -71,36 +73,36 @@ namespace System.Resources.Extensions.Tests
             {
                 writer.AddResource("duplicate", "value");
 
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", "value"));
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", new object()));
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", new byte[0]));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", "value"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", new object()));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", new byte[0]));
 
                 using (var stream = new MemoryStream())
                 {
-                    Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", stream));
-                    Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", stream, true));
-                    Assert.Throws<ArgumentException>(null, () => writer.AddActivatorResource("duplicate", stream, "System.DayOfWeek", false));
+                    AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", stream));
+                    AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", stream, true));
+                    AssertExtensions.Throws<ArgumentException>(null, () => writer.AddActivatorResource("duplicate", stream, "System.DayOfWeek", false));
                 }
 
-                Assert.Throws<ArgumentException>(null, () => writer.AddBinaryFormattedResource("duplicate", new byte[1], "System.DayOfWeek"));
-                Assert.Throws<ArgumentException>(null, () => writer.AddTypeConverterResource("duplicate", new byte[1], "System.DayOfWeek"));
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", "Monday", "System.DayOfWeek"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddBinaryFormattedResource("duplicate", new byte[1], "System.DayOfWeek"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddTypeConverterResource("duplicate", new byte[1], "System.DayOfWeek"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicate", "Monday", "System.DayOfWeek"));
 
 
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("Duplicate", "value"));
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("dUplicate", new object()));
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("duPlicate", new byte[0]));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("Duplicate", "value"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("dUplicate", new object()));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duPlicate", new byte[0]));
 
                 using (var stream = new MemoryStream())
                 {
-                    Assert.Throws<ArgumentException>(null, () => writer.AddResource("dupLicate", stream));
-                    Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplIcate", stream, true));
-                    Assert.Throws<ArgumentException>(null, () => writer.AddActivatorResource("dupliCate", stream, "System.DayOfWeek", false));
+                    AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("dupLicate", stream));
+                    AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplIcate", stream, true));
+                    AssertExtensions.Throws<ArgumentException>(null, () => writer.AddActivatorResource("dupliCate", stream, "System.DayOfWeek", false));
                 }
 
-                Assert.Throws<ArgumentException>(null, () => writer.AddBinaryFormattedResource("duplicAte", new byte[1], "System.DayOfWeek"));
-                Assert.Throws<ArgumentException>(null, () => writer.AddTypeConverterResource("duplicaTe", new byte[1], "System.DayOfWeek"));
-                Assert.Throws<ArgumentException>(null, () => writer.AddResource("duplicatE", "Monday", "System.DayOfWeek"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddBinaryFormattedResource("duplicAte", new byte[1], "System.DayOfWeek"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddTypeConverterResource("duplicaTe", new byte[1], "System.DayOfWeek"));
+                AssertExtensions.Throws<ArgumentException>(null, () => writer.AddResource("duplicatE", "Monday", "System.DayOfWeek"));
             }
         }
 
@@ -340,7 +342,7 @@ namespace System.Resources.Extensions.Tests
                 }
             }
         }
-        [Fact]
+        [ConditionalFact(nameof(AllowsCustomResourceTypes))]
         [ActiveIssue("https://github.com/dotnet/runtime/issues/34495", TestPlatforms.Windows, TargetFrameworkMonikers.Netcoreapp, TestRuntimes.Mono)]
         public static void TypeConverterByteArrayResources()
         {
@@ -373,7 +375,7 @@ namespace System.Resources.Extensions.Tests
             }
         }
 
-        [Fact]
+        [ConditionalFact(nameof(AllowsCustomResourceTypes))]
         public static void TypeConverterStringResources()
         {
             var values = TestData.StringConverter;
@@ -403,7 +405,7 @@ namespace System.Resources.Extensions.Tests
             }
         }
 
-        [Fact]
+        [ConditionalFact(nameof(AllowsCustomResourceTypes))]
         [ActiveIssue("https://github.com/dotnet/runtime/issues/34495", TestPlatforms.Windows, TargetFrameworkMonikers.Netcoreapp, TestRuntimes.Mono)]
         [ActiveIssue("https://github.com/dotnet/runtime/issues/34008", TestPlatforms.Linux, TargetFrameworkMonikers.Netcoreapp, TestRuntimes.Mono)]
         public static void StreamResources()
@@ -469,7 +471,7 @@ namespace System.Resources.Extensions.Tests
             }
         }
 
-        [Fact]
+        [ConditionalFact(nameof(AllowsCustomResourceTypes))]
         public static void ResourceManagerLoadsCorrectReader()
         {
             ResourceManager resourceManager = new ResourceManager(typeof(TestData));
@@ -524,9 +526,15 @@ namespace System.Resources.Extensions.Tests
 
             const int Threads = 10;
             using Barrier barrier = new(Threads);
-            Task task = Task.WhenAll(Enumerable.Range(0, Threads).Select(_ => Task.Run(WaitForBarrierThenEnumerateResources)));
+            Task[] tasks = Enumerable.Range(0, Threads)
+                .Select(_ => Task.Factory.StartNew(
+                    WaitForBarrierThenEnumerateResources,
+                    CancellationToken.None,
+                    TaskCreationOptions.LongRunning,
+                    TaskScheduler.Default))
+                .ToArray();
 
-            Assert.True(task.Wait(TimeSpan.FromSeconds(30)));
+            Assert.True(Task.WaitAll(tasks, TimeSpan.FromSeconds(30)));
 
             void WaitForBarrierThenEnumerateResources()
             {
