@@ -1,8 +1,11 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
+using System.ComponentModel;
+
 namespace System.Runtime.InteropServices.ComTypes
 {
+    [EditorBrowsable(EditorBrowsableState.Never)]
     public struct FORMATETC
     {
         [MarshalAs(UnmanagedType.U2)]
