@@ -3,6 +3,7 @@
 
 using System.Collections.Generic;
 using System.Globalization;
+using System.Text;
 using Xunit;
 
 namespace System.Tests
@@ -101,7 +102,7 @@ namespace System.Tests
         {
             foreach (NumberFormatInfo defaultFormat in new[] { null, NumberFormatInfo.CurrentInfo })
             {
-                foreach (string defaultSpecifier in new[] { "G", "G\0", "\0N222", "\0", "" })
+                foreach (string defaultSpecifier in new[] { "G", "G\0", "\0N222", "\0", "", "R" })
                 {
                     yield return new object[] { short.MinValue, defaultSpecifier, defaultFormat, "-32768" };
                     yield return new object[] { (short)-4567, defaultSpecifier, defaultFormat, "-4567" };
@@ -115,21 +116,26 @@ namespace System.Tests
                 yield return new object[] { (short)4567, "D99\09", defaultFormat, "000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000004567" };
                 yield return new object[] { (short)-4567, "D99", defaultFormat, "-000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000004567" };
 
+                yield return new object[] { (short)0, "x", defaultFormat, "0" };
                 yield return new object[] { (short)0x2468, "x", defaultFormat, "2468" };
                 yield return new object[] { (short)-0x2468, "x", defaultFormat, "db98" };
+
+                yield return new object[] { (short)0, "b", defaultFormat, "0" };
+                yield return new object[] { (short)0x2468, "b", defaultFormat, "10010001101000" };
+                yield return new object[] { (short)-0x2468, "b", defaultFormat, "1101101110011000" };
+
                 yield return new object[] { (short)2468, "N", defaultFormat, string.Format("{0:N}", 2468.00) };
-
-
             }
 
             NumberFormatInfo invariantFormat = NumberFormatInfo.InvariantInfo;
-            yield return new object[] { (short)32, "C100", invariantFormat, "¤32.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000" };
+            yield return new object[] { (short)32, "C100", invariantFormat, "\u00A432.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000" };
             yield return new object[] { (short)32, "P100", invariantFormat, "3,200.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000 %" };
             yield return new object[] { (short)32, "D100", invariantFormat, "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000032" };
             yield return new object[] { (short)32, "E100", invariantFormat, "3.2000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000E+001" };
             yield return new object[] { (short)32, "F100", invariantFormat, "32.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000" };
             yield return new object[] { (short)32, "N100", invariantFormat, "32.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000" };
             yield return new object[] { (short)32, "X100", invariantFormat, "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000020" };
+            yield return new object[] { (short)32, "B100", invariantFormat, "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000100000" };
 
             var customFormat = new NumberFormatInfo()
             {
@@ -162,7 +168,7 @@ namespace System.Tests
             string lowerExpected = expected.ToLowerInvariant();
 
             bool isDefaultProvider = (provider == null || provider == NumberFormatInfo.CurrentInfo);
-            if (string.IsNullOrEmpty(format) || format.ToUpperInvariant() == "G")
+            if (string.IsNullOrEmpty(format) || format.ToUpperInvariant() is "G" or "R")
             {
                 if (isDefaultProvider)
                 {
@@ -186,10 +192,6 @@ namespace System.Tests
         public static void ToString_InvalidFormat_ThrowsFormatException()
         {
             short i = 123;
-            Assert.Throws<FormatException>(() => i.ToString("r")); // Invalid format
-            Assert.Throws<FormatException>(() => i.ToString("r", null)); // Invalid format
-            Assert.Throws<FormatException>(() => i.ToString("R")); // Invalid format
-            Assert.Throws<FormatException>(() => i.ToString("R", null)); // Invalid format
             Assert.Throws<FormatException>(() => i.ToString("Y")); // Invalid format
             Assert.Throws<FormatException>(() => i.ToString("Y", null)); // Invalid format
         }
@@ -213,6 +215,7 @@ namespace System.Tests
             yield return new object[] { "123", NumberStyles.HexNumber, null, (short)0x123 };
             yield return new object[] { "abc", NumberStyles.HexNumber, null, (short)0xabc };
             yield return new object[] { "ABC", NumberStyles.HexNumber, null, (short)0xabc };
+            yield return new object[] { "101010111100", NumberStyles.BinaryNumber, null, (short)0b101010111100 };
             yield return new object[] { "1000", NumberStyles.AllowThousands, null, (short)1000 };
             yield return new object[] { "(123)", NumberStyles.AllowParentheses, null, (short)-123 }; // Parentheses = negative
 
@@ -220,6 +223,7 @@ namespace System.Tests
 
             yield return new object[] { "123", NumberStyles.Any, emptyFormat, (short)123 };
             yield return new object[] { "12", NumberStyles.HexNumber, emptyFormat, (short)0x12 };
+            yield return new object[] { "10010", NumberStyles.BinaryNumber, emptyFormat, (short)0b10010 };
             yield return new object[] { "$1,000", NumberStyles.Currency, customFormat, (short)1000 };
         }
 
@@ -271,8 +275,11 @@ namespace System.Tests
             yield return new object[] { "-32769", NumberStyles.Integer, null, typeof(OverflowException) }; // < min value
             yield return new object[] { "32768", NumberStyles.Integer, null, typeof(OverflowException) }; // > max value
 
-            yield return new object[] { "FFFFFFFF", NumberStyles.HexNumber, null, typeof(OverflowException) }; // Hex number < 0
+            yield return new object[] { "FFFFF", NumberStyles.HexNumber, null, typeof(OverflowException) }; // Hex number < 0
             yield return new object[] { "10000", NumberStyles.HexNumber, null, typeof(OverflowException) }; // Hex number > max value
+
+            yield return new object[] { "11111111111111111", NumberStyles.BinaryNumber, null, typeof(OverflowException) }; // Binary number < 0
+            yield return new object[] { "10000000000000000", NumberStyles.BinaryNumber, null, typeof(OverflowException) }; // Binary number > max value
         }
 
         [Theory]
@@ -313,16 +320,18 @@ namespace System.Tests
         }
 
         [Theory]
-        [InlineData(NumberStyles.HexNumber | NumberStyles.AllowParentheses, null)]
-        [InlineData(unchecked((NumberStyles)0xFFFFFC00), "style")]
-        public static void TryParse_InvalidNumberStyle_ThrowsArgumentException(NumberStyles style, string paramName)
+        [InlineData(NumberStyles.HexNumber | NumberStyles.AllowParentheses)]
+        [InlineData(NumberStyles.BinaryNumber | NumberStyles.AllowParentheses)]
+        [InlineData(NumberStyles.HexNumber | NumberStyles.BinaryNumber)]
+        [InlineData(unchecked((NumberStyles)0xFFFFFC00))]
+        public static void TryParse_InvalidNumberStyle_ThrowsArgumentException(NumberStyles style)
         {
             short result = 0;
-            AssertExtensions.Throws<ArgumentException>(paramName, () => short.TryParse("1", style, null, out result));
+            AssertExtensions.Throws<ArgumentException>("style", () => short.TryParse("1", style, null, out result));
             Assert.Equal(default(short), result);
 
-            AssertExtensions.Throws<ArgumentException>(paramName, () => short.Parse("1", style));
-            AssertExtensions.Throws<ArgumentException>(paramName, () => short.Parse("1", style, null));
+            AssertExtensions.Throws<ArgumentException>("style", () => short.Parse("1", style));
+            AssertExtensions.Throws<ArgumentException>("style", () => short.Parse("1", style, null));
         }
 
         public static IEnumerable<object[]> Parse_ValidWithOffsetCount_TestData()
@@ -336,6 +345,8 @@ namespace System.Tests
             yield return new object[] { "-32768", 0, 5, NumberStyles.Integer, null, (short)-3276 };
             yield return new object[] { "abc", 0, 2, NumberStyles.HexNumber, null, (short)0xab };
             yield return new object[] { "abc", 1, 2, NumberStyles.HexNumber, null, (short)0xbc };
+            yield return new object[] { "1010101100", 0, 8, NumberStyles.BinaryNumber, null, (short)0b10101011 };
+            yield return new object[] { "11101010110011", 2, 8, NumberStyles.BinaryNumber, null, (short)0b10101011 };
             yield return new object[] { "(123)", 1, 3, NumberStyles.AllowParentheses, null, (short)123 };
             yield return new object[] { "123", 0, 1, NumberStyles.Integer, new NumberFormatInfo(), (short)1 };
             yield return new object[] { "$1,000", 1, 5, NumberStyles.Currency, new NumberFormatInfo() { CurrencySymbol = "$" }, (short)1000 };
@@ -384,46 +395,51 @@ namespace System.Tests
         }
 
         [Theory]
-        [MemberData(nameof(ToString_TestData))]
-        public static void TryFormat(short i, string format, IFormatProvider provider, string expected)
+        [MemberData(nameof(Parse_ValidWithOffsetCount_TestData))]
+        public static void Parse_Utf8Span_Valid(string value, int offset, int count, NumberStyles style, IFormatProvider provider, short expected)
         {
-            char[] actual;
-            int charsWritten;
+            short result;
+            ReadOnlySpan<byte> valueUtf8 = Encoding.UTF8.GetBytes(value, offset, count);
 
-            // Just right
-            actual = new char[expected.Length];
-            Assert.True(i.TryFormat(actual.AsSpan(), out charsWritten, format, provider));
-            Assert.Equal(expected.Length, charsWritten);
-            Assert.Equal(expected, new string(actual));
-
-            // Longer than needed
-            actual = new char[expected.Length + 1];
-            Assert.True(i.TryFormat(actual.AsSpan(), out charsWritten, format, provider));
-            Assert.Equal(expected.Length, charsWritten);
-            Assert.Equal(expected, new string(actual, 0, charsWritten));
-
-            // Too short
-            if (expected.Length > 0)
+            // Default style and provider
+            if (style == NumberStyles.Integer && provider == null)
             {
-                actual = new char[expected.Length - 1];
-                Assert.False(i.TryFormat(actual.AsSpan(), out charsWritten, format, provider));
-                Assert.Equal(0, charsWritten);
+                Assert.True(short.TryParse(valueUtf8, out result));
+                Assert.Equal(expected, result);
             }
 
-            if (format != null)
-            {
-                // Upper format
-                actual = new char[expected.Length];
-                Assert.True(i.TryFormat(actual.AsSpan(), out charsWritten, format.ToUpperInvariant(), provider));
-                Assert.Equal(expected.Length, charsWritten);
-                Assert.Equal(expected.ToUpperInvariant(), new string(actual));
+            Assert.Equal(expected, short.Parse(valueUtf8, style, provider));
 
-                // Lower format
-                actual = new char[expected.Length];
-                Assert.True(i.TryFormat(actual.AsSpan(), out charsWritten, format.ToLowerInvariant(), provider));
-                Assert.Equal(expected.Length, charsWritten);
-                Assert.Equal(expected.ToLowerInvariant(), new string(actual));
+            Assert.True(short.TryParse(valueUtf8, style, provider, out result));
+            Assert.Equal(expected, result);
+        }
+
+        [Theory]
+        [MemberData(nameof(Parse_Invalid_TestData))]
+        public static void Parse_Utf8Span_Invalid(string value, NumberStyles style, IFormatProvider provider, Type exceptionType)
+        {
+            if (value != null)
+            {
+                short result;
+                ReadOnlySpan<byte> valueUtf8 = Encoding.UTF8.GetBytes(value);
+
+                // Default style and provider
+                if (style == NumberStyles.Integer && provider == null)
+                {
+                    Assert.False(short.TryParse(valueUtf8, out result));
+                    Assert.Equal(0, result);
+                }
+
+                Assert.Throws(exceptionType, () => short.Parse(Encoding.UTF8.GetBytes(value), style, provider));
+
+                Assert.False(short.TryParse(valueUtf8, style, provider, out result));
+                Assert.Equal(0, result);
             }
         }
+
+        [Theory]
+        [MemberData(nameof(ToString_TestData))]
+        public static void TryFormat(short i, string format, IFormatProvider provider, string expected) =>
+            NumberFormatTestHelper.TryFormatNumberTest(i, format, provider, expected);
     }
 }
