@@ -87,10 +87,8 @@ namespace System.Security.AccessControl
             {
                 privilegeLock.EnterReadLock();
 
-                if (luids.ContainsKey(privilege))
+                if (luids.TryGetValue(privilege, out luid))
                 {
-                    luid = luids[privilege];
-
                     privilegeLock.ExitReadLock();
                 }
                 else
@@ -99,7 +97,7 @@ namespace System.Security.AccessControl
 
                     if (false == Interop.Advapi32.LookupPrivilegeValue(null, privilege, out luid))
                     {
-                        int error = Marshal.GetLastWin32Error();
+                        int error = Marshal.GetLastPInvokeError();
 
                         if (error == Interop.Errors.ERROR_NOT_ENOUGH_MEMORY)
                         {
@@ -134,9 +132,8 @@ namespace System.Security.AccessControl
 
                 if (privilegeLock.IsWriteLockHeld)
                 {
-                    if (!luids.ContainsKey(privilege))
+                    if (luids.TryAdd(privilege, luid))
                     {
-                        luids[privilege] = luid;
                         privileges[luid] = privilege;
                     }
 
@@ -177,7 +174,7 @@ namespace System.Security.AccessControl
                                             TokenAccessLevels.Duplicate,
                                             out localProcessHandle))
                             {
-                                cachingError = Marshal.GetLastWin32Error();
+                                cachingError = Marshal.GetLastPInvokeError();
                                 success = false;
                             }
                             processHandle = localProcessHandle;
@@ -201,7 +198,7 @@ namespace System.Security.AccessControl
 
                     if (error != 0)
                     {
-                        if (success == true)
+                        if (success)
                         {
                             this.threadHandle = threadHandleBefore;
 
@@ -212,7 +209,7 @@ namespace System.Security.AccessControl
 
                             System.Diagnostics.Debug.Assert(this.isImpersonating == false, "Incorrect isImpersonating state");
 
-                            if (success == true)
+                            if (success)
                             {
                                 error = 0;
                                 if (false == Interop.Advapi32.DuplicateTokenEx(
@@ -223,12 +220,12 @@ namespace System.Security.AccessControl
                                                 System.Security.Principal.TokenType.TokenImpersonation,
                                                 ref this.threadHandle))
                                 {
-                                    error = Marshal.GetLastWin32Error();
+                                    error = Marshal.GetLastPInvokeError();
                                     success = false;
                                 }
                             }
 
-                            if (success == true)
+                            if (success)
                             {
                                 error = SetThreadToken(this.threadHandle);
                                 unchecked { error &= ~(int)0x80070000; }
@@ -239,7 +236,7 @@ namespace System.Security.AccessControl
                                 }
                             }
 
-                            if (success == true)
+                            if (success)
                             {
                                 this.isImpersonating = true;
                             }
@@ -363,10 +360,7 @@ namespace System.Security.AccessControl
 
         public Privilege(string privilegeName)
         {
-            if (privilegeName == null)
-            {
-                throw new ArgumentNullException(nameof(privilegeName));
-            }
+            ArgumentNullException.ThrowIfNull(privilegeName);
 
             this.luid = LuidFromPrivilege(privilegeName);
         }
@@ -459,9 +453,9 @@ namespace System.Security.AccessControl
                                   &previousState,
                                   &previousSize))
                 {
-                    error = Marshal.GetLastWin32Error();
+                    error = Marshal.GetLastPInvokeError();
                 }
-                else if (Interop.Errors.ERROR_NOT_ALL_ASSIGNED == Marshal.GetLastWin32Error())
+                else if (Interop.Errors.ERROR_NOT_ALL_ASSIGNED == Marshal.GetLastPInvokeError())
                 {
                     error = Interop.Errors.ERROR_NOT_ALL_ASSIGNED;
                 }
@@ -554,7 +548,7 @@ namespace System.Security.AccessControl
                                       null,
                                       null))
                     {
-                        error = Marshal.GetLastWin32Error();
+                        error = Marshal.GetLastPInvokeError();
                         success = false;
                     }
                 }
