@@ -33,8 +33,10 @@ namespace System.Security.Cryptography.Xml
         // A helper function that determines if a namespace node is a committed attribute
         internal static bool IsCommittedNamespace(XmlElement element, string prefix, string value)
         {
-            if (element == null)
+            if (element is null)
+            {
                 throw new ArgumentNullException(nameof(element));
+            }
 
             string name = ((prefix.Length > 0) ? "xmlns:" + prefix : "xmlns");
             if (element.HasAttribute(name) && element.GetAttribute(name) == value) return true;
@@ -43,13 +45,15 @@ namespace System.Security.Cryptography.Xml
 
         internal static bool IsRedundantNamespace(XmlElement element, string prefix, string value)
         {
-            if (element == null)
+            if (element is null)
+            {
                 throw new ArgumentNullException(nameof(element));
+            }
 
-            XmlNode ancestorNode = ((XmlNode)element).ParentNode;
+            XmlNode? ancestorNode = ((XmlNode)element).ParentNode;
             while (ancestorNode != null)
             {
-                XmlElement ancestorElement = ancestorNode as XmlElement;
+                XmlElement? ancestorElement = ancestorNode as XmlElement;
                 if (ancestorElement != null)
                     if (HasNamespace(ancestorElement, prefix, value)) return true;
                 ancestorNode = ancestorNode.ParentNode;
@@ -58,9 +62,9 @@ namespace System.Security.Cryptography.Xml
             return false;
         }
 
-        internal static string GetAttribute(XmlElement element, string localName, string namespaceURI)
+        internal static string? GetAttribute(XmlElement element, string localName, string namespaceURI)
         {
-            string s = (element.HasAttribute(localName) ? element.GetAttribute(localName) : null);
+            string? s = (element.HasAttribute(localName) ? element.GetAttribute(localName) : null);
             if (s == null && element.HasAttribute(localName, namespaceURI))
                 s = element.GetAttribute(localName, namespaceURI);
             return s;
@@ -71,12 +75,12 @@ namespace System.Security.Cryptography.Xml
             return element.HasAttribute(localName) || element.HasAttribute(localName, namespaceURI);
         }
 
-        internal static bool VerifyAttributes(XmlElement element, string expectedAttrName)
+        internal static bool VerifyAttributes(XmlElement element, string? expectedAttrName)
         {
             return VerifyAttributes(element, expectedAttrName == null ? null : new string[] { expectedAttrName });
         }
 
-        internal static bool VerifyAttributes(XmlElement element, string[] expectedAttrNames)
+        internal static bool VerifyAttributes(XmlElement element, string[]? expectedAttrNames)
         {
             foreach (XmlAttribute attr in element.Attributes)
             {
@@ -114,7 +118,7 @@ namespace System.Security.Cryptography.Xml
 
         internal static bool IsEmptyDefaultNamespaceNode(XmlNode n)
         {
-            return IsDefaultNamespaceNode(n) && n.Value.Length == 0;
+            return IsDefaultNamespaceNode(n) && n.Value!.Length == 0;
         }
 
         internal static string GetNamespacePrefix(XmlAttribute a)
@@ -128,7 +132,7 @@ namespace System.Security.Cryptography.Xml
             return GetNamespacePrefix(a).Equals(nsPrefix);
         }
 
-        internal static bool IsNonRedundantNamespaceDecl(XmlAttribute a, XmlAttribute nearestAncestorWithSamePrefix)
+        internal static bool IsNonRedundantNamespaceDecl(XmlAttribute a, XmlAttribute? nearestAncestorWithSamePrefix)
         {
             if (nearestAncestorWithSamePrefix == null)
                 return !IsEmptyDefaultNamespaceNode(a);
@@ -136,12 +140,6 @@ namespace System.Security.Cryptography.Xml
                 return !nearestAncestorWithSamePrefix.Value.Equals(a.Value);
         }
 
-        internal static bool IsXmlPrefixDefinitionNode(XmlAttribute a)
-        {
-            return false;
-            //            return a.Prefix.Equals("xmlns") && a.LocalName.Equals("xml") && a.Value.Equals(NamespaceUrlForXmlPrefix);
-        }
-
         internal static string DiscardWhiteSpaces(string inputBuffer)
         {
             return DiscardWhiteSpaces(inputBuffer, 0, inputBuffer.Length);
@@ -179,14 +177,14 @@ namespace System.Security.Cryptography.Xml
             }
         }
 
-        internal static XmlReader PreProcessStreamInput(Stream inputStream, XmlResolver xmlResolver, string baseUri)
+        internal static XmlReader PreProcessStreamInput(Stream inputStream, XmlResolver? xmlResolver, string? baseUri)
         {
             XmlReaderSettings settings = GetSecureXmlReaderSettings(xmlResolver);
             XmlReader reader = XmlReader.Create(inputStream, settings, baseUri);
             return reader;
         }
 
-        internal static XmlReaderSettings GetSecureXmlReaderSettings(XmlResolver xmlResolver)
+        internal static XmlReaderSettings GetSecureXmlReaderSettings(XmlResolver? xmlResolver)
         {
             XmlReaderSettings settings = new XmlReaderSettings();
             settings.XmlResolver = xmlResolver;
@@ -198,8 +196,10 @@ namespace System.Security.Cryptography.Xml
 
         internal static XmlDocument PreProcessDocumentInput(XmlDocument document, XmlResolver xmlResolver, string baseUri)
         {
-            if (document == null)
+            if (document is null)
+            {
                 throw new ArgumentNullException(nameof(document));
+            }
 
             MyXmlDocument doc = new MyXmlDocument();
             doc.PreserveWhitespace = document.PreserveWhitespace;
@@ -218,10 +218,12 @@ namespace System.Security.Cryptography.Xml
             return doc;
         }
 
-        internal static XmlDocument PreProcessElementInput(XmlElement elem, XmlResolver xmlResolver, string baseUri)
+        internal static XmlDocument PreProcessElementInput(XmlElement elem, XmlResolver xmlResolver, string? baseUri)
         {
-            if (elem == null)
+            if (elem is null)
+            {
                 throw new ArgumentNullException(nameof(elem));
+            }
 
             MyXmlDocument doc = new MyXmlDocument();
             doc.PreserveWhitespace = true;
@@ -241,12 +243,12 @@ namespace System.Security.Cryptography.Xml
 
         internal static XmlDocument DiscardComments(XmlDocument document)
         {
-            XmlNodeList nodeList = document.SelectNodes("//comment()");
+            XmlNodeList? nodeList = document.SelectNodes("//comment()");
             if (nodeList != null)
             {
                 foreach (XmlNode node1 in nodeList)
                 {
-                    node1.ParentNode.RemoveChild(node1);
+                    node1.ParentNode!.RemoveChild(node1);
                 }
             }
             return document;
@@ -264,7 +266,7 @@ namespace System.Security.Cryptography.Xml
 
             do
             {
-                XmlNode rootNode = (XmlNode)elementList[index];
+                XmlNode rootNode = (XmlNode)elementList[index]!;
                 // Add the children nodes
                 XmlNodeList childNodes = rootNode.ChildNodes;
                 if (childNodes != null)
@@ -278,10 +280,10 @@ namespace System.Security.Cryptography.Xml
                     }
                 }
                 // Add the attribute nodes
-                XmlAttributeCollection attribNodes = rootNode.Attributes;
+                XmlAttributeCollection? attribNodes = rootNode.Attributes;
                 if (attribNodes != null)
                 {
-                    foreach (XmlNode attribNode in rootNode.Attributes)
+                    foreach (XmlNode attribNode in rootNode.Attributes!)
                     {
                         if (attribNode.LocalName == "xmlns" || attribNode.Prefix == "xmlns")
                             namespaceList.Add(attribNode);
@@ -307,7 +309,7 @@ namespace System.Security.Cryptography.Xml
             return nodeList;
         }
 
-        internal static bool NodeInList(XmlNode node, XmlNodeList nodeList)
+        internal static bool NodeInList(XmlNode? node, XmlNodeList nodeList)
         {
             foreach (XmlNode nodeElem in nodeList)
             {
@@ -337,9 +339,9 @@ namespace System.Security.Cryptography.Xml
             return idref;
         }
 
-        internal static string ExtractIdFromLocalUri(string uri)
+        internal static string ExtractIdFromLocalUri(string? uri)
         {
-            string idref = uri.Substring(1);
+            string idref = uri!.Substring(1);
 
             // Deal with XPointer of type #xpointer(id("ID")). Other XPointer support isn't handled here and is anyway optional
             if (idref.StartsWith("xpointer(id(", StringComparison.Ordinal))
@@ -358,8 +360,8 @@ namespace System.Security.Cryptography.Xml
         // This removes all children of an element.
         internal static void RemoveAllChildren(XmlElement inputElement)
         {
-            XmlNode child = inputElement.FirstChild;
-            XmlNode sibling = null;
+            XmlNode? child = inputElement.FirstChild;
+            XmlNode? sibling;
 
             while (child != null)
             {
@@ -376,7 +378,7 @@ namespace System.Security.Cryptography.Xml
         internal static long Pump(Stream input, Stream output)
         {
             // Use MemoryStream's WriteTo(Stream) method if possible
-            MemoryStream inputMS = input as MemoryStream;
+            MemoryStream? inputMS = input as MemoryStream;
             if (inputMS != null && inputMS.Position == 0)
             {
                 inputMS.WriteTo(output);
@@ -418,7 +420,7 @@ namespace System.Security.Cryptography.Xml
             return set;
         }
 
-        internal static string EscapeWhitespaceData(string data)
+        internal static string EscapeWhitespaceData(string? data)
         {
             StringBuilder sb = new StringBuilder();
             sb.Append(data);
@@ -426,7 +428,7 @@ namespace System.Security.Cryptography.Xml
             return sb.ToString();
         }
 
-        internal static string EscapeTextData(string data)
+        internal static string EscapeTextData(string? data)
         {
             StringBuilder sb = new StringBuilder();
             sb.Append(data);
@@ -455,7 +457,7 @@ namespace System.Security.Cryptography.Xml
             return sb.ToString();
         }
 
-        internal static XmlDocument GetOwnerDocument(XmlNodeList nodeList)
+        internal static XmlDocument? GetOwnerDocument(XmlNodeList nodeList)
         {
             foreach (XmlNode node in nodeList)
             {
@@ -465,7 +467,7 @@ namespace System.Security.Cryptography.Xml
             return null;
         }
 
-        internal static void AddNamespaces(XmlElement elem, CanonicalXmlNodeList namespaces)
+        internal static void AddNamespaces(XmlElement elem, CanonicalXmlNodeList? namespaces)
         {
             if (namespaces != null)
             {
@@ -496,18 +498,18 @@ namespace System.Security.Cryptography.Xml
         }
 
         // This method gets the attributes that should be propagated
-        internal static CanonicalXmlNodeList GetPropagatedAttributes(XmlElement elem)
+        internal static CanonicalXmlNodeList? GetPropagatedAttributes(XmlElement? elem)
         {
             if (elem == null)
                 return null;
 
             CanonicalXmlNodeList namespaces = new CanonicalXmlNodeList();
-            XmlNode ancestorNode = elem;
+            XmlNode? ancestorNode = elem;
             bool bDefNamespaceToAdd = true;
 
             while (ancestorNode != null)
             {
-                XmlElement ancestorElement = ancestorNode as XmlElement;
+                XmlElement? ancestorElement = ancestorNode as XmlElement;
                 if (ancestorElement == null)
                 {
                     ancestorNode = ancestorNode.ParentNode;
@@ -617,11 +619,11 @@ namespace System.Security.Cryptography.Xml
         }
 
         // Mimic the behavior of the X509IssuerSerial constructor with null and empty checks
-        internal static X509IssuerSerial CreateX509IssuerSerial(string issuerName, string serialNumber)
+        internal static X509IssuerSerial CreateX509IssuerSerial(string? issuerName, string? serialNumber)
         {
-            if (issuerName == null || issuerName.Length == 0)
+            if (string.IsNullOrEmpty(issuerName))
                 throw new ArgumentException(SR.Arg_EmptyOrNullString, nameof(issuerName));
-            if (serialNumber == null || serialNumber.Length == 0)
+            if (string.IsNullOrEmpty(serialNumber))
                 throw new ArgumentException(SR.Arg_EmptyOrNullString, nameof(serialNumber));
 
             return new X509IssuerSerial()
@@ -634,7 +636,7 @@ namespace System.Security.Cryptography.Xml
         internal static X509Certificate2Collection BuildBagOfCerts(KeyInfoX509Data keyInfoX509Data, CertUsageType certUsageType)
         {
             X509Certificate2Collection collection = new X509Certificate2Collection();
-            ArrayList decryptionIssuerSerials = (certUsageType == CertUsageType.Decryption ? new ArrayList() : null);
+            ArrayList? decryptionIssuerSerials = (certUsageType == CertUsageType.Decryption ? new ArrayList() : null);
             if (keyInfoX509Data.Certificates != null)
             {
                 foreach (X509Certificate2 certificate in keyInfoX509Data.Certificates)
@@ -645,7 +647,7 @@ namespace System.Security.Cryptography.Xml
                             collection.Add(certificate);
                             break;
                         case CertUsageType.Decryption:
-                            decryptionIssuerSerials.Add(CreateX509IssuerSerial(certificate.IssuerName.Name, certificate.SerialNumber));
+                            decryptionIssuerSerials!.Add(CreateX509IssuerSerial(certificate.IssuerName.Name, certificate.SerialNumber));
                             break;
                     }
                 }
@@ -666,7 +668,7 @@ namespace System.Security.Cryptography.Xml
             {
                 if (stores[index] != null)
                 {
-                    X509Certificate2Collection filters = null;
+                    X509Certificate2Collection? filters = null;
                     // We don't care if we can't open the store.
                     try
                     {
@@ -748,9 +750,9 @@ namespace System.Security.Cryptography.Xml
             return false;
         }
 
-        internal static AsymmetricAlgorithm GetAnyPublicKey(X509Certificate2 certificate)
+        internal static AsymmetricAlgorithm? GetAnyPublicKey(X509Certificate2 certificate)
         {
-            AsymmetricAlgorithm algorithm = (AsymmetricAlgorithm)certificate.GetRSAPublicKey() ?? certificate.GetECDsaPublicKey();
+            AsymmetricAlgorithm? algorithm = (AsymmetricAlgorithm?)certificate.GetRSAPublicKey() ?? certificate.GetECDsaPublicKey();
 
 #if NETCOREAPP
             if (algorithm is null && !OperatingSystem.IsTvOS() && !OperatingSystem.IsIOS())
