@@ -10,78 +10,77 @@ using Xunit;
 
 namespace System.Security.Cryptography.Tests
 {
-    public class HmacSha3_384Tests : HmacTests<HmacSha3_384Tests.Traits>
+    public class HmacSha3_512Tests : HmacTests<HmacSha3_512Tests.Traits>
     {
         public sealed class Traits : IHmacTrait
         {
-            public static bool IsSupported => HMACSHA3_384.IsSupported;
-            public static int HashSizeInBytes => HMACSHA3_384.HashSizeInBytes;
+            public static bool IsSupported => HMACSHA3_512.IsSupported;
+            public static int HashSizeInBytes => HMACSHA3_512.HashSizeInBytes;
         }
 
-        protected override int BlockSize => 104;
-        protected override int MacSize => HMACSHA3_384.HashSizeInBytes;
+        protected override int BlockSize => 72;
+        protected override int MacSize => HMACSHA3_512.HashSizeInBytes;
 
-        protected override HMAC Create() => new HMACSHA3_384();
-        protected override HMAC Create(byte[] key) => new HMACSHA3_384(key);
-        protected override HashAlgorithm CreateHashAlgorithm() => SHA3_384.Create();
+        protected override HMAC Create() => new HMACSHA3_512();
+        protected override HMAC Create(byte[] key) => new HMACSHA3_512(key);
+        protected override HashAlgorithm CreateHashAlgorithm() => SHA3_512.Create();
         protected override byte[] HashDataOneShot(byte[] key, byte[] source) =>
-            HMACSHA3_384.HashData(key, source);
+            HMACSHA3_512.HashData(key, source);
 
         protected override byte[] HashDataOneShot(ReadOnlySpan<byte> key, ReadOnlySpan<byte> source) =>
-            HMACSHA3_384.HashData(key, source);
+            HMACSHA3_512.HashData(key, source);
 
         protected override int HashDataOneShot(ReadOnlySpan<byte> key, ReadOnlySpan<byte> source, Span<byte> destination) =>
-            HMACSHA3_384.HashData(key, source, destination);
+            HMACSHA3_512.HashData(key, source, destination);
 
         protected override bool TryHashDataOneShot(ReadOnlySpan<byte> key, ReadOnlySpan<byte> source, Span<byte> destination, out int written) =>
-            HMACSHA3_384.TryHashData(key, source, destination, out written);
+            HMACSHA3_512.TryHashData(key, source, destination, out written);
 
         protected override byte[] HashDataOneShot(ReadOnlySpan<byte> key, Stream source) =>
-            HMACSHA3_384.HashData(key, source);
+            HMACSHA3_512.HashData(key, source);
 
         protected override byte[] HashDataOneShot(byte[] key, Stream source) =>
-            HMACSHA3_384.HashData(key, source);
+            HMACSHA3_512.HashData(key, source);
 
         protected override int HashDataOneShot(ReadOnlySpan<byte> key, Stream source, Span<byte> destination) =>
-            HMACSHA3_384.HashData(key, source, destination);
+            HMACSHA3_512.HashData(key, source, destination);
 
         protected override ValueTask<int> HashDataOneShotAsync(
             ReadOnlyMemory<byte> key,
             Stream source,
             Memory<byte> destination,
-            CancellationToken cancellationToken) => HMACSHA3_384.HashDataAsync(key, source, destination, cancellationToken);
+            CancellationToken cancellationToken) => HMACSHA3_512.HashDataAsync(key, source, destination, cancellationToken);
 
         protected override ValueTask<byte[]> HashDataOneShotAsync(
             ReadOnlyMemory<byte> key,
             Stream source,
-            CancellationToken cancellationToken) => HMACSHA3_384.HashDataAsync(key, source, cancellationToken);
+            CancellationToken cancellationToken) => HMACSHA3_512.HashDataAsync(key, source, cancellationToken);
 
         protected override ValueTask<byte[]> HashDataOneShotAsync(
             byte[] key,
             Stream source,
-            CancellationToken cancellationToken) => HMACSHA3_384.HashDataAsync(key, source, cancellationToken);
+            CancellationToken cancellationToken) => HMACSHA3_512.HashDataAsync(key, source, cancellationToken);
 
         private static readonly byte[][] s_testKeys = new byte[][]
         {
-            // From: https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/HMAC_SHA3-384.pdf
+            // From: https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/HMAC_SHA3-512.pdf
             null,
             ByteUtils.HexToByteArray(
-                "000102030405060708090a0b0c0d0e0f1011121314151617" +
-                "18191a1b1c1d1e1f202122232425262728292a2b2c2d2e2f"),
+                "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f" +
+                "202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f"),
             ByteUtils.HexToByteArray(
                 "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f" +
                 "202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f" +
-                "404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f" +
-                "6061626364656667"),
+                "4041424344454647"),
             ByteUtils.HexToByteArray(
                 "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f" +
                 "202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f" +
                 "404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f" +
                 "606162636465666768696a6b6c6d6e6f707172737475767778797a7b7c7d7e7f" +
-                "808182838485868788898a8b8c8d8e8f9091929394959697"),
+                "8081828384858687"),
             ByteUtils.HexToByteArray(
                 "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f" +
-                "202122232425262728292a2b2c2d2e2f"),
+                "202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f"),
         };
 
         private static readonly byte[][] s_testData = new byte[][]
@@ -97,25 +96,25 @@ namespace System.Security.Cryptography.Tests
         {
             null,
             ByteUtils.HexToByteArray(
-                "d588a3c51f3f2d906e8298c1199aa8ff6296218127f6b38a" +
-                "90b6afe2c5617725bc99987f79b22a557b6520db710b7f42"),
+                "4efd629d6c71bf86162658f29943b1c308ce27cdfa6db0d9c3ce81763f9cbce5" +
+                "f7ebe9868031db1a8f8eb7b6b95e5c5e3f657a8996c86a2f6527e307f0213196"),
             ByteUtils.HexToByteArray(
-                "a27d24b592e8c8cbf6d4ce6fc5bf62d8fc98bf2d486640d9" +
-                "eb8099e24047837f5f3bffbe92dcce90b4ed5b1e7e44fa90"),
+                "544e257ea2a3e5ea19a590e6a24b724ce6327757723fe2751b75bf007d80f6b3" +
+                "60744bf1b7a88ea585f9765b47911976d3191cf83c039f5ffab0d29cc9d9b6da"),
             ByteUtils.HexToByteArray(
-                "e5ae4c739f455279368ebf36d4f5354c95aa184c899d3870" +
-                "e460ebc288ef1f9470053f73f7c6da2a71bcaec38ce7d6ac"),
-            ByteUtils.HexToByteArray("25f4bf53606e91af79d24a4bb1fd6aecd44414a30c8ebb0a"), // Truncated
+                "5f464f5e5b7848e3885e49b2c385f0694985d0e38966242dc4a5fe3fea4b37d4" +
+                "6b65ceced5dcf59438dd840bab22269f0ba7febdb9fcf74602a35666b2a32915"),
+            ByteUtils.HexToByteArray("7bb06d859257b25ce73ca700df34c5cbef5c898bac91029e0b27975d4e526a08"), // Truncated
         };
 
 
-        public HmacSha3_384Tests() : base(s_testKeys, s_testData, s_testMacs)
+        public HmacSha3_512Tests() : base(s_testKeys, s_testData, s_testMacs)
         {
         }
 
         [ConditionalTheory(nameof(IsSupported))]
         [MemberData(nameof(TestCaseIds))]
-        public void HmacSha3_384_VerifyTestCases(int caseId)
+        public void HmacSha3_512_VerifyTestCases(int caseId)
         {
             VerifyHmac(caseId, s_testMacs[caseId]);
         }
@@ -132,110 +131,117 @@ namespace System.Security.Cryptography.Tests
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public void HmacSha3_384_Rfc2104_2()
+        public void HmacSha3_512_Rfc2104_2()
         {
             VerifyHmacRfc2104_2();
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public void HmacSha3_384_ThrowsArgumentNullForNullConstructorKey()
+        public void HmacSha3_512_ThrowsArgumentNullForNullConstructorKey()
         {
-            AssertExtensions.Throws<ArgumentNullException>("key", () => new HMACSHA3_384(null));
+            AssertExtensions.Throws<ArgumentNullException>("key", () => new HMACSHA3_512(null));
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public void HmacSha3_384_EmptyKey()
+        public void HmacSha3_512_EmptyKey()
         {
             VerifyRepeating(
                 input: "Crypto is fun!",
                 1,
                 hexKey: "",
-                output: "16C079F7D15505E9E541421E63C432A063F39C1E3E953E6DC7B8A81FE5620AFFA430C3E6BE6A0F605755C7C5EE4E347E");
+                output: "8F1658BB962C7048A50BEA174FA7697596F3F5F127228EEA64589DFFB0C1A07C" +
+                        "98792648C97886B3DD9E63AB962581C67DA5EE04F2B15263555B1796782CB556");
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public void HmacSha3_384_Stream_MultipleOf4096()
+        public void HmacSha3_512_Stream_MultipleOf4096()
         {
             // Verfied with:
-            // for _ in {1..1024}; do echo -n "0102030405060708"; done | openssl sha3-384 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
+            // for _ in {1..1024}; do echo -n "0102030405060708"; done | openssl sha3-512 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
             VerifyRepeating(
                 input: "0102030405060708",
                 1024,
                 hexKey: "000102030405060708090A0B0C0D0E0F",
-                output: "17D03C8153AF1719F5829C8CBF328D4200900ED1AB038A399B4F256A490BD4D2AB311C71D2ED0C20ABA96E57768CCA6E");
+                output: "43A635D00606EFB4797B0B50B3C2CCAACDC8C2DA38D1369EA49CDD93EDE27824" +
+                        "317D9014C429DB18E5A6BFD811F7B484922471085F17ED31F6A7EB4E07BFFA97");
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public void HmacSha3_384_Stream_NotMultipleOf4096()
+        public void HmacSha3_512_Stream_NotMultipleOf4096()
         {
             // Verfied with:
-            // for _ in {1..1025}; do echo -n "0102030405060708"; done | openssl sha3-384 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
+            // for _ in {1..1025}; do echo -n "0102030405060708"; done | openssl sha3-512 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
             VerifyRepeating(
                 input: "0102030405060708",
                 1025,
                 hexKey: "000102030405060708090A0B0C0D0E0F",
-                output: "5360719A6A9DFEB1143C2866A7F72EA29404C3DBF37F244A0497F400DA126B2728118863454288F26E3796BE72238958");
+                output: "90D378A7546A66804E6C0ADF3203D3836244FD8CF628294E7F3AD95539EDF6D9" +
+                        "E86DECE850D50DE76386CB293FA832778C7D6607A4F00AD666DA3EFFD6143E70");
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public void HmacSha3_384_Stream_Empty()
+        public void HmacSha3_512_Stream_Empty()
         {
             // Verfied with:
-            // echo -n "" | openssl sha3-384 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
+            // echo -n "" | openssl sha3-512 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
             VerifyRepeating(
                 input: "",
                 0,
                 hexKey: "000102030405060708090A0B0C0D0E0F",
-                output: "5B0196779BDAF859E99869A63C9FDF3821E9100A370B5E9B88F76B9DA87410F99846E7DBB4F8A69368C5C5A834B3128D");
+                output: "908D1BE1A2CC5CC4B8C62E98D09AB6E967529FCB24F4177CB94CB072F5968D01" +
+                        "CA58633748DC80D4615E3D21228BB3A5F535FA1CB963DF463CC28ABAF1A9B2D1");
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public async Task HmacSha3_384_Stream_MultipleOf4096_Async()
+        public async Task HmacSha3_512_Stream_MultipleOf4096_Async()
         {
             // Verfied with:
-            // for _ in {1..1024}; do echo -n "0102030405060708"; done | openssl sha3-384 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
+            // for _ in {1..1024}; do echo -n "0102030405060708"; done | openssl sha3-512 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
             await VerifyRepeatingAsync(
                 input: "0102030405060708",
                 1024,
                 hexKey: "000102030405060708090A0B0C0D0E0F",
-                output: "17D03C8153AF1719F5829C8CBF328D4200900ED1AB038A399B4F256A490BD4D2AB311C71D2ED0C20ABA96E57768CCA6E");
+                output: "43A635D00606EFB4797B0B50B3C2CCAACDC8C2DA38D1369EA49CDD93EDE27824" +
+                        "317D9014C429DB18E5A6BFD811F7B484922471085F17ED31F6A7EB4E07BFFA97");
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public async Task HmacSha3_384_Stream_NotMultipleOf4096_Async()
+        public async Task HmacSha3_512_Stream_NotMultipleOf4096_Async()
         {
             // Verfied with:
-            // for _ in {1..1025}; do echo -n "0102030405060708"; done | openssl sha3-384 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
+            // for _ in {1..1025}; do echo -n "0102030405060708"; done | openssl sha3-512 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
             await VerifyRepeatingAsync(
                 input: "0102030405060708",
                 1025,
                 hexKey: "000102030405060708090A0B0C0D0E0F",
-                output: "5360719A6A9DFEB1143C2866A7F72EA29404C3DBF37F244A0497F400DA126B2728118863454288F26E3796BE72238958");
+                output: "90D378A7546A66804E6C0ADF3203D3836244FD8CF628294E7F3AD95539EDF6D9" +
+                        "E86DECE850D50DE76386CB293FA832778C7D6607A4F00AD666DA3EFFD6143E70");
         }
 
         [ConditionalFact(nameof(IsSupported))]
-        public async Task HmacSha3_384_Stream_Empty_Async()
+        public async Task HmacSha3_512_Stream_Empty_Async()
         {
             // Verfied with:
-            // echo -n "" | openssl sha3-384 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
+            // echo -n "" | openssl sha3-512 -hex -mac HMAC -macopt hexkey:000102030405060708090A0B0C0D0E0F
             await VerifyRepeatingAsync(
                 input: "",
                 0,
                 hexKey: "000102030405060708090A0B0C0D0E0F",
-                output: "5B0196779BDAF859E99869A63C9FDF3821E9100A370B5E9B88F76B9DA87410F99846E7DBB4F8A69368C5C5A834B3128D");
+                output: "908D1BE1A2CC5CC4B8C62E98D09AB6E967529FCB24F4177CB94CB072F5968D01" +
+                        "CA58633748DC80D4615E3D21228BB3A5F535FA1CB963DF463CC28ABAF1A9B2D1");
         }
 
         [Fact]
-        public void HmacSha3_384_HashSizes()
+        public void HmacSha3_256_HashSizes()
         {
-            Assert.Equal(384, HMACSHA3_384.HashSizeInBits);
-            Assert.Equal(48, HMACSHA3_384.HashSizeInBytes);
+            Assert.Equal(512, HMACSHA3_512.HashSizeInBits);
+            Assert.Equal(64, HMACSHA3_512.HashSizeInBytes);
         }
 
         [Fact]
-        public void HmacSha3_384_IsSupported_AgreesWithPlatformVersion()
+        public void HmacSha3_512_IsSupported_AgreesWithPlatformVersion()
         {
-            Assert.Equal(PlatformDetection.SupportsSha3, HMACSHA3_384.IsSupported);
+            Assert.Equal(PlatformDetection.SupportsSha3, HMACSHA3_512.IsSupported);
         }
     }
 }
