@@ -3,7 +3,7 @@
 
 namespace System.Security.Permissions
 {
-#if NET5_0_OR_GREATER
+#if NETCOREAPP
     [Obsolete(Obsoletions.CodeAccessSecurityMessage, DiagnosticId = Obsoletions.CodeAccessSecurityDiagId, UrlFormat = Obsoletions.SharedUrlFormat)]
 #endif
     public sealed partial class SecurityPermission : CodeAccessPermission, IUnrestrictedPermission
