@@ -6,7 +6,7 @@ using System.Security.Permissions;
 
 namespace System.Transactions
 {
-#if NET5_0_OR_GREATER
+#if NETCOREAPP
     [Obsolete(Obsoletions.CodeAccessSecurityMessage, DiagnosticId = Obsoletions.CodeAccessSecurityDiagId, UrlFormat = Obsoletions.SharedUrlFormat)]
 #endif
     public sealed class DistributedTransactionPermission : CodeAccessPermission, IUnrestrictedPermission
