@@ -4,12 +4,136 @@
 // Changes to this file must follow the https://aka.ms/api-review process.
 // ------------------------------------------------------------------------------
 
+#if !BUILDING_CORELIB_REFERENCE
+namespace System.Threading
+{
+    public partial class Barrier : System.IDisposable
+    {
+        public Barrier(int participantCount) { }
+        public Barrier(int participantCount, System.Action<System.Threading.Barrier>? postPhaseAction) { }
+        public long CurrentPhaseNumber { get { throw null; } }
+        public int ParticipantCount { get { throw null; } }
+        public int ParticipantsRemaining { get { throw null; } }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public long AddParticipant() { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public long AddParticipants(int participantCount) { throw null; }
+        public void Dispose() { }
+        protected virtual void Dispose(bool disposing) { }
+        public void RemoveParticipant() { }
+        public void RemoveParticipants(int participantCount) { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void SignalAndWait() { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool SignalAndWait(int millisecondsTimeout) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool SignalAndWait(int millisecondsTimeout, System.Threading.CancellationToken cancellationToken) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void SignalAndWait(System.Threading.CancellationToken cancellationToken) { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool SignalAndWait(System.TimeSpan timeout) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool SignalAndWait(System.TimeSpan timeout, System.Threading.CancellationToken cancellationToken) { throw null; }
+    }
+    public partial class BarrierPostPhaseException : System.Exception
+    {
+        public BarrierPostPhaseException() { }
+        public BarrierPostPhaseException(System.Exception? innerException) { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
+        protected BarrierPostPhaseException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
+        public BarrierPostPhaseException(string? message) { }
+        public BarrierPostPhaseException(string? message, System.Exception? innerException) { }
+    }
+    public partial class CountdownEvent : System.IDisposable
+    {
+        public CountdownEvent(int initialCount) { }
+        public int CurrentCount { get { throw null; } }
+        public int InitialCount { get { throw null; } }
+        public bool IsSet { get { throw null; } }
+        public System.Threading.WaitHandle WaitHandle { get { throw null; } }
+        public void AddCount() { }
+        public void AddCount(int signalCount) { }
+        public void Dispose() { }
+        protected virtual void Dispose(bool disposing) { }
+        public void Reset() { }
+        public void Reset(int count) { }
+        public bool Signal() { throw null; }
+        public bool Signal(int signalCount) { throw null; }
+        public bool TryAddCount() { throw null; }
+        public bool TryAddCount(int signalCount) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void Wait() { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool Wait(int millisecondsTimeout) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool Wait(int millisecondsTimeout, System.Threading.CancellationToken cancellationToken) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void Wait(System.Threading.CancellationToken cancellationToken) { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool Wait(System.TimeSpan timeout) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public bool Wait(System.TimeSpan timeout, System.Threading.CancellationToken cancellationToken) { throw null; }
+    }
+    public partial class HostExecutionContext : System.IDisposable
+    {
+        public HostExecutionContext() { }
+        public HostExecutionContext(object? state) { }
+        protected internal object? State { get { throw null; } set { } }
+        public virtual System.Threading.HostExecutionContext CreateCopy() { throw null; }
+        public void Dispose() { }
+        public virtual void Dispose(bool disposing) { }
+    }
+    public partial class HostExecutionContextManager
+    {
+        public HostExecutionContextManager() { }
+        public virtual System.Threading.HostExecutionContext? Capture() { throw null; }
+        public virtual void Revert(object previousState) { }
+        public virtual object SetHostExecutionContext(System.Threading.HostExecutionContext hostExecutionContext) { throw null; }
+    }
+    public partial struct LockCookie : System.IEquatable<System.Threading.LockCookie>
+    {
+        private int _dummyPrimitive;
+        public override bool Equals([System.Diagnostics.CodeAnalysis.NotNullWhenAttribute(true)] object? obj) { throw null; }
+        public bool Equals(System.Threading.LockCookie obj) { throw null; }
+        public override int GetHashCode() { throw null; }
+        public static bool operator ==(System.Threading.LockCookie a, System.Threading.LockCookie b) { throw null; }
+        public static bool operator !=(System.Threading.LockCookie a, System.Threading.LockCookie b) { throw null; }
+    }
+    public sealed partial class ReaderWriterLock : System.Runtime.ConstrainedExecution.CriticalFinalizerObject
+    {
+        public ReaderWriterLock() { }
+        public bool IsReaderLockHeld { get { throw null; } }
+        public bool IsWriterLockHeld { get { throw null; } }
+        public int WriterSeqNum { get { throw null; } }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void AcquireReaderLock(int millisecondsTimeout) { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void AcquireReaderLock(System.TimeSpan timeout) { }
+        public void AcquireWriterLock(int millisecondsTimeout) { }
+        public void AcquireWriterLock(System.TimeSpan timeout) { }
+        public bool AnyWritersSince(int seqNum) { throw null; }
+        public void DowngradeFromWriterLock(ref System.Threading.LockCookie lockCookie) { }
+        public System.Threading.LockCookie ReleaseLock() { throw null; }
+        public void ReleaseReaderLock() { }
+        public void ReleaseWriterLock() { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public void RestoreLock(ref System.Threading.LockCookie lockCookie) { }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public System.Threading.LockCookie UpgradeToWriterLock(int millisecondsTimeout) { throw null; }
+        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
+        public System.Threading.LockCookie UpgradeToWriterLock(System.TimeSpan timeout) { throw null; }
+    }
+}
+#endif // !BUILDING_CORELIB_REFERENCE
 namespace System.Threading
 {
     public partial class AbandonedMutexException : System.SystemException
     {
         public AbandonedMutexException() { }
         public AbandonedMutexException(int location, System.Threading.WaitHandle? handle) { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
         protected AbandonedMutexException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
         public AbandonedMutexException(string? message) { }
         public AbandonedMutexException(string? message, System.Exception? inner) { }
@@ -50,77 +174,7 @@ namespace System.Threading
     {
         public AutoResetEvent(bool initialState) : base (default(bool), default(System.Threading.EventResetMode)) { }
     }
-#if !BUILDING_CORELIB_REFERENCE
-    public partial class Barrier : System.IDisposable
-    {
-        public Barrier(int participantCount) { }
-        public Barrier(int participantCount, System.Action<System.Threading.Barrier>? postPhaseAction) { }
-        public long CurrentPhaseNumber { get { throw null; } }
-        public int ParticipantCount { get { throw null; } }
-        public int ParticipantsRemaining { get { throw null; } }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public long AddParticipant() { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public long AddParticipants(int participantCount) { throw null; }
-        public void Dispose() { }
-        protected virtual void Dispose(bool disposing) { }
-        public void RemoveParticipant() { }
-        public void RemoveParticipants(int participantCount) { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void SignalAndWait() { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool SignalAndWait(int millisecondsTimeout) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool SignalAndWait(int millisecondsTimeout, System.Threading.CancellationToken cancellationToken) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void SignalAndWait(System.Threading.CancellationToken cancellationToken) { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool SignalAndWait(System.TimeSpan timeout) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool SignalAndWait(System.TimeSpan timeout, System.Threading.CancellationToken cancellationToken) { throw null; }
-    }
-    public partial class BarrierPostPhaseException : System.Exception
-    {
-        public BarrierPostPhaseException() { }
-        public BarrierPostPhaseException(System.Exception? innerException) { }
-        protected BarrierPostPhaseException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
-        public BarrierPostPhaseException(string? message) { }
-        public BarrierPostPhaseException(string? message, System.Exception? innerException) { }
-    }
-#endif
     public delegate void ContextCallback(object? state);
-#if !BUILDING_CORELIB_REFERENCE
-    public partial class CountdownEvent : System.IDisposable
-    {
-        public CountdownEvent(int initialCount) { }
-        public int CurrentCount { get { throw null; } }
-        public int InitialCount { get { throw null; } }
-        public bool IsSet { get { throw null; } }
-        public System.Threading.WaitHandle WaitHandle { get { throw null; } }
-        public void AddCount() { }
-        public void AddCount(int signalCount) { }
-        public void Dispose() { }
-        protected virtual void Dispose(bool disposing) { }
-        public void Reset() { }
-        public void Reset(int count) { }
-        public bool Signal() { throw null; }
-        public bool Signal(int signalCount) { throw null; }
-        public bool TryAddCount() { throw null; }
-        public bool TryAddCount(int signalCount) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void Wait() { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool Wait(int millisecondsTimeout) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool Wait(int millisecondsTimeout, System.Threading.CancellationToken cancellationToken) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void Wait(System.Threading.CancellationToken cancellationToken) { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool Wait(System.TimeSpan timeout) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public bool Wait(System.TimeSpan timeout, System.Threading.CancellationToken cancellationToken) { throw null; }
-    }
-#endif
     public enum EventResetMode
     {
         AutoReset = 0,
@@ -144,6 +198,8 @@ namespace System.Threading
         public static System.Threading.ExecutionContext? Capture() { throw null; }
         public System.Threading.ExecutionContext CreateCopy() { throw null; }
         public void Dispose() { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
         public void GetObjectData(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
         public static bool IsFlowSuppressed() { throw null; }
         public static void Restore(System.Threading.ExecutionContext executionContext) { }
@@ -151,24 +207,6 @@ namespace System.Threading
         public static void Run(System.Threading.ExecutionContext executionContext, System.Threading.ContextCallback callback, object? state) { }
         public static System.Threading.AsyncFlowControl SuppressFlow() { throw null; }
     }
-#if !BUILDING_CORELIB_REFERENCE
-    public partial class HostExecutionContext : System.IDisposable
-    {
-        public HostExecutionContext() { }
-        public HostExecutionContext(object? state) { }
-        protected internal object? State { get { throw null; } set { } }
-        public virtual System.Threading.HostExecutionContext CreateCopy() { throw null; }
-        public void Dispose() { }
-        public virtual void Dispose(bool disposing) { }
-    }
-    public partial class HostExecutionContextManager
-    {
-        public HostExecutionContextManager() { }
-        public virtual System.Threading.HostExecutionContext? Capture() { throw null; }
-        public virtual void Revert(object previousState) { }
-        public virtual object SetHostExecutionContext(System.Threading.HostExecutionContext hostExecutionContext) { throw null; }
-    }
-#endif
     public static partial class Interlocked
     {
         public static int Add(ref int location1, int value) { throw null; }
@@ -187,6 +225,8 @@ namespace System.Threading
         public static int CompareExchange(ref int location1, int value, int comparand) { throw null; }
         public static long CompareExchange(ref long location1, long value, long comparand) { throw null; }
         public static System.IntPtr CompareExchange(ref System.IntPtr location1, System.IntPtr value, System.IntPtr comparand) { throw null; }
+        [System.CLSCompliantAttribute(false)]
+        public static System.UIntPtr CompareExchange(ref System.UIntPtr location1, System.UIntPtr value, System.UIntPtr comparand) { throw null; }
         [return: System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("location1")]
         public static object? CompareExchange(ref object? location1, object? value, object? comparand) { throw null; }
         public static float CompareExchange(ref float location1, float value, float comparand) { throw null; }
@@ -206,6 +246,8 @@ namespace System.Threading
         public static int Exchange(ref int location1, int value) { throw null; }
         public static long Exchange(ref long location1, long value) { throw null; }
         public static System.IntPtr Exchange(ref System.IntPtr location1, System.IntPtr value) { throw null; }
+        [System.CLSCompliantAttribute(false)]
+        public static System.UIntPtr Exchange(ref System.UIntPtr location1, System.UIntPtr value) { throw null; }
         [return: System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("location1")]
         public static object? Exchange([System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("value")] ref object? location1, object? value) { throw null; }
         public static float Exchange(ref float location1, float value) { throw null; }
@@ -229,9 +271,9 @@ namespace System.Threading
         public static uint Or(ref uint location1, uint value) { throw null; }
         [System.CLSCompliantAttribute(false)]
         public static ulong Or(ref ulong location1, ulong value) { throw null; }
-        public static long Read(ref long location) { throw null; }
+        public static long Read(ref readonly long location) { throw null; }
         [System.CLSCompliantAttribute(false)]
-        public static ulong Read(ref ulong location) { throw null; }
+        public static ulong Read(ref readonly ulong location) { throw null; }
     }
     public static partial class LazyInitializer
     {
@@ -241,20 +283,11 @@ namespace System.Threading
         public static T EnsureInitialized<T>([System.Diagnostics.CodeAnalysis.NotNullAttribute] ref T? target, System.Func<T> valueFactory) where T : class { throw null; }
         public static T EnsureInitialized<T>([System.Diagnostics.CodeAnalysis.NotNullAttribute] ref T? target, [System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("syncLock")] ref object? syncLock, System.Func<T> valueFactory) where T : class { throw null; }
     }
-#if !BUILDING_CORELIB_REFERENCE
-    public partial struct LockCookie
-    {
-        private int _dummyPrimitive;
-        public override bool Equals([System.Diagnostics.CodeAnalysis.NotNullWhenAttribute(true)] object? obj) { throw null; }
-        public bool Equals(System.Threading.LockCookie obj) { throw null; }
-        public override int GetHashCode() { throw null; }
-        public static bool operator ==(System.Threading.LockCookie a, System.Threading.LockCookie b) { throw null; }
-        public static bool operator !=(System.Threading.LockCookie a, System.Threading.LockCookie b) { throw null; }
-    }
-#endif
     public partial class LockRecursionException : System.Exception
     {
         public LockRecursionException() { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
         protected LockRecursionException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
         public LockRecursionException(string? message) { }
         public LockRecursionException(string? message, System.Exception? innerException) { }
@@ -329,32 +362,6 @@ namespace System.Threading
         public void ReleaseMutex() { }
         public static bool TryOpenExisting(string name, [System.Diagnostics.CodeAnalysis.NotNullWhenAttribute(true)] out System.Threading.Mutex? result) { throw null; }
     }
-#if !BUILDING_CORELIB_REFERENCE
-    public sealed partial class ReaderWriterLock : System.Runtime.ConstrainedExecution.CriticalFinalizerObject
-    {
-        public ReaderWriterLock() { }
-        public bool IsReaderLockHeld { get { throw null; } }
-        public bool IsWriterLockHeld { get { throw null; } }
-        public int WriterSeqNum { get { throw null; } }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void AcquireReaderLock(int millisecondsTimeout) { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void AcquireReaderLock(System.TimeSpan timeout) { }
-        public void AcquireWriterLock(int millisecondsTimeout) { }
-        public void AcquireWriterLock(System.TimeSpan timeout) { }
-        public bool AnyWritersSince(int seqNum) { throw null; }
-        public void DowngradeFromWriterLock(ref System.Threading.LockCookie lockCookie) { }
-        public System.Threading.LockCookie ReleaseLock() { throw null; }
-        public void ReleaseReaderLock() { }
-        public void ReleaseWriterLock() { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public void RestoreLock(ref System.Threading.LockCookie lockCookie) { }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public System.Threading.LockCookie UpgradeToWriterLock(int millisecondsTimeout) { throw null; }
-        [System.Runtime.Versioning.UnsupportedOSPlatformAttribute("browser")]
-        public System.Threading.LockCookie UpgradeToWriterLock(System.TimeSpan timeout) { throw null; }
-    }
-#endif
     public partial class ReaderWriterLockSlim : System.IDisposable
     {
         public ReaderWriterLockSlim() { }
@@ -399,6 +406,8 @@ namespace System.Threading
     public partial class SemaphoreFullException : System.SystemException
     {
         public SemaphoreFullException() { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
         protected SemaphoreFullException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
         public SemaphoreFullException(string? message) { }
         public SemaphoreFullException(string? message, System.Exception? innerException) { }
@@ -479,6 +488,8 @@ namespace System.Threading
     public partial class SynchronizationLockException : System.SystemException
     {
         public SynchronizationLockException() { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
         protected SynchronizationLockException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
         public SynchronizationLockException(string? message) { }
         public SynchronizationLockException(string? message, System.Exception? innerException) { }
@@ -500,26 +511,27 @@ namespace System.Threading
     }
     public static partial class Volatile
     {
-        public static bool Read(ref bool location) { throw null; }
-        public static byte Read(ref byte location) { throw null; }
-        public static double Read(ref double location) { throw null; }
-        public static short Read(ref short location) { throw null; }
-        public static int Read(ref int location) { throw null; }
-        public static long Read(ref long location) { throw null; }
-        public static System.IntPtr Read(ref System.IntPtr location) { throw null; }
+        public static bool Read(ref readonly bool location) { throw null; }
+        public static byte Read(ref readonly byte location) { throw null; }
+        public static double Read(ref readonly double location) { throw null; }
+        public static short Read(ref readonly short location) { throw null; }
+        public static int Read(ref readonly int location) { throw null; }
+        public static long Read(ref readonly long location) { throw null; }
+        public static System.IntPtr Read(ref readonly System.IntPtr location) { throw null; }
         [System.CLSCompliantAttribute(false)]
-        public static sbyte Read(ref sbyte location) { throw null; }
-        public static float Read(ref float location) { throw null; }
+        public static sbyte Read(ref readonly sbyte location) { throw null; }
+        public static float Read(ref readonly float location) { throw null; }
         [System.CLSCompliantAttribute(false)]
-        public static ushort Read(ref ushort location) { throw null; }
+        public static ushort Read(ref readonly ushort location) { throw null; }
         [System.CLSCompliantAttribute(false)]
-        public static uint Read(ref uint location) { throw null; }
+        public static uint Read(ref readonly uint location) { throw null; }
         [System.CLSCompliantAttribute(false)]
-        public static ulong Read(ref ulong location) { throw null; }
+        public static ulong Read(ref readonly ulong location) { throw null; }
         [System.CLSCompliantAttribute(false)]
-        public static System.UIntPtr Read(ref System.UIntPtr location) { throw null; }
+        public static System.UIntPtr Read(ref readonly System.UIntPtr location) { throw null; }
         [return: System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("location")]
-        public static T Read<T>([System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("location")] ref T location) where T : class? { throw null; }
+        public static T Read<T>([System.Diagnostics.CodeAnalysis.NotNullIfNotNullAttribute("location")] ref readonly T location) where T : class? { throw null; }
+
         public static void Write(ref bool location, bool value) { }
         public static void Write(ref byte location, byte value) { }
         public static void Write(ref double location, double value) { }
@@ -543,6 +555,8 @@ namespace System.Threading
     public partial class WaitHandleCannotBeOpenedException : System.ApplicationException
     {
         public WaitHandleCannotBeOpenedException() { }
+        [System.ObsoleteAttribute("This API supports obsolete formatter-based serialization. It should not be called or extended by application code.", DiagnosticId = "SYSLIB0051", UrlFormat = "https://aka.ms/dotnet-warnings/{0}")]
+        [System.ComponentModel.EditorBrowsableAttribute(System.ComponentModel.EditorBrowsableState.Never)]
         protected WaitHandleCannotBeOpenedException(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context) { }
         public WaitHandleCannotBeOpenedException(string? message) { }
         public WaitHandleCannotBeOpenedException(string? message, System.Exception? innerException) { }
