@@ -104,18 +104,13 @@ namespace System.Threading.Tests
         }
 
         [Fact]
-        public static async Task CaptureAndRunOnFlowSupressedContext()
+        public static async Task CaptureAndRunOnFlowSuppressedContext()
         {
-            ExecutionContext.SuppressFlow();
-            try
+            using (ExecutionContext.SuppressFlow())
             {
                 ExecutionContext ec = ExecutionContext.Capture();
                 Assert.Throws<InvalidOperationException>(() => ExecutionContext.Run(ec, _ => { }, null));
             }
-            finally
-            {
-                ExecutionContext.RestoreFlow();
-            }
         }
 
         [Fact]
@@ -598,16 +593,11 @@ namespace System.Threading.Tests
             // Check Running with the contexts captured when setting the locals
             TestCapturedExecutionContexts();
 
-            ExecutionContext.SuppressFlow();
-            try
+            using (ExecutionContext.SuppressFlow())
             {
                 // Re-check restoring, but starting with a suppressed flow
                 TestCapturedExecutionContexts();
             }
-            finally
-            {
-                ExecutionContext.RestoreFlow();
-            }
 
             // -- Local functions --
             void ValidateAsyncLocalsValuesNull()
@@ -689,7 +679,7 @@ namespace System.Threading.Tests
                 }
             }
 
-            // Synchronous function is async to create different ExectutionContexts for each set, and check async unwinding
+            // Synchronous function is async to create different ExecutionContexts for each set, and check async unwinding
             async Task SetLocalsRecursivelyAsync(int index)
             {
                 // Set AsyncLocal
