@@ -6,15 +6,12 @@ namespace System.Transactions.Configuration
     internal sealed class DefaultSettingsSection // ConfigurationSection
     {
         private static readonly DefaultSettingsSection s_section = new DefaultSettingsSection();
-        private static TimeSpan s_timeout = TimeSpan.Parse(ConfigurationStrings.DefaultTimeout);
+        private static readonly TimeSpan s_timeout = TimeSpan.Parse(ConfigurationStrings.DefaultTimeout);
 
         internal static DefaultSettingsSection GetSection() => s_section;
 
-        public string DistributedTransactionManagerName { get; set; } = ConfigurationStrings.DefaultDistributedTransactionManagerName;
+        public static string DistributedTransactionManagerName { get; set; } = ConfigurationStrings.DefaultDistributedTransactionManagerName;
 
-        public TimeSpan Timeout
-        {
-            get { return s_timeout; }
-        }
+        public static TimeSpan Timeout => s_timeout;
     }
 }
