@@ -187,7 +187,8 @@ namespace
     fx_definition_t* resolve_framework_reference(
         const fx_reference_t & fx_ref,
         const pal::string_t & oldest_requested_version,
-        const pal::string_t & dotnet_dir)
+        const pal::string_t & dotnet_dir,
+        const bool disable_multilevel_lookup)
     {
 #if defined(DEBUG)
         assert(!fx_ref.get_fx_name().empty());
@@ -202,7 +203,7 @@ namespace
             fx_ref.get_fx_name().c_str(), fx_ref.get_fx_version().c_str());
 
         std::vector<pal::string_t> hive_dir;
-        get_framework_and_sdk_locations(dotnet_dir, &hive_dir);
+        get_framework_and_sdk_locations(dotnet_dir, disable_multilevel_lookup, &hive_dir);
 
         pal::string_t selected_fx_dir;
         pal::string_t selected_fx_version;
@@ -380,7 +381,7 @@ void fx_resolver_t::update_newest_references(
 // - host_info
 //     Information about the host - mainly used to determine where to search for frameworks.
 // - override_settings
-//     Framework resolution settings which will win over anything found (settings comming from command line).
+//     Framework resolution settings which will win over anything found (settings coming from command line).
 //     Passed as fx_reference_t for simplicity, the version part of that structure is ignored.
 // - config
 //     Parsed runtime configuration to process.
@@ -404,6 +405,7 @@ void fx_resolver_t::update_newest_references(
 //     InvalidConfigFile - reading of a runtime config for some of the processed frameworks has failed.
 StatusCode fx_resolver_t::read_framework(
     const host_startup_info_t & host_info,
+    bool disable_multilevel_lookup,
     const runtime_config_t::settings_t& override_settings,
     const runtime_config_t & config,
     const fx_reference_t * effective_parent_fx_ref,
@@ -449,8 +451,8 @@ StatusCode fx_resolver_t::read_framework(
 
             m_effective_fx_references[fx_name] = new_effective_fx_ref;
 
-            // Resolve the effective framework reference against the the existing physical framework folders
-            fx_definition_t* fx = resolve_framework_reference(new_effective_fx_ref, m_oldest_fx_references[fx_name].get_fx_version(), host_info.dotnet_root);
+            // Resolve the effective framework reference against the existing physical framework folders
+            fx_definition_t* fx = resolve_framework_reference(new_effective_fx_ref, m_oldest_fx_references[fx_name].get_fx_version(), host_info.dotnet_root, disable_multilevel_lookup);
             if (fx == nullptr)
             {
                 trace::error(
@@ -459,8 +461,8 @@ StatusCode fx_resolver_t::read_framework(
                     _X("App: %s\n")
                     _X("Architecture: %s"),
                     app_display_name != nullptr ? app_display_name : host_info.host_path.c_str(),
-                    get_arch());
-                display_missing_framework_error(fx_name, new_effective_fx_ref.get_fx_version(), pal::string_t(), host_info.dotnet_root);
+                    get_current_arch_name());
+                display_missing_framework_error(fx_name, new_effective_fx_ref.get_fx_version(), pal::string_t(), host_info.dotnet_root, disable_multilevel_lookup);
                 return FrameworkMissingFailure;
             }
 
@@ -489,7 +491,7 @@ StatusCode fx_resolver_t::read_framework(
                 return StatusCode::InvalidConfigFile;
             }
 
-            rc = read_framework(host_info, override_settings, new_config, &new_effective_fx_ref, fx_definitions, app_display_name);
+            rc = read_framework(host_info, disable_multilevel_lookup, override_settings, new_config, &new_effective_fx_ref, fx_definitions, app_display_name);
             if (rc)
             {
                 break; // Error case
@@ -529,6 +531,7 @@ fx_resolver_t::fx_resolver_t()
 
 StatusCode fx_resolver_t::resolve_frameworks_for_app(
     const host_startup_info_t & host_info,
+    bool disable_multilevel_lookup,
     const runtime_config_t::settings_t& override_settings,
     const runtime_config_t & app_config,
     fx_definition_vector_t & fx_definitions,
@@ -542,7 +545,7 @@ StatusCode fx_resolver_t::resolve_frameworks_for_app(
     do
     {
         fx_definitions.resize(1); // Erase any existing frameworks for re-try
-        rc = resolver.read_framework(host_info, override_settings, app_config, /*effective_parent_fx_ref*/ nullptr,  fx_definitions, app_display_name);
+        rc = resolver.read_framework(host_info, disable_multilevel_lookup, override_settings, app_config, /*effective_parent_fx_ref*/ nullptr, fx_definitions, app_display_name);
     } while (rc == StatusCode::FrameworkCompatRetry && retry_count++ < Max_Framework_Resolve_Retries);
 
     assert(retry_count < Max_Framework_Resolve_Retries);
