@@ -30,6 +30,7 @@ file_free_func (void *object);
 
 static
 void
+DN_CALLBACK_CALLTYPE
 stack_hash_value_free_func (void *entry);
 
 static
@@ -57,7 +58,7 @@ file_write_event_to_block (
 	uint64_t capture_thread_id,
 	uint32_t sequence_number,
 	uint32_t stack_id,
-	bool is_sotred_event);
+	bool is_sorted_event);
 
 static
 bool
@@ -74,6 +75,16 @@ static
 uint32_t
 file_get_file_minimum_version (EventPipeSerializationFormat format);
 
+static
+uint32_t
+DN_CALLBACK_CALLTYPE
+stack_hash_key_hash_func (const void *key);
+
+static
+bool
+DN_CALLBACK_CALLTYPE
+stack_hash_key_equal_func (const void *key1, const void *key2);
+
 /*
  * EventPipeFile.
  */
@@ -93,15 +104,15 @@ file_fast_serialize_func (void *object, FastSerializer *fast_serializer)
 	EP_ASSERT (fast_serializer != NULL);
 
 	EventPipeFile *file = (EventPipeFile *)object;
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->file_open_system_time, sizeof (file->file_open_system_time));
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->file_open_timestamp, sizeof (file->file_open_timestamp));
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->timestamp_frequency, sizeof (file->timestamp_frequency));
+	ep_fast_serializer_write_system_time (fast_serializer, &file->file_open_system_time);
+	ep_fast_serializer_write_timestamp (fast_serializer, file->file_open_timestamp);
+	ep_fast_serializer_write_int64_t (fast_serializer, file->timestamp_frequency);
 
 	// the beginning of V3
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->pointer_size, sizeof (file->pointer_size));
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->current_process_id, sizeof (file->current_process_id));
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->number_of_processors, sizeof (file->number_of_processors));
-	ep_fast_serializer_write_buffer (fast_serializer, (const uint8_t *)&file->sampling_rate_in_ns, sizeof (file->sampling_rate_in_ns));
+	ep_fast_serializer_write_uint32_t (fast_serializer, file->pointer_size);
+	ep_fast_serializer_write_uint32_t (fast_serializer, file->current_process_id);
+	ep_fast_serializer_write_uint32_t (fast_serializer, file->number_of_processors);
+	ep_fast_serializer_write_uint32_t (fast_serializer, file->sampling_rate_in_ns);
 }
 
 static
@@ -121,6 +132,7 @@ file_vtable = {
 
 static
 void
+DN_CALLBACK_CALLTYPE
 stack_hash_value_free_func (void *entry)
 {
 	ep_stack_hash_entry_free ((StackHashEntry *)entry);
@@ -152,18 +164,18 @@ file_get_stack_id (
 	EP_ASSERT (file->stack_block != NULL);
 
 	uint32_t stack_id = 0;
-	EventPipeStackContents *stack_contents = ep_event_instance_get_stack_contents_ref (event_instance);
+	EventPipeStackContentsInstance *stack_contents = ep_event_instance_get_stack_contents_instance_ref (event_instance);
 	EventPipeStackBlock *stack_block = file->stack_block;
-	ep_rt_stack_hash_map_t *stack_hash = &file->stack_hash;
-	StackHashEntry *entry = NULL;
+	dn_umap_t *stack_hash = file->stack_hash;
 	StackHashKey key;
 	ep_stack_hash_key_init (&key, stack_contents);
-	if (!ep_rt_stack_hash_lookup (stack_hash, &key, &entry)) {
+	dn_umap_it_t found = dn_umap_find (stack_hash, &key);
+	if (dn_umap_it_end (found)) {
 		stack_id = file->stack_id_counter + 1;
 		file->stack_id_counter = stack_id;
-		entry = ep_stack_hash_entry_alloc (stack_contents, stack_id, ep_stack_hash_key_get_hash (&key));
+		StackHashEntry *entry = ep_stack_hash_entry_alloc (stack_contents, stack_id, ep_stack_hash_key_get_hash (&key));
 		if (entry) {
-			if (!ep_rt_stack_hash_add (stack_hash, ep_stack_hash_entry_get_key_ref (entry), entry))
+			if (!dn_umap_insert (stack_hash, ep_stack_hash_entry_get_key_ref (entry), entry).result)
 				ep_stack_hash_entry_free (entry);
 			entry = NULL;
 		}
@@ -177,7 +189,7 @@ file_get_stack_id (
 				EP_UNREACHABLE ("Should never fail to add event to a clear block. If we do the max size is too small.");
 		}
 	} else {
-		stack_id = ep_stack_hash_entry_get_id (entry);
+		stack_id = ep_stack_hash_entry_get_id (dn_umap_it_value_t (found, StackHashEntry *));
 	}
 
 	ep_stack_hash_key_fini (&key);
@@ -194,7 +206,9 @@ file_get_metadata_id (
 	EP_ASSERT (ep_event != NULL);
 
 	uint32_t metadata_ids;
-	if (ep_rt_metadata_labels_hash_lookup (&file->metadata_ids, ep_event, &metadata_ids)) {
+	dn_umap_it_t found = dn_umap_ptr_uint32_find (file->metadata_ids, ep_event);
+	if (!dn_umap_it_end (found)) {
+		metadata_ids = dn_umap_it_value_uint32_t (found);
 		EP_ASSERT (metadata_ids != 0);
 		return metadata_ids;
 	}
@@ -218,7 +232,7 @@ file_write_event_to_block (
 	uint64_t capture_thread_id,
 	uint32_t sequence_number,
 	uint32_t stack_id,
-	bool is_sotred_event)
+	bool is_sorted_event)
 {
 	EP_ASSERT (file != NULL);
 	EP_ASSERT (event_instance != NULL);
@@ -237,14 +251,14 @@ file_write_event_to_block (
 		block = (EventPipeEventBlockBase *)file->metadata_block;
 	}
 
-	if (ep_event_block_base_write_event (block, event_instance, capture_thread_id, sequence_number, stack_id, is_sotred_event))
+	if (ep_event_block_base_write_event (block, event_instance, capture_thread_id, sequence_number, stack_id, is_sorted_event))
 		return; // the block is not full, we added the event and continue
 
 	// we can't write this event to the current block (it's full)
 	// so we write what we have in the block to the serializer
 	ep_file_flush (file, flags);
 
-	bool result = ep_event_block_base_write_event (block, event_instance, capture_thread_id, sequence_number, stack_id, is_sotred_event);
+	bool result = ep_event_block_base_write_event (block, event_instance, capture_thread_id, sequence_number, stack_id, is_sorted_event);
 	if (!result)
 		EP_UNREACHABLE ("Should never fail to add event to a clear block. If we do the max size is too small.");
 }
@@ -260,13 +274,8 @@ file_save_metadata_id (
 	EP_ASSERT (ep_event != NULL);
 	EP_ASSERT (metadata_id > 0);
 
-	// If a pre-existing metadata label exists, remove it.
-	uint32_t old_id;
-	if (ep_rt_metadata_labels_hash_lookup (&file->metadata_ids, ep_event, &old_id))
-		ep_rt_metadata_labels_hash_remove (&file->metadata_ids, ep_event);
-
-	// Add the metadata label.
-	return ep_rt_metadata_labels_hash_add (&file->metadata_ids, ep_event, metadata_id);
+	// Add/Replace the metadata label.
+	return dn_umap_ptr_uint32_insert_or_assign (file->metadata_ids, ep_event, metadata_id).result;
 }
 
 static
@@ -304,6 +313,11 @@ ep_file_alloc (
 	StreamWriter *stream_writer,
 	EventPipeSerializationFormat format)
 {
+	dn_umap_custom_alloc_params_t params = {0, };
+	params.hash_func = stack_hash_key_hash_func;
+	params.equal_func = stack_hash_key_equal_func;
+	params.value_dispose_func = stack_hash_value_free_func;
+
 	EventPipeFile *instance = ep_rt_object_alloc (EventPipeFile);
 	ep_raise_error_if_nok (instance != NULL);
 
@@ -337,11 +351,11 @@ ep_file_alloc (
 
 	instance->sampling_rate_in_ns = (uint32_t)ep_sample_profiler_get_sampling_rate ();
 
-	ep_rt_metadata_labels_hash_alloc (&instance->metadata_ids, NULL, NULL, NULL, NULL);
-	ep_raise_error_if_nok (ep_rt_metadata_labels_hash_is_valid (&instance->metadata_ids));
+	instance->metadata_ids = dn_umap_alloc ();
+	ep_raise_error_if_nok (instance->metadata_ids != NULL);
 
-	ep_rt_stack_hash_alloc (&instance->stack_hash, ep_stack_hash_key_hash, ep_stack_hash_key_equal, NULL, stack_hash_value_free_func);
-	ep_raise_error_if_nok (ep_rt_stack_hash_is_valid (&instance->stack_hash));
+	instance->stack_hash = dn_umap_custom_alloc (&params);
+	ep_raise_error_if_nok (instance->stack_hash != NULL);
 
 	// Start at 0 - The value is always incremented prior to use, so the first ID will be 1.
 	ep_rt_volatile_store_uint32_t (&instance->metadata_id_counter, 0);
@@ -376,8 +390,8 @@ ep_file_free (EventPipeFile *file)
 	ep_metadata_block_free (file->metadata_block);
 	ep_stack_block_free (file->stack_block);
 	ep_fast_serializer_free (file->fast_serializer);
-	ep_rt_metadata_labels_hash_free (&file->metadata_ids);
-	ep_rt_stack_hash_free (&file->stack_hash);
+	dn_umap_free (file->metadata_ids);
+	dn_umap_free (file->stack_hash);
 
 	// If file has not been initialized, stream_writer ownership
 	// have not been passed along and needs to be freed by file.
@@ -398,7 +412,7 @@ ep_file_initialize_file (EventPipeFile *file)
 	bool success = true;
 	if (file->format >= EP_SERIALIZATION_FORMAT_NETTRACE_V4) {
 		const ep_char8_t header[] = "Nettrace";
-		const uint32_t bytes_to_write = (uint32_t)(EP_ARRAY_SIZE (header) - 1);
+		const uint32_t bytes_to_write = (uint32_t)(STRING_LENGTH (header));
 		uint32_t bytes_written = 0;
 		success = ep_stream_writer_write (file->stream_writer, (const uint8_t *)header, bytes_to_write, &bytes_written) && bytes_written == bytes_to_write;
 	}
@@ -457,7 +471,7 @@ ep_file_write_event (
 	file_write_event_to_block (file, event_instance, metadata_id, capture_thread_id, sequence_number, stack_id, is_sorted_event);
 
 ep_on_exit:
-	ep_event_metdata_event_free (metadata_instance);
+	ep_event_metadata_event_free (metadata_instance);
 	return;
 
 ep_on_error:
@@ -487,7 +501,7 @@ ep_file_write_sequence_point (
 
 	// stack cache resets on sequence points
 	file->stack_id_counter = 0;
-	ep_rt_stack_hash_remove_all (&file->stack_hash);
+	dn_umap_clear (file->stack_hash);
 
 ep_on_exit:
 	return;
@@ -540,13 +554,13 @@ ep_stack_hash_entry_get_key (StackHashEntry *stack_hash_entry)
 
 StackHashEntry *
 ep_stack_hash_entry_alloc (
-	const EventPipeStackContents *stack_contents,
+	const EventPipeStackContentsInstance *stack_contents,
 	uint32_t id,
 	uint32_t hash)
 {
 	EP_ASSERT (stack_contents != NULL);
 
-	uint32_t stack_size = ep_stack_contents_get_size (stack_contents);
+	uint32_t stack_size = ep_stack_contents_instance_get_size (stack_contents);
 	StackHashEntry *entry = (StackHashEntry *)ep_rt_byte_array_alloc (offsetof (StackHashEntry, stack_bytes) + stack_size);
 	ep_raise_error_if_nok (entry != NULL);
 
@@ -554,7 +568,7 @@ ep_stack_hash_entry_alloc (
 	entry->key.hash = hash;
 	entry->key.stack_size_in_bytes = stack_size;
 	entry->key.stack_bytes = entry->stack_bytes;
-	memcpy (entry->stack_bytes, ep_stack_contents_get_pointer (stack_contents), stack_size);
+	memcpy (entry->stack_bytes, ep_stack_contents_instance_get_pointer (stack_contents), stack_size);
 
 ep_on_exit:
 	return entry;
@@ -590,36 +604,19 @@ hash_bytes (const uint8_t *data, size_t data_len)
 	return hash;
 }
 
-StackHashKey *
-ep_stack_hash_key_init (
-	StackHashKey *key,
-	const EventPipeStackContents *stack_contents)
-{
-	EP_ASSERT (key != NULL);
-	EP_ASSERT (stack_contents != NULL);
-
-	key->stack_bytes = ep_stack_contents_get_pointer (stack_contents);
-	key->stack_size_in_bytes = ep_stack_contents_get_size (stack_contents);
-	key->hash = hash_bytes (key->stack_bytes, key->stack_size_in_bytes);
-
-	return key;
-}
-
-void
-ep_stack_hash_key_fini (StackHashKey *key)
-{
-	;
-}
-
+static
 uint32_t
-ep_stack_hash_key_hash (const void *key)
+DN_CALLBACK_CALLTYPE
+stack_hash_key_hash_func (const void *key)
 {
 	EP_ASSERT (key != NULL);
 	return ((const StackHashKey *)key)->hash;
 }
 
+static
 bool
-ep_stack_hash_key_equal (const void *key1, const void *key2)
+DN_CALLBACK_CALLTYPE
+stack_hash_key_equal_func (const void *key1, const void *key2)
 {
 	EP_ASSERT (key1 != NULL);
 	EP_ASSERT (key2 != NULL);
@@ -631,10 +628,31 @@ ep_stack_hash_key_equal (const void *key1, const void *key2)
 		!memcmp (stack_hash_key1->stack_bytes, stack_hash_key2->stack_bytes, stack_hash_key1->stack_size_in_bytes);
 }
 
+StackHashKey *
+ep_stack_hash_key_init (
+	StackHashKey *key,
+	const EventPipeStackContentsInstance *stack_contents)
+{
+	EP_ASSERT (key != NULL);
+	EP_ASSERT (stack_contents != NULL);
+
+	key->stack_bytes = ep_stack_contents_instance_get_pointer (stack_contents);
+	key->stack_size_in_bytes = ep_stack_contents_instance_get_size (stack_contents);
+	key->hash = hash_bytes (key->stack_bytes, key->stack_size_in_bytes);
+
+	return key;
+}
+
+void
+ep_stack_hash_key_fini (StackHashKey *key)
+{
+	;
+}
+
 #endif /* !defined(EP_INCLUDE_SOURCE_FILES) || defined(EP_FORCE_INCLUDE_SOURCE_FILES) */
 #endif /* ENABLE_PERFTRACING */
 
-#ifndef EP_INCLUDE_SOURCE_FILES
+#if !defined(ENABLE_PERFTRACING) || (defined(EP_INCLUDE_SOURCE_FILES) && !defined(EP_FORCE_INCLUDE_SOURCE_FILES))
 extern const char quiet_linker_empty_file_warning_eventpipe_file;
 const char quiet_linker_empty_file_warning_eventpipe_file = 0;
 #endif
