@@ -8,7 +8,7 @@ public class NegCollect
     public static int Main()
     {
         bool retVal = true;
-        GCCollectionMode[] invalidInputs = { (GCCollectionMode)(GCCollectionMode.Default - 1), (GCCollectionMode)(GCCollectionMode.Optimized + 1) };
+        GCCollectionMode[] invalidInputs = { (GCCollectionMode)(GCCollectionMode.Default - 1), (GCCollectionMode)(GCCollectionMode.Aggressive + 1) };
 
         for (int i = 0; i < invalidInputs.Length; i++)
         {
