@@ -4,7 +4,6 @@
 
 using System;
 using System.Diagnostics;
-using System.Reflection;
 
 public class Test_Collect
 {
@@ -14,7 +13,7 @@ public class Test_Collect
         GC.Collect();
         sw.Stop();
         TimeSpan elapsed = sw.Elapsed;
-        TimeSpan totalPauseDuration = (TimeSpan)typeof(GC).GetMethod("GetTotalPauseDuration", BindingFlags.Public | BindingFlags.Static).Invoke(null, null);
+        TimeSpan totalPauseDuration = GC.GetTotalPauseDuration();
         GCMemoryInfo memoryInfo = GC.GetGCMemoryInfo();
         TimeSpan lastGcDuration = memoryInfo.PauseDurations[0];
 
