@@ -12,7 +12,7 @@
 using System;
 using System.Runtime.InteropServices;
 
-public class Test
+public class Test_Alloc_neg
 {
     public static int Main()
     {
