@@ -5,10 +5,11 @@ namespace Dynamic
 {
     using System;
     using TestLibrary;
+    using Xunit;
 
     class Program
     {
-        static int Main(string[] doNotUse)
+        static int Main()
         {
             // RegFree COM is not supported on Windows Nano
             if (Utilities.IsWindowsNanoServer)
