@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 #include "ClientTests.h"
+#include <windows_version_helpers.h>
 
 template<COINIT TM>
 struct ComInit
@@ -23,6 +24,11 @@ using ComMTA = ComInit<COINIT_MULTITHREADED>;
 
 int __cdecl main()
 {
+    if (is_windows_nano() == S_OK)
+    {
+        ::puts("RegFree COM is not supported on Windows Nano. Auto-passing this test.\n");
+        return 100;
+    }
     ComMTA init;
     if (FAILED(init.Result))
         return -1;
