@@ -5,7 +5,8 @@
 
 using System;
 using System.Runtime.CompilerServices;
-public class BringUpTest
+using Xunit;
+public class BringUpTest_FPConvF2F
 {
     const int Pass = 100;
     const int Fail = -1;
@@ -16,7 +17,8 @@ public class BringUpTest
     [MethodImplAttribute(MethodImplOptions.NoInlining)]
     public static float FPConvF2F(double x) { return (float) x; }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int result = Fail;
         double x = FPConvF2F(3f);
