@@ -5,7 +5,8 @@
 
 using System;
 using System.Runtime.CompilerServices;
-public class BringUpTest
+using Xunit;
+public class BringUpTest_JTrueLtDbl
 {
     const int Pass = 100;
     const int Fail = -1;
@@ -25,7 +26,8 @@ public class BringUpTest
         return returnValue;
     }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int returnValue = Pass;
 
