@@ -6,6 +6,7 @@
 using System;
 using System.Text;
 using System.Runtime.CompilerServices;
+using Xunit;
 
 struct vc
 {
@@ -15,12 +16,13 @@ struct vc
     public vc (int xx, int yy, int zz) { x = xx; y = yy; z = zz; }
 }
 
-class child
+public class child
 {
     const int Pass = 100;
     const int Fail = -1;
 
-    static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int result = mul2(3);
         if (result == 15)
