@@ -11,12 +11,14 @@
 
 using System;
 using System.Runtime.CompilerServices;
+using Xunit;
 
 namespace Test
 {
-    static class App
+    public static class App
     {
-        static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             new DerivedClass<int>(3);
             new DerivedClass<int>(8);
