@@ -5,8 +5,9 @@
 #define USE_STRUCT
 using System.Runtime.CompilerServices;
 using System;
+using Xunit;
 
-class SP2
+public class SP2
 {
 
 #if USE_STRUCT
@@ -43,7 +44,8 @@ class SP2
 #endif
     }
 
-    public static int Main(String[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         long res = M(1, 2);
         Console.WriteLine("M(1, 2) is {0}.", res);
