@@ -4,8 +4,9 @@
 
 using System.Runtime.CompilerServices;
 using System;
+using Xunit;
 
-class SP2c
+public class SP2c
 {
 
     // Struct in reg (int, long)
@@ -30,7 +31,8 @@ class SP2c
         return Foo(i2, l0, s); // r0 <= inarg[4]; r2/r3 <= r0/r1; outarg[0] <= inarg[0]; outarg[8/12] <= r2/r3
     }
 
-    public static int Main(String[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         long res = M(4, 2, 3, 1);
         Console.WriteLine("M(4, 2, 3, 1) is {0}.", res);
