@@ -6,8 +6,9 @@
 
 using System;
 using System.Runtime.InteropServices;
+using Xunit;
 
-internal class Program
+public class Program
 {
     private static void Eval(int testCase, bool b1, bool b2)
     {
@@ -636,7 +637,8 @@ internal class Program
         }
     }  // end of test case 0003
 
-    private static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         try
         {
