@@ -3,6 +3,7 @@
 
 using System;
 using System.Threading;
+using Xunit;
 
 internal delegate T GenDelegate<T>(T p1, out T p2);
 
@@ -15,9 +16,10 @@ internal struct Foo<T>
     }
 }
 
-internal class Test
+public class Test_Delegate030
 {
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int i, j;
         GenDelegate<int> MyDelegate = new GenDelegate<int>(Foo<int>.Function);
@@ -25,7 +27,7 @@ internal class Test
 
         if ((i != 10) || (j != 10))
         {
-            Console.WriteLine("Failed Sync Invokation");
+            Console.WriteLine("Failed Sync Invocation");
             return 1;
         }
 
