@@ -2,10 +2,14 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 
-internal class test
+namespace Test_i8flat_cs
 {
-    public static int Main()
+public class test
+{
+    [Fact]
+    public static int TestEntryPoint()
     {
         Int64 x;
         Int64 y;
@@ -1237,3 +1241,4 @@ internal class test
             return 1;
     }
 }
+}
