@@ -2,10 +2,11 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 
-namespace BoxTest
+namespace BoxTest_tailjump_cs
 {
-    internal class Test
+    public class Test
     {
         protected object Fibonacci(object num, object flag)
         {
@@ -28,7 +29,8 @@ namespace BoxTest
             return N;
         }
 
-        private static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             new Test().Fibonacci(20, true);
             Console.WriteLine();
