@@ -3,7 +3,8 @@
 //
 
 using System;
-class r8NaNadd
+using Xunit;
+public class r8NaNadd
 {
 
     //user-defined class that overloads operator +
@@ -77,7 +78,8 @@ class r8NaNadd
         public double d_vt_test3_op1;
         public double d_vt_test3_op2;
     }
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         bool passed = true;
         //initialize class
