@@ -3,11 +3,13 @@
 
 // static field
 using System;
+using Xunit;
 namespace Precise
 {
-    class Driver
+    public class Driver_xprecise4
     {
-        public static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             try
             {
