@@ -4,7 +4,10 @@
 // Throw from a finally block
 
 using System;
+using Xunit;
 
+namespace Test_throwinfinally_basics
+{
 public class a
 {
     private static TestUtil.TestLog testLog;
@@ -38,7 +41,8 @@ public class a
         }
     }
 
-    public static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         //Start recording
         testLog.StartRecording();
@@ -59,3 +63,4 @@ public class a
         return testLog.VerifyOutput();
     }
 }
+}
