@@ -3,10 +3,11 @@
 
 // Try finally, non error case
 using System;
+using Xunit;
 
-namespace hello
+namespace hello_tryfinally_basics_cs
 {
-    class Class1
+    public class Class1
     {
         private static TestUtil.TestLog testLog;
 
@@ -34,7 +35,8 @@ namespace hello
             Console.WriteLine("in Finally");
         }
 
-        static public int Main(string[] args)
+        [Fact]
+        static public int TestEntryPoint()
         {
             //Start recording
             testLog.StartRecording();
