@@ -3,10 +3,11 @@
 
 // try/finally embedded in a try catch with a nonlocal exit 
 using System;
+using Xunit;
 
-namespace hello
+namespace hello_simplenonlocalexit_finallyexec_cs
 {
-    class Class1
+    public class Class1
     {
         private static TestUtil.TestLog testLog;
 
@@ -23,14 +24,15 @@ namespace hello
             testLog = new TestUtil.TestLog(expectedOut);
         }
 
-        static public int Main(string[] args)
+        [Fact]
+        static public int TestEntryPoint()
         {
             //Start recording
             testLog.StartRecording();
 
             try
             {
-                if (args.Length == 0) goto done;
+                goto done;
                 Console.WriteLine("in try");
             }
             finally
