@@ -2,8 +2,9 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 
-namespace Test
+namespace Test_refarg_s_cs
 {
     internal class AA
     {
@@ -23,7 +24,7 @@ namespace Test
         }
     }
 
-    internal class App
+    public class App
     {
         private static AA s_aa = new AA();
 
@@ -50,7 +51,8 @@ namespace Test
             return 100;
         }
 
-        private static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             int exitCode = Test(ref s_aa.mm);
             GC.Collect();
