@@ -1,14 +1,14 @@
 // Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
-/* 
+/*
  * Bug info:
- * Asserstion prop was comparing 2 range assertions incorrectly and keeping the wrong one as a result.
+ * Assertstion prop was comparing 2 range assertions incorrectly and keeping the wrong one as a result.
  * Thus assuming it didn't need the narrowing convert to byte. It thus returned the full 32 bit value instead of the lower 8 bits  alone.
- * 
+ *
  * Repro Steps:
  * Compile this program with option /optimize and execute it from a console window:
- * 
+ *
  * Actual Results:
  * value : 256, lowByte : 0, lowByteInt : 256
  *
@@ -17,8 +17,11 @@
 */
 
 using System;
+using Xunit;
 
-internal class Program
+namespace Test_intToByte_cs
+{
+public class Program
 {
     private struct MyStruct
     {
@@ -31,7 +34,8 @@ internal class Program
 
     static private MyStruct[] s_myObjects = { new MyStruct(0x0100) };
 
-    private static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         MyStruct obj = s_myObjects[0];
         ushort value = obj.value;
@@ -41,3 +45,4 @@ internal class Program
         return (lowByteInt == 0) ? 100 : 101;
     }
 }
+}
