@@ -2,7 +2,10 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
-internal unsafe class testout1
+using Xunit;
+namespace Test_5w1d_04
+{
+public unsafe class testout1
 {
     public struct VT_0
     {
@@ -31,7 +34,8 @@ internal unsafe class testout1
         return Convert.ToDouble((asgop0 / asgop1));
     }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int retval;
         retval = Convert.ToInt32(Func_0());
@@ -43,3 +47,4 @@ internal unsafe class testout1
         return retval;
     }
 }
+}
