@@ -2,13 +2,11 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
-using Microsoft.Xunit.Performance;
+
 using System;
 using System.Runtime.CompilerServices;
 using Xunit;
 
-[assembly: OptimizeForBenchmarks]
-
 namespace Benchstone.BenchF
 {
 public static class BenchMk2
@@ -40,27 +38,10 @@ public static class BenchMk2
         return true;
     }
 
-    [Benchmark]
-    public static void Test()
-    {
-        foreach (var iteration in Benchmark.Iterations)
-        {
-            using (iteration.StartMeasurement())
-            {
-                Bench();
-            }
-        }
-    }
-
-    private static bool TestBase()
+    [Fact]
+    public static int TestEntryPoint()
     {
         bool result = Bench();
-        return result;
-    }
-
-    public static int Main()
-    {
-        bool result = TestBase();
         return (result ? 100 : -1);
     }
 }
