@@ -2,7 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
-using Microsoft.Xunit.Performance;
 
 namespace Functions
 {
@@ -13,21 +12,6 @@ namespace Functions
         private const double acosDoubleDelta = 0.0004;
         private const double acosDoubleExpectedResult = 7852.4108380716079;
 
-        [Benchmark(InnerIterationCount=AcosDoubleIterations)]
-        public static void AcosDoubleBenchmark()
-        {
-            foreach (var iteration in Benchmark.Iterations)
-            {
-                using (iteration.StartMeasurement())
-                {
-                    for (int i = 0; i < Benchmark.InnerIterationCount; i++)
-                    {
-                        AcosDoubleTest();
-                    }
-                }
-            }
-        }
-
         public static void AcosDoubleTest()
         {
             var result = 0.0; var value = -1.0;
