@@ -2,7 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
-using Microsoft.Xunit.Performance;
 
 namespace Functions
 {
@@ -14,21 +13,6 @@ namespace Functions
         private const double atan2DoubleDeltaY = 0.0004;
         private const double atan2DoubleExpectedResult = 3926.99081698702;
 
-        [Benchmark(InnerIterationCount = Atan2DoubleIterations)]
-        public static void Atan2DoubleBenchmark()
-        {
-            foreach (var iteration in Benchmark.Iterations)
-            {
-                using (iteration.StartMeasurement())
-                {
-                    for (int i = 0; i < Benchmark.InnerIterationCount; i++)
-                    {
-                        Atan2DoubleTest();
-                    }
-                }
-            }
-        }
-
         public static void Atan2DoubleTest()
         {
             var result = 0.0; var valueX = 1.0; var valueY = -1.0;
