@@ -2,7 +2,6 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
-using Microsoft.Xunit.Performance;
 
 namespace Functions
 {
@@ -13,21 +12,6 @@ namespace Functions
         private const double expDoubleDelta = 0.0004;
         private const double expDoubleExpectedResult = 5877.1812477590884;
 
-        [Benchmark(InnerIterationCount = ExpDoubleIterations)]
-        public static void ExpDoubleBenchmark()
-        {
-            foreach (var iteration in Benchmark.Iterations)
-            {
-                using (iteration.StartMeasurement())
-                {
-                    for (int i = 0; i < Benchmark.InnerIterationCount; i++)
-                    {
-                        ExpDoubleTest();
-                    }
-                }
-            }
-        }
-
         public static void ExpDoubleTest()
         {
             var result = 0.0; var value = -1.0;
