@@ -9,14 +9,9 @@
 
 #define INTF_FOR_TASK
 
-using Microsoft.Xunit.Performance;
 using System;
-using System.Collections.Generic;
-
-[assembly: OptimizeForBenchmarks]
-
-// using System.Diagnostics;
-// using System.Text.RegularExpressions;
+using System.Runtime.CompilerServices;
+using Xunit;
 
 namespace V8.Richards
 {
@@ -105,13 +100,16 @@ namespace V8.Richards
 
         public const int DATA_SIZE = 4;
 
-        public static int Main(String[] args)
+        [Fact]
+        public static int TestEntryPoint()
         {
-            int n = 1;
-            if (args.Length > 0)
-            {
-                n = Int32.Parse(args[0]);
-            }
+            return Test(null);
+        }
+
+        [MethodImpl(MethodImplOptions.NoInlining)]
+        private static int Test(int? arg)
+        {
+            int n = arg ?? 1;
             bool result = Measure(n);
             return (result ? 100 : -1);
         }
@@ -130,23 +128,6 @@ namespace V8.Richards
                               n, dur.TotalMilliseconds, (1000.0 * dur.TotalMilliseconds) / n);
             return result;
         }
-
-        [Benchmark]
-        public static void Bench()
-        {
-            const int Iterations = 5000;
-
-            foreach (var iteration in Benchmark.Iterations)
-            {
-                using (iteration.StartMeasurement())
-                {
-                    for (int i = 0; i < Iterations; i++)
-                    {
-                        runRichards();
-                    }
-                }
-            }
-        }
     }
 
     internal class Scheduler
