@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
+using Xunit;
 namespace DefaultNamespace
 {
     //@BEGINRENAME; Verify this renames
@@ -10,7 +11,8 @@ namespace DefaultNamespace
 
     public class Bug
     {
-        public static int Main(String[] args)
+        [Fact]
+        public static int TestEntryPoint()
         {
             //Console.WriteLine (DateTime.GetNow().ToString());
             Console.WriteLine(DateTime.Now.ToString());
