@@ -3,11 +3,12 @@
 
 
 using System;
+using Xunit;
 
 
 namespace DefaultNamespace
 {
-    internal class X
+    public class X
     {
         public virtual int Blah(int what)
         {
@@ -27,7 +28,8 @@ namespace DefaultNamespace
         }
 
 
-        public static int Main(String[] argv)
+        [Fact]
+        public static int TestEntryPoint()
         {
             int i = 0;
             X x = new X();
