@@ -3,11 +3,13 @@
 //
 
 using System;
+using Xunit;
 
 public class test
 {
     static byte by = 13;
-    public static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         byte by1 = (byte)(-by);
 
