@@ -16,11 +16,13 @@
 // <Code> 
 
 using System;
+using Xunit;
 
-class MyClass
+public class MyClass
 {
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         long test1 = long.MinValue;
         long test2 = 0;
