@@ -2,11 +2,12 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
+using Xunit;
 namespace Test
 {
     using System;
 
-    struct AA
+    public struct AA
     {
         private ulong[] m_aulDummyField;
         private static object[] m_axField4;
@@ -29,7 +30,8 @@ namespace Test
                 } while (AA.Static1(null, new bool[7]));
             }
         }
-        static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             try
             {
