@@ -2,13 +2,15 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
+using Xunit;
 namespace Test
 {
     using System;
 
-    class AA
+    public class AA
     {
-        public static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             int L = 1;
             try
