@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
+using Xunit;
 namespace Test
 {
     using System;
@@ -10,10 +11,11 @@ namespace Test
     {
         public static sbyte m_sb = 0;
     }
-    struct CC
+    public struct CC
     {
         float Method1() { return 0; }
-        static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             CC[] cc = new CC[10];
             byte[] ab = new byte[10];
