@@ -2,9 +2,11 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 public class SamplesArray
 {
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int[] myLens = new int[1] { 5 };
         int[] myLows = new int[1] { -2 };
