@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
+using Xunit;
 namespace Test
 {
     using System;
@@ -19,7 +20,8 @@ namespace Test
             catch (Exception) { }
             return 103;
         }
-        static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             try
             {
