@@ -2,9 +2,11 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 public class test
 {
-    public static int Main(String[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         bool flag = false;
         for (int i = 1; i <= -1; i++)
