@@ -4,13 +4,15 @@
 
 //COMMAND LINE: csc /nologo /optimize- /debug- /w:0 bug.cs
 using System;
+using Xunit;
 public class BB
 {
     byte Method1(sbyte[,][][,] param2)
     {
         return new byte[][, ,] { }[0][Math.Sign(1), Math.Sign(1), Math.Min(0, 0)];
     }
-    static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         try
         {
