@@ -2,7 +2,8 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
-internal unsafe class testout1
+using Xunit;
+public unsafe class testout1
 {
     public struct VT_1
     {
@@ -25,7 +26,8 @@ internal unsafe class testout1
         return retval_1;
     }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         CL cl = new CL();
 
