@@ -3,6 +3,7 @@
 //
 
 using System;
+using Xunit;
 class C
 {
     private string s = "This is private";
@@ -13,9 +14,10 @@ class B
     public string t = "This is safe";
 }
 
-class Class1
+public class Class1
 {
-    public static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         B[,] ab = new B[1, 1];
         object[,] ao = ab;
