@@ -10,8 +10,9 @@
 
 using System;
 using System.Runtime.CompilerServices;
+using Xunit;
 
-class My
+public class My
 {
 
     [MethodImplAttribute(MethodImplOptions.NoInlining)]
@@ -30,7 +31,8 @@ class My
         return null;
     }
 
-    static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         Object o = new Object();
         foo(o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, o, bar(o), o);
