@@ -5,10 +5,12 @@ using System;
 using System.Threading;
 using System.Runtime.InteropServices;
 using System.Runtime.CompilerServices;
+using Xunit;
 
 public class Test
 {
-    public static int Main(String[] arguments)
+    [Fact]
+    public static int TestEntryPoint()
     {
         bool testCaseSucceeded = false;
         string[] theArray = { "Wrong =0", "Correct =1", "Wrong =2", "Wrong =3" };
