@@ -5,6 +5,7 @@
 // to analyze a bounds check.
 
 using System;
+using Xunit;
 
 public struct MyStruct
 {
@@ -26,9 +27,10 @@ public struct MyReader
     }
 }
 
-class GitHub_19454
+public class GitHub_19454
 {
-    static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         MyReader r = new MyReader();
         r.ReadBytesInner(0);
