@@ -5,12 +5,14 @@ using System;
 using System.Numerics;
 
 using Point = System.Numerics.Vector2;
+using Xunit;
 
 namespace GitHub_21854
 {
     public class test
     {
-        static int Main(string[] args)
+        [Fact]
+        public static int TestEntryPoint()
         {
             try {
                 var unused = new object[] { Vector<int>.Zero };
