@@ -4,17 +4,19 @@
 
 using System;
 using System.Runtime.CompilerServices;
+using Xunit;
 
 // The jit should null check 'this' in NextElement
 
-unsafe struct GitHub_23791
+public unsafe struct GitHub_23791
 {
     fixed byte A[10];
 
     [MethodImpl(MethodImplOptions.NoInlining)]
     byte NextElement(int i) => A[1+i];
 
-    static int Main() 
+    [Fact]
+    public static int TestEntryPoint() 
     {
         int result = -1;
         GitHub_23791* x = null;
