@@ -3,8 +3,9 @@
 
 using System;
 using System.Runtime.CompilerServices;
+using Xunit;
 
-class GitHub_26417
+public class GitHub_26417
 {
     static int   _a;
 
@@ -30,7 +31,8 @@ class GitHub_26417
         }
     }
 
-    static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         int result = 100;
         try {
