@@ -5,22 +5,24 @@
 
 using System;
 using System.Runtime.CompilerServices;
+using Xunit;
 
 struct RefWrapper
 {
     public Object a; // a ref field
 }
 
-class TestStructs
+public class TestStructs
 {
     static RefWrapper[] arr;
 
-    public static RefWrapper GetElement() // 8 byte size return will be retyped as a ref.
+    static RefWrapper GetElement() // 8 byte size return will be retyped as a ref.
     {
         return arr[0];
     }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         RefWrapper a = new RefWrapper();
         arr = new RefWrapper[1];
