@@ -4,13 +4,14 @@
 using System;
 using System.Runtime.CompilerServices;
 using System.Threading;
+using Xunit;
 
 interface I
 {
     public Decimal F();
 }
 
-class Runtime_53549 : I
+public class Runtime_53549 : I
 {
     Decimal z;
 
@@ -25,12 +26,13 @@ class Runtime_53549 : I
     // we allow GDV on i.F().
     //
     [MethodImpl(MethodImplOptions.NoInlining)]
-    public static int H(I i)
+    static int H(I i)
     {
         return G(i.F()) ? 100 : -1;
     }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         Runtime_53549 x = new Runtime_53549();
         x.z = 100M;
