@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System.Runtime.CompilerServices;
+using Xunit;
 
 public class Runtime_55131
 {
@@ -17,7 +18,8 @@ public class Runtime_55131
     static ushort s_6;
     static uint[] s_15 = new uint[] { 0 };
     static bool s_19 = false;
-    private static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         bool condition = False();
         int result = 100;
