@@ -1,7 +1,8 @@
-﻿// Licensed to the .NET Foundation under one or more agreements.
+// Licensed to the .NET Foundation under one or more agreements.
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 
 namespace Runtime_56930
 {
@@ -14,11 +15,12 @@ namespace Runtime_56930
         }
     }
 
-    class Program
+    public class Program
     {
         static C0 s_2 = new C0(1);
 
-        public static int Main()
+        [Fact]
+        public static int TestEntryPoint()
         {
             // Since the following statement is a dead store the next two statements will become
             // a NULLCHECK(s_2) followed by STOREIND(&s_2[8], 0)
