@@ -2,10 +2,12 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 //
 
-class Bug
+using Xunit;
+public class Bug
 {
     static short s1 = 8712, s2 = -973;
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         short s3 = (short)(s1 / s2);
         short s4 = (short)(s1 % s2);
