@@ -3,6 +3,7 @@
 //
 
 using System;
+using Xunit;
 
 struct MyValueClass
 {
@@ -14,9 +15,10 @@ struct MyValueClass
     public int i6;
 };
 
-class T
+public class T
 {
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         MyValueClass mvc = foo();
 
@@ -37,7 +39,7 @@ class T
         return 100;
     }
 
-    public static MyValueClass foo()
+    private static MyValueClass foo()
     {
         MyValueClass mvcRetVal = new MyValueClass();
 
