@@ -82,19 +82,21 @@ struct S
     }
 }
 
-public class Test_struct1_4
+public class Test_struct4_4
 {
-    private static void c(S s1)
+    private static void c(S s1, S s2, S s3)
     {
-        Console.WriteLine(s1.str);
+        Console.WriteLine(s1.str + s2.str + s3.str);
     }
 
     [Fact]
     public static int TestEntryPoint()
     {
         S sM = new S("test");
+        S sM2 = new S("test2");
+        S sM3 = new S("test3");
 
-        c(sM);
+        c(sM, sM2, sM3);
         return 100;
     }
 }
