@@ -3,6 +3,7 @@
 //
 
 using System;
+using Xunit;
 
 struct Pad
 {
@@ -38,16 +39,15 @@ struct Pad
     public double d29;
     public double d30;
 #pragma warning restore 0414
-
 }
 
 struct S
 {
+    public String str;
+    public Pad pad;
 #pragma warning disable 0414
     public String str2;
 #pragma warning restore 0414
-    public String str;
-    public Pad pad;
 
     public S(String s)
     {
@@ -86,20 +86,26 @@ struct S
     }
 }
 
-class Test
+public class Test_structret6_2
 {
-    public static S c(S s1, S s2)
+    private static S c(S s1, S s2, S s3, S s4, S s5)
     {
-        s1.str = (s1.str + s2.str);
-        return s1;
+        S r;
+        s1.str = s1.str + s2.str + s3.str + s4.str + s5.str;
+        r = s1;
+        return r;
     }
 
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         S sM = new S("test");
         S sM2 = new S("test2");
+        S sM3 = new S("test3");
+        S sM4 = new S("test4");
+        S sM5 = new S("test5");
 
-        Console.WriteLine(c(sM, sM2));
+        Console.WriteLine(c(sM, sM2, sM3, sM4, sM5));
         return 100;
     }
 }
