@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 public struct T : IComparable
 {
     public int x;
@@ -26,9 +27,10 @@ public struct T : IComparable
     }
 }
 
-internal class foo
+public class foo
 {
-    public static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         bar<T> b = new bar<T>();
         return b.test(new T(1, 2, 3));
