@@ -2,8 +2,10 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using System.Runtime.CompilerServices;
+using Xunit;
 
-internal unsafe class Test
+public unsafe class Test
 {
     private static uint GetIndex(int v)
     {
@@ -22,7 +24,11 @@ internal unsafe class Test
         return i;
     }
 
-    public static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint() => Run(new string[0]);
+
+    [MethodImpl(MethodImplOptions.NoInlining)]
+    public static int Run(string[] args)
     {
         byte* table = stackalloc byte[257];
 
