@@ -4,8 +4,9 @@
 
 using System.Runtime.InteropServices;
 using System;
+using Xunit;
 
-internal class NullableTest
+public class NullableTest
 {
     private static bool BoxUnboxToNQGen<T>(T o)
     {
@@ -27,7 +28,8 @@ internal class NullableTest
         return ((NotEmptyStruct?)o) == null;
     }
 
-    private static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         NotEmptyStruct? s = null;
 
