@@ -13,8 +13,9 @@
 
 using System.Runtime.InteropServices;
 using System;
+using Xunit;
 
-internal class NullableTest
+public class NullableTest
 {
     private static bool BoxUnboxToNQGen<T>(T o)
     {
@@ -36,7 +37,8 @@ internal class NullableTest
         return ((MarshalAsStruct?)(ValueType)o) == null;
     }
 
-    private static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         MarshalAsStruct? s = null;
 
