@@ -4,8 +4,9 @@
 //Simple dev unit test for constant propagation assertion.
 
 using System;
+using Xunit;
 
-internal class Sample1
+public class Sample1
 {
     [System.Runtime.CompilerServices.MethodImplAttribute(System.Runtime.CompilerServices.MethodImplOptions.NoInlining)]
     private static int func(int a)
@@ -18,7 +19,8 @@ internal class Sample1
         else
             return y;
     }
-    private static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         bool failed = false;
         if (func(0) != 5)
