@@ -2,6 +2,7 @@
 // The .NET Foundation licenses this file to you under the MIT license.
 
 using System;
+using Xunit;
 internal class Buffer
 {
     private byte[] _buffer;
@@ -39,9 +40,10 @@ internal class Buffer
 }
 
 
-internal class Test
+public class Test_bug595776
 {
-    private static int Main()
+    [Fact]
+    public static int TestEntryPoint()
     {
         Buffer b1 = new Buffer("Abra-cadabra");
         int result = 0;
