@@ -4,8 +4,9 @@
 using System;
 using System.Diagnostics;
 using System.Runtime.CompilerServices;
+using Xunit;
 
-class IntegerSumLoop
+public class IntegerSumLoop
 {
     [MethodImpl(MethodImplOptions.NoInlining)]
     public static int F(int from, int to)
@@ -18,7 +19,8 @@ class IntegerSumLoop
         return result;
     }
 
-    public static int Main(string[] args)
+    [Fact]
+    public static int TestEntryPoint()
     {
         int final = 1_000_000;
         long frequency = Stopwatch.Frequency;
