@@ -7,7 +7,7 @@ using System.IO;
 using System.Reflection;
 using System.Runtime.Loader;
 
-using TestLibrary;
+using Xunit;
 
 using ResolutionStage = BinderTracingTests.ResolutionAttempt.ResolutionStage;
 using ResolutionResult = BinderTracingTests.ResolutionAttempt.ResolutionResult;
@@ -73,7 +73,8 @@ namespace BinderTracingTests
         //   ResolutionAttempted : DefaultAssemblyLoadContextFallback   (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AssemblyLoadContextResolvingEvent    (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AppDomainAssemblyResolveEvent        (CustomALC)     [AssemblyNotFound]
-        [BinderTest(isolate: true, testSetup: nameof(LoadSubdirectoryAssembly_InstanceALC))]
+        [BinderTest(isolate: true, testSetup: nameof(LoadSubdirectoryAssembly_InstanceALC),
+            activeIssue: "https://github.com/dotnet/runtime/issues/68521")] // Emit-based Invoke causes an extra load.
         public static BindOperation FindInLoadContext_CustomALC_IncompatibleVersion()
         {
             var assemblyName = new AssemblyName($"{SubdirectoryAssemblyName}, Version=4.3.2.1");
@@ -180,7 +181,9 @@ namespace BinderTracingTests
         //   ResolutionAttempted : ApplicationAssemblies                (DefaultALC)    [MismatchedAssemblyName]
         //   ResolutionAttempted : AssemblyLoadContextResolvingEvent    (DefaultALC)    [AssemblyNotFound]
         //   ResolutionAttempted : AppDomainAssemblyResolveEvent        (DefaultALC)    [AssemblyNotFound]
-        [BinderTest(isolate: true, additionalLoadsToTrack: new string[] { DependentAssemblyName + "_Copy" } )]
+        [BinderTest(isolate: true,
+            additionalLoadsToTrack: new string[] { DependentAssemblyName + "_Copy" },
+            activeIssue: "https://github.com/dotnet/runtime/issues/68521")] // Emit-based Invoke causes AssemblyNotFound instead of MismatchedAssemblyName.
         public static BindOperation ApplicationAssemblies_MismatchedAssemblyName()
         {
             var assemblyName = new AssemblyName($"{DependentAssemblyName}_Copy, Culture=neutral, PublicKeyToken=null");
@@ -250,7 +253,7 @@ namespace BinderTracingTests
             var assemblyPath = Helpers.GetAssemblyInSubdirectoryPath(assemblyName.Name);
             CustomALC alc = new CustomALC(nameof(AssemblyLoadContextLoad), true /*throwOnLoad*/);
 
-            Assert.Throws<FileLoadException, Exception>(() => alc.LoadFromAssemblyName(assemblyName));
+            AssertExtensions.ThrowsWithInnerException<FileLoadException, Exception>(() => alc.LoadFromAssemblyName(assemblyName));
 
             return new BindOperation()
             {
@@ -336,7 +339,8 @@ namespace BinderTracingTests
         //   ResolutionAttempted : ApplicationAssemblies                (DefaultALC)    [AssemblyNotFound]
         //   ResolutionAttempted : DefaultAssemblyLoadContextFallback   (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AssemblyLoadContextResolvingEvent    (CustomALC)     [Success]
-        [BinderTest(isolate: true)]
+        [BinderTest(isolate: true,
+            activeIssue: "https://github.com/dotnet/runtime/issues/68521")] // Emit-based Invoke causes an extra load.
         public static BindOperation AssemblyLoadContextResolvingEvent_CustomALC()
         {
             var assemblyName = new AssemblyName(SubdirectoryAssemblyName);
@@ -405,14 +409,15 @@ namespace BinderTracingTests
         //   ResolutionAttempted : ApplicationAssemblies                (DefaultALC)    [AssemblyNotFound]
         //   ResolutionAttempted : DefaultAssemblyLoadContextFallback   (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AssemblyLoadContextResolvingEvent    (CustomALC)     [Exception]
-        [BinderTest(isolate: true)]
+        [BinderTest(isolate: true,
+            activeIssue: "https://github.com/dotnet/runtime/issues/68521")] // Emit-based Invoke causes an extra load.
         public static BindOperation AssemblyLoadContextResolvingEvent_CustomALC_Exception()
         {
             var assemblyName = new AssemblyName(SubdirectoryAssemblyName);
             CustomALC alc = new CustomALC(nameof(AssemblyLoadContextResolvingEvent_CustomALC_Exception));
             using (var handlers = new Handlers(HandlerReturn.Exception, alc))
             {
-                Assert.Throws<FileLoadException, BinderTestException>(() => alc.LoadFromAssemblyName(assemblyName));
+                AssertExtensions.ThrowsWithInnerException<FileLoadException, BinderTestException>(() => alc.LoadFromAssemblyName(assemblyName));
 
                 return new BindOperation()
                 {
@@ -444,7 +449,7 @@ namespace BinderTracingTests
             var assemblyName = new AssemblyName(SubdirectoryAssemblyName);
             using (var handlers = new Handlers(HandlerReturn.Exception, AssemblyLoadContext.Default))
             {
-                Assert.Throws<FileLoadException, BinderTestException>(() => AssemblyLoadContext.Default.LoadFromAssemblyName(assemblyName));
+                AssertExtensions.ThrowsWithInnerException<FileLoadException, BinderTestException>(() => AssemblyLoadContext.Default.LoadFromAssemblyName(assemblyName));
 
                 return new BindOperation()
                 {
@@ -471,7 +476,8 @@ namespace BinderTracingTests
         //   ResolutionAttempted : DefaultAssemblyLoadContextFallback   (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AssemblyLoadContextResolvingEvent    (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AppDomainAssemblyResolveEvent        (CustomALC)     [Success]
-        [BinderTest(isolate: true)]
+        [BinderTest(isolate: true,
+            activeIssue: "https://github.com/dotnet/runtime/issues/68521")] // Emit-based Invoke causes an extra load.
         public static BindOperation AppDomainAssemblyResolveEvent_CustomALC()
         {
             var assemblyName = new AssemblyName(SubdirectoryAssemblyName);
@@ -544,14 +550,15 @@ namespace BinderTracingTests
         //   ResolutionAttempted : DefaultAssemblyLoadContextFallback   (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AssemblyLoadContextResolvingEvent    (CustomALC)     [AssemblyNotFound]
         //   ResolutionAttempted : AppDomainAssemblyResolveEvent        (CustomALC)     [Exception]
-        [BinderTest(isolate: true)]
+        [BinderTest(isolate: true,
+            activeIssue: "https://github.com/dotnet/runtime/issues/68521")] // Emit-based Invoke causes an extra load.
         public static BindOperation AppDomainAssemblyResolveEvent_Exception()
         {
             var assemblyName = new AssemblyName(SubdirectoryAssemblyName);
             CustomALC alc = new CustomALC(nameof(AppDomainAssemblyResolveEvent_Exception));
             using (var handlers = new Handlers(HandlerReturn.Exception))
             {
-                Assert.Throws<FileLoadException, BinderTestException>(() => alc.LoadFromAssemblyName(assemblyName));
+                AssertExtensions.ThrowsWithInnerException<FileLoadException, BinderTestException>(() => alc.LoadFromAssemblyName(assemblyName));
 
                 return new BindOperation()
                 {
@@ -585,7 +592,7 @@ namespace BinderTracingTests
 
             Assembly asm = AssemblyLoadContext.Default.LoadFromAssemblyPath(assemblyPath);
 
-            Assert.AreNotEqual(assemblyPath, asm.Location);
+            Assert.NotEqual(assemblyPath, asm.Location);
             return new BindOperation()
             {
                 AssemblyName = asm.GetName(),
