@@ -12,7 +12,7 @@ class Foo
 		
 }
 
-public class Test
+public class Test_method001
 {
 	public static int counter = 0;
 	public static bool result = true;
