@@ -12,7 +12,7 @@ public class Foo
 		
 }
 
-public class Test
+public class Test_method001a
 {
 	public static int counter = 0;
 	public static bool result = true;
