@@ -12,7 +12,7 @@ class Foo<U>
 		
 }
 
-public class Test
+public class Test_method004
 {
 	public static int counter = 0;
 	public static bool result = true;
