@@ -18,7 +18,7 @@ class Foo : IFoo
 		
 }
 
-public class Test
+public class Test_method007
 {
 	public static int counter = 0;
 	public static bool result = true;
