@@ -25,7 +25,7 @@ class Gen<T> : IGenX<T[]>, IGenY<T>
   	}
 }
 
-public class Test
+public class Test_MultipleInterface01
 {
 	public static int counter = 0;
 	public static bool result = true;
