@@ -37,6 +37,8 @@ class InheritingFromGrowingBase : GrowingBase
     public int x;
 }
 
+struct LocallyDefinedStructure {}
+
 
 static class OpenClosedDelegateExtension
 {
@@ -63,7 +65,7 @@ class Program
             // Make sure the constrained call to ToString doesn't box
             var mystruct = new MyStructWithVirtuals();
             mystruct.ToString();
-            Assert.AreEqual(mystruct.X, "Overriden");
+            Assert.AreEqual(mystruct.X, "Overridden");
         }
     }
 
@@ -134,7 +136,7 @@ class Program
         MyClass.TestStaticFields();
 
 #if false // TODO: Enable once LDFTN is supported
-        Task.Run(() => { 
+        Task.Run(() => {
            MyClass.ThreadStaticStringField = "Garbage";
            MyClass.ThreadStaticIntField = 0xBAAD;
            MyClass.ThreadStaticDateTimeField = DateTime.Now;
@@ -330,7 +332,7 @@ class Program
 
         public void TestMultipleLoads()
         {
-            Assembly a = LoadFromAssemblyPath(Path.Combine(Directory.GetCurrentDirectory(), "test.ni.dll"));
+            Assembly a = LoadFromAssemblyPath(Path.Combine(Directory.GetCurrentDirectory(), "test.dll"));
             Assert.AreEqual(AssemblyLoadContext.GetLoadContext(a), this);
         }
 
@@ -357,7 +359,7 @@ class Program
 
     static void TestOpenClosedDelegate()
     {
-        // This test is verifying the the fixups for open vs. closed delegate created against the same target
+        // This test is verifying the fixups for open vs. closed delegate created against the same target
         // method are encoded correctly.
 
         Func<string, string, object> idOpen = OpenClosedDelegateExtension.OpenClosedDelegateTarget;
@@ -366,11 +368,11 @@ class Program
         Func<string, object> idClosed = "World".OpenClosedDelegateTarget;
         Assert.AreEqual(idClosed("hey"), "World, hey");
     }
-    
+
     static void GenericLdtokenFieldsTest()
     {
         Func<FieldInfo, string> FieldFullName = (fi) => fi.FieldType + " " + fi.DeclaringType.ToString() + "::" + fi.Name;
-        
+
         IFieldGetter getter1 = new FieldGetter<string>();
         IFieldGetter getter2 = new FieldGetter<object>();
         IFieldGetter getter3 = new FieldGetter<int>();
@@ -390,7 +392,7 @@ class Program
             string expectedDllField3 = "System.Collections.Generic.List`1[???] MyGeneric`2[???,???]::m_Field3".Replace("???", instArg.ToString());
             string expectedDllField4 = "System.Collections.Generic.KeyValuePair`2[???,System.Int32] MyGeneric`2[???,???]::m_Field4".Replace("???", instArg.ToString());
             string expectedDllField5 = "System.Int32 MyGeneric`2[???,???]::m_Field5".Replace("???", instArg.ToString());
-            
+
             Assert.AreEqual(expectedField1, FieldFullName(getter.GetGenT_Field1()));
             Assert.AreEqual(expectedField2, FieldFullName(getter.GetGenT_Field2()));
             Assert.AreEqual(expectedField3, FieldFullName(getter.GetGenT_Field3()));
@@ -412,70 +414,137 @@ class Program
             Assert.AreEqual(value[i], (byte)(9 - i));
     }
 
+    // public constructor, so we run something when loading from byte array in the test below
+    public Program()
+    {
+        // do something in the constructor to see if it works
+        TestVirtualMethodCalls();
+    }
+
     static void TestLoadR2RImageFromByteArray()
     {
         Assembly assembly1 = typeof(Program).Assembly;
-        
+
         byte[] array = File.ReadAllBytes(assembly1.Location);
         Assembly assembly2 = Assembly.Load(array);
-        
+
         Assert.AreEqual(assembly2.FullName, assembly1.FullName);
+
+        assembly2.CreateInstance("Program");
+    }
+
+    [MethodImplAttribute(MethodImplOptions.NoInlining)]
+    static void TestILBodyChange()
+    {
+        int actualMethodCallResult = (int)typeof(ILInliningTest).GetMethod("TestDifferentIntValue").Invoke(null, new object[]{});
+        Console.WriteLine(actualMethodCallResult);
+        Assert.AreEqual(ILInliningTest.TestDifferentIntValue(), actualMethodCallResult);
+    }
+
+    private class CallDefaultVsExactStaticVirtual<T> where T : IDefaultVsExactStaticVirtual
+    {
+        public static string CallMethodOnGenericType() => T.Method();
+    }
+
+    [MethodImplAttribute(MethodImplOptions.NoInlining)]
+    static void TestDefaultVsExactStaticVirtualMethodImplementation()
+    {
+        Assert.AreEqual(CallDefaultVsExactStaticVirtual<DefaultVsExactStaticVirtualClass>.CallMethodOnGenericType(), "DefaultVsExactStaticVirtualMethod");
+        // Naively one would expect that the following should do, however Roslyn fails to compile it claiming that the type DVESVC doesn't contain 'Method':
+        // Assert.AreEqual(DefaultVsExactStaticVirtualClass.Method(), "DefaultVsExactStaticVirtualMethod");
     }
 
     static void RunAllTests()
     {
+        Console.WriteLine("TestVirtualMethodCalls");
         TestVirtualMethodCalls();
+        Console.WriteLine("TestMovedVirtualMethod");
         TestMovedVirtualMethods();
 
+        Console.WriteLine("TestConstrainedMethodCalls");
         TestConstrainedMethodCalls();
 
+        Console.WriteLine("TestConstrainedMethodCalls_Unsupported");
         TestConstrainedMethodCalls_Unsupported();
 
+        Console.WriteLine("TestInterop");
         TestInterop();
 
+        Console.WriteLine("TestStaticFields");
         TestStaticFields();
 
+        Console.WriteLine("TestPreInitializedArray");
         TestPreInitializedArray();
 
+        Console.WriteLine("TestMultiDimmArray");
         TestMultiDimmArray();
 
+        Console.WriteLine("TestGenericVirtualMethod");
         TestGenericVirtualMethod();
+        Console.WriteLine("TestMovedGenericVirtualMethod");
         TestMovedGenericVirtualMethod();
+        Console.WriteLine("TestGenericNonVirtualMethod");
         TestGenericNonVirtualMethod();
 
+        Console.WriteLine("TestGenericOverStruct");
         TestGenericOverStruct();
 
+        Console.WriteLine("TestInstanceFields");
         TestInstanceFields();
 
+        Console.WriteLine("TestInstanceFieldsWithLayout");
         TestInstanceFieldsWithLayout();
 
+        Console.WriteLine("TestInheritingFromGrowingBase");
         TestInheritingFromGrowingBase();
 
+        Console.WriteLine("TestGrowingStruct");
         TestGrowingStruct();
+        Console.WriteLine("TestChangingStruct");
         TestChangingStruct();
+        Console.WriteLine("TestChangingHFAStruct");
         TestChangingHFAStruct();
 
+        Console.WriteLine("TestGetType");
         TestGetType();
 
+        Console.WriteLine("TestMultipleLoads");
         TestMultipleLoads();
 
+        Console.WriteLine("TestFieldLayoutNGenMixAndMatch");
         TestFieldLayoutNGenMixAndMatch();
 
+        Console.WriteLine("TestStaticBaseCSE");
         TestStaticBaseCSE();
 
+        Console.WriteLine("TestIsInstCSE");
         TestIsInstCSE();
 
+        Console.WriteLine("TestCastClassCSE");
         TestCastClassCSE();
 
+        Console.WriteLine("TestRangeCheckElimination");
         TestRangeCheckElimination();
 
+        Console.WriteLine("TestOpenClosedDelegate");
         TestOpenClosedDelegate();
-        
+
+        Console.WriteLine("GenericLdtokenFieldsTest");
         GenericLdtokenFieldsTest();
 
+        Console.WriteLine("RVAFieldTest");
         RVAFieldTest();
-        
+
+        Console.WriteLine("TestLoadR2RImageFromByteArray");
         TestLoadR2RImageFromByteArray();
+
+        Console.WriteLine("TestILBodyChange");
+        TestILBodyChange();
+        
+        Console.WriteLine("TestDefaultVsExactStaticVirtualMethodImplementation");
+        TestDefaultVsExactStaticVirtualMethodImplementation();
+        
+        ILInliningVersioningTest<LocallyDefinedStructure>.RunAllTests(typeof(Program).Assembly);
     }
 
     static int Main()
@@ -484,7 +553,10 @@ class Program
         for (int i = 0; i < 3; i++)
            RunAllTests();
 
-        Console.WriteLine("PASSED");
+        if (!Assert.HasAssertFired)
+            Console.WriteLine("PASSED");
+        else
+            Console.WriteLine("FAILED");
         return Assert.HasAssertFired ? 1 : 100;
     }
 
