@@ -15,12 +15,11 @@
 [assembly: System.Reflection.AssemblyCompany("Microsoft Corporation")]
 [assembly: System.Reflection.AssemblyProduct("Microsoft® .NET Framework")]
 [assembly: System.Reflection.AssemblyCopyright("© Microsoft Corporation.  All rights reserved.")]
-[assembly: System.Reflection.AssemblyFileVersion("1.0.24212.01")]
-[assembly: System.Reflection.AssemblyInformationalVersion("1.0.24212.01. Commit Hash: 9688ddbb62c04189cac4c4a06e31e93377dccd41")]
+[assembly: System.Reflection.AssemblyFileVersion("4.6.23123.00")]
+[assembly: System.Reflection.AssemblyInformationalVersion("4.6.23123.00 built by: PROJECTKREL")]
 [assembly: System.CLSCompliant(true)]
-[assembly: System.Reflection.AssemblyMetadata(".NETFrameworkAssembly", "")]
-[assembly: System.Reflection.AssemblyMetadata("Serviceable", "True")]
-[assembly: System.Reflection.AssemblyVersionAttribute("4.1.0.0")]
+[assembly: System.Reflection.AssemblyMetadata("", "")]
+[assembly: System.Reflection.AssemblyVersionAttribute("4.0.10.0")]
 [assembly: System.Reflection.AssemblyFlagsAttribute((System.Reflection.AssemblyNameFlags)0x70)]
 namespace System
 {
@@ -743,8 +742,6 @@ namespace System
 
         public static bool HasShutdownStarted { get { throw null; } }
 
-        public static string MachineName { get { throw null; } }
-
         public static string NewLine { get { throw null; } }
 
         public static int ProcessorCount { get { throw null; } }
@@ -753,16 +750,12 @@ namespace System
 
         public static int TickCount { get { throw null; } }
 
-        public static void Exit(int exitCode) { }
-
         public static string ExpandEnvironmentVariables(string name) { throw null; }
 
         public static void FailFast(string message, Exception exception) { }
 
         public static void FailFast(string message) { }
 
-        public static string[] GetCommandLineArgs() { throw null; }
-
         public static string GetEnvironmentVariable(string variable) { throw null; }
 
         public static Collections.IDictionary GetEnvironmentVariables() { throw null; }
