@@ -209,6 +209,61 @@ namespace Microsoft.Build.Tasks.Git.UnitTests
             Assert.Equal(workingDir2.Path, repository.WorkingDirectory);
         }
 
+        [Fact]
+        public void OpenRepository_Version1_Extensions()
+        {
+            using var temp = new TempRoot();
+
+            var workingDir = temp.CreateDirectory();
+            var gitDir = workingDir.CreateDirectory(".git");
+
+            gitDir.CreateFile("HEAD");
+            gitDir.CreateFile("config").WriteAllText(@"
+[core]
+	repositoryformatversion = 1
+[extensions]
+    noop = 1
+    preciousObjects = true
+    partialClone = promisor_remote
+    worktreeConfig = true
+");
+
+            Assert.True(GitRepository.TryFindRepository(gitDir.Path, out var location));
+            Assert.Equal(gitDir.Path, location.CommonDirectory);
+            Assert.Equal(gitDir.Path, location.GitDirectory);
+            Assert.Null(location.WorkingDirectory);
+
+            var repository = GitRepository.OpenRepository(location, GitEnvironment.Empty);
+            Assert.Equal(gitDir.Path, repository.CommonDirectory);
+            Assert.Equal(gitDir.Path, repository.GitDirectory);
+            Assert.Null(repository.WorkingDirectory);
+        }
+
+        [Fact]
+        public void OpenRepository_Version1_UnknownExtension()
+        {
+            using var temp = new TempRoot();
+
+            var homeDir = temp.CreateDirectory();
+
+            var workingDir = temp.CreateDirectory();
+            var gitDir = workingDir.CreateDirectory(".git");
+
+            gitDir.CreateFile("HEAD").WriteAllText("ref: refs/heads/master");
+            gitDir.CreateDirectory("refs").CreateDirectory("heads").CreateFile("master").WriteAllText("0000000000000000000000000000000000000000");
+            gitDir.CreateDirectory("objects");
+
+            gitDir.CreateFile("config").WriteAllText(@"
+[core]
+	repositoryformatversion = 1
+[extensions]
+	newExtension = true");
+
+            var src = workingDir.CreateDirectory("src");
+
+            Assert.Throws<NotSupportedException>(() => GitRepository.OpenRepository(src.Path, new GitEnvironment(homeDir.Path)));
+        }
+
         [Fact]
         public void OpenRepository_VersionNotSupported()
         {
@@ -223,7 +278,7 @@ namespace Microsoft.Build.Tasks.Git.UnitTests
             gitDir.CreateDirectory("refs").CreateDirectory("heads").CreateFile("master").WriteAllText("0000000000000000000000000000000000000000");
             gitDir.CreateDirectory("objects");
 
-            gitDir.CreateFile("config").WriteAllText("[core]repositoryformatversion = 1");
+            gitDir.CreateFile("config").WriteAllText("[core]repositoryformatversion = 2");
 
             var src = workingDir.CreateDirectory("src");
 
@@ -398,13 +453,7 @@ namespace Microsoft.Build.Tasks.Git.UnitTests
             var repository = new GitRepository(GitEnvironment.Empty, GitConfig.Empty, gitDir.Path, gitDir.Path, workingDir.Path);
 
             var submodules = repository.GetSubmodules();
-            AssertEx.Equal(new[]
-            {
-                "S10: 'sub10' 'http://github.com'",
-                "S11: 'sub11' 'http://github.com'",
-                "S6: 'sub6' 'http://github.com'",
-                "S9: 'sub9' 'http://github.com'"
-            }, submodules.Select(s => $"{s.Name}: '{s.WorkingDirectoryRelativePath}' '{s.Url}'"));
+            Assert.Empty(submodules);
 
             var diagnostics = repository.GetSubmoduleDiagnostics();
             AssertEx.Equal(new[]
@@ -413,12 +462,6 @@ namespace Microsoft.Build.Tasks.Git.UnitTests
               string.Format(Resources.InvalidSubmodulePath, "S1", "  "),
               // The path of submodule 'S2' is missing or invalid: ''
               string.Format(Resources.InvalidSubmodulePath, "S2", ""),
-              // Could not find a part of the path 'sub3\.git'.
-              TestUtilities.GetExceptionMessage(() => File.ReadAllText(Path.Combine(workingDir.Path, "sub3", ".git"))),
-              // Could not find a part of the path 'sub4\.git'.
-              TestUtilities.GetExceptionMessage(() => File.ReadAllText(Path.Combine(workingDir.Path, "sub4", ".git"))),
-              // Could not find a part of the path 'sub5\.git'.
-              TestUtilities.GetExceptionMessage(() => File.ReadAllText(Path.Combine(workingDir.Path, "sub5", ".git"))),
               // The format of the file 'sub7\.git' is invalid.
               string.Format(Resources.FormatOfFileIsInvalid, Path.Combine(workingDir.Path, "sub7", ".git")),
               // Path specified in file 'sub8\.git' is invalid.
@@ -459,8 +502,8 @@ namespace Microsoft.Build.Tasks.Git.UnitTests
             submoduleRefsHeadsDir.CreateFile("master").WriteAllText("0000000000000000000000000000000000000000");
             submoduleGitDir.CreateFile("HEAD").WriteAllText("ref: refs/heads/master");
 
-            var repository = new GitRepository(GitEnvironment.Empty, GitConfig.Empty, gitDir.Path, gitDir.Path, workingDir.Path);
-            Assert.Equal("0000000000000000000000000000000000000000", repository.ReadSubmoduleHeadCommitSha(submoduleWorkingDir.Path));
+            Assert.Equal("0000000000000000000000000000000000000000",
+                GitRepository.GetSubmoduleReferenceResolver(submoduleWorkingDir.Path)?.ResolveHeadReference());
         }
 
         [Fact]
@@ -479,8 +522,22 @@ namespace Microsoft.Build.Tasks.Git.UnitTests
             oldStyleSubmoduleRefsHeadDir.CreateFile("branch1").WriteAllText("1111111111111111111111111111111111111111");
             oldStyleSubmoduleGitDir.CreateFile("HEAD").WriteAllText("ref: refs/heads/branch1");
 
-            var repository = new GitRepository(GitEnvironment.Empty, GitConfig.Empty, gitDir.Path, gitDir.Path, workingDir.Path);
-            Assert.Equal("1111111111111111111111111111111111111111", repository.ReadSubmoduleHeadCommitSha(oldStyleSubmoduleWorkingDir.Path));
+            Assert.Equal("1111111111111111111111111111111111111111",
+                GitRepository.GetSubmoduleReferenceResolver(oldStyleSubmoduleWorkingDir.Path)?.ResolveHeadReference());
+        }
+
+        [Fact]
+        public void GetSubmoduleHeadCommitSha_NoGitFile()
+        {
+            using var temp = new TempRoot();
+
+            var gitDir = temp.CreateDirectory();
+            var workingDir = temp.CreateDirectory();
+
+            var submoduleGitDir = temp.CreateDirectory();
+            var submoduleWorkingDir = workingDir.CreateDirectory("sub").CreateDirectory("abc");
+
+            Assert.Null(GitRepository.GetSubmoduleReferenceResolver(submoduleWorkingDir.Path)?.ResolveHeadReference());
         }
     }
 }
